﻿var background = chrome.extension.getBackgroundPage();

var processMulti = false;
var multiValue = 0;
var searchTerm = "";
var wizardOpen = false;
var trackingOpen = false;
var messageAreaWidth = 0;
var whatProductApp = "10"; //Free

var currentContact;
var currentContactEmail;

var growthTrialEndDate;
var renderTrialCountdown = false;
var daysRemainingOnTrial = 0;

var currentlyInGrowthTrial = false;
var ebstaGrowthLicence = false;

/**
 * Global variable which stores values for Custom Bar Searches
 */
var gCustomBarSearchValues = {
    /**
     * Array of objects containing each sections found SalesforceId
     */
    EbstaCustomBarValues: [],
    /**
     * Adds an object to the EbstaCustomBarValues array which contains a Section number and SalesforceId
     * @param {} section 
     * @param {} idResult 
     * @returns {} 
     */
    AddCustomBarValue: function(section, idResult) {
        this.EbstaCustomBarValues.push(this.CreateEbstaCustomBarValue(section, idResult));
    },
    /**
     * Creates an object which stores the sections resulting ID of the Salesforce object found
     * @param {} section 
     * @param {} idResult 
     * @returns {} 
     */
    CreateEbstaCustomBarValue : function(section, idResult) {
        return {
            SectionNumber: section, 
            SalesforceId: idResult,
            GetId : function() {
                return this.SalesforceId;
            }
        }
    },
    /**
     * Sort the array ascending
     * @returns {} 
     */
    SortEbstaCustomBarValuesBySectionNumber : function() {
        this.EbstaCustomBarValues.sort(function (a, b) {
            if (a["SectionNumber"] < b["SectionNumber"])
                return -1;
            if (a["SectionNumber"] > b["SectionNumber"])
                return 1;
            return 0;
        });
    },
    /**
     * Names of the last person searched
     */
    NameSearched: "",
    /**
     * Email of the last person searched
     */
    EmailSearched: "",
    /**
     * Flag to state whether we've run the searches for the 3rd and 4th section
     */
    RelatedSearchesRun: false,
    /**
     * Sets the RelatedSearchesRun flag  to true
     * @returns {} 
     */
    SetRelatedSearchesRunFlag : function() {
        this.RelatedSearchesRun = true;
    }
}

function getParameterByName(name) {
    //console.log("CleverBarLocalProcess: " + name);
    name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
    var regexS = "[\\?&]" + name + "=([^&#]*)";
    var regex = new RegExp(regexS);
    var results = regex.exec(window.location.search);
    if (results == null)
        return "";
    else
        return decodeURIComponent(results[1].replace(/\+/g, " "));
}

function ReformatBarLayout() {
    var spaceAvailable = CalculateSpareBarSpace();

    var iconSpaceThreshold = 180 + messageAreaWidth;
    var messageSpaceThreshold = 315 + messageAreaWidth;
    var lowerMessageThreshold = messageAreaWidth + 65; // Bit of a hack to resize the text element of the Bar

    var thresholdModifier = 0;

    //if we are rendering trial messaging in the bar we need to alter the spacing thresholds to ensure it is displayed correctly at each resize stage.
    if (renderTrialCountdown) {
        thresholdModifier = 200;
    }

    if (spaceAvailable < (280 + thresholdModifier)) { //This should be the smallest bar possible on a 1024px resolution, we just want the details, wizard and close buttons
        $(".logoSection").hide();
        $("#wizardLoader").hide();
        $(".trimOnResize").hide();
        $(".trimOnResizeMore").hide();
        $(".ebsta-results-bttn").hide();

        if (renderTrialCountdown) {
            $(".ebsta-clickprompt").hide();
        }

    } else if (spaceAvailable < (lowerMessageThreshold + thresholdModifier)) { //remove ebsta logo, question mark and cut call to action text to just display number of results and ebsta logo.
        $(".logoSection").hide();
        $("#wizardLoader").hide();
        $(".trimOnResize").hide();
        $(".trimOnResizeMore").hide();
        $(".ebsta-results-bttn").show();
        $(".ebsta-clickprompt").show();
    } else if (spaceAvailable < (iconSpaceThreshold + thresholdModifier)) { //remove ebsta logo, question mark and cut call to action text.
        $(".logoSection").hide();
        $("#wizardLoader").hide();
        $(".trimOnResize").hide();
        $(".trimOnResizeMore").show();
        $(".ebsta-results-bttn").show();
        $(".ebsta-clickprompt").show();
    } else if (spaceAvailable < (messageSpaceThreshold + thresholdModifier)) { //remove ebsta logo and question mark.
        $(".logoSection").hide();
        $("#wizardLoader").hide();
        $(".trimOnResize").show();
        $(".trimOnResizeMore").show();
        $(".ebsta-results-bttn").show();
        $(".ebsta-clickprompt").show();
    } else { //show all sections in layout.
        $(".logoSection").show();
        $("#wizardLoader").show();
        $(".trimOnResize").show();
        $(".trimOnResizeMore").show();
        $(".ebsta-results-bttn").show();
        $(".ebsta-clickprompt").show();
    }
}

//detect window resize changes and adjust layout accordingly.
$(window).resize(function () {
    ReformatBarLayout();
});

function CalculateSpareBarSpace() {
    var windowWidth = $(window).width();

    var resPanelWidth = 0;

    var barSections = $(".barContent");
    var seperators = $(".seperator");

    $(barSections).each(function () {
        if ($(this).is(":visible")) {
            resPanelWidth = resPanelWidth + $(this).outerWidth();
        }
    });

    $(seperators).each(function () {
        if ($(this).is(":visible")) {
            resPanelWidth = resPanelWidth + $(this).outerWidth();
        }
    });

    var spareSpace = windowWidth - resPanelWidth;

    return spareSpace;
}

//set up the wizard menu events.
$("body").delegate(".menuSection", "click", function () {

    var identifier = $(this).attr("identifier");
    var selectedIdentifier = $(".selected").children().attr("identifier");

    if (selectedIdentifier != identifier) {
        $(".selected").removeClass("selected");
        $("." + identifier).parent().addClass("selected");

        $("." + selectedIdentifier + "content").fadeOut("fast", function () {
            $("." + identifier + "content").fadeIn("fast");
        });

        if ($(this).parent().attr("class").indexOf("lastMenuItem") != -1) {
            $(".nextSpan").empty();

            if ($(".previousSpan").html() == '') {
                RenderWizardButtonPrevious();
            }
        } else {
            if ($(".previousSpan").html() == '') {
                RenderWizardButtonPrevious();
            }

            if ($(this).parent().attr("class").indexOf("firstMenuItem") != -1) {
                $(".previousSpan").empty();

                if ($(".nextSpan").html() == '') {
                    RenderWizardButtonNext();
                }
            } else {
                if ($(".nextSpan").html() == '') {
                    RenderWizardButtonNext();
                }
            }
        }
    }
});

$("body").delegate(".nextButton", "click", function () {

    $(".contentSection").stop();

    var identifier = $(".selected").children().attr("identifier");
    var newIdentifier = $(".selected").next().children().attr("identifier");

    var selectedClass = $(".selected").attr("class");

    if (selectedClass.indexOf("lastMenuItem") != -1) {
        $(".selected").removeClass("selected");
        $(".firstMenuItem").addClass("selected");

        $(".lastContentItem").fadeOut("fast", function () {
            $(".firstContentItem").fadeIn("fast");
        });
    } else {
        if ($(".selected").next().attr("class").indexOf("lastMenuItem") != -1) {
            $(".nextSpan").empty();
        } else {
            if ($(".previousSpan").html() == '') {
                RenderWizardButtonPrevious();
            }
        }

        $(".selected").removeClass("selected").next().addClass("selected");

        $("." + identifier + "content").fadeOut("fast", function () {
            $("." + newIdentifier + "content").fadeIn("fast");
        });
    }
});

$("body").delegate(".previousButton", "click", function () {

    $(".contentSection").stop();

    var identifier = $(".selected").children().attr("identifier");
    var newIdentifier = $(".selected").prev().children().attr("identifier");

    var selectedClass = $(".selected").attr("class");

    if (selectedClass.indexOf("firstMenuItem") != -1) {
        $(".selected").removeClass("selected");
        $(".lastMenuItem").addClass("selected");

        $(".firstContentItem").fadeOut("fast", function () {
            $(".lastContentItem").fadeIn("fast");
        });
    } else {
        if ($(".selected").prev().attr("class").indexOf("firstMenuItem") != -1) {
            $(".previousSpan").empty();
        } else {
            if ($(".nextSpan").html() == '') {
                RenderWizardButtonNext();
            }
        }

        $(".selected").removeClass("selected").prev().addClass("selected");

        $("." + identifier + "content").fadeOut("fast", function () {
            $("." + newIdentifier + "content").fadeIn("fast");
        });
    }
});

function RenderWizardButtonPrevious() {
    var nextLink = document.createElement('a');
    nextLink.href = '#';
    nextLink.className = 'previousButton';
    nextLink.appendChild(document.createTextNode("Previous"));

    $(".previousSpan").append(nextLink);

    $("body").undelegate(".previousButton", "click");
    $("body").delegate(".previousButton", "click", function () {

        $(".contentSection").stop();

        var identifier = $(".selected").children().attr("identifier");
        var newIdentifier = $(".selected").prev().children().attr("identifier");

        var selectedClass = $(".selected").attr("class");

        if (selectedClass.indexOf("firstMenuItem") != -1) {
            $(".selected").removeClass("selected");
            $(".lastMenuItem").addClass("selected");

            $(".firstContentItem").fadeOut("fast", function () {
                $(".lastContentItem").fadeIn("fast");
            });
        } else {
            if ($(".selected").prev().attr("class").indexOf("firstMenuItem") != -1) {
                $(".previousSpan").empty();
            } else {
                if ($(".nextSpan").html() == '') {
                    RenderWizardButtonNext();
                }

            }

            $(".selected").removeClass("selected").prev().addClass("selected");

            $("." + identifier + "content").fadeOut("fast", function () {
                $("." + newIdentifier + "content").fadeIn("fast");
            });
        }
    });
}

function RenderWizardButtonNext() {
    var nextLink = document.createElement('a');
    nextLink.href = '#';
    nextLink.className = 'nextButton';
    nextLink.appendChild(document.createTextNode("Next"));

    $(".nextSpan").append(nextLink);

    $("body").undelegate(".nextButton", "click");
    $("body").delegate(".nextButton", "click", function () {

        $(".contentSection").stop();

        var identifier = $(".selected").children().attr("identifier");
        var newIdentifier = $(".selected").next().children().attr("identifier");

        var selectedClass = $(".selected").attr("class");

        if (selectedClass.indexOf("lastMenuItem") != -1) {
            $(".selected").removeClass("selected");
            $(".firstMenuItem").addClass("selected");

            $(".lastContentItem").fadeOut("fast", function () {
                $(".firstContentItem").fadeIn("fast");
            });
        } else {
            if ($(".selected").next().attr("class").indexOf("lastMenuItem") != -1) {
                $(".nextSpan").empty();
            } else {
                if ($(".previousSpan").html() == '') {
                    RenderWizardButtonPrevious();
                }
            }

            $(".selected").removeClass("selected").next().addClass("selected");

            $("." + identifier + "content").fadeOut("fast", function () {
                $("." + newIdentifier + "content").fadeIn("fast");
            });
        }
    });
}

function CheckTrialMessaging() {
    //console.log('renderTrialCountdown: ' + renderTrialCountdown);
    //console.log('daysRemainingOnTrial: ' + daysRemainingOnTrial);

    //if we need to display the trial countdown.
    if (renderTrialCountdown) {

        var trialMessage = "Trial expires in " + daysRemainingOnTrial + " days";

        if (daysRemainingOnTrial + "" === "1") {
            trialMessage = "Trial expires in " + daysRemainingOnTrial + " day";
        }

        //set the counter in the button.
        $(".ebsta-daysleftontrial-span").text(trialMessage);

        //display the div.
        $(".ebsta-bt-trialmessage-bar-container").show();

        //set up the click event.
        $("body").undelegate(".ebsta-bt-trialmessage-bar", "click");
        $("body").delegate(".ebsta-bt-trialmessage-bar", "click", function () {
            var redirectUrl = "https://www.ebsta.com/pricing/";
            window.open(redirectUrl, "_blank");
        });
    }
}

window.addEventListener("DOMContentLoaded", function () {

    AbCheckForEmailTracking();

    RunSearch();

    ListenForBarClickEvents();

    $("body").delegate("#close", "click", function () {
        chrome.extension.sendMessage({ MessageType: "CloseBar" });
    });

    $("body").delegate("#wizardLoader", "click", function () {

        // First we'll send a message to background.js to find the current url of the active page so 
        // we know wether to bring up the Onboarding process or normal LinkedIn Wizard
        chrome.extension.sendMessage({ MessageType: "GetPageUrl" }, function (response) {
            if (response.url) {
                if (response.url.indexOf("google.com") == -1) {
                    if (wizardOpen) {
                        wizardOpen = false;
                        $(".footerLinks").hide();
                        $("#WizardPanel").slideToggle("slow", function () {
                            chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '0' });
                        });
                    }
                    else {
                        wizardOpen = true;
                        // email tracking
                        if (trackingOpen) {
                            CloseTrackingPanel(true);
                        } else {
                            // old part
                            $(".footerLinks").show();
                            chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '100001' });
                            $("#WizardPanel").slideToggle("slow");
                        }
                    }
                }
                else {
                    //sends message to open wizards calls HandleBarOpenWizardModals() in bar.js
                    chrome.extension.sendMessage({ MessageType: "BarOpenWizardModals" });
                }
            }
        });
    });

    $("body").delegate(".close", "click", function () {

        if (wizardOpen) {
            wizardOpen = false;
            $("#WizardPanel").slideToggle("slow", function () {
                chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '0' });
            });
            $(".footerLinks").hide();
        } else {
            if (trackingOpen) {
                trackingOpen = false;
                wizardOpen = false;
                $("#TrackingPanel").slideToggle("slow", function () {
                    CleanUpAfterTrackingBar();
                    chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '0' });
                });
            } else {
                trackingOpen = true;
            }

            //wizardOpen = true;
        }

        CloseTrackingPanel(false);
    });

    // open/close trackingPanel
    WireUpTrackingPanel();
    WireUpSettingsPanel();
}, false);

function RunSearch() {

    var searchType = "" + getParameterByName('st');

    var isChatterOnlyAccount = '' + getParameterByName('co');

    var name = getParameterByName('name');

    //console.log("RunSearch(): " + searchType);

    //console.log("cleverBarLocalProcess: " + name);
    if (searchType != "3") {
        //console.log("cleverBarLocalProcess: within the tidy up")
        name = EbstaNameSanitizer.TidyName(name);
    }

    //console.log("cleverBarLocalProcess: RunSearch()");
    //console.log(name);
    //console.log(searchType);
    //console.log(personname);

    // email tracking
    var unknownWebsiteValue = "400";
    var knownWebsiteValue = "401";
    if (searchType == unknownWebsiteValue || searchType == knownWebsiteValue) {
        $(".resultsPanel").hide();
        $(".gmailWelcomePanel").show();

        chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '100001' });
        $("#TrackingPanel").slideToggle("slow");
        OpenTrackingPanel();
        return;
    }



    if (searchType == "300") {
        $(".resultsPanel").hide();
        $(".gmailWelcomePanel").show();

        $(".ebsta-results-bttn").html('<span class="additionalResultsMessage"></span> Click<span class="btIcon"> Ebsta</span> to open the Ebsta bubble');

        wizardOpen = true;
        $(".footerLinks").show();
        chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '100001' });
        $("#WizardPanel").slideToggle("slow");
    }

    var movePage = getParameterByName('changeView');

    var personname = '';
    if (searchType === "4") {
        personname = getParameterByName('personname');
        
        if (!CheckSearchStringIsGood(name)) return;

        ProcessCustomBar(name, personname);
    }
    else if (searchType == "50") {
        ClearBar();

        chrome.extension.sendMessage({
            MessageType: "SetUpgradeDay"
        }, function (response) {
        }
         );

        $(".ebstabar").addClass("expiredbar");
        var aLinke = document.createElement("span");
        aLinke.setAttribute("class", "openPro");
        aLinke.innerText = "Upgrade to Ebsta Pro to view, edit and update Salesforce without leaving this website. See Pricing";

        $('#messageText').text('');
        $('#messageText').append(aLinke);

        $('.openPro').click(function () {
            chrome.extension.sendMessage({
                MessageType: "PaymentPageOpen"
            }, function (response) {
            }
          );

        }
        );
    }
    else if (searchType == "40") {
        ClearBar();
        ClearBar();

        $(".ebstabar").addClass("expiredbar");
        var searchingForText = document.createTextNode('Start a Free trial of Ebsta Pro to put Salesforce inside this site with Ebsta');

        $('#messageText').text('');
        $('#messageText').append(searchingForText);


    }
    else if (isChatterOnlyAccount == "1") {
        var chatterUserText = document.createTextNode('Chatter User? Contact <a href="mailto:support@ebsta.com" target="_top" style="text-decoration:underline;color:#ffffff;">support@ebsta.com</a> for more information on how to view Salesforce results for ' + name + ' in Ebsta.');

        $('#messageText').text('');
        $('#messageText').append(chatterUserText);
    } else if (searchType == "33") {

        $(".ebstabar").removeClass("redbar");
        $(".ebstabar").removeClass("greenbar");
        $(".ebstabar").removeClass("bluebar");
        $(".ebstabar").addClass("greybar");
        $(".ebstabar").text('');
    } else if (searchType == "2") {
        
        if (!CheckSearchStringIsGood(name)) return;

        var searchingForText = document.createTextNode('Searching for ' + name + ' in companies');
        
        $('#messageText').text('');
        $('#messageText').append(searchingForText);

        chrome.extension.sendMessage({ MessageType: "BarCompanySearch", SearchTerm: "" + name, ChangeView: movePage }, function (response) {
            if (response.result == "notloggedin") {
                $(".noLoginPanel").show();
                $(".resultsPanel").hide();
                $(".noResultsPanel").hide();
            }
        });

    } else if (searchType == "26") { //load from the background activity poll...       

        // Removed all the rubbish.
        var blueBarText = '';
        $('#messageText').text('');
        $('#messageText').append(blueBarText);

        $(".ebstabar").removeClass("redbar");
        $(".ebstabar").removeClass("greenbar");
        $(".ebstabar").addClass("bluebar");
    } else {

        if (!CheckSearchStringIsGood(name)) return;

        var customBarSection = null;
        if (searchType === BarSearchTypes.CustomBarAccount.URL
            || searchType === BarSearchTypes.CustomBarAccount.LINKEDINUSER
            || searchType === BarSearchTypes.CustomBarAccount.LINKEDINCOMPANY)
        {
            customBarSection = document.getElementById("customBarAccount");
            $(".resultsPanel").hide();
            if (customBarSection) {
                ShowElementBlock(customBarSection)
            };
        }

        var barText = document.createTextNode('Searching for ' + name + ' in people.');

        $('#messageText').text('');
        $('#messageText').append(barText);

        chrome.extension.sendMessage({
            MessageType: "BarSearch",
            SearchTerm: "" + name,
            ChangeView: "" + movePage,
            BarSearchType: searchType
        }, function (response) {
            if (response) {
                if (response.result == "LoggedOut") {
                    BarDisplayLoginState();
                } else if (response.result == "notloggedin") {
                    BarDisplayLoginState();
                } else {
                    ProcessCustomBarSearchResponse(response, customBarSection);
                }
            }
        });
    }
}

/**
 * Checks certain criteria to ensure the search string is good enough to be sent up to the server to be searched on
 * @param {any} searchString - Original search term
 * @returns {boolean} - If the string is a good string to search for
 */
function CheckSearchStringIsGood(searchString) {
    if (searchString && searchString.length < 3) {
        RenderSearchTermNotLongEnoughMessage();
        return false;
    }

    return true;
}

/** When a string is not long enough to be searched for we'll display a message to the user for this */
function RenderSearchTermNotLongEnoughMessage() {
    $('.resultsPanel').hide();
    $('.noResultsPanel').show();
    $('.noResultsPanel .textRow').text("Your search term is too broad. Please search again with a minimum of 3 characters.");
}

/**
 * Takes a request result and finds if we managed to find any results matching the 
 * criteria, if so show the result if not show that we didn't find anything
 * @param {object} response
 * @param {element} customBarSectionElement
 */
function ProcessCustomBarSearchResponse(response, customBarSectionElement) {
    var jsonObject = response.d;
    if (jsonObject) {

        chrome.tabs.getCurrent(function (currentTab) {
            background.UpdateLastCustomSearch(null, jsonObject.ActualSearchTerm, currentTab.id, currentTab.url);

            var valuesObject = jsonObject.Values;
            if (valuesObject && valuesObject.length > 0) {
                var parsedFirstValue = JSON.parse(valuesObject[0]);
                if (parsedFirstValue.CustomBarColumns) {
                    ProcessCustomAccountBar(parsedFirstValue.CustomBarColumns, customBarSectionElement);
                }
            } else {
                //No results found
                HideElement(customBarSectionElement);
                RenderNoResultsBar(jsonObject.ActualSearchTerm);
            }
        });
    }
}

/**
 * Renders information into the bar from a given data set
 * @param {array} columnsData
 * @param {element} customBarSectionElement
 */
function ProcessCustomAccountBar(columnsData, customBarSectionElement) {
    if (customBarSectionElement) {
        customBarSectionElement.innerHTML = "";
        customBarSectionElement.appendChild(CreateElement("div", "mainIcon ContactIcon"));
        ShowElementBlock(customBarSectionElement);
        for (var i = 0; i < columnsData.length; i++) {
            var newSection = gBarSectionCreator.CreateSection([columnsData[i].RowOne, columnsData[i].RowTwo]);
            customBarSectionElement.appendChild(newSection);
            if (columnsData.length - 1 != i) {
                customBarSectionElement.appendChild(CreateElement("div", "seperator"));
            }
        }
    }
}

/** Simple module to help create elements within the bar within the bar */
var gBarSectionCreator = (function () {

    /**
     * Creates a section within the bar
     * @param {array} leftSideValues - The values you'd like to see in the left column of the section (This should be an array of objects
     * in the structure of e.g. {Label: "AccountName", Type: "reference", Value: "Test Account"})
     */
    function CreateSection(leftSideValues) {
        var section = CreateElement("div", "barsection barContent");
        var textBlock = CreateElement("div", "textBlock");
        var leftSideBlock = CreateElement("div", "leftBlock");

        //Set up the left column NOTE: This is only setup to handle 2 rows currently
        for (var i = 0; i < leftSideValues.length; i++) {
            var leftSideValue = leftSideValues[i];
            if (leftSideValue) {
                var label = null;
                if (leftSideValue.Label) {
                    label = leftSideValue.Label + " : ";
                }

                var rowDataElement = CreateElement("span", "textRow", label);
                if (i == 1) {
                    AddClass(rowDataElement, "bottomRow");
                }

                if (leftSideValue.Value != null) {
                    //Handle reference fields by treating them as anchors and linking them to what they represent
                    if (leftSideValue.Type === "reference") {
                        rowDataElement.appendChild(CreateElement("a", null, leftSideValue.Value, "", { href: leftSideValue.Link, target: "_blank" }))
                    } else {
                        rowDataElement.innerText += leftSideValue.Value;
                    }
                }

                leftSideBlock.appendChild(rowDataElement);
            }
        }

        textBlock.appendChild(leftSideBlock);
        section.appendChild(textBlock);
        return section;
    }

    return {
        CreateSection: CreateSection
    }

})();

chrome.extension.onMessage.addListener(
 function (request, sender, sendResponse) {
     var userFailAuth = false;
     if (request.DataObject != null) {
         if (request.DataObject.indexOf('REAUTH:') != -1) {
             userFailAuth = true;
         }
     }

     if (request.MessageType == "BarSearchResults" && !userFailAuth) {
         var displayText = request.SearchType;
         var arrowString = ' <img id="ebstaarrow" src="../images/curvedarrow.png" />';
         var whyNotAdd = ' Click to create a new record.';
         var databaseBeingSearched = '';
         var dbTypeId = request.CrmDBType;

         switch (dbTypeId) {
             case '2':
                 databaseBeingSearched = 'Salesforce';
                 break;
             case '20':
                 databaseBeingSearched = 'Jobscience';
                 break;
             case '21':
                 databaseBeingSearched = 'Talent Rover';
                 break;
             case '22':
                 databaseBeingSearched = 'Target Recruit';
                 break;
             case '23':
                 databaseBeingSearched = 'Click To Cloud';
                 break;
             default:
                 databaseBeingSearched = 'Salesforce';
                 break;
         }

         if (request.SearchType != "MissingAuth") {
             sendResponse({ farewell: "goodbye" });
             if (request.SearchType == "client") {
                 displayText = "companies";
             }

             if (request.SearchType == "sfcontact") {
                 displayText = "contacts";
             }

             if (request.SearchType == "sfaccount") {
                 displayText = "accounts";
             }

             if (request.SearchType == "people-auto") {
                 displayText = "people";
             }

             if (request.SearchType == "people-manual") {
                 displayText = "people";
             }

             if (request.SearchType == "client-auto") {
                 displayText = "client";
             }

             if (request.SearchType == "client-manual") {
                 displayText = "client";
             }

             if (request.SearchType == "company-auto") {
                 displayText = "company";
             }

             if (request.SearchType == "company-manual") {
                 displayText = "company";
             }

             if (IsPopup(window)) {
                 $("#messageText").addClass('popuparrow');
             }

         } else {
             BarDisplayLoginState();
         }

         var emailData = request.EmailDataObject;
         searchTerm = request.SearchTerm;

         var resultCounter = request.Result;
         var searchResultData = request.DataObject;

         var renderingInGmail = false;

         if (typeof emailData != "undefined" && emailData != null) {
             searchResultData = emailData;
             renderingInGmail = true;
         }

         if (resultCounter == 1) { //render single record bar...
             processMulti = false;
             RenderSingleResultsBar(searchResultData, renderingInGmail, searchTerm);
         } else if (resultCounter == 0 && request.SearchType != "MissingAuth") { //render no record bar...
             processMulti = false;
             RenderNoResultsBar(searchTerm, request.SearchType);
             CheckSearchTermTrackingEvents(searchTerm);
         } else if (resultCounter > 1) { //render multi record bar...)
             multiValue = resultCounter;
             processMulti = true;
             RenderMultiResultsBar(searchResultData, renderingInGmail, searchTerm);
         }

         $("#messageText").delegate("#close", "click", function () {
             chrome.extension.sendMessage({ MessageType: "CloseBar" });
         });
     }

     if (!userFailAuth) {
         if (request.MessageType == "ProcessBarSections") {
             RenderBarSegments(request);
             messageAreaWidth = 490;
             ReformatBarLayout();
         }

         //send a section number field as we need to process 4 sections
         if (request.MessageType == "ProcessCustomBarSections") {
             RenderBarSegments(request);
             messageAreaWidth = 490;
             ReformatBarLayout();
         }
     }

     // email tracking - items
     if (request.MessageType == "LoadTrackItems") {
         LoadTrackItems(request.generalfields);
     }
     if (request.MessageType == "OpenCloseTrackingPanelMessage") {
         OpenCloseTrackingPanel();
     }

     if (request.MessageType == "ProccessContactHasTrackingEvents") {
         ProccessCheckContactHasTrackingEvents(request.HasTrackingEvents);
     }

     if (request.MessageType === "UpdateEmailTrackingAlertIcon") {
         UpdateEmailTrackingAlertIcon(request);
     }

 });

function CheckNull(inputString, objectType, isPlural) {
    if (inputString == null || inputString == "null" || inputString == "") {
        var suffix = '';
        if (isPlural) {
            suffix = 's';
        }

        return 'No ' + objectType + suffix;
    }

    return inputString;
}
//check for null/empty fields and return a label instead if we find any
function CheckNullCustomField(inputString, label)
{
    if (inputString == null || inputString == "null" || inputString == "") {
        return "No " + label;
    }
    return inputString;
}

function RenderSpecialCharacters(input) {
    return $(document.createElement('div')).html(input).text();
}

function RenderBarSegments(request) {
    var instanceUrl = request.InstanceUrl;
    //console.log("RenderBarSegments: InstanceUrl:");
    //console.log(instanceUrl);
    //console.log(request);
    //render individual request sections here - sections should be made visible via initial request type.

    var db = JSON.parse(request.DataObject);
    //console.log("CleverBarLocalProcess.js: RenderBarSegments: UseCustomBarSearch: " + db.d.Values);
    //console.log("CleverBarLocalProcess.js: RenderBarSegments: ");
    //console.log(request.EntityType);
    if (request.EntityType == "cbcontactbyid") {
        if (db.d.Values.length > 0) {
            ReInstateStandardIcons()
            var recordFound = JSON.parse(db.d.Values[0]);

            var contactName = CheckNull(recordFound.Name, "Contact", true);
            var contactId = recordFound.Id;
            var contactTitle = CheckNull(recordFound.Title, "title", false);
            var contactEmail = CheckNull(recordFound.Email, "email address", false);

            var contactPhone = '';

            if (recordFound.MobilePhone != null && recordFound.MobilePhone != "null" && recordFound.MobilePhone != "") {
                contactPhone = CheckNull(recordFound.MobilePhone, "phone number", false);
            } else if (recordFound.Phone != null && recordFound.Phone != "null" && recordFound.Phone != "") {
                contactPhone = CheckNull(recordFound.Phone, "phone number", false);
            } else {
                contactPhone = CheckNull(recordFound.OtherPhone, "phone number", false);
            }

            var accountName = '';
            var accountType = '';
            var accountUrl = '';
            var accountId = '';

            var accountLink = document.createElement('a');

            if (recordFound.Account != null && typeof recordFound.Account != "undefined") {
                accountName = CheckNull(recordFound.Account.Name, "Account", false);
                accountType = CheckNull(recordFound.Account.Type, "Account type", false);
                accountId = recordFound.Account.Id;

                accountLink.href = instanceUrl + "/" + accountId;
                accountLink.appendChild(document.createTextNode(accountName));
                accountLink.target = "_blank";
                accountLink.className = "barAccountName ebsta-bar-field";

                $(".accountNameSpan").empty();
                $(".accountNameSpan").append(accountLink);
            } else {
                accountName = 'No Account';
                accountType = 'No Account type';
                $(".accountNameSpan").empty();
                $(".accountNameSpan").text(accountName);

                //render no opporunities if we have no accountid in gmail.
                if (request.Url != null) {
                    if (request.Url.indexOf("mail.google.com") != -1) {

                        $(".oppOpenNumber").empty();
                        $(".oppOpenSpan").empty();
                        $(".oppOpenValue").empty();

                        $(".oppOpenNumber").append(document.createTextNode('0'));
                        $(".oppOpenSpan").append(document.createTextNode('Open: '));
                        $(".oppOpenValue").append(document.createTextNode('(0)'));

                        $(".oppClosedNumber").empty();
                        $(".oppClosedSpan").empty();
                        $(".oppClosedValue").empty();

                        $(".oppClosedNumber").append(document.createTextNode('0'));
                        $(".oppClosedSpan").append(document.createTextNode('Closed Won: '));
                        $(".oppClosedValue").append(document.createTextNode('(0)'));

                        $(".thirdSectionInner").show();
                        $(".thirdSectionLoader").hide();
                    }
                }
            }

            var nameLink = document.createElement('a');
            nameLink.href = instanceUrl + "/" + contactId;
            nameLink.appendChild(document.createTextNode(contactName));
            nameLink.target = "_blank";
            nameLink.className = "barContactName ebsta-bar-field";
            nameLink.setAttribute("data-contact-name", contactName);

            var emailLink = document.createElement('a');
            emailLink.href = "mailto:" + contactEmail;
            emailLink.appendChild(document.createTextNode(contactEmail));
            emailLink.target = "_top";
            emailLink.className = "barContactEmail ebsta-bar-field";
            emailLink.setAttribute("data-contact-email", contactEmail);

            if (recordFound.Email != null && recordFound.Email != "null" && typeof recordFound.Email != "undefined") {
                //Check if email has tracking events (for changing tracking bar icon)
                if (contactEmail.length > 0) {
                    CheckIfEmailHasTrackingEvents(recordFound.Email);
                }
            }

            currentContact = contactName;
            currentContactEmail = contactEmail;

            $(".nameSpan").empty();
            $(".jobTitleSpan").empty();
            $(".emailSpan").empty();

            $(".nameSpan").append(nameLink);
            $(".jobTitleSpan").append(document.createTextNode(contactTitle));

            if (recordFound.Email == null || recordFound.Email == "null" || recordFound.Email == "") {
                $(".emailSpan").append(document.createTextNode(contactEmail));
            } else {
                $(".emailSpan").append(emailLink);
            }

            $(".phoneSpan").empty();
            $(".accountTypeSpan").empty();

            $(".phoneSpan").append(document.createTextNode(contactPhone));
            $(".accountTypeSpan").append(document.createTextNode(accountType));

            $(".primaryDetails").show();
            $(".secondSectionInner").show();
            $(".mainRecordLoader").hide();
            $(".accountLoader").hide();

            var viewPrompt = document.createElement('span');
            viewPrompt.className = "trimOnResize";
            viewPrompt.appendChild(document.createTextNode('to view more.'));

            $(".clickToText").text('');
            $(".clickToText").append(viewPrompt);
        }
    }

    if (request.EntityType == "cbleadinfo") {
        if (db.d.Values.length > 0) {
            //ReInstateStandardIcons()
            var recordFound = JSON.parse(db.d.Values[0]);

            var leadName = CheckNull(recordFound.Name, "Lead", true);
            var leadId = recordFound.Id;
            var leadTitle = CheckNull(recordFound.Title, "title", false);
            var leadEmail = CheckNull(recordFound.Email, "email address", false);

            var leadPhone = '';

            if (recordFound.MobilePhone != null && recordFound.MobilePhone != "null" && recordFound.MobilePhone != "") {
                leadPhone = CheckNull(recordFound.MobilePhone, "phone number", false);
            } else {
                leadPhone = CheckNull(recordFound.Phone, "phone number", false);
            }

            var nameLink = document.createElement('a');
            nameLink.href = instanceUrl + "/" + leadId;
            nameLink.appendChild(document.createTextNode(leadName));
            nameLink.target = "_blank";
            nameLink.className = "barLeadName";
            nameLink.setAttribute("data-contact-name", leadName);

            var emailLink = document.createElement('a');
            emailLink.href = "mailto:" + leadEmail;
            emailLink.appendChild(document.createTextNode(leadEmail));
            emailLink.target = "_top";
            emailLink.className = "barContactEmail";
            emailLink.setAttribute("data-contact-email", leadEmail);

            if (recordFound.Email != null && recordFound.Email != "null" && typeof recordFound.Email != "undefined") {
                //Check if email has tracking events (for changing tracking bar icon)
                if (recordFound.Email.length > 0) {
                    CheckIfEmailHasTrackingEvents(recordFound.Email);
                }
            }

            $(".nameSpan").empty();
            $(".jobTitleSpan").empty();
            $(".emailSpan").empty();

            $(".nameSpan").append(nameLink);
            $(".jobTitleSpan").append(document.createTextNode(leadTitle));

            if (recordFound.Email == null || recordFound.Email == "null" || recordFound.Email == "") {
                $(".emailSpan").append(document.createTextNode(leadEmail));
            } else {
                $(".emailSpan").append(emailLink);
            }

            $(".phoneSpan").empty();

            $(".phoneSpan").append(document.createTextNode(leadPhone));

            $(".primaryDetails").show();
            $(".mainRecordLoader").hide();

            var viewPrompt = document.createElement('span');
            viewPrompt.className = "trimOnResize";
            viewPrompt.appendChild(document.createTextNode('to view more.'));

            $(".clickToText").text('');
            $(".clickToText").append(viewPrompt);

            $(".ContactIcon").removeClass(".ContactIcon").addClass("LeadIcon");
            $(".primaryRecord.barSection").attr("title", "Lead");
        }
    }

    if (request.EntityType == "cbleadactivityinfo") {
        $(".relatedContactDetails").hide();

        var nextActivityFound = false;
        var lastActivityFound = false;

        if (db.d.Values.length > 0) {
            //ReInstateStandardIcons()
            for (var dt = 0; dt < db.d.Values.length; dt++) {
                var record = JSON.parse(db.d.Values[dt]);

                var requestType = record.requesttype;
                var objectType = record.attributes.type;

                var subject = CheckNull(record.Subject, "subject", false);
                var startDateTime = '';
                var activityId = record.Id;

                if (record.ActivityDate != null) {
                    startDateTime = record.ActivityDate;
                } else {
                    startDateTime = record.LastModifiedDate;
                }

                var activityDate = null;
                var shortDate = '';

                if (startDateTime != null) {
                    activityDate = new Date(startDateTime);
                    if (activityDate.toString().indexOf("Invalid") == -1) {
                        shortDate = $.datepicker.formatDate('mm/dd/yy', new Date(activityDate));
                        shortDate = CheckNull(shortDate, "Date", false);
                    }
                }

                var whoId = '';
                var whoName = '';

                var whoLink = document.createElement('a');

                if (record.What != null && record.What != "null") {
                    whoId = record.What.Id;
                    whoName = CheckNull(record.What.Name, "related record", false);

                    whoLink.href = instanceUrl + "/" + whoId;
                    whoLink.appendChild(document.createTextNode(CheckStringLength(whoName, 20)));
                    whoLink.target = "_blank";
                }
                else if (record.Owner != null && record.Owner != "null") {
                    whoId = record.Owner.Id;
                    whoName = CheckNull(record.Owner.Name, "owner", false);

                    whoLink.href = instanceUrl + "/" + whoId;
                    whoLink.appendChild(document.createTextNode(CheckStringLength(whoName, 20)));
                    whoLink.target = "_blank";

                } else {
                    whoName = 'N/A';
                }

                if (requestType == "Next") {
                    var nextActivityLink = document.createElement('a');
                    nextActivityLink.href = instanceUrl + "/" + activityId;
                    nextActivityLink.appendChild(document.createTextNode(CheckStringLength(subject, 11)));
                    nextActivityLink.target = "_blank";
                    nextActivityLink.className = "barNextActivityName ebsta-bar-field";

                    $(".nextActivity").text('');
                    $(".nextActivity").append(nextActivityLink);

                    whoLink.className = "barNextActivityWho";
                    $(".nextWhoSpan").text('');
                    $(".nextWhoSpan").append(whoLink);

                    $(".nextDateSpan").empty();
                    $(".nextDateSpan").append(document.createTextNode(shortDate));

                    nextActivityFound = true;
                }

                if (requestType == "Last") {
                    var lastActivityLink = document.createElement('a');
                    lastActivityLink.href = instanceUrl + "/" + activityId;
                    lastActivityLink.appendChild(document.createTextNode(CheckStringLength(subject, 11)));
                    lastActivityLink.target = "_blank";
                    lastActivityLink.className = "barLastActivityName ebsta-bar-field";

                    $(".lastActivity").text('');
                    $(".lastActivity").append(lastActivityLink);

                    whoLink.className = "barLastActivityWho";
                    $(".lastWhoSpan").text('');
                    $(".lastWhoSpan").append(whoLink);

                    $(".lastDateSpan").empty();
                    $(".lastDateSpan").append(document.createTextNode(shortDate));

                    lastActivityFound = true;
                }
            }
        }

        if (!nextActivityFound) {
            $(".nextActivity").empty();
            $(".nextActivity").append(document.createTextNode("No Activity found"));
            $(".nextActivity").parent().attr("colspan", "2");
        }

        if (!lastActivityFound) {
            $(".lastActivity").empty();
            $(".lastActivity").append(document.createTextNode("No Activity found"));
            $(".lastActivity").parent().attr("colspan", "2");
        }

        $(".forthSectionInner").show();
        $(".forthSectionLoader").hide();
    }

    if (request.EntityType == "cbaccountinfo") {
        if (db.d.Values.length > 0) {
            ReInstateStandardIcons()
            var recordFound = JSON.parse(db.d.Values[0]);

            var accountName = RenderSpecialCharacters(CheckNull(recordFound.Name, "Account", false));
            var accountId = recordFound.Id;
            var accountType = RenderSpecialCharacters(CheckNull(recordFound.Type, "account type", false));
            var accountPhone = RenderSpecialCharacters(CheckNull(recordFound.Phone, "phone number", false));
            var accountWebsite = RenderSpecialCharacters(CheckNull(recordFound.Website, "website", false));

            var nameLink = document.createElement('a');
            nameLink.href = instanceUrl + "/" + accountId;
            nameLink.appendChild(document.createTextNode(accountName));
            nameLink.target = "_blank";

            //We are standardising the layout of the bar
            //put fields into the account section which should now universally be the second section
            $(".accountNameSpan").empty();
            $(".accountTypeSpan").empty();

            $(".accountNameSpan").append(nameLink);
            $(".accountTypeSpan").append(document.createTextNode(accountType));

            $(".primaryDetails").show();
            $(".secondSectionInner").show();
            $(".accountLoader").hide();

            var viewPrompt = document.createElement('span');
            viewPrompt.className = "trimOnResize";
            viewPrompt.appendChild(document.createTextNode('to view more.'));

            $(".clickToText").text('');
            $(".clickToText").append(viewPrompt);
        }
    }

    if (request.EntityType == "cbaccountcontacts") {
        ReInstateStandardIcons()

        if (db.d.Values.length > 0) {
            var recordFound = JSON.parse(db.d.Values[0]);

            var contactName = RenderSpecialCharacters(CheckNull(recordFound.Name, "Contact", true));
            var contactId = recordFound.Id;
            var contactTitle = RenderSpecialCharacters(CheckNull(recordFound.Title, "title", false));

            var nameLink = document.createElement('a');
            nameLink.href = instanceUrl + "/" + contactId;
            nameLink.appendChild(document.createTextNode(contactName));
            nameLink.target = "_blank";
            //standardising the order of the sections in the bar
            //want to add this into the first section rather than the relatedcontactrecord section
            $(".nameSpan").append(nameLink);
            $(".jobTitleSpan").append(document.createTextNode(contactTitle));

            $(".secondSectionInner").show();
            $(".mainRecordLoader").hide();

            if (db.d.Values.length == 2) {
                var secondRecordFound = JSON.parse(db.d.Values[1]);

                var contactName2 = RenderSpecialCharacters(CheckNull(secondRecordFound.Name, "Contact", true));
                var contactId2 = secondRecordFound.Id;
                var contactTitle2 = RenderSpecialCharacters(CheckNull(secondRecordFound.Title, "title", false));

                var nameLink2 = document.createElement('a');
                nameLink2.href = instanceUrl + "/" + contactId2;
                nameLink2.appendChild(document.createTextNode(contactName2));
                nameLink2.target = "_blank";

                $(".emailSpan").empty();
                $(".phoneSpan").empty();

                $(".emailSpan").append(nameLink2);
                $(".phoneSpan").append(document.createTextNode(contactTitle2));


            } else if (db.d.Values.length > 2) {
                $(".emailSpan").empty();

                $(".emailSpan").append(document.createTextNode("+ other Contacts"));
                $(".emailSpan").attr("colspan", "2");
                $(".phoneSpan").remove();
                $(".phoneSpan").addClass("textRight");
            }
        } else {
            //render something for no related contacts.

            $(".nameSpan").empty();
            $(".nameSpan").append(document.createTextNode("No related Contacts"));
            $(".mainRecordLoader").hide();

            //$(".contact1NameSpan").css("padding-top", "10px");
        }

        $(".fifthSectionLoader").hide();
    }

    if (request.EntityType == "cbopenopportunitescount") {

        $(".thirdSectionInner").show();
        $(".thirdSectionLoader").hide();
    }

    if (request.EntityType == "cbtaskeventinfo") {

        $(".relatedContactRecord").hide();

        var nextActivityFound = false;
        var lastActivityFound = false;
        //ReInstateStandardIcons()

        if (db.d.Values.length > 0) {
            for (var dt = 0; dt < db.d.Values.length; dt++) {
                var record = JSON.parse(db.d.Values[dt]);

                var requestType = record.requesttype;
                var objectType = record.attributes.type;

                var subject = RenderSpecialCharacters(CheckNull(record.Subject, "subject", false));
                var startDateTime = '';
                var activityId = record.Id;

                var shortDate = '';

                if (record.ActivityDate != null) {
                    shortDate = record.ActivityDate;
                } else if (record.LastModifiedDate) {
                    shortDate = record.LastModifiedDate;
                } else if (record.StartDateTime) {
                    shortDate = record.StartDateTime;
                }

                var whoId = '';
                var whatId = '';
                var whoName = '';
                var whatName = '';

                var whoLink = document.createElement('a');

                if (record.Who != null && record.Who != "null") {
                    whoId = record.Who.Id;
                    whoName = CheckNull(record.Who.Name, "related person", false);

                    whoLink.href = instanceUrl + "/" + whoId;
                    whoLink.appendChild(document.createTextNode(CheckStringLength(whoName, 20)));
                    whoLink.target = "_blank";
                } else if (record.What != null && record.What != "null") {
                    whoId = record.What.Id;
                    whoName = CheckNull(record.What.Name, "related person", false);

                    whoLink.href = instanceUrl + "/" + whoId;
                    whoLink.appendChild(document.createTextNode(CheckStringLength(whoName, 20)));
                    whoLink.target = "_blank";

                } else {
                    whoName = 'N/A';
                }

                if (requestType == "Next") {
                    var nextActivityLink = document.createElement('a');
                    nextActivityLink.href = instanceUrl + "/" + activityId;
                    nextActivityLink.appendChild(document.createTextNode(CheckStringLength(subject, 11)));
                    nextActivityLink.target = "_blank";
                    nextActivityLink.className = "barNextActivityName ebsta-bar-field";

                    $(".nextActivity").text('');
                    $(".nextActivity").append(nextActivityLink);

                    whoLink.className = "barNextActivityWho";
                    $(".nextWhoSpan").text('');
                    $(".nextWhoSpan").append(whoLink);

                    $(".nextDateSpan").empty();
                    $(".nextDateSpan").append(document.createTextNode(shortDate));

                    nextActivityFound = true;
                }

                if (requestType == "Last") {
                    var lastActivityLink = document.createElement('a');
                    lastActivityLink.href = instanceUrl + "/" + activityId;
                    lastActivityLink.appendChild(document.createTextNode(CheckStringLength(subject, 11)));
                    lastActivityLink.target = "_blank";
                    lastActivityLink.className = "barLastActivityName ebsta-bar-field";

                    $(".lastActivity").text('');
                    $(".lastActivity").append(lastActivityLink);

                    whoLink.className = "barLastActivityWho ebsta-bar-field";
                    $(".lastWhoSpan").text('');
                    $(".lastWhoSpan").append(whoLink);

                    $(".lastDateSpan").empty();
                    $(".lastDateSpan").append(document.createTextNode(shortDate));

                    lastActivityFound = true;
                }
            }
        }

        if (!nextActivityFound) {
            $(".nextActivity").empty();
            $(".nextActivity").append(document.createTextNode("No Activity found"));
            $(".nextActivity").parent().attr("colspan", "2");
        }

        if (!lastActivityFound) {
            $(".lastActivity").empty();
            $(".lastActivity").append(document.createTextNode("No Activity found"));
            $(".lastActivity").parent().attr("colspan", "2");
        }
        $(".forthSectionContainer").show();
        $(".forthSectionInner").show();
        $(".forthSectionLoader").hide();
    }

    if (request.EntityType == "cballopportunities") {
        ReInstateStandardIcons()

        if (db.d.ErrorMessage == "NOACCESS" || localStorage.CrmDBType == '21') { //hide opportunities section if no access.
            $(".thirdSection").hide();
            $(".breaker3").hide();
            $(".thirdSectionLoader").hide();
        } else if (db.d.Values.length > 0) {

            var shouldWeDisplayAmountData = false;

            for (var dt = 0; dt < db.d.Values.length; dt++) {
                var record = JSON.parse(db.d.Values[dt]);

                if (record.requesttype == "OpenOp") {

                    if (record.expr0 != null) {
                        var openOpportunities = record.expr0;
                        var openOpportunityValue = 0;

                        $(".oppOpenNumber").empty();
                        $(".oppOpenSpan").empty();
                        $(".oppOpenValue").empty();

                        //Don't show the value of oppoturnity part if they didn't have amount as a valid field on their opportnity object
                        if (record.expr1 != null) {
                            shouldWeDisplayAmountData = true;
                            if (parseInt(openOpportunities) != 0) {
                                //If the number is a decimal then round it up
                                if (record.expr1 % 1 != 0) {
                                    record.expr1 = record.expr1.toFixed(2)
                                }
                                openOpportunityValue = '' + record.expr1;
                            }
                        }

                        $(".oppOpenNumber").append(document.createTextNode(openOpportunityValue));

                        $(".oppOpenSpan").append(document.createTextNode('Open: '));
                        $(".oppOpenValue").append(document.createTextNode('(' + openOpportunities + ')'));

                    }
                }

                if (record.requesttype == "ClosedOp") {

                    if (record.expr0 != null) {
                        var closedOpportunities = record.expr0;
                        var closedOpportunityValue = 0;

                        $(".oppClosedNumber").empty();
                        $(".oppClosedSpan").empty();
                        $(".oppClosedValue").empty();

                        //Don't show the value of oppoturnity part if they didn't have amount as a valid field on their opportnity object
                        if (record.expr1 != null) {
                            shouldWeDisplayAmountData = true;
                            if (parseInt(closedOpportunities) != 0) {
                                //If the number is a decimal then round it up
                                if (record.expr1 % 1 != 0) {
                                    record.expr1 = record.expr1.toFixed(2);
                                }

                                closedOpportunityValue = '' + record.expr1;
                            }
                        }

                        $(".oppClosedNumber").append(document.createTextNode(closedOpportunityValue));

                        $(".oppClosedSpan").append(document.createTextNode('Closed Won: '));
                        $(".oppClosedValue").append(document.createTextNode('(' + closedOpportunities + ')'));
                    }
                }
            }

            //Hide the value fields if we've found no amount fields or there are no open opportunities
            if (!shouldWeDisplayAmountData) {
                $(".oppOpenNumber, .oppClosedNumber").hide();
            }
        }

        $(".thirdSectionInner").show();
        $(".thirdSectionLoader").hide();
    }

    // Bar Customisation. Last field returned is always the ID so we can create linkable objects
    if (request.EntityType === "cbcustombarsection" && request.MessageType === "ProcessCustomBarSections") {
        RenderCustomBarSections(request, db, instanceUrl);
    }

    if (processMulti) {

        processMulti = false;

        var otherResults = multiValue - 1;
        var resultString = 'results';

        if (otherResults == 1) {
            resultString = 'result';
        }

        var additionalRecordsSpan = document.createElement('span');
        additionalRecordsSpan.appendChild(document.createTextNode(otherResults));
        additionalRecordsSpan.className = "boldNumber";

        var foundPrompt = document.createElement('span');
        foundPrompt.className = "trimOnResize";
        foundPrompt.appendChild(document.createTextNode('Salesforce found '));

        var resultsText = document.createElement('span');
        resultsText.className = "trimOnResizeMore";
        resultsText.appendChild(document.createTextNode(' other ' + resultString + ' for ' + searchTerm + '. '));

        $(".additionalResultsCounter").text('');
        $(".additionalResultsCounter").append(document.createTextNode(otherResults + '+'));

        $(".additionalResultsMessage").show();
        $(".additionalResultsMessage").text('');
        $(".additionalResultsMessage").append(foundPrompt);
        $(".additionalResultsMessage").append(additionalRecordsSpan);
        $(".additionalResultsMessage").append(resultsText);
    }
}

/**
 * Renders the sections for custom bar search
 * @param {} request 
 * @param {} db 
 * @returns {} 
 */
function RenderCustomBarSections(request, db, instanceUrl) {
    var sectionNumber = request.SectionNumber;

    //As we no longer return an empty result set we need to do a different check.
    if (db.d.Values.length > 0) {
        var recordFound = JSON.parse(db.d.Values[0]);
        var nullcheck = recordFound["Results"].length;

        if (nullcheck > 0) {

            var fieldsFound = recordFound["Results"][0]["lstFields"];
            var objectType = recordFound["Results"][0]["ObjectName"];
            if (fieldsFound != null) {
                var numberOfFields = fieldsFound.length;
                var id = fieldsFound[numberOfFields - 1].fieldValue;

                var classElement = null;

                gCustomBarSearchValues.AddCustomBarValue(sectionNumber - 1, id);

                if (sectionNumber === 1) {
                    $(".nameSpan, .jobTitleSpan, .emailSpan, .phoneSpan").empty();

                    RenderFieldData(["nameSpan", "jobTitleSpan", "emailSpan", "phoneSpan"], fieldsFound, id, instanceUrl, numberOfFields);

                    classElement = ".primaryDetails";
                    $(".primaryRecord").attr('title', objectType);
                    $(".mainRecordLoader").hide();
                    $(".primaryDetails").show();
                }

                if (sectionNumber === 2) {
                    $(".accountNameSpan, .accountTypeSpan").empty();

                    RenderFieldData(["accountNameSpan", "accountTypeSpan"], fieldsFound, id, instanceUrl, numberOfFields);

                    $(".secondSection").attr('title', objectType);
                    $(".accountLoader").hide();
                    classElement = ".secondSectionInner";
                    $(".secondSectionInner").show();
                }

                if (sectionNumber === 3) {
                    $(".oppOpenSpan").empty().removeClass("opportunityGreenText");
                    $(".oppClosedSpan").empty();

                    RenderFieldData(["oppOpenSpan", "oppClosedSpan"], fieldsFound, id, instanceUrl, numberOfFields);

                    $(".thirdSectionLoader").hide();
                    $(".thirdSectionContainer").attr('title', objectType);
                    classElement = ".thirdSectionInner";
                    $(".thirdSectionInner").show();
                }

                if (sectionNumber === 4) {
                    $(".contact1NameSpan, .contact2NameSpan").empty();

                    RenderFieldData(["contact1NameSpan", "contact2NameSpan"], fieldsFound, id, instanceUrl, numberOfFields);

                    $(".relatedContactRecord").attr('title', objectType).show();
                    $(".fifthSectionLoader").hide();

                    classElement = ".relatedContactDetails";
                    $(".relatedContactDetails").show();
                }

                //Update the icon in the bar
                if (classElement != null) {
                    ChangeIconsInCustomBar(objectType, classElement);
                }
            }
            else {
                processNoCustomBarResults(db, sectionNumber);
            }
        }
        else {
            processNoCustomBarResults(db, sectionNumber);
        }
    }
    else {
        processNoCustomBarResults(db, sectionNumber);
    }

    CheckAndRunAdditionalSectionSearch();
}

/**
 * Takes a list of fields return from a custom bar section search
 * @param {} listOfElements 
 * @param {} numberOfFields 
 * @returns {} 
 */
function RenderFieldData(listOfElements, fieldsFound, id, instanceUrl, numberOfFields) {
    function GetFieldLabel(field) {
        return field["fieldLabel"];
    }

    function GetFieldValue(field) {
        return field["fieldValue"];
    }

    for (var i = 0; i < numberOfFields; i++) {
        var fieldLabel = GetFieldLabel(fieldsFound[i]);
        if (fieldLabel !== "Id") {
            var fieldValue = CheckCustomBarValue(GetFieldValue(fieldsFound[i]), fieldLabel, id, instanceUrl);
            var fieldElement = document.getElementsByClassName(listOfElements[i]);
            if (fieldElement.length > 0) {
                if (typeof fieldValue === 'string') {
                    fieldValue = CreateTextNode(fieldValue);
                }
                fieldElement[0].appendChild(fieldValue);
            }
        }
    }
}

/**
 * Checks whether we have enough values to run searches on the additional bar sections as they may need Salesforce Id's passed up in order for them to work
 * i.e. Section 4 is set to show Tasks, this will look for Tasks related to either Section 1/2
 * @returns {} 
 */
function CheckAndRunAdditionalSectionSearch() {
    //If we've received first 2 sections back we can now fire the last section search off
    if (gCustomBarSearchValues.EbstaCustomBarValues.length === 2 && !gCustomBarSearchValues.RelatedSearchesRun) {
        gCustomBarSearchValues.SortEbstaCustomBarValuesBySectionNumber();
        var idToSearchFor = null;
        for (var i = 0; i < gCustomBarSearchValues.EbstaCustomBarValues.length; i++) {
            idToSearchFor = gCustomBarSearchValues.EbstaCustomBarValues[i].GetId();
            if (idToSearchFor) break;
        }

        //Helps prevent potential double tapping
        gCustomBarSearchValues.SetRelatedSearchesRunFlag();

        chrome.extension.sendMessage({
            MessageType: "LoadCustomBarSegments",
            Email: "" + gCustomBarSearchValues.EmailSearched,
            NameSearchTerm: "" + gCustomBarSearchValues.NameSearched,
            SalesforceId: idToSearchFor, SectionsToLoad: [3, 4]
        });
    }
}

/**
 * Checks what field it is and cleans up the field value
 * @param {} fieldValue 
 * @param {} fieldLabel 
 * @param {} id 
 * @param {} instanceUrl 
 * @returns {} 
 */
function CheckCustomBarValue(fieldValue, fieldLabel, id, instanceUrl) {
    fieldValue = CheckNullCustomField(fieldValue, fieldLabel);

    //Lower case the Label for use 
    fieldLabel = fieldLabel.toLowerCase();

    fieldValue = CheckIfLinkableField(fieldValue, fieldLabel, id, instanceUrl);
    fieldValue = CheckIfEmailField(fieldValue, fieldLabel);
    fieldValue = CheckifDateField(fieldValue, fieldLabel);
    return fieldValue;
}

function processNoCustomBarResults(db, sectionNumber) {

    //Update global variable to state this section didn't have any results
    gCustomBarSearchValues.AddCustomBarValue(sectionNumber - 1, null);

    if (db.d.Values.length > 0) {
        var fieldsFound = db.d.Fields;
        var numberOfFields = fieldsFound.length;
        var objType = fieldsFound[numberOfFields - 1].Name;
        if (sectionNumber === 1) {
            $(".nameSpan").empty();
            $(".nameSpan").text("No " + objType + " results found");
            $(".mainRecordLoader").hide();
            $(".primaryRecord").attr('title', objType);
            var classElement = ".primaryDetails";
            ChangeIconsInCustomBar(objType, classElement);
            $(".primaryDetails").find(".leftBlock").attr('class', '');
            $(".primaryDetails").show();

        }
        if (sectionNumber === 2) {
            $(".accountNameSpan").empty();
            $(".accountNameSpan").text("No " + objType + " results found");
            $(".accountLoader").hide();
            $(".secondSection").attr('title', objType);

            var classElement = ".secondSectionInner";
            ChangeIconsInCustomBar(objType, classElement);
            $(".secondSectionInner").show();

        }
        if (sectionNumber === 3) {
            $(".oppOpenSpan").empty();
            $(".oppOpenSpan").text("No " + objType + " results found");
            $(".oppOpenSpan").removeClass("opportunityGreenText");
            $(".thirdSectionLoader").hide();
            $(".thirdSectionContainer").attr('title', objType);
            var classElement = ".thirdSectionInner";
            ChangeIconsInCustomBar(objType, classElement);
            $(".thirdSectionInner").find(".leftBlock").attr('class', '');
            $(".thirdSectionInner").show();

        }
        if (sectionNumber === 4) {
            $(".contact1NameSpan").empty();
            $(".contact1NameSpan").text("No " + objType + " results found");
            $(".fifthSectionLoader").hide();
            $(".relatedContactRecord").attr('title', objType);

            var classElement = ".relatedContactDetails";
            ChangeIconsInCustomBar(objType, classElement);
            $(".relatedContactRecord").show();
            $(".relatedContactDetails").show();

        }
    }
    else
    {
        if(sectionNumber === 1)
        {
            $(".nameSpan").empty();
            $(".mainRecordLoader").hide();
        }
        if (sectionNumber === 2) {
            $(".accountNameSpan").empty();
            $(".accountLoader").hide();
        }
        if (sectionNumber === 3) {
            $(".oppOpenSpan").empty();
            $(".thirdSectionLoader").hide();
        }
        if (sectionNumber === 4) {
            $(".contact1NameSpan").empty();
            $(".fifthSectionLoader").hide();
        }
    }

}

/**
 * Alter the icons in the Custom Bar based on the object type
 */
function ChangeIconsInCustomBar(objectType, classToCheck) {
    switch (objectType) {
        case "Activity":
        case "Task":
        case "Event":
        case "Tasks/Activities":
            $(classToCheck).find(".mainIcon").attr('class', 'mainIcon ActivityIcon');
            break;

        case "Account":
            $(classToCheck).find(".mainIcon").attr('class', 'mainIcon AccountIcon');
            break;

        case "Contact":
            $(classToCheck).find(".mainIcon").attr('class', 'mainIcon ContactIcon');
            break;

        case "Lead":
            $(classToCheck).find(".mainIcon").attr('class', 'mainIcon LeadIcon');
            break;

        case "Opportunity":
            $(classToCheck).find(".mainIcon").attr('class', 'mainIcon OpportunityIcon');
            break;

        case "Task", "Event":
            $(classToCheck).find(".mainIcon").attr('class', 'mainIcon TaskIcon');
            break;

        default:
            $(classToCheck).find(".mainIcon").attr('class', 'mainIcon CustomIcon');
    }
}

//Bar Customisation. When we display custom sections we change the icons, so now we need to change them back
function ReInstateStandardIcons() {
    $(".primaryDetails").find(".mainIcon").attr('class', 'mainIcon ContactIcon');
    $(".secondSectionInner").find(".mainIcon").attr('class', 'mainIcon AccountIcon');
    $(".thirdSectionInner").find(".mainIcon").attr('class', 'mainIcon OpportunityIcon');
    $(".relatedContactDetails").find(".mainIcon").attr('class', 'mainIcon relatedContactIcon');
}

function CheckifDateField(checkMe,labelName)
{
    if (labelName.includes("date"))
    {
        var activityDate = null;
        var shortDate = '';

        if (checkMe != null) {
            activityDate = new Date(checkMe);
            if (activityDate.toString().indexOf("Invalid") == -1) {
                shortDate = $.datepicker.formatDate('mm/dd/yy', new Date(activityDate));
                return CheckNull(shortDate, "Date", false);
            }
            else
            {
                return checkMe;
            }
        }
        else
        {
            return checkMe;
        }
    }
    else
        return checkMe;

}


//Bar Customisation. If we have an email field make it a clickable email link
function CheckIfEmailField(checkMe,labelName)
{
    if (checkMe != '' && checkMe != null && checkMe != "null") {
        if (labelName === "email") {
                if (checkMe.toLowerCase() != "no email") {
                    var emailLink = document.createElement('a');
                    emailLink.href = "mailto:" + checkMe;
                    emailLink.appendChild(document.createTextNode(checkMe));
                    emailLink.target = "_top";
                    emailLink.className = "barContactEmail";
                    emailLink.setAttribute("data-contact-email", checkMe);
                    return emailLink;
                }
            }
        }

        return checkMe;
}

//Bar Customisation. If we have a salesforce object field make it a clickable link
function CheckIfLinkableField(checkMe, labelname, idValue, Url)
{
    if (checkMe.toLowerCase() != "no website") {
        if (labelname === "name" || labelname === "website") {
            if (idValue != '' && idValue != null && idValue != "null") {
                var nameLink = document.createElement('a');
                nameLink.appendChild(document.createTextNode(checkMe));
                nameLink.target = "_blank";
                nameLink.className = "barAccountName";
                nameLink.setAttribute("data-contact-name", checkMe);

                if (labelname === "name") {
                    nameLink.href = Url + "/" + idValue;
                }
                else {
                    nameLink.href = checkUrl(checkMe);
                }
                return nameLink;
            }
        }

    }
        return checkMe;
}

function checkUrl(url) {
    var pattern = /^((http|https|ftp):\/\/)/;

    if (!pattern.test(url)) {
        url = "http://" + url;
    }
    return url;
}


    function CheckStringLength(inputString, trimIndex) {
        if (inputString != null) {
            if (inputString.length > trimIndex) {
                inputString = inputString.substring(0, trimIndex) + '...';

                return inputString;
            } else {
                return inputString;
            }
        }

        return inputString;
    }

    function IsPopup(window) {
        if (window.menubar.visible == false && window.opener == null) {
            return true;
        }
        else {
            return false;
        }
    }

    function RenderSingleResultsBar(searchResultData, renderingInGmail, searchTerm) {
        var recordFound = null;
        var recordType = null;

        if (renderingInGmail && searchResultData.length > 0) {
            recordFound = searchResultData[0];
            recordType = recordFound.Type;
        } else if (searchResultData.length > 0) {
            var db = JSON.parse(searchResultData)
            recordFound = JSON.parse(db.d.Values[0]);
            recordType = recordFound.attributes.type;
        }

        //Check if we have result of type Contact / Lead / Account - if so call func to proccess that result and exit
        if (recordType == 'Contact') {
            ProcessContactBar(recordFound);
            return;
        }

        if (recordType == 'Lead') {
            ProcessLeadBar(recordFound);
            return;
        }

        if (recordType == 'Account') {
            ProcessAccountBar(recordFound);
            return;
        }

        //If the result type is not Account / Lead / Contact - display empty result.
        RenderNoResultsBar(searchTerm);
    }

    function RenderMultiResultsBar(searchResultData, renderingInGmail, searchTerm) {

        var recordFound = null;
        var recordType = null;

        if (renderingInGmail) {
            for (var i = 0; i < searchResultData.length; i++) {
                recordFound = searchResultData[i];
                recordType = recordFound.Type;

                if (recordFound != null && typeof recordFound != "undefined") {
                    break;
                }
            }
        } else {
            var db = JSON.parse(searchResultData)
            for (var i = 0; i < db.d.Values.length; i++) {
                recordFound = JSON.parse(db.d.Values[i]);
                recordType = recordFound.attributes.type;

                if (recordFound != null && typeof recordFound != "undefined") {
                    break;
                }
            }
        }

        var defaultBarSearch = true;        
        if (typeof localStorage.UseDefaultBarSearch != "undefined" && localStorage.UseDefaultBarSearch != null)
        {
            defaultBarSearch = localStorage.UseDefaultBarSearch;
        }

        if (defaultBarSearch) {
            //Check if we have result of type Contact / Lead / Account - if so call func to proccess that result and exit
            if (recordType == 'Contact') {
                ProcessContactBar(recordFound);
                return;
            }

            if (recordType == 'Lead') {
                ProcessLeadBar(recordFound);
                return;
            }

            if (recordType == 'Account') {
                ProcessAccountBar(recordFound);
                return;
            }

        }
        else {
            return;

        }

        //If the result type is not Account / Lead / Contact - display empty result.
        RenderNoResultsBar(searchTerm);
    }

    function RenderNoResultsBar(searchTerm, searchType) {
        //console.log("CleverBarLocalProcess: RenderNoResultsBar");
        $(".clickToText").text('');
        $(".noResultsPanel").show();
        $(".resultsPanel").hide();
        $(".noLoginPanel").hide();

        $(".clickToText").empty();
        $(".searchTerm").empty();

        //Find if this email is on the blacklist for searching in Gmail to display a special message
        var emailBlackListHit = false;
        if (searchType == "Contact") {
            var emailSplit = searchTerm.split('@');
            if (emailSplit.length > 1) {
                if (localStorage.EmailBlacklist) {
                    if (localStorage.EmailBlacklist.indexOf(emailSplit[1]) != -1) {
                        emailBlackListHit = true;
                    }
                }
            }
        }

        if (!emailBlackListHit) {
            $(".clickToText").append(document.createTextNode("to create a new Salesforce record."));
            $(".searchTerm").append(document.createTextNode(searchTerm));
            $(".viewMoreSpanNoResult").show();
            $(".viewMoreSpanNoResult").addClass("ebsta-clickprompt");
        } else {
            $('.textRow').hide();
            $('.blackListResult').show();
            $(".searchTerm").append(document.createTextNode(emailSplit[1]));
        }

        $(".smallLogoIcon").addClass("smallWhiteLogo");

        $(".viewMoreSpan").hide();
        $(".viewMoreSpan").removeClass("ebsta-clickprompt");
    }

function ProcessContactBar(recordFound) {

    var accountUrl = '';
    var accountId = '';

        if (recordFound.Account != null && typeof recordFound.Account != "undefined") {
            accountUrl = recordFound.Account.attributes.url;

            var urlSplit = accountUrl.split('/');

            accountId = urlSplit[urlSplit.length - 1];
        }

        chrome.extension.sendMessage({ MessageType: "LoadSingleContactSegments", ObjectId: '' + recordFound.Id, AccountId: '' + accountId });
    }

    function ProcessLeadBar(recordFound) {

        $(".breaker1").hide();
        $(".secondSection").hide();
        $(".breaker2").hide();
        $(".thirdSection").hide();

        chrome.extension.sendMessage({ MessageType: "LoadSingleLeadSegments", LeadId: '' + recordFound.Id });
    }

    function ProcessAccountBar(recordFound) {

        chrome.extension.sendMessage({ MessageType: "LoadSingleAccountSegments", AccountId: '' + recordFound.Id });
    }

    function ProcessCustomBar(searchTerm, namesearchTerm) {
        gCustomBarSearchValues.EmailSearched = searchTerm;
        gCustomBarSearchValues.NameSearched = namesearchTerm;

        $(".relatedContactRecord").find(".mainIcon").attr('class', 'mainIcon');

        $(".fifthSectionLoader").show();

        $(".forthSection").hide();
        $(".breaker3").hide();
        $(".breaker4").show();
        $(".relatedContactRecord").show();
        $(".relatedContactDetails").hide();

        chrome.extension.sendMessage({ MessageType: "LoadCustomBarSegments", Email: '' + searchTerm, NameSearchTerm: '' + namesearchTerm, SectionsToLoad : [1,2] });
    }

    //Track Bar Click Events
    function ListenForBarClickEvents() {

        document.addEventListener("click", function (event) {

            var elementClicked = "Bar Click - ";
            var logEvent = false;

            if (event.toElement.tagName.toLowerCase() == 'a') {
                logEvent = true;
                if (event.toElement.className.length > 0) {
                    elementClicked += event.toElement.className;

                    if (elementClicked.indexOf("bar") != -1) {
                        elementClicked = elementClicked.replace("bar", "");
                    }

                    //Wizard next and previous buttons
                    if (elementClicked.indexOf("Button") != -1) {
                        elementClicked = elementClicked.replace("-", "- Wizard");
                    }

                    //Wizard header sections
                    if (elementClicked.indexOf("section1") != -1) {
                        elementClicked = "Bar Click - Wizard First Section";
                    }
                    else if (elementClicked.indexOf("section2") != -1) {
                        elementClicked = "Bar Click - Wizard Second Section";
                    }
                    else if (elementClicked.indexOf("section3") != -1) {
                        elementClicked = "Bar Click - Wizard Third Section";
                    }
                }
                else {
                    logEvent = true;

                    idElement = event.toElement.id;
                    //Find if wizard is being shown or closed using the wizard button (? button)
                    if (idElement.toLowerCase().indexOf("wizard") != -1) {
                        var wizVisible = document.getElementById("WizardPanel").style.height;

                        if (wizVisible < '250px') {
                            elementClicked += "Wizard Opened";
                        }
                        else {
                            elementClicked += "Wizard Closed";
                        }
                    }
                        //Ebsta Close Button
                    else if (idElement.toLowerCase().indexOf("close") != -1) {
                        elementClicked += "Ebsta Bar Closed";
                    }
                }


            } else if (event.toElement.title.indexOf("Close Wizard") != -1) {
                logEvent = true;
                elementClicked += "Wizard Closed (X - Button)";
            }

            if (logEvent) {
                chrome.extension.sendMessage({ MessageType: "LogBarClick", Element: '' + elementClicked });
            }
        });



    }


    function WireUpTrackingPanel() {

        $("body").delegate("#trackingLoader", "click", function () {

            if (trackingOpen) {
                CloseTrackingPanel(false);
            } else {
                OpenTrackingPanel();
                chrome.extension.sendMessage({ MessageType: "LogOpenButtonClicked" });
            }

        });
    }

function WireUpSettingsPanel() {

    $("body").undelegate("#settingsLoader", "click");
    $("body").delegate("#settingsLoader", "click", function () {
        var redirectUrl = 'https://console.ebsta.com/LoginSalesforce.aspx?k=' + localStorage.WebUserId + '&j=crmintegration&objectLinker=Account';
        window.open(redirectUrl, "_blank");
    });

}

// email tracking functions
function OpenCloseTrackingPanel() {

        if (trackingOpen) {
            CloseTrackingPanel(false);
        } else {
            OpenTrackingPanel();
            chrome.extension.sendMessage({ MessageType: "LogOpenButtonClicked" });
        }
    }

    function OpenTrackingPanel() {

        if (trackingOpen) {
            return;
        }

        trackingOpen = true;

        if (wizardOpen) {
            wizardOpen = false;
            $("#WizardPanel").slideToggle("slow", function () {
                chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '0' });
                // callback wait for slideToggle
                OpenTrackingPanelAfterWizardBarClosed();
            });
            $(".footerLinks").hide();

            return;
        }

        OpenTrackingPanelAfterWizardBarClosed();
    }
    
    /**
     * Searches for an email address within the bar if one exists
     * @returns {} 
     */
    function GetEmailFromBar() {
        return $('.cleverbar').find('.barContactEmail').attr('data-contact-email');
    }

    function OpenTrackingPanelAfterWizardBarClosed() {
        $('#TrackingPanel').hide();
        $('#TrackingPanel').removeClass('DisplayNone');
        $('#TrackingPanel').addClass('TrackingPanelVisible');

        TrackingSearch.SetSearchText(GetEmailFromBar());
        TrackingSearch.SetFilterCombo();
        TrackingSearch.Search();

        chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '100001' });
        $("#TrackingPanel").slideDown("slow");
    }

    function CloseTrackingPanel(wizardWillBeOpened) {

        if (trackingOpen) {
            trackingOpen = false;

            $("#TrackingPanel").slideUp("slow", function () {

                if (!wizardWillBeOpened) {
                    chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '0' });
                }

                CleanUpAfterTrackingBar();

                if (wizardWillBeOpened) {
                    OpenWizardPanelAfterTrackingBarClosed();
                }

            });
        }
    }

    function CleanUpAfterTrackingBar() {

        $('.trackingrowdiv').remove();
        $('.Noresult').remove();
        $('#TrackingPanel').removeClass('TrackingPanelVisible');
        $('#TrackingPanel').addClass('DisplayNone');
    }

    function CreateTrackingTitle() {

        var trackingPanelDiv = document.getElementById("TrackingPanel");

        var panelHook = document.createElement('span');
        panelHook.setAttribute('id', 'TrackingPanelHook');
        var title = document.createElement("h1");
        title.setAttribute("id", "h1Tracking");
        var titleLabel = 'Email tracking';

        var titleNode = document.createTextNode(titleLabel);
        title.appendChild(titleNode);

        panelHook.appendChild(title);

        var br = document.createElement("br");
        title.setAttribute("id", "brTracking");
        panelHook.appendChild(br);

        var trackingMainContentDiv = document.createElement('div');
        trackingMainContentDiv.setAttribute('id', 'trackingMainContentDiv');

        panelHook.appendChild(trackingMainContentDiv);

        trackingPanelDiv.appendChild(panelHook);
    }


    function OpenWizardPanelAfterTrackingBarClosed() {

        $(".footerLinks").show();
        chrome.extension.sendMessage({ MessageType: "RestyleBar", ZIndexValue: '100001' });
        $("#WizardPanel").slideToggle("slow");

    }


    // ab check for email tracking
    function AbCheckForEmailTracking() {

        chrome.extension.sendMessage({
            MessageType: "BARSETUP"
        }, function (response) {

            //if the user is not logged in, display login message in bar
            if (response.LoggedIn != null && response.LoggedIn == false) {
                BarDisplayLoginState(); return;
            }

            var abRules = JSON.parse(response.AbRules);
            if (abRules.EnableGmailEmailTracking + "" == "1") {
                $('#trackingLoader').css('display', 'block');
            }

            //get the licencing flags for ebsta growth trials.
            currentlyInGrowthTrial = response.EbstaGrowthTrial;
            daysRemainingOnTrial = response.GrowthTrialDaysRemaining;

            if (currentlyInGrowthTrial + "" === "true") {
                renderTrialCountdown = true;
            } else {
                renderTrialCountdown = false;
            }

            CheckTrialMessaging();

          
            return true;
        }
        );
    }

    function ProccessCheckContactHasTrackingEvents(hasTrackingEvents) {
        $('#trackingLoader').removeClass('hasTrackingEvents');
        if (hasTrackingEvents || hasTrackingEvents + '' == 'true') {
            $('#trackingLoader').addClass('hasTrackingEvents');
        }
    }

    function BarDisplayLoginState() {
        //Show login message + btn
        $(".noLoginPanel").show();
        //Hide other bar elements
        $(".resultsPanel").hide();
        $(".noResultsPanel").hide();
        $(".viewMoreSpan").hide();
        $(".viewMoreSpanNoResult").hide();
        //$(".viewMoreSpanNoResult").removeClass("ebsta-clickprompt");
        $("#trackingLoaderContainer").hide();
        //Setup click event for login btn
        SetupLoginClickEvent();
    }

    function SetupLoginClickEvent() {
        $('.barLoginBtn').on("click", function () {
            LoginMeIn();
        });
    }

    function LoginMeIn() {
        switch (whatProductApp) {
            //If what product app is valid:
            case "10":
            case "20":
            case "30":
                //Call func. to send msg to background - using tabs api to update tab
                OpenSFAuthTab("https://console.ebsta.com/Salesforce/RequestAccess.aspx?ver=" + whatProductApp);
                break;
            default:
                //If what product app is not a valid val. - use default 10
                //Call func. to send msg to background - using tabs api to update tab
                OpenSFAuthTab("https://console.ebsta.com/Salesforce/RequestAccess.aspx?ver=10");
                break;
        }

    }

    function OpenSFAuthTab(uri, newTab) {
        //Send message to bg and use chrome tabs api to open a new tab.
        chrome.extension.sendMessage({ MessageType: "OpenSFAuthTabURI", URI: uri });
}

