﻿/*

    ContentGet.js - processing rules, parsing data

*/

// 

var rules = null;

/**
 * Object containing inner objects which help handle the properties within a LayoutManager object
 * @param {} layoutManagerObject 
 * @returns {} 
 */
function LayoutManager(layoutManagerObject) {

    /**
     * Representing the CompanySettings property on a LayoutManager object
     * @param {} layoutManager 
     * @returns {} 
     */
    function CompanySettings(layoutManager) {
        var disableLinkedInProfileParsing = false;

        //Search through the layoutmanager object to find the values we need
        if (layoutManager && layoutManager.d) {
            if (layoutManager.d.CompanySettings) {
                var disableLinkedInProfileParsingValue = layoutManager.d.CompanySettings.DisableLinkedInProfileParsing;
                disableLinkedInProfileParsing = disableLinkedInProfileParsingValue != null ? disableLinkedInProfileParsingValue : false;
            }
        }
        return {
            /**
             * Bool to represent if we're stopping the parsing of profiles on LinkedIn
             * @returns {} 
             */
            ShouldDisableLinkedInProfileParsing: function () {
                return disableLinkedInProfileParsing;
            }
        }
    }

    var companySettings = new CompanySettings(layoutManagerObject);

    return {
        /**
         * Returns a CompanySettings object
         */
        CompanySettings: companySettings
    }
}


chrome.extension.sendMessage({
    action: "GetRules"
}, function(response) {

    rules = response.Rules;
    var debugMode = response.DebugMode;

    var cvContent = "";
    if (document.URL.indexOf(".linkedin.") > -1) {
        if (response.LayoutManager) {
            var layoutManager = new LayoutManager(response.LayoutManager);
            if (layoutManager.CompanySettings) {
                if (layoutManager.CompanySettings.ShouldDisableLinkedInProfileParsing() === false) {
                    cvContent = ParseContent();
                }
            } else {
                cvContent = ParseContent();
            }
        }
    } else {
        cvContent = ParseContent();
    }

    chrome.extension.sendMessage({
        action: "content",
        host: document.location.hostname,
        content: cvContent,
        ParseInformation: ParsePageDetails2(debugMode)
    }, function(response) {

    });
});


function noNumbers(string) {
    var returnVal = true;    
    for (var i = 0; i < 10; i++)
    {
        if (string.indexOf(i) != -1)
        {
            returnVal = false;
        }        
    }
    return returnVal;
}

function ParseContent() {
    var url = '' + document.URL.toLowerCase();
    var node = document.body;
    var parsedRules = (JSON.parse(rules).d);
    var lookingForSplits = false;
    var html = "";

    if (url.indexOf(".monster.") > -1) {
        if (url.indexOf("jcm/resumesearch/searchresults.aspx") > -1) {

            html = GetHtmlByXpath('//*[@id="divResumeDataBody"]/div[4]');
            if (html.length != 0) {
                return html;
            }

        }
    }


    try {
        for (var i = 0; i < parsedRules.length; i++) {
            var currentRule = parsedRules[i];

            if (url.indexOf(currentRule.URL.toLowerCase()) > -1) {
                if (currentRule.Field == "Main Text") {
                    var gotCvWithRule = false;
                    if (html.length == 0) { //if we don't have a CV yet
                        html += GetHtmlByXpath(currentRule.Xpath); // get a CV
                        if (html.length > 0) { // if we found a CV with current rule
                            gotCvWithRule = true;
                        }
                    }
                    else { // else, we already have a CV
                        if (lookingForSplits == true) { // if we are looking a split CV
                            var htmlLen1 = html.length;
                            if (currentRule.Split) { // if we have a rule for a split CV
                                html += GetHtmlByXpath(currentRule.Xpath); // get more CV
                            } // otherwise skip
                            if (html.length > htmlLen1) { // if we found a CV with current rule
                                gotCvWithRule = true;
                            }
                        } // otherwise skip
                    }
                    if (gotCvWithRule == true) { // if we found a CV with current rule
                        if (currentRule.Split == 1) {
                            lookingForSplits = true;
                        }
                    }
                }
            }
        }

        if (url.indexOf(".monster.") > -1) {
            if (html.indexOf('<div class="TabInnerWindow CareerSummary" id="divCareerDataBody">') > -1) {
                var split = html.split('<div class="TabInnerWindow CareerSummary" id="divCareerDataBody">');
                if (split[0] != null) {
                    html = split[0];
                }
            }
        }

        //get cv for cw jobs manually.
        if (document.URL.indexOf("recruiter.cwjobs.co.uk") != -1) {
            var resumeSection = document.getElementById("ctl00_cphCentralPanel_winPreviewCV");
            var iframeDocument = resumeSection.contentWindow.document;

            html = iframeDocument.body.innerHTML;
        }

        if (url.indexOf("source.gild.com") > -1) {

            var copyhtml;
            var copyHtml = html;

            //Take top section of profile up to the salary graph
            if (html.indexOf('<div class="estimated-salary">') > -1) {
                var split = html.split('<div class="estimated-salary">');
                if (split[0] != null) {
                    html = split[0];
                }

                //Start from employment history and get all infor up to the project section
                if (copyHtml.indexOf('<div class="section" data-intro="profile_view-employment-history" id="section-employment-history">') > -1) {
                    split = copyHtml.split('<div class="section" data-intro="profile_view-employment-history" id="section-employment-history">');

                    //Find the last section before the projects section and remove it
                    if (split[1].indexOf('<div class="section" data-intro="profile_view-code-projects" data-pdf-hide="" id="section-code-projects">') > -1) {
                        split = split[1].split('<div class="section" data-intro="profile_view-code-projects" data-pdf-hide="" id="section-code-projects">');
                        if (split[0] != null) {
                            html += split[0];
                        }
                    }
                }
            }

        }

        html = FormatParsedCV(html);
        //Strip out clearfix
        var str = html.replace(/clearfix/g, "LSB-CV");
        html = str;

        if (html != '') {
            return html;
        }

    } catch (error) {
    }

 

    return html;

}

//FormatParsedCV tidies up CV before submitting to Bullhorn.
//We do some site specific stuff (removing images, adjusting CSS)

//This function could be known as MakeDanRemediosHappy()
function FormatParsedCV(html) {



    if (document.domain == "employer.dice.com") {
        var div = document.createElement('div');
        div.innerHTML = html;
        var styles = div.getElementsByTagName('style');
        for (var i = 0; i < styles.length; i++) {
            styles[i].innerHTML = "";
        }

        return div.innerHTML;

    }
    if (document.domain == "www.reed.co.uk") {
        // 
        var nameAndDetails = document.createElement('div');
        nameAndDetails.innerHTML = html;
        nameAndDetails = nameAndDetails.getElementsByClassName('nameAndDetails')[0];



        var profile = document.createElement('div');
        profile.innerHTML = html;
        profile = profile.getElementsByTagName('section')[0];
        if (profile != null) {
            profile.innerHTML = profile.innerHTML.replace("Recently Viewed", "");
            var icons = profile.getElementsByClassName('sectionIcon');
            profile.innerHTML = profile.innerHTML.replace('<i class="sectionIcon"></i>', '');
            profile.innerHTML = profile.innerHTML.replace(/<br>\s*<br>\s*<br>\s*/g, "<br>");
        }

        var div = document.createElement('div');
        div.innerHTML = html.replace("undefined", "");

        var img = div.getElementsByTagName('img')[0];
        var link = img.getAttribute('src');
        link = link.split("&pageNo=")[0];

        var ret = '<div>';
        if (nameAndDetails != null) {
            ret += nameAndDetails.outerHTML;
            ret += "<br/>";
        }


        if (profile != null) {
            ret += profile.outerHTML;
            ret += '<br/>';
        }
        ret += '<a href="https://' + document.domain + link + '&pageNo=1' + '" target="_blank">View CV Page 1</a>';
        ret += '<br/>';
        ret += '<a href="https://' + document.domain + link + '&pageNo=2' + '" target="_blank">View CV Page 2</a>';
        ret += '<br/>';
        ret += '<a href="https://' + document.domain + link + '&pageNo=3' + '" target="_blank">View CV Page 3</a>';
        ret += '</div>';
        return ret;
    }
    if (document.domain == "www.entelo.com") {
        // 
        var div = document.createElement('div');
        div.innerHTML = html;
        var similar = div.getElementsByClassName("similar-candidates")[0];
        similar.setAttribute("style", "display: none;");
        var fav = div.getElementsByClassName("fav")[0];
        fav.setAttribute("style", "display: none;");
        return div.innerHTML;
    }
    if (document.domain == "www.discoverydb.com") {
        var page = document.createElement('div');
        page.innerHTML = html;
        var headers = page.getElementsByClassName('name');
        for (var i = 0; i < headers.length; i++) {
            headers[i].setAttribute("style", "font-weight: bold; margin-top: 10px");
        }

        return page.innerHTML;


    }
    if (document.domain == "hiring.monster.ca") {
        // 
    }

    html = StripUnwantedElements(html);

    // 
    return html;

}

//Removes images and alters links in HTML before sending to Bullhorn.
//Consider expanding to remove stylesheets etc
function StripUnwantedElements(html) {
    //alert('stripping unwanted elements...');
    //alert(html);

    var page = document.createElement('div');
    page.innerHTML = html;

    //alert('html...' + html);

    //Strip out images
    var images = page.getElementsByTagName('img');
    if (images.length) {
        for (var i = 0; i < images.length; i++) {
            images[i].setAttribute("style", "display: none;");
            //alert(images[i]);
        }
    }

    //Replace local links with hard links
    var links = page.getElementsByTagName('a');
    if (links.length) {
        for (var i = 0; i < links.length; i++) {
            var thisLinkURL = links[i].getAttribute("href");
            if (thisLinkURL) {
                if (thisLinkURL.indexOf("http") != 0) {
                    if (thisLinkURL.indexOf("/") == 0) {
                        links[i].setAttribute("href", window.location.protocol + "//" + document.domain + thisLinkURL);
                    }
                    if (thisLinkURL.indexOf("../") == 0) {
                        thisLinkURL = thisLinkURL.substring(2, thisLinkURL.length - 1);
                        links[i].setAttribute("href", window.location.protocol + "//" + document.domain + thisLinkURL);
                    }
                    if (thisLinkURL.indexOf("x_") == 0) {
                        links[i].setAttribute("href", window.location.protocol + "//" + document.domain + "/" + thisLinkURL);
                    }
                }

                if (document.URL.indexOf('.doccafe.') > -1) {
                    if (links[i].href.indexOf('cv_view') == -1) {
                        links[i].setAttribute('style', 'display: none');
                    }
                }
                links[i].setAttribute("target", "_blank");
            }
        }
    }

    //Strip out inputs
    var inputs = page.getElementsByTagName('input');
    if (inputs.length) {
        for (var i = 0; i < inputs.length; i++) {
            inputs[i].setAttribute("style", "display: none");
        }

    }



    return page.innerHTML;
}

//This is the main 'Get Info from page' function.phone
//It relies on the RulesEngine SQL table. 
function ParsePageDetails2(debugMode) {

    var jsonPerson = {};
    var parsedRules = (JSON.parse(rules).d);    
    var fullname = "";
    var firstname = "";
    var surname = "";
    var email = "";
    var otherEmail = "";
    var mobile = "";
    var homePhone = "";
    var workPhone = "";
    var phone = "";
    var fax = "";
    var employer = "";
    var jobtitle = "";
    var titleCompany = "";
    var postCode = "";
    var city = "";
    var state = "";
    var address = "";
    var country = "";
    var cityCountry = "";
    var skills = "";
    var webAddress = "";
    var publicProfileURL = "";
    var twitter = "";
    var description = "";
    var source = document.domain.toString();
    var street = "";
    var dunsNumber = "";

    //Groupon fields
    var dealDescription = "",
        dealPrice = "",
        dealSaving = "",
        dealDiscount = "",
        dealQuantity = "";

    //LightIron fields
        filmAka = "",
        filmLastRelease = "",
        filmNextRelease = "",
        filmPersonGender = "",
        filmProjectName = "",
        filmProjectStatus = "",
        filmProjectType = "",
        filmProjectGenre = "",
        filmProjectStartDate = "",
        filmProjectWrapDate = "",
        filmProjectDiStartDate = "",
        filmProjectAirDate = "",
        filmProjectLocations = "",
        filmProjectLogLine = "",
        filmProjectBudget = "",
        filmProjectDevNotes = "",
        filmProjectReleaseDate = "";
        filmDirector = "";
        filmProductionCompany = "";
        filmWriters = "";
        filmCast = "";
        filmDistributor = "";
        filmProducers = "";
        filmEditors = "";
        filmProductionManagers = "";
      
    //channel navigator fields...
    var legalName = "",
        cnMailingAddress = "",
        linkedInUrl = "",
        accountSource = "",
        accountSourceDetails = "",
        stateIncorporated = "",
        startYear = "",
        fiscalEndYear = "",
        ownership = "",
        serviceOverview = "",
        cnEmployees = "",
        billableEmployees = "",
        annualRevenue = "",
        otherServices = "",
        runRate = "",
        partnersAndCertifications = "",
        otherCustomerFocus = "",
        activeCustomers = "",
        customerBaseSize = "",
        department = "",
        type = "",
        cnAddress1 = "",
        cnAddress2 = "",
        sst = "",
        solutions = "",
        services = "",
        technologies = "",
        awardsRankings = "",
        awards = "",
        rankings = "",
        fullBillingAddress = "",
        verticalMarkets = "",
        vendorPartnerships = "",
        revenueRange = "",
        communicationNotes = "";    

    var successes = 0;
    
    //This is the start of the loop
    //Loop for each rule in the table
    for (var i = 0; i < parsedRules.length; i++) {
        var currentRule = parsedRules[i];        

        var urlText = '' + document.URL.toLowerCase();

        //Any rules that start with CustomGmail are a custom rule that we'd liked to use 
        //to overwrite any values that we may already have found
        var isThisACustomGmail = false;
        if (currentRule.URL && urlText.toLowerCase().indexOf("mail.google.com") !== -1) {
            //Skip over any rules we have for gmail in rules engine as we don't want to us them currently
            if (currentRule.URL.toLowerCase().indexOf("mail.google.com") !== -1) continue;
            isThisACustomGmail = (currentRule.URL.toLowerCase().indexOf("customgmail") >= 0);
        }

        //If the current rule is for our current URL, then proceed:        
        if (urlText.indexOf(currentRule.URL.toLowerCase()) > -1 || isThisACustomGmail) {
            /* Reinstate this for debugging */
            //            if (GetValueByXpath(currentRule.Xpath).length > 0) {
            //                 
            //            }


            //From here, we're just processing each type of rule differently
            //Typically, we grab the string via Xpath, then split/format it so it'll display correctly
            //Also we'll validate that what we've got is what we want via regex
            // 
            if (currentRule.Field == "Full Name" && fullname.length == 0) {
                if (GetValueByXpath(currentRule) != "") {
                    if (GetValueByXpath(currentRule).indexOf("@") > -1) {
                        fullname = GetValueByXpath(currentRule);
                        continue;
                    }
                    else {
                        fullname = GetValueByXpath(currentRule);
                        
                        if (document.URL.indexOf("jobsite.co.uk/cvs-bin/cvdisplay_text?") != -1) {
                            fullname = TidyPersonNameJobSite(fullname);
                        }
                        
                        fullname = EbstaNameSanitizer.TidyName(fullname);

                        if (source.indexOf(".entelo.com") > -1) {
                            fullname = fullname.split("Hide")[0];
                        }

                        //do some bits for cw jobs and totaljobs
                        if (document.URL.indexOf("cwjobs.co.uk") != -1 || document.URL.indexOf("recruiter.totaljobs.com/Recruitment/CandidateSearch") != -1) {
                            var split = fullname.split(":");
                            if (split.length > 0) {
                                fullname = split[0].trim();
                            }
                        }

                        if (document.URL.indexOf("boardex.com/director/profile") != -1) {
                            fullname = TidyPersonNameBoardEx(fullname);
                        }
                        if (document.URL.indexOf("talentsearch.seek.com.au") > -1) {
                            if (fullname.length > 1) {
                                if (fullname.substr(fullname.length - 1) == ".") {
                                    fullname = fullname.substring(0, fullname.length - 1);
                                }
                            }
                        }
                        
                        if (currentRule.Split == true) {
                            if (fullname.indexOf('<') == -1 && fullname.indexOf('>') == -1) {
                                //Splitting by spaces
                                var names = SplitString(fullname, " ");
                                var newNames = new Array();
                                var x = 0;
                                for (var i = 0; i < names.length; i++) {
                                    if (names[i] != "") {
                                        newNames[x] = names[i];
                                        x++;
                                    }
                                }
                                names = newNames;
                                if (names.length == 2) {
                                    firstname = names[0];
                                    surname = names[1];
                                }
                                else {
                                    firstname = names[0];
                                    surname = fullname.replace(names[0] + " ", "");
                                }
                                if (names[0] == "Dr." ^ names[0] == "Dr" ^ names[0] == "Mr." ^ names[0] == "Mr" ^ names[0] == "Mrs." ^ names[0] == "Ms") //Expand with an array of titles...
                                {
                                    if (names.length == 3) {
                                        firstname = names[1];
                                        surname = names[2];
                                    }
                                    else {
                                        firstname = names[1];
                                        surname = fullname.replace(names[0] + " ", "");
                                        surname = fullname.replace(names[1] + " ", "");
                                    }

                                }
                                continue;
                            }
                        }
                    }
                }
            }
            if (currentRule.Field == "First Name" && firstname.length == 0) {
                if (GetValueByXpath(currentRule) != "" && currentRule.Split == false) {
                    var s = GetValueByXpath(currentRule);
                    //We're checking here to make sure that there are no digits
                    var getDigit = /\d/;
                    if (s.indexOf("@") > -1) {
                        firstname = s;
                        successes++;
                        continue;
                    }
                    else {
                        if (s.search(getDigit, s) == -1) {
                            firstname = GetValueByXpath(currentRule);
                            successes++;
                            continue;
                        }
                    }
                }
            }

            if (currentRule.Field == "Surname" && surname.length == 0) {
                if (GetValueByXpath(currentRule) != "" && currentRule.Split == false) {
                    var s = GetValueByXpath(currentRule);
                    //We're checking here to make sure that there are no digits
                    var getDigit = /\d/;
                    if (s.indexOf("@") > -1) {
                        surname = s;
                        successes++;
                        continue;
                    }
                    else {
                        if (s.search(getDigit, s) == -1) {
                            surname = GetValueByXpath(currentRule);
                            successes++;
                            continue;
                        }
                    }
                }
            }

            //Title/Company can be split a lot of different ways - e.g. by comma, by hyphen, by 'at'...
            if (currentRule.Field == "Title+Company" && titleCompany.length == 0) {
                if (GetValueByXpath(currentRule) != "") {
                    titleCompany = GetValueByXpath(currentRule);

                    if (source.indexOf("entelo.com") > -1) {
                        titleCompany = titleCompany.split(" - ")[0];
                    }
                    if (source.indexOf("doximity") > -1) {
                        var _split = titleCompany.split(",");
                        for (i = 0; i < _split.length; i++) {

                            if (_split.length != 1) {
                                titleCompany = _split[i];
                                employer = _split[i];
                            }

                        }
                        jobtitle = _split[0];
                    }
                    if (urlText.indexOf("xing.com/xtm/profile") > -1) {
                        if (titleCompany.indexOf(",") > -1) {
                            var _split = titleCompany.split(",");
                            if (_split.length == 2) {
                                jobtitle = _split[0];
                                employer = _split[1];
                            }
                            else {
                                jobtitle = _split[0];
                            }
                        }
                    }

                    if (!(source.indexOf("facebook.com") > -1 && titleCompany.indexOf("Studie") > -1)) {
                        if (currentRule.Split == true) {
                            if (source.indexOf("oilcareers.") > -1) {
                                if (titleCompany.indexOf(",") > -1) {
                                    var _split = SplitString(titleCompany, ",");
                                    if (_split.length > 3) {

                                        if (jobtitle.length == 0) {
                                            jobtitle = _split[0];
                                        }
                                        employer = _split[2];
                                    }
                                    else {

                                        if (jobtitle.length == 0) {
                                            jobtitle = _split[0];
                                        }
                                    }
                                }
                                else {

                                    if (jobtitle.length == 0) {
                                        jobtitle = titleCompany;
                                    }
                                }
                            }

                            else {
                                //Splitting on the word 'at'
                                var _split = SplitString(titleCompany, " at ");

                                if (_split.length == 2) {

                                    if (jobtitle.length == 0) {
                                        jobtitle = _split[0];
                                    }
                                    employer = _split[1];
                                }
                                if (_split.length == 1)
                                    //Then the above didn't work, so try: 
                                {
                                    _split = SplitString(titleCompany, ",");
                                    if (_split.length == 2) {
                                        if (jobtitle.length == 0) {
                                            jobtitle = _split[0];
                                        }
                                        employer = _split[1];
                                    }
                                    if (_split.length > 2) {

                                        if (jobtitle.length == 0) {
                                            jobtitle = _split[0];
                                        }
                                        employer = _split.join().replace(jobtitle, "");
                                        employer = employer.replace(",", "");

                                    }

                                    if (_split.length === 1) {
                                        _split = SplitString(titleCompany, "-");

                                        if (_split.length == 2) {
                                            if (jobtitle.length == 0) {
                                                jobtitle = _split[0];
                                            }
                                            employer = _split[1];
                                        }

                                        if (_split.length > 2) {

                                            if (jobtitle.length == 0) {
                                                jobtitle = _split[0];
                                            }
                                            employer = _split.join().replace(jobtitle, "");
                                            employer = employer.replace(",", "");

                                        }


                                    }

                                }
                            }

                        }
                        continue;
                    }
                }
            }
            if (currentRule.Field == "Email" && email.length == 0) {
                var s = GetValueByXpath(currentRule);
                if (s) {
                    if (s != "" && currentRule.Split == false) {
                        //If it contains an @ symbol then we know it's good

                        if (s.indexOf('@') > -1 && s.indexOf('.') > -1) {
                            email = s;
                            email = email.replace("Email:", "");
                            successes++;
                            if (source.indexOf('facebook.com') > -1 && email.indexOf('@face') > -1) {
                                var _split = email.split('@');
                                email = _split[0] + "@facebook.com";
                            }
                            continue;
                        }
                    }
                    //Right now this is a special case just for entelo
                    //Could potentially be expanded into any email where email is embedded into a tag
                    if (s === "" && source.indexOf("entelo.com") > -1) {
                        s = GetHtmlByXpath(currentRule.Xpath);
                        if (s != "") {
                            var div = document.createElement('div');
                            div.innerHTML = s;
                            email = div.getElementsByTagName('input')[0].getAttribute('value');
                        }
                        continue;

                    }
                }
            }
            if (currentRule.Field === "OtherEmail" && otherEmail.length === 0) {
                otherEmail = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "Mobile" && mobile.length == 0) {

                var s = GetValueByXpath(currentRule);

                if (s.toLowerCase().indexOf("work") != -1) {
                    workPhone = TidyPhone(s);
                }
                else if (s.indexOf("(home)") != -1) {
                    homePhone = TidyPhone(s);
                }
                else {

                    if (s != "" && currentRule.Split == false) {
                        if (s.indexOf('<') == -1 && s.indexOf('>') == -1) {
                            //Here we're making sure there's some numbers
                            var getDigit = /\d/;
                            if (s.search(getDigit, s) > -1) {
                                if (source.indexOf("oilcareers") == -1) {
                                    mobile = TidyPhone(s);
                                }
                                if (source.indexOf("oilcareers") > -1) {
                                    if (s.indexOf("Mobile Tel:") > -1) {
                                        mobile = TidyPhone(s);
                                    }
                                    if (s.indexOf("Phone:") > -1) {
                                        homePhone = TidyPhone(s);
                                    }
                                    if (source.indexOf("xing.com") > -1) {
                                        var check = GetValueByXpath("//*[@id='lightbox']/div[2]/output/div/dl/dt[1]");
                                        if (check == "Cell phone" | check == "Cell Phone" | check == "cell phone" | check == "Mobile" | check == "mobile") {
                                            mobile = TidyPhone(s);
                                        }
                                    }
                                }
                                continue;
                            }
                        }
                    }
                }


            }
            if (currentRule.Field == "HomePhone" && homePhone.length == 0) {
                var s = GetValueByXpath(currentRule.Xpath);


                var s = GetValueByXpath(currentRule);
                //// 

                if (s != "" && currentRule.Split == false) {
                    if (s.indexOf('<') == -1 && s.indexOf('>') == -1) {
                        //Here we're making sure there's some numbers
                        var getDigit = /\d/;
                        if (s.search(getDigit, s) > -1) {
                            if (source.indexOf("oilcareers") == -1) {
                                homePhone = TidyPhone(s);
                            }
                            if (source.indexOf("oilcareers") > -1) {
                                if (s.indexOf("Phone:") > -1) {
                                    homePhone = TidyPhone(s);
                                }
                                if (s.indexOf("Mobile Tel:") > -1) {
                                    mobile = TidyPhone(s);
                                }
                            }
                            continue;
                        }
                    }
                }
            }
            if (currentRule.Field == "WorkPhone" && workPhone.length == 0) {
                    var s = GetValueByXpath(currentRule);
                    if (s != "" && currentRule.Split == false) {
                        if (s.indexOf('<') == -1 && s.indexOf('>') == -1) {
                            //Here we're making sure there's some numbers
                            var getDigit = /\d/;
                            if (s.search(getDigit, s) > -1) {
                                workPhone = TidyPhone(s);
                                successes++;
                                continue;
                            }
                        }
                    }
            }
            if (currentRule.Field == "Phone" && phone.length == 0) {
                var tel = GetValueByXpath(currentRule);
                if (tel != "" && currentRule.Split == false) {
                    if (tel.indexOf('<') == -1 && tel.indexOf('>') == -1) {
                        var gd = /\d/;
                        if (tel.search(gd, tel) > -1) {
                            phone = TidyPhone(tel);
                            successes++;
                            continue;
                        }
                    }
                }
            }
            if (currentRule.Field == "Fax" && fax.length == 0) {
                fax = GetValueByXpath(currentRule);
                fax = TidyPhone(fax);
            }

            jobtitle = jobtitle.trim();

            if (currentRule.Field == "Job Title" && jobtitle.length == 0) {
                if (GetValueByXpath(currentRule) != "" && currentRule.Split == false) {

                    jobtitle = GetValueByXpath(currentRule);

                    jobtitle = jobtitle.replace("(Current)", "");

                    if (jobtitle.indexOf(":") > -1) {
                        jobtitle = jobtitle.split(':')[1];
                    }
                    if (jobtitle.indexOf(",") > -1 && document.URL.indexOf("doximity") != -1) {
                        jobtitle = jobtitle.split(',')[0];
                    }

                    if (jobtitle.lastIndexOf(",") == (jobtitle.length - 1)) {
                        jobtitle = jobtitle.substring(0, jobtitle.lastIndexOf(","));
                    }
                    continue;
                }
            }
            if (parsedRules[i].Field == "Employer" && employer.length == 0) {
                
                if (GetValueByXpath(currentRule) != "" && currentRule.Split == false) {
                    employer = GetValueByXpath(currentRule);
                    
                    if (employer.indexOf(":") == 0) {
                        employer = employer.substring(1);
                    }
                    if (employer.indexOf("Not Specified") > -1) {
                        employer = "";
                    }
                    /* Split Experteer employer string to only contain employer. */
                    if (document.URL.indexOf('https://www.experteer.co.uk/recruiting/candidate/') > -1) {
                        if (employer.indexOf(',') > -1) {
                            employer = employer.split(',')[0];
                        }
                    }

                    if (document.URL.indexOf("dice.com") > -1) {
                        var diceChop = employer.split(/\sat\s/);
                        if (diceChop.length > 1) {
                            if (diceChop[1] != null || diceChop[1] != "") {
                                employer = diceChop[1];
                            }
                        }
                    }
                    if (document.URL.indexOf("boardex.com") > -1) {
                        employer = TidyCompanyNameBoardEx(employer);
                    }
                    continue;
                }
            }

            //Street 
            if (currentRule.Field == "Street" && street.length == 0) {
                street = GetValueByXpath(currentRule);

                if (document.URL.indexOf(".boardex.com") > -1 && address.length > 0)
                {
                    street = street + "\n" + address;
                }

                if (document.URL.indexOf(".termscout.com/") > -1) {
                    //if address over two lines - state country bottom line
                    //street addr first line
                    var streetLineSplit = street.split("\n");
                    street = streetLineSplit[0];
                }

                if (document.URL.indexOf(".yelp.") > -1) {

                    var streetLineSplit = street.split("\n"); // splits the street by carriage returns

                    // looks for this property ie <meta content="US" itemprop="addressCountry"> so we can determine the country for us
                    var profileLocationObject = document.querySelector('[itemprop="addressCountry"]');
                    var countryAttribute = "";

                    // Another check for US addresses if the address doesnot contain address.
                    if (typeof profileLocationObject != "undefined" && profileLocationObject != null)
                    {
                        countryAttribute = profileLocationObject.getAttribute("content");
                    }

                    // checks if the street address is in the United States
                    if (countryAttribute.toUpperCase() === "US" || country.toUpperCase() === "UNITED STATES") {
                        if (streetLineSplit.length === 1) {
                            street = streetLineSplit[0];
                        }
                    }
                    // checks if the uk address has 3 elements, the first two elements are street details
                    else {
                        if (streetLineSplit.length === 2) {
                            street = streetLineSplit[0] + "\n" + streetLineSplit[1];
                        }
                        else {
                            street = streetLineSplit[0];
                        }
                    }
                }
            }

            if (currentRule.Field == "State" && state.length == 0) {
                state = GetValueByXpath(currentRule);

                if (state.length > 0) {
                    //trim down so we dont have to worry about this in xpath
                    state = state.trim();
                    //ok we got a US shorthand for state - translate to full state name
                    if (state.length === 2) {
                        state = TranslateState(state);
                    }
                }

                if (document.URL.indexOf(".yelp.") != -1)
                {
                    state = TranslateState(state);
                }

                if (document.URL.toLowerCase().indexOf('subscriber.hoovers.com') != -1 || isThisACustomGmail) {
                    //console.time("translatingState");
                    state = TranslateState(state);
                    //console.timeEnd("translatingState");
                }

                if (document.URL.indexOf(".termscout.com/") !== -1) {

                    //states are in caps, need titlecase
                    if (state.length > 0) {
                        state = ToTitleCase(state);
                    }
                }

                //If we don't have a country buy have a state then we should at least try and find that country
                if (country.length === 0 && state.length > 0) {
                    country = getCountryFromState(state);
                }
            }

            if (currentRule.Field == "Postcode" && postCode.length == 0) {


                if (GetValueByXpath(currentRule) != "" && currentRule.Split == false) {
                    var s = GetValueByXpath(currentRule);                    
                    var getDigit = /\d/;
                    if (s.search(getDigit, s) > -1) {
                        postCode = s;
                       
                        continue;
                    }         
                }
            }

            if (currentRule.Field == "City" && city.length == 0) {
                if (GetValueByXpath(currentRule) != "" && currentRule.Split == false) {
                    var s = GetValueByXpath(currentRule);
                    //Split out any extraneous words
                    var split = s.split(',');
                    s = split[0];

                    if (document.domain.indexOf("source.gild.com") > -1) {
                        if (split.length > 1) {
                            state = split[1];
                        }

                        if (split.length > 2) {
                            country = split[2];
                        }
                    }
                
                    if (document.domain.indexOf("zoominfo") > -1) {
                        if (split.length > 0) {
                            city = split[0];
                        }
                        if (city != undefined) {
                            citySplit = city.split(" ");
                        }
                        if (citySplit != undefined) {
                            if (citySplit.length > 3) {
                                city = citySplit[0];
                                postCode = citySplit[1] + " " + citySplit[2];
                            }
                        }

                        var pc = "";
                        if (split[1] != undefined) {
                            if (split[1]) {
                                pc += split[1].toString();
                            }
                        }
                        pc = pc.trim();
                        var sp = pc.split(" ");

                        if (sp != undefined)
                            {
                            if (sp.length > 1) {
                                if (sp[0] != null && sp[0] != "" && sp[0] != undefined) {
                                    if (state.length == 0) {
                                        state = sp[0];
                                        //this section deals with US postcodes with states with 2 words e.g New York 98765
                                        if (sp.length > 2) {
                                            if (sp[2].length > 3) {
                                                postCode = sp[2];
                                                state = sp[0] + " " + sp[1];
                                            }
                                        }
                                        //this section will fix UK addresses with tw word county and uk postcode , e.g. West Yorkshire WY1 9UT
                                        if (sp.length > 3) {
                                            state = sp[0] + " " + sp[1];
                                            postCode = sp[2] + " " + sp[3];
                                        }
                                    }
                                }
                                if (postCode.length < 1) {
                                    if (sp[1] != null && sp[1] != "") {
                                        postCode = sp[1];
                                        if (sp.length > 2) {
                                            var length = sp.length;
                                            var postCode1 = length - 2;
                                            var posCode2 = length - 1;
                                            postCode = sp[postCode1] + " " + sp[posCode2];
                                        }
                                    }
                                }
                            }
                        }
                        //postCode = pc.substring(pc.indexOf(" ") + 1, pc.length);
                    }                    //Break down address string for Postcode and City params.
                    if (document.domain === 'www.careerbuilder.co.uk') {
                        var locationParts = s.split("-");
                        city = locationParts[2];
                        postCode = locationParts[locationParts.length - 1];
                    }

                    if (document.domain == 'www.mdjobsite.com') {
                        var pcode = s.split('-')[2];
                        s = s.split('-')[0];
                        postCode = pcode;
                    }
                    if (document.domain.indexOf("data.com") > -1) {
                        if (split[1] != "" && split[1] != null) {
                            var s2 = split[1];
                            var split2 = s2.split(" ");
                            if (split2[split2.length - 1] != "" && split2[split2.length - 1] != null) {
                                postCode = split2[split2.length - 1];
                            }
                        }
                    }
                    if (document.URL.indexOf("workopolis.com/CandidateSourcing") > -1) {
                        if (split[0] != "" && split[0] != null) {
                            city = split[0];
                        }
                        if (split[split.length - 1] != "" && split[split.length - 1] != null) {
                            postCode = split[split.length - 1];
                        }
                        if (split.length == 3) {
                            if (state.length == 0) {
                                state = split[1];
                            }
                        }
                    }
                    if (document.URL.indexOf("nursingjobs.us") > -1) {
                        city = s;
                        var split2 = split[1].split(" ");
                        for (var i = 0; i < (split2.length - 1) ; i++) {
                            state += split2[i] + " ";
                        }
                        postCode = split2[split2.length - 1];
                    }

                    if (document.URL.indexOf("boardex.com") > -1) {
                        city = s;
                        if (split.length > 1) {
                            state = split[1];
                        }
                    }

                    // Taking in consideration of the united kingdom and the united states yelp profiles
                    if (document.URL.indexOf("yelp.co.uk/user_details") > -1 || document.URL.indexOf("yelp.com/user_details") > -1) {
                        if (split.length > 1) {
                            city = split[0]; 
                            country = split[1];
                        }

                        if (split.length > 2) {
                            state = split[1];
                            country = split[2];
                        }
                    }

                    //If it has a word in it
                    var getWord = /\w+/;
                    if (s.search(getWord, s) > -1) {
                        //And no digits
                        var getDigit = /\d/;

                        if (s.search(getDigit, s) == -1) {
                            //// 
                            city = s;
                            successes++;
                        }
                        //Dirty, I know!
                        if (document.URL.indexOf("members.ocpinfo.com/search/") != -1) {
                            if (city == surname) {
                                city = '';
                            }
                        }
                        continue;
                    }
                }

            }
            if (currentRule.Field == "Country" && country.length == 0) {
                if (GetValueByXpath(currentRule) != "" && currentRule.Split == false) {
                    var s = GetValueByXpath(currentRule);

                    if (document.URL.indexOf("crunchbase") != -1)
                    {
                        if (noNumbers(s)) {
                            if (s.indexOf) {
                                var getWord = /\w+/;
                                if (s.search(getWord, s) > -1) {
                                    country = s;
                                    if (country.indexOf(' (Map)') > -1) {
                                        country = country.replace(" (Map)", "");
                                    }
                                    successes++;
                                    continue;
                                }
                            }
                        }
                    }

                    else {
                        if (s.indexOf) {
                            var getWord = /\w+/;
                            if (s.search(getWord, s) > -1) {
                                country = s;
                                if (country.indexOf(' (Map)') > -1) {
                                    country = country.replace(" (Map)", "");
                                }
                                successes++;
                                continue;
                            }
                        }
                    }
                }

            }
            if (currentRule.Field == "City+Country" && cityCountry.length == 0) {               
                if (GetValueByXpath(currentRule) != "") {
                    cityCountry = GetValueByXpath(currentRule);


                    //for Lead411, we get City, State and Postcode from this rule - lets split them out here
                    if (source.indexOf(".lead411.com") != -1)
                    {                        
                        var cityCountySplit = cityCountry.split(",");                        
                        if (cityCountySplit.length > 1)
                        {                            
                            city = cityCountySplit[0];                            
                            var stateAndPostcode = cityCountySplit[1].trim();                            
                            //now let's get state and postcode                            
                            state = stateAndPostcode.split(" ")[0];                      
                            postCode = stateAndPostcode.split(" ")[1];                            
                        }
                    }
                    else {
                        if (currentRule.Split == true) {
                            //Splitting at a comma
                            var _split = SplitString(cityCountry, ",");
                            if (_split.length == 2) {
                                if (city == "") city = _split[0];
                                if (country == "") country = _split[1];
                                if (country.indexOf(' (Map)') > -1) {
                                    country = country.replace(" (Map)", "");
                                }
                            }

                        }
                    }
                    continue;
                }
            }

            //best use of this rule would be when the state translation is dependant on the country
            //for example the WA shorthand would normally be Washington (US), but we can make a decision in TranslateState()
            //that if country is Australia, then WA shorthand means Western Australia.Therefor we must get the country first.
            if (currentRule.Field == "State+Country" &&
                (state.length === 0 || country.length === 0)) {

                var stateCountryVal = GetValueByXpath(currentRule);
                if (stateCountryVal !== "") {
                    var splitValue = stateCountryVal.split(",");
                    if (splitValue.length > 1) {
                        //ensure that we haven't already gotten country from another rule
                        if (country.length === 0) {
                            country = splitValue[1].trim(); //we expect the country to be the last value
                            //less then 4 characters we can assume is a shorthand like US / AU  / UAE etc
                            if (country.length < 4) {
                                country = TranslateCountry(country);
                            }
                        }

                        //ensure we need to get state
                        if (state.length === 0) {
                            state = splitValue[0].trim(); //we expect state to be the first value
                            //if its shorter then 4 characters its probably a shorthand
                            if (state.length < 4) {
                                state = TranslateStateWithCountry(state, country);
                            }
                        }
                    }
                }
            }

            if (currentRule.Field == "Twitter" && twitter.length == 0) {
                if (source.indexOf("twitter.com") > -1) {
                    twitter = source.toString();
                }
                else {
                    var s = GetHtmlByXpath(currentRule.Xpath);
                    if (s.indexOf("twitter") > -1) {
                        if (s != "" && s != null && currentRule.Split == false) {
                            var temp = document.createElement('div');
                            temp.innerHTML = s;
                            //// 
                            var link = temp.firstChild.getAttribute('href');
                            if (link != null) {
                                twitter = decodeURIComponent(link);
                                if (source.indexOf("linkedin.com") > -1) {

                                    twitter = twitter.substring(0, twitter.indexOf('&urlhash='));
                                }
                                continue;
                            }
                        }
                    }
                }

            }
            if (currentRule.Field == "Skills" && skills.length == 0) {
                if (GetHtmlByXpath(currentRule.Xpath) != "") {
                    var skillsHTML = GetHtmlByXpath(currentRule.Xpath);
                    var skillsDiv = document.createElement('div');
                    skillsDiv.innerHTML = skillsHTML;

                    var skillsArray = new Array();
                    //LinkedIn
                    if (skillsDiv.getElementsByClassName("endorse-item-name-text").length > 0) {

                        var listOfSkills = skillsDiv.getElementsByClassName("endorse-item-name-text");
                        for (var i = 0; i < listOfSkills.length; i++) {
                            if (listOfSkills[i].innerText) {
                                if (listOfSkills[i].innerText != '') {
                                    if (listOfSkills[i].innerText.indexOf("Endorsement") == -1) {

                                        //// 
                                        skillsArray.push(listOfSkills[i].innerText);
                                    }
                                }
                            }
                        }
                        skills = skillsArray.join(",");
                    }
                    //<li> tags
                    if (document.URL.indexOf(".linkedin.") == -1) {
                        if (skillsDiv.getElementsByTagName("li").length > 0) {
                            var listOfSkills = skillsDiv.getElementsByTagName("li");
                            //Max 10 skills
                            for (var i = 0; i < listOfSkills.length; i++) {
                                //// 
                                if (listOfSkills[i].innerText) {
                                    if (listOfSkills[i].innerText != '') {
                                        skillsArray.push(listOfSkills[i].innerText);
                                        if (i >= 10) {
                                            break;
                                        }
                                    }
                                }
                            }
                            skills = skillsArray.join(",");
                        }
                    }
                }
            }
            if (currentRule.Field == "WebAddress" && webAddress.length == 0) {

                var s = GetHtmlByXpath(currentRule.Xpath);
                //

                if (s != "" && s != null && currentRule.Split == false) {
                    var temp = document.createElement('div');
                    temp.innerHTML = s;
                    var link = '';
                    var firstChild = temp.firstChild;

                    if (firstChild.nodeType == 3) {
                        link = s;
                    } else {
                        link = temp.firstChild.getAttribute('href');
                    }
                    
                    if (link != null) {
                        webAddress = decodeURIComponent(link);
                        if (source.indexOf("linkedin.com") > -1) {
                            webAddress = webAddress.substring(0, webAddress.indexOf('&urlhash='));
                        }
                        continue;
                    }
                    webAddress = GetValueByXpath(currentRule);
                }

            }
            if (currentRule.Field == "PublicProfileURL" && publicProfileURL.length == 0) {
                // 
                var pubProfile = GetHtmlByXpath(currentRule.Xpath);
                // 
                // 
                if (pubProfile != "" && pubProfile != null && currentRule.Split == false) {
                    publicProfileURL = pubProfile;
                    // 
                    // 
                }
            }

            if (currentRule.Field == "Address") {
                //If you ever get time, you should totally make a super-smart Address parser
                //For now it's just a pretty dumb backup method
                address = CleanUpHTML(GetValueByXpath(currentRule));
                if (document.URL.indexOf("biospace.com") > -1) {
                    var splits = address.split(",");
                    city = splits[0];
                    var splits2 = splits[1].split(" ");
                    var pc = '';
                    for (var sp = 0; sp < splits2.length; sp++) {
                        var x = splits2[sp];
                        if (x != '' && x.indexOf("\n") < 0) {
                            pc += x;
                        }
                    }
                    postCode = pc;
                }
                if (document.URL.indexOf("careerbuilder.com/JobPoster/Jobs/AppDetails") > -1) {
                    var add = GetValueByXpath(currentRule);
                    var split = add.split("-");
                    if (split[0] != "" && split[0] != null) {
                        country = split[0];
                    }
                    if (split[2] != "" && split[2] != null) {
                        city = split[2];
                    }
                    if (split[3] != "" && split[3] != null) {
                        postCode = split[3];
                    }
                }
                if (document.URL.toLowerCase().indexOf("workopolis.com") > -1) {
                    var add = GetValueByXpath(currentRule);
                    var split = add.split("\n");
                    if (split.length == 1) {
                        var citySplit = split[0].split(",");
                        if (citySplit.length > 1) {
                            city = citySplit[0];
                            state = citySplit[1].trim();
                            country = getCountryFromState(state);
                            if (citySplit.length == 3) {
                                postCode = citySplit[2];
                            }
                        }
                        address = "";
                    } else if (split.length > 1) {
                        street = split[0].replace(",","");
                        var citySplit = split[1].split(",");
                        if (citySplit.length > 2) {
                            city = citySplit[0];
                            state = citySplit[1].trim();
                            country = getCountryFromState(state);
                            postCode = citySplit[2];
                        }
                        if (split.length > 2) {
                            country = split[2];
                        }
                    }
                }
                if (document.URL.indexOf("jigsaw.com/id") > -1) {
                    var address2 = "";
                    var addSplit = address.split("\n");
                    for (var addSpCt = 0; addSpCt < addSplit.length; addSpCt++) {
                        if (addSplit[addSpCt].indexOf(",") == -1) {
                            if (address2.length > 0) {
                                address2 = address2 + ", " + addSplit[addSpCt];
                            }
                            else {
                                address2 = addSplit[addSpCt];
                            }
                        }
                        else {
                            var addSplit2 = addSplit[addSpCt].split(",");
                            if (addSplit2.length == 4) {
                                city = addSplit2[0];
                                state = addSplit2[1];
                                postCode = addSplit2[2];
                                country = addSplit2[3];
                            }
                            else {
                                if (addSplit2.length == 3) {
                                    city = addSplit2[0];
                                    postCode = addSplit2[1];
                                    country = addSplit2[2];
                                }
                                else {
                                    city = addSplit2[0];
                                    state = addSplit2[addSplit2.length - 2];
                                    country = addSplit2[addSplit2.length - 1];
                                }
                            }
                        }
                    }
                    country = country.replace(/\smap/,"");
                    address = address2;
                }
                if (document.URL.indexOf("monster") > -1) {
                    var add2 = GetValueByXpath(currentRule);
                    var split = add2.split(",");
                    if (split.length > 1) {
                        city = split[0];
                        var split2 = split[1].trim().split(" ");
                        if (split2.length > 1) {
                            state = split2[0];
                            postCode = split2[1];
                        }
                    }
                }
                if (document.URL.toLowerCase().indexOf("discoverydb.com/exec/siteimprov") > -1) {

                    var add2 = GetValueByXpath(currentRule);
                    var split = add2.split("\n");

                    if (split.length > 2) {
                        address = split[1];
                        street = split[1];
                        var citySplit = split[2].split(",");
                        if (citySplit.length > 1) {
                            city = citySplit[0];
                            var split2 = citySplit[1].trim().split(" ");
                            if (split2.length > 1) {
                                state = split2[0];
                                postCode = split2[1];
                            }
                        }
                    }
                }
                if (document.URL.indexOf("nursingjobs.us") > -1) {
                    address = GetValueByXpath(currentRule);
                }
                if (document.URL.indexOf("employers.greatinsurancejobs.com/candidate-detail") > -1) {
                    var a = GetValueByXpath(currentRule);
                    var cityStateSplit = a.split(",");
                    if (cityStateSplit.length == 1)
                    {
                        state = cityStateSplit[0];
                    }
                    else if (cityStateSplit.length > 1)
                    {
                        if (cityStateSplit[0] > 0) {
                            postCode = cityStateSplit[0];
                        } else {
                            city = cityStateSplit[0];
                        }
                        state = cityStateSplit[1];
                    }
                    if (state.length > 0) {
                        country = getCountryFromState(state);
                    }
                    
                }
                if (document.URL.indexOf("connect.data.com") > -1)
                {
                    if (street.indexOf(address) == -1)
                    {
                        street = address + "\n" + street;
                    }                    
                    
                }
                if (document.URL.indexOf("universobit.com.ar/empresas") > -1)
                {
                    var addSplit = address.split(",");
                    var splitSize = addSplit.length;
                    if (splitSize == 4)
                    {
                        street = addSplit[0];
                        city = addSplit[1];
                        state = addSplit[2];
                        country = addSplit[3];
                    }
                    else if (splitSize > 4)
                    {
                        street = addSplit[0] + " " + addSplit[1];
                        city = addSplit[2];
                        state = addSplit[3];
                        country = addSplit[4];
                    }
                }
                if (document.URL.indexOf("jobsite.co.uk/client/CVview") > -1) {
                    var addSplit = address.split(",");
                    var splitSize = addSplit.length;
                    if (splitSize) {

                        addSplit[0] = addSplit[0].replace("Address ", "");

                        if (splitSize > 1) {
                            //Last 2 always seem to be Postcode and State/County
                            state = addSplit[splitSize - 2].trim();
                            postCode = addSplit[splitSize - 1].trim();

                            if (addSplit.length == 2) {
                                address = "";
                            } else if (addSplit.length == 3) {
                                address = addSplit[0].trim();
                            } else if (addSplit.length == 4) {
                                address = addSplit[0];
                                address += " " + addSplit[1].trim();
                            } else if (addSplit.length == 5) {
                                address = addSplit[0];
                                address += " " + addSplit[1].trim();
                                address += "\n" + addSplit[2].trim();
                            } else if (addSplit.length == 6) {
                                address = addSplit[0];
                                address += "\n" + addSplit[1].trim();
                                address += "\n" + addSplit[2].trim();
                                city = addSplit[3].trim();
                            }
                        }
                    }
                }

                if (document.URL.indexOf("talentsearch.seek.com.au") > -1) {
                    var addSplit = address.split(",");
                    if (addSplit.length) {
                        if (addSplit.length > 1) {
                            address = "";
                            street = "";
                            city = addSplit[0];
                            state = addSplit[1];
                        }
                    }
                }

                if (document.URL.indexOf("jobsdb.co") > -1) {
                    var addSplit = address.split(",");
                    if (addSplit.length) {
                        address = "";
                        country = addSplit[0];
                        if (addSplit.length > 1) {
                            state = addSplit[1];
                        }
                        if (addSplit.length > 2) {
                            city = addSplit[2];
                        }
                    }
                }
                
                if (city == '') {
                    if (address.indexOf("London") > -1) {
                        city = "London";
                    }
                    if (address.indexOf("New York") > -1) {
                        city = "New York";
                    }
                    if (address.indexOf("Hong Kong") > -1) {
                        city = "Hong Kong";
                    }

                }
                continue;
            }

            if (currentRule.Field == 'IndeedDetails') {             // string from Indeed Candidate profile format: " | {phoneNumber} | {city},{state} "
                var indeedDetails = GetValueByXpath(currentRule);   // If the is no Phon number then address is in it's place.
                if (indeedDetails.length > 0) { //if the rule get something
                    var detailsSplit = indeedDetails.split("|"); // split on "|"
                    var hasWrds = /[A-za-z]/; // regex to identify if string has letters in it
                    if (detailsSplit.length == 3) { //if in the complete format, we know where things are
                        if (workPhone.length < 1) { //if not already got tel number
                            workPhone = detailsSplit[1];
                        }
                        addSplit = detailsSplit[2].split(","); //split city and state
                        if (addSplit.length == 2) {
                            if (state.length < 1) {
                                state = addSplit[1];
                            }
                            if (city.length < 1) {
                                city = addSplit[0];
                            }
                        }
                        else { // put details in city field
                            if (city.length < 1) {
                                city = detailsSplit[dts];
                            }
                        }
                    }
                    else { // if not in the complete format, need to work out what we have
                        for (var dts = 0; dts < detailsSplit.length; dts++) {
                            if (detailsSplit[dts].search(hasWrds, detailsSplit) == -1) { // if string does not have and letters, it's a phone number
                                if (workPhone.length < 1) {
                                    workPhone = detailsSplit[dts];
                                }
                            }
                            else { // otherwise it's the address so we split it up again
                                addSplit = detailsSplit[dts].split(",");
                                if (addSplit.length == 2) {
                                    if (state.length < 1) {
                                        state = addSplit[1];
                                    }
                                    if (city.length < 1) {
                                        city = addSplit[0];
                                    }
                                }
                                else {
                                    if (city.length < 1) {
                                        city = detailsSplit[dts];
                                    }
                                }
                            }
                        }
                    }
                }
            }

            if (currentRule.Field == "description" && description.length == 0) {                                
                description = GetValueByXpath(currentRule);
            }

            //Groupon Fields - when custom fields are included, I presume this will all come through dynamically...
            if (currentRule.Field == "DealDescription") {
                dealDescription = GetValueByXpath(currentRule);
               
            }
            if (currentRule.Field == "DealPrice") {
                dealPrice = GetValueByXpath(currentRule);
                
            }
            if (currentRule.Field == "DealSaving") {
                dealSaving = GetValueByXpath(currentRule);
               
            }
            if (currentRule.Field == "DealDiscount") {
                dealDiscount = GetValueByXpath(currentRule);
            
            }
            if (currentRule.Field == "DealQuantity") {
                dealQuantity = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "Gender" && filmPersonGender.length == 0) {
                filmPersonGender= GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "AKA" && filmAka.length == 0) {
                filmAka = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "LastRelease" && filmLastRelease.length == 0) {
                filmLastRelease = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "NextRelease" && filmNextRelease.length == 0) {
                filmNextRelease = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectName" && filmProjectName.length == 0) {             
                filmProjectName = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectStatus" && filmProjectStatus.length == 0) {
                filmProjectStatus = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectType" && filmProjectType.length == 0) {
                filmProjectType = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectGenre" && filmProjectGenre.length == 0) {
                filmProjectGenre = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "ProjectBudget" && filmProjectBudget.length == 0) {
                filmProjectBudget = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectDevNotes" && filmProjectDevNotes.length == 0) {
                filmProjectDevNotes= GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectDiStartDate" && filmProjectDiStartDate.length == 0) {
                filmProjectDiStartDate= GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectLogLine" && filmProjectLogLine.length == 0) {
                filmProjectLogLine = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectRelease" && filmProjectReleaseDate.length == 0) {
                filmProjectReleaseDate = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectStart" && filmProjectStartDate.length == 0) {
                filmProjectStartDate = GetValueByXpath(currentRule);
            }
            if (currentRule.Field == "ProjectWrapDate" && filmProjectWrapDate.length == 0) {
                filmProjectWrapDate = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Director" && filmDirector.length == 0)
            {
                filmDirector = GetValueByXpath(currentRule);
                //imbd may have multiple values, in incremental spans - if so we need to loop through them and append the values
                if ((document.URL.indexOf("pro-labs.imdb.") != -1) && (document.URL.indexOf("filmmakers") == -1))
                {
                    var oldxpath = currentRule.Xpath;
                    for (var x = 2; x < 5; x++)
                    {
                        var newxpath = currentRule.Xpath
                        newxpath = newxpath.replace('//*[@id="filmmaker_list"]/ul/li[1]/span[2]/span[1]/a/text()', '//*[@id="filmmaker_list"]/ul/li[1]/span[2]/span[' + x + ']/a/text() ');
                        currentRule.Xpath = newxpath;
                        var nextDir = GetValueByXpath(currentRule);                       
                        if (nextDir.length > 1)
                        {                           
                            if (nextDir.indexOf("See all") == -1)
                            {
                                filmDirector = filmDirector + ", " + nextDir;
                            }
                        }
                        currentRule.Xpath = oldxpath;
                    }
                }                
                //if we are on the separate 'filmmakers' page of IMDB, we have to get the multiple values in a slightly differennt way'
                if ((document.URL.indexOf("pro-labs.imdb.") != -1) && (document.URL.indexOf("filmmakers") != -1))
                {
                    var oldxpath = currentRule.Xpath;
                    for (var x = 2; x < 5; x++) {
                        var newxpath = currentRule.Xpath
                        newxpath = newxpath.replace('//*[@id="filmmakers"]/ul[1]/li[1]/span[1]/span/a/text()', '//*[@id="filmmakers"]/ul[1]/li[' + x + ']/span[1]/span/a/text()');
                        currentRule.Xpath = newxpath;
                        var nextDir = GetValueByXpath(currentRule);
                        if (nextDir.length > 1) {
                            if (nextDir.indexOf("See all") == -1) {
                                filmDirector = filmDirector + ", " + nextDir;
                            }
                        }
                        currentRule.Xpath = oldxpath;
                    }

                }

            }

            if (currentRule.Field == "ProductionCompany" && filmProductionCompany.length == 0)
            {          
                filmProductionCompany = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Writer" && filmWriters.length == 0)
            {
                filmWriters = GetValueByXpath(currentRule);
                    //imbd may have multiple values, in incremental spans - if so we need to loop through them and append the values
                    if ((document.URL.indexOf("pro-labs.imdb.") != -1)  && (document.URL.indexOf("filmmakers") == -1))
                    {
                        var oldxpath = currentRule.Xpath;
                        for (var x = 2; x < 30; x++)
                        {
                            var newxpath = currentRule.Xpath
                            newxpath = newxpath.replace('/span[2]/span[1]/a', '/span[2]/span[' + x + ']/a');
                            currentRule.Xpath = newxpath;
                            var nextWriter = GetValueByXpath(currentRule);
                            if (nextWriter.length > 1)
                            {
                                if (nextWriter.indexOf("See all") == -1)
                                {
                                    filmWriters = filmWriters + ", " + nextWriter;
                                }
                            }                            
                            currentRule.Xpath = oldxpath;
                        }
                    }
                    //if we are on the separate 'filmmakers' page of IMDB, we have to get the multiple values in a slightly different way'
                    if ((document.URL.indexOf("pro-labs.imdb.") != -1) && (document.URL.indexOf("filmmakers") != -1))
                    {
                        var oldxpath = currentRule.Xpath;
                        for (var x = 2; x < 30; x++) {
                            var newxpath = currentRule.Xpath
                            newxpath = newxpath.replace('/ul[2]/li[1]/span[1]/', '/ul[2]/li[' + x + ']/span[1]/');
                            currentRule.Xpath = newxpath;
                            var nextWriter = GetValueByXpath(currentRule);
                            if (nextWriter.length > 1) {
                                if (nextWriter.indexOf("See all") == -1)
                                {
                                    filmWriters = filmWriters + ", " + nextWriter;
                                }
                            }
                            currentRule.Xpath = oldxpath;
                        }
                    }
            }

            if (currentRule.Field == "Cast" && filmCast.length == 0)
            {
                filmCast = GetValueByXpath(currentRule);
                //imbd may have multiple values, in incremental spans - if so we need to loop through them and append the values
                if ((document.URL.indexOf("pro-labs.imdb.") != -1) && (document.URL.indexOf("filmmakers") == -1))
                {
                    var oldxpath = currentRule.Xpath;
                    for (var x = 2; x < 300; x++) {
                        var newxpath = currentRule.Xpath;
                        newxpath = newxpath.replace('[@id="cast_list"]/ul/li[1]', '[@id="cast_list"]/ul/li[' + x + ']');
                        currentRule.Xpath = newxpath;
                        var nextCastMember = GetValueByXpath(currentRule);
                        if (nextCastMember.length > 1)
                        {
                            if (nextCastMember.indexOf("See all") == -1)
                            {
                                filmCast = filmCast + ", " + nextCastMember;
                            }
                        }
                        currentRule.Xpath = oldxpath;   
                    }
                }

            }


            if (currentRule.Field == "Distributor" && filmDistributor.length == 0)
            {
                filmDistributor = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Producer" && filmProducers.length == 0)
            {
                filmProducers = GetValueByXpath(currentRule);
                //imbd may have multiple values, in incremental spans - if so we need to loop through them and append the values
                if ((document.URL.indexOf("pro-labs.imdb.") != -1) && (document.URL.indexOf("filmmakers") == -1))
                {
                    var oldxpath = currentRule.Xpath;
                    for (var x = 2; x < 30; x++)
                    {
                        var newxpath = currentRule.Xpath;
                        newxpath = newxpath.replace('/span[2]/span[1]/a', '/span[2]/span[' + x + ']/a');
                        currentRule.Xpath = newxpath;
                        var nextProducer = GetValueByXpath(currentRule);
                        if (nextProducer.length > 1)
                        {
                            if (nextProducer.indexOf("See all") == -1)
                            {
                                filmProducers = filmProducers + ", " + nextProducer;
                            }
                        }
                        currentRule.Xpath = oldxpath;
                    }
                }
                //if we are on the separate 'filmmakers' page of IMDB, we have to get the multiple values in a slightly different way'
                if ((document.URL.indexOf("pro-labs.imdb.") != -1) && (document.URL.indexOf("filmmakers") != -1))
                {
                    var oldxpath = currentRule.Xpath;
                    for (var x = 2; x < 30; x++)
                    {
                        var newxpath = currentRule.Xpath;
                        newxpath = newxpath.replace('/ul[3]/li[1]/span[1]/', '/ul[3]/li[' + x + ']/span[1]/');
                        currentRule.Xpath = newxpath;
                        var nextProducer = GetValueByXpath(currentRule);
                        if (nextProducer.length > 1)
                        {
                            if (nextProducer.indexOf("See all") == -1)
                            {
                                filmProducers = filmProducers + ", " + nextProducer;
                            }
                        }
                        currentRule.Xpath = oldxpath;
                    }
                }
            }

            if (currentRule.Field == "Editor" && filmEditors.length == 0)
            {
                filmEditors = GetValueByXpath(currentRule);
                //imbd may have multiple values, in incremental spans - if so we need to loop through them and append the values
                if ((document.URL.indexOf("pro-labs.imdb.") != -1) && (document.URL.indexOf("filmmakers") == -1))
                {
                    var oldxpath = currentRule.Xpath;
                    for (var x = 2; x < 30; x++) {
                        var newxpath = currentRule.Xpath;
                        newxpath = newxpath.replace('/span[2]/span[1]/a', '/span[2]/span[' + x + ']/a');
                        currentRule.Xpath = newxpath;
                        var nextEditor = GetValueByXpath(currentRule);
                        if (nextEditor.length > 1) {

                            if (nextEditor.indexOf('See all') == -1) {
                                filmEditors = filmEditors + ", " + nextEditor;
                            }
                        }
                        currentRule.Xpath = oldxpath;
                    }
                }
                //if we are on the separate 'filmmakers' page of IMDB, we have to get the multiple values in a slightly different way'
                if ((document.URL.indexOf("pro-labs.imdb.") != -1) && (document.URL.indexOf("filmmakers") != -1))
                {
                    var oldxpath = currentRule.Xpath;
                    for (var x = 2; x < 30; x++)
                    {
                        var newxpath = currentRule.Xpath;
                        newxpath = newxpath.replace('/ul[6]/li[1]/span[1]/', '/ul[6]/li[' + x + ']/span[1]/');
                        currentRule.Xpath = newxpath;
                        var nextEditor = GetValueByXpath(currentRule);
                        if (nextEditor.length > 1)
                        {
                            if (nextEditor.indexOf('See all') == -1)
                            {
                                filmEditors = filmEditors + ", " + nextEditor;
                            }
                        }
                        currentRule.Xpath = oldxpath;
                    }
                }
            }

            
            if (currentRule.Field == "ProductionManager" && filmProductionManagers.length == 0)
            {
                filmProductionManagers = GetValueByXpath(currentRule);
                //imbd may have multiple values, in incremental spans - if so we need to loop through them and append the values
                if ((document.URL.indexOf("pro-labs.imdb.") != -1) && (document.URL.indexOf("filmmakers") != -1))
                {
                    var oldxpath = currentRule.Xpath;
                    for (var x = 2; x < 100; x++)
                    {
                        var newxpath = currentRule.Xpath
                        newxpath = newxpath.replace('//*[@id="filmmakers"]/ul[13]/li[1]/span[1]/span/a/text()', '//*[@id="filmmakers"]/ul[13]/li[' + x + ']/span[1]/span/a/text()');
                        currentRule.Xpath = newxpath;
                        var nextProdMgr = GetValueByXpath(currentRule);
                        if (nextProdMgr.length > 1)
                        {
                            if (nextWriter.indexOf("See all") == -1)
                            {
                                filmProductionManagers = filmProductionManagers + ", " + nextProdMgr;
                            }
                        }
                        currentRule.Xpath = oldxpath;
                    }
                }
            }

            //channel navigator fields...
            
            if (currentRule.Field == "LegalName" && legalName.length == 0) {
                legalName = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "MailingAddress" && cnMailingAddress.length == 0) {
                cnMailingAddress = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Description" && description.length == 0) {
                description = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "LinkedInUrl" && linkedInUrl.length == 0) {
                if (document.URL.indexOf('data.channel-navigator.com') !== -1) {
                    //The name is trimmed down so we need the href attribute of the node
                    var element = document.evaluate(currentRule.Xpath, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                    if (element) {
                        var hrefAttr = element.getAttribute('href');
                        if (hrefAttr) {
                            linkedInUrl = hrefAttr;
                        }
                    }
                } else {
                    linkedInUrl = GetValueByXpath(currentRule);
                }
            }

            if (currentRule.Field == "AccountSource" && accountSource.length == 0) {
                accountSource = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "AccountSourceDetails" && accountSourceDetails.length == 0) {
                accountSourceDetails = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "StateIncorporated" && stateIncorporated.length == 0) {
                stateIncorporated = GetValueByXpath(currentRule);
                stateIncorporated = TranslateState(stateIncorporated);
            }

            if (currentRule.Field == "StartYear" && startYear.length == 0) {
                startYear = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "FiscalEndYear" && fiscalEndYear.length == 0) {
                fiscalEndYear = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Ownership" && ownership.length == 0) {
                ownership = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "ServiceOverview" && serviceOverview.length == 0) {
                serviceOverview = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Employees" && cnEmployees.length == 0) {
                cnEmployees = GetValueByXpath(currentRule);
                if (document.URL.indexOf("connect.data.com" != -1))
                {
                    var split = cnEmployees.split(" ");

                    cnEmployees = split[split.length-1].replace(">", "").replace("K", "000").replace("-", "").trim();
                }
                if (document.URL.indexOf("boardex.com") != -1 && cnEmployees.length > 0)
                {
                    cnEmployees = cnEmployees.replace(",", "");

                    //Make sure it's a number before putting it into the field
                    if (isNaN(cnEmployees)) {
                        cnEmployees = "";
                    }
                }
            }

            if (currentRule.Field == "BillableEmployees" && billableEmployees.length == 0) {
                billableEmployees = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "AnnualRevenue" && annualRevenue.length == 0) {
                annualRevenue = GetValueByXpath(currentRule);

                if (document.URL.indexOf("boardex.com") != -1 && annualRevenue.length > 0)
                {
                    //Replace commas for larger numbers then try to parse the number to an int
                    annualRevenue = annualRevenue.replace(",", "");
                    
                    //Make sure it's a number before putting it into the field
                    if (isNaN(annualRevenue)){
                        annualRevenue = "";
                    } else {
                        annualRevenue = parseInt(annualRevenue);
                        annualRevenue = annualRevenue * 1000000;
                    }
                    annualRevenue = annualRevenue.toString();
                }
            }

            if (currentRule.Field == "OtherServices" && otherServices.length == 0) {
                otherServices = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "RunRate" && runRate.length == 0) {
                runRate = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "PartnersAndCertifications" && partnersAndCertifications.length == 0) {
                partnersAndCertifications = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "OtherCustomerFocus" && otherCustomerFocus.length == 0) {
                otherCustomerFocus = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "ActiveCustomers" && activeCustomers.length == 0) {
                activeCustomers = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "CustomerBaseSize" && customerBaseSize.length == 0) {
                customerBaseSize = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Department" && department.length == 0) {
                department = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Type" && type.length == 0) {                
                type = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "CNAddress1" && cnAddress1.length == 0) {
                cnAddress1 = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "CNAddress2" && cnAddress2.length == 0) {
                cnAddress2 = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Solutions" && solutions.length == 0) {
                solutions = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Services" && services.length == 0) {
                services = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Technologies" && technologies.length == 0) {
                technologies = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Awards" && awards.length == 0) {
                awards = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "Rankings" && rankings.length == 0) {
                rankings = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "VerticalMarkets" && verticalMarkets.length == 0) {
                verticalMarkets = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "VendorPartnerships" && vendorPartnerships.length == 0) {
                vendorPartnerships = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "RevenueRange" && revenueRange.length == 0) {
                revenueRange = GetValueByXpath(currentRule);
            }

            if (currentRule.Field == "CommunicationNotes" && communicationNotes.length == 0) {
                communicationNotes = GetValueByXpath(currentRule);
            }
            
            //Key/Value pairs are a little different - this is what we'll use when the page has insufficient structure for us to Xpath straight to it
            //The key determines what field we put the data into
            //Alas, because every HTML page is different, we need to split the pair using a special method
            // 
            if (currentRule.Field == "KeyValuePair") {

                var pair = GetHtmlByXpath(currentRule.Xpath);
                var key;
                var value;
                // 
                // 
                // 
                if (pair != "" && s != null && currentRule.Split == false) {
                    var temp = document.createElement('div');
                    temp.innerHTML = pair;
                    if (document.URL.toLowerCase().indexOf("dice.com/talentmatch") > -1) {
                        key = temp.getElementsByTagName('dt')[0].innerText;
                        value = temp.getElementsByTagName('dd')[0].innerText;
                        if (key == "Desired position:") {
                            jobtitle = value;
                        }
                        if (key == "Primary Phone:") {
                            homePhone = value;
                        }
                        if (key == "Secondary Phone:") {
                            workPhone = value;
                        }
                        if (key == "Recent Employer:") {
                            employer = value;
                        }
                    }
                    if (document.URL.toLowerCase().indexOf("rs.drapersjobs.com/cv-match/candidate/") > -1) {
                        if (temp.getElementsByClassName('itemLabel').length) {
                            key = temp.getElementsByClassName('itemLabel')[0].innerText;
                        } else key = '';
                        if (temp.getElementsByClassName('inputField').length) {
                            value = temp.getElementsByClassName('inputField')[0].innerText;
                        } else value = '';
                        if (key == "Mobile") {
                            if (mobile == '') {
                                mobile = TidyPhone(value);
                            }
                            if (homePhone == '') {
                                homePhone = TidyPhone(value);
                            }
                        }
                        if (key == "Email") {
                            email = value;
                        }
                        if (key == "Website") {
                            webAddress = value;
                        }
                        if (key == "Home telephone") {
                            if (homePhone == '') {
                                homePhone = TidyPhone(value);
                            }
                        }

                    }

                    if (document.URL.toLowerCase().indexOf("s1jobs.com") > -1) {
                        var pairsArray = pair.split(/\n/);
                        for (var j = 0; j < pairsArray.length; j++) {
                            var splitAtComma = pairsArray[j].split(':');

                            if (splitAtComma.length == 2) {
                                key = CleanUpHTML(splitAtComma[0]);
                                value = CleanUpHTML(splitAtComma[1]);
                                // 
                                // 
                                if (key.indexOf("Daytime") > -1) {
                                    // 
                                    workPhone = TidyPhone(value);
                                }
                                if (key.indexOf("Evening tel") > -1) {
                                    homePhone = TidyPhone(value);
                                }
                                if (key.indexOf("Mobile tel") > -1) {
                                    mobile = TidyPhone(value);
                                }
                            }

                        }

                    }
                    if (document.URL.toLowerCase().indexOf("monster.com.hk") > -1) {
                        // 

                        var listOfNodes = temp.innerHTML.split('<br>');
                        for (var activeNode = 0; activeNode < listOfNodes.length; activeNode++) {

                            var thisNodeAsDiv = document.createElement('div');
                            thisNodeAsDiv.innerHTML = listOfNodes[activeNode];

                            var keyValuePair = thisNodeAsDiv.innerText.split(':');
                            if (keyValuePair.length > 1) {
                                if (keyValuePair[0] == "Phone") {
                                    homePhone = TidyPhone(keyValuePair[1]);
                                }
                                if (keyValuePair[0] == "Mobile") {
                                    mobile = TidyPhone(keyValuePair[1]);
                                }
                            }
                        }
                    }

                }
            }

            if (currentRule.Field == "DUNSNumber") {
                dunsNumber = GetValueByXpath(currentRule);
            }
        }
    }

    //Fill and blank  specific phone number fields with generic Phone if we have it
    //the if below is to accomodate a specific request from LightIron re not auto filling other types of phone number
    if (document.URL.indexOf(".imdb.") == -1 && document.URL.indexOf("studiosystem.") == -1
        && document.URL.indexOf(".channel-navigator.") == -1)
    {
        if (phone.length > 0) {
            if (mobile.length == 0) {
                mobile = phone;
            }
            if (workPhone.length == 0) {
                workPhone = phone;
            }
            if (homePhone.length == 0) {
                homePhone = phone;
            }
        }
        if (workPhone.length > 0) {
            if (phone.length == 0) {
                phone = workPhone;
            }
            if (mobile.length == 0) {
                mobile = workPhone;
            }
            if (homePhone.length == 0) {
                homePhone = workPhone;
            }
        }
        if (mobile.length > 0) {
            if (phone.length == 0) {
                phone = mobile;
            }
            if (workPhone.length == 0) {
                workPhone = mobile;
            }
            if (homePhone.length == 0) {
                homePhone = mobile;
            }
        }
        if (homePhone.length > 0) {
            if (phone.length == 0) {
                phone = homePhone;
            }
            if (mobile.length == 0) {
                mobile = homePhone;
            }
            if (workPhone.length == 0) {
                workPhone = homePhone;
            }
        }
    }

    if(state.length > 0)
    {
        var split = CheckForPostcode(state);

        if (split) {
            var splitter = state.split(" ");
            state = splitter[0];
            postCode = splitter[1];
        }
    }

    //For Doximity and nursingjobs.us always set country to 'United States'                      
    if (document.URL.indexOf("doximity.com") != -1 || document.URL.indexOf("nursingjobs.us") > -1) {
        country = "United States";
    }

    if (document.URL.indexOf("channel-navigator.com") != -1) {

        employer = employer.replace(new RegExp("inc", "ig"), '');
        employer = employer.replace(new RegExp("llc", "ig"), '');

        accountSourceDetails = "" + document.URL;
        type = 'AC - Suspect';

        fullBillingAddress = '' + cnAddress1 + ', ' + cnAddress2;
     
        street = fullBillingAddress;

        sst = '' + solutions;
        
        if (solutions.length > 0) {
            sst = sst + '; ' + services;
        } else {
            sst = sst + services;
        }

        if (services.length > 0) {
            sst = sst + '; ' + technologies;
        } else {
            sst = sst + technologies;
        }

        awardsRankings = '' + awards + '; ' + rankings;

        if (awards.length == 0 || rankings.length == 0) {
            awardsRankings = awardsRankings.replace(';', '');
        }

        if (cnAddress1.length == 0 || cnAddress2.length == 0) {
            fullBillingAddress = fullBillingAddress.replace(',', '');
            street = street.replace(',', '');
        }

        if (annualRevenue.length > 0) {
            var annualRevenueInt = parseFloat(annualRevenue);

            if (annualRevenueInt != null && typeof annualRevenueInt != "undefined") {
                annualRevenue = '' + (annualRevenueInt * 1000000);
            }
        }
    }

    //many different formats and configs for State / Postcode in Crunchbase organisations
    //this sorts through the date we have in that line and populate valid values
    if (document.URL.indexOf("crunchbase") != -1)
    {
        state = state.trim();
        var stateSplit = state.split(" ");
        if (stateSplit.length >= 1)
        {
            if (noNumbers(stateSplit[0])) {                
                state = stateSplit[0];
                postCode = "";
                for (var i = 1; i < stateSplit.length; i++) {
                        postCode = postCode + " " + stateSplit[i];                    
                }
            }
            else
            {
                //if our state value has numbers, we want to discard it
                state = "";
                //and then rebuild our postcode here
                postCode = "";
                for (var i = 0; i < stateSplit.length; i++) {
                    postCode = postCode + " " + stateSplit[i];
                }
            }
        }

        if (postCode.length < 1 || postCode == "" || postCode == null)
        {
            //last try to get a postcode
            if (stateSplit.length > 0)
            {
                //it must contain numbers
                if (!noNumbers(stateSplit[0]))
                {
                    postCode = stateSplit[0];
                }
            }
        }
    }

    source = TidySource(source);
    //Strip titles from surname, eg. Phd
    firstname = EbstaNameSanitizer.TidyName(firstname);
    surname = EbstaNameSanitizer.TidyName(surname);
    jsonPerson["Url"] = document.URL;
    jsonPerson["Firstname"] = CleanUpHTML(firstname);
    jsonPerson["Surname"] = CleanUpHTML(surname);
    jsonPerson["Email"] = CleanUpHTML(email);
    jsonPerson["OtherEmail"] = CleanUpHTML(otherEmail);
    jsonPerson["Mobile"] = RemoveSkype(CleanUpHTML(mobile));
    jsonPerson["HomePhone"] = RemoveSkype(CleanUpHTML(homePhone));
    jsonPerson["WorkPhone"] = RemoveSkype(CleanUpHTML(workPhone));
    jsonPerson["Phone"] = RemoveSkype(CleanUpHTML(phone));
    jsonPerson["Fax"] = CleanUpHTML(fax);
    jsonPerson["Company"] = CleanUpHTML(employer);
    jsonPerson["Title"] = CleanUpHTML(jobtitle);
    jsonPerson["Address"] = CleanUpHTML(address);
    jsonPerson["Postcode"] = CleanUpHTML(postCode);
    jsonPerson["City"] = CleanUpHTML(city);
    jsonPerson["State"] = CleanUpHTML(state);
    jsonPerson["Country"] = CleanUpHTML(country);
    jsonPerson["Twitter"] = CleanUpHTML(twitter);
    jsonPerson["WebAddress"] = CleanUpHTML(webAddress);
    jsonPerson["PublicProfileURL"] = CleanUpHTML(publicProfileURL);
    jsonPerson["Skills"] = CleanUpHTML(skills);
    jsonPerson["Source"] = CleanUpHTML(source);
    jsonPerson["Description"] = CleanUpHTML(description);
    jsonPerson["Street"] = CleanUpHTML(street);
    jsonPerson["DunsNumber"] = CleanUpHTML(dunsNumber);
    
    //Custom fields
    jsonPerson["DealDescription"] = CleanUpHTML(dealDescription);
    jsonPerson["DealPrice"] = CleanUpHTML(dealPrice);
    jsonPerson["DealDiscount"] = CleanUpHTML(dealDiscount);
    jsonPerson["DealSaving"] = CleanUpHTML(dealSaving);
    //Currency is £ for now - link to locale
    jsonPerson["DealRRP"] =  GetSum(dealPrice, dealSaving);
    jsonPerson["DealQuantity"] = CleanUpHTML(dealQuantity);
    //DealLink is the same as URL
    jsonPerson["DealLink"] = document.URL;
    jsonPerson["CompetitorName"] = CleanUpHTML(source);
    //LightIron
    jsonPerson["Gender"] = CleanUpHTML(filmPersonGender);
    jsonPerson["Aka"] = CleanUpHTML(filmAka);
    jsonPerson["LastRelease"] = CleanUpHTML(filmLastRelease);
    jsonPerson["NextRelease"] = CleanUpHTML(filmNextRelease);
    jsonPerson["ProjectType"] = CleanUpHTML(filmProjectType);
    jsonPerson["ProjectName"] = CleanUpHTML(filmProjectName);
    jsonPerson["ProjectStatus"] = CleanUpHTML(filmProjectStatus);
    jsonPerson["ProjectGenre"] = CleanUpHTML(filmProjectGenre);
    jsonPerson["ProjectBudget"] = CleanUpHTML(filmProjectBudget);
    jsonPerson["ProjectDevNotes"] = CleanUpHTML(filmProjectDevNotes);
    jsonPerson["ProjectDiStartDate"] = CleanUpHTML(filmProjectDiStartDate);
    jsonPerson["ProjectLogLine"] = CleanUpHTML(filmProjectLogLine);
    jsonPerson["ProjectReleaseDate"] = CleanUpHTML(filmProjectReleaseDate);
    jsonPerson["ProjectStartDate"] = CleanUpHTML(filmProjectStartDate);
    jsonPerson["ProjectWrapDate"] = CleanUpHTML(filmProjectWrapDate);
    jsonPerson["ProjectDirector"] = CleanUpHTML(filmDirector);
    jsonPerson["ProjectProductionCompany"] = CleanUpHTML(filmProductionCompany);
    jsonPerson["ProjectWriters"] = CleanUpHTML(filmWriters);
    jsonPerson["ProjectCast"] = CleanUpHTML(filmCast);
    jsonPerson["ProjectDistributor"] = CleanUpHTML(filmDistributor);
    jsonPerson["ProjectProducers"] = CleanUpHTML(filmProducers);
    jsonPerson["ProjectEditors"] = CleanUpHTML(filmEditors);
    jsonPerson["ProjectProductionManagers"] = CleanUpHTML(filmProductionManagers);

    jsonPerson["LinkedInPro"] = '1';

    //channel navigator...
    jsonPerson["LegalName"] = CleanUpHTML(legalName);
    jsonPerson["MailingAddress"] = CleanUpHTML(cnMailingAddress);
    //jsonPerson["Description"] = CleanUpHTML(description);
    jsonPerson["LinkedInUrl"] = CleanUpHTML(linkedInUrl);
    jsonPerson["AccountSource"] = CleanUpHTML(accountSource);
    jsonPerson["AccountSourceDetails"] = CleanUpHTML(accountSourceDetails);
    jsonPerson["StateIncorporated"] = CleanUpHTML(stateIncorporated);
    jsonPerson["StartYear"] = CleanUpHTML(startYear);
    jsonPerson["FiscalEndYear"] = CleanUpHTML(fiscalEndYear);
    jsonPerson["Ownership"] = CleanUpHTML(ownership);
    jsonPerson["ServiceOverview"] = CleanUpHTML(serviceOverview);
    jsonPerson["Employees"] = CleanUpHTML(cnEmployees);
    jsonPerson["BillableEmployees"] = CleanUpHTML(billableEmployees);
    jsonPerson["AnnualRevenue"] = CleanUpHTML(annualRevenue);
    jsonPerson["OtherServices"] = CleanUpHTML(otherServices);
    jsonPerson["RunRate"] = CleanUpHTML(runRate);
    jsonPerson["PartnersAndCertifications"] = CleanUpHTML(partnersAndCertifications);
    jsonPerson["OtherCustomerFocus"] = CleanUpHTML(otherCustomerFocus);
    jsonPerson["ActiveCustomers"] = CleanUpHTML(activeCustomers);
    jsonPerson["CustomerBaseSize"] = CleanUpHTML(customerBaseSize);
    jsonPerson["Department"] = CleanUpHTML(department);
    jsonPerson["Type"] = CleanUpHTML(type);

    jsonPerson["FullBillingAddress"] = CleanUpHTML(fullBillingAddress);
    jsonPerson["SST"] = CleanUpHTML(sst);
    jsonPerson["AwardsRankings"] = CleanUpHTML(awardsRankings);

    jsonPerson["VerticalMarkets"] = CleanUpHTML(verticalMarkets);
    jsonPerson["VendorPartnerships"] = CleanUpHTML(vendorPartnerships);
    jsonPerson["RevenueRange"] = CleanUpHTML(revenueRange);
    jsonPerson["CommunicationNotes"] = CleanUpHTML(communicationNotes);

    //This appends a div into the page when we're running in debug mode
    //Only for parsing monitor purposes
    if (debugMode == 'true') {
        // inject information into the page. 
        this._view = document.body.insertBefore(document.createElement("div"), document.body.firstChild);
        this._view.id = "ebsta-debug";
        this._view.innerText = jsonPerson;
    }

    return jsonPerson;
}

//fix for lightiron states/zipcodes
function CheckForPostcode(state) {
    var re = /^([A-Z][A-Z]\s\b\b\b\b\b\b)/;
    return re.test(state);
}

function RemoveSkype(variable) {

    // 
    return variable.toString();
}

function CleanUpHTML(variable) {

    //// 
    // Handle Linkedin Highlight. 
    variable = variable.replace('begin_of_the_skype_highlighting', '');
    variable = variable.replace('end_of_the_skype_highlighting', '');

    variable = variable.replace('<strong class=\"highlight\">', '');
    variable = variable.replace('<\/strong>', '');
    //Linkedin external links
    variable = variable.replace('/redir/redirect?url=', '');
    variable = variable.replace('&urlhash=xYPG', '/');
    variable = variable.replace('&urlhash=_kfX', '/');

    variable = variable.replace('<strong class=\"highlight\">', '');
    variable = variable.replace('<\/strong>', '');

    variable = variable.replace('send SMS', '');

    variable = variable.replace('<p>', '');
    variable = variable.replace('<br>', '');
    variable = variable.replace('<span>', '');
    variable = variable.replace('</span>', '');

    //Change to cope with Talentbin's varying layouts...
    variable = variable.replace('Upgrade to see full name', '');

    variable = variable.replace('(Download CV )', '');

    variable = variable.trim();

    return variable;
}

function SplitString(input, splitter) {
    return input.split(splitter);
}

function TidySource(source) {

    var _return = '';
    var split = SplitString(source, '.');
    var s = split[1];

    if (s == "com" || s == "co") {
        s = split[0];
    }

    switch (s) {
        case "linkedin":
            _return = "LinkedIn";
            break;
        case "cv-library":
            _return = "CV-Library";
            break;
        case "theitjobboard":
            _return = "TheITJobBoard";
            break;
        case "discoverydb":
            _return = "Discover.org";
            break;
        case "theladders":
            _return = "The Ladders";
            break;
        case "dc3":
            __return = "PageUpPeople";
            break;
        case "adcourier":
            _return = "Broadbean";
        case "data":
            _return = "Data.com";
            break;
        case "studiosystem":
            _return = "StudioSystem";
            break;
        case "channel-navigator":
            _return = "Channel Navigator";
            break;
        default:
            _return = s.charAt(0).toUpperCase() + s.slice(1);

    }

    return _return;

}

function TidyPhone(s) {
    s = s.trim();
    s = s.replace(/\"/g, "");
    s = s.replace("Mobile Tel: ", "");
    s = s.replace("Mobile phone:", "");
    s = s.replace("Phone: ", "");
    s = s.replace("Phone:", "");
    s = s.replace("Work Tel:", "");
    s = s.replace("Fax: ", "");
    s = s.replace("Fax:", "");
    s = s.replace("(mobile)", "");
    s = s.replace("(home)", "");
    s = s.replace("&nbsp;", "");
    s = s.replace("(work)", "");
    s = s.replace(',', '');
    s = s.replace('HQ Phone', '');
    s = s.replace('– Preferred', '');
    s = s.replace('| ', '');
    s = s.replace('Direct Phone : ', '');
    s = s.replace('HQ Phone : ', '');
    s = s.replace('Fax', '');
    s = s.replace('fax', '');
    s = s.replace('FAX', '');
    s = s.replace('Fax:', '');
    s = s.replace('fax:', '');
    s = s.replace('FAX:', '');

    if (s == "-") {
        s = "";
    }
    if (s == "Not Specified") {
        s = "";
    }

    return s;
}

function GetHtmlByXpath(path) {

    var valueStr = '';
    var regexRule = false;

    try {
        regexRule = rule.Regex;
    } catch (err) {

    }

    try {

        if (regexRule) {


            var str = document.body.innerHTML;
            var pat = new RegExp(rule.Xpath, 'gim');

            var matches = pat.exec(str);

            if (matches != null) {
                valueStr = matches[1];
            }


        }
        else {
            //Right now we turn on iFrames for each specific domain where they're used
            //One day I hope to never have any page-specific code in here.

            //One day...
            if (document.domain == 'www.oilandgasjobsearch.com' |
                document.domain == 'recruiter.cwjobs.co.uk' |
                document.domain == 'recruiter.totaljobs.com' |
                document.domain === 'recruiter.caterer.com' |
                document.domain === 'property2.costar.com') {

                if (document.getElementsByTagName("iframe").length > 0) {
                    var iframes = document.getElementsByTagName("iframe");
                    for (var i = 0; i < iframes.length; i++) {
                        var this_iframe = iframes[i];
                        var frameDomain = (SplitString(this_iframe.src, '/'))[2];
                        //Only bother to evaluate if iframe comes from the same place as the site itself
                        if (frameDomain == document.domain || frameDomain == "docs.google.com" & document.domain == "www.ziprecruiter.com") {
                            //// 

                            var frameDoc = this_iframe.contentDocument;
                            try {
                                var element = frameDoc.evaluate(path.Xpath, frameDoc, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                                // 
                                if (element != null) {
                                    valueStr = element.outerHTML.trim();
                                    return valueStr;
                                }

                            }
                            catch (error) {
                                ////  
                            }
                        }
                    }
                }
            }

            try {
                var element = document.evaluate(path, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;

                if (element != null) {
                    //// 
                    //   //// 
                    var pathValueUntrimmed = element.outerHTML;

                    if (typeof pathValueUntrimmed != "undefined") {
                        valueStr = pathValueUntrimmed.trim();
                    } else if (element.length != 0 && element != null && typeof element != "undefined") {
                        valueStr = element.textContent;
                    }

                    //valueStr = element.outerHTML.trim();
                }

            }
            catch (error) {
                //// 
            }
        }
    }
    catch (error) {
        // error
    }
    // 
    return valueStr;
}

//This is the most important function in the whole page!
function GetValueByXpath(rule) {

    var valueStr = '';
    var regexRule = false;
    var dateTimeFormat = false;

    try {
        if (rule.FormatType != null) {
            if (rule.FormatType.length != 0) {
                dateTimeFormat = true;
            }
        }

    } catch(err) {
        
    }

    try {
        regexRule = rule.Regex;
    } catch (err) {

    }

    try {

        if (regexRule) {

            
            var str = document.body.innerHTML;
            var pat = new RegExp(rule.Xpath, 'gim');
            var matches = pat.exec(str);
            if (matches != null) {
                valueStr = matches[1];
                if (valueStr) {
                    //Remove potential html tags included within result
                    valueStr = valueStr.replace(/<[^>]*>/g, '');

                    //Put value into a element and read text out so the characters are decoded for us by the browser
                    var elem = document.createElement('textarea');
                    elem.innerHTML = valueStr;
                    valueStr = elem.textContent;
                }
            }


        } else {
            var element = document.evaluate(rule.Xpath, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;

            if (element != null) {
                //// 
                if (element.innerText) {
                    valueStr = htmlspecialchars_decode(element.innerText).trim();
                } else {
                    valueStr = htmlspecialchars_decode(element.textContent).trim();
                }
            }
        }

    }
    catch (error) {
        ////  
    }

    //Have a retry process in case the xpath we used doesn't return a node value
    if (!valueStr) {
        valueStr = EbstaXpath.GetStringByAnyType(rule.Xpath);
    }

    //iframes evaluated if conventional method fails
    if (valueStr == '' && document.domain == 'www.oilandgasjobsearch.com') {
        if (document.getElementsByTagName("iframe").length > 0) {
            var iframes = document.getElementsByTagName("iframe");
            for (var i = 0; i < iframes.length; i++) {

                var this_iframe = iframes[i];
                var frameDomain = (SplitString(this_iframe.src, '/'))[2];
                //Only bother to evaluate if iframe comes from the same place as the site itself
                if (frameDomain == document.domain) {
                    var frameDoc = this_iframe.contentDocument;
                    try {
                        var element = frameDoc.evaluate(rule.Xpath, frameDoc, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                        //// 
                        if (element != null) {
                            if (element.innerText) {
                                valueStr = htmlspecialchars_decode(element.innerText).trim();

                            }
                            else {
                                valueStr = htmlspecialchars_decode(element.textContent).trim();
                            }                            
                        }

                    }
                    catch (error) {
                        ////  
                    }
                }
            }
        }
    }

    if (valueStr == '' && document.domain == 'property2.costar.com') {
        if (document.getElementsByTagName("iframe").length > 0) {
            var iframes = document.getElementsByTagName("iframe");
            for (var i = 0; i < iframes.length; i++) {

                var this_iframe = iframes[i];
                var frameDomain = (SplitString(this_iframe.src, '/'))[2];
                //Only bother to evaluate if iframe comes from the same place as the site itself
                if (frameDomain == document.domain) {
                    var frameDoc = this_iframe.contentDocument;
                    try {

                        var element = frameDoc.evaluate(rule.Xpath, frameDoc, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                        //// 
                        if (element != null) {
                            if (element.innerText) {
                                valueStr = htmlspecialchars_decode(element.innerText).trim();

                            }
                            else {
                                valueStr = htmlspecialchars_decode(element.textContent).trim();
                            }
                        }

                    }
                    catch (error) {
                        ////  
                    }
                }
            }
        }
    }
    
    if (dateTimeFormat) {
        // We need to format the DAtes . 
        //if (rule.FormatType == "mm/dd/yyyy") {
        //    valueStr = FormatDateStandard(valueStr, rule.FormatType); 
        //}
        //else if (rule.FormatType == "mmmDD,yyyy") {
        //    valueStr = valueStr.replace(",", "");
        //    //var retSplt = valueStr.split(/\n/);
        //    var retSplt = valueStr.split(/\s/);
        //    if (retSplt.length > 2) {
        //        valueStr = retSplt[0] + "/" + retSplt[1] + "/" + retSplt[2];
        //        valueStr = FormatDateStandard(valueStr, rule.FormatType);
        //    }
        //}
        valueStr = FormatDateStandard(valueStr, rule.FormatType);
    }

    return valueStr;
}

// All need to come back..yy-mm-dd 
function FormatDateStandard(value, formatType) {
    // We need to format the DAtes . 
    var spl;
    if (formatType == "mm/dd/yyyy") {
        spl = value.split('/');
    }
    if (formatType == "mmmDD,yyyy") {
        spl = value.split(/\s/);
    }
    var mth = spl[0];
    var dd = spl[1];
    var year = spl[2];

    //if (year.length == 4) {
    //    year = year.substring(1, 2);
    //}

    
    if (typeof year == "undefined" || typeof mth == "undefined" || typeof dd == "undefined") {
        return '';
    } else {
        if (mth != null) {
            switch (mth.toLowerCase()) {
            case "jan":
                mth = "01";
                break;
            case "feb":
                mth = "02";
                break;
            case "mar":
                mth = "03";
                break;
            case "apr":
                mth = "04";
                break;
            case "may":
                mth = "05";
                break;
            case "jun":
                mth = "06";
                break;
            case "jul":
                mth = "07";
                break;
            case "aug":
                mth = "08";
                break;
            case "sep":
                mth = "09";
                break;
            case "oct":
                mth = "10";
                break;
            case "nov":
                mth = "11";
                break;
            case "dec":
                mth = "12";
                break;
            }

            return year + "-" + mth + "-" + dd;
        } else {
            return '';
        }
    }

}

function htmlspecialchars_decode(string, quote_style) {
    // http://kevin.vanzonneveld.net
    // +   original by: Mirek Slugen
    // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
    // +   bugfixed by: Mateusz "loonquawl" Zalega
    // +      input by: ReverseSyntax
    // +      input by: Slawomir Kaniecki
    // +      input by: Scott Cariss
    // +      input by: Francois
    // +   bugfixed by: Onno Marsman
    // +    revised by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
    // +   bugfixed by: Brett Zamir (http://brett-zamir.me)
    // +      input by: Ratheous
    // +      input by: Mailfaker (http://www.weedem.fr/)
    // +      reimplemented by: Brett Zamir (http://brett-zamir.me)
    // +    bugfixed by: Brett Zamir (http://brett-zamir.me)
    // *     example 1: htmlspecialchars_decode("<p>this -&gt; &quot;</p>", 'ENT_NOQUOTES');
    // *     returns 1: '<p>this -> &quot;</p>'
    // *     example 2: htmlspecialchars_decode("&amp;quot;");
    // *     returns 2: '&quot;'
    var optTemp = 0,
    i = 0,
    noquotes = false;
    if (typeof quote_style === 'undefined') {
        quote_style = 2;
    }
    string = string.toString().replace(/&lt;/g, '<').replace(/&gt;/g, '>');
    var OPTS = {
        'ENT_NOQUOTES': 0,
        'ENT_HTML_QUOTE_SINGLE': 1,
        'ENT_HTML_QUOTE_DOUBLE': 2,
        'ENT_COMPAT': 2,
        'ENT_QUOTES': 3,
        'ENT_IGNORE': 4
    };
    if (quote_style === 0) {
        noquotes = true;
    }
    if (typeof quote_style !== 'number') { // Allow for a single string or an array of string flags
        quote_style = [].concat(quote_style);
        for (i = 0; i < quote_style.length; i++) {
            // Resolve string input to bitwise e.g. 'PATHINFO_EXTENSION' becomes 4
            if (OPTS[quote_style[i]] === 0) {
                noquotes = true;
            } else if (OPTS[quote_style[i]]) {
                optTemp = optTemp | OPTS[quote_style[i]];
            }
        }
        quote_style = optTemp;
    }
    if (quote_style & OPTS.ENT_HTML_QUOTE_SINGLE) {
        string = string.replace(/&#0*39;/g, "'"); // PHP doesn't currently escape if more than one 0, but it should
        // string = string.replace(/&apos;|&#x0*27;/g, "'"); // This would also be useful here, but not a part of PHP
    }
    if (!noquotes) {
        string = string.replace(/&quot;/g, '"');
    }
    // Put this in last place to avoid escape being double-decoded
    string = string.replace(/&amp;/g, '&');

    return string;
}

function AreWeProLinkedin(url) {

    var vUrl = '' + url;

    if (vUrl.indexOf("linkedin.com") != -1) {
        return false;
    } else {
        return true;
    }
}

function GetSum(cashStringOne, cashStringTwo) {
    try {
        var numOne = Number(cashStringOne.replace(/[^0-9\.]+/g, ""));
        var numTwo = Number(cashStringTwo.replace(/[^0-9\.]+/g, ""));

        if (numOne > 0 && numTwo > 0) {
            //currency is currently pounds
            return "£" + ((numOne + numTwo).toFixed(2).toString());
        }
        return "";
    }
    catch (e) {
        return "";
    }
}

function getCountryFromState(state) {

    var country = "";

    switch (state.trim()) {
        case "AL":
            country = "United States";
            break;
        case "AK":
            country = "United States";
            break;
        case "AZ":
            country = "United States";
            break;
        case "AR":
            country = "United States";
            break;
        case "CA":
            country = "United States";
            break;
        case "CO":
            country = "United States";
            break;
        case "CT":
            country = "United States";
            break;
        case "DE":
            country = "United States";
            break;
        case "FL":
            country = "United States";
            break;
        case "GA":
            country = "United States";
            break;
        case "HI":
            country = "United States";
            break;
        case "ID":
            country = "United States";
            break;
        case "IL":
            country = "United States";
            break;
        case "IN":
            country = "United States";
            break;
        case "IA":
            country = "United States";
            break;
        case "KS":
            country = "United States";
            break;
        case "KY":
            country = "United States";
            break;
        case "LA":
            country = "United States";
            break;
        case "ME":
            country = "United States";
            break;
        case "MD":
            country = "United States";
            break;
        case "MA":
            country = "United States";
            break;
        case "MI":
            country = "United States";
            break;
        case "MN":
            country = "United States";
            break;
        case "MS":
            country = "United States";
            break;
        case "MO":
            country = "United States";
            break;
        case "MT":
            country = "United States";
            break;
        case "NE":
            country = "United States";
            break;
        case "NV":
            country = "United States";
            break;
        case "NH":
            country = "United States";
            break;
        case "NJ":
            country = "United States";
            break;
        case "NM":
            country = "United States";
            break;
        case "NY":
            country = "United States";
            break;
        case "NC":
            country = "United States";
            break;
        case "ND":
            country = "United States";
            break;
        case "OH":
            country = "United States";
            break;
        case "OK":
            country = "United States";
            break;
        case "OR":
            country = "United States";
            break;
        case "PA":
            country = "United States";
            break;
        case "RI":
            country = "United States";
            break;
        case "SC":
            country = "United States";
            break;
        case "SD":
            country = "United States";
            break;
        case "TN":
            country = "United States";
            break;
        case "TX":
            country = "United States";
            break;
        case "UT":
            country = "United States";
            break;
        case "VT":
            country = "United States";
            break;
        case "VA":
            country = "United States";
            break;
        case "WA":
            country = "United States";
            break;
        case "WV":
            country = "United States";
            break;
        case "WI":
            country = "United States";
            break;
        case "WY":
            country = "United States";
            break;
        case "DC":
            country = "United States";
            break;
        case "PR":
            country = "United States";
            break;
        case "VI":
            country = "United States";
            break;
        case "AS":
            country = "United States";
            break;
        case "GU":
            country = "United States";
            break;
        case "MP":
            country = "United States";
            break;
        case "FM":
            country = "United States";
            break;
        case "MH":
            country = "United States";
            break;
        case "PW":
            country = "United States";
            break;
        case "AB":
            country = "Canada";
            break;
        case "BC":
            country = "Canada";
            break;
        case "MB":
            country = "Canada";
            break;
        case "NB":
            country = "Canada";
            break;
        case "NL":
            country = "Canada";
            break;
        case "NS":
            country = "Canada";
            break;
        case "ON":
            country = "Canada";
            break;
        case "PE":
            country = "Canada";
            break;
        case "QC":
            country = "Canada";
            break;
        case "SK":
            country = "Canada";
            break;
        case "NT":
            country = "Canada";
            break;
        case "NU":
            country = "Canada";
            break;
        case "YT":
            country = "Canada";
            break;
        case "Alabama":
            country = "United States";
            break;
        case "Alaska":
            country = "United States";
            break;
        case "Arizona":
            country = "United States";
            break;
        case "Arkansas":
            country = "United States";
            break;
        case "California":
            country = "United States";
            break;
        case "Colorado":
            country = "United States";
            break;
        case "Connecticut":
            country = "United States";
            break;
        case "Delaware":
            country = "United States";
            break;
        case "Florida":
            country = "United States";
            break;
        case "Georgia":
            country = "United States";
            break;
        case "Hawaii":
            country = "United States";
            break;
        case "Idaho":
            country = "United States";
            break;
        case "Illinois":
            country = "United States";
            break;
        case "Indiana":
            country = "United States";
            break;
        case "Iowa":
            country = "United States";
            break;
        case "Kansas":
            country = "United States";
            break;
        case "Kentucky":
            country = "United States";
            break;
        case "Louisiana":
            country = "United States";
            break;
        case "Maine":
            country = "United States";
            break;
        case "Maryland":
            country = "United States";
            break;
        case "Massachusetts":
            country = "United States";
            break;
        case "Michigan":
            country = "United States";
            break;
        case "Minnesota":
            country = "United States";
            break;
        case "Mississippi":
            country = "United States";
            break;
        case "Missouri":
            country = "United States";
            break;
        case "Montana":
            country = "United States";
            break;
        case "Nebraska":
            country = "United States";
            break;
        case "Nevada":
            country = "United States";
            break;
        case "New Hampshire":
            country = "United States";
            break;
        case "New Jersey":
            country = "United States";
            break;
        case "New Mexico":
            country = "United States";
            break;
        case "New York":
            country = "United States";
            break;
        case "North Carolina":
            country = "United States";
            break;
        case "North Dakota":
            country = "United States";
            break;
        case "Ohio":
            country = "United States";
            break;
        case "Oklahoma":
            country = "United States";
            break;
        case "Oregon":
            country = "United States";
            break;
        case "Pennsylvania":
            country = "United States";
            break;
        case "Rhode Island":
            country = "United States";
            break;
        case "South Carolina":
            country = "United States";
            break;
        case "South Dakota":
            country = "United States";
            break;
        case "Tennessee":
            country = "United States";
            break;
        case "Texas":
            country = "United States";
            break;
        case "Utah":
            country = "United States";
            break;
        case "Vermont":
            country = "United States";
            break;
        case "Virginia":
            country = "United States";
            break;
        case "Washington":
            country = "United States";
            break;
        case "West Virginia":
            country = "United States";
            break;
        case "Wisconsin":
            country = "United States";
            break;
        case "Wyoming":
            country = "United States";
            break;
        case "Washington DC":
            country = "United States";
            break;
        case "Puerto Rico":
            country = "United States";
            break;
        case "U.S. Virgin Islands":
            country = "United States";
            break;
        case "American Samoa":
            country = "United States";
            break;
        case "Guam":
            country = "United States";
            break;
        case "Northern Mariana Islands":
            country = "United States";
            break;
        case "Federated States of Micronesia":
            country = "United States";
            break;
        case "Marshall Islands":
            country = "United States";
            break;
        case "Palau":
            country = "United States";
            break;
        case "Alberta":
            country = "Canada";
            break;
        case "British Columbia":
            country = "Canada";
            break;
        case "Manitoba":
            country = "Canada";
            break;
        case "New Brunswick":
            country = "Canada";
            break;
        case "Newfoundland and Labrador":
            country = "Canada";
            break;
        case "Nova Scotia":
            country = "Canada";
            break;
        case "Ontario":
            country = "Canada";
            break;
        case "Prince Edward Island":
            country = "Canada";
            break;
        case "Quebec":
            country = "Canada";
            break;
        case "Saskatchewan":
            country = "Canada";
            break;
        case "Northwest Territories":
            country = "Canada";
            break;
        case "Nunavut":
            country = "Canada";
            break;
        case "Yukon Territory":
            country = "Canada";
            break;

        //Australia states
        case "NSW":
        case "New South Wales":
        case "QLD":
        case "Queensland":
        case "SA":
        case "South Australia":
        case "TAS":
        case "Tasmania":
        case "VIC":
        case "Victoria":
        case "ACT":
        case "Australian Capital Territory":
            country = "Australia";
            break;

    }

    return country;
}

/**
 * Translates certain states where the shorthand may exist in multiple
 * countries, such as WA exists in US and AUS as Washington and Western Australia 
 * respectivly. So if you are AUS we will set you to the correct state.
 * Otherwise translate as normal with TranslateState
 * @param {any} state - the state to translate
 * @param {any} country - the countr you are translating for
 */
function TranslateStateWithCountry(state, country) {
    switch (state) {
        case "WA":
            if (country && country.toLowerCase() === "australia") {
                state = "Western Australia";
            } else {
                state = "Washington";
            }
            break;
        default:
            TranslateState(state);
            break;
    }
    return state;
}

function TranslateState(state) {

    switch (state.toUpperCase()) {
        case "AL":
            state = "Alabama";
            break;
        case "AK":
            state = "Alaska";
            break;
        case "AS":
            state = "American Samoa";
            break;
        case "AZ":
            state = "Arizona";
            break;
        case "AR":
            state = "Arkansas";
            break;
        case "CA":
            state = "California";
            break;
        case "CO":
            state = "Colorado";
            break;
        case "CT":
            state = "Connecticut";
            break;
        case "DE":
            state = "Delaware";
            break;
        case "DC":
            state = "District Of Columbia";
            break;
        case "FM":
            state = "Federated States Of Micronesia";
            break;
        case "FL":
            state = "Florida";
            break;
        case "GA":
            state = "Georgia";
            break;
        case "GU":
            state = "Guam Gu";
            break;
        case "HI":
            state = "Hawaii";
            break;
        case "ID":
            state = "Idaho";
            break;
        case "IL":
            state = "Illinois";
            break;
        case "IN":
            state = "Indiana";
            break;
        case "IA":
            state = "Iowa";
            break;
        case "KS":
            state = "Kansas";
            break;
        case "KY":
            state = "Kentucky";
            break;
        case "LA":
            state = "Louisiana";
            break;
        case "ME":
            state = "Maine";
            break;
        case "MH":
            state = "Marshall Islands";
            break;
        case "MD":
            state = "Maryland";
            break;
        case "MA":
            state = "Massachusetts";
            break;
        case "MI":
            state = "Michigan";
            break;
        case "MN":
            state = "Minnesota";
            break;
        case "MS":
            state = "Mississippi";
            break;
        case "MO":
            state = "Missouri";
            break;
        case "MT":
            state = "Montana";
            break;
        case "NE":
            state = "Nebraska";
            break;
        case "NV":
            state = "Nevada";
            break;
        case "NH":
            state = "New Hampshire";
            break;
        case "NJ":
            state = "New Jersey";
            break;
        case "NM":
            state = "New Mexico";
            break;
        case "NY":
            state = "New York";
            break;
        case "NC":
            state = "North Carolina";
            break;
        case "ND":
            state = "North Dakota";
            break;
        case "MP":
            state = "Northern Mariana Islands";
            break;
        case "OH":
            state = "Ohio";
            break;
        case "OK":
            state = "Oklahoma";
            break;
        case "OR":
            state = "Oregon";
            break;
        case "PW":
            state = "Palau";
            break;
        case "PA":
            state = "Pennsylvania";
            break;
        case "PR":
            state = "Puerto Rico";
            break;
        case "RI":
            state = "Rhode Island";
            break;
        case "SC":
            state = "South Carolina";
            break;
        case "SD":
            state = "South Dakota";
            break;
        case "TN":
            state = "Tennessee";
            break;
        case "TX":
            state = "Texas";
            break;
        case "UT":
            state = "Utah";
            break;
        case "VT":
            state = "Vermont";
            break;
        case "VI":
            state = "Virgin Islands";
            break;
        case "VA":
            state = "Virginia";
            break;
        case "WA":
            state = "Washington";
            break;
        case "WV":
            state = "West Virginia";
            break;
        case "WI":
            state = "Wisconsin";
            break;
        case "WY":
            state = "Wyoming";
            break;

            //Canada
        case "AB":
            state = "Alberta";
            break;
        case "BC":
            state = "British Columbia";
            break;
        case "B.C.":
            state = "British Columbia";
            break;
        case "MB":
            state = "Manitoba";
            break;
        case "NB":
            state = "New Brunswick";
            break;
        case "NL":
            state = "Newfoundland and Labrador";
            break;
        case "NT":
            state = "Northwest Territories";
            break;
        case "NS":
            state = "Nova Scotia";
            country = "Canada";
            break;
        case "NU":
            state = "Nunavut";
            break;
        case "ON":
            state = "Ontario";
            break;
        case "PE":
            state = "Prince Edward Island";
            break;
        case "QC":
            state = "Quebec";
            break;
        case "SK":
            state = "Saskatchewan";
            break;
        case "YT":
            state = "Yukon";
            break;

        //Australia - note WA is translated above along with 
        //US Wasthington, based on the country value
        case "NSW":
            state = "New South Wales";
            break;
        case "QLD":
            state = "Queensland";
            break;
        case "SA":
            state = "South Australia";
            break;
        case "TAS":
            state = "Tasmania";
            break;
        case "VIC":
            state = "Victoria";
            break;
        case "ACT":
            state = "Australian Capital Territory";
            break;
    }
    return state;
}

//Fixes Company names in BoardEx.com as some are displayed like PROLOGIS (Prologis Trust prior to 06/2002) (De-listed 06/2011) (Quoted)
function TidyCompanyNameBoardEx(name) {
    //Remove any bracketted details
    name = name.replace(/\(.+\)/g, '');

    //Lets looks for the last tag of a company and remove any titled information i.e. INC, CORP, LTD
    var companyNameSplit = name.trim().split(" ");
    if (companyNameSplit.length > 1) {
        var companyTags = ['ltd', 'corp', 'inc', 'plc', 'lp', 'sa', 'limited'];

        var lastWord = companyNameSplit[companyNameSplit.length - 1].toLowerCase();
        if (companyTags.indexOf(lastWord) != -1) {
            name = '';
            for (var i = 0; i < companyNameSplit.length - 1; i++) {
                name += companyNameSplit[i] + " ";
            }
        }
    }

    name = name.trim();
    return name;
}

//Tidy's Contact names in BoardEx.com as some have 1 initial middle names which effect the search, this helps it make it be a little more smarter
function TidyPersonNameBoardEx(name)
{
    //Remove any bracketted details
    name = name.replace(/\(.+\)/g, '');

    //Remove some titles that appear
    name = name.replace(/Professor.? /g, '');
    name = name.replace(/Dr.? /g, '');
    name = name.replace(/Doctor.? /g, '');
    name = name.replace(/The Hon.? /g, '');
    name = name.replace(/Lady.? /g, '');
    name = name.replace(/Dame.? /g, '');
    name = name.replace(/Major.? /g, '');
    name = name.replace(/General.? /g, '');
    name = name.replace(/Brigadier.? /g, '');
    name = name.replace(/Sir.? /g, '');
    name = name.replace(/Jr/g, '');
    name = name.replace(/Sr/g, '');
    name = name.trim();

    return name;
}

//The name appears within text on JobSite.co.uk that is hard to extract straight off using xpath so we'll need to tidy it up here
function TidyPersonNameJobSite(name) {
    //Example formats;
    // Jobsite CV for	Enda O'Flaherty	27-Aug-2015 15:16  
    // telecomsjobsource.co.uk CV for	Tejeshwari Injaganeri	27-Aug-2015 12:59 

    var regexMatch = new RegExp("CV for(.*?)\\s+\\d");
    var match = regexMatch.exec(name.trim());

    if (match) {
        return match[1];
    }
}

//Need to titlecase from some sites, to match e.g state
function ToTitleCase(str) {
    if (str != null && str.length > 1) {
        return str.replace(/\w\S*/g, function (txt) { return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase(); });
    }
}
