﻿/// <reference path="~/Aventador/js/Shared/domcreation.js" />

/**
 * Holds all contents in regards to running searches for Email Tracking items
 */
var TrackingSearch = (function () {
    /**
     * Types of filters that can be applies
     */
    var emailTypeRequest = {
        All: "all",
        Opened: "opened",
        UnOpened: "unopened"
    };
    
    /**
     * Tabs avalable to select
     */
    var tab = {
        ActivityFeed: 1,
        AllTrackedEmails: 2
    };

    /**
     * Current State of tab and filter
     */
    var currentFilterState = emailTypeRequest.All;
    var currentTabState = tab.ActivityFeed;

    /**
     * Tab elements
     */
    var activityFeedOption = document.getElementById("btn-trackinglivefeed");
    var allTrackedEmailsOption = document.getElementById("btn-trackingallevents");
    
    /**
     * Validation criteria for the search box
     */
    var searchValidators = [];
    searchValidators.push(new EbstaSearchBoxValidator(function (text) {
        return text.length >= 3;
    }, "Please enter at least 3 characters."));
    searchValidators.push(new EbstaSearchBoxValidator(ValidateEmail, "Please enter a valid email address"));

    /**
     * Search box
     */
    var emailTrackingSearchBox = EbstaSearchBox(document.getElementById("trackingSearchTextBox"), Search, Search, searchValidators);

    /**
     * Set the display to reflect Activity Feed has been chosen
     * @returns {} 
     */
    function SetActivityFeedDisplay() {
        if (activityFeedOption) {
            AddClass(activityFeedOption, "active");
        }

        if (allTrackedEmailsOption) {
            RemoveClass(allTrackedEmailsOption, "active");
        }
    }

    /**
     * Set the display to reflect All Tracked Emails has been chosen
     * @returns {} 
     */
    function SetAllTrackedEmailsDisplay() {
        if (activityFeedOption) {
            RemoveClass(activityFeedOption, "active");
        }

        if (allTrackedEmailsOption) {
            AddClass(allTrackedEmailsOption, "active");
        }
    }

    /**
     * Runs the search according to the set parameters
     * @param {} ebstaSearchBoxCallback 
     * @returns {} 
     */
    function Search(ebstaSearchBoxCallback) {
        ClearTrackingButtonsActive();
        ShowTrackingHeaderProgessTypeChange();
        var searchValue = emailTrackingSearchBox.GetText();
        if (currentTabState === tab.ActivityFeed) {
            SetActivityFeedDisplay();
            if (searchValue) {
                TrackingPanelGetEmailTrackingHeadersForContact(searchValue, emailTypeRequest.Opened, ebstaSearchBoxCallback);
            } else {
                TrackingPanelGetEmailTrackingHeadersForOpenedMails();
            }

            HideComboBoxFilter(true);

        } else if (currentTabState === tab.AllTrackedEmails) {
            SetAllTrackedEmailsDisplay();
            if (searchValue) {
                TrackingPanelGetEmailTrackingHeadersForContact(searchValue, currentFilterState, ebstaSearchBoxCallback);
            } else {
                TrackingPanelGetEmailTrackingHeadersAllEvents(currentFilterState);
            }

            HideComboBoxFilter(false);
        }
    }
    
    if (activityFeedOption) {
        activityFeedOption.addEventListener("click", function () {
            currentTabState = tab.ActivityFeed;
            RemoveClass(allTrackedEmailsOption, "active");
            AddClass(this, "active");
            Search(null);
        });
    }

    if (allTrackedEmailsOption) {
        allTrackedEmailsOption.addEventListener("click", function () {
            currentTabState = tab.AllTrackedEmails;
            RemoveClass(activityFeedOption, "active");
            AddClass(this, "active");
            Search(null);
        });
    }

    var filterComboBox = document.getElementById("trackingPanelComboBoxFilterText");

    /**
     * Checks the saved state of the filter and sets filter combo box value to this
     * @returns {} 
     */
    function SetTextForComboUsingEmailTypeRequest() {
        if (currentFilterState === emailTypeRequest.All) filterComboBox.textContent = "All";
        else if (currentFilterState === emailTypeRequest.Opened) filterComboBox.textContent = "Opened Emails";
        else if (currentFilterState === emailTypeRequest.UnOpened) filterComboBox.textContent = "Unopened Emails";
    }

    return {
        EmailTypeRequest: emailTypeRequest,
        Tab: tab,
        GetFilter: function() {
            return currentFilterState;
        },
        SetFilter: function(filter) {
            currentFilterState = filter;
        },
        GetTab: function() {
            return currentTabState;
        },
        SetTab: function(tab) {
            currentTabState = tab;
        },
        SetFilterCombo: function() {
            SetTextForComboUsingEmailTypeRequest();
        },
        Search: function(searchBoxCallBack) {
            Search(searchBoxCallBack);
        },
        GetSearchText: function() {
            return emailTrackingSearchBox.GetText();
        },
        SetSearchText: function (text) {
            if(text)
            {
                emailTrackingSearchBox.SetText(text);
            }
        },
    };
})();

/**
 * Validates an email with the given parameter
 * @param {} email 
 * @returns {} 
 */
function ValidateEmail(email) {
    var re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(email);
}

/**
 * Sets the default functionality needed for the entire page
 * @returns {} 
 */
function Init() {
    HideComboBoxFilter(true);
    SetupTrackingButtonsEvents();
}
Init();

/**
 * Sets a various amount of events in order for the page to operate correctly
 * @returns {} 
 */
function SetupTrackingButtonsEvents() {
    SetupMuteNotification();
    SetupFilter();
    SetupTrackingAnalysisClick();
}

/**
 * Sets the functionality to enable/disable all notifications
 * @returns {} 
 */
function SetupMuteNotification() {
    var notificationToggle = document.getElementsByClassName("trackingPanelNotificationToggle")[0];
    if (notificationToggle) {
        MuteNotificationSetup(notificationToggle);
        Update_Notification_Switch(notificationToggle);
    }
}

/**
 * Sets the functionality where the user can filter the results they seess
 * @returns {} 
 */
function SetupFilter() {
    var comboBoxFilter = document.getElementsByClassName("trackingPanelComboBoxFilter")[0];
    if (comboBoxFilter) {
        SetHoverEventsForComboBox(comboBoxFilter);
        SetClickEventsHandlerForComboBox(comboBoxFilter);
    }
}

function GetExpectedlines(trackId) {
    var expectedLines = 1;
    var expLines = $("#trackingview" + trackId).attr('data-tracking-trackingview-' + trackId);

    if (!isNaN(expLines)) {
        expectedLines = parseInt(expLines);
    }

    return expectedLines;
}

// fetching data 
function GetTrackItems(trackId) {

    var detailsDiv = CreateDetailsDiv(trackId);
    ShowTrackingItemProgess(trackId);
    chrome.extension.sendMessage({ MessageType: "GetEmailTrackingItems", trackId: trackId });

    $(detailsDiv).slideDown();

    return false;
}

function LoadTrackItems(generalfields) {

    if (generalfields == null) {
        return;
    }

    var jsonParsedGeneral = JSON.parse(generalfields);
    if (jsonParsedGeneral.d == null) {
        return;
    }

    if (jsonParsedGeneral.d.Values.length != 1) {
        return;
    }

    var jsonString = jsonParsedGeneral.d.Values[0];
    var fields = JSON.parse(jsonString);

    if (fields.length == 0) {
        return;
    }
    var trackId = fields[0].TrackingLogId;


    //padding
    var detailsPaddingDiv = document.getElementById('trackingmainDetailsPadding' + trackId);
    // creating details row
    var detailsDiv = document.getElementById('trackingmainDetails' + trackId);

    $(".resultsLoaderTrackingItemdiv").hide();

    // items
    for (var t = 0; t < fields.length; t++) {

        var field = fields[t];

        var rowDiv = elementDivBuilder('trackingrowemb');
        rowDiv.setAttribute('data-tracking-rowemb', 'trackingrowemb' + trackId);

        var item = GetFormattedItem(field, true);

        // separator line between items
        var separatorDiv = elementDivBuilder('separatoremb');

        // frame row
        rowDiv.appendChild(item);
        rowDiv.appendChild(separatorDiv);
        detailsDiv.appendChild(rowDiv);
        //main row
        detailsPaddingDiv.appendChild(detailsDiv);
    }
}

function CreateDetailsDiv(trackId) {

    var expectedLines = GetExpectedlines(trackId);

    var parentDiv = document.getElementById(trackId);
    
    var detailsPaddingDiv = elementDivBuilderWithDataAttr('trackingmainDetailsPadding', trackId);
    var detailsDiv = elementDivBuilderWithDataAttr('trackingmainDetails', trackId);

    var base = 48;
    var newHeight = (base * expectedLines) + expectedLines;
    var newHeightWithPx = "" + newHeight + "px";

    $(detailsDiv).css('height', newHeightWithPx);
    detailsPaddingDiv.appendChild(detailsDiv);
    $(detailsPaddingDiv).hide();
    parentDiv.appendChild(detailsPaddingDiv);

    return detailsPaddingDiv;
}

// wiring up click events
function AddClickeventPerRow(trackId) {

    $('#trackingmain' + trackId).off().on('click', function (e) {
        e.preventDefault();

        $("span[data-tracking-trackingviewid='" + trackId + "']").text('View less');
        GetTrackItems(trackId);
        ChangeSelectedRowCss(trackId);

        CloseDetails(trackId);
        return false;
    });
};

function RenderNoResultInfo(contentDiv, isContactFilter) {

    var rowDiv = componentBuilder('Noresult', 'No notifications yet!');
    rowDiv.setAttribute('class', "Noresult");

    var link = document.createElement('a');
    link.href = 'https://ebstalimited.zendesk.com/hc/en-us/sections/206408747-Email-Tracking';
    link.target = '_blank';
    link.appendChild(document.createTextNode('Learn more'));

    var rowDiv2 = document.createElement('div');
    if (isContactFilter) {
        rowDiv2.appendChild(document.createTextNode('We’ll let you know as soon as ' + TrackingSearch.GetSearchText() + ' opens an email. '));
    } else {
        rowDiv2.appendChild(document.createTextNode('We’ll let you know as soon as someone opens an email. '));
    }
    rowDiv2.appendChild(link);
    rowDiv2.appendChild(document.createTextNode('.'));
    rowDiv2.className = 'Noresult noResultSubHeading';

    contentDiv.appendChild(rowDiv);
    contentDiv.appendChild(rowDiv2);
}

var CloseDetails = function (trackId) {

    $('#trackingmain' + trackId).off().on('click', function (e) {
        e.preventDefault();
        RemoveItemRows(trackId);
        $("span[data-tracking-trackingviewid='" + trackId + "']").text('View more');
        RemoveSelectedRowCss(trackId);
        AddClickeventPerRow(trackId);
        return false;
    });

    return;
}

/**
 * Creates the DOM to display for rows that show the email that was tracked as well as the associated events
 * @param {Object} field - The core object containing the event details
 * @param {boolean} embedded - Is this an email or an tracking event of an email
 * @param {Object} trackingHeaderDisplayObject - Represents a TrackingHeaderDisplay object
 * @returns {DOMElement} - The DOM to render
 */
var GetFormattedItem = function (field, embedded, trackingHeaderDisplayObject) {

    embedded = (typeof embedded === "undefined") ? false : embedded;

    var postFix = "";
    var viewedLabel = '';
    var viewedLabelClose = '';

    //Are these values to associated with an tracked email in the sub list
    if (embedded) {
        postFix = "emb";
        viewedLabel = 'Viewed "';
        viewedLabelClose = '"';
    }

    var params = [];
    var subjectText = '';

    var isTrackingLink = false;

    if (!embedded) {

        var recripientSplit = [];

        if (field.Recipient != null && typeof field.Recipient != "undefined") {
            recripientSplit = field.Recipient.split(',');
        }

        var additionalRecipientString = '';

        if (recripientSplit.length > 1) {
            additionalRecipientString = ' +' + (recripientSplit.length - 1);
        }

        var recipientSpan = elementSpanBuilder('trackingrecipient', 'To ' + CheckStringLength(recripientSplit[0], 35));

        if (recripientSplit.length > 1) {

            var otherRecipientsList = document.createElement('div');
            otherRecipientsList.className = 'otherRecipientsDiv';
            otherRecipientsList.id = 'otherRecipientsDiv' + field.Id;

            var additionalTo = document.createElement('span');
            additionalTo.appendChild(document.createTextNode(additionalRecipientString));
            additionalTo.className = 'additionalRecipientsClass';
            additionalTo.setAttribute("recId", "" + field.Id);
            recipientSpan.appendChild(additionalTo);

            for (var j = 0; j < recripientSplit.length; j++) {
                if (j != 0) {
                    var lineBreaker = document.createElement('br');

                    if (j > 1)
                    {
                        otherRecipientsList.appendChild(lineBreaker);
                    }

                    otherRecipientsList.appendChild(document.createTextNode(recripientSplit[j]));
                }
            }

            recipientSpan.appendChild(otherRecipientsList);
        }

        params.push(recipientSpan);

        if (isNullOrUndefined(field.Subject)) {
            subjectText = "Empty Subject";
        } else {
            subjectText = field.Subject;
        }
    }
    else {

        if (field.IsLink != null && field.TrackingLinkUri != null) {
            if (field.IsLink && field.TrackingLinkUri !== "") {
                isTrackingLink = true;
                subjectText = field.TrackingLinkUri;
            }
        }

        if (!isTrackingLink) {
            var parentDiv = document.getElementById(field.TrackingLogId);
            var child = parentDiv.getElementsByClassName('trackingsubjectspan')[0];
            if (child != null) {
                subjectText = child.innerText;
            }
        }
    }

    // This checks to see which data string property should be used as the old data string is still being used

    var formattedDate;

    // new data string property
    if (field.OpeningDateString != null) {
        formattedDate = FormatDateTime(field.OpeningDateString);
    } else {
        // old data string property
        formattedDate = FormatDateTime(field.OpeningDate);
    }

    var dateTimeBreakerSplit = formattedDate.split(' ');
    var dateTimeBreaker = ' on ';

    if (!embedded) {
        if (trackingHeaderDisplayObject.GetLastLinkClicked()) {
            dateTimeBreaker = 'Last Clicked on ';
        } else {
            dateTimeBreaker = 'Last Viewed on ';
        }
    }

    if (dateTimeBreakerSplit.length >= 5) {
        for(var i = 0; i < dateTimeBreakerSplit.length; i++)
        {
            if (i == 4) {
                dateTimeBreaker = dateTimeBreaker + " at " + dateTimeBreakerSplit[i];
            } else {
                if (i > 0) {
                    dateTimeBreaker = dateTimeBreaker + " ";
                }

                dateTimeBreaker = dateTimeBreaker + dateTimeBreakerSplit[i];
            }            
        }        
    }

    var formattedDateSent = FormatDateTime(field.SentDateTime);

    var dateTimeBreakerSplitSent = formattedDateSent.split(' ');
    var dateTimeBreakerSent = 'Sent on ';

    if (dateTimeBreakerSplitSent.length >= 5) {
        for (var i = 0; i < dateTimeBreakerSplitSent.length; i++) {
            if (i == 4) {
                dateTimeBreakerSent = dateTimeBreakerSent + " at " + dateTimeBreakerSplitSent[i];
            } else {
                if (i > 0) {
                    dateTimeBreakerSent = dateTimeBreakerSent + " ";
                }

                dateTimeBreakerSent = dateTimeBreakerSent + dateTimeBreakerSplitSent[i];
            }
        }
    }

    if (!embedded) {
        subjectText = CheckStringLength(subjectText, 35);
        params.push(elementSpanBuilder('trackingsubject' + postFix, subjectText));
        var sentSpan = elementSpanBuilder('trackingsent' + postFix, dateTimeBreakerSent);
        params.push(sentSpan);
    } else {
        params.push(CreateTrackingDetails(isTrackingLink, subjectText, dateTimeBreaker));
    }

    // future use
    if (embedded) {
        var deviceType = GetDevice(field.Device);
        var deviceFriendlyTitle = GetDeviceFriendlyTitle(deviceType);
    
        var locationSpan = elementSpanBuilder('trackinglocation' + deviceType + postFix, "");

        if (deviceFriendlyTitle != null && deviceFriendlyTitle.length > 0) {
            locationSpan.setAttribute("title", deviceFriendlyTitle);
        }

        params.push(locationSpan);
    }
 
    if (!embedded) {
        var totalOfViewsAndLinkClicks = trackingHeaderDisplayObject.GetNoOfViews() + trackingHeaderDisplayObject.GetNoOfLinkClicks();
        if (totalOfViewsAndLinkClicks > 0) {
            var openedSpan = elementSpanBuilder('trackingopened' + postFix, dateTimeBreaker);
            params.push(openedSpan);

            var viewMore = "View more";
            var viewSpan = elementSpanBuilder('trackingview', viewMore);
            viewSpan.setAttribute('data-tracking-trackingview-' + field.TrackingLogId, totalOfViewsAndLinkClicks);
            viewSpan.setAttribute('data-tracking-trackingviewId', field.TrackingLogId);
            viewSpan.setAttribute('id', 'trackingview' + field.TrackingLogId);
            params.push(viewSpan);
        }

        //Insert Bell Glyphicon
        var wrapperDiv = CreateElement("span", "eb-muteNotification", null, null);

        var spinnerLoader = CreateElement("span", "eb-spinnerloader", null, null);
        spinnerLoader.style.display = "none";
        wrapperDiv.appendChild(spinnerLoader);

        var notificationSwitch = CreateElement("span", "ebsta-glyphicon ebsta-glyphicon-bell", null, null);
        wrapperDiv.appendChild(notificationSwitch);

        var noNotificationSwitch = CreateElement("span", "ebsta-right-diagional-red-line ebsta-mute-notification-line", null, null);
        if (!field.HasEmailNotificationsBeenMuted) {
            noNotificationSwitch.style.display = "none";
        }
        wrapperDiv.appendChild(noNotificationSwitch);

        Setup_Click_Events_For_Mute_Icon(wrapperDiv, field.TrackingLogId);
        params.push(wrapperDiv);
    }

    // main
    var mainDiv = elementDivBuilder('trackingmain' + postFix);
    if (!embedded) {
        mainDiv.setAttribute('id', 'trackingmain' + field.TrackingLogId);
    }

    AddElementsToRoot(mainDiv, params);

    $("body").delegate(".additionalRecipientsClass", "mouseover", function (evt) {
        var idAttr = $(this).attr("recId");
        $('#otherRecipientsDiv' + idAttr).show();

        $('#otherRecipientsDiv' + idAttr).css({
            top: evt.pageY,
            left: evt.pageX
        })
    });

    $("body").delegate(".additionalRecipientsClass", "mouseleave", function () {
        var idAttr = $(this).attr("recId");
        $('#otherRecipientsDiv' + idAttr).hide();
    });


    return mainDiv;
};

//Creates the span element which displays the Link Tracking Icon
function CreateTrackingLinkIconSpan() {
    return CreateElement("span", "ebEmailLinkTrackingIcon ebsta-glyphicon ebsta-glyphicon-link ");
}

//Creates the eye image to associate with a tracked email being open (within the view more section)
function CreateTrackingIconSpan() {
    return CreateElement("span", "trackingsubjectembspan");
}

//Creates the div which contains the viewed and click details for the chosen tracked email
function CreateTrackingDetails(isTrackingLink, subjectText, dateTimeInfo) {
    var iconWithTextSpan = CreateElement("div", "ebEmailTrackingDetailsDiv", null, null);
    var detailsText = CreateElement("span", "ebEmailTrackingDetailsText", null, null);

    //If this is a tracking link we'd like to create a link so the user can navigate straight to it
    if (isTrackingLink) {
        detailsText.appendChild(CreateTextNode("Clicked "));

        var aElement = CreateElement("a", null, null, null);
        aElement.href = subjectText;
        aElement.target = "_blank";
        aElement.appendChild(CreateElement("div", "truncate", subjectText, null));

        detailsText.className = detailsText.className + ' ebsta-tracking-feed-link';

        detailsText.appendChild(aElement);

        //Create the Tracking Link Icon
        iconWithTextSpan.appendChild(CreateTrackingLinkIconSpan());
    } else {
        //Lets create the usual Email Opened view
        detailsText.appendChild(CreateTextNode("Viewed "));
        detailsText.appendChild(CreateElement("div", "truncate", subjectText, null));
        iconWithTextSpan.appendChild(CreateTrackingIconSpan());
    }

    //Append the time the email was viewed or a link was clicked
    detailsText.appendChild(CreateTextNode(" " + dateTimeInfo));
    iconWithTextSpan.appendChild(detailsText);
    return iconWithTextSpan;
}

//Attachs events onto the Bell glyphicon to trigger certain events off
function Setup_Click_Events_For_Mute_Icon(muteIconWrapper, trackingLogId) {
    if (muteIconWrapper != null) {

        var bellIcon = muteIconWrapper.getElementsByClassName("ebsta-glyphicon-bell");
        var banCircleIcon = muteIconWrapper.getElementsByClassName("ebsta-mute-notification-line");
        var spinnerLoader = muteIconWrapper.getElementsByClassName("eb-spinnerloader");

        muteIconWrapper.addEventListener("click", function (event) {
            event.stopImmediatePropagation();
            if (banCircleIcon.length === 1 && bellIcon.length === 1 && spinnerLoader.length === 1) {
                bellIcon[0].style.display = "none";
                if (banCircleIcon[0].style.display === "none") {
                    //Muting notification
                    spinnerLoader[0].style.display = "";
                    Send_Mute_Request(true, trackingLogId);
                } else {
                    //Unmuting Notification
                    spinnerLoader[0].style.display = "";
                    banCircleIcon[0].style.display = "none";
                    Send_Mute_Request(false, trackingLogId);
                }
            }
        });
    }
}

//Send request off to background.js
function Send_Mute_Request(mute, trackingLogId) {
    chrome.extension.sendMessage({
        MessageType: "Update_Tracking_Mute_Request",
        Mute: mute,
        TrackingLogId: trackingLogId
    });
}

//Receives the original request sent by Send_Mute_Request and deciphers it to update the Bell icon within the display
function UpdateEmailTrackingAlertIcon(originalRequestObject) {
    if (originalRequestObject != null) {
        if (originalRequestObject.ExtraInfo != null) {
            if (originalRequestObject.ExtraInfo.TrackingLogId != null) {
                var trackingRowElement = document.getElementById(originalRequestObject.ExtraInfo.TrackingLogId);
                if (trackingRowElement != null) {
                    var muteNotificationDiv = trackingRowElement.getElementsByClassName("eb-muteNotification");
                    if (muteNotificationDiv.length === 1) {
                        var bellIcon = muteNotificationDiv[0].getElementsByClassName("ebsta-glyphicon-bell");
                        var banCircleIcon = muteNotificationDiv[0].getElementsByClassName("ebsta-mute-notification-line");
                        var spinnerLoader = muteNotificationDiv[0].getElementsByClassName("eb-spinnerloader");

                        if (bellIcon.length > 0) {
                            bellIcon[0].style.display = "";
                        }

                        if (spinnerLoader.length > 0) {
                            spinnerLoader[0].style.display = "none";
                        }

                        if (originalRequestObject.ExtraInfo.Mute != null) {
                            if (originalRequestObject.ExtraInfo.Mute) {
                                if (banCircleIcon.length > 0) {
                                    banCircleIcon[0].style.display = "";
                                }
                            }
                        }
                    }
                }
            }
        }
    }
}

function CheckStringLength(inputString, trimIndex) {

    if (inputString != null) {
        if (inputString.length > trimIndex) {
            inputString = inputString.substring(0, trimIndex) + '...';

            return inputString;
        } else {
            return inputString;
        }
    }

    return inputString;
}

function RemoveItemRows(trackId) {

    log('remove  ' + trackId);

    //trackingmainDetails
    $("#trackingmainDetailsPadding" + trackId).slideUp(function () {
        $("div[data-tracking-trackingmaindetailspadding='trackingmainDetailsPadding" + trackId + "']").remove();
    });


    log('trackingrowembFrame ' + trackId + 'removed');
}

function ChangeSelectedRowCss(trackId) {

    $('#' + trackId).css("background-color", "#effbfb").css("border", '1px solid #dee7ed');
}

function RemoveSelectedRowCss(trackId) {

    $('#' + trackId).css("background-color", "").css("border", "");
}

function ShowTrackingItemProgess(trackId) {
 
    var loadingResults = elementDivBuilder("resultsLoaderTrackingItem");
    loadingResults.setAttribute("id", "resultsLoaderEmailTrackingItem");
    loadingResults.setAttribute('data-tracking-resultsLoaderTrackingItem', 'resultsLoaderTrackingItem' + trackId);
    var loadingGif = CreateImg("https://console.ebsta.com/images/google/ajax-loader.gif", "Loading ...");

    loadingResults.appendChild(loadingGif);
    var root = document.getElementById('trackingmainDetails' + trackId);
    root.appendChild(loadingResults);
}

function ShowTrackingHeaderProgessTypeChange() {
    var loadingResults = elementDivBuilder("resultsLoaderTrackingTypeChange");
    loadingResults.setAttribute("id", "resultsLoaderEmailTracking");
    loadingResults.setAttribute('data-tracking-resultsLoaderEmailTrackingHeader', 'resultsLoaderEmailTrackingHeader');
    var loadingGif = CreateImg("../images/ajax-loader.gif", "Loading tracking events...");

    loadingResults.appendChild(loadingGif);
    var root = document.getElementById('trackingMainContentDiv');
    root.innerHTML = "";
    root.appendChild(loadingResults);
}

function HideProgress() {

    $("#resultsLoaderEmailTracking").remove();
    $('#resultsLoaderTrackingTypeChange').remove();
}

function HideTrackingItemProgess() {
    $("#resultsLoaderEmailTrackingItem").remove();
}

var GetDevice = function (deviceType) {
    
    var device = 0;
    if (!isNaN(deviceType)) {
        device = parseInt(deviceType);
    } else {
        return "";
    }
    //UnKnown = 0,
    //ImageProxy = 1,
    //OtherOs = 2,
    //Android = 3,
    //Iphone = 4,
    //WindowsPhone = 5,
    //Blackberry = 6,
    //Windows = 7,
    //Mac = 8,
    //Linux = 9
    switch (device) {
        case 0:
            return "";
        case 1:
            return "ImageProxyGmail";
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
            return "mobileDevice";
        case 7:
        case 8:
        case 9:
            return "computerDevice";
        default:
            return "";
    }
}

// from bar.js.
function CreateImg(src, alt) {
    var spanHeader = document.createElement('img');
    spanHeader.setAttribute("src", src);
    spanHeader.setAttribute("alt", alt);
    return spanHeader;
}


var AddElementsToRoot = function (main, params) {
    for (i = 0; i < params.length; i++) {
        main.appendChild(params[i]);
    }

    return main;
}

var elementBuilder = function (type, name) {

    var cssClass = name + type;
    var element = document.createElement(type);
    element.setAttribute('class', cssClass);
    return element;
}

var elementDivBuilder = function (name) {
    var element = elementBuilder('div', name);
    return element;
}

var elementDivBuilderWithDataAttr = function (name, id) {

    var element = elementBuilder('div', name);
    element.setAttribute('id', name + id);
    element.setAttribute('data-tracking-' + name, name + id);
    return element;
}

var elementSpanBuilder = function (name, value) {

    var element = elementBuilder('span', name);
    if (!isNullOrUndefined(value)) {
        var val = document.createTextNode(value);
        element.appendChild(val);
    }
    return element;
}

var componentBuilder = function (name, value) {

    var div = elementDivBuilder(name);
    var span = elementSpanBuilder(name);

    if (!isNullOrUndefined(value)) {
        var val = document.createTextNode(value);
        span.appendChild(val);
    }

    div.appendChild(span);
    return div;
}

var isNullOrUndefined = function (value) {

    if (value != null && typeof value != "undefined") {
        return false;
    }

    return true;
}

function GetDateDisplayOptions() {

    var options = {
        weekday: "long", year: "numeric", month: "short",
        day: "numeric", hour: "2-digit", minute: "2-digit"
    };

    return options;
}

function GetLocale() {

    var localeId = 'localeId';
    var locale = "en-GB";

    try {
        var localeHidden = document.getElementById(localeId);
        locale = localeHidden.value;
    } catch (e) {
        locale = "en-GB";
    }

    return locale;
}

function FormatDateTime(dateString) {


    var date = new Date(dateString);
    return date.toDateString() + ' ' + date.toLocaleTimeString(navigator.language, { hour: '2-digit', minute: '2-digit' });
}

function log(message) {

    var debug = false;
    if (debug) {
        //console.log(message);
    }
}

function CreateTextNode(text) {
    return document.createTextNode(text);
}

/**
 * Click listener setup for the "Mute Notifications" switch
 * @param {} mainElement 
 * @returns {} 
 */
function MuteNotificationSetup(mainElement) {
    var iconDiv = mainElement.getElementsByClassName("trackingPanelNotificationToggleOn")[0],
        textSpanDiv = mainElement.getElementsByTagName("span")[0],
        redDiagonalLine = mainElement.getElementsByClassName("ebsta-mute-notification-line")[0],
        spinnerLoader = mainElement.getElementsByClassName("eb-spinnerloader")[0];

    if (iconDiv && textSpanDiv && redDiagonalLine && spinnerLoader) {
        mainElement.addEventListener("click", function() {
            if (iconDiv.className.indexOf("trackingPanelNotificationToggleOff") !== -1) {
                iconDiv.style.display = "none";
                iconDiv.className = iconDiv.className.replace("trackingPanelNotificationToggleOff", "trackingPanelNotificationToggleOn");
                textSpanDiv.textContent = "Mute All Notifications";
                redDiagonalLine.style.display = "none";
                spinnerLoader.style.display = "";
                setTimeout(function() {
                    spinnerLoader.style.display = "none";
                    Send_Notification_Message(false);
                    mainElement.style.display = "";
                    iconDiv.style.display = "";
                }, 1000);
            } else if (iconDiv.className.indexOf("trackingPanelNotificationToggleOn") !== -1) {
                iconDiv.style.display = "none";
                iconDiv.className = iconDiv.className.replace("trackingPanelNotificationToggleOn", "trackingPanelNotificationToggleOff");
                textSpanDiv.textContent = "Unmute All Notifications";
                spinnerLoader.style.display = "";
                setTimeout(function() {
                    spinnerLoader.style.display = "none";
                    Send_Notification_Message(true);
                    mainElement.style.display = "";
                    iconDiv.style.display = "";
                    redDiagonalLine.style.display = "";
                }, 1000);
            }
        });
    }
}

//Checks localstorage to see if this has has previously set these switch and update the button to display this
function Update_Notification_Switch(muteNotification) {
    chrome.extension.sendMessage({
        MessageType: "Get_LocalStorage_Item_And_Return_JSON",
        LocalStorageKey: "EmailTrackingSettings"
    }, function (response) {
        if (response) {
            if (response.LocalStorageItem) {
                if (response.LocalStorageItem.TurnOffNotifications != null) {
                    var toggleButton = $(muteNotification).find('.ebsta-glyphicon-bell');
                    if (response.LocalStorageItem.TurnOffNotifications) {
                        if ($(toggleButton).hasClass('trackingPanelNotificationToggleOn')) {
                            $(muteNotification)[0].click();
                        };
                    } else {
                        if ($(toggleButton).hasClass('trackingPanelNotificationToggleOff')) {
                            $(muteNotification)[0].click();
                        };
                    }
                }
            }
        }
    });
}

//Updates localstorage with the users selectiong
function Send_Notification_Message(turnOffNotifications) {
    chrome.extension.sendMessage({
        MessageType: "UpdateNotificationOption",
        TurnOffNotifications: turnOffNotifications
    });
}

function SetHoverEventsForComboBox(comboBoxDiv) {
    $(comboBoxDiv).hover(function ()
    {
        //Hover Over

        ShowOrHideTrackingPanelOptions("show");
    }, function () {
        //Hover out
        ShowOrHideTrackingPanelOptions("hide");
    });
}

function ShowOrHideTrackingPanelOptions(hideOrShow) {
    $('.trackingPanelComboBoxFilterOption').each(function ()
    {
        if (hideOrShow.toLowerCase() == "show")
        {
            var heightOfEachOptionDiv = 12;
            var numberOfOptions = $('.trackingPanelComboBoxFilterOption').length;

            $(this).removeClass("hidden");
            $(this).closest('.trackingPanelComboBoxFilter').css("min-height", numberOfOptions * heightOfEachOptionDiv);
        }
        else if (hideOrShow == "hide")
        {
            $(this).closest('.trackingPanelComboBoxFilter').css("min-height", "12px");
            $(this).addClass("hidden");
        }
    });
}

function SetClickEventsHandlerForComboBox(comboBoxDiv) {

    $(comboBoxDiv).find('.trackingPanelComboBoxFilterOption').off().on("click", function (e) {
        if ($(this).text()) {
            var chosenOption = $(this).text();
            $('#trackingPanelComboBoxFilterText').text(chosenOption);
            
            //BE CAREFUL HERE - AS THIS IS USING TEXT TO SET A PROPERTY ELSEWHERE, ANY VARIANCE AND THIS WONT WORK
            TrackingSearch.SetFilter($(this).attr("filter"));
            TrackingSearch.Search();

            //Hide the select options once a value has been chosen
            $('.trackingPanelComboBoxFilterOption').each(function () {
                ShowOrHideTrackingPanelOptions("hide");
            });
        }
    });
}

//Hides the combo box depending on the parameter thats passed in
function HideComboBoxFilter(shouldWeHideThis) {
    if (shouldWeHideThis) {
        $('.trackingPanelComboBoxFilterContainer').hide();
    } else {
        $('.trackingPanelComboBoxFilterContainer').css('display', '');
    }
}

function RenderContactTrackingEmptyResult() {
    //Get the main tracking div
    var mainContentDiv = document.getElementById("trackingMainContentDiv");
    if (mainContentDiv != null && typeof mainContentDiv != "undefined") {
        //Remove previous result
        $(mainContentDiv).empty();
        //Display no result message for contact
        RenderNoResultInfo(mainContentDiv, true);
    }
}

function ClearTrackingButtonsActive() {
    $('.trackinpaneltypebuttons .active').removeClass('active');
}

function HideTrackingPanelTypeButtons() {

    var buttonsDiv = document.getElementById("trackingButtonsDiv");
    if (typeof buttonsDiv != "undefined" && buttonsDiv != null) {
        buttonsDiv.innerHTML = "";
    }
}

function CheckIfEmailHasTrackingEvents(searchEmail) {
    chrome.extension.sendMessage({ MessageType: "CheckContactHasTrackingEvents", ContactEmail: searchEmail });
}

function CheckSearchTermTrackingEvents(searchTerm) {
    if (searchTerm != null && typeof searchTerm != "undefined") {
        //Check if search term is an email (or probably is so)
        if (ValidateEmail(searchTerm)) {
            //Set the data-contact-email attribute -> used to search for the email
            $(".searchTerm").attr("data-contact-email", searchTerm);
            $(".searchTerm").addClass("barContactEmail");
            //Fire off search to check for tracking events (for bar alert / icon change)
            CheckIfEmailHasTrackingEvents(searchTerm);
        }
    }
}
 
function SetupTrackingAnalysisClick() {
    $("body").undelegate(".ebsta-tracking-analysis-link", "click");
    $("body").delegate(".ebsta-tracking-analysis-link", "click", function () {

        if (localStorage.TrackingAnalysis == "true")
        {
            // goes to NEW Tracking analysis Dashboard
            var authUrl = "https://console.ebsta.com/LoginSalesforce.aspx?k=" + localStorage.WebUserId + "&j=130";
        }
        else
        {
            // Goes to current Tracking Analysis Dashboard
            var authUrl = "https://console.ebsta.com/LoginSalesforce.aspx?k=" + localStorage.WebUserId + "&j=7";
        }
        window.open(authUrl, "_blank");
    });
}

/**
 * Populates email tracking results into the view
 * @param {} generalfields 
 * @returns {} 
 */
function PopulateEmailTrackingResultsIntoView(generalfields, contactResult) {
    var fields = GetFields(generalfields);
    BuildScreen(fields, contactResult);
    HideProgress();
}

/**
 * Gets all email tracking events by status (All, Opened or Unopened)
 * @param {} mailStatus 
 * @returns {} 
 */
function TrackingPanelGetEmailTrackingHeadersAllEvents(mailStatus) {
    chrome.extension.sendMessage({ MessageType: "GetEmailTrackingHeadersAllEvents", Page: 0, MailStatus: mailStatus }, function (response) {
        PopulateEmailTrackingResultsIntoView(response);
    });
}

/**
 * Searchs for Email Tracking items by email and a status type
 * @param {string} contactEmail - search value to look for
 * @param {string} mailStatus - Type of emails to be shown, all, opened , unopened etc
 * @param {function} searchBoxCallBack - This should be related to the searchbox which was used for the search so we can update that at the end of the request
 * @returns {} 
 */
function TrackingPanelGetEmailTrackingHeadersForContact(contactEmail, mailStatus, searchBoxCallBack) {
    chrome.extension.sendMessage({ MessageType: "GetEmailTrackingHeadersForContact", Page: 0, ContactEmail: contactEmail, MailStatus: mailStatus }, function (response) {
        if (response) {
            PopulateEmailTrackingResultsIntoView(response, true);
            if (searchBoxCallBack) {
                searchBoxCallBack();
            }
        }
    });
}

/**
 * Gets most recently opened emails
 * @returns {} 
 */
function TrackingPanelGetEmailTrackingHeadersForOpenedMails() {
    chrome.extension.sendMessage({ MessageType: "GetEmailTrackingHeadersForOpenedMails", Page: 0 }, function(response) {
        PopulateEmailTrackingResultsIntoView(response);
    });
}


function BuildScreen(fields, contact) {

    var mainContentDiv = document.getElementById("trackingMainContentDiv");

    //Empty the container before inserting new records
    $(mainContentDiv).empty();

    for (var t = 0; t < fields.length; t++) {

        var field = fields[t];

        // opening div, with id
        var rowDiv = elementDivBuilder('trackingrow');
        rowDiv.setAttribute('id', field.TrackingLogId);

        let convertedTrackedItem = new TrackingHeaderDisplay(field);

        var item = GetFormattedItem(field, false, convertedTrackedItem);

        rowDiv.appendChild(item);
        mainContentDiv.appendChild(rowDiv);

        if (convertedTrackedItem.GetLastLinkClicked()) {
            rowDiv.classList.add("linkClickLastEvent");
            AddClickeventPerRow(field.TrackingLogId);
        } else {
            if (convertedTrackedItem.GetNoOfViews() > 0) {
                rowDiv.className += ' rowTrackingOpened';
                AddClickeventPerRow(field.TrackingLogId);
            }
        }
    }


    var contactResult = false;
    if (!isNullOrUndefined(contact) && contact == true)
    {
        contactResult = true;
    }


    if (fields == "undefined" || fields.length == 0 ) {
        RenderNoResultInfo(mainContentDiv, contactResult);
    }

}


function GetFields(generalfields)
{
    var fields = [];
    if (generalfields === null) {
        return fields;
    }

    if (generalfields.d === null) {
        return fields;
    }

    //var kind = JSON.parse(jsonParsedGeneral.d.EntityType);
    
    for (i = 0; i < generalfields.d.Values.length; i++) {

        var elem = JSON.parse(generalfields.d.Values[i]);
        Array.prototype.push.call(fields, elem);
    }

    return fields;
}

//Returns a friendly title for tracking location device types
function GetDeviceFriendlyTitle(deviceType) {

    var deviceFriendly = "";

    if (deviceType != null) {
        switch (deviceType.toLowerCase()) {
            case "computerdevice":
                deviceFriendly = "Desktop";
                break;
            case "mobiledevice":
                deviceFriendly = "Mobile";
                break;
            case "imageproxygmail":
                deviceFriendly = "Gmail";
                break; 
        }
    }

    return deviceFriendly;
}

/**
 * Converts an individual tracked email object received from our webservices request into an object
 * @param {Object} trackingHeaderDisplayObject - The JSON object received from the request
 * @returns {Object} - Returns helper functions to get the properties of the
 */
function TrackingHeaderDisplay(trackingHeaderDisplayObject) {

    if (!trackingHeaderDisplayObject) throw new Error("You must specify a valid TrackingHeaderDisplay object");

    let emailViews = 0, //No of opens the email has had
        emailLinkClicks = 0; //No of links that have been clicked
    let lastLinkClicked = null; //What the last linked click was

    let views = trackingHeaderDisplayObject.Views;
    if (views != null) {
        views = parseInt(views);
        if (!isNaN(views)) {
            emailViews = views;
        }
    }

    let linkClicks = trackingHeaderDisplayObject.LinkClicks;
    if (linkClicks != null) {
        linkClicks = parseInt(linkClicks);
        if (!isNaN(linkClicks)) {
            emailLinkClicks = linkClicks;
        }
    }

    const linkClicked = trackingHeaderDisplayObject.LinkClicked;
    if (linkClicked) {
        lastLinkClicked = linkClicked;
    }

    return {
        GetNoOfViews: function() {
            return emailViews;
        },
        GetNoOfLinkClicks: function() {
            return emailLinkClicks;
        },
        GetLastLinkClicked: function() {
            return lastLinkClicked;
        }
    };
}