﻿var eventSetupDom = false;  

//Helper Functions to Create DOM elements
function CreateElement(elementType, className, text, id, attributes) {
    var element = document.createElement(elementType);

    if (className) {
        element.setAttribute("class", className);
    }

    if (text) {
        element.innerText = text;
    }

    if (id) {
        element.setAttribute("id", id);
    }

    if (attributes) {
        Object.keys(attributes).forEach(function (key) {
            element.setAttribute(key, attributes[key]);
        });
    }

    return element;
}

/**
 * Creates and returns a text node element
 * @param {any} text
 */
function CreateTextNode(text) {
    return document.createTextNode(text);
}

//Creates an Image Element
function CreateImg(src, alt, className) {
    var spanHeader = document.createElement('img');

    if (src) {
        spanHeader.setAttribute("src", src);
    }

    if (alt) {
        spanHeader.setAttribute("alt", alt);
    }

    if (className) {
        spanHeader.setAttribute("class", className);
    }
    return spanHeader;
}

function CreateOptionElement(text, value) {
    var optionElem = CreateElement("option", null, text, null);
    optionElem.value = value;
    return optionElem;
}

function CreateBoostrapRow(additionalClass) {
    var newElem = CreateElement("div", "ebBootStrapRow", "", "");
    if (additionalClass != null) {
        newElem.className = newElem.className + " " + additionalClass;
    }
    return newElem;
}

/**
 * Takes a node to be inserted into the DOM, and another node of where it needs to be positioned
 * next to and will be inserted at this point
 * @param {} newNode 
 * @param {} referenceNode 
 * @returns {} 
 */
function insertAfter(newNode, referenceNode) {
    referenceNode.parentNode.insertBefore(newNode, referenceNode.nextSibling);
}

function ReplaceNodeWithNewNode(elementToReplaceWith, elementToBeReplace) {
    if (elementToBeReplace && elementToBeReplace.parentNode) {
        elementToBeReplace.parentNode.replaceChild(elementToReplaceWith, elementToBeReplace);
    }
}

/**
 * Clones a node then overwrites it with the newly created clone node
 * @param {} Node to copy 
 * @returns {} The newly created cloned node
 */
function CloneAndOverwriteNode(element) {
    var clonedNode = element.cloneNode(true);
    ReplaceNodeWithNewNode(clonedNode, element);
    return clonedNode;
}

/**
 * Takes a node to be inserted into the DOM, and another node of where it needs to be positioned
 * before and will be inserted before this element
 * @param {} newNode 
 * @param {} referenceNode 
 * @returns {} 
 */
function insertBefore(nodeToInsert, childNewNodeToBeInsertedBefore) {
    childNewNodeToBeInsertedBefore.parentNode.insertBefore(nodeToInsert, childNewNodeToBeInsertedBefore);
}

/**
 * Prepends a new element to the given element
 * @param {} nodeToInsert 
 * @param {} elementToPrepand 
 * @returns {} 
 */
function Prepend(nodeToInsert, elementToPrependTo) {
    if (elementToPrependTo != null) {
        if (elementToPrependTo.firstChild != null) {
            insertBefore(nodeToInsert, elementToPrependTo.firstChild);
        } else {
            //If there are no childs just append
            elementToPrependTo.appendChild(nodeToInsert);
        }
    }
}

/**
 * Removes the element provided
 * @param {} element 
 * @returns {} 
 */
function RemoveElement(element) {
    if (element != null && element.parentNode != null) {
        element.parentNode.removeChild(element);
    }
}

/**
 * Hides an element using Javascript only
 * @param {} element 
 * @returns {} 
 */
function HideElement(element) {
    if (element != null && element.style != null) {
        element.style.display = "none";
    }
}

/**
 * Hides an element using Javascript only
 * @param {} element 
 * @returns {} 
 */
function HideElementUsingVisibility(element) {
    if (element != null && element.style != null) {
        element.style.visibility = "hidden";
    }
}

/**
 * Hides an element using Javascript only
 * @param {} element 
 * @returns {} 
 */
function ShowElementRemovingVisibility(element) {
    if (element != null && element.style != null) {
        element.style.visibility = "";
    }
}

/**
 * Shows an element by removing the display: none style attribute
 * @param {} element 
 * @returns {} 
 */
function ShowElement(element) {
    if (element != null && element.style != null) {
        element.style.display = "inline-block";
    }
}

/**
 * Show an element using block
 * @param {} element 
 * @returns {} 
 */
function ShowElementBlock(element) {
    if (element != null && element.style != null) {
        element.style.display = "block";
    }
}

/**
 * Emptys all elements from an element
 * @param {} element 
 * @returns {} 
 */
function EmptyElement(element) {
    while (element.firstChild) {
        element.removeChild(element.firstChild);
    }
}

/**
 * Check if an element contains a classname
 * @param {} element 
 * @param {} className 
 * @returns {} 
 */
function HasClass(element, className) {
    return element.classList.contains(className);
}

/**
 * Removes a class from an element
 * @param {} element 
 * @param {} className 
 * @returns {} 
 */
function RemoveClass(element, className) {
    element.classList.remove(className);
}

/**
 * Adds a class to an element
 * @param {} element 
 * @param {} className 
 * @returns {} 
 */
function AddClass(element, className) {
    element.classList.add(className);
}

/**
 * Uses Jquerys implementation of checking if a element is hidden
 * @param {} e 
 * @returns {} 
 */
function IsVisible(e) {
    if (e != null) {
        return !!(e.offsetWidth || e.offsetHeight || e.getClientRects().length);
    }
    return false;
}

/**
 * Checks if an element exists within a given node
 * @param {} elementToFind 
 * @param {} nodeToSearch 
 * @returns {} 
 */
function IsElementInDom(elementToFind, nodeToSearch) {
    if (elementToFind && nodeToSearch) return nodeToSearch.contains(elementToFind);
    return false;
}

/**
 * Creates a Custom Check Box element which is entirely handled by the CSS (for more info check http://wtfforms.com/)
 * @param {} optionText 
 * @returns {} 
 */
function CreateCustomCheckBox(optionText, checkboxChecked) {
    var labelHeader = CreateElement("label", "ebsta-control ebsta-control-checkbox");
    labelHeader.appendChild(document.createTextNode(optionText));

    var inputAttributes = { "type": "checkbox" };

    if (checkboxChecked) {
        inputAttributes.checked = "checked";
    }

    var hiddenInput = CreateElement("input", null, null, null, inputAttributes);
    labelHeader.appendChild(hiddenInput);

    var controlIndicatorDiv = CreateElement("div", "ebsta-control-indicator");
    labelHeader.appendChild(controlIndicatorDiv);

    return labelHeader;
}

/**
 * Triggers an event type on a given element
 * @param {} eventType 
 * @param {} element 
 * @returns {} 
 */
function TriggerEventOnElement(eventType, element) {
    if (eventType != null && element != null) {
        var evt = document.createEvent("HTMLEvents");
        evt.initEvent(eventType, false, true);
        element.dispatchEvent(evt);
    }
}

/** Simulates a keyboard event */
function SimulateKeyboardEventOnElement(eventType, element, key, keyCode) {
    var evt = new KeyboardEvent(eventType, { bubbles: true, cancelable: true, key: key, keyCode: keyCode });
    element.dispatchEvent(evt);
}

/**
 * Updates the custom check box with the specified value
 * @param {} checkBoxContainer 
 * @param {} checkedAttribute 
 * @returns {} 
 */
function UpdateCheckedAttributeOnCheckBox(checkBoxContainer, checkboxChecked) {
    var findInputElement = checkBoxContainer.getElementsByTagName("input");
    if (findInputElement.length === 1) {
        findInputElement[0].checked = checkboxChecked === true ? true : false;
    }
}

/**
 * Creates a Custom Check Box element which is entirely handled by the CSS and attach handler fed in (for more info check http://wtfforms.com/)
 * @param {} optionText 
 * @returns {} 
 */
function CreateCustomCheckBoxWithListenerAttached(optionText, eventType, handler, checkboxChecked, id, classString) {
    var labelHeader = CreateElement("label", "ebsta-control ebsta-control-checkbox");
    labelHeader.appendChild(document.createTextNode(optionText));

    var inputAttributes = { "type": "checkbox"};

    if (checkboxChecked) {
        inputAttributes.checked = "checked";
    }

    var hiddenInput = CreateElement("input", null, null, null, inputAttributes);
    if (handler) {
        hiddenInput.addEventListener(eventType, handler);
    }
    labelHeader.appendChild(hiddenInput);

    var controlIndicatorDiv = CreateElement("div", "ebsta-control-indicator " + classString);
    controlIndicatorDiv.id = id;
    labelHeader.appendChild(controlIndicatorDiv);

    return labelHeader;
}

/**
 * Creates a Custom Check Box element which is entirely handled by the CSS (for more info check http://wtfforms.com/)
 * @param {} optionText 
 * @param {} id 
 * @returns {} 
 */
function CreateCustomCheckBoxV2(optionText, checkboxid) {
    var labelHeader = CreateElement("label", "ebsta-control ebsta-control-checkbox");
    labelHeader.appendChild(document.createTextNode(optionText));

    var hiddenInput = CreateElement("input", null, null, null, { "type": "checkbox", "checked": "checked" });
    hiddenInput.id = checkboxid;
    labelHeader.appendChild(hiddenInput);

    var controlIndicatorDiv = CreateElement("div", "ebsta-control-indicator", "", "EbstaChk-" + checkboxid);
    labelHeader.appendChild(controlIndicatorDiv);

    return labelHeader;
}

var EbstaCustomDropdown = (function () {

    /**
     * In relation to our Custom Dropdown element we need a list of values to a have specific structure. This method helps take a list of objects 
     * with an unknown structure be able to convert into an acceptable format for the custom dropdown so that it renders and functions 
     * correctly.
     * 
     * i.e. If you had an object like below;
     * 
     * {
     *      DisplayText: "Display Value 1",
     *      Core: "value"
     * }
     * 
     * You'd provide an object like the below;
     * 
     * {
     *      Value: "Core"
     *      Text: "DisplayText"
     * }
     * 
     * This will let the converter know that what properties to look for.
     * 
     * Each object for the custom dropdown needs to have a "value" and "text" property;
     * 
     * Text = What we should display as the text on the front end
     * Value = What the actualy value of the list item is
     * 
     * There are also additional details such as;
     * 
     * ImageUrl = If you would like to have an image prepended onto each list item you can set the URL for it here
     * @param {Object[]} listOfValues - The list of objects that need to be converted
     * @param {Object} valueMap - This needs to be an object which helps us map the properties of the list of objects provided into an 
     * object which is acceptable for our custom dropdown
     * @param {string} valueMap.Value - What is the name of property that holds the value
     * @param {string} valueMap.Text -  What is the name of property that holds the the display text
     * @returns {Object[]} - Array of objects we can use to provided our custom dropdown implementation.
     */
    function ConvertListOfValuesIntoValuesForCustomDropdown(listOfValues, valueMap) {
        const rtnArray = [];

        if (Array.isArray(listOfValues)) {
            for (let i = 0; i < listOfValues.length; i++) {
                const listValue = listOfValues[i];
                let value = listValue[valueMap.Value];
                let displayText = listValue[valueMap.Text];
                let imageUrl = listValue[valueMap.ImageUrl];
                if (value && displayText) {
                    rtnArray.push({
                        "fullDetails": listValue,
                        "value": value,
                        "text": displayText,
                        "imageUrl": imageUrl
                    });
                }
            }
        }

        return rtnArray;
    }

    /**
     * Creates our custom dropdown element
     * 
     * On selection of an item from the list it will automatically update the img and text so that it reflects the selected value
     * 
     * @param {string} id - The id for the whole of element
     * @param {object} selectedValueObject - The starting value for the dropdown
     * @param {string} selectedValueObject.value - The value of the starting object
     * @param {string} selectedValueObject.text - The display value of the starting object
     * @param {string} selectedValueObject.imageUrl - The url of the image we should display next to the selected value
     * @param {object[]} arrayOfValuesForTheList - A list of objects to render in the dropdown
     * @param {string} arrayOfValuesForTheList[].value - The value of the starting object
     * @param {string} arrayOfValuesForTheList[].text - The display value of the starting object
     * @param {string} arrayOfValuesForTheList[].imageUrl - The url of the image we should display next to the selected value
     * @param {function} selectValueCallback - A function to fire when a value is selected in the dropdown
     * @returns {DOMElement} - The element to render
     */
    function CreateCustomDropDown(id, selectedValueObject, arrayOfValuesForTheList, selectValueCallback) {

        const mainDiv = CreateElement("div", "ebsta-dd ebsta-wrapper-dropdown", null, id);

        //This is the container for where we'll show the selected value from the dropdown
        const selectedContainer = CreateElement("div", "ebsta-wrapper-dropdown-selected");

        /**
         * The image to prepnd next to the selected value, we'll hide this initally and will only 
         * be displayed if a image url has been set
         */
        const selectValueImg = CreateElement("img", null, null, null);
        HideElement(selectValueImg);

        const selectedValueTextSpan = CreateElement("span", "ebsta-selectedtext ebsta-ellipsis");

        /*
         * If a value has been selected then apply this to the selected value elements so that it looks like 
         * this value is currently the selected value
         */
        if (selectedValueObject) {

            //Only show and change if a imageUrl is set
            if (selectedValueObject.imageUrl) {
                ShowElement(selectValueImg);
                selectValueImg.src = selectedValueObject.imageUrl;
            }

            selectedValueTextSpan.innerText = selectedValueObject.text;
            mainDiv.setAttribute('selectedvalue', selectedValueObject.value);
        }

        selectedContainer.appendChild(selectValueImg);
        selectedContainer.appendChild(selectedValueTextSpan);
        mainDiv.appendChild(selectedContainer);

        //The container for all the items that can be selected
        let ul = CreateElement("ul", "ebsta-dropdown");

        //Loop through all the values to create a list item for each of them
        if (arrayOfValuesForTheList && Array.isArray(arrayOfValuesForTheList)) {
            for (let i = 0; i < arrayOfValuesForTheList.length; i++) {
                const listValue = arrayOfValuesForTheList[i];

                const listElement = CreateElement("li");

                //Simple container so we could style items a bit easier
                const containerDiv = CreateElement("div");

                //Only create an img element if a imageUrl is set
                if (listValue.imageUrl) {
                    containerDiv.appendChild(CreateElement("img", null, null, null, { src: listValue.imageUrl }));
                }

                const aTag = CreateElement("a", "ebsta-ellipsis");
                aTag.href = '#';
                aTag.innerText = listValue.text;
                aTag.setAttribute('val', listValue.value);

                /*
                 * Set up the click listener on each item so when a user clicks the item in a list we update the 
                 * display to respect the selection and also fire the callback(if this any)
                 */
                (function (lSelectedValueObject, lSelectValueCallback) {
                    listElement.addEventListener("click", function (event) {
                        if (lSelectedValueObject.imageUrl) {
                            ShowElement(selectValueImg);
                            selectValueImg.src = lSelectedValueObject.imageUrl;
                        }
                        selectedValueTextSpan.innerText = lSelectedValueObject.text;
                        mainDiv.setAttribute('selectedvalue', lSelectedValueObject.value);
                        mainDiv.classList.remove("active");

                        if (typeof lSelectValueCallback === "function") {
                            lSelectValueCallback(lSelectedValueObject);
                        }

                        event.preventDefault();
                        event.stopPropagation();
                    });
                })(listValue, selectValueCallback);

                containerDiv.appendChild(aTag);
                listElement.appendChild(containerDiv);
                ul.appendChild(listElement);
            }
        }

        mainDiv.appendChild(ul);

        AssignClickEventToCustomDropDown(mainDiv);

        return mainDiv;
    }

    /**
     * Applies a click listener on list item on our custom dropdown. As long as the dropdown is "Active" we'll hide
     * the dropdown after the click event.
     * @param {DOMElement} customDropDownElement - The whole dropdown element
     */
    function AssignClickEventToCustomDropDown(customDropDownElement) {
        customDropDownElement.addEventListener("click", function (event) {
            //Put a disabled attribute on the element if you want to stop user from selecting anything
            if (!customDropDownElement.hasAttribute("disabled")) {

                // Checks if the drop down has an active class
                const isActive = customDropDownElement.classList.contains("active");

                // firstly remove "active" class from any other ebsta wrapper dropdowns
                $(".ebsta-wrapper-dropdown").removeClass("active");

                // if there is not an active class on the dropdown then add a class "active"
                if (isActive !== true) {
                    customDropDownElement.classList.add("active");
                }
            }

            /*
             * Helps prevent trigger the event we have on the whole document which removes to ehe "Active" class form the dropdown
             * as this would just hide the dropdown each time you tried to open it.
             */
            event.preventDefault();
            event.stopPropagation();
        });
    }

    return {
        ConvertListOfValuesIntoValuesForCustomDropdown: ConvertListOfValuesIntoValuesForCustomDropdown,
        CreateCustomDropDown: CreateCustomDropDown
    };
})();

/**
 * Related to CreateTabs function, you must pass in a list of these to create tabs
 * @param {} tabHeader 
 * @param {} sectionId 
 * @param {} tabContent 
 * @returns {} 
 */
function CreateTabObject(tabHeader, sectionId, tabContent) {
    return {
        TabHeader: tabHeader,
        SectionId: sectionId,
        TabContent: tabContent
    }
}

/**
 * Takes in a list of CreateTabObject() and creates tabs based on their values, this does not set the tabs up though you will need to use
 * new CBPFWTabs(DOMELEMENTHERE) to set the styling and events 
 * @param {} elementId 
 * @param {} tabObjects 
 * @returns {} 
 */
function CreateTabs(elementId, tabObjects) {
    if (tabObjects != null && tabObjects.length > 0) {
        var mainTabsDiv = CreateElement("div", "ebsta-tabs ebsta-tabs-style-linebox", null, elementId);

        var mainTabsNav = CreateElement("nav");
        var mainTabUnorderedList = CreateElement("ul");

        var contentSections = CreateElement("div", "ebsta-content-wrap");

        tabObjects.forEach(function (tabObject) {
            //Creates the actual tab itself
            var listElement = CreateElement("li");
            var aTag = CreateElement("a", null, null, null, { href: "#" + tabObject.SectionId });
            var labelSpan = CreateElement("span", null, tabObject.TabHeader);
            aTag.appendChild(labelSpan);
            listElement.appendChild(aTag);
            mainTabUnorderedList.appendChild(listElement);

            //Create Content Section where the main info will sit and the click event on the tabs will show once clicke
            var contentSection = CreateElement("section", null, null, tabObject.SectionId);
            contentSection.appendChild(tabObject.TabContent);
            contentSections.appendChild(contentSection);
        });

        mainTabsNav.appendChild(mainTabUnorderedList);
        mainTabsDiv.appendChild(mainTabsNav);
        mainTabsDiv.appendChild(contentSections);
        return mainTabsDiv;
    }

    return null;
}

/**
 * ToolTip closure to help create a tooltip on hover of element. 
 * You need to append it to the element you wish for it to appear when hovering over and give the element that you're appending it to this class - "ebsta-tooltip"
 * @returns {} 
 */
function ToolTip() {
    return {
        CreateToolTip: function (text) {
            var toolTipContainer = CreateElement("div", "ebsta-tooltip-content ebsta-tooltip-content-uparrow");

            var toolTipText = CreateElement("div", "ebsta-tooltip-text");
            var toolTipInner = CreateElement("div", "ebsta-tooltip-inner", text);
            toolTipText.appendChild(toolTipInner);
            toolTipContainer.appendChild(toolTipText);

            return toolTipContainer;
        },
        CreateSmallToolTip: function (text) {
            var toolTipContainer = CreateElement("div", "ebsta-tooltip-content ebsta-tooltip-content-small ebsta-tooltip-content-small-uparrow");

            var toolTipText = CreateElement("div", "ebsta-tooltip-text");
            var toolTipInner = CreateElement("div", "ebsta-tooltip-inner", text);
            toolTipText.appendChild(toolTipInner);
            toolTipContainer.appendChild(toolTipText);

            return toolTipContainer;
        }
    }
}

/**
 * Tells you if the string provided a valid JSON
 * @param {} str 
 * @returns {} 
 */
function IsValidJSON(str) {
    try {
        return (JSON.parse(str) && !!str);
    } catch (e) {
        return false;
    }
}

/**
 * Couples with EbstaSearchBox. Provides a way of authenticating values within the search box.
 * Providing a function which has one parameter, which would be the textbox value, 
 * then running that function to return a bool to denote if the validation has passed. 
 * 
 * @param { A function that has one parameter and returns a boolean } validationFunction 
 * @param { A string which will be display in the placeholder if not valid} errorMessage 
 * @returns {} 
 */
function EbstaSearchBoxValidator(validationFunction, errorMessage) {
    
    return {
        Validate : function(inputText) {
            return validationFunction(inputText);
        },
        GetErrorMessage : function() {
            return errorMessage;
        }
    }
}

/**
 * Creates a search box conforming to the EbstaSearchBox closure
 * @param {string} placeHolderText - Placeholder text for the search box
 * @returns {DOMelement} - The search element
 */
function CreateEbstaSearchBox(placeHolderText) {
    let parentDiv = CreateElement("div", "ebsta-search-textbox");
    parentDiv.appendChild(CreateElement("i", "fa fa-search"));
    parentDiv.appendChild(CreateElement("input", null, null, null, { type: "text", placeholder: placeHolderText }));
    parentDiv.appendChild(CreateElement("i", "loaderAndClear fa"));
    return parentDiv;
}

/**
 * Custom SearchBox setup, it must follow the below structure;
 * 
 * <div class="ebsta-search-textbox">
        <i class="fa fa-search"></i>
        <input type="text"/>
        <i class="loaderAndClear fa"></i>
    </div>

 * Running a search will display a loader within the textbox, there is a callback which is fired 
 * into functionToCallOnSearch which needs to be run once the search has 
 * finished which will update the icon and also the state of the search so they can run additional searches.
 *
 * @param {DOMelement} containerElement - The DOM containing the whole of element as defined by teh strucute above
 * @param {function} functionToCallOnSearch - A function which has one parameter (a callback)
 * @param {function} functionToCallOnClear - What to call when a user clicks the clear icon
 * @param {array} ebstaSearchBoxValidators - List of EbstaSearchBoxValidator's
 * @returns {object} - functions accessing properties of the text box 
 */
function EbstaSearchBox(containerElement, functionToCallOnSearch, functionToCallOnClear, ebstaSearchBoxValidators) {
    this.containerElement = containerElement;
    var inputElement = containerElement.getElementsByTagName("input")[0];
    var clearSearchElement = containerElement.getElementsByClassName("loaderAndClear")[0];
    
    if (!inputElement || !clearSearchElement) throw error("Not all elements found to setup Ebsta Search Box. Please create the DOM as specified.");

    var searchModeEnum = {
        ReadyToSearch: 1,
        Searching: 2,
        FinishedSearch: 3
    };

    var searchMode = searchModeEnum.ReadyToSearch;
    var clearTextIconClass = "fa-times";
    var arrayOfClassesForLoader = ["fa-spinner", "fa-pulse", "fa-fw"];

    /**
     * Adds or removes the classes associated with making the icon a loader
     * @param {} addClasses 
     * @returns {} 
     */
    function AddOrRemoveClassesForSpinner(addClasses) {
        arrayOfClassesForLoader.forEach(function (className) {
            if (addClasses) {
                AddClass(clearSearchElement, className);
            } else {
                RemoveClass(clearSearchElement, className);
            }
        });
    }

    /**
     * Sets up the textbox to have a clear icon
     * @returns {} 
     */
    function SetClearIcon() {
        AddOrRemoveClassesForSpinner(false);
        AddClass(clearSearchElement, clearTextIconClass);
        searchMode = searchModeEnum.FinishedSearch;
    }

    /**
     * Original state of placeholder that we can return to if needed
     */
    var originalPlaceHolder = inputElement.placeholder;

    /**
     * Runs all EbstaSearchBoxValidator's to ensure value is good
     * @returns {} 
     */
    function Validate() {
        if (ebstaSearchBoxValidators && ebstaSearchBoxValidators.length > 0) {
            for (var i = 0; i < ebstaSearchBoxValidators.length; i++) {
                var ebstaSearchBoxValidator = ebstaSearchBoxValidators[i];
                if (ebstaSearchBoxValidator.Validate(inputElement.value) === false) {
                    return ebstaSearchBoxValidator.GetErrorMessage();
                }
            }
        }

        return null;
    }

    /**
     * Clears text box
     * @returns {} 
     */
    function ClearInput() {
        inputElement.value = "";
    }

    /**
     * Validates the input then runs a search if it passes if not displays error message
     * @returns {} 
     */
    function Search() {
        var validateError = Validate();
        if (validateError) {
            ClearInput();
            inputElement.placeholder = validateError;
            AddClass(containerElement, "error");
            RemoveClass(clearSearchElement, clearTextIconClass);
        } else {
            RemoveClass(containerElement, "error");
            AddOrRemoveClassesForSpinner(true);
            if (typeof functionToCallOnSearch === 'function') {
                functionToCallOnSearch(SetClearIcon);
            }
            searchMode = searchModeEnum.Searching;
            inputElement.placeholder = originalPlaceHolder;
        }
    }
    
    function ClearSearchClickListener() {
        if (searchMode === searchModeEnum.FinishedSearch) {
            ClearInput();
            if (functionToCallOnClear) {
                functionToCallOnClear();
            }
            RemoveClass(clearSearchElement, clearTextIconClass);
            searchMode = searchModeEnum.ReadyToSearch;
        }
    }

    clearSearchElement.addEventListener("click", ClearSearchClickListener);

    inputElement.addEventListener("keyup", function (event) {
        //Make sure a search isn't already running before running another one
        if (searchMode === searchModeEnum.ReadyToSearch || searchMode === searchModeEnum.FinishedSearch) {
            searchMode = searchModeEnum.ReadyToSearch;
            //Enter
            if (event.keyCode === 13) {
                Search();
            }
        }
    });

    return {
        SetText: function (text) {
            inputElement.value = text;
            SetClearIcon();
        },
        GetText: function () {
            return inputElement.value;
        },
        Search: Search
    };
}

/**
 * Queries the entire DOM to find the current element being hovered over
 * @returns {} 
 */
function GetInnermostHoveredElement() {
    var n = document.querySelector(":hover");
    var nn;
    while (n) {
        nn = n;
        n = nn.querySelector(":hover");
    }
    return nn;
}

/**
 * Creates a retry process around a given function, and fires off the corresponding callback if it succeeds or fails
 * @param {} howManyTimesToRetry 
 * @param {} waitBetweenEachRetryInSeconds 
 * @param { The function which is called at each iteration, this must return a true to determine uf we should stop the retry process } functionToReturnBool 
 * @param {} successfulcallback 
 * @param {} unsuccessFulCallBack 
 * @returns {} 
 */
function RetryWithCallback(howManyTimesToRetry, waitBetweenEachRetryInSeconds, functionToReturnBool, successfulcallback, unsuccessFulCallBack) {
    var retryCount = 0;
    waitBetweenEachRetryInSeconds = waitBetweenEachRetryInSeconds * 1000;

    var retryInterval = setInterval(function () {
        if (retryCount < howManyTimesToRetry) {
            retryCount++;
            if (functionToReturnBool() === true) {
                clearInterval(retryInterval);
                if (successfulcallback) {
                    successfulcallback();
                }
            }
        } else {
            clearInterval(retryInterval);
            if (unsuccessFulCallBack) {
                unsuccessFulCallBack();
            }
        }

    }, waitBetweenEachRetryInSeconds);
}

/**
 * Opens a new tab for the given uri
 * @param {any} uri
 */
function OpenNewTab(uri) {
    window.open(uri, "_tab");
}

/**
 * Validates an email with the given parameter
 * @param {} email 
 * @returns {} 
 */
function ValidateEmail(email) {
    var re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(email);
}

/**
 * Get a parameter from a URL by name
 * @param {any} name - name of url parameter
 * @returns {string} - A string value of the parameter or a empty string
 */
function GetParameterByName(name) {
    name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
    var regexS = "[\\?&]" + name + "=([^&#]*)";
    var regex = new RegExp(regexS);
    var results = regex.exec(window.location.search);
    if (results === null)
        return "";
    else
        return decodeURIComponent(results[1].replace(/\+/g, " "));
}

/**
* Contains all functions around displaying messages
*/
var ebstaMessaging = (function () {

    /**
     * Creates a small message that overlays and disappears after a small time
     * @param {string} text - Text of message
     * @param {boolean} successfull - If true the styling will reflect a success, false and it won't
     */
    function CreateAndShowMessage(text, successfull) {
        var msgElement = CreateElement("div", "ebsta-message-alert ebsta-alert", text, null);

        if (successfull) {
            AddClass(msgElement, "ebsta-alert-success");
        } else {
            AddClass(msgElement, "ebsta-alert-danger");
        }

        document.body.appendChild(msgElement);

        setTimeout(function () {
            (function (msgElement) {
                msgElement.remove();
            })(msgElement);
        }, 2500);
    }

    if (eventSetupDom === false) {
        if (typeof window.addEventListener != 'undefined') {
            window.addEventListener("message", EbstaMessageListener, false);
            eventSetupDom = true;
        }
    }

    /**
     * Detects message events we receive
     * @param {MessagEvent} event - MessagEvent
     */
    function EbstaMessageListener(event) {
        if (event && event.data) {
            var eventData = event.data;
            switch (eventData.RequestType) {
                case "SHOWMESSAGE":
                    if (eventData.MessageText !== null) {
                        ebstaMessaging.ShowMessage(eventData.MessageText, eventData.Successful);
                    }
                    break;
                default: break;
            }
        }
    }

    return {
        ShowMessage: function (text, successfull) {
            CreateAndShowMessage(text, successfull);
        }
    };
})();

/** 
 * Helps tidy up a name we parse from the DOM 
 * @returns {object} - Public functions avaliable to call
 */
var EbstaNameSanitizer = (function () {

    /**
     * Remove range of unwanted utf characters - emoticons
     * Miscellaneous Symbols
     * Miscellaneous Symbols and Pictographs
     * @param {any} stringIn - String to clean
     * @returns {string} - cleanedup string
     */
    function RemoveUnwatedUTFCharRange(stringIn) {
        var cleanedString = "";

        if (stringIn !== "") {
            //split regex for readability 

            //BMP only - dont need surrogates 
            //[\uE000-\uF8FF] : Private usage area
            //[\u20A0-\u20CF] : Currency symbols
            //[\u2190-\u2FFF] : Wide cover -  Arrows to Ideographic Description Characters
            //cannot remove set - includes extended latin chars
            cleanedString = stringIn.replace(/([\uE000-\uF8FF]|[\u20A0-\u20CF]|[\u2190-\u2FFF])/g, '');

            //supplementary - need to use surrogates
            //\uD83C[\uDF00-\uDFFF] : Miscellaneous Symbols 
            //\uD83D[\uDC00-\uDDFF] : Miscellaneous Symbols and Pictographs
            //\uD83D[\uDE01-\uDE4F] : Emoticons: surrogate - \uD83D
            //\uD83D[\uDE80-\uDEFF] : Transport and map symbols: surrogate - \uD83D
            //\uD83C[\uDD70-\uDE51] : Enclosed Alphanumerics: surrogate - \D83C
            if (cleanedString !== "") {
                cleanedString = cleanedString.replace(/(\uD83C[\uDF00-\uDFFF]|\uD83D[\uDC00-\uDDFF]|[\uD83D[\uDE01-\uDE4F]|[\uD83D[\uDE80-\uDEFF]|\uD83C[\uDD70-\uDE51])/g, '');
            }

            cleanedString = cleanedString.trim();
        }

        return cleanedString;
    }

    /**
     * Tidys up a string found on the DOM removing generic values so we have a nice trimmed down search value
     * @param {string} name - the string to tidy up
     * @param {any} searchType - what type of search (1 = contact, 2 = company)
     * @returns {string} - The tidied string
     */
    function TidyName(name, searchType) {
        name = decodeURIComponent(name);

        //clean out emoticons
        var utfRangeCleaned = "";
        utfRangeCleaned = RemoveUnwatedUTFCharRange(name);
        if (utfRangeCleaned !== "") {
            name = utfRangeCleaned;
        }

        //Some names have bracketed titles in the name e.g. (John Smith (CBIP)), we need to remove these from the full name if they are present
        var nameRemoveBracketedTitle = name.split(" ");
        for (var i = 0; i < nameRemoveBracketedTitle.length; i++) {
            var len = nameRemoveBracketedTitle[i].length;
            len--;
            var x = nameRemoveBracketedTitle[i].toString;

            if (nameRemoveBracketedTitle[i].indexOf("(") === 0 && nameRemoveBracketedTitle[i].indexOf(")") === len) {
                name = name.replace(nameRemoveBracketedTitle[i], '');
            }
        }

        //in some instances (e.g. nursingjobs.us) the name and jobtitle are together in one div.
        //if so we need to separate out the name in order to trigger a positive drop
        if (name.indexOf(" - ") !== -1) {
            name = name.split(" - ")[0];
        }

        //clean out spaces caused by trimming out any bracketed titles
        name = name.trim();

        //Now carry on with the tidy
        var fullname = name,
            blacklist = [
                "(", ")", "/", ",",
                "Dr.",
                "Dr",
                "Mr.",
                "Mr",
                "Mrs.",
                "Mrs",
                "Miss",
                "Ms",
                "Prof.",
                "CA",
                "CPA",
                "CFA",
                "CTP",
                "MACC",
                "MBA",
                "CFE",
                "CBV",
                "MTAX",
                "MTax",
                "CGMA",
                "CIA",
                "CISA",
                "CGA",
                "CMA",
                "Download CV ",
                "CIOB",
                "C.I.O.B",
                "FIOB",
                "F.I.O.B",
                "MRICS",
                "M.R.I.C.S",
                "FRICS",
                "F.R.I.C.S",
                "CMIOSH",
                "C.M.I.O.S.H",
                "NEBOSH",
                "MProp",
                "MSISV",
                "MRISM",
                "LEED",
                "GA",
                "CIMA",
                "Bsc",
                "BSc",
                "BSC",
                "Msc",
                "MSc",
                "MSC",
                "M.Sc",
                "M.Sc.",
                "B.Sc",
                "B.Sc.",
                "MA",
                "Ma",
                "Md",
                "MD",
                "IEng",
                "MIET",
                "PMP",
                "FIRP",
                "CV",
                "ACCA",
                "LION",
                "PQS",
                "B.Arch",
                "C.P.M.",
                "C.P.M",
                "CIPD",
                "BEng",
                "MIoR",
                "MIOR",
                "MIRP",
                "PhD",
                "FCA",
                "FCCA"
            ];

        var whiteList = [
            //dutch / south african
            "van",
            "vander",
            "von",
            "der",
            "den",
            "vd",
            //spanish
            "de",
            "del",
            "el",
            //irish
            "Ó"
        ];

        if (fullname.indexOf(',') > -1) {
            fullname = fullname.substring(0, fullname.indexOf(','));
        }
        if (fullname.indexOf(';') > -1) {
            fullname = fullname.substring(0, fullname.indexOf(';'));
        }
        if (fullname.indexOf(':') > -1) {
            fullname = fullname.substring(0, fullname.indexOf(':'));
        }

        var nameSplit = fullname.split(/\s|\(|\)|\,/);
        for (let i = 0; i < blacklist.length; i++) {
            for (var j = 0; j < nameSplit.length; j++) {
                if (nameSplit[j] != undefined) {
                    var x = nameSplit[j];
                    if (x == '-') {
                        //remove hypen from the name array
                        nameSplit.splice(j, 1);
                        //delete nameSplit[j];
                    }
                    else {
                        if (x.indexOf('-') == 0) {
                            x = x.substring(1, x.length);
                            nameSplit[j] = x;
                        }
                    }

                    if (nameSplit[j] === blacklist[i]) {
                        //remove the blacklisted item fully from the name array
                        nameSplit.splice(j, 1);
                        //delete nameSplit[j];
                    }
                }
            }
        }

        //and are we searching for people - not company: search type 2 = company
        if (searchType != "2") {

            //after clean up - do we have multiple words?
            if (nameSplit.length > 1) {

                //if we do - the fullname should only be the first word and last word - discard any middle names
                //unless the first or last word are single characters - then we may have something unexpected 
                //- may be an emoticon or similar, fallback to restore the name
                if (nameSplit.length > 2) {

                    //Find if any of the middle names are whitelisted by us
                    var whiteMiddle = false;
                    for (var i = 1; i < nameSplit.length - 1; i++) {
                        for (var j = 0; j < whiteList.length; j++) {
                            if (nameSplit[i].toLowerCase() == whiteList[j].toLowerCase()) {
                                whiteMiddle = true;
                                break;
                            }
                        }
                    }

                    //If any of the middle names are whitelisted then create a string using the middle names
                    //and include in the whole middle name within the end result
                    if (whiteMiddle) {
                        const removeFirstName = nameSplit.slice(1);
                        removeFirstName.pop();
                        whiteMiddle = removeFirstName.join(" ");
                    }

                    //check for single char first and last word - if found fallback to restore name
                    //- mitigates risk of running unexpected search on for example an emoticon
                    if (nameSplit[0].length === 1 || nameSplit[nameSplit.length - 1].length === 1) {
                        fullname = nameSplit.join(" ");
                    } else {
                        //if we have middle parts of name that was whitelisted - add
                        if (whiteMiddle != '') {
                            fullname = nameSplit[0] + " " + whiteMiddle + " " + nameSplit[nameSplit.length - 1];
                        } else { //if not, use only first and last word
                            fullname = nameSplit[0] + " " + nameSplit[nameSplit.length - 1];
                        }
                    }

                } else { //if we only have two words use first and last word
                    fullname = nameSplit.join(" ");
                }

            } else {
                //if only single word - use as search term
                fullname = nameSplit[0];
            }
        } else {
            //if its a company - put the full remaning string back together
            fullname = nameSplit.join(" ");
        }

        //Removes a fullstop off the end of search term if there is one
        if (fullname.endsWith(".")) {
            fullname = fullname.substring(0, fullname.length - 1);
        }

        return fullname;
    }

    return {
        TidyName: TidyName
    };
})();

/** Module container generic helper functions to create DOM elements */
var ebstaDomElementCreator = (function () {

    /**
     * Creates a simple dropdown using <select> and <option> tags
     * @param {string} className - The class name to be used for the select element
     * @param {string} id - The id to be used for the select element
     * @param {array} optionValues - At list of values to render as the options
     * @param {string} optionValues.text - The display text for the option
     * @param {string} optionValues.value - The value for the option
     * @returns {DOMelement} - The <select> dropdown element
     */
    function CreateDropDown(className, id, optionValues) {
        var select = CreateElement("select", className, null, id);
        if (optionValues && optionValues.length > 0) {
            for (let i = 0; i < optionValues.length; i++) {
                select.appendChild(CreateElement("option", null, optionValues[i].text, null, { value: optionValues[i].value }));
            }
        }
        return select;
    }

    return {
        CreateDropDown: CreateDropDown
    };
})();

/**
 * Provides a validation object which is used to verify values put into EbstaTagsInput
 * @param {function} validationFunction - A function which takes its first argument as a string so it can run the validation process on it
 * @param {string} errorMessage - The related error message to the validation function
 * @return {Object<EbstaTagValidator>} - The public functions
 */
function EbstaTagValidator(validationFunction, errorMessage) {
    return {
        Validate: function(inputText) {
            return validationFunction(inputText);
        },
        GetErrorMessage: function() {
            return errorMessage;
        }
    };
}

/**
 * Module to create an input which creates removable tags
 * @param {Array<EbstaTagValidator>} listOfEbstaTagValidators - A list of EbstaTagValidator which will verify the values entered into the input
 * @return {Object} - An object with functions give you access to to the inputs properties
 */
function EbstaTagsInput(listOfEbstaTagValidators) {
    var mainContainer = CreateElement("div", "ebsta-tags-input-container");

    var input = CreateElement("input");
    mainContainer.appendChild(input);

    /**
     * When clicking in this container make sure to set the focus to the text box
     */
    mainContainer.addEventListener("click", function () {
        input.focus();
        ScrollToBottomOfContainer();
    });

    /**
     * When the user ventures away from the text box try and add a tag
     */
    input.addEventListener("blur", function (event) {
        AddTag(this.value);
    });

    /**
     * Detect key events to create tags
     */
    input.addEventListener("keydown", function (event) {
        switch (event.keyCode) {
            case 9: //TAB
            case 13: //Enter
                if (this.value) {
                    event.preventDefault();
                    AddTag(this.value);
                    input.focus();
                }
                break;
            case 8: //BackSpace
                if (!this.value) {
                    RemoveLastAddedTag();
                }
                break;
            default:
                break;
        }
    });

    input.addEventListener("input", function () {
        if (this.value.length < 40) {
            input.size = this.value.length;
        }
    });

    input.addEventListener("paste", function (event) {
        // Stop the default pasting event and intercept the content. 
        // Split the content up by a space or ';' and try and verify the contents
        event.preventDefault();
        if (event && event.clipboardData) {
            const clipBoardText = event.clipboardData.getData("text");
            if (clipBoardText) {
                const splitString = clipBoardText.split(/\s|\;/);
                for (let i = 0; i < splitString.length; i++) {
                    AddTag(splitString[i]);
                }
            }
        }
    });

    /** Scrolls the main container to the bottom */
    function ScrollToBottomOfContainer() {
        mainContainer.scrollTop = mainContainer.scrollHeight;
    }

    /**
     * Removes the last added tag from the container
     */
    function RemoveLastAddedTag() {
        const allTags = mainContainer.getElementsByClassName("ebsta-tags-input-tag");
        if (allTags.length > 0) {
            allTags[allTags.length - 1].remove();
        }
    }

    /**
     * Removes the tag when a user clicks on the cross icon
     * @param {DOMElement} crossIcon - The cross symbol within the tag
     * @param {DOMElement} wholeTagElement - The whole tag element we need to remove
     */
    function SetupCrossIconListener(crossIcon, wholeTagElement) {
        (function (crossIcon, wholeTagElement) {
            crossIcon.addEventListener("click",
                function (event) {
                    wholeTagElement.remove();

                    // Stops the event from bubbling up and triggering any click events on containers i.e. we have 
                    // a click which scrolls to the bottom in the parent container so this helps prevent that from 
                    // happening each time a tag is deleted
                    event.stopPropagation();
                });
        })(crossIcon, wholeTagElement);
    }

    /**
     * Iterates through a list of validation functions and returns the error message for the first one that fails
     * @param {string} inputValue - The text to validate
     * @return {string} - Returns the error message for the first validation function that failed
     */
    function ValidateTag(inputValue) {
        if (Array.isArray(listOfEbstaTagValidators)) {
            for (let i = 0; i < listOfEbstaTagValidators.length; i++) {
                if (typeof listOfEbstaTagValidators[i].Validate === "function") {
                    if (listOfEbstaTagValidators[i].Validate(inputValue) === false) {
                        return listOfEbstaTagValidators[i].GetErrorMessage();
                    }
                }
            }
        }

        return null;
    }

    /**
     * Adds a tag with the value in the current input
     */
    function AddTag(inputValue) {
        if (inputValue) {
            inputValue = inputValue.trim();
            if (inputValue) {
                const tagContainer = CreateElement("div", "ebsta-tags-input-tag", null, null, { title: inputValue });
                const tagContent = CreateElement("span",
                    "ebsta-tags-input-tag-value",
                    inputValue,
                    null,
                    {
                        "tag-value": inputValue
                    });

                const ebstaTagValidationMessage = ValidateTag(inputValue);
                if (ebstaTagValidationMessage !== null) {
                    tagContainer.classList.add("error");
                }

                const crossIcon = CreateElement("i", "fa fa-times");
                SetupCrossIconListener(crossIcon, tagContainer);
              
                tagContainer.appendChild(tagContent);
                tagContainer.appendChild(crossIcon);
                mainContainer.insertBefore(tagContainer, input);
                input.value = "";
                input.size = 1;
                ScrollToBottomOfContainer();
            }
        }
    }

    /**
     * Gets all the valid tags currently within the input
     * @return {Array<String>} - A list of string values represent the tags selected
     */
    function GetValidValues() {
        const validValues = [];

        const allTags = mainContainer.getElementsByClassName("ebsta-tags-input-tag-value");

        for (let i = 0; i < allTags.length; i++) {
            const tagValue = allTags[i].getAttribute("tag-value");
            const isError = allTags[i].classList.contains("error");
            if (isError === false && tagValue && validValues.indexOf(tagValue) === -1) {
                validValues.push(tagValue);
            }
        }

        return validValues;
    }

    return {
        AddTag: AddTag,
        GetElement : function() {
            return mainContainer;
        },
        GetValidValues: GetValidValues
    };
}