﻿/**
 * Module containining all Complex Add creation functions as well as values selected
 * @returns {} 
 */
function ComplexAdd() {
    var lComplexAddValues = null; // Values selected within the complex add window
    var lComplexAddButton = null; // Complex Add button within the compose window

    //We need to create this straight away so the EbstaComposeButtons closure can detect this and associate the click event to it
    var lSendEmailButton = CreateElement("button", "ebsta-button-dark sfBtSendEmail", "Send Email", null);

    function AddComplexAdd(obj) {
        var divBox = document.createElement('div');

        //Add listener to register clicks outside of the search box to make it disappear
        //AddListenerForComplexAddBox(divBox);

        divBox.id = "ebstaBox";
        divBox.setAttribute("class", "sfBox front sfComplexAddBox");

        if (document.getElementsByClassName("G-atb D E").length != null) {
            if (document.getElementsByClassName("G-atb D E").length > 0) {
                document.getElementsByClassName("G-atb D E")[0].style.zIndex = 0;
            }
        }

        //create the title element for complex add popup.
        var titleElement = CreateElement("span", "sftitle-text", "Add Email to Salesforce");
        //create the 'customize fields' link in the complex add header.
        var customiseFieldSpan = CreateElement("a", "sfLink ebCustomizeFields", "Customize Fields", "EbstaAnchorComplexAddCustomizeFields",
            { href: 'https://console.ebsta.com/LoginSalesforce.aspx?k=' + webUserId + '&j=crmintegration&objectLinker=ComplexAdd' });

        //create a container for all the header items in complex add.
        var headerContainer = document.createElement("div");
        headerContainer.className = 'ebsta-header-container';

        //append the title/link to the header container.
        headerContainer.appendChild(titleElement);
        headerContainer.appendChild(customiseFieldSpan);

        //append the header container to the main modal.
        divBox.appendChild(headerContainer);

        var helpIconComplexAdd = EbstaModalTools.CreateHelpIcon("ebsta-help-icon-right",
            "https://ebstalimited.zendesk.com/hc/en-us/articles/115000096093-Add-Email-to-Salesforce",
            "Visit Our Knowledge Base");

        helpIconComplexAdd.id = "EbstaAnchorComplexAddHelp﻿";

        headerContainer.appendChild(helpIconComplexAdd);

        headerContainer.appendChild(customiseFieldSpan);

        //append the header container to the main modal.
        divBox.appendChild(headerContainer);

        divBox.appendChild(CreateElement("span", "sftitle-close"));

        //Subject Details Row
        var divrow2 = CreateElement("div", "sfSubjectRow");
        var formGroupDiv = CreateElement("div", "ebsta-form-group");
        formGroupDiv.appendChild(CreateElement("label", "sfSubject ebsta-label", "Subject"));

        var subjectTextBox = CreateElement("input", "ebsta-form-control sfTextSubject", null, "txtSubject", {
            type: "textbox", value: obj.Title != null ? obj.Title : ""
        });

        formGroupDiv.appendChild(subjectTextBox);
        divrow2.appendChild(formGroupDiv);
        divBox.appendChild(divrow2);

        //Hide Subject if its for campaigns for the time being
        if (obj.CampaignManager || obj.HasCampaignBeenSelected) {
            HideElement(divrow2);
        }

        var divrow3 = CreateElement("div", "row");
        divrow3.appendChild(CreateElement("div", "sfNameHeader", "Name"));
        divrow3.appendChild(CreateElement("div", "sfRelatedToHeader", "Related To"));
        divBox.appendChild(divrow3);

        GetComplexAddSettings(obj, divBox);

        var messageId = getMessageId($("*[current-emailadd*='true'"));

        var complexAddContainer = CreateElement("div", "sfComplexAddContainer");
        complexAddContainer.setAttribute('gmailmessageid', messageId);

        //We only need this value when in Gmail of which we should have injected the GmailTools module
        if (typeof GmailTools === "object"
            && typeof GmailTools.MessageView === "object"
            && typeof GmailTools.MessageView.GetNewFormatMessageId === 'function') {
            complexAddContainer.setAttribute('newformatgmailmessageid', GmailTools.MessageView.GetNewFormatMessageId($("*[current-emailadd*='true'")[0]));
        }

        var divrow3 = CreateDivWithId("", "", "sfComplexAddRowsContainer");
        complexAddContainer.appendChild(divrow3);
        divBox.appendChild(complexAddContainer);

        //Create Div for complete box
        var divCompleteRow = CreateElement("div", "sfRowComplete");
        divCompleteRow.setAttribute("id", "sfRowComplete");

        //Save/Add on Send button - depending on if this is a save on send or not
        var saveButton = CreateBt("Save", "ebsta-button-dark sfBtSave", "EbstaBtComplexAddSave");
        divCompleteRow.appendChild(saveButton);
        //If there are values already pre-selected render a remove button to clear state
        if (obj.ComplexAddClosure != null) {
            if (!obj.ComplexAddClosure.IsComplexAddValuesEmpty()) {
                divCompleteRow.appendChild(obj.ComplexAddClosure.CreateRemoveButton(obj.FunctionToUpdateSendButtons));
            }

            if (!obj.CampaignManager) {
                saveButton.innerText = "Add on Send";

                //Append the "Send Email" button when not in campaign manager
                var sendEmailButton = obj.ComplexAddClosure.GetSendEmailButton();
                if (sendEmailButton != null) {
                    sendEmailButton.ComplexAddObject = obj;
                    divCompleteRow.appendChild(sendEmailButton);
                }
            }
        }

        divBox.appendChild(divCompleteRow);

        (document.body || document.documentElement).appendChild(divBox);

        //If this complex add on an email
        if (obj.SaveOnSend == 0 && obj.Id != "") {
            CreateContactSuggestions(obj);
            CreateNewComplexAddRow(obj.Id, obj.ObjectName, obj.SFType, obj.EmailText, obj.WhatId, obj.WhatName, obj.WhatRecordType);
            CreateNewComplexAddRow();
        } else {
            // If this is not a save on send action then check if the user has already selected values for this 
            // compose window to see if we need to pre-render the objects within the Complex Add screen
            RenderSelectedObjectsForComplexAdd(obj, subjectTextBox);
        }

        $('#btCancel').click(function () {
            $('#ebstaBox').remove();
            $('.sfEBComplexAddSearchObjectChoiceDiv').hide();
        });

        $('.sftitle-close').click(function () {
            ////LogThis("Fired title");
            $(".sfComplexAddFired").removeClass("sfComplexAddFired");
            $(".sfQuickAddFired").removeClass("sfQuickAddFired");

            $('#ebstaBox').remove();
            $('.sfEBComplexAddSearchObjectChoiceDiv').hide();
        });


        if (obj.SaveOnSend + "" == "1") {
            $('#btSave').text("Send + Add to Salesforce");
        }

        //Complex add save/send email buttons
        $('#btSave,.sfBtSave').click(function () {

            if (VerifyCustomFieldInputs($(this)[0])) {
                SaveComplexAddNew(obj);
                $('#ebstaBox').remove();
            }

            $('.sfEBComplexAddSearchObjectChoiceDiv').hide();

        });
    }

    //Diables fields when a lead is selected
    function CheckForLeads() {

        //Skip this if either of the objects are not available
        if (!whoWhatEnabledObject.whatEnabled || !whoWhatEnabledObject.whoEnabled) {
            return false;
        }

        $('#ebstaBox, #emailTemplateBox').find('.sfAddComplexRow, .sfEmailTemplateSearch').each(function () {
            var foundElement = false;

            var isThisForTemplates = false;

            if ($(this).hasClass('sfEmailTemplateSearch')) {
                isThisForTemplates = true;
            }

            $(this).find('.sfSuggestLead').each(function () {
                //if this is a lead not in a results window
                if ($(this).attr('recordtype') != null && $(this).parent()[0].className != "resultsWindow") {
                    //find the corresponding row and disable the what box
                    $(this).closest('.sfAddComplexRow, .sfEmailTemplateSearch').each(function () {
                        //Resest attributes
                        if (!isThisForTemplates) {
                            $(this).attr("sfaddcomplexrowwhatids", "");
                        }

                        $(this).find('.sfWhatSearch').find('.CABoxStyle').each(function () {
                            $(this).remove();
                        });

                        $(this).find('.sfSearchWhatTextBox').each(function () {
                            $(this).attr('disabled', 'disabled');
                            $(this).val("Cannot add objects to Leads");
                            $(this).closest('.sfWhatSearch').addClass("sfGreyOutBox");
                            $(this).addClass("sfGreyOutBox");
                            $(this).css("display", "inline-block");
                        });

                        $(this).find('.sfWhatSearchBtn, .ebsta-glyphicon').each(function () {
                            $(this).attr('disabled', 'disabled');
                            $(this)[0].style.display = "none";
                        });

                        $(this).find('.sfGetSuggestRow').each(function () {
                            $(this)[0].style.display = "none";
                        });

                        $(this).find('.sfRowWhatSuggest').each(function () {
                            $(this)[0].style.display = "none";
                        });

                    });
                }

                foundElement = true;
            });


            if (!foundElement) {
                //Extra Styling for Templates when leads aren't detected
                if (isThisForTemplates) {

                    var searchQuery = "";

                    $(this).find('.sfSearchWhatTextBox').each(function () {
                        //If the div has the 'Cannot add object to lead' text then remove
                        if ($(this).val().indexOf("Cannot add objects to Leads") != -1) {
                            $(this).val("");
                        } else {
                            searchQuery = $(this).val();
                        }
                        $(this).removeClass("sfGreyOutBox");
                        $(this).prop("disabled", false);
                    });

                    $(this).find('.sfWhatSearch').each(function () {
                        $(this).removeClass("sfGreyOutBox");

                        //If there are no objects selected display search/cross icons back
                        if (!$(this).find('.CABoxStyle').length) {
                            $(this).find('.sfWhatSearchBtn, .ebsta-glyphicon-option-after').each(function () {
                                $(this).prop("disabled", false);
                                $(this)[0].style.display = "inline-block";
                            });
                        }
                    });

                    if (searchQuery.length == 0) {
                        $(this).find('.crossSearchIcon').each(function () {
                            $(this).removeClass("crossSearchIcon").addClass('magnifySearchIcon');
                        });
                    }
                }
                else {
                    $(this).find('.sfSearchWhatTextBox').each(function () {
                        $(this).prop("disabled", false);
                        //$(this).val("");
                    });

                    $(this).find('.sfWhatSearch').each(function () {
                        $(this).removeClass("sfGreyOutBox");

                        //If there are no objects selected display search/cross icons back
                        if (!$(this).find('.CABoxStyle').length) {
                            $(this).find('.sfWhatSearchBtn').each(function () {
                                $(this).prop("disabled", false);
                                $(this)[0].style.display = "block";
                            });
                        }
                    });
                }

            }
        });
    }

    return {
        AddComplexAdd: AddComplexAdd,
        CheckForLeads: CheckForLeads,
        /**
         * Creates a remove button for the complex add screen
         * @returns {} 
         */
        CreateRemoveButton: function (functionToUpdateSendButtons) {
            var removeButton = CreateElement("button", "ebsta-button-dark ebsta-ca-remove", "Remove");

            var thisClosureInstance = this;
            removeButton.addEventListener("click", function () {
                thisClosureInstance.RemoveButtonClickEvent();
                if (functionToUpdateSendButtons) {
                    functionToUpdateSendButtons();
                }
            });

            return removeButton;
        },
        /**
         * Click Event for the Remove button within the complex add screen
         * @returns {} 
         */
        RemoveButtonClickEvent: function () {
            RemoveElement(document.getElementById("ebstaBox"));
            this.ResetComplexAddValues();
            this.UpdateComplexAddButton(false);
        },
        /**
         * Returns the Send Email button element
         * @returns {} 
         */
        GetSendEmailButton: function () {
            return lSendEmailButton;
        },
        SetComplexAddButton: function (button) {
            lComplexAddButton = button;
        },
        /**
         * Creates a new complex add screen
         * @param {} complexAddButton 
         * @param {} lComplexAddClosure 
         * @param {} hasCampaignBeenSelected 
         * @param {} functionToUpdateSendButtons - This is a function that is called when either "Add on Send" or "Remove" buttons are clicked
         * @returns {} 
         */
        SetupAndCreateComplexAdd: function (complexAddButton, lComplexAddClosure, hasCampaignBeenSelected, functionToUpdateSendButtons) {
            this.CloseComplexAddScreen();
            //Set button
            lComplexAddButton = complexAddButton;

            $('#emailTemplateBox').remove();
            createCampaignMgrObject().RemoveCampaignManagerModal();

            //Set all complex add buttons to current = falsle
            $("[current-complexadd*='true'").attr("current-complexadd", "false");

            $("[current-emailAdd*='true'").attr("current-emailAdd", "false");
            //Set the clicked complex add button as the current complex add button
            $(complexAddButton).attr("current-complexadd", "true");

            $(complexAddButton).attr("current-emailAdd", "true");

            var email = $(complexAddButton).attr("sfEmailAddress");
            var type = $(complexAddButton).attr("sfType");
            var sfname = $(complexAddButton).attr("sfname");

            var d = new Date();
            //21 July 2014 18:03
            var lasttime = "" + d.getDate() + " " + FormatMonth(d.getMonth()) + " " + d.getFullYear() + " " + FormatNumber(d.getHours()) + ":" + FormatNumber(d.getMinutes());
            var title = "";
            var message = "";
            var originalSubject = "", composeId = null;

            var composeWindowContainer = $(complexAddButton).closest('.I5');

            // Message Rules.
            $(complexAddButton).closest('.I5').find('*[aria-label="Message Body"]').each(function () {
                message = StripHtml($(this).html());
            });

            $(composeWindowContainer).find('*[name="composeid"]').each(function () {
                composeId = $(this).val();
            });

            //Is this new gmail UI
            $(composeWindowContainer).find('*[name="draft"]').each(function () {
                var draftId = $(this).val();
                if (draftId.indexOf("#msg-a:") > -1) {
                    composeId = draftId;
                }
            });

            if (message.length == 0) {
                $(complexAddButton).closest('.aVN').find('*[name="body"]').each(function () {
                    message = StripHtml($(this).val());
                });
            }

            // Email.
            var emailAddressesSentTo = new Array();
            $(complexAddButton).closest('.I5').find("[email]").each(function () {
                if (!$(this).parent().hasClass('oL aDm az9')) {
                    email += $(this).attr("email") + " OR ";
                    emailAddressesSentTo.push($(this).attr("email"));
                }
            });

            if (email != null) {
                if (email.length == 0) {
                    $(complexAddButton).closest('.aVN').find("[email]").each(function () {
                        email = $(this).attr("email");

                    });
                }
            }

            if (title.length == 0) {
                title = "Email Sent: " + $(complexAddButton).closest('.I5').find('.aoT').val();
                originalSubject = $(complexAddButton).closest('.I5').find('.aoT').val();
            }

            // Title
            if (title.length == 0) {
                $(complexAddButton).closest('*[class="nH Hd"]').find('.aYF').each(function () {
                    title = "Email Sent: ";
                    var subjectDiv = $(this).closest('*[class="nH Hd"]').find('input[name="subject"]');
                    if (subjectDiv) {
                        if (subjectDiv.attr('value')) {
                            if (subjectDiv.attr('value') != "") {
                                title += $(this).text();
                                originalSubject = $(this).text();
                            }
                        }
                    }
                });
            }

            if (title.length == 0) {
                $(complexAddButton).closest('.aVN').find('*[name="subject"]').each(function () {
                    //title = "Email: " + $(this).text();
                    title = "Email Sent:" + $(this).val();
                    originalSubject = $(this).val();
                });
            }

            var secondaryEmailSearch = new Array();

            for (var i = 0; i < emailAddressesSentTo.length; i++) {
                var copy = secondaryEmailSearch.indexOf(emailAddressesSentTo[i]);

                if (copy == -1) {
                    secondaryEmailSearch.push(emailAddressesSentTo[i]);
                }
            }

            email = secondaryEmailSearch[0];
            secondaryEmailSearch.splice(0, 1);

            var saveOnSendObj = "";

            if (complexAddButton) {
                saveOnSendObj = complexAddButton;
            }

            //We'll first go and check Local Storage for any 'Related To' objects previous chosen for the chosen WhoId
            chrome.extension.sendMessage({
                MessageType: "GetLocalStorageRelatedToSelections"
            }, function (response) {

                var whatId = null, whatName = null, whatRecordType = null;

                //If we've got some history of a 'Related To' object for this record then lets pre-populate the Related To box instantly
                if (response.relatedToHistory) {
                    listOfRelatedToHistory = response.relatedToHistory;
                    var result = LookForRelatedToObject(email);

                    if (result) {
                        if (result.WhatRecordId) {
                            whatId = result.WhatRecordId;
                        }

                        if (result.WhatRecordName) {
                            whatName = result.WhatRecordName;
                        }

                        if (result.WhatRecordType) {
                            whatRecordType = result.WhatRecordType;
                        }
                    }
                }

                AddComplexAdd({
                    MessageType: "AddTaskEmail",
                    Id: email,
                    EmailText: email,
                    SecondaryEmailSearch: secondaryEmailSearch,
                    Time: lasttime,
                    Title: title,
                    Msg: message,
                    SFType: type,
                    ObjectName: sfname,
                    SaveOnSend: 1,
                    SaveOnSendObj: saveOnSendObj,
                    FoundDate: false,
                    OriginalSubject: originalSubject,
                    WhatId: whatId,
                    WhatName: whatName,
                    WhatRecordType: whatRecordType,
                    ComplexAddClosure: lComplexAddClosure,
                    ComposeId: composeId,
                    HasCampaignBeenSelected: hasCampaignBeenSelected,
                    FunctionToUpdateSendButtons: functionToUpdateSendButtons
                });
            });
        },
        SetComplexAddValues: function (value) {
            lComplexAddValues = value;
        },
        GetComplexAddValues: function () {
            return lComplexAddValues;
        },
        IsComplexAddValuesEmpty: function () {
            if (lComplexAddValues != null) {
                if (lComplexAddValues.ComplexAddWhoAndWhatIds != null && lComplexAddValues.ComplexAddWhoAndWhatIds.length > 0)
                    return false;
                if (lComplexAddValues.ComplexAddAdditionalFields != null && lComplexAddValues.ComplexAddAdditionalFields.length > 0)
                    return false;
            }
            return true;
        },
        HasComplexAddBeenSet: function () {
            return lComplexAddValues != null;
        },
        UpdateComplexAddButton: function (selected) {
            if (lComplexAddButton != null) {
                if (selected) {
                    AddClass(lComplexAddButton, "ebsta-btn-success");
                } else {
                    RemoveClass(lComplexAddButton, "ebsta-btn-success");
                }
            }
        },
        /**
         * Adds a new set paid for Who/Whatids for tasks to be created and associated with in SF
         * @param {} whoId 
         * @param {} whoName 
         * @param {} whoRecordType 
         * @param {} whoEmail 
         * @param {} whatId 
         * @param {} whatRecordName 
         * @param {} whatRecordType 
         * @returns {} 
         */
        CreateComplexWhoAndWhatId: function (whoId, whoName, whoRecordType, whoEmail, whatId, whatRecordName, whatRecordType, gmailAttachments) {
            return {
                WhoId: whoId != null ? whoId : null,
                WhoName: whoName != null ? whoName : null,
                WhoRecordType: whoRecordType,
                WhoEmail: whoEmail != null ? whoEmail : null,
                WhatId: whatId != null ? whatId : null,
                WhatRecordName: whatRecordName != null ? whatRecordName : null,
                WhatRecordType: whatRecordType,
                GmailAttachmentIds: gmailAttachments
            };
        },
        /**
         * Sets the string we'll used to log for the
         * @param {} logEventString 
         * @returns {} 
         */
        SetLogEventString: function (logEventString) {
            lComplexAddValues.LogEvent = logEventString;
        },
        /**
         * Return the log event string
         * @param {} complexAddRow 
         * @returns {} 
         */
        GetLogEventString: function () {
            return lComplexAddValues.LogEvent;
        },
        /**
         * Adds a new row into the lComplexAddValues array which will be used to add objects to Salesforce
         * @param {} complexAddRowObject 
         * @param {} gmailAttachmentIds 
         * @returns {} 
         */
        AddComplexAddRow: function (complexAddRowObject) {
            // If this a blank row but is the first then continue to add this as a user may want to add a Task with No who or WhatId's, 
            // if it's not the first row then we'll presume they don't want to add a blank task
            if (lComplexAddValues.ComplexAddWhoAndWhatIds.length > 0 && (complexAddRowObject == null || (!complexAddRowObject.WhoId && !complexAddRowObject.WhatId)))
                return true;

            lComplexAddValues.ComplexAddWhoAndWhatIds.push(complexAddRowObject);
        },
        /**
         * Sets the additional fields that are contained in the top part of the Complex Add screen
         * @param {} listOfFields 
         * @returns {} 
         */
        SetComplexAddAdditionalFields: function (listOfFields) {
            for (var i = 0; i < listOfFields.length; i++) {
                lComplexAddValues.ComplexAddAdditionalFields.push({
                    FieldName: listOfFields[i].FieldName,
                    FieldType: listOfFields[i].FieldType,
                    FieldValue: listOfFields[i].FieldValue,
                    ReferenceRecordType: listOfFields[i].ReferenceRecordType,
                    ReferenceRecordName: listOfFields[i].ReferenceRecordName
                });
            }
        },
        InitialiseComplexAddValues: function () {
            lComplexAddValues = {
                ComplexAddAdditionalFields: new Array(),
                ComplexAddWhoAndWhatIds: new Array()
            }
        },
        ResetComplexAddValues: function () {
            lComplexAddValues = null;
        },
        SetComposeId: function (composeId) {
            lComplexAddValues.ComposeId = composeId;
        },
        SetOriginalSubject: function (originalSubject) {
            lComplexAddValues.OriginalSubject = originalSubject;
        },
        SetSubject: function (subject) {
            lComplexAddValues.Subject = subject;
        },
        GetSubject: function () {
            return lComplexAddValues.Subject;
        },
        CloseComplexAddScreen: function () {
            var complexAdd = document.getElementById("ebstaBox");
            if (complexAdd != null) {
                complexAdd.remove();
            }
        }
    }
}