﻿/// <reference path="bar.js" />


(function () {

    function ParseUrlIntoParams(url) {
        var regex = /[?&]([^=#]+)=([^&#]*)/g;
        var params = {};
        var match;

        while (match = regex.exec(url)) {
            params[match[1]] = match[2];
        }

        return params;
    }
    /**
     * Checks by the URL and Body of a gmail request if this is a new email that has been loaded
     * A request for a new email will contain a single object that has two properties named 1 and 2:
     * 1 is an object that contains two propertes 1 and 3, where 1 contains the message and thread idn and 3 contains the message id.
     * 2 is a property that contains a single digit value.
     * Regex here matches very narrowly on the body of the request being this type of object.
     * @param {any} urlParamsObject - the paramaters in the url
     * @param {any} reqBody - the request body
     */
    function IsGmailNewMessageLoaded(urlParamsObject, reqBody) {
        if (urlParamsObject && typeof urlParamsObject.hl === 'string' && typeof reqBody === 'string') {
            var regNewMsg = /{"1":"thread-f:\d{19}[|]msg-f:\d{19}"."3":\["msg-f:\d{19}"]}],"2":\d}$/;
            return regNewMsg.test(reqBody);
        }
    }

    ///Parses params in, this allows us to look for specific events. 
    var lastDt = new Date();

    function parseRequest(params) {
        try {
       
            params.rawUrl = params.url;

            params.url = ParseUrlIntoParams(params.url);

            //if we have loaded a new email - we need to go and run a new inbox search to re load icons
            //these typically come in multiple requests for multiple emails at a time, so we have a timeout to prevent us from running
            //NEWEMAIL message to often
            if (typeof params.url.ik == 'string' && typeof params.url.search == 'string' && params.body.length == 0 && typeof params.url._reqid == 'string'
                || IsGmailNewMessageLoaded(params.url, params.body)) {

                var processor = setTimeout(function() {
                    lastDt = new Date();
                    var last = lastDt;

                    setTimeout(function() {
                        //console.log("Setting : " + last.getTime() + " - - " + lastDt.getTime());

                        if (last.getTime() === lastDt.getTime()) {
                           // console.log('Did match should be last.');
                            window.postMessage({ type: "NEWEMAIL", text: "" }, "*");
                        }
                    }, 700);

                }, 100);
            }

            //if (typeof params.url._reqid == 'string' && params.url.view === 'tl' && params.url.auto != undefined) {
            //    console.log("NEw Emails...");
            //    window.postMessage({ type: "NEWEMAIL", text: "" }, "*");

            //}
            return params;
        } catch (error) {
            //console.log(" Failure " + error);

        }

    }

    /**
     * Looks for a saved draft event within the request picked up by the proxy, if so sends a 
     * message into bar.js to determine what we need to do with it
     * @param {} parsedParams 
     * @param {} responseText 
     * @param {} body 
     * @returns {} 
     */
    function DetectSaveDraftChanges(parsedParams, responseText, body) {
        if (parsedParams != null && responseText != null) {
            if (parsedParams.url != null) {
                if (parsedParams.url.act === "sd") {
                    window.postMessage({ type: "DRAFT_RESPONSE", ResponseText: responseText, ParsedParams: parsedParams, BodyContent: body }, "*");
                }
            }
        }
    }

    /**
     * Finds the elements where the HTTP request are fired off
     * @returns {} 
     */
    function GetWindowElementForRequests() {
        // In Old Gmail the javascript for firing request seems to be hidden behind a iframe so we need to find 
        // this frame in order to intecept the requests
        var frameWithXhrRequestJavascript = top.document.getElementById("js_frame");

        //If we still don't have the frame we're looking for by now, intercept the whole window's XHR requests
        if (!frameWithXhrRequestJavascript) {
            frameWithXhrRequestJavascript = top;
        }

        if (!frameWithXhrRequestJavascript) throw new Error("Unable to find XHR Frame");

        if (frameWithXhrRequestJavascript.contentDocument && frameWithXhrRequestJavascript.contentDocument.defaultView) {
            return frameWithXhrRequestJavascript.contentDocument.defaultView;
        }

        return frameWithXhrRequestJavascript;
    }

    var win = GetWindowElementForRequests();
    var proxied = win.XMLHttpRequest.prototype.open;
    var proxSend = win.XMLHttpRequest.prototype.send;

    win.XMLHttpRequest.prototype.open = function (method, url, async, user, password) {
        this.xhrParams = {
            method: method.toString(),
            url: url.toString()
        };

        return proxied.apply(this, [].slice.call(arguments));
    };

    win.XMLHttpRequest.prototype.send = function(body) {

        var curr_onreadystatechange = this.onreadystatechange;
        var xhr = this;

        var parsedParams = null;

        if (this.xhrParams) {
            this.xhrParams.body = body;
            parsedParams = parseRequest(this.xhrParams);
        } else {
            console.log("No XhrParams");
        }

        this.onreadystatechange = function(progress) {
            if (this.readyState === this.DONE) {

                if (progress.target.responseType != null) {
                    if (progress.target.responseType != 'blob') {
                        xhr.xhrResponse = progress.target.responseText;

                        if (progress.target.responseURL != null) {
                            
                            if (progress.target.responseURL.indexOf('act=sm') != -1) {
                                var resp = progress.target.responseText + " " + body;
                                window.postMessage({ type: "FROM_PAGE", text: resp }, "*");
                            }

                            if (progress.target.responseText) {
                                window.postMessage({
                                    type: "GMAILHTTPREQUEST",
                                    text: progress.target.responseText,
                                    url: progress.target.responseURL
                                }, "*");
                            }
                        }

                        if (parsedParams != null) {
                            DetectSaveDraftChanges(parsedParams, progress.target.responseText, body);
                        }
                    }
                }
            }

            if (curr_onreadystatechange) {
                curr_onreadystatechange.apply(this, arguments);
            }
        };


        return proxSend.apply(this, arguments);
    };
})();

 //console.log("Proxy setup");
