﻿/** Hides the Campaign Manager Iframe*/
function HideCampaignIFrame() {
    global_SalesforceHelper.HideIframe();
}

/** Module for all things Salesforce */
var global_SalesforceHelper = (function () {
    var gCampaignIframe = null, // iframe where campaigns will live
        gMutationObserverLightning = null, //Mutation observer for salesforve lightning
        gSalesforcePageType = 0; //Keep track of what type of page we're currently on

    /** ENUM type object representing what type of page we're currently on*/
    var salesforcePageTypeEnum = {
        ClassicRecordView: 1,
        LightningRecordView: 2,
        ClassicListView: 3,
        LightningListView: 4,
        ClassicCampaignMemberView: 5,
        LightningCampaignMemberView: 6,
        ClassicCampaignMemberViewListView: 7,
        LightningCampaignMemberViewListView: 8,
        ClassicJobOrdersApplicationsListView: 9,
        LightningJobOrdersApplicationsListView: 10,
        LightningApplicationsRelatedListView: 11
    }

    /** Checks if a Salesforce ID is a valid Contact or Lead Id */
    function isIdAContactOrLead(recordId) {
        return /^003(.{12}|.{15})|00Q(.{12}|.{15})/.test(recordId) === true;
    }

    /** 
     *  Tries to find records selected in Lists in either Salesforce Classic
     *  @returns {array} - Array of Salesforce ids
     */
    function GetSelectedRowsFromListViewClassic() {
        var arrayOfRecords = [];
        var getListContainer = document.getElementsByClassName("listBody")[0];
        if (getListContainer) {
            var getAllCheckBoxes = getListContainer.getElementsByTagName("input");
            for (var i = 0; i < getAllCheckBoxes.length; i++) {
                var checkBox = getAllCheckBoxes[i];
                if (checkBox.checked) {
                    var checkBoxId = checkBox.id;
                    if (checkBoxId) {
                        //Id's should be 15 or 18 iin length, anythine else and this value won't be correct
                        var checkBoxLength = checkBoxId.length;
                        if (checkBoxLength === 15 || checkBoxLength === 18) {
                            arrayOfRecords.push(checkBoxId);
                        }
                    }
                }
            }
        }
        return arrayOfRecords;
    }

    /** 
     *  Tries to find records selected in Lists in either Salesforce Lightning
     *  @returns {array} - Array of Salesforce ids
     */
    function GetSelectedRowsFromListViewInLightning() {
        var arrayOfRecords = [];
        var getAllTableRowsSelected = document.querySelectorAll("table tr.selected");
        if (getAllTableRowsSelected.length > 0) {
            for (var i = 0; i < getAllTableRowsSelected.length; i++) {
                const recordElements = getAllTableRowsSelected[i].querySelectorAll("a[data-recordid]");
                recordElements.forEach(recordElement => {                
                    if (recordElement) {
                        var recordId = recordElement.getAttribute("data-recordid");
                        if (recordId) {
                            if (isIdAContactOrLead(recordId)) {
                                arrayOfRecords.push(recordId);
                                return;
                            }
                        }
                    }
                });
            }
        }
        return arrayOfRecords;
    }

    /**
     * Extracts the salesforce id from the URL for Classic
     * @returns {string} - The id or null
     */
    function GetSalesforceRecordIdFromClassicUrl() {
        var regexRule = new RegExp("force\.com\/([0-9a-zA-Z]{15})");
        var regexResult = regexRule.exec(document.URL);
        if (regexResult !== null && regexResult.length > 1) {
            return regexResult[1];
        }
        return null;
    }

    /**
     * Extracts the salesforce id from the URL for Lightning
     * @returns {string} - The id or null
     */
    function GetSalesforceRecordIdFromLightningUrl() {
        var regexRule = new RegExp("\.force\.com\/.*([0-9a-zA-Z]{18})\/view");
        var regexResult = regexRule.exec(document.URL);
        if (regexResult !== null && regexResult.length > 1) {
            return regexResult[1];
        }
        return null;
    }

    /** 
    *  Gets all id's from a CampaignMember list view (Clasic)
    *  @returns {array} - Array of Salesforce ids
    */
    function GetSelectedRowsFromCampaignMemberListViewInClassic() {
        const arrayOfRecords = [];
        const rowCheckboxes = document.querySelectorAll("#null_grid input");
        if (rowCheckboxes.length > 0) {
            for (var i = 0; i < rowCheckboxes.length; i++) {
                const checkbox = rowCheckboxes[i];
                if (checkbox.checked === true) {
                    const allATagsInRow = checkbox.closest("tr").querySelectorAll("a")
                    for (let j = 0; j < allATagsInRow.length; j++) {
                        const aTag = allATagsInRow[j];
                        if (aTag) {
                            const hrefAttr = aTag.getAttribute("href");
                            if (hrefAttr) {
                                const recordId = hrefAttr.replace("/", "");
                                if (recordId) {
                                    //Id's should be 15 or 18 iin length, anythine else and this value won't be correct
                                    if (/^003(.{12}|.{15})|00Q(.{12}|.{15})/.test(recordId) === true) {
                                        arrayOfRecords.push(recordId);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return arrayOfRecords;
    }

    /** 
    *  Gets all id's from a CampaignMember list view
    *  @returns {array} - Array of Salesforce ids
    */
    function GetSelectedRowsFromCampaignMemberListViewInLightning() {
        var arrayOfRecords = [];
        var getAllTableRowsSelected = document.querySelectorAll("table tr.selected");
        if (getAllTableRowsSelected.length > 0) {
            for (var i = 0; i < getAllTableRowsSelected.length; i++) {
                var recordElement = getAllTableRowsSelected[i].querySelector("a[data-recordid]");
                if (recordElement) {
                    var recordId = recordElement.getAttribute("data-recordid");
                    if (recordId) {
                        //Id's should be 15 or 18 iin length, anythine else and this value won't be correct
                        if (/^003(.{12}|.{15})|00Q(.{12}|.{15})/.test(recordId) === true) {
                            arrayOfRecords.push(recordId);
                        }
                    }
                }
            }
        }
        return arrayOfRecords;
    }

    function GetSelectedRowsFromJobOrderApplicationsRelatedListView() {
        var arrayOfRecords = [];

        const applicationsTable = EbstaXpath.GetFirstNode('//h3[text()="Applications"]/ancestor::form[1]//div[@class="pbBody"]/table');

        if (!applicationsTable) return arrayOfRecords;

        const rowCheckboxes = applicationsTable.querySelectorAll("input[type=checkbox]");
        if (rowCheckboxes.length > 0) {
            for (var i = 0; i < rowCheckboxes.length; i++) {
                const checkbox = rowCheckboxes[i];
                if (checkbox.checked === true) {
                    const allATagsInRow = checkbox.closest("tr").querySelectorAll("a")
                    for (let j = 0; j < allATagsInRow.length; j++) {
                        const aTag = allATagsInRow[j];
                        if (aTag) {
                            const hrefAttr = aTag.getAttribute("href");
                            if (hrefAttr) {
                                const recordId = hrefAttr.replace("/", "");
                                if (recordId) {
                                    //Id's should be 15 or 18 iin length, anythine else and this value won't be correct
                                    if (/^003(.{12}|.{15})|00Q(.{12}|.{15})/.test(recordId) === true) {
                                        arrayOfRecords.push(recordId);
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return arrayOfRecords;
    }

    /**
     * Gets the selected rows for a JobOrder applications list. It will stem from the button clicked as the same list can appear on 
     * a page twice due to the hover over DOM that appears for Applications when on a Job Order page
     * @param {any} campaignButtonClicked
     */
    function GetSelectedRowsFromJobOrderApplicationsSectionsInLightning(campaignButtonClicked) {
        const arrayOfRecords = [];

        const listViewContainer = campaignButtonClicked.closest('.forceListViewManager');

        if (!listViewContainer) {
            // Check for other containers as there maybe be different DOM for different list views
            return GetSelectedRowsFromJobOrderApplicationsTabInLightning(campaignButtonClicked);
        }

        const getAllTableRowsSelected = listViewContainer.querySelectorAll("table tr.selected");
        if (getAllTableRowsSelected.length > 0) {
            for (var i = 0; i < getAllTableRowsSelected.length; i++) {
                const recordElement = getAllTableRowsSelected[i].querySelector("a[data-recordid]");
                if (recordElement) {
                    const recordId = recordElement.getAttribute("data-recordid");
                    if (recordId) {
                        //Id's should be 15 or 18 iin length, anythine else and this value won't be correct
                        if (/^003(.{12}|.{15})|00Q(.{12}|.{15})/.test(recordId) === true) {
                            arrayOfRecords.push(recordId);
                        }
                    }
                }
            }
        }
        return arrayOfRecords;
    }

    /**
     * Gets the ids of the selected record in the applications tab on a Job Order page
     * @param {HTMLElement} campaignButtonClicked - The element representing the ebsta campaign button that was clicked
     * @returns {Array} - A list of contact or lead Salesforce ID's
     */
    function GetSelectedRowsFromJobOrderApplicationsTabInLightning(campaignButtonClicked) {
        const arrayOfRecords = [];

        const listViewContainer = campaignButtonClicked.closest('lst-common-list');

        if (!listViewContainer) {
            return arrayOfRecords;
        }
        
        const contactOrLeadRegex = /003(.{12}|.{15})|00Q(.{12}|.{15})/g;
        
        const getAllTableRowsSelected = listViewContainer.querySelectorAll("table tr.slds-is-selected");
        if (getAllTableRowsSelected.length > 0) {
            for (var i = 0; i < getAllTableRowsSelected.length; i++) {
                const recordElement = getAllTableRowsSelected[i].querySelector("a");
                if (recordElement) {
                    const href = recordElement.getAttribute("href");
                    if (href) {
                        const regexMatch = href.match(contactOrLeadRegex);
                        if (regexMatch) {
                            arrayOfRecords.push(regexMatch[0]);
                        }
                    }
                }
            }
        }

        return arrayOfRecords;
    }

    /** 
     *  Tries to find records selected in Lists in either Salesforce Classic or Lightning
     *  @returns {array} - Array of Salesforce ids
     */
    function GetSelectedRowsFromListView(campaignButtonClicked) {
        switch (gSalesforcePageType) {
            case salesforcePageTypeEnum.ClassicListView:
                return GetSelectedRowsFromListViewClassic();
            case salesforcePageTypeEnum.LightningListView:
            case salesforcePageTypeEnum.LightningApplicationsRelatedListView:
                return GetSelectedRowsFromListViewInLightning();
            case salesforcePageTypeEnum.ClassicRecordView:
                return new Array(GetSalesforceRecordIdFromClassicUrl());
            case salesforcePageTypeEnum.LightningRecordView:
                return new Array(GetSalesforceRecordIdFromLightningUrl());
            case salesforcePageTypeEnum.ClassicCampaignMemberView:
                return new Array(GetRecordIdFromCampaignMemberViewClassic());
            case salesforcePageTypeEnum.LightningCampaignMemberView:
                return new Array(GetRecordIdFromCampaignMemberViewLightning());
            case salesforcePageTypeEnum.ClassicCampaignMemberViewListView:
                return GetSelectedRowsFromCampaignMemberListViewInClassic();
            case salesforcePageTypeEnum.LightningCampaignMemberViewListView:
                return GetSelectedRowsFromCampaignMemberListViewInLightning();
            case salesforcePageTypeEnum.ClassicJobOrdersApplicationsListView:
                return GetSelectedRowsFromJobOrderApplicationsRelatedListView();
            case salesforcePageTypeEnum.LightningJobOrdersApplicationsListView:
                return GetSelectedRowsFromJobOrderApplicationsSectionsInLightning(campaignButtonClicked);
            default:
                return null;
        }
    }

    /** Finds the Contact or Leads Id from a CampaignMember view in Classic mode*/
    function GetRecordIdFromCampaignMemberViewClassic() {
        const contactOrLeadElement = document.querySelector("#Contact_ileinner a, #Lead_ileinner a");
        if (contactOrLeadElement) {
            const hrefAttribute = contactOrLeadElement.getAttribute("href");
            if (hrefAttribute) {
                return hrefAttribute.replace("/", "");
            }
        }
    }

    /** Finds the Contact or Leads Id from a CampaignMember view in Lightning mode */
    function GetRecordIdFromCampaignMemberViewLightning() {
        const allElementsWithARecordId = document.querySelectorAll(".row-main a[data-recordid]");
        for (const element of allElementsWithARecordId) {
            if (element.dataset) {
                const dataRecordId = element.dataset.recordid;
                if (dataRecordId) {
                    if (/^003|00Q/.test(dataRecordId)) {
                        return dataRecordId;
                    }
                }
            }
        }
    }

    /** 
     * Creates a button we'll use for campaigns across salesforce classic  
     * @returns {DOMElement}- He button element
     */
    function CreateCampaignButtonForClassic() {
        return CreateElement("input", "btn ebstaButtonCampaign", null, "EbstaBtAddToCampaignSalesforce", {
            type: "button",
            value: "Add to Ebsta Cadence",
            title: "Add to Ebsta Cadence",
            name: "ebstaCampaign"
        });
    }

    /** 
     * Creates a button we'll use for campaigns across salesforce lightning  
     * @returns {DOMElement}- He button element
     */
    function CreateCampaignButtonForLightning() {
        let button = CreateElement("li", "slds-button slds-button--neutral slds-button_neutral slds-truncate ebstaButtonCampaign", null, null);
        button.appendChild(CreateElement("a", null, "Add To Ebsta Cadence", "EbstaBtAddToCampaignSalesforce", {
            title: "Add to Ebsta Cadence"
        }));
        return button;
    }

    /** 
     *  Creates and inserts a button which conforms to the same styling as other buttons for list view
     *  @returns {function} - The functon which inserts the button into salesforce
     */
    function CreateCampaignButtonInClassicListView() {
        let campaignButton = CreateCampaignButtonForClassic();

        let insertButtonFunction = function () {
            return (function (ebstaCampaignButton) {
                var topNavTabs = document.getElementsByClassName("topNavTab");
                for (var i = 0; i < topNavTabs.length; i++) {
                    var topNavTab = topNavTabs[i];
                    if (topNavTab.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                        topNavTab.appendChild(ebstaCampaignButton);
                        return true;
                    }
                }
                return false;
            })(campaignButton);
        };

        return {
            InsertButtonFunction: insertButtonFunction,
            CampaignButton: campaignButton
        };
    }

    /** 
    *  Creates and inserts a button which conforms to the same styling as other buttons in the Campaign Member list view
    *  @returns {function} - The functon which inserts the button into salesforce
    */
    function CreateCampaignButtonInClassicRecordViewForCampaignMemberListView() {
        let campaignButton = CreateCampaignButtonForClassic();

        campaignButton.setAttribute('style', 'float:right !important; margin-right:10px!important');

        let insertButtonFunction = function () {
            return (function (ebstaCampaignButton) {
                var editColumnElements = document.getElementById("editColumns");
                if (editColumnElements !== null) {
                    if (editColumnElements.parentElement.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                        insertBefore(ebstaCampaignButton, editColumnElements);
                        return true;
                    }
                }
                return false;
            })(campaignButton);
        };

        return {
            InsertButtonFunction: insertButtonFunction,
            CampaignButton: campaignButton
        };
    }

    /** 
    *  Creates and inserts a button which conforms to the same styling as other buttons for a record view
    *  @returns {function} - The functon which inserts the button into salesforce
    */
    function CreateCampaignButtonInClassicRecordView() {
        let campaignButton = CreateCampaignButtonForClassic();

        let insertButtonFunction = function () {
            return (function (ebstaCampaignButton) {
                var topButtonRow = document.getElementById("topButtonRow");
                if (topButtonRow !== null) {
                    if (topButtonRow.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                        topButtonRow.appendChild(ebstaCampaignButton);
                    }
                    return true;
                }
                return false;
            })(campaignButton);
        };

        return {
            InsertButtonFunction: insertButtonFunction,
            CampaignButton: campaignButton
        };
    }

    /** 
     *  Creates and inserts a button which conforms to the same styling as other buttons in lightning
     *  @returns {DOMElement} - The button created
     */
    function CreateAndInsertCampaignButtonLightning(isListView) {

        let campaignButton = CreateCampaignButtonForLightning();

        let insertButtonFunction = function () {

            return (function (ebstaCampaignButton) {

                // if we're in list view mode.
                if (isListView) {
                    var ribbonElements = document.querySelectorAll(".oneActionsRibbon");

                    if (ribbonElements && ribbonElements.length > 0) {

                        for (var i = 0; i < ribbonElements.length; i++) {
                            var ribbonElement = ribbonElements[i];
                            if (ribbonElement.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                                Prepend(ebstaCampaignButton, ribbonElement);
                            }
                        }
                    }
                } else { // otherwise we're viewing a record - layout is different.

                    // lightning hides page contents when navigating between pages - select the active one.
                    var activePageContentDiv = document.querySelectorAll(".oneContent.active");

                    if (activePageContentDiv && activePageContentDiv.length > 0) {
                        // then select the force action buttons ribbon.
                        var alternateRibbonElements = activePageContentDiv[0].querySelectorAll(".actionsContainer .oneActionsRibbon");

                        // if we found the element go append the button to it.
                        if (alternateRibbonElements && alternateRibbonElements.length > 0) {
                            var ribbonElementA = alternateRibbonElements[0];
                            if (ribbonElementA.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                                Prepend(ebstaCampaignButton, ribbonElementA);
                            }
                        }
                        else {
                            //temp fix covering SF temp fix 
                            alternateRibbonElements = activePageContentDiv[0].querySelectorAll(".actionsContainer");
                            // if we found the element go append the button to it.
                            if (alternateRibbonElements && alternateRibbonElements.length > 0) {
                                var ribbonElementA = alternateRibbonElements[0];
                                if (ribbonElementA.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                                    Prepend(ebstaCampaignButton, ribbonElementA);
                                }
                            }
                            else
                            {
                                // covering when they have the sidebar out--need to go all doc
                                alternateRibbonElements = document.querySelectorAll("div.slds-grid div.forceActionsContainer");
                                if (alternateRibbonElements && alternateRibbonElements.length > 0) {
                                    alternateRibbonElements.forEach(alternateRibbonElement => {
                                        if (alternateRibbonElement) {
                                            if (alternateRibbonElement.className == "slds-grid forceActionsContainer") {
                                                if (alternateRibbonElement.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                                                    alternateRibbonElement.appendChild(ebstaCampaignButton);
                                                }
                                            }
                                        }
                                    });

        
                                }
                            }

                        }

                    }
                }
            })(campaignButton);
        };

        return {
            InsertButtonFunction: insertButtonFunction,
            CampaignButton: campaignButton
        };
    }

    /** 
      *  Creates and inserts a button which conforms to the same styling as other buttons in the Applications Related List on the JobOrder page
      *  @returns {object} - The button element and the function which inserts the button into salesforce
      */
    function CreateCampaignButtonInClassicRecordViewForJobOrderApplicationsSection() {
        let campaignButton = CreateCampaignButtonForClassic();

        let insertButtonFunction = function () {
            return (function (ebstaCampaignButton) {
                const applicationsTable = EbstaXpath.GetFirstNode('//h3[text()="Applications"]/ancestor::table[1]');
                if (applicationsTable !== null) {
                    const tableButtonsContainer = applicationsTable.getElementsByClassName("pbButton")[0];
                    if (tableButtonsContainer) {
                        if (tableButtonsContainer.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                            tableButtonsContainer.appendChild(ebstaCampaignButton);                            
                        }
                        return true;
                    }
                }
                return false;
            })(campaignButton);
        };

        return {
            InsertButtonFunction: insertButtonFunction,
            CampaignButton: campaignButton
        };
    }

    /** 
     *  Creates and inserts a button which conforms to the same styling as other buttons in the Applications tab on the JobOrder page
     *  @returns {object} - The button element and the function which inserts the button into salesforce
     */
    function CreateCampaignButtonInLightningViewForJobOrderApplicationsTab() {
        let campaignButton = CreateCampaignButtonForLightning();

        let insertButtonFunction = function () {
            return (function (ebstaCampaignButton) {
                let applicationsTabButtonInserted = false, applicationsHoverOverButtonInsered = false;

                const applicationsTabButtonHeader = EbstaXpath.GetFirstNode('//div[contains(@class,"windowViewMode-normal")]//div[contains(@class,"slds-p-left_xx-small actionsContainer")]//ul[contains(@class,"slds-button-group-list")]');
                if (applicationsTabButtonHeader !== null) {
                    if (applicationsTabButtonHeader.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                        Prepend(ebstaCampaignButton, applicationsTabButtonHeader);
                    }
                    applicationsTabButtonInserted = true;
                }

                const applicationsHoverOverDiv = EbstaXpath.GetFirstNode('//div[contains(@class,"uiContainerManager")]//h2//a[text()="Applications"]/ancestor::div[contains(@class,"uiContainerManager")]//ul[contains(@class,"oneActionsRibbon")]');
                if (applicationsHoverOverDiv !== null) {
                    if (applicationsHoverOverDiv.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                        Prepend(ebstaCampaignButton, applicationsHoverOverDiv);
                    }
                    applicationsHoverOverButtonInsered = true;
                }
                // If both buttons are inserted return true so we don't keep attempting to insert as not needed
                return applicationsTabButtonInserted && applicationsHoverOverButtonInsered;
            })(campaignButton);
        };

        return {
            InsertButtonFunction: insertButtonFunction,
            CampaignButton: campaignButton
        };
    }

    /** 
 *  Creates and inserts a button which conforms to the same styling as other buttons in the Applications tab on the JobOrder page
 *  @returns {object} - The button element and the function which inserts the button into salesforce
 */
    function CreateCampaignButtonInLightningViewForApplicationsListView() {
        let campaignButton = CreateCampaignButtonForLightning();

        let insertButtonFunction = function () {
            return (function (ebstaCampaignButton) {

                const applicationsButtonHeader = document.querySelector(".active .oneActionsRibbon");
                if (applicationsButtonHeader) {
                    if (applicationsButtonHeader.getElementsByClassName("ebstaButtonCampaign").length === 0) {
                        Prepend(ebstaCampaignButton, applicationsButtonHeader);                       
                    }
                    return true;
                }

                return false;
            })(campaignButton);
        };

        return {
            InsertButtonFunction: insertButtonFunction,
            CampaignButton: campaignButton
        };
    }

    /**
     * Ensures we only try to intialise the whole closure if we're on a page which we'll end up
     * showing the "Add to Ebsta Cadence" button. This should help save on calls up to our server.
     * @returns {boolean} - True if we should try to insert button, false if not
     */
    function DoesUrlMatchUrlForInitialisation() {
        //Contact and Lead list views in Classic
        if (/force\.com\/00(3|Q)\?fcf=/.test(document.URL) === true
            ||
            //Contact and Lead records in Classic
            /force\.com\/00(3|Q|v)([0-9a-zA-Z]{12})/.test(document.URL) === true) {
            return true;
        }

        //Saleforce lighting - This is a SPA so when we land on any Salesforce lightning page we need to use
        //mutation observers to detect page changes and if we should be inserting the button from there on
        if (document.URL.toLowerCase().indexOf("lightning.force.com") !== -1) return true;

        // Campaign Member list view in Classic view
        if (/force.com\/ui\/campaign\/CampaignManageMembersExistingTab/.test(document.URL) === true) return true;

        // Job Order (Classic View) (ts2__Job__c)
        if (/force\.com\/a0K([0-9a-zA-Z]{12})/.test(document.URL) === true) return true;

        return false;
    }

    /**
     * Core setup for the module
     * @param {any} webUserId - users id
     */
    function Initialize(webUserId) {
        if (!DoesUrlMatchUrlForInitialisation()) return;

        InsertCampaignButtonIfUserHasMailboxesToUse(true, function () {
            InsertCampaignButton(webUserId);

            if (document.URL.toLowerCase().indexOf("lightning.force.com") !== -1) {
                if (gMutationObserverLightning === null) {
                    SetMutationObserverInLightning(function () {
                        (function (webUserId) {
                            InsertCampaignButton(webUserId);
                        })(webUserId);
                    });
                }
            }
        });
    }

    /**
     * Checks to see if a user has any mailboxes to use within Campaigns and if so will fire the callback
     * @param {boolean} forceHttpRequestRetry - If we should fire a fresh HTTP request to get the list of mailboxes for a users
     * @param {function} callBack - If we find a user has mailboxes we'll fire this function which should be the core functionality
     *                              around inserting buttons into Salesforce
     */
    function InsertCampaignButtonIfUserHasMailboxesToUse(forceHttpRequestRetry, callBack) {
        chrome.extension.sendMessage({
            MessageType: "GetCampaignSetup",
            ForceHttpRequestRetry: forceHttpRequestRetry
        },
            function (response) {
                if (response.success) {
                    const responseValues = response.values;
                    if (responseValues) {
                        if (responseValues.SendFromMailboxes) {
                            if (responseValues.SendFromMailboxes.length > 0) {
                                if (typeof callBack === "function") callBack();
                            }
                        }
                    }
                }
            });
    }

    /**
     * Inserts a button into salesforce where a user can send campaigns
     * @param {string} webUserId - The id of the user
     */
    function InsertCampaignButton(webUserId) {

        let rtnObject = null, insertButtonFunction = null, campaignButton = null;

        //Detect what type of page we're on
        if (document.URL.indexOf("force.com/003?fcf=") !== -1 ||
            document.URL.indexOf("force.com/00Q?fcf=") !== -1) {
            gSalesforcePageType = salesforcePageTypeEnum.ClassicListView;
            rtnObject = CreateCampaignButtonInClassicListView();
        } else if (/force\.com\/.*\/Contact\/list/.test(document.URL) === true ||
            /force\.com\/.*\/Lead\/list/.test(document.URL) === true) {
            gSalesforcePageType = salesforcePageTypeEnum.LightningListView;
            rtnObject = CreateAndInsertCampaignButtonLightning(true);
        } else if (/force\.com\/003([0-9a-zA-Z]{12})/.test(document.URL) === true ||
            /force\.com\/00Q([0-9a-zA-Z]{12})/.test(document.URL) === true) {
            gSalesforcePageType = salesforcePageTypeEnum.ClassicRecordView;
            rtnObject = CreateCampaignButtonInClassicRecordView();
        } else if (/force\.com\/.*\/003([0-9a-zA-Z]{15})\/view/.test(document.URL) === true ||
            /force\.com\/.*\/00Q([0-9a-zA-Z]{15})\/view/.test(document.URL) === true) {
            gSalesforcePageType = salesforcePageTypeEnum.LightningRecordView;
            rtnObject = CreateAndInsertCampaignButtonLightning(false);
        } else if (/force\.com\/00v([0-9a-zA-Z]{12})/.test(document.URL) === true) {
            // Campaign Member View (Classic)
            gSalesforcePageType = salesforcePageTypeEnum.ClassicCampaignMemberView;
            rtnObject = CreateCampaignButtonInClassicRecordView();
        } else if (/force\.com\/.*\/00v([0-9a-zA-Z]{15})\/view/.test(document.URL) === true) {
            // Campaign Member View (Lightning)
            gSalesforcePageType = salesforcePageTypeEnum.LightningCampaignMemberView;
            rtnObject = CreateAndInsertCampaignButtonLightning(false);
        } else if (/force.com\/ui\/campaign\/CampaignManageMembersExistingTab/.test(document.URL)) {
            // Campaign Member - Manage members List View (Classic)
            gSalesforcePageType = salesforcePageTypeEnum.ClassicCampaignMemberViewListView;
            rtnObject = CreateCampaignButtonInClassicRecordViewForCampaignMemberListView();
        } else if (/force\.com\/.*\/701([0-9a-zA-Z]{15})\/related\/CampaignMembers\/view/.test(document.URL)) {
            // Campaign Member List View (Lightning)
            gSalesforcePageType = salesforcePageTypeEnum.LightningCampaignMemberViewListView;
            rtnObject = CreateAndInsertCampaignButtonLightning(true);
        } else if (/force\.com\/a0K([0-9a-zA-Z]{12})/.test(document.URL) === true) {
            // Job Order Applications Related List View (Classic)
            gSalesforcePageType = salesforcePageTypeEnum.ClassicJobOrdersApplicationsListView;
            rtnObject = CreateCampaignButtonInClassicRecordViewForJobOrderApplicationsSection();
        } else if (/force\.com\/.*\/a0K([0-9a-zA-Z]{15})\/view/.test(document.URL) === true) {
            // Job Order Applications Related List View (Lightning)
            gSalesforcePageType = salesforcePageTypeEnum.LightningJobOrdersApplicationsListView;
            rtnObject = CreateCampaignButtonInLightningViewForJobOrderApplicationsTab();
        } else if (/force\.com\/.*\/a0K([0-9a-zA-Z]{15})\/related\/ts2__Applications__r\/view/.test(document.URL) === true) {
            // Applications Related List View (Lightning)
            gSalesforcePageType = salesforcePageTypeEnum.LightningApplicationsRelatedListView;
            rtnObject = CreateCampaignButtonInLightningViewForApplicationsListView();
        }

        if (rtnObject !== null) {
            insertButtonFunction = rtnObject.InsertButtonFunction;
            campaignButton = rtnObject.CampaignButton;

            //We should only need to create the iframe once per script execution
            if (gCampaignIframe === null) {
                gCampaignIframe = CreateElement("iframe",
                    "ebstaIFrameOverlay",
                    "Add To Ebsta Cadence",
                    "ebstaCampaignFrame",
                    {
                        src: "chrome-extension://" +
                            chrome.i18n.getMessage("@@extension_id") +
                            "/html/CampaignManager.html?wb=" +
                            webUserId
                    });

                document.body.appendChild(gCampaignIframe);
            }

            if (campaignButton) {
                campaignButton.addEventListener("click",
                    function () {
                        (function (iframeElement) {
                            iframeElement.style.display = "block";
                            iframeElement.contentWindow.postMessage({
                                "Message": "OPENCAMPAIGN",
                                "RecordsSelected": GetSelectedRowsFromListView(campaignButton)
                            }, "*");
                        })(gCampaignIframe);
                    });
            }

            if (typeof insertButtonFunction === 'function') {
                insertButtonFunction();
                RetryWithCallback(10, 0.5, insertButtonFunction, null, null);
            }
        }
    }

    /**
     * Sets a mutation observr on the whole DOM in Salesforce lightning as the majority of loading is 
     * done asynchronously so DOMContentLoaded events aren't always fired to help action what we need to action
     * @param {any} callback - What function to be call on each change detection
     */
    function SetMutationObserverInLightning(callback) {
        var mutationObserverCallback = function (mutations) {
            callback();
        };

        // Attach a mutation obeserver onto the Draft element within the compose window so 
        // we can keep track of any draft id changes
        // TODO: MUTATION
        gMutationObserverLightning = new MutationObserver(mutationObserverCallback);
        var config = { subtree: true, childList: true };
        gMutationObserverLightning.observe(document.body, config);
    }

    /** Hide the iframe we injected */
    function HideIframe() {
        if (gCampaignIframe !== null) {
            HideElement(gCampaignIframe);
        }
    }

    /** Stops the mutation observer */
    function DisconnectMutationObserver() {
        if (gMutationObserverLightning) {
            gMutationObserverLightning.disconnect();
        }
    }

    return {
        Initialize: Initialize,
        HideIframe: HideIframe,
        DisconnectMutationObserver: DisconnectMutationObserver,
        GetSelectedRowsFromListView: GetSelectedRowsFromListView
    };
})();