﻿//Closure represent the Send Later UI within an object
var global_SendLaterObject = new SendLaterObject();

Date.prototype.addHours = function (h) {
    this.setHours(this.getHours() + h);
    return this;
}

//Takes an element within a compose window and will return you the container for the window
function GetComposeWindowFromElement(elementWithinCompose) {
    return elementWithinCompose.closest(".I5");
}

//Closure object representing SendLater information
function SendLaterObject() {
    var schedulePopUpElement = null; //The Send Later object
    var calendarElement = null; // The Calendar object
    var hourElement = null; // Hour drop down object
    var minuteElement = null; // Minute drop down object
    var amPmElement = null; // AM/PM drop down object
    var chosenFieldElements = null; //All drop down objects
    var lastDateSelected = new Date(); //Date selected to be used and the date we'll schedule the email for
    var timeSpan = null; // Text element where we inform the user when the email will be sent
    var scheduleButton = null; // Schedule button object
    var cancelScheduleButton = null; // Cancel schedule button object
    var sendLaterId = null; //Ebsta unique id for this drafts scheduled send
    var scheduledDate = null; // The date this send later instance is scheduled for
    var ebstaSendLaterButton = null; // The button which appears in the compose window
    var monthNames = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"],
        lEbstaComposeButtonsClosure = null;

    return {
        SetlEbstaComposeButtonsClosure : function(closure) {
            lEbstaComposeButtonsClosure = closure;
        },
        /**
         * Shows or creates the Send Later Pop Up
         */
        ShowSchedulePopUp : function() {
            if (schedulePopUpElement == null) {
                this.CreateSchedule();
            }

            if (scheduledDate != null) {
                lastDateSelected = scheduledDate;
                this.EnabledCancelScheduleButton();
            } else {
                lastDateSelected = new Date().addHours(1);
            }

            this.SetDateOnCalendar(lastDateSelected);
            this.SetTimeAndDateEmailIsScheduledFor();
            this.SetTimeDateDropDownFieldsToDate(lastDateSelected);
            this.CheckLastDateSelected();

            schedulePopUpElement.style.display = "block";

            //Append pop up element to the buttons container where all the ebsta buttons currently sit
            var buttonsContainer = ebstaSendLaterButton.closest(".n1tfz");

            if (typeof buttonsContainer == "undefined" || buttonsContainer == null || buttonsContainer.length < 1) {
                buttonsContainer = ebstaSendLaterButton.closest('.btC');
            }

            if (buttonsContainer != null) {
                buttonsContainer.appendChild(schedulePopUpElement);
            }
        },
        CreateSchedule : function() {
            var $this = this;
            schedulePopUpElement = this.CreateSchedulePopUp();
            chosenFieldElements = $(schedulePopUpElement).find(".chosen-select");
            $(chosenFieldElements).chosen({ width: '30%' }).change(function() {
                $this.SetLastSelectedDate();
            });
            return schedulePopUpElement;
        },
        /**
         * Create the Send later button which appears in the compose window
         * @returns {} 
         */
        CreateSendLaterButton: function (addClickEvent) {
            var td = Createtd("gU ebstaTD");

            ebstaSendLaterButton = CreateElement("div", "sfEbSendLaterBtn", null, "EbstaBtSendLater");
            ebstaSendLaterButton.title = 'Send later';

            var sendLaterClosureInstance = this;

            if (addClickEvent) {
                ebstaSendLaterButton.addEventListener("click", function () {
                    sendLaterClosureInstance.SendLaterButtonClickEvent();
                });
            }

            td.appendChild(ebstaSendLaterButton);
            return td;
        },
        /**
         * Click event on the Send later button within the compose window
         * @returns {} 
         */
        SendLaterButtonClickEvent : function() {
            this.ShowSchedulePopUp();
        },
        HideSchedulePopUp: function () {
            if (schedulePopUpElement != null) {
                schedulePopUpElement.style.display = "none";
            }
        },
        SetLastSelectedDate :function() {
            this.SetTimeLastDateSelected();
            this.SetTimeAndDateEmailIsScheduledFor();
            this.CheckLastDateSelected();
        },
        SetLastDateSelected : function(date) {
            lastDateSelected = date;
        },
        SetTimeLastDateSelected: function () {
            var hour = parseInt(this.GetHourDropDownValue());
            var amOrPm = this.GetAmOrPmDropDownValue();
            if (amOrPm === "PM" && hour !== 12) hour += 12;
            else if (amOrPm === "AM" && hour === 12) hour = 0;
            lastDateSelected.setHours(hour);
            lastDateSelected.setMinutes(this.GetMinuteDropDownValue());
        },
        CheckLastDateSelected: function () {
            if (lastDateSelected != null && lastDateSelected < new Date()) {
                timeSpan.innerText = "";
                timeSpan.appendChild(CreateTextNode("Date is not in the future."));
                timeSpan.classList.remove("ebsta-alert-info");
                timeSpan.classList.add("ebsta-alert-danger");
                this.DisableScheduleButton();
            } else {
               timeSpan.classList.add("ebsta-alert-info");
               timeSpan.classList.remove("ebsta-alert-danger");
              this.EnabledScheduleButton();
          }
        },
        CreateCalendar: function (calendarContainerDiv) {
            var $this = this;
            calendarElement = new CibulCalendar(calendarContainerDiv, {
                range: false,
                onSelect: function (selected) {
                    lastDateSelected = selected;
                    $this.SetLastSelectedDate();
                }
            });
        },
        SetDateOnCalendar : function(date) {
            calendarElement.setSelected({ begin: date, end: date });
        },
        CreateTimeDropDowns : function(columnDiv) {
            this.CreateHourDropDown(columnDiv);
            this.CreateMinuteDropDown(columnDiv);
            this.CreateAMPMDropDown(columnDiv);
        },
        CreateHourDropDown: function (columnDiv) {
            hourElement = CreateElement("select", "chosen-select", "", "EbstaDdSendLaterHour");
            for (var i = 1; i < 13;i++) {
                var opt = CreateOptionElement(i, i);
                hourElement.appendChild(opt);
            }
            columnDiv.appendChild(hourElement);
        },
        CreateMinuteDropDown: function (columnDiv) {
            minuteElement = CreateElement("select", "chosen-select", "", "EbstaDdSendLaterMinute");
            for (var i = 0; i < 60; i++) {               
                var minuteText = i < 10 ? "0" + i : i;
                var opt = CreateOptionElement(minuteText, minuteText);
                minuteElement.appendChild(opt);
            }
            columnDiv.appendChild(minuteElement);
        },
        CreateAMPMDropDown: function (columnDiv) {
            amPmElement = CreateElement("select", "chosen-select", "", "EbstaDdSendLaterAMPM");
            var amOption = CreateOptionElement("AM", "AM");
            var pmOption = CreateOptionElement("PM", "PM");
            amPmElement.appendChild(amOption);
            amPmElement.appendChild(pmOption);

            columnDiv.appendChild(amPmElement);
        },
        SetTimeDateDropDownFieldsToDate: function(date) {
            var hour = date.getHours();
            this.SetHourDropDown(this.GetFriendlyHourFormat(hour));
            this.SetMinuteDropDown(this.GetFriendlyMinuteFormat(date.getMinutes()));
            this.SetAmOrPmDropDown(this.Get12HourFormat(hour));
            $(chosenFieldElements).trigger("chosen:updated");
        },
        Get12HourFormat : function(hour) {
            if (hour > 11) {
                return "PM";
            } else {
                return "AM";
            }
        },
        GetHourDropDownValue: function () {
            return $(hourElement).val();
        },
        SetHourDropDown : function(value) {
            hourElement.value = value;
        },
        GetMinuteDropDownValue: function () {
            return $(minuteElement).val();
        },
        SetMinuteDropDown: function (value) {
            minuteElement.value = value;
        },
        GetAmOrPmDropDownValue: function () {
            return $(amPmElement).val();
        },
        SetAmOrPmDropDown : function(value) {
            amPmElement.value = value;
        },
        CreateTimeSpan : function(elementToAppendTo) {
            timeSpan = CreateElement("span", "ebsta-alert ebsta-alert-info sfEbScheduledTime", null, null);
            elementToAppendTo.appendChild(timeSpan);
        },
        GetTimeAndDateEmailIsScheduledFor: function () {
            return new Date(lastDateSelected);
        },
        GetFriendlyNameForTimeAndDateIsScheduledFor: function (lastDateSelected) {
            var day = this.AppendOrdinalIndicatorOntoDay(lastDateSelected.getDate());
            var month = monthNames[lastDateSelected.getMonth()];
            var time = this.GetFriendlyTimeFormat(lastDateSelected);
            //var timeZone = this.GetTimeZoneFromDate(lastDateSelected);
            return month + " " + day + " " + time; // + " (" + timeZone + ")";
        },
        GetFriendlyHourFormat :function(hour) {
            return (hour + 24) % 12 || 12;
        },
        GetFriendlyMinuteFormat : function(minutes) {
            if (minutes < 10) minutes = "0" + minutes;
            return minutes;
        },
        GetFriendlyTimeFormat: function (date) {
            var amOrPm = this.Get12HourFormat(date.getHours());
            var hr = this.GetFriendlyHourFormat(date.getHours());
            var minutes = this.GetFriendlyMinuteFormat(date.getMinutes());
            
            return hr + ":" + minutes + " " + amOrPm;
        },
        GetTimeZoneFromDate: function (date) {
            var fullTz = date.toString().match(/\(([A-Za-z\s].*)\)/)[1];
            var parts = fullTz.split(' ');
            var tz = "";
            parts.forEach(function(element) {
                 tz += element.substring(0, 1);
            });
            return tz;
        },
        SetTimeAndDateEmailIsScheduledFor: function () {
            timeSpan.innerText = "";
            timeSpan.appendChild(CreateTextNode(this.GetFriendlyNameForTimeAndDateIsScheduledFor(lastDateSelected)));
        },
        AppendOrdinalIndicatorOntoDay: function (number) {
            var j = number % 10,
                k = number % 100;
            if (j == 1 && k != 11) {
                return number + "st";
            }
            if (j == 2 && k != 12) {
                return number + "nd";
            }
            if (j == 3 && k != 13) {
                return number + "rd";
            }
            return number + "th";
        },
        SetScheduleButton: function (scheduleButtonElem) {
            scheduleButton = scheduleButtonElem;
            $this = this;
            scheduleButton.addEventListener("click", function () {
                $this.ScheduleButtonClick(lastDateSelected);
            });
        },
        /**
         * Copys the schedule button by cloning it which removes all listenered attached to it
         * @returns {} 
         */
        ResetScheduleButton: function () {
            scheduleButton = CloneAndOverwriteNode(scheduleButton);
        },
        /**
         * Takes a function and argumnets and adds an event listener to the schedule button
         * @param {} functionToCall 
         * @param {} args 
         * @returns {} 
         */
        AddEventListenerToScheduleButton: function (functionToCall, args) {
            if (scheduleButton != null) {
                $this = this;
                scheduleButton.addEventListener("click", function () {
                    functionToCall.apply(this, args);
                });
            }
        },
        SetCancelScheduleButton: function (cancelScheduleButtonElem) {
            cancelScheduleButton = cancelScheduleButtonElem;
            $this = this;
            cancelScheduleButton.addEventListener("click", function () {
                $this.CancelScheduleButtonClick(cancelScheduleButton);
            });
        },
        /**
         * Copys the cancel schedule button by cloning it which removes all listenered attached to it
         * @returns {} 
         */
        ResetCancelScheduleButton : function() {
            cancelScheduleButton = CloneAndOverwriteNode(cancelScheduleButton);
        },
        /**
         * Takes a function and argumnets and adds an event listener to the cancel schedule button
         * @param {} functionToCall 
         * @param {} args 
         * @returns {} 
         */
        AddEventListenerToCancelButton: function (functionToCall, args) {
            if (cancelScheduleButton != null) {
                $this = this;
                cancelScheduleButton.addEventListener("click", function() {
                    functionToCall.apply(this,args);
                });
            }
        },
        DisableScheduleButton: function () {
            $(scheduleButton).addClass("disabled");
        },
        EnabledScheduleButton: function () {
            $(scheduleButton).removeClass("disabled");
        },
        DisableCancelScheduleButton : function() {
            $(cancelScheduleButton).addClass("disabled");
        },
        EnabledCancelScheduleButton : function() {
            $(cancelScheduleButton).removeClass("disabled");
        },
        ScheduleButtonClick: function (dateSelected) {
            this.HideSchedulePopUp();
            scheduledDate = dateSelected;
            this.SetAttributesOnSendLaterButton(ebstaSendLaterButton, dateSelected, null);
            this.ChangeSendLaterButtonToGreen(ebstaSendLaterButton);
            lEbstaComposeButtonsClosure.UpdateSendEmailButton();
        },
        SetAttributesOnSendLaterButton : function(sendLaterBtn, date, sendLaterId) {
            if (sendLaterId != null) {
                sendLaterBtn.setAttribute("ebSendLaterId", sendLaterId);
            }
        },
        CancelScheduleButtonClick: function () {
            this.RemoveAttributesOffSendLaterButton(ebstaSendLaterButton);
            scheduledDate = null;
            this.HideSchedulePopUp();
            this.ResetSendLaterButton(ebstaSendLaterButton);
            this.CancelSendLaterRequest();
            lEbstaComposeButtonsClosure.UpdateSendEmailButton();
        },
        /**
         * Clears attributes off the send later button
         * @param {} ebstaSendLaterButton 
         * @returns {} 
         */
        RemoveAttributesOffSendLaterButton : function(ebstaSendLaterButton) {
            ebstaSendLaterButton.removeAttribute("ebsendlaterid");
        },
        GetEbstaSendButton : function (lastSendLaterButtonClicked) {
            var buttonsContainer = lastSendLaterButtonClicked.closest(".n1tfz");

            if (typeof buttonsContainer == "undefined" || buttonsContainer == null || buttonsContainer.length < 1) {
                buttonsContainer = lastSendLaterButtonClicked.closest('.btC');
            }

            if (buttonsContainer != null) {
                var ebstaSendButton = buttonsContainer.getElementsByClassName("sfEbSendBtn");
                if (ebstaSendButton.length > 0) {
                    return ebstaSendButton[0];
                }
            }
            return null;
        },
        ShowEbstaSendButton: function (lastSendLaterButtonClicked) {
            var ebstaSendButton = this.GetEbstaSendButton(lastSendLaterButtonClicked);
            if (ebstaSendButton != null) {
                ebstaSendButton.style.display = null;
            }
        },
        HideEbstaSendButton: function (lastSendLaterButtonClicked) {
            var ebstaSendButton = this.GetEbstaSendButton(lastSendLaterButtonClicked);
            if (ebstaSendButton != null) {
                ebstaSendButton.style.display = "none";
            }
        },
        GetGmailSendButton : function(lastSendLaterButtonClicked) {
            var buttonsContainer = lastSendLaterButtonClicked.closest(".n1tfz");

            if (typeof buttonsContainer == "undefined" || buttonsContainer == null || buttonsContainer.length < 1) {
                buttonsContainer = lastSendLaterButtonClicked.closest('.btC');
            }

            if (buttonsContainer != null) {
                var gmailButton = buttonsContainer.getElementsByClassName("Up");
                if (gmailButton.length > 0) {
                    return gmailButton[0];
                }
            }
            return null;
        },
        GetScheduledDate : function() {
            return scheduledDate;
        },
        IsScheduled : function() {
            return scheduledDate != null;
        },
        SaveSendLaterRequest: function (messageId, dateEmailScheduledFor, sendLaterButton, complexAddValues, isEmailTrackingOn, sfTemplateId) {
            chrome.extension.sendMessage({
                MessageType: "sendlatersaveschedule",
                MessageId: messageId,
                CurrentSendLaterId: sendLaterId,
                ScheduledDate: dateEmailScheduledFor,
                GmailUsername: GetGmailUsersEmailAddress(),
                IsEmailTracked: isEmailTrackingOn,
                ComplexAddValues: complexAddValues,
                SfTemplateId: sfTemplateId
            });
        },
        IsEmailTracked: function (sendLaterButton) {
            var gmailButtonsContainer = sendLaterButton.closest(".n1tfz");

            if (typeof gmailButtonsContainer == "undefined" || gmailButtonsContainer == null || gmailButtonsContainer.length < 1) {
                gmailButtonsContainer = sendLaterButton.closest('.btC');
            }

            if (gmailButtonsContainer != null) {
                var ebstaSendButton = gmailButtonsContainer.getElementsByClassName("sfEbstaGmailEmailTrackingSelected");
                if (ebstaSendButton.length > 0) {
                    return true;
                }
            }
            return false;
        },
        CancelSendLaterRequest: function () {
            if (sendLaterId) {
                chrome.extension.sendMessage({
                    MessageType: "sendlatercancelschedule",
                    SendLaterId: sendLaterId
                });
            }
        },
        GetScheduleDetailRequest: function (gmailSendButton, ebstaComposeButtonsClosure) {
            var messageId = GmailTools.Compose.GetDraftMessageIdValueFromCompose(gmailSendButton);
            if (messageId != null) {
                var thisClosureInstance = this;
                chrome.extension.sendMessage({
                    MessageType: "selectsendlaterschedule",
                    MessageId: messageId 
                }, function (response) {
                    if (response != null) {
                        thisClosureInstance.HandleCallBackFromGetScheduleDetailRequest(response, ebstaComposeButtonsClosure);
                    }
                });
            }
        },
        ChangeSendLaterButtonToGreen : function(buttonElement) {
            buttonElement.classList.add("ebsta-btn-success");
            buttonElement.title = "Email scheduled for " + this.GetFriendlyNameForTimeAndDateIsScheduledFor(scheduledDate);
        },
        ResetSendLaterButton : function(buttonElement) {
            buttonElement.classList.remove("ebsta-btn-success");
            buttonElement.title = "Email not scheduled";
        },
        HandleCallBackFromSavingDraft : function(returnedSuccess) {
            if (returnedSuccess != null) {
                var elem = this.CreateResponseMessage(returnedSuccess);
                document.body.appendChild(elem);

                setTimeout(function() {
                    elem.remove();
                }, 2500);
            }
        },
        CreateResponseMessage : function(wasRequestSuccesful) {
            var text = "", className = "ebSendLaterResponseMessage ebsta-alert";
            if (wasRequestSuccesful) {
                text = "Email successfully scheduled.";
                className += " ebsta-alert-success";
            } else {
                text = "Ebsta failed to schedule this email please try again";
                className += " ebsta-alert-danger";
            }
            return CreateElement("div", className, text, null);
        },
        /**
         * Handles the request received from our server which has details about this draft and if it has previously been scheduled
         * @param {} values 
         * @returns {} 
         */
        HandleCallBackFromGetScheduleDetailRequest: function (values, ebstaComposeButtonsClosure) {
            var parsedJson = JSON.parse(values[0]);
            if (parsedJson != null) {
                var messageId = parsedJson.EmailId;
                if (messageId != null) {
                    //Set date that it was previously set for (append a Z onto the time to designate that this is a UTC)
                    scheduledDate = new Date(parsedJson.SendDateTime + "Z");
                    //Set the our id of this on the element so we can track changes and removals
                    sendLaterId = parsedJson.SendLaterId;
                    if (sendLaterId && scheduledDate instanceof Date) {
                        this.ChangeSendLaterButtonToGreen(ebstaSendLaterButton);
                    }
                }

                //Update email tracking button to reflect what the user chose for this
                var isEmailTracked = parsedJson.IsEmailTracked;
                if (isEmailTracked != null) {
                    ebstaComposeButtonsClosure.UpdateEmailTrackingButtonWithState(isEmailTracked);
                }

                //Update Complex Add buttons to show the details the user chose for this
                if (parsedJson.ComplexAddValues != null) {
                    var complexAddClosure = ebstaComposeButtonsClosure.GetComplexAddClosure();
                    complexAddClosure.SetComplexAddValues(parsedJson.ComplexAddValues);
                    ebstaComposeButtonsClosure.SetComplexAddValues(complexAddClosure);
                }

                ebstaComposeButtonsClosure.UpdateSendEmailButton();
            }
        },
        FindComposeWindowByDraftId: function (messageId) {
            var query = document.querySelector("td.I5 input[value='" + messageId + "']");
            if (query != null) {
                return GetComposeWindowFromElement(query);
            }
        },
        ScheduleEmail: function (ebstaSendbutton, complexAddValues, isEmailTrackingOn, sfTemplateId) {
            var containerDiv = ebstaSendbutton.closest(".n1tfz");

            if (typeof containerDiv == "undefined" || containerDiv == null || containerDiv.length < 1) {
                containerDiv = ebstaSendbutton.closest('.btC');
            }

            if (containerDiv != null) {
                var composeContainerDiv = GmailTools.Compose.GetWholeComposeWindow(containerDiv);
                if (composeContainerDiv != null) {
                    //We need to find the Send Later button as this will have the date value attached to it which we'll parse later on
                    var sendLaterBtnArray = containerDiv.getElementsByClassName("sfEbSendLaterBtn");
                    var closeButton = composeContainerDiv.getElementsByClassName("Ha");
                    if (sendLaterBtnArray.length === 1) {

                        var isThisAComposeWindow = closeButton.length === 1; //A compose window not a reply/forward window
                        var sendLaterButton = sendLaterBtnArray[0];
                        if (scheduledDate != null) {
                            GmailTools.Compose.TriggerChangeOnCompose(composeContainerDiv);
                            var loadingOverlay = GmailTools.Compose.InsertOverlay(composeContainerDiv, "Saving email, please wait...");

                            var sendLaterInstance = this;

                            //Lets put a decent timeout to make sure we catch the last draft save update
                            setTimeout(function () {
                                sendLaterInstance.ScheduleEmailDelayed(composeContainerDiv, sendLaterButton, isThisAComposeWindow, closeButton[0], complexAddValues, isEmailTrackingOn, sfTemplateId);
                                loadingOverlay.remove();
                            }, 5000);
                        }
                    }
                }
            }
        },
        /**
         * Call made from ScheduleEmail once the timeout has reached as we need to wait a some time to make sure we gain the right draft message
         * @param {} composeContainerDiv 
         * @param {} sendLaterInstance 
         * @param {} sendLaterButton 
         * @param {} isThisAComposeWindow 
         * @param {} closeButton 
         * @returns {} 
         */
        ScheduleEmailDelayed: function (composeContainerDiv, sendLaterButton, isThisAComposeWindow, closeButton, complexAddValues, isEmailTrackingOn, sfTemplateId) {
            // Get Draft Id
            var messageId = GmailTools.Compose.GetDraftMessageIdValueFromCompose(composeContainerDiv);

            if (messageId != null && messageId !== "undefined") {
                this.SaveSendLaterRequest(messageId, scheduledDate.toISOString(), sendLaterButton, complexAddValues, isEmailTrackingOn, sfTemplateId);
            }

            //No close button, then we'll presumse this is a draft
            if (!isThisAComposeWindow) {
                var findTheWholeComposeContainer = composeContainerDiv.closest(".aoI");
                if (findTheWholeComposeContainer) {
                    HideElement(findTheWholeComposeContainer);
                }
            } else {
                //Mimic the close event on the compose window
                SimulateMouseEventOnElement("mouseup", closeButton);
            }
        },
        /**
         * Creates the main Send Later pop up
         * @returns {} 
         */
        CreateSchedulePopUp : function () {
            var mainDiv = CreateElement("div", "sfEbSendLaterSchedule", "", "");

            var firstRow = CreateBoostrapRow(null);

            //render the calendar on the left column
            var leftColumnDiv = CreateElement("div", "calendar-canvas col-xs-5", "", "sfEbCalendarSchedule");
            this.CreateCalendar(leftColumnDiv);
            firstRow.appendChild(leftColumnDiv);

            //Right column
            var rightColumnDiv = CreateElement("div", "col-xs-5", "", "sfEbScheduleRightCol");

            //Time drop downs
            var rightColumnFirstRow = CreateBoostrapRow(null);
            rightColumnDiv.appendChild(rightColumnFirstRow);
            firstRow.appendChild(rightColumnDiv);

            this.CreateTimeDropDowns(rightColumnFirstRow);
            this.SetTimeDateDropDownFieldsToDate(new Date().addHours(1));

            //Text details on when email will be sent
            var rightColumnSecondRow = CreateBoostrapRow("ebScheduleTextDetails");
            var scheduledTimeDiv = CreateElement("div", "sfEbScheduledTimeText", "Email will be sent at:", null);
            rightColumnSecondRow.appendChild(scheduledTimeDiv);
            rightColumnDiv.appendChild(rightColumnSecondRow);

            this.CreateTimeSpan(rightColumnSecondRow);
            this.SetTimeAndDateEmailIsScheduledFor();

            //Schedule and Cancel buttons
            var rightColumnThirdRow = CreateBoostrapRow("ebScheduleButtons");

            var cancelScheduleButton = CreateButton("ebCancelScheduleButton", "Cancel Schedule", "ebsta-btn ebsta-btn-danger");
            this.SetCancelScheduleButton(cancelScheduleButton);
            rightColumnThirdRow.appendChild(cancelScheduleButton);

            var scheduleButton = CreateButton("ebScheduleButton", "Schedule", "ebsta-btn ebsta-btn-success");
            this.SetScheduleButton(scheduleButton);

            rightColumnThirdRow.appendChild(scheduleButton);
            rightColumnDiv.appendChild(rightColumnThirdRow);

            mainDiv.appendChild(firstRow);

            return mainDiv;
        },
        /**
         * Verifys all the contacts we found have an email address
         * @param {} contactArray 
         * @returns {} 
         */
        VerifyRecipients : function(contactArray) {
            if (contactArray && contactArray.length > 0) {
                for (var i = 0 ; i < contactArray.length; i++) {
                    var contact = contactArray[i];
                    if (!contact.GetEmailAddress()) {
                        ebstaMessaging.ShowMessage("You have an invalid recipient, please check all recipients are correct.", false);
                        return false;
                    }
                }
                return true;
            } else {
                ebstaMessaging.ShowMessage("Please specify at least one recipient.", false);
            }

            return false;
        }
    };
}
