﻿/// <reference path="bar.js" />
/// <reference path="gmailtools.js" />
/// <reference path="Shared/ebModalDialog.js" />
/// <reference path="Shared/domcreation.js" />

const CampaignRequestType = {
    GetSalesforceRecordDetailsFromId : 1,
    GetLiveCampaignInfoForContact : 2
};

/**
 * Module for Froala (Our WYSIWYG Editor)
 * NOTE: This has some custom integration for campaigns here so be careful if you wish to remove this to use elsewhere
 */
function FroalaClosure() {
    var gFroalaDiv = null;
    var gFroalaTextPanel = null;
    var gSubjectElement = null;
    var gViewOnlyMode = null;
    var gTemplate = null;
    var gTaskEditObj = null;
    var gModalWindow = null;

    return {
        GetFroalaDiv: function() {
            return gFroalaDiv;
        },
        SetupFroala: function (viewOnlyMode, selector, template) {
            gViewOnlyMode = viewOnlyMode;
            
            if (template ) {
                
                var froalaEditorDiv = $(selector);

                if (froalaEditorDiv.length > 0) {

                    gModalWindow = froalaEditorDiv[0];
                    this.EnableHtmlEditor();
                    this.SetTemplate(template);

                    froalaEditorDiv.froalaEditor({
                        key: "kKC1KXDF1INBh1KPe2TK==",
                        charCounterCount: false,
                        charCounterMax: -1,
                        theme: 'gray',
                        toolbarButtons: ['bold', 'italic', 'underline', 'fontFamily', 'fontSize', '|', 'color', 'paragraphFormat', 'formatOL', 'formatUL', 'insertLink', 'html'],
                        toolbarButtonsMD: ['bold', 'italic', 'underline', 'fontFamily', 'fontSize', '|', 'color', 'paragraphFormat', 'formatOL', 'formatUL', 'insertLink', 'html'],
                        toolbarButtonsSM: ['bold', 'italic', 'underline', 'fontFamily', 'fontSize', '|', 'color', 'paragraphFormat', 'formatOL', 'formatUL', 'insertLink', 'html'],
                        toolbarButtonsXS: ['bold', 'italic', 'underline', 'fontFamily', 'fontSize', '|', 'color', 'paragraphFormat', 'formatOL', 'formatUL', 'insertLink', 'html'],
                        imageEditButtons: [],
                        height: 200,
                        width: 700,
                        toolbarBottom: true,
                        fontFamily: {
                            "Arial,Helvetica,sans-serif": "Arial",
                            "Georgia,sans-serif": "Georgia",
                            "Impact,Charcoal,sans-serif": "Impact",
                            "MS PGothic, sans-serif": "MS PGothic",
                            "Tahoma,Geneva,sans-serif": "Tahoma",
                            "Times New Roman,Times,serif,-webkit-standard": "Times New Roman",
                            "Verdana,Geneva,sans-serif": "Verdana",
                            "Yu Gothic,sans-serif": "Yu Gothic"
                        }
                    });

                    gFroalaTextPanel = froalaEditorDiv[0];
                }
            }
        },
        InsertFroalaCustomHeader(taskEditObj, getIconsObject) {
            var froala = $('.fr-box');

            if ($(froala).length > 0) {
                gFroalaDiv = $(froala)[0];
                this.InsertCustomHeader(taskEditObj, getIconsObject);
            }
        },
        SetEditMode: function() {
            gViewOnlyMode = false;
        },
        SetViewMode: function() {
            gViewOnlyMode = true;
        },
        InsertCustomHeader: function (taskEditObj, scheduleLaterDiv) {
            this.SetTaskEditObj(taskEditObj);
            var customEbstaHeader = CreateElement("div", "froala-ebsta-header bg-blue-lightest");

            var subjectDiv = CreateElement("div", null, null, null);
            gSubjectElement = CreateElement("input");
            subjectDiv.appendChild(gSubjectElement);

            customEbstaHeader.appendChild(subjectDiv);

            if (scheduleLaterDiv != null) {
                customEbstaHeader.appendChild(scheduleLaterDiv);
            }

            //this.SaveButtonListener();
            gFroalaDiv.childNodes[0].parentNode.insertBefore(customEbstaHeader, gFroalaDiv.childNodes[0]);
        },
        SetSubject: function (subject, disableSubjectTextBox) {
            gSubjectElement.value = subject;
            gSubjectElement.disabled = disableSubjectTextBox;
        },
        GetSubject: function() {
            return gSubjectElement.value;
        },
        GetHtmlBody: function() {
            return $(gFroalaTextPanel).froalaEditor('html.get');
        },
        SetHtmlBody: function(html) {
            $(gFroalaTextPanel).froalaEditor('html.set', html);
        },
        DisableTextEditor: function() {
            $(gFroalaTextPanel).froalaEditor('edit.off');
        },
        EnableTextEditor: function() {
            $(gFroalaTextPanel).froalaEditor('edit.on');
        },
        HideToolbar: function() {
            $(gFroalaTextPanel).froalaEditor('toolbar.hide');
        },
        ShowToolbar: function() {
            $(gFroalaTextPanel).froalaEditor('toolbar.show');
        },
        DisableHtmlEditor: function() {
            gModalWindow.classList.add('ebsta-disable');
        },
        EnableHtmlEditor: function() {
            gModalWindow.classList.remove('ebsta-disable');
        },
        SetTemplate: function(template) {
            gTemplate = template;
        },
        GetTemplate: function() {
            return gTemplate;
        },
        SetTaskEditObj: function (obj) {
            gTaskEditObj = obj;
        },
        GetTaskEditObj: function () {
            return gTaskEditObj;
        }
    }
}


/**
 * Centralized way of creating a Contact object which is used to send a campaign to singular or multiple
 * @param {} emailAddress 
 * @param {} firstName 
 * @param {} lastName 
 * @param {} company 
 * @param {} csvMergeFields 
 * @param {string=} salesforceId - The SalesforceId for this contact
 * @returns {}
 */
function Contact(emailAddress, firstName, lastName, company, csvMergeFields, salesforceId) {
    var EmailAddress = emailAddress ? emailAddress : null;
    var Firstname = firstName ? firstName : null;
    var LastName = lastName ? lastName : null;
    var Company = company ? company : null;
    var CsvMergeFields = csvMergeFields ? csvMergeFields : null;
    var SalesforceId = salesforceId ? salesforceId : null;

    this.GetContact = function () {
        return {
            Company: Company,
            EmailAddress: EmailAddress,
            Firstname: Firstname,
            LastName: LastName,
            CsvMergeFields: CsvMergeFields,
            SalesforceId: SalesforceId
        }
    };

    this.GetEmailAddress = function () {
        return EmailAddress;
    };

    this.GetCompany = function () {
        return Company;
    };

    this.GetFirstName = function () {
        return Firstname;
    };

    this.GetLastName = function () {
        return LastName;
    };

    this.GetSalesforceId = function () {
        return SalesforceId;
    };

    this.GetFullName = function () {
        return (Firstname == null ? "" : Firstname) + " " + (LastName == null ? "" : LastName);
    }

    this.ResetContact = function () {
        EmailAddress = null;
        Firstname = null;
        LastName = null;
        Company = null;
        CsvMergeFields = null;
        SalesforceId = null;
    }
}

//creates an empty campaign manager object that will
//always return HasCampaignBeenSelected false, used when disabling camapgins
function createDisabledCampaignMgrObject() {
    return {
        HasCampaignBeenSelected: function () {
            return false;
        }
    }
}

/*
 * Tracks headers rendered for csv upload table.
 */
var csvTableHeaders = []

// create campain management object
function createCampaignMgrObject(GmailTools) {

    var starterPopup = null,
        recentlyUsedCampaignsPopup = null,
        mainCampaignMgrModel = null,
        itemsPerPage = 7,
        pageNumber = 1, //Current page number on pagination type requests
        maxResultsFound = null,
        campaignSortField = 0, // the field to sort list of campaigns by
        campaignSortAsc = false, // the direction of sort in list of campaigns
        currentSearchTerm,
        tasksWithDecisions = new Array(), // All tasks with their related decisions
        companyUsers = new Array(), //All users for current users company
        emailTrackingButtonState = false,
        campaignComposeMode = false, //States whether the campaign screen has been triggered from a compose window
        csvManager = null, //object for handling csv file
        lThisCampaignManagerInstance = null, //Helps keep track of this particular instance of this closure
        lEbstaComposeButtonsClosure = null,//Closure for the compose window this campaign manager object is associated with
        lComplexAddClosure = new ComplexAdd(), //Store complex add details
        campaignDetailsSelected, //Details for the current campaign if selected
        enableAutoRelateComplexAddForMultiRecipient = false,//Enables the complex add button for multi-recipients
        autoRelateMultiRecipientCampaign = false, //Keep track if the user has selected to auto relate the campaign to Sf records
        listOfMailboxesToSendCampaignFrom = null, //List of mailboxes allowed to send the campaign from
        listOfSalesforceCampaigns = null, //List of Salesforce Campaigns
        /**
         * Object representing the current selected mailbox in the below format;
         * {
         *    details: {
         *          Mailbox: <<mailbox name>>,
         *          Webuserid: <<user id of mailbox owner>>,
         *          IsSharedMailbox: <<has this mailbox been shared with this user>>
         *      }
         * }
         */
        selectedGoogleMailbox = null, //An object represent what has been selected in the From dropdown
        isInitialisationInProgress = false, //Are we still in the initialisation phase for the object
        isSalesforceRecordListRequestActive = false, //Flagged as true when we have an active request to salesforce running to retrieve records to pre-populate into the campaign
        isGmailMode = typeof GmailTools !== 'undefined', //If the module has been injected in then we know we're in Gmail Mode
        isSelectedMailboxAuthenticated = false, //Keeps track to know if the current mailbox select is fully authenticated
        /**
         * Unique key to help catch concurrent requests overwriting each other and display the correct campaign details last requested.
         * Set to null once the request has completed
         */
        lastGetCampaignDetailsRequestKey = null,
        /**
         * Is the first step in the selected campaign an email?
         */
        isFirstStepAnEmail = false,
        /*
         * Allows upload of extra csv fields in campaigns.
         */
        enableCsvMergeFields = false,
        /**
         * The id of the last campaign that was selected
         */
        lastLoadedCampaignId = null,
        /**
         * Enables the auto check feature where we'll go and check if selected recipients are already live/paused on
         * a campaign already
         */
        enableCampaignDupeCheck = false;

    //Help keep consitentency in naming attributes also enables using 'dataset' on an element if you want to
    var dataAttributes = {
        TaskStepNo: "data-TaskStepNo",
        CampaignId: "data-campaignId",
        ContactId: "data-campaigncontactId",
        CampaignTaskTypeId: "data-campaigntasktypeid",
        CampaignDecisionTaskTypeId: "data-CampaignDecisionTaskTypeId",
        ConnectedStepNo: "data-ConnectedStep",
        NegativeDecision: "data-negative",
        PositiveDecision: "data-positive",
        TaskInterfaceNumber: "data-taskinterface",
        TemplateId: "data-templateId",
        TaskId: "data-campaignTaskId",
        SubjectName: "data-campaignSubject",
        ebstaCampaignUniqueClientId: "data-ebstaCampaignUniqueClientId",
        ebstaCampaignScheduledLater: "data-ebstaCampaignScheduledLater"
    }

    var icons = {
        RightArrow: "fa fa-caret-right fa-lg",
        SendEmail: "fa fa-paper-plane-o fa-lg ebsta-paddingright-ten",
        Clock: "fa fa-clock-o fa-lg ebsta-paddingright-ten",
        Trash: "fa fa-trash-o fa-lg",
        Salesforce: "fa fa-cloud fa-lg ebsta-paddingright-ten",
        PlusSign: "fa fa-plus-square-o fa-lg ebsta-paddingright-ten",
        StopSign: "fa fa-ban fa-lg",
        Pencil: "fa fa-pencil fa-lg ebsta-campaign-edit-pencil",
        PencilSquare: "fa fa-pencil-square fa-lg ebsta-campaign-edit-pencil",
        BackArrow: "fa fa-chevron-left fa-lg ebsta-paddingright-ten",
        Download: "fa fa-cloud-download fa-lg ebsta-paddingright-ten",
        Bullhorn: "fa fa-bullhorn fa-lg",
        Spinner: "fa fa-spinner fa-pulse fa-fw ebsta-fontawesome-spinner",
        MagnifyingGlass: "fa fa-search",
        Refresh: "fa fa-refresh",
        Calendar: "fa fa-calendar ebsta-paddingright-ten",
        CalendarScheduled: "fa fa-calendar-check-o ebsta-paddingright-ten",
        QuestionMark: "fa fa-question-circle fa-lg ebsta-paddingright-ten",
        ExclamationMarkCircle: "fa fa-exclamation-circle"
    }

    //List of classnames for specific elements (helps keeps consistency)
    var campaignClassNames = {
        Task: "campaignTask", // Task container name
        TaskOption: "campaignTaskOption",
        TaskHeader: "campaignTaskHeader",
        DecisionHeader: "campaignDecisionHeader",
        TaskContainer: "campaignTaskContainer",
        TaskCircle: "campaignTaskCircle",
        SideTask: "campaignSideTask", //
        SideTaskOption: "campaignSideTaskOption", // 
        DecisionContainer: "campaignDecisionContainer",
        Decision: "campaignDecision", // Decision container name
        TaskConnector: "taskConnector", // Connects a decision to a task
        SideTaskConnector: "sideTaskConnector", // Branching off to right task
        SideTaskConnectorOptionContainer: "sideTaskConnectorOptionContainer", // Branching off to right task
        TaskContent: "taskContent",
        DecisionContent: "decisionContent",
        EmailBody: "emailContent", // Main content of email
        EmailSubject: "emailSubject", // Subject of email
        SfTaskSubject: "sfTaskSubject",
        SfTaskBody: "sfTaskBody",
        SfTaskStatus: "sfTaskStatus",
        DecisionTime: "decisionTime",
        DecisionTimeGroup: "decisionTimeGroup",
        SearchResult: "campaignSearchResult",
        SelectedCircle: "circle-selected",
        WholeTaskContent: "campaignTaskWholeCont",
        TaskContentEmail: "sfEbstaCampaignTaskEmail",
        TaskContentNotify: "sfEbstaCampaignTaskAddToSalesforce",
        WholeTaskContentMain: "campaignTaskWholeContMain",
        TaskExecutionDay: "campaignTaskDay",
        TaskExecutionDayNumber: "campaignTaskDayNumber",
        TaskExecutionDayContainer: "campaignTaskDayContainer",
        MultiRecipientFirstNameColumn: "sfEbSendCampaignMultiRecipientsFirstNameCol", //First name input in MutliRecipients table
        MultiRecipientEmailColumn: "sfEbSendCampaignMultiRecipientsEmailCol", //Email input in MutliRecipients table
        MultiRecipientLastNameColumn: "sfEbSendCampaignMultiRecipientsLastNameCol", //Last name input in MutliRecipients table
        MultiRecipientCompanyColumn: "sfEbSendCampaignMultiRecipientsCompanyCol", //Company input in MutliRecipients table
        // dynamic csv data column.
        MultiRecipientCsvDataColumn: "sfEbSendCampaignMultiRecipientsCsvDataCol"
    }

    //Help calculate the total minutes decisions need to be set for
    var minuteCalculations = {
        minute: 1,
        hour: 60,
        day: 1440,
        week: 10080
    }

    var campaignMgrTaskTypeInterfaceToCallEnum = {
        SendingAnEmail: 1,
        AddingObjectToSalesforce: 2
    }

    /**
    * The types of decisions we offer
    */
    var campaignMgrDecisionTaskTypeInterfaceToCallEnum = {
        AlwaysTrue: 1,
        CheckIfNoOneHasReplied: 2,
        CheckIfTaskIsClosed: 3
    };

    //List of currently stored DOM elements, saves using selectors to go get them again and again
    var domElements = {
        DefaultCampaignMenuPopUp: null,
        CampaignComposeRecentCampaignLoader: null, //The loading element for recent campaigns pop up in compose window
        CampaignStepsDetailContainer: null,
        CampaignUsageContainer: null,
        CampaignSequenceLabel: null,
        CampaignUsageLabel: null,
        FirstNameInput: null,
        SecondNameInput: null,
        EmailInput: null,
        CampaignName: null,
        CampaignMetaTable: null,
        CampaignMetaTableBody: null,
        SidebarCampaigns: null,
        SidebarContacts: null,
        TaskDecisionPopUp: null,
        TaskPopUpSection: null,
        DecisionPopUpSection: null,
        NewCampaignButton: null,
        SaveCampaignButton: null,
        HomePageTableBody: null,
        CampaignEditIcon: null,
        CampaignTitleInput: null,
        SearchMagnify: null,
        SearchInput: null,
        PagingNext: null,
        PagingBack: null,
        PageNumbersContainer: null,
        SendLaterButton: null,
        EmailTrackingButton: null,
        ComplexAddButton: null,
        MultiRecipientModal: null, //Main Modal for selecting multiple recipientes
        MultiRecipientsTotalContactsSelectedHeaderText: null, // The text to display the total count of contacts selected in the multi recipient modal
        MultiRecipientsTextSpan: null, // The text to display the total count of contacts selected in the campaign manager modal
        MultiRecipientTableData: null, // The table holding the contacts selected in multi recipients for manually added contacts
        MultiRecipientCsvTableData: null, // The table holding the contacts selected in multi recipients for manually added contacts
        MultiRecipientSalesforceTableData: null, // The table holding the contacts selected in multi recipients for salesforce list views,
        MultiRecipientFirstNameInput: null, //recipient firstname input 
        MultiRecipientLastNameInput: null, //recipient lastname input 
        MultiRecipientEmailInput: null, //recipient email input 
        MultiRecipientCompanyInput: null, //recipient company input 
        MultiRecipientCsvInput: null, //recipient company input 
        MultiRecipientSalesforceObjectDropdown: null, //drop down for salesforce object selection 
        MultiRecipientSalesforceListviewDropdown: null, //drop down for salesforce list view selection 
        ComposeWindowButton: null, //The campaign manager button which appears in the compose window
        LoadingOverlay: null, //Last inserted overlay on the compose window
        SearchPage: {
            CampaignOwnerSelectDropDown: null
        },
        /**
         * Containing elements within the details page
         */
        DetailsPage: {
            CcTagsInput: null,
            BccTagsInput: null,
            SalesforceCampaignDropDown: null
        }
    };

    var currentPage = {
        SelectCampaign: false,
        RecentlyUsedCampaignDetail: false,
        CampaignDetail: false,
        EditEmailTemplate: false,
        ScheduleLater: false
    }

    var currentState = {
        button: null, // Last CampaignManager button within a compose window clicked
        selectedCampaign: null
    }
    var clickedFromMainModel = false;

    //Contact Variables
    var contactObject = new Contact(),
        contactObjectAll = new Array(), //Keeps track of all recipients in a compose window (To's onlys)
        multipleContactSelectedBool = false, // Has multi recipients been selected
        multipleContactArray = new Array(), // List of contacts chosen for multiple recipients
        maximumMultiRecipients = 250, // Global value of the maxium amount of recipients you can select
        multiRecipientRowSelectedCounter = 0, //Keeps track of the amount of contacts selected in the multi recipient popup
        multiRecipientComposeErrorCounter = 0, // Keeps track of the number of contacts we found in the compose window which don't have the full details
        ccComposeRecipients = new Array(),
        bccComposeRecipients = Array();

    var mergeFields = {
        EmailMergeFields: {
            1: "{!Contact.Email}",
            2: "{{{Recipient.Email}}}"
        },
        FullNameMergeFields: {
            1: "{!Contact.Name}",
            2: "{{{Recipient.Name}}}"
        },
        FirstNameMergeFields: {
            1: "{!Contact.FirstName}",
            2: "{{{Recipient.FirstName}}}"
        },
        SecondNameMergeFields: {
            1: "{!Contact.LastName}",
            2: "{{{Recipient.LastName}}}"
        },
        CompanyMergeFields: {
            1: "{!Contact.Company}",
            2: "{{{Recipient.Company}}}"
        }
    };

    var mainModelScheduler = null;
    var calendarElement = null;
    var lastDateSelected = null;
    var timeSpan = null;
    var hourElement = null;
    var minuteElement = null;
    var amPmElement = null;
    var chosenFieldElements = null;

    // if true, user has scheduled first email to be sent at a later time
    var firstEmailScheduledLater = false;
    var firstEmailScheduledLaterFinalDate = null;

    var loadingSelectedCampaign = false; //Informs us if a campaign has already been selected

    var clickedThroughToScheduleLaterFromFlowPage = false;

    var monthNames = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];

    /**
     * Creates the loading elements for recent campaigns with the compose window menu
     * @returns {} 
     */
    function CreateRecentCampaignsLoader() {
        if (domElements.CampaignComposeRecentCampaignLoader != null) {
            domElements.CampaignComposeRecentCampaignLoader.remove();
        }

        var divRecenylyUsedCampaignsLoader = CreateElement("div", "sfEbSendCampaignMgrStarterPopupItem sfEbComposeMenuOption", null, "sfEbSendCampaignMgrStarterPopupLoader");
        divRecenylyUsedCampaignsLoader.appendChild(CreateElement("div", icons.Bullhorn + " sfEbSendCampaignMgrStarterPopupItemBullhornIcon", null, null));
        divRecenylyUsedCampaignsLoader.appendChild(CreateElement("div", icons.Spinner, null, "sfEbSendCampaignMgrStarterPopupLoaderDiv"));
        Prepend(divRecenylyUsedCampaignsLoader,domElements.DefaultCampaignMenuPopUp);
        domElements.CampaignComposeRecentCampaignLoader = divRecenylyUsedCampaignsLoader;
    }

    /**
     * Updates the Multi Recipients text at the top to show the amount of contacts selected
     * @param {} numberOfContactsSelected 
     * @returns {} 
     */
    function UpdateMultiRecipientsTotalContactsSelectedHeaderText() {
        if (domElements.MultiRecipientsTotalContactsSelectedHeaderText != null && domElements.MultiRecipientsTotalContactsSelectedHeaderText.parentNode != null) {
            RemoveClass(domElements.MultiRecipientsTotalContactsSelectedHeaderText.parentNode, "ebsta-text-red");

            var contactsSelectText = multiRecipientRowSelectedCounter + "/" + maximumMultiRecipients + " Contacts Selected (" + maximumMultiRecipients + " maximum)";
            if (multiRecipientRowSelectedCounter === maximumMultiRecipients) {
                contactsSelectText = multiRecipientRowSelectedCounter + "/" + maximumMultiRecipients + " Maximum Contacts Selected";
            } else if (multiRecipientRowSelectedCounter > maximumMultiRecipients) {
                AddClass(domElements.MultiRecipientsTotalContactsSelectedHeaderText.parentNode, "ebsta-text-red");
                contactsSelectText = multiRecipientRowSelectedCounter + "/" + maximumMultiRecipients + " - Too Many Contacts Selected (" + maximumMultiRecipients + " maximum)";
            }
            domElements.MultiRecipientsTotalContactsSelectedHeaderText.innerText = contactsSelectText;
        }
    };

    /**
     * Creates an input fields for the Campaign Manager contact details
     * @param {} outerDiv 
     * @param {} elementId 
     * @returns {} 
     */
    function CreateAndPopulateContactFields(outerDiv, elementId, inputValue) {
        if (outerDiv) {
            AddClass(outerDiv, "sfEbSendCampaignDetailsRowSendingTo");

            //Find the element, if not found create it
            var contactInputElement = document.getElementById(elementId);
            if (contactInputElement == null) {
                contactInputElement = CreateElement("input", "sfEbStaCampaignMgrInput", "", elementId);
                outerDiv.appendChild(contactInputElement);
            } 

            if (contactInputElement != null && inputValue && inputValue.length > 0) {
                contactInputElement.value = inputValue;
            }
        }
    }

    /**
     * Shows the Contact container within the campaign details screen
     * @returns {} 
     */
    function ShowContactContainer () {
        var contactContainer = document.getElementById("sfEbSendCampaignDetailsContactContainer");
        if (contactContainer) {
            ShowElementBlock(contactContainer);
        }
    }

    /** 
     * Inserts a loader over the top of the contact info to inform the user we're looking for any 
     * matches for this email address in Salesforce 
     */
    function InsertContactLoadingOverLay(textOfmessage) {
        var contactContainer = document.getElementById('sfEbSendCampaignDetailsContactContainer');
        if (contactContainer) {
            var overlayMain = CreateElement("div", "loadingOverlay");
            var overlayContainer = CreateElement("div", "loadingOverlayContent");
            overlayContainer.appendChild(CreateElement('div', icons.Spinner));
            overlayContainer.appendChild(CreateElement("span", null, textOfmessage));
            overlayMain.appendChild(overlayContainer);
            contactContainer.appendChild(overlayMain);
            return overlayMain;
        }
    }

    /** Uses the saved Contact object to prepopulate the Contact fields for single campaigns */
    function InsertContactDetails() {
        if (contactObject != null) {
            var sendingToEmail = document.getElementById('sfEbSendCampaignDetailsSendingTo');
            if (sendingToEmail) {
                var rows = sendingToEmail.getElementsByClassName('sfEbSendCampaignDetailsRow');
                if (rows && rows.length === 4) {
                    CreateAndPopulateContactFields(rows[0], "sfEbSendCampaignDetailsEmailValue", contactObject.GetEmailAddress());
                    CreateAndPopulateContactFields(rows[1], "sfEbSendCampaignDetailsFirstNameValue", contactObject.GetFirstName());
                    CreateAndPopulateContactFields(rows[2], "sfEbSendCampaignDetailsLastNameValue", contactObject.GetLastName());
                    CreateAndPopulateContactFields(rows[3], "sfEbSendCampaignDetailsCompanyNameValue", contactObject.GetCompany());
                }
            }
        }
    }

    /**
     * Populates the contact object into the input fields within the campaign modal
     * @returns {} 
     */
    function PopulateSingleContactFromComposeToCampaign() {
        //If the campaign has already been chosen don't re-run a search just load the saved contact details
        if (!loadingSelectedCampaign && contactObject.GetEmailAddress() && contactObject.StopSalesforceLookup !== true) {
            var overlayDiv = InsertContactLoadingOverLay("Searching Salesforce for matching record");
            isSalesforceRecordListRequestActive = true;
            chrome.extension.sendMessage({ MessageType: "RunEmailSearch", EmailAddress: contactObject.GetEmailAddress() },
                function (response) {
                    EmailSearchResponseHandler(response);
                    InsertContactDetails();
                    if (overlayDiv) overlayDiv.remove();
                    isSalesforceRecordListRequestActive = false;
                    ShouldWeShowSendCampaignButton();
                });
        } else if (isSalesforceRecordListRequestActive) {
            let overlayDiv = InsertContactLoadingOverLay("Retrieving selected Salesforce records");

            RetryWithCallback(60, 1, function () {
                return !isSalesforceRecordListRequestActive;
            }, function () {
                PrePopulateContactsIntoCampaignModal();
                overlayDiv.remove();
                ShouldWeShowSendCampaignButton();
            });
        }
        else {
            InsertContactDetails();
        }
    }

    /** 
     * Centralises the place where we show the Send/Select buttons, this helps only show the buttons
     * once we have everything loaded correctly 
     */
    function ShouldWeShowSendCampaignButton() {
        if (currentPage.CampaignDetail === true) {
            if (isSalesforceRecordListRequestActive === false && isSelectedMailboxAuthenticated === true
                && lastGetCampaignDetailsRequestKey === null) {
                if (multipleContactSelectedBool === true || campaignComposeMode === false) {
                    ShowCampaignSendButton("Send");
                    HideCampaignSelectButton();
                } else {
                    if (campaignComposeMode === true) {
                        if (isFirstStepAnEmail === true) {
                            ShowCampaignSelectButton("Select");
                            HideCampaignSendButton();
                        } else {
                            ShowCampaignSendButton("Send");
                            HideCampaignSelectButton();
                        }
                    }
                }
            }
        }
    }

    /**
     * Handles a response from getting a result from searching for the email address the user is ending the campaign to, if
     * a match is found then we'll replace the Contact information we have with the Salesforce information
     * @param {any} response
     */
    function EmailSearchResponseHandler(response) {
        if (response) {
            var firstName = response["FirstName"];
            var lastName = response["LastName"];
            var accountName = response["Account"] ? response["Account"]["Name"] : null;

            if (accountName == null) {
                //Try and find in Company if it's a lead
                accountName = response["Company"];
            }

            var salesforceId = response["Id"];
            //If we have a first or last name then lets replace the Contact object so we display the Salesforc details
            if (firstName || lastName) {
                contactObject = new Contact(contactObject.GetEmailAddress(), firstName, lastName, accountName, null, salesforceId);
            }
        }
    }

    /**
    * Creates help icon for display on recipients tab in multi-recipients.
    * @returns {} 
    */
    function CreateHelpIconRecipientTab() {

        // create a div with the class name ebsta-recipientTab-helpIcon
        var helpIcon = document.createElement("div");
        helpIcon.className = 'ebsta-recipientTab-helpIcon';

        // create an anchor tag of the knowledge base article recipients
        var kbRecipientsAnchorTag = CreateElement("a", null, null, "EbstaAnchorAddRecipientHelp", {
            href: "https://ebstalimited.zendesk.com/hc/en-us/articles/115005084167",
            target: "_blank"
        });

        // append the anchor tag to the new div created above
        helpIcon.appendChild(kbRecipientsAnchorTag);

        // create the font awesome icon
        var faQuestionIcon = document.createElement("i");
        faQuestionIcon.className = 'fa fa-question-circle fa-2x';

        //append the i element into the anchor tag
        kbRecipientsAnchorTag.appendChild(faQuestionIcon);

        return helpIcon;
    }

    /**
     * select all recipients in the table of recipeients except the ones that are not valid
     * @returns {} 
     */
    function SelectAllRecipientInMultiRecipientTable(event) {
        var targetInput = event && event.target;
        if (targetInput) {
            var tableContainer = targetInput.closest('.sfEbSendCampaignMultiRecipientsTabSectionContent');
            if (tableContainer) {
                var table = tableContainer.getElementsByClassName('sfEbSendCampaignMultiRecipientsTableContent')[0];
                if (table) {
                    var inputs = table.getElementsByTagName('input');
                    if (inputs && inputs.length > 0) {
                        var i = 0;

                        for (i; i < inputs.length; i++) {
                            var inputChecked = inputs[i].checked;
                            if (inputs[i].closest('.ebstaInvalid') === null) {
                                inputs[i].checked = targetInput.checked;
                                //Only trigger the event if the state has been changed
                                if (inputChecked != targetInput.checked) {
                                    var evt = document.createEvent("HTMLEvents");
                                    evt.initEvent("change", false, true);
                                    inputs[i].dispatchEvent(evt);
                                }
                            }
                        }
                    }
                }
            }
        }
    }


    /**
     * when check box against a recipient is clicked, add selection class to the row
     * @param {} event 
     * @returns {} 
     */
    function SelectRecipientInMultiRecipientTable(event) {
        var targetInput = event && event.target;
        if (targetInput) {
            var row = targetInput.closest('.sfEbSendCampaignMultiRecipientsTableContentRow');
            if (row) {
                if (targetInput.checked) {
                    multiRecipientRowSelectedCounter++;
                    UpdateMultiRecipientsTotalContactsSelectedHeaderText();
                    row.classList.add('sfEbSendCampaignMultiRecipientsSelected');
                } else {
                    if (multiRecipientRowSelectedCounter > 0) {
                        multiRecipientRowSelectedCounter--;
                    }

                    UpdateMultiRecipientsTotalContactsSelectedHeaderText();
                    row.classList.remove('sfEbSendCampaignMultiRecipientsSelected');
                }
            }
        }
    }

    /**
     * Validate the name value to make sure it exists
     * @param {} name 
     * @returns {} 
     */
    function ValidateName(name) {
        if (name && name.length > 0) {
            return true;
        }
        return false;
    }


    /**
     * When the input loses focus this wil be run to help validate the input
     * @returns {} 
     */
    function MultiRecipientInputEvent() {
        var rowElement = this.RowElement;
        var checkBoxElement = this.CheckBoxElement;
        var invalidData = false;

        if (rowElement != null && checkBoxElement != null) {
            var getAllInputs = rowElement.getElementsByTagName("input");
            for (var i = 0; i < getAllInputs.length; i++) {
                var invalidColumn = false;
                if(HasClass(getAllInputs[i], campaignClassNames.MultiRecipientFirstNameColumn)) {
                    if (!ValidateName(getAllInputs[i].value)) {
                        invalidData = true;
                        invalidColumn = true;
                    };
                } else if(HasClass(getAllInputs[i], campaignClassNames.MultiRecipientEmailColumn)) {
                    if (!ValidateEmail(getAllInputs[i].value.trim())) {
                        invalidData = true;
                        invalidColumn = true;
                    };
                }

                //Update column flags for the input so we can highlight the issue ot the user
                if (invalidColumn) {
                    AddClass(getAllInputs[i].parentNode, "ebstaInvalid");
                } else {
                    RemoveClass(getAllInputs[i].parentNode, "ebstaInvalid");
                }
            }
        }

        if (invalidData) {          
            if (!HasClass(rowElement, "ebstaInvalid"))
            {
                if (HasClass(rowElement, "sfEbSendCampaignMultiRecipientsSelected")) {
                    multiRecipientRowSelectedCounter--;
                }

                AddClass(rowElement, "ebstaInvalid");
                RemoveClass(rowElement, "sfEbSendCampaignMultiRecipientsSelected");
                UpdateCheckedAttributeOnCheckBox(checkBoxElement, false);                
            }
        } else {
            if (HasClass(rowElement, "ebstaInvalid")) {
                RemoveClass(rowElement, "ebstaInvalid");
                AddClass(rowElement, "sfEbSendCampaignMultiRecipientsSelected");
                UpdateCheckedAttributeOnCheckBox(checkBoxElement, true);
                multiRecipientRowSelectedCounter++;
            }
        }

        TriggerEventOnElement("change", checkBoxElement);
        UpdateMultiRecipientsTotalContactsSelectedHeaderText();
    }

    /**
     * Takes a column no and returns the appropriate class name for the inputs in multi recpients
     * @param {} columnNo 
     * @returns {} 
     */
    function GetMultiRecipientColumnName(columnNo) {
        switch (columnNo) {
            //First name
            case 0:
                return campaignClassNames.MultiRecipientFirstNameColumn;
            //Last Name
            case 1:
                return campaignClassNames.MultiRecipientLastNameColumn;
             //Email address
            case 2:
                return campaignClassNames.MultiRecipientEmailColumn;
            //Company
            case 3:
                return campaignClassNames.MultiRecipientCompanyColumn;
            default:
                break;
        }

        return null;
    }

    /**
     * Takes the row container which has all inputs and loops through them to find the contact details
     * @param {} rowContainer 
     * @returns {} 
     */
    function CreateContactFromRow(contactRowElement) {
        var getAllInputs = contactRowElement.getElementsByTagName("input");

        var emailAddress = null,
            firstName = null,
            lastName = null,
            company = null,
            csvFields = {},
            salesforceid = contactRowElement.getAttribute("salesforceid");

        //Loop through inputs to get values for the contact
        for (var j = 0; j < getAllInputs.length; j++) {

            var inputField = getAllInputs[j];

            var value = inputField.value;
            if (HasClass(inputField, campaignClassNames.MultiRecipientFirstNameColumn)) {
                firstName = value ? value : null;
            } else if (HasClass(inputField, campaignClassNames.MultiRecipientLastNameColumn)) {
                lastName = value ? value : null;
            } else if (HasClass(inputField, campaignClassNames.MultiRecipientEmailColumn)) {
                emailAddress = value ? value.trim() : null;
            } else if (HasClass(inputField, campaignClassNames.MultiRecipientCompanyColumn)) {
                company = value ? value : null;
            }

            if (HasClass(inputField, campaignClassNames.MultiRecipientCsvDataColumn)) {
                // get the field name via attribute on the element.
                var fieldName = '' + inputField.getAttribute('fieldname');

                // lowercase to field names as these are becoming dictionary keys.
                fieldName = fieldName.toLowerCase();

                // push the field value to the dictionary.
                csvFields[fieldName] = '' + value;
            }
        }

        return new Contact(emailAddress, firstName, lastName, company, csvFields, salesforceid);
    }

    /**
     * Used to build the row element for the csv recipient table
     * @param {} data 
     * @param {} validate 
     * @returns {} 
     */
    function CreateRecipientRow(data, validate) {

        if (data && data.length > 0) {
            var divContainer = CreateElement("tr", "sfEbSendCampaignMultiRecipientsTableContentRow");

            //create the checkbox element.
            var checkBox = CreateCustomCheckBoxWithListenerAttached("", "change", SelectRecipientInMultiRecipientTable, false, "", "ebsta-campaign-recipient-checkbox");

            // add value column container
            var tableCellsToAppend = [];
            var validRow = true;
            var i = 0;
            for (i; i < data.length; i++) {
                var validColumn = true;
                // validation
                switch (i) {
                    case 0:
                        // first name
                        if (validate && !ValidateName(data[i])) {
                            validRow = false;
                            validColumn = false;
                        }
                        break;
                    case 2:
                        // email address
                        if (validate &&
                            (!data[i] || !ValidateEmail(data[i].trim()))) {
                            validRow = false;
                            validColumn = false;
                        }
                        break;
                }

                //Ignore any data in columns past the 4th column and append the data
                if (i < 4) {
                    var newCell = CreateElement("td", "minimum-width sfEbDataCell");
                    var cellContainer = CreateElement("div");
                    var inputElement = CreateElement("input", GetMultiRecipientColumnName(i) + ' ' + campaignClassNames.MultiRecipientCsvDataColumn, null, null,
                        {
                            value: data[i] ? data[i].trim() : "",
                            spellcheck: false,
                            fieldname: csvTableHeaders[i]
                        });

                    //Special actions for only the first name and email columns
                    if (i === 0 || i === 2) {
                        inputElement.placeholder = "Enter ";
                        if (i == 0) inputElement.placeholder += "First Name";
                        else if (i == 2) inputElement.placeholder += "Email Address";
                        inputElement.addEventListener("input", MultiRecipientInputEvent);
                        inputElement.CheckBoxElement = checkBox;
                        inputElement.RowElement = divContainer;

                        //Exclamation mark to highlight field isn't currently valid
                        var exclamationMark = CreateElement("i", icons.ExclamationMarkCircle + " ebsta-text-red");
                        cellContainer.appendChild(exclamationMark);
                    }

                    cellContainer.appendChild(inputElement);
                    newCell.appendChild(cellContainer);

                    if (!validColumn) {
                        AddClass(newCell, "ebstaInvalid");
                    }

                    tableCellsToAppend.push(newCell);
                } else {

                    // we should be rendering extra fields in the csv.
                    if (enableCsvMergeFields) {
                        // now render bonus cells in the csv file.
                        var inputContainer = CreateElement("td", "minimum-width sfEbDataCell");
                        var inputElement = CreateElement("input", campaignClassNames.MultiRecipientCsvDataColumn, null, null,
                            {
                                value: data[i] ? data[i].trim() : "",
                                spellcheck: false,
                                fieldname: csvTableHeaders[i]
                            });

                        inputContainer.appendChild(inputElement);
                        tableCellsToAppend.push(inputContainer);                        
                    } else {
                        // exit the loop, no point running through the rest of the columns.
                        break;
                    }
                }
            }
            
            // add check box for the recipient
            var checkBoxContainer = CreateElement("td", "sfEbSendCampaignMultiRecipientsCheckBoxContainer");                
                       
            if (!validRow) {
                AddClass(divContainer, "ebstaInvalid");
            } else {
                multiRecipientRowSelectedCounter++;
                AddClass(divContainer, "sfEbSendCampaignMultiRecipientsSelected");
                UpdateCheckedAttributeOnCheckBox(checkBox, true);
            }

            checkBoxContainer.appendChild(checkBox);          

            divContainer.appendChild(checkBoxContainer);
            
            if (enableCampaignDupeCheck) {
                const campaignDupeAlertCell = CreateElement("td", "sfEbSendCampaignMultiRecipientsTableDupeAlert");
                const campaignDupeAlertATag = CreateElement("a");
                campaignDupeAlertCell.appendChild(campaignDupeAlertATag);
                divContainer.appendChild(campaignDupeAlertCell);
            }          

            tableCellsToAppend.forEach(tableCellToAppend => {
                divContainer.appendChild(tableCellToAppend)
            });

            UpdateMultiRecipientsTotalContactsSelectedHeaderText();

            return divContainer;
        }
    }

    /**
     * get or create object for reading csv file
     * @returns {} 
     */
    function GetOrCreateCsvReaderObject() {
        if (csvManager != null) {
            return csvManager;
        } else {
            csvManager = CreateCsvManagerObject(enableCsvMergeFields);
            return csvManager;
        }
    }

    /** Gets all the email address values from the input elements for the email column in CSV tab */
    function getAllEmailAddressesInCsvTab(recipientListContainer) {
        const allEmailAddressInputs =
            recipientListContainer.querySelectorAll('input[class~="sfEbSendCampaignMultiRecipientsEmailCol"]');
        return Array.from(allEmailAddressInputs).map(x => x.value).filter(x => x);
    }

    /**
    * read from csv and populate the right multi recipient table
    * @returns {} 
    */
    function UploadCSVIntoMultiRecipientTable(files, targetElement, rowProducer, callBack) {
        if (files && files.length > 0) {
            var csvReader = GetOrCreateCsvReaderObject();
            csvReader.InsertCsvIntoTable(files[0], targetElement, rowProducer, callBack);
        }
    }

    /**
     * add single recipeint into table
     * @param {} button 
     * @param {} table 
     * @returns {string|null} - Returns the email address if valid, null if not
     */
    function AddRecipientIntoMultiRecipientTable(button, table, rowProducer) {
        var buttonParent = button.parentElement;
        var processSelection = true;

        // valid first name
        var firstName = "";
        var firstnameInput = buttonParent.getElementsByClassName("sfEbSendCampaignMultiRecipientsFirstNameInput")[0];
        if (firstnameInput) {
            firstName = firstnameInput.value;
        }

        if (!ValidateName(firstName)) {
            processSelection = false;
            AddClass(firstnameInput, "ebError");
        } else {
            RemoveClass(firstnameInput, "ebError");
        }

        // valid last name
        var lastName = "";
        var lastNameInput = buttonParent.getElementsByClassName("sfEbSendCampaignMultiRecipientsLastNameInput")[0];
        if (lastNameInput) {
            lastName = lastNameInput.value;
        }

        // valid email
        var email = "";
        var emailInput = buttonParent.getElementsByClassName("sfEbSendCampaignMultiRecipientsEmailInput")[0];
        if (emailInput) {
            email = emailInput.value;
        }

        if (!ValidateEmail(email.trim())) {
            processSelection = false;
            AddClass(emailInput, "ebError");
        } else {
            RemoveClass(emailInput, "ebError");
        }

        // valid company
        var company = "";
        var companyInput = buttonParent.getElementsByClassName("sfEbSendCampaignMultiRecipientsCompanyInput")[0];
        if (companyInput) {
            company = companyInput.value;
        }

        if (processSelection) {

            const noRecipientsMessage = table.querySelector(".sfEbSendCampaignMultiRecipientsTableNoRecipientsRow");
            if (noRecipientsMessage) {
                noRecipientsMessage.remove();
            }
            
            var divContainer = rowProducer([firstName, lastName, email, company], false);

            table.appendChild(divContainer);

            //clear the text boxes.
            $("#sfEbSendCampaignMultiRecipientsRecipientsTabSection .sfEbStaCampaignMgrInput").val("");

            return email;
        }

        return null;
    }

    /**
     * Clears out the specified div and populates it with the given table data
     * @param {} tableContentDiv 
     * @param {} tableData 
     * @returns {} 
     */
    function ClearAndPopulateMultiRecipientTableData(tableContentDiv, tableData) {

        tableContentDiv.innerHTML = "";
        if (tableData && tableData.length > 0) {
            // render data into table
            var i = 0;
            for (i; i < tableData.length; i++) {
                tableContentDiv.appendChild(CreateRecipientRow([tableData[i].GetFirstName(), tableData[i].GetLastName(),tableData[i].GetEmailAddress(), tableData[i].GetCompany()], true));
            }
        } else {
            const row = CreateElement("tr", "sfEbSendCampaignMultiRecipientsTableNoRecipientsRow");
            const cell = CreateElement("td", null, "No recipients added", null, { colspan: 100 });
            row.appendChild(cell);
            tableContentDiv.appendChild(row);
        }
    }
    /**

    * creates dom element to be used as the table for holding recipients in the multi recipient section
    * @returns {} 
    */
    function CreateMultiRecipientsTableElement(tab, inputAction, tableData) {

        var tableContainer = CreateElement("div", "sfEbSendCampaignMultiRecipientsTableContainer");

        // content section, where recipients will be displayed
        var contentContainer = CreateElement("table", "sfEbSendCampaignMultiRecipientsTabSectionContent");
        tableContainer.appendChild(contentContainer);

        // table headers
        var tableHeader = CreateElement("thead", "sfEbSendCampaignMultiRecipientsTableHeaderContainer");
        var tableHeaderRow = CreateElement("tr");
        tableHeader.appendChild(tableHeaderRow);

        // check box for recipient selection
        var checkBoxContainer = CreateElement("th", "sfEbSendCampaignMultiRecipientsTableHeader sfEbSendCampaignMultiRecipientsCheckBoxContainer");
        var selectAllCheckbox = CreateCustomCheckBoxWithListenerAttached("", "change", SelectAllRecipientInMultiRecipientTable, true, "EbstaChkCampaignRecipientSelectAll");
        checkBoxContainer.appendChild(selectAllCheckbox);
        tableHeaderRow.appendChild(checkBoxContainer);

        if (enableCampaignDupeCheck) {
            tableHeaderRow.appendChild(CreateElement("th", "sfEbSendCampaignMultiRecipientsTableHeader", null, null, {dupecheck:null}));
        }

        tableHeaderRow.appendChild(CreateElement("th", "sfEbSendCampaignMultiRecipientsTableHeader", "First Name *"));
        tableHeaderRow.appendChild(CreateElement("th", "sfEbSendCampaignMultiRecipientsTableHeader", "Last Name"));
        tableHeaderRow.appendChild(CreateElement("th", "sfEbSendCampaignMultiRecipientsTableHeader email", "Email *"));
        // The colespan attribute is to give the effect that the sticker header goes right across the screen
        tableHeaderRow.appendChild(CreateElement("th", "sfEbSendCampaignMultiRecipientsTableHeader", "Company", null));

        contentContainer.appendChild(tableHeader);

        // table content
        var tableContent = CreateElement("tbody", "sfEbSendCampaignMultiRecipientsTableContent");
        ClearAndPopulateMultiRecipientTableData(tableContent, tableData);

        if (tab.csvSection) {
            domElements.MultiRecipientCsvTableData = tableContent;
            if (inputAction) {

                // add listener to input to load the csv data into the table content
                inputAction.addEventListener("click", function() {
                    this.value = null;
                });
                inputAction.addEventListener("change", function () {
                    tableContent.innerHTML = "";
                    multiRecipientRowSelectedCounter = domElements.MultiRecipientModal.getElementsByClassName("sfEbSendCampaignMultiRecipientsSelected").length;
                    
                    let files = this.files;                   
                    
                    UploadCSVIntoMultiRecipientTable(files, tableContent, CreateRecipientRow, function () {
                        const allEmailAddresses = getAllEmailAddressesInCsvTab(tableContent);

                        if (enableCampaignDupeCheck) {

                            const loadingMessageContainer = createDuplicateLoaderElement(true);
                            tableContainer
                                .closest(".sfEbSendCampaignMultiRecipientsTabSectionContainer")
                                .appendChild(loadingMessageContainer);
                            
                            checkIfRecipientsAreLiveOnACampaign(allEmailAddresses, [tableContent], duplicateWarningMessageType.MultipleContact, function () {
                                loadingMessageContainer.remove();
                            });
                        }
                    });

                    if (files != null) {
                        if (files.length > 0) {
                            var filename = files[0].name;

                            if (filename.length > 0) {
                                $(".fileupload-text").text(filename);
                            } else {
                                $(".fileupload-text").text("Browse");
                            }
                        }
                    }
                });
            }
        } else if (tab.tableDataSecion) {
            domElements.MultiRecipientTableData = tableContent;

            if (inputAction) {
                // on button press, load user in
                inputAction.addEventListener("click", function (event) {
                    var button = event && event.target;
                    const emailAddressOfContact = AddRecipientIntoMultiRecipientTable(button, tableContent, CreateRecipientRow);
                    UpdateMultiRecipientsTotalContactsSelectedHeaderText();
                        
                    if (enableCampaignDupeCheck && emailAddressOfContact != null) {

                        const loadingMessageContainer = createDuplicateLoaderElement(false);
                        tableContainer
                            .closest(".sfEbSendCampaignMultiRecipientsTabSectionContainer")
                            .appendChild(loadingMessageContainer);
                                
                        checkIfRecipientsAreLiveOnACampaign([emailAddressOfContact], [tableContent], duplicateWarningMessageType.SingularContact, function () {
                            loadingMessageContainer.remove();
                        });
                    }                    
                });
            }
        } else if (tab.salesforceSection) {
            domElements.MultiRecipientSalesforceTableData = tableContent;
        }

        contentContainer.appendChild(tableContent);

        return tableContainer;
    }

    /**
    * Creates the recipients tab section
    * @param {} sectionId 
    * @returns {} 
    */
    function CreateRecipientsSection(sectionId) {
        var recipientsSection = CreateElement("div", "sfEbSendCampaignMultiRecipientsTabSectionContainer");

        // header section where user will upload csv
        var headerContainer = CreateElement("div", "sfEbSendCampaignMultiRecipientsTabSectionHeader");

        // first name input
        var firstNameContainer = CreateElement("div");
        firstNameContainer.appendChild(CreateElement("label", null, "First Name *"));
        var firstNameInputContainer = CreateElement("div");
        domElements.MultiRecipientFirstNameInput = CreateElement("input", "sfEbStaCampaignMgrInput sfEbSendCampaignMultiRecipientsFirstNameInput", null, null, { type: "text" });
        firstNameInputContainer.appendChild(domElements.MultiRecipientFirstNameInput);
        firstNameContainer.appendChild(firstNameInputContainer);

        // last name input
        var lastNameContainer = CreateElement("div");
        lastNameContainer.appendChild(CreateElement("label", null, "Last Name"));
        var lastNameInputContainer = CreateElement("div");
        domElements.MultiRecipientLastNameInput = CreateElement("input", "sfEbStaCampaignMgrInput sfEbSendCampaignMultiRecipientsLastNameInput", null, null, { type: "text" });
        lastNameInputContainer.appendChild(domElements.MultiRecipientLastNameInput);
        lastNameContainer.appendChild(lastNameInputContainer);

        // email input
        var emailContainer = CreateElement("div");
        emailContainer.appendChild(CreateElement("label", null, "Email *"));
        var emailInputContainer = CreateElement("div");
        domElements.MultiRecipientEmailInput = CreateElement("input", "sfEbStaCampaignMgrInput sfEbSendCampaignMultiRecipientsEmailInput", null, null, { type: "text" });
        emailInputContainer.appendChild(domElements.MultiRecipientEmailInput);
        emailContainer.appendChild(emailInputContainer);


        // company input
        var companyContainer = CreateElement("div");
        companyContainer.appendChild(CreateElement("label", null, "Company"));
        var companyInputContainer = CreateElement("div");
        domElements.MultiRecipientCompanyInput = CreateElement("input", "sfEbStaCampaignMgrInput sfEbSendCampaignMultiRecipientsCompanyInput", null, null, { type: "text" });
        companyInputContainer.appendChild(domElements.MultiRecipientCompanyInput);
        companyContainer.appendChild(companyInputContainer);

        var addRecipientButton = CreateElement("div", "sfEbSendCampaignMgrButton", "Add", "EbstaBtCampaignsAddRecipient");

        // help icon for recipients tab
        var helpIcon = CreateHelpIconRecipientTab();

        headerContainer.appendChild(firstNameContainer);
        headerContainer.appendChild(lastNameContainer);
        headerContainer.appendChild(emailContainer);
        headerContainer.appendChild(companyContainer);
        headerContainer.appendChild(addRecipientButton);

        // append the helpIcon div to the headerContainer
        headerContainer.appendChild(helpIcon);
        recipientsSection.appendChild(headerContainer);

        // create table for multi recipients
        var contentContainer = CreateMultiRecipientsTableElement({ tableDataSecion: true}, addRecipientButton, contactObjectAll);

        recipientsSection.appendChild(contentContainer);

        return recipientsSection;
    }

    /**
     * Creates help icon for display on salesforce tab in multi-recipients.
     * @returns {} 
     */
    function CreateHelpIconSalesforceTab () {
        // create a div with the class name ebsta-helpIcon
        var helpIcon = document.createElement("div");
        helpIcon.className = 'ebsta-salesforceTab-helpIcon';

        var kbSalesforceAnchorTag = CreateElement("a", null, null, "EbstaAnchorSalesforceRecipientHelp", {
            href: "https://ebstalimited.zendesk.com/hc/en-us/articles/115005084167",
            target: "_blank"
        });

        // append the anchor tag to the new div created above
        helpIcon.appendChild(kbSalesforceAnchorTag);

        // create the font awesome icon
        var faQuestionIcon = document.createElement("i");
        faQuestionIcon.className = 'fa fa-question-circle fa-2x';

        //append the i element into the anchor tag
        kbSalesforceAnchorTag.appendChild(faQuestionIcon);

        return helpIcon;
    }

    /**
     * make call to get salesforce objects user has access to for multi recipient list view
     * @returns {} 
     */
    function GetCampaignSalesforceListViewObjects() {

        var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceObjectLoader');
        if (loaderDiv) {
            loaderDiv.innerHTML = '';
            loaderDiv.appendChild(CreateSpinnerLoaderDiv());
        }

        var request = {
            MessageType: "GetCampaignSalesforceListViewObjects"
        };
                
        SendMessageToBackGround(request);
    }

    /**
     * creates section for selecting salesforce list view contactd and leads into recipient table
     * @returns {} 
     */
    function CreateMultiRecipientSalesforceSection() {
        var recipientsSection = CreateElement("div", "sfEbSendCampaignMultiRecipientsTabSectionContainer");

        // header section where user will choose salesforce object and list view
        var headerContainer = CreateElement("div");

        // selection for salesforce object type and list view 
        var selectionContainer1 = CreateElement("div", "sfEbSendCampaignMultiRecipientsTabSalesforceSelectionContainer");                

        // salesforce object type label
        var labelContainer = CreateElement("div");
        labelContainer.appendChild(CreateElement("label", null, "Select Object"));

        selectionContainer1.appendChild(labelContainer);

        // salesforce object type selection
        var selection1 = CreateElement("div");
        domElements.MultiRecipientSalesforceObjectDropdown = EbstaCustomDropdown.CreateCustomDropDown('sfEbSendCampaignMultiRecipientsSalesforceObjectType');
        selection1.appendChild(domElements.MultiRecipientSalesforceObjectDropdown);

        selectionContainer1.appendChild(selection1);

        const loadingGifContainer =
            CreateElement("div", null, null, 'sfEbSendCampaignMultiRecipientsTabSalesforceObjectLoader');
        loadingGifContainer.appendChild(CreateSpinnerLoaderDiv());

        // loader and error handling 
        selectionContainer1.appendChild(loadingGifContainer);

        //create the help icon.
        var helpIcon = CreateHelpIconSalesforceTab();

        // append the helpicon div to the selectionContainer
        selectionContainer1.appendChild(helpIcon);

        // selection for salesforce list views by object
        var selectionContainer2 = CreateElement("div", "sfEbSendCampaignMultiRecipientsTabSalesforceSelectionContainer");

        // salesforce object type label
        var labelContainer2 = CreateElement("div");
        labelContainer2.appendChild(CreateElement("label", null, "Select View"));

        selectionContainer2.appendChild(labelContainer2);


        // salesforce object type selection
        var selection2 = CreateElement("div");
        domElements.MultiRecipientSalesforceListviewDropdown = EbstaCustomDropdown.CreateCustomDropDown('sfEbSendCampaignMultiRecipientsSalesforceListView');
        domElements.MultiRecipientSalesforceListviewDropdown.classList.add('ebsta-disable');
        selection2.appendChild(domElements.MultiRecipientSalesforceListviewDropdown);

        selectionContainer2.appendChild(selection2);

        // loader and error handling 
        selectionContainer2.appendChild(CreateElement("div", null, null, 'sfEbSendCampaignMultiRecipientsTabSalesforceListviewLoader'));

        headerContainer.appendChild(selectionContainer1);
        headerContainer.appendChild(selectionContainer2);

        recipientsSection.appendChild(headerContainer);

        // create table for multi recipients
        var contentContainer = CreateMultiRecipientsTableElement({ salesforceSection : true});

        recipientsSection.appendChild(contentContainer);

        return recipientsSection;
    }

    /**
    * Creates help icon for display on csv tab in multi-recipients.
    * @returns {} 
    */
    function CreateHelpIconCsvTab() {
            
        // create a div with the class name ebsta-helpIcon
        var helpIcon = document.createElement("div");
        helpIcon.className = 'ebsta-csvTab-helpIcon';

        // create an anchor tag of the knowledge base article recipients
        var kbCsvHelpIconAnchorTag = CreateElement("a", null, null, "EbstaAnchorCampaignCsvHelp", {
            href: "https://ebstalimited.zendesk.com/hc/en-us/articles/115005084167#csv",
            target: "_blank"
        });

        // append the anchor tag to the new div created above
        helpIcon.appendChild(kbCsvHelpIconAnchorTag);

        // create the font awesome icon
        var faQuestionIcon = document.createElement("i");
        faQuestionIcon.className = 'fa fa-question-circle fa-2x';

        //append the i element into the anchor tag
        kbCsvHelpIconAnchorTag.appendChild(faQuestionIcon);

        return helpIcon;
    }

    /**
     * Creates the CSV tab section
     * @param {} sectionId 
     * @returns {} 
     */
    function CreateCsvSection(sectionId) {
        var recipientsSection = CreateElement("div", "sfEbSendCampaignMultiRecipientsTabSectionContainer");

        // header section where user will upload csv
        var headerContainer = CreateElement("div", "sfEbSendCampaignMultiRecipientsTabSectionHeader");

        var labelContainer = CreateElement("label", "ebsta-fileupload", "", "EbstaFileAddRecipientsCsv");

        var fileInput = CreateElement("input", "ebsta-fileupload-input", null, "sfEbSendCampaignMultiRecipientsCsvInput", { type: "file", name: "file", accept: ".csv", pattern: "^.+\.(csv)$" });
        var inputSpan = CreateElement("span", "fileupload-text", "Choose File");
        var inputSpan2 = CreateElement("span", "input-combo-icon-left");
        var iconSpan = CreateElement("i", "fa fa-file-text-o");
        iconSpan.setAttribute("aria-hidden", true);

        inputSpan2.appendChild(iconSpan);

        labelContainer.appendChild(fileInput);
        labelContainer.appendChild(inputSpan);
        labelContainer.appendChild(inputSpan2);

        headerContainer.appendChild(labelContainer);

        domElements.MultiRecipientCsvInput = fileInput;
        recipientsSection.appendChild(headerContainer);

        // create a div with the classname ebsta-kb-csv
        var kbCsv = document.createElement("div");
        kbCsv.className = 'ebsta-kb-csv';

        // create an anchor tag of the knowledge base CSV Format Requirements
        var kbCsvAnchorTag = CreateElement("a", "", "CSV Format Requirements", "EbstaAnchorCampaignCsvRequirements", {
            href: "https://ebstalimited.zendesk.com/hc/en-us/articles/115005084167#csv",
            target: "_blank"
        });

        // append the anchor tag to the new div created above
        kbCsv.appendChild(kbCsvAnchorTag);

        // append the new div to the header container
        headerContainer.appendChild(kbCsv);

        // create help icon in the csv tab 
        var helpIcon = CreateHelpIconCsvTab();

        // append the helpicon div to the selectionContainer
        headerContainer.appendChild(helpIcon);

        // create table for multi recipients
        var contentContainer = CreateMultiRecipientsTableElement({ csvSection: true }, domElements.MultiRecipientCsvInput);

        recipientsSection.appendChild(contentContainer);

        return recipientsSection;
    }

    /**
    * Creates each tab section within the Multi Recipients modal
    * @returns {} 
    */
    function CreateMultiRecipientTabsSection() {
        var recipientsSectionId = "sfEbSendCampaignMultiRecipientsRecipientsTabSection";
        var recipientsTabSection = CreateRecipientsSection(recipientsSectionId);
        var recipientsTabObject = CreateTabObject("Recipients", recipientsSectionId, recipientsTabSection);

        var salesforceSectionId = "sfEbSendCampaignMultiRecipientsSalesforceTabSection";
        var salesforceTabSection = CreateMultiRecipientSalesforceSection(salesforceSectionId);
        var salesforceTabObject = CreateTabObject("Salesforce View", salesforceSectionId, salesforceTabSection);

        var csvSectionId = "sfEbSendCampaignMultiRecipientsCsvTabSection";
        var csvTabSection = CreateCsvSection(csvSectionId);
        var csvTabObject = CreateTabObject("CSV", csvSectionId, csvTabSection);

        return CreateTabs("sfEbSendCampaignMultiRecipientTabs", [recipientsTabObject, salesforceTabObject, csvTabObject]);
    }

    /**
     * Gets all email addresses from selected recipients from all tabs in the multi recipient window 
     * @returns {Array} - An array of contacts from all selected rows
     */
    function getAllSelectedMultiRecipientEmailAddresses() {
        const emailAddresses = [];

        const multiRecipientModal = document.getElementById("sfEbSendCampaignMultiRecipientModal");
        if (multiRecipientModal) {
            const contactRecipientRowElements = multiRecipientModal.getElementsByClassName("sfEbSendCampaignMultiRecipientsTableContentRow sfEbSendCampaignMultiRecipientsSelected");
            if (contactRecipientRowElements && contactRecipientRowElements.length) {
                for (i = 0; i < contactRecipientRowElements.length; i++) {  
                    const newContactObject = CreateContactFromRow(contactRecipientRowElements[i]);
                    if (newContactObject) {
                        emailAddresses.push(newContactObject.GetEmailAddress());
                    }
                }
            }
        }

        return emailAddresses;
    }

    /**
     * Shows an alert dialog to state the user needs to select at least 1 contact
     */
    function ShowNoSelectedContactsWarning() {
        window.GLOBAL_EBSTA_MODAL.SetText("Please select at least 1 contact.");
        window.GLOBAL_EBSTA_MODAL.SetOkButtonMode();
        window.GLOBAL_EBSTA_MODAL.ShowModal();
        window.GLOBAL_EBSTA_MODAL.ShowDialog();
    }

    /**
     * Runs a check against all the selected recipients in the multi-recipient window to check if they're currently live on a campaign
     */
    function DupeCheckAllSelectedMultiRecipients() {
        const allSelectedEmailAddresses = getAllSelectedMultiRecipientEmailAddresses();
         
        // CHeck we actually have selected recipients before running a request
        if (Array.isArray(allSelectedEmailAddresses) && allSelectedEmailAddresses.length == 0) {
            ShowNoSelectedContactsWarning();
            return;
        }

        const allTableBodys = document.getElementsByClassName("sfEbSendCampaignMultiRecipientsTableContent");
        const multiRecipientModal = document.getElementById("sfEbSendCampaignMultiRecipientModal");
        const loadingMessageContainer = createDuplicateLoaderElement(true);
         
        multiRecipientModal.appendChild(loadingMessageContainer);

        checkIfRecipientsAreLiveOnACampaign(allSelectedEmailAddresses, allTableBodys,
            duplicateWarningMessageType.MultipleContact, () => {
                loadingMessageContainer.remove();                
                const countOfSelectedContacts = getAllSelectedMultiRecipientEmailAddresses().length;
                
                // If we've deselected all multi-recipients then ensure we switch back to single-recipient mode
                // so the user cannot send the campaign to the duplicate contacts
                if (countOfSelectedContacts === 0) {
                    HideMultiRecipientsText();
                    contactObject.ResetContact();
                    multipleContactArray = new Array();
                    UpdateMultipleContactSelectedBool(false);
                    InsertContactDetails();
                    ShowContactContainer();
                }
            }, true);
    }

    /**
     * The click event logic for the Confirm button in the multi-recipient modal
     * @param {HTMLElement} confirmButton - The confirm button element
     */
    function MultiRecipientConfirmButtonClickEvent(confirmButton) {
        if (enableCampaignDupeCheck) {
            DupeCheckAllSelectedMultiRecipients();
        } else {
            lThisCampaignManagerInstance.SetMultipleRecipientsSelected();
        }
        confirmButton.blur();
    }

    /**
     * Attachs a click listener onto the Confirm button situated in the mutli recipient modal
     * @param {} confirmButton - The confirm button element
     */
    function AttachClickListenerOnConfirmMultiRecipientsButtons(confirmButton) {
        confirmButton.addEventListener("click", function () {
            MultiRecipientConfirmButtonClickEvent(this);
        });
    }

    /**
    * Creates the multi recipient modal where the user can select recipients
    * @returns {} 
    */
    function CreateMultiRecipientModal() {
        var mainContainer = CreateElement("div", "sfEbSendCampaignMgrModal", null, "sfEbSendCampaignMultiRecipientModal");

        //Title
        var titleHeader = CreateElement("header");
        titleHeader.appendChild(CreateElement("h3", null, "Select Recipients"));
        var multiRecipientsHeaderText = CreateElement("span", "sfEbSendCampaignMultiRecipientModalHeader");
        domElements.MultiRecipientsTotalContactsSelectedHeaderText = CreateElement("h3", null, "0/" + maximumMultiRecipients + " Contacts Selected");
        multiRecipientsHeaderText.appendChild(domElements.MultiRecipientsTotalContactsSelectedHeaderText);
        titleHeader.appendChild(multiRecipientsHeaderText);

        var closeIcon = CreateElement("span", "sftitle-close");
        closeIcon.addEventListener("click", function () {
            HideElement(mainContainer);
        });
        insertBefore(closeIcon, multiRecipientsHeaderText);

        mainContainer.appendChild(titleHeader);

        //Main Content Section
        var contentSection = CreateElement("section", "sfEbSendCampaignMultiRecipientMainSection");
        mainContainer.appendChild(contentSection);

        //Tabs
        var tabsSection = CreateMultiRecipientTabsSection();
        mainContainer.appendChild(tabsSection);
        new CBPFWTabs(tabsSection);

        //Create Footer for buttons
        var footerContainer = CreateElement("div", null, null, "sfEbSendCampaignMultiRecipientMainFooter");
        var confirmButton = CreateElement("div", "sfEbSendCampaignMgrButton", "Confirm", "EbstaBtCampaignsConfirmRecipients");
        AttachClickListenerOnConfirmMultiRecipientsButtons(confirmButton);
        footerContainer.appendChild(confirmButton);

        var cancelButton = CreateElement("div", "sfEbSendCampaignMgrButton", "Cancel", "sfEbSendCampaignMultiRecipientCancel");
        if (multipleContactSelectedBool) {
            cancelButton.style.display = "none";
        }
        footerContainer.appendChild(cancelButton);
        cancelButton.addEventListener("click", function() {
            HideElement(domElements.MultiRecipientModal);
        });

        mainContainer.appendChild(footerContainer);

        domElements.MultiRecipientModal = mainContainer;

        mainCampaignMgrModel.appendChild(domElements.MultiRecipientModal);
    }

    /**
    * hide the cancel button in the multi recipient model
    * @returns {} 
    */
    function HideMultiRecipientCancelButton() {
        HideElement(document.getElementById('sfEbSendCampaignMultiRecipientCancel'));
    }

    /**
    * show the cancel button in the multi recipient model
    * @returns {} 
    */
    function ShowMultiRecipientCancelButton() {
        ShowElement(document.getElementById('sfEbSendCampaignMultiRecipientCancel'));
    }

    /**
    * Creates or Shows the multiple recipient modal
    * @returns {} 
    */
    function CreateOrShowMultiRecipientModal() {
        if (domElements.MultiRecipientModal != null && IsElementInDom(domElements.MultiRecipientModal, mainCampaignMgrModel)) {
            ShowElement(domElements.MultiRecipientModal);
            
            // if multi recipients are selected hide cancel button, otheer wise show cancel button
            if (multipleContactSelectedBool) {
                HideMultiRecipientCancelButton();
            } else {
                ShowMultiRecipientCancelButton();
            }
        } else {
            lThisCampaignManagerInstance.ResetMultiRecipientTableData();
            CreateMultiRecipientModal();
            GetCampaignSalesforceListViewObjects();
        }
    }

    /**
     * Hides the Multi Recipients text (250 Recipients Selected)
     * @returns {} 
     */
    function HideMultiRecipientsText() {
        HideElement(domElements.MultiRecipientsTextSpan);
    }

    /**
     * Show the Multi Recipients text (250 Recipients Selected)
     * @returns {} 
     */
    function ShowMultiRecipientsText() {
        ShowElement(domElements.MultiRecipientsTextSpan);
    }

    /**
    * Hides the contact fields and replaces it with a text element stating how many people have been selected for this campaign
    * @param {} recipientsSelected 
    * @returns {} 
    */
    function ReplaceContactDisplayWithMultiRecipientLink() {

        var contactContainer = document.getElementById('sfEbSendCampaignDetailsContactContainer');

        if (contactContainer) {
            HideElement(contactContainer);

            //Text to display on the main campaign modal when mutli recipients have been selected and confirmed
            var textMessage = "";
            if (multiRecipientRowSelectedCounter > 0) {
                textMessage += multiRecipientRowSelectedCounter + " recipient";

                if (multiRecipientRowSelectedCounter !== 1) textMessage += "s";
                textMessage += " selected";
            }

            //Create or Update text element saying how many recipients were selected
            if (domElements.MultiRecipientsTextSpan != null
                && IsElementInDom(domElements.MultiRecipientsTextSpan, mainCampaignMgrModel)) {
                domElements.MultiRecipientsTextSpan.childNodes[0].textContent = textMessage;
                ShowMultiRecipientsText();
            } else {
                domElements.MultiRecipientsTextSpan = CreateElement("span", "ebsta-link-text", textMessage);
                domElements.MultiRecipientsTextSpan.appendChild(CreateElement("div", icons.Pencil));

                insertAfter(domElements.MultiRecipientsTextSpan, contactContainer);

                //Re-show the modal when clicking on the message - always set the click event.
                domElements.MultiRecipientsTextSpan.addEventListener("click", function () {
                    CreateOrShowMultiRecipientModal();
                });
            }
        }
    }

    /**
     * Updates the state of the complex add button
     * @param {} buttonElement 
     * @param {} selected 
     * @returns {} 
     */
    function UpdateComplexAddButton(buttonElement, selected) {
        if (buttonElement != null) {
            if (selected) {
                AddClass(buttonElement, "ebsta-btn-success");
            } else {
                RemoveClass(buttonElement, "ebsta-btn-success");
            }
        }
    }

    /** 
     * Sets the Complex add button to be a simple switch which either inform us 
     * whether the user wants us to auto relate the emails/tasks we add into Salesforce
     */
    function SetComplexAddButtonForMultiRecipientMode() {
        if (domElements.ComplexAddButton != null) {
            var cloneOfComplexAddButton = domElements.ComplexAddButton.cloneNode(false);
            ReplaceNodeWithNewNode(cloneOfComplexAddButton, domElements.ComplexAddButton);
            domElements.ComplexAddButton = cloneOfComplexAddButton;

            UpdateComplexAddButton(domElements.ComplexAddButton, false);    

            //Keep the button disabled if the user doesn't have access to this feature yet
            if (!enableAutoRelateComplexAddForMultiRecipient) {
                AddClass(domElements.ComplexAddButton, "ebsta-button-disabled");
                domElements.ComplexAddButton.appendChild(new ToolTip().CreateSmallToolTip("Disabled for Multiple Recipients"));
            } else {
                cloneOfComplexAddButton.addEventListener("click", function () {
                    UpdateComplexAddButton(this, !autoRelateMultiRecipientCampaign);
                    autoRelateMultiRecipientCampaign = !autoRelateMultiRecipientCampaign;
                });
            }                   
        }
    }

    /**
     * Attaches a click event to the complex add button in campaigns
     * @param {} addToSalesforceButton 
     * @param {} complexAddClosure 
     * @returns {} 
     */
    function AttachClickListenerToAddToSalesforceButton(addToSalesforceButton, complexAddClosure) {
        if (addToSalesforceButton != null) {
            if (complexAddClosure != null) complexAddClosure.SetComplexAddButton(addToSalesforceButton);
            addToSalesforceButton.addEventListener("click", function() {
                var emailElement = document.getElementById('sfEbSendCampaignDetailsEmailValue');

                complexAddClosure.AddComplexAdd({
                    MessageType: "AddTaskEmail",
                    Id: emailElement == null ? null : emailElement.value,
                    EmailText: emailElement == null ? null : emailElement.value,
                    CampaignManager: true,
                    ComplexAddClosure: complexAddClosure,
                    SecondaryEmailSearch: new Array()
                });
            });
        }
    }

    /**
     * Resets the click listener on the complex add button by replacing the node
     * @param {} complexAddClosure 
     * @param {} deepCopy 
     * @returns {} 
     */
    function ResetClickListenerOnAddToSalesButton(complexAddClosure, deepCopy) {
        //Recreate complex add button node so we remove the original attached listener
        if (domElements.ComplexAddButton) {
            var cloneOfComplexAddButton = domElements.ComplexAddButton.cloneNode(deepCopy);
            ReplaceNodeWithNewNode(cloneOfComplexAddButton, domElements.ComplexAddButton);
            domElements.ComplexAddButton = cloneOfComplexAddButton;
            AttachClickListenerToAddToSalesforceButton(domElements.ComplexAddButton, complexAddClosure);
        }
    }

    /**
     * Sets complex add button to be enabled in its normal state
     * @returns {} 
     */
     function SetComplexAddButtonToBeEnabled() {
        if (domElements.ComplexAddButton != null) {
            RemoveClass(domElements.ComplexAddButton, "ebsta-button-disabled");
            UpdateComplexAddButton(domElements.ComplexAddButton, false);
            ResetClickListenerOnAddToSalesButton(lComplexAddClosure, false);
        }
     }

    /**
     * Displays the "Send" button with the desired text
     * @param {} text 
     * @returns {} 
     */
    function ShowCampaignSendButton(text) {
        var send = document.getElementById('sfEbSendCampaignMgrMainModelSend');
        if (send) {
            send.innerText = text;
            ShowElement(send);
        }
    }

    /**
     * Shows the Campaign Select button which is used for campaigns in compose mode
     * @param {any} text - text to display inside the button
     */
    function ShowCampaignSelectButton(text) {
        var select = document.getElementById('sfEbSendCampaignMgrMainModelSelect');
        if (select) {
            select.innerText = text;
            select.style.display = 'inline-block';
        }
    }

    /** Hides the Send Button */
    function HideCampaignSendButton() {
        var send = document.getElementById('sfEbSendCampaignMgrMainModelSend');
        if (send) {
            HideElement(send);
        }
    }

    /**
     * Hides the select button within the campaign modal
     * @returns {} 
     */
    function HideCampaignSelectButton () {
        var select = document.getElementById('sfEbSendCampaignMgrMainModelSelect');
        if (select) {
            select.style.display = 'none';
        }
    }

    /**
     * Updates the multipleContactSelected variable with the given value
     * @param {} selected 
     * @returns {} 
     */
    function UpdateMultipleContactSelectedBool(selected) {
        multipleContactSelectedBool = selected;
    }

    /**
    * When opening the campaign modal from a compose window this will determine whether this needs to be set up 
    * for a single recipient or a multi recipient
    * @returns {} 
    */
    function PrePopulateContactsIntoCampaignModal() {
        //Multi Recipients
        if (contactObjectAll != null && contactObjectAll.length > 1
            || multipleContactArray != null && multipleContactArray.length > 1) {
            UpdateMultipleContactSelectedBool(true);
            ReplaceContactDisplayWithMultiRecipientLink();
            SetComplexAddButtonForMultiRecipientMode();
            //If there were errors parsing anyone over from the compose window open the multi-recipiens window straight off
            if (multiRecipientComposeErrorCounter > 0) {
                lThisCampaignManagerInstance.ResetMultiRecipientTableData();
                lThisCampaignManagerInstance.HideMultiRecipientModal();
                CreateOrShowMultiRecipientModal();
            } else if (enableCampaignDupeCheck) {
                CreateOrShowMultiRecipientModal();
                DupeCheckAllSelectedMultiRecipients();
            }
        } else {
            //Single Recipient
            PopulateSingleContactFromComposeToCampaign();
            SetComplexAddButtonToBeEnabled();
        }
    }

    /**
    * Checks the URL of the current gmail tab used to send a campaign and if it's in a 
    * particular view mode we should remove the tab just like Gmail do
    */
    function CloseTab() {
        if (GmailTools && GmailTools.IsThisAExternalGmailComposeScreen()) {
            close();
        }
    }

    /**
     * Clears all recent campaigns from the container
     * @returns {} 
     */
    function ClearRecentCampaigns() {
        if (domElements.DefaultCampaignMenuPopUp != null) {
            var recentCampaigns = domElements.DefaultCampaignMenuPopUp.getElementsByClassName("sfEbstaCampaign");
            for (var i = recentCampaigns.length - 1; i > -1; i--) {
                recentCampaigns[i].remove();
            }
        }
    }

    /**
     * Removes the last inserted overlay
     * @returns {} 
     */
    function RemoveOverlay() {
        if (domElements.LoadingOverlay != null) {
            domElements.LoadingOverlay.remove();
            domElements.LoadingOverlay = null;
        }
    }

    /**
     * Attachs a click listener on the container for the campaign items so that when clicked we go find the item click to find which campaign had been clicked
     * @param {} element 
     * @returns {} 
     */
    /**
     * Attachs a click listener on the container for the campaign items so that when clicked we go find the item click to find which campaign had been clicked
     * @param {} element 
     * @param { Informs us that this Campaign selected was not selected from a compose window } notComposeSelection 
     * @returns {} 
     */
    function AttachClickThroughToCampaignFlowView(element, notComposeSelection) {
        if (element != null) {
            element.addEventListener('click', function (e) {
                var target = e.target;
                var div = target.closest('.sfEbstaCampaign');
                if (div) {
                    var id = div.getAttribute("sfebsendcampaignkey");

                    if (id !== null && typeof id === 'string' && id.length > 0) {
                        lThisCampaignManagerInstance.GetCampaignDetails(id, notComposeSelection);
                    }
                }
                e.stopPropagation();

                lThisCampaignManagerInstance.HideCampaignComposeMenu();
                CampaignManagerResizing().ResizeCampaignManager();
            });
        }
    }

    /**
     * Triggers the trash icon within a compose window and sets a default style so that we hide the 
     * gmail message which appears when you discard a draft
     * @param {} trashIcon 
     * @returns {} 
     */
    function TriggerDiscardDraftComposeIcon(trashIcon) {
        // Hide he container before we close the compose screen down, just to ensure it doesn't flicker on the screen
        // between the time from closing the compose window to removing the message
        InjectTemporaryStyle('.b8.UC.bAp { display: none; }', 1000);

        //Trigger the event on the "discard draft" button
        SimulateMouseEventOnElement("click", trashIcon);

        RetryWithCallback(60, 0.1, function() {
            return document.getElementsByClassName("b8 UC bAp").length === 1;
        }, function() {
            var discardMessage = document.getElementsByClassName("b8 UC bAp");
            if (discardMessage.length === 1) {
                discardMessage = discardMessage[0];

                var contentsContainer = discardMessage.getElementsByClassName("vh")[0];
                if (contentsContainer) {
                    EmptyElement(contentsContainer);
                    RemoveClass(discardMessage, "bAp");
                    
                    //This is how gmail hides the element itself once its now viewable so copy them to save any issue with adding classed/properties etc
                    discardMessage.style.position = "relative";
                    discardMessage.style.top = "-1000px";
                }
            }
        });
    }

    /**
     * Finds the email tracking button and looks for class which sets it to be on
     * @returns {} 
     */
    function IsCampaignTracked() {
        if (HasClass(domElements.EmailTrackingButton, "sfEbstaGmailEmailTrackingSelected")) {
            return true;
        }
        return false;
    }

    /**
     * Gets the user(s) selected in the search dropdown
     * @returns {array} - An array of owner ids. Empty if "All Campaigns" is selected.
     */
    function GetSelectedCampaignOwners() {
        var rtnArray = new Array();
        if (domElements.SearchPage.CampaignOwnerSelectDropDown) {
            var selectedValue = $(domElements.SearchPage.CampaignOwnerSelectDropDown).val();
            if (selectedValue && selectedValue !== "all") {
                rtnArray.push(selectedValue);
            }
        }
        return rtnArray;
    }

    /**
     * Creates the user select dropdown on the main campaign search screen
     * @param {DOMElement} elementToAppendTo - The element to append the dropdown to
     */
    function CreateUserDropdown(elementToAppendTo) {
        var loadingDropDown = CreateElement("select", null, "Loading...", "ddCampaignOwner", null);
        elementToAppendTo.appendChild(loadingDropDown);

        chrome.extension.sendMessage({ MessageType: "GetLastSearchedCampaignUsers" },
            function(previouslySavedValues) {

                var helperArray = [];
                var starterObject = null;
                var customIdValueForAllCampaigns = "all";

                for (var i = 0; i < companyUsers.length; i++) {
                    var name = companyUsers[i].Name;
                    var id = companyUsers[i].Id;
                    var currentUser = companyUsers[i].CurrentUser;

                    if (name && id) {
                        if (currentUser === true) {
                            //Insert All Campaigns always after My Campaigns so the default starting selectiong is My Campaigns
                            var allCampaignsSelection = { value: customIdValueForAllCampaigns, text: "All Cadences" };

                            //The array saved should be empty if All Campaigns was selected last
                            if (previouslySavedValues && previouslySavedValues.length == 0) {
                                starterObject = allCampaignsSelection;
                            }
                            helperArray.unshift(allCampaignsSelection);

                            var myCampaignsSelection = { value: id, text: "My Cadences" };
                            if (starterObject == null) {
                                starterObject = myCampaignsSelection;
                            }
                            helperArray.unshift(myCampaignsSelection);
                        } else {
                            var userSelection = { value: id, text: name };
                            if (previouslySavedValues && previouslySavedValues.indexOf(id) != -1) {
                                starterObject = userSelection;
                            }
                            helperArray.push(userSelection);
                        }
                    }
                }
                var userDropDown = ebstaDomElementCreator.CreateDropDown("chosen-select", "ddCampaignOwner", helperArray);
                ReplaceNodeWithNewNode(userDropDown, loadingDropDown);
                $(userDropDown).chosen({ width: "200px", search_contains: true }).on("change", function () {
                    RunSearch(1);
                });
                $(userDropDown).val(starterObject.value).trigger("chosen:updated");
                domElements.SearchPage.CampaignOwnerSelectDropDown = userDropDown;
                RunSearch(1);
            });
    }

    /**
     * Sends a message to background to fire a request off, response handlers are in "SendMessageResponseHandlers"
     * @param {object} request - The request object
     * @param {function} callback - A function that can be used to call if anything extra needs to happen when the response from the message happens
     */
    function SendMessageToBackGround (request, callback) {
        if (request) {
            chrome.extension.sendMessage(request, function (response) {
                lThisCampaignManagerInstance.SendMessageResponseHandlers(response, callback);
            });
        }
    }

    /**
     * Handles the response for getting setup values for the campaign
     * @param {object} response - The object that is returned from the message request
     * @param {boolean} response.success - If the request was successful
     * @param {object} response.values - The values need to setup campaign
     * @param {boolean} response.values.EnableAutoRelateComplexAddForMultiRecipient - Is the auto relate feature enabled
     * @param {array} response.values.SendFromMailboxes - A list of mailboxes a user can send a campaign from
     * @param {array} response.values.TaskAndDecisions - The list of different tasks and decisions used to help render the campaign flow
     * @param {array} response.values.SalesforceCampaigns - A list of Salesforce Campaigns
     */
    function HandleGetCampaignSetupResponse(response) {
        if (response.success) {
            var responseValues = response.values;
            if (responseValues) {
                companyUsers = responseValues.CompanyUsers;
                enableAutoRelateComplexAddForMultiRecipient = responseValues.EnableAutoRelateComplexAddForMultiRecipient;

                if (responseValues.SendFromMailboxes) {
                    listOfMailboxesToSendCampaignFrom = responseValues.SendFromMailboxes;
                }

                if (responseValues.TaskAndDecisions) {
                    tasksWithDecisions = new Array();
                    for (var i = 0; i < responseValues.TaskAndDecisions.length; i++) {
                        tasksWithDecisions.push(JSON.parse(responseValues.TaskAndDecisions[i]));
                    }
                }

                if (responseValues.SalesforceCampaigns) {
                    listOfSalesforceCampaigns = responseValues.SalesforceCampaigns;

                    //Sort the campaigns by name for potential later use
                    listOfSalesforceCampaigns.sort(function (x, y) {
                        if (x.Name > y.Name) {
                            return 1;
                        }

                        if (x.Name < y.Name) {
                            return -1;
                        }

                        return 0;
                    });
                }

                if (responseValues.EnableCsvMergeFields !== undefined) {
                    enableCsvMergeFields = responseValues.EnableCsvMergeFields;
                }

                if (responseValues.EnableCampaignDupeCheck !== undefined) {
                    enableCampaignDupeCheck = responseValues.EnableCampaignDupeCheck === true;
                }
            }
        } else {
            throw new Error("Failed to get setup for campaigns")
        }
    }

    /**
     * Get values needed for campaign manager to function correctly
     * @param {boolean} forceHttpRequestRetry - Enforces a whole new HTTP request
     * @param {any} callback - Function to call when receiving message back
     * @returns {object} - Functions to access this closure
     */
    function GetSetupValues(forceHttpRequestRetry, callback) {
        var isRequestActive = true;

        chrome.extension.sendMessage({
            MessageType: "GetCampaignSetup",
            ForceHttpRequestRetry: forceHttpRequestRetry
        }, function response(request) {
            HandleGetCampaignSetupResponse(request);
            if (callback) {
                callback();
            }

            isRequestActive = false;
        });

        return {
            IsRequestActive: function() {
                return isRequestActive;
            }
        };
    }

    /** Creates our own ebsta spinner loader */
    function CreateSpinnerLoaderDiv() {
        var loaderDiv = CreateElement('div', icons.Spinner + " ebsta-campaign-loaderDiv", null, );
        return loaderDiv;
    }

    /** Adds a loading gif to the search result panel to show the its currently loading */
    function AddLoaderToMainModelListItems () {
        var resultList = document.getElementById("sfEbSendCampaignMgrMainModelListItems");
        if (resultList) {
            resultList.innerHTML = '';
            resultList.appendChild(CreateSpinnerLoaderDiv());
        }
    }

    /**
     * Runs a search for campaigns
     * @param {any} searchTerm
     * @param {any} page
     */
    function RunSearch(page) {

        if(page > 0) pageNumber = page;

        var searchInput = document.getElementById("sfEbSendCampaignMgrSearch");
        if (searchInput) {
            currentSearchTerm = searchInput.value;
        }

        AddLoaderToMainModelListItems();

        var webusersToSearchFor = GetSelectedCampaignOwners();

        var request = {
            MessageType: "SearchCampaigns",
            SearchTerm: currentSearchTerm,
            PageNumber: pageNumber,
            ItemsPerPage: itemsPerPage,
            OrderField: campaignSortField,
            SortAsc: campaignSortAsc,
            WebuserIds: webusersToSearchFor
        };

        SendMessageToBackGround(request);

        chrome.extension.sendMessage({
            MessageType: "SaveLastSearchedCampaignUsers",
            SavedIds: webusersToSearchFor
        });
    }

    /**
     * These are the core bit of data we need within the module in order for it to function correctly, we 
     * shouldn't proceed with opening the window if we don't have these values
     */
    function DoWeHaveNeededStartUpValues() {
        if (tasksWithDecisions.length > 0 &&
            companyUsers.length > 0 && 
            listOfMailboxesToSendCampaignFrom != null && listOfMailboxesToSendCampaignFrom.length > 0)
        {
            return true;
        }
        return false;
    }

    /** Closes down the screen */
    var closeCampaignMgr = function () {
        if (mainCampaignMgrModel) {
            mainCampaignMgrModel.remove();
        }
    }

    /**
     * Override the current method to close down the campaign manager screen
     * @param {any} functionToCall
     */
    function SetCloseCampaignFunction(functionToCall) {
        closeCampaignMgr = functionToCall;
    }

    /**
     * Response handler for when a user selected multiple id's from a list view in Salesforce to be populated into a Campaign
     * @param {any} response
     */
    function GetSalesforceRecordDetailsFromIdRequestTypeResponseHandler(response) {
        //wipe contact object on new campaign setup
        contactObject.ResetContact();
        contactObjectAll = new Array();
        multiRecipientRowSelectedCounter = 0;
        multiRecipientComposeErrorCounter = 0;

        for (var i = 0; i < response.values.length; i++) {
            var result = response.values[i];

            if (result) {
                var parsedResult = JSON.parse(result);
                var firstName = parsedResult["FirstName"];
                var lastName = parsedResult["LastName"];
                var email = parsedResult["Email"];
                var company = parsedResult["Company"];

                var newContact = new Contact(email, firstName, lastName, company);

                // if this is the first one, add to contactObject object and always add to contactObjectAll Object
                if (contactObjectAll.length === 0) {
                    contactObject = newContact;
                    //As this has come from Salesforce already we shouldn't need to do an extra Salesforce lookup for this match like we do already
                    contactObject.StopSalesforceLookup = true;
                }

                contactObjectAll.push(newContact);

                if (newContact.GetEmailAddress() !== null && newContact.GetFirstName() !== null) {
                    multipleContactArray.push(newContact.GetContact());
                    multiRecipientRowSelectedCounter++;
                } else {
                    multiRecipientComposeErrorCounter++;
                }
            }
        }
    }

    /**
     * Sends a Campaign Request
     * @param {number} campaignRequestType - The type of request (CampaignRequestType)
     * @param {object} requestBody - The body of the request
     * @param {object} callBack - The callback to be called when response has returned
     */
    function SendCampaignRequest(campaignRequestType, requestBody, callBack) {
        // Keep within a closure so we can store a local instance of values do we 
        // can process multiple concurrent request and the response is handled correctly
        (function (localCampaignRequestType) {
            var request = {
                MessageType: "SendCampaignRequest",
                CampaignRequestType: localCampaignRequestType,
                RequestBody: requestBody
            };

            if (campaignRequestType === CampaignRequestType.GetSalesforceRecordDetailsFromId) {
                isSalesforceRecordListRequestActive = true;
            }

            chrome.extension.sendMessage(request, function Response(response) {
                if (response.success === true && response.values !== null) {
                    switch (localCampaignRequestType) {
                        case 1:
                            GetSalesforceRecordDetailsFromIdRequestTypeResponseHandler(response);
                            break;
                        case CampaignRequestType.GetLiveCampaignInfoForContact:
                            callBack(response.values);
                            return;
                        default:
                            break;
                    }
                }

                isSalesforceRecordListRequestActive = false;
            });
        })(campaignRequestType);
    }

    /**
     * Opens up the Campaign screen
     * @param {DOMelement} elementSelectedToOpenCampaign - The element that was clicked in order to open the campaign screen
     * @param {boolean} isComposeMode - Is campaigns selected from a Gmail compose window
     * @param {array} listOfSalesforceIds - A list of salesforceids to pre-populate the multi-recipient screen with
     */
    function OpenModal(elementSelectedToOpenCampaign, isComposeMode, listOfSalesforceIds) {

        /** The main function in order to make the campaign screen to appear */
        function CampaignModalOpening() {
            lThisCampaignManagerInstance.ShowMainModelContentPage();
            campaignComposeMode = isComposeMode;
            if (!campaignComposeMode) {
                contactObject.ResetContact();
            }
            lThisCampaignManagerInstance.ResetMultipleContactArray();
            HideCampaignSendButton();
            lThisCampaignManagerInstance.ResetMultiRecipientTableData();
            HideElement(domElements.MultiRecipientModal);

            if (elementSelectedToOpenCampaign) {
                PreSelectContactFromComposeWindowIntoCampaign(elementSelectedToOpenCampaign);
                PreSelectCcAndBccContactsFromComposeWindowIntoCampaign(elementSelectedToOpenCampaign);
            } else if (listOfSalesforceIds !== null && Array.isArray(listOfSalesforceIds) && listOfSalesforceIds.length > 0) {
                SendCampaignRequest(1, { recordIds: listOfSalesforceIds });
            }

            lThisCampaignManagerInstance.SHowMainModelContentDiv();
            lThisCampaignManagerInstance.HideMainModelDetailsDiv();
            CampaignManagerResizing().ResizeCampaignManager();
        }

        //Always get the setup values from background in case they've been updated since last time the campaign window was created/opened         
        GetSetupValues(false, function () {
            if (DoWeHaveNeededStartUpValues()) {
                CampaignModalOpening();
            } else {
                //If we didn't have the values at start, something has gone wrong, put in a retry process to try get the values again
                GetSetupValues(true, function () {
                    if (DoWeHaveNeededStartUpValues()) {
                        CampaignModalOpening();
                    } else {
                        SendMessageToDisplayAlert(false, "An error has occured, please try again or refresh your page.");
                    }
                });
            }
        });
    }

    /**
     * Checks if we're still initiliaing the closure, before opening the modal
     * @param {DOMelement} elementSelectedToOpenCampaign - What element was selected to open the screen
     * @param {boolean} isComposeMode - Was this selected from a Gmail compose window
     * @param {array} listOfSalesforceIds - List of salesforce ids selected to pre-poopulate into multi-recipient table
     */
    function Open(elementSelectedToOpenCampaign, isComposeMode, listOfSalesforceIds) {
        (function (elementSelectedToOpenCampaign, isComposeMode, listOfSalesforceIds) {
            RetryWithCallback(60, 0.1,
                function () { return !isInitialisationInProgress; },
                function successCallbk() {
                    OpenModal(elementSelectedToOpenCampaign, isComposeMode, listOfSalesforceIds);
                },
                function failureCallbk() {
                    OpenModal(elementSelectedToOpenCampaign, isComposeMode, listOfSalesforceIds);
                });
        })(elementSelectedToOpenCampaign, isComposeMode, listOfSalesforceIds);
    }

    /**
     * Creates a list of validation functions for the tags input
     * @return {Array<EbstaTagValidator>} - List of validation functions
     */
    function EbstaTagValidationFunctions() {
        const arrayOfValidationFunctions = [];
        const emailValidation = new EbstaTagValidator(ValidateEmail, "Not a valid email address");
        arrayOfValidationFunctions.push(emailValidation);
        return arrayOfValidationFunctions;
    }

    /**
     * Creates a recipient section for CC or BCC
     * @param {string} label - The label for the section which will appear beside the input
     * @return {DOMElement} - The container
     */
    function CreateAdditionalRecipientContainer(label) {
        const containerElement = CreateElement("div", "sfEbSendCampaignAdditionalRecipients");
        containerElement.appendChild(CreateElement("span", null, label));

        const recipients = CreateElement("div");

        const ebstaTagsInput = EbstaTagsInput(EbstaTagValidationFunctions());
        recipients.appendChild(ebstaTagsInput.GetElement());
        containerElement.appendChild(recipients);

        return {
            Element: containerElement,
            TagsInput: ebstaTagsInput
        };
    }

    /**
     * Creates a loader type visual for the 'FROM' selector so we can gather the most up-to-date list information before rendering the list
     * @returns {DOMElement} - The DOM for the loader 
     **/
    function CreateSendCampaignFromMailboxLoader() {
        var sendFromDropDownContainer = CreateElement("div", "sfEbSendCampaignFromContainer");
        sendFromDropDownContainer.appendChild(CreateElement("span", null, "From: "));
        sendFromDropDownContainer.appendChild(CreateSpinnerLoaderDiv());
        sendFromDropDownContainer.appendChild(CreateElement("span", null, "Loading mailboxes.."));
        return sendFromDropDownContainer;
    }

    /**
     * Replaces the content within the From container with the given element
     * @param {DOMElement} domToReplaceWith - The DOM that will replace the current DOM
     */
    function ReplaceCampaignSendFromElement(domToReplaceWith) {
        var currentElement = document.getElementsByClassName("sfEbSendCampaignFromContainer")[0];
        if (currentElement) {
            ReplaceNodeWithNewNode(domToReplaceWith, currentElement);
        }
    }

    /**
     * Create the header where the user can select what email to send the campaign from
     * @return {DOMElement} - The DOM for the mailbox selection dropdown
     */
    function CreateSendCampaignFromElement() {
        var sendFromDropDownContainer = CreateElement("div", "sfEbSendCampaignFromContainer");
        sendFromDropDownContainer.appendChild(CreateElement("span",null, "From:"));

        var activeAccountsToDisplay = [], defaultValue = null, foundLastSelected = false;

        if (listOfMailboxesToSendCampaignFrom) {
            for (var i = 0; i < listOfMailboxesToSendCampaignFrom.length; i++) {
                var mailbox = listOfMailboxesToSendCampaignFrom[i];
                if (mailbox.Mailbox != null) {
                    var mailboxName = mailbox.Mailbox;

                    var objectToAddToList = { text: mailboxName, value: mailboxName, details: mailbox};
                    activeAccountsToDisplay.push(objectToAddToList);

                    //If we have now default value then set the first object we come into
                    if (defaultValue === null) {
                        defaultValue = objectToAddToList;
                    }

                    if (mailbox.LastSelected === true) {
                        defaultValue = objectToAddToList;
                        foundLastSelected = true;
                    }

                    ///If the default value we selected is Gmail and we havent found a previously selected and this new value 
                    // is not a gmail email then prioritise this over the selected gmail account
                    if (!foundLastSelected) {
                        if (defaultValue.value.indexOf("@gmail.") !== -1 && mailboxName.indexOf("gmail.") === -1) {
                            defaultValue = objectToAddToList;
                        }
                    }

                    var defaultImageUrl = "";

                    switch (mailbox.TokenType) {
                        case 1:
                            defaultImageUrl = chrome.extension.getURL("images/mailicons/gmail.jpg");
                            break;
                        case 2:
                            defaultImageUrl = chrome.extension.getURL("images/mailicons/office365.png");;
                            break;
                    }

                    objectToAddToList.imageUrl = defaultImageUrl;
                }
            }
        }

        selectedGoogleMailbox = defaultValue;

        var mailboxDropDownElement = EbstaCustomDropdown.CreateCustomDropDown("ddCampaignMailbox", defaultValue, activeAccountsToDisplay,
            function (selectedValueObject) {
                selectedGoogleMailbox = selectedValueObject;

                //When a new value is selected fire off another auth check
                CheckGmailAccountIsAuthedBeforeSend();
            });
        sendFromDropDownContainer.appendChild(mailboxDropDownElement);

        return sendFromDropDownContainer;
    }

    /** Pre-populates the CC and BCC recipients extracted from the compose window into the CC and BCC fields in the campaign screen*/
    function PrePopulateCcAndBccTags() {
        for (let i = 0; i < ccComposeRecipients.length; i++) {
            const localContactObject = ccComposeRecipients[i];
            if (localContactObject.GetEmailAddress() != null) {
                domElements.DetailsPage.CcTagsInput.AddTag(localContactObject.GetEmailAddress());
            }
        }

        for (let i = 0; i < bccComposeRecipients.length; i++) {
            const localContactObject = bccComposeRecipients[i];
            if (localContactObject.GetEmailAddress() != null) {
                domElements.DetailsPage.BccTagsInput.AddTag(localContactObject.GetEmailAddress());
            }
        }
    }

    /**
     * Creates the section where a user can choose the mailbox to send the campaign from along with any CC or BCC recipients
     * @return {DOMElement} - The contents of the section to render
     */
    function CreateFromAndRecipientHeader() {
        const sendCampaignHeader = CreateElement("div", "sfEbSendCampaignHeader");
        const sendFromHeader = CreateElement("div", "sfEbSendCampaignFromHeader", null, "");

        sendFromHeader.appendChild(CreateSendCampaignFromMailboxLoader()); 
        
        const additionalRecipientsContainer = CreateElement("div", "sfEbSendCampaignAdditionRecipientsContainer");
        sendFromHeader.appendChild(additionalRecipientsContainer);
        const bccHyperlink = CreateElement("a", null, "Bcc", "EbstaAnchorCampaignBcc");
        const ccHyperlink = CreateElement("a", null, "Cc", "EbstaAnchorCampaignCc");

        additionalRecipientsContainer.appendChild(ccHyperlink);
        additionalRecipientsContainer.appendChild(bccHyperlink);
        sendCampaignHeader.appendChild(sendFromHeader);

        const ccSection = CreateAdditionalRecipientContainer("Cc:");
        domElements.DetailsPage.CcTagsInput = ccSection.TagsInput;

        const bccSection = CreateAdditionalRecipientContainer("Bcc:");
        domElements.DetailsPage.BccTagsInput = bccSection.TagsInput;

        PrePopulateCcAndBccTags();

        sendCampaignHeader.appendChild(ccSection.Element);
        sendCampaignHeader.appendChild(bccSection.Element);

        HideElement(ccSection.Element);
        HideElement(bccSection.Element);

        bccHyperlink.addEventListener("click",
            function () {
                bccSection.Element.style.display = bccSection.Element.style.display === "" ? "none" : "";
            });

        ccHyperlink.addEventListener("click",
            function () {
                ccSection.Element.style.display = ccSection.Element.style.display === "" ? "none" : "";
            });

        return sendCampaignHeader;
    }

    /** Core setup process to ensure we have the values we need for the campaign module to work */
    function Initialize() {
        if (!window["GLOBAL_EBSTA_MODAL"]) {
            window["GLOBAL_EBSTA_MODAL"] = new EbstaModal(new Array());
        }

        isInitialisationInProgress = true;
        GetSetupValues(false, function () {
            isInitialisationInProgress = false;
        });        
    }

    /**
     * Creates the DOM to render next to the FROM mailbox selector when there has been an error
     * @param {string} errorText - The text to display
     * @returns {DOMelement} - The element to display
     */
    function CreateMailboxSelectionFailureElement(errorText) {
        var invalidAuthenticationElement = CreateElement("div", "authenticatorText");
        invalidAuthenticationElement.appendChild(CreateElement("i", "fa fa-exclamation-triangle ebTextError"));
        invalidAuthenticationElement.appendChild(CreateElement("span", "ebTextError", errorText));
        return invalidAuthenticationElement;
    }

    /**
     * Checks if the selected gmail account is authed before allowing the user to send it
     */
    function CheckGmailAccountIsAuthedBeforeSend() {

        HideCampaignSelectButton();
        HideCampaignSendButton();

        isSelectedMailboxAuthenticated = false;

        var mailboxDropdown = document.getElementById("ddCampaignMailbox");

        if (mailboxDropdown) {
            RemoveClass(mailboxDropdown, "ebError");

            //If there is a sibling this means that we're already showing a message next to the div so remove it so we can we re-render the next message
            if (mailboxDropdown.nextSibling) {
                mailboxDropdown.nextSibling.remove();
            }

            var loaderElement = CreateElement("div", "authenticatorText");
            loaderElement.appendChild(CreateSpinnerLoaderDiv());
            loaderElement.appendChild(CreateElement("span", null, "Checking mailbox..."));

            insertAfter(loaderElement, mailboxDropdown);

            //Runs a check to see if the mailbox selected is authenticated and still shared with the user to use
            (function (lSelectedGoogleMailbox) {
                chrome.extension.sendMessage({
                    MessageType: "CampaignMailboxAuthCheck",
                    MailboxAddress: lSelectedGoogleMailbox.details.Mailbox,
                    IsSharedMailbox: lSelectedGoogleMailbox.details.IsSharedMailbox,
                    SharedMailboxUserId: lSelectedGoogleMailbox.details.WebuserId,
                    TokenType: lSelectedGoogleMailbox.details.TokenType
                }, function (responseData) {
                    //Make sure this request matches the current selection in case we had multiple requests firing at the same time for different email addresses
                    if (lSelectedGoogleMailbox.details.Mailbox === selectedGoogleMailbox.details.Mailbox) {
                        if (responseData.d) {
                            if (responseData.d.Success === true) {
                                if (responseData.d.Values !== null) {
                                    if (responseData.d.Values.length > 0) {
                                        let parsedValues = JSON.parse(responseData.d.Values);
                                        
                                        if (parsedValues.IsMailboxAuthenticated === true) {
                                            isSelectedMailboxAuthenticated = true;
                                        } else {
                                            AddClass(mailboxDropdown, "ebError");
                                            //Show some new text and buttons to inform the user they won't be able to send until they re-authenticate that gmail account
                                            insertAfter(CreateMailboxSelectionFailureElement("Account not authenticated."), mailboxDropdown);
                                        }

                                        ShouldWeShowSendCampaignButton();
                                    }
                                }
                            } else {
                                var errorMessage = "Please try another mailbox.";
                                if (responseData.d.ErrorMessage) {
                                    errorMessage = responseData.d.ErrorMessage;
                                }
                                AddClass(mailboxDropdown, "ebError");
                                insertAfter(CreateMailboxSelectionFailureElement(errorMessage), mailboxDropdown);
                            }
                        }
                    }
                    loaderElement.remove();
                });
            })(selectedGoogleMailbox);
        }
    }

    /**
     * Sends a message to the top DOM to display a message which should have 
     * domcreation.js injected so it receives this communication message to display the chosen message below
     * @param {any} successType
     * @param {any} messageText
     */
    function SendMessageToDisplayAlert(successType, messageText) {
        window.top.postMessage({ RequestType: "SHOWMESSAGE", MessageText: messageText, Successful: successType }, "*");
    }

    /**
     * Handles the response back onces a campaign has been scheduled by a user
     * @param {any} response
     */
    function SaveTaskStagesForContactHandleResponse(response) {
        var wasRequestSuccesful = false;
        var messageText = "";

        if (response) {
            if (response.success === true) {
                wasRequestSuccesful = true;
                messageText = "Cadence successfully scheduled";

                setTimeout(function () {
                    if (campaignComposeMode === true) {
                        CloseTab();
                    }
                },2500);
            } else {
                if (typeof response.errorMessage != "undefined" && response.errorMessage != null) {
                    messageText = response.errorMessage;
                } else {
                    messageText = "Ebsta failed to schedule this campaign please try again";
                }
            }

            SendMessageToDisplayAlert(wasRequestSuccesful, messageText);
        }
    }

    /**
     * Creates the empty results message when there are no campaigns returned in a search request
     * @param {any} isTheAUserSearch - Was this a search with using any sort of search criteria as we'll display a slighlty different message for these searches
     * @returns {DOMelement} - The message DOMElement
     */
    function CreateNoCampaignMessage(isTheAUserSearch) {
        var text = "You have no Cadences currently.";
        if (isTheAUserSearch) {
            text = "You have no Cadences by that name.";
        }

        var messageDiv = CreateElement('div', "ebsta-campaign-emptymessage", null, null);
        messageDiv.appendChild(CreateElement('div', null, text, null));
        messageDiv.appendChild(CreateElement('br', null, null, null));
        messageDiv.appendChild(CreateElement('br', null, null, null));
        messageDiv.appendChild(document.createTextNode("Click "));
        var uri = "";
        if (webUserId != null) {
            uri = "https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=31";
        }
        var linkElement = CreateElement("a", null, 'here', null);
        linkElement.setAttribute("href", uri);
        linkElement.target = '_blank';
        messageDiv.appendChild(linkElement);
        messageDiv.appendChild(document.createTextNode(" to create a Cadence."));

        return messageDiv;
    }

    /**
     * Find all the TO contacts within a compose window.
     *
     * If;
     *
     * - There is only 1 recipient all the details found will be pre-populated into the campaign details screen.
     * - There are multiple recipients then it'll pre-select them all into a multi-recipient campaign.
     *
     * @param {DOMElement} elementClicked - The element clicked within a compose window 
     */
    function PreSelectContactFromComposeWindowIntoCampaign(elementClicked) {
        if (GmailTools) {
            //wipe contact object on new campaign setup
            contactObject.ResetContact();
            contactObjectAll = new Array();
            multiRecipientRowSelectedCounter = 0;
            multiRecipientComposeErrorCounter = 0;

            var composeWindowContacts = GmailTools.Compose.GetContactsFromComposeWindow(elementClicked, [GmailTools.Compose.RecipientAddressTypes.To]);

            for (i = 0; i < composeWindowContacts.length; i++) {
                var localContactObject = composeWindowContacts[i];

                // if this is the first one, add to contactObject object and always add to contactObjectAll Object
                if (contactObjectAll.length === 0) {
                    contactObject = localContactObject;
                }

                contactObjectAll.push(localContactObject);

                if (localContactObject.GetEmailAddress() != null && localContactObject.GetFirstName() != null) {
                    multipleContactArray.push(localContactObject.GetContact());
                    multiRecipientRowSelectedCounter++;
                } else {
                    multiRecipientComposeErrorCounter++;
                }
            }
        }
    }

    /**
     * Find all the CC and BCC contacts within a compose window, storing them for later use
     * @param {DOMElement} elementClicked - The element clicked within a compose window 
     */
    function PreSelectCcAndBccContactsFromComposeWindowIntoCampaign(elementClicked) {
        if (GmailTools) {
            ccComposeRecipients = GmailTools.Compose.GetContactsFromComposeWindow(elementClicked, [GmailTools.Compose.RecipientAddressTypes.Cc]);
            bccComposeRecipients = GmailTools.Compose.GetContactsFromComposeWindow(elementClicked, [GmailTools.Compose.RecipientAddressTypes.Bcc]);
        }
    }

    /**
     * Creates a select dropdown element containing Salesforce Campaigns
     * @param {string} currentRelatedSalesforceCampaignId - The Salesforce Id of the campaign currently related to this Ebsta Campaign
     * @returns {DOMElement|null} - Returns the element if there is a list of campaigns to render, null if not
     */
    function CreateSalesforceCampaignDropdown(currentRelatedSalesforceCampaignId) {
        if (listOfSalesforceCampaigns) {
            var selectElement = CreateElement("select");              

            //The Chosen plugin needs an empty selection for the first option so we can have selecting nothing as an option
            selectElement.appendChild(CreateElement("option", null, name, null));

            for (var i = 0; i < listOfSalesforceCampaigns.length; i++) {
                var name = listOfSalesforceCampaigns[i].Name;
                var id = listOfSalesforceCampaigns[i].Id;
                if (id && name) {
                    var option = CreateElement("option", null, name, null, { value: id });
                    if (id === currentRelatedSalesforceCampaignId) {
                        option.setAttribute("selected", "");
                    }
                    //If the Salesforce campaign in the list is already being related to another Ebsta Campaign then disable the selection
                    else if (listOfSalesforceCampaigns[i].RelatedEbstaCampaignName != null) {
                        option.setAttribute("disabled", "");
                    }
                    selectElement.appendChild(option);
                }
            }
            domElements.DetailsPage.SalesforceCampaignDropDown = selectElement;
            return selectElement;
        }
        return null;
    }

    /**
     * Gets the current value selected in the Related Salesforce Campaign dropdown
     * @returns {string|null} - The value if present or null if the element doesn't exist
     */
    function GetRelatedSalesforceCampaignValue() {
        if (domElements.DetailsPage && domElements.DetailsPage.SalesforceCampaignDropDown) {
            return $(domElements.DetailsPage.SalesforceCampaignDropDown).val();
        }
        return null;
    }

    /**
     * Renders the name of the related Salesforce campaign if an Ebsta Campaign is related to one
     * @param {string} relatedSalesforceCampaignId - The ID of a Salesforce campaign
     * @param {boolean} isUserAllowedToChangeRelation - If user is allowed we'll render a dropdown instead of text so the user can select another Salesforce Campaign
     */
    function PopulateRelatedSalesforceCampaignData(relatedSalesforceCampaignId, isUserAllowedToChangeRelation) {
        //Reset this each time we attemp to populate data so we know we have a fresh dropdown if the user has access to change the campaign
        if (domElements.DetailsPage && domElements.DetailsPage.SalesforceCampaignDropDown) {
            domElements.DetailsPage.SalesforceCampaignDropDown = null;
        }

        const salesforceCampaignField = document.getElementById("sfEbSendCampaignDetailsSalesforceRelationValue");
        if (salesforceCampaignField) {
            if (isUserAllowedToChangeRelation === true) {
                var campaignDropDown = CreateSalesforceCampaignDropdown(relatedSalesforceCampaignId);
                if (campaignDropDown) {
                    AddClass(salesforceCampaignField, "dropdown");
                    salesforceCampaignField.innerHTML = "";
                    salesforceCampaignField.appendChild(campaignDropDown);
                    $(campaignDropDown).chosen({ width: "190px", search_contains: true, allow_single_deselect: true });
                }
                return;
            }

            if (salesforceCampaignField && listOfSalesforceCampaigns) {
                for (let i = 0; i < listOfSalesforceCampaigns.length; i++) {
                    const name = listOfSalesforceCampaigns[i].Name;
                    const id = listOfSalesforceCampaigns[i].Id;
                    if (id && name) {
                        if (id === relatedSalesforceCampaignId) {
                            salesforceCampaignField.innerText = name;
                            salesforceCampaignField.title = name;
                            return;
                        }
                    }
                }
            }
        }
    }

    /**
     * Renders the Campaign's details such as name, creator, no of recipients
     * @param {object} campaignDetailsObject - The entire object containing all details for a campaign
     * @param {string} campaignDetailsObject.CampaignName - The name of the campaign
     * @param {string} campaignDetailsObject.Owner - The owner of the campaign
     * @param {string|int} campaignDetailsObject.Recipients - The total number of recipients
     * @param {string|null} campaignDetailsObject.RelatedSalesforceCampaignId - The id of the Salesforce Campaign that it's related to, if set
     * @param {boolean} campaignDetailsObject.SendFollowUpsAsReplies - Is the campaign set to send emails as replies
     * @param {boolean} campaignDetailsObject.SendWeekdaysOnly - The the campaign set to only send on weekdays
     * @param {string} preSelectedSalesforceCampaign - If we're re-loading a campaign that a user has already selected, pass in the value selected previously for the Related Salesforce Campaign
     */
    function RenderCampaignMetaData(campaignDetailsObject, preSelectedSalesforceCampaign) {
        if (campaignDetailsObject.CampaignName && campaignDetailsObject.CampaignName.length > 0) {
            var nameValueField = document.getElementById("sfEbSendCampaignDetailsCampaignNameValue");
            if (nameValueField) {
                nameValueField.title = campaignDetailsObject.CampaignName;
                nameValueField.innerHTML = campaignDetailsObject.CampaignName;
            }
        }

        if (campaignDetailsObject.Owner && campaignDetailsObject.Owner.length > 0) {
            var ownerValueField = document.getElementById("sfEbSendCampaignDetailsOwnerValue");
            if (ownerValueField) {
                ownerValueField.innerHTML = campaignDetailsObject.Owner;
            }
        }

        if (campaignDetailsObject.Recipients !== null && typeof campaignDetailsObject.Recipients !== 'undefined') {
            var recipientsValueField = document.getElementById("sfEbSendCampaignDetailsRecipientsValue");
            if (recipientsValueField) {
                recipientsValueField.innerHTML = campaignDetailsObject.Recipients;
            }
        }

        var relatedSaleforceCampaignId = campaignDetailsObject.RelatedSalesforceCampaignId;
        //Check to see if we're re-loading a campaign that has already been selected, if so find the previously value that was chosen
        if (preSelectedSalesforceCampaign != null) {
            relatedSaleforceCampaignId = preSelectedSalesforceCampaign;
        }
            
        PopulateRelatedSalesforceCampaignData(relatedSaleforceCampaignId, campaignDetailsObject.IsRequesterAllowedToSetRelatedSalesforceCampaign);

        var sendFollowUpsAsRepliesInput = document.getElementById("ebsta-follupsreplies-checkbox");
        if (sendFollowUpsAsRepliesInput) {
            sendFollowUpsAsRepliesInput.checked = campaignDetailsObject.SendFollowUpsAsReplies;
        }

        var weekdaysonlyInput = document.getElementById("ebsta-weekdaysonly-checkbox");
        if (weekdaysonlyInput) {
            weekdaysonlyInput.checked = campaignDetailsObject.SendWeekdaysOnly;
        }        
    }

    /**
     * Sets up the Email Tracking button to toggle between different states when clicked
     * @param {DOMElement} emailTrackingButton - The email tracking button DOMElement
     * @param {boolean} startingValue - What state the button should be in the first (true = on, false = off)
     */
    function AttachClickListenerToEmailTrackingButton(emailTrackingButton, startingValue) {
        if (emailTrackingButton != null) {

            /**
             * So we don't have to use classes on the element to track the state of the button
             * use a closure which will store the state during the whole of the click event listener's life
             */
            (function (startingValue) {
                emailTrackingButtonState = startingValue;
                UpdateEmailTrackingButton(startingValue);
                emailTrackingButton.addEventListener("click",
                    function() {
                        if (emailTrackingButtonState === true) {
                            ShowEmailTrackingOffWarning(function() {
                                emailTrackingButtonState = !emailTrackingButtonState;
                                UpdateEmailTrackingButton(emailTrackingButtonState);
                            });
                        } else {
                            emailTrackingButtonState = !emailTrackingButtonState;
                            UpdateEmailTrackingButton(emailTrackingButtonState);
                        }
                    });
            })(startingValue);
        }
    }

    /**
     * Shows the user a confirmation dialog when they try to switch the tracking off in campaigns
     * @param {function} confirmCallback - A function to call when Confirm is clicked
     */
    function ShowEmailTrackingOffWarning(confirmCallback) {
        window.GLOBAL_EBSTA_MODAL.SetText(
            "Without tracking, we will not be able to report on the campaign performance. Are you sure you want to disable tracking?");
        window.GLOBAL_EBSTA_MODAL.SetConfirmButtonMode();
        window.GLOBAL_EBSTA_MODAL.HandleConfirmButton(function () {
            if (typeof confirmCallback === 'function') confirmCallback();
        });
        window.GLOBAL_EBSTA_MODAL.ShowModal();
        window.GLOBAL_EBSTA_MODAL.ShowDialog();
    };

    /**
     * Toggles the state of the Email Tracking button 
     * @param {boolean} isTrackingOn - True if the button should show as switched on, false if not
     */
    function UpdateEmailTrackingButton(isTrackingOn) {
        if (domElements.EmailTrackingButton != null) {
            if (isTrackingOn === true) {
                RemoveClass(domElements.EmailTrackingButton, "sfEbstaGmailEmailTrackingNotSelected");
                AddClass(domElements.EmailTrackingButton, "sfEbstaGmailEmailTrackingSelected");
                domElements.EmailTrackingButton.title = "Email Tracking On";
            } else {
                RemoveClass(domElements.EmailTrackingButton, "sfEbstaGmailEmailTrackingSelected");
                AddClass(domElements.EmailTrackingButton, "sfEbstaGmailEmailTrackingNotSelected");
                domElements.EmailTrackingButton.title = "Email Tracking Off";
            }
        }
    }

    /**
    * Gets the label for when a decision has a negative side to it
    * @param {int} decisionInterfaceEnum - The int version of the decision interface
    * @returns {string} - The text to display
    */
    function GetNegativeDecisionTextLabel(decisionInterfaceEnum) {
        switch (decisionInterfaceEnum) {
            case campaignMgrDecisionTaskTypeInterfaceToCallEnum.CheckIfNoOneHasReplied:
                return "If recipient replies";
            case campaignMgrDecisionTaskTypeInterfaceToCallEnum.CheckIfTaskIsClosed:
                return "If Task is not completed in 1 month";
        }

        return null;
    }

    /**
     * Returns the CampaignMgrTaskTypeId of a Task
     * @param {object} task - Should represent a Task within a campaign
     * @returns {string} - The id from the given object
     */
    function GetTaskGuid(task) {
        return task.CampaignMgrTaskTypeId;
    }

    /**
     * Assigns a click event onto the element when a user wishes to edit a email step in the campaign
     * @param {DOMElement} element - The element to assign the click listener too
     */
    function AttachListenerEditTemplate(element) {
        if (element) {
            element.addEventListener('click', function (e) {
                var target = e.target;
                var id = target && target.id;
                if (id && id.length > 0) {
                    var taskContent = target.closest('.taskContent');
                    var taskId = taskContent.dataset.campaigntaskid;
                    var task = lThisCampaignManagerInstance.GetTaskStageById(taskId);
                    var templateEditObj = { subject: task.Subject, campaignId: taskContent.getAttribute('data-campaignid'), taskId: taskId };

                    // if we are in send follow ups as replies mode, set right flag so we do not take the reply subjects over when updating a template
                    if (task.SendFollowUpsAsReplies) {
                        templateEditObj.SendFollowUpsAsReplies = true;
                        templateEditObj.SubjectCore = task.SubjectCore;
                    }

                    //set whether or not this item is disabled or not.
                    templateEditObj.isDisabled = $(this).parent().hasClass("ebsta-stage-disabled");

                    lThisCampaignManagerInstance.GetTemplateById(id, templateEditObj);
                }
                CampaignManagerResizing().ResizeCampaignManager();
            });
        }
    }

    /**
     * Creates the DOMElement when displayinn a task such as an Email or Adding a Task To Salesforce
     * @param {int} interfaceToCall - Related to the 'campaignMgrTaskTypeInterfaceToCallEnum' object to determine what type of Task this is
     * @param {string} templateId - The TemplateId representing the content of the email for this step
     * @returns {DOMElement} - The created DOMElement to render
     */
    function CreateTaskContentSectionGmail(interfaceToCall, templateId) {
        var section = CreateElement("div", campaignClassNames.TaskContent, null, null);
        section.setAttribute(dataAttributes.TaskInterfaceNumber, interfaceToCall);

        if (interfaceToCall === campaignMgrTaskTypeInterfaceToCallEnum.SendingAnEmail) {
            section.classList.add(campaignClassNames.TaskContentEmail);
            section.appendChild(CreateElement("div", campaignClassNames.EmailSubject, null, null));
            var editSpan = CreateElement("div", icons.Pencil, null, templateId);
            AttachListenerEditTemplate(editSpan);
            section.appendChild(editSpan);
        } else if (interfaceToCall === campaignMgrTaskTypeInterfaceToCallEnum.AddingObjectToSalesforce) {
            section.classList.add(campaignClassNames.TaskContentNotify);
            section.appendChild(CreateElement("div", campaignClassNames.SfTaskSubject, null, null));
        }

        return section;
    }

    /**
     * Creates the container which holds the details for the Task Element
     * @param {object} task - An object containing details for the type of Task this step is
     * @param {boolean} isPositiveStep - Is this step a positive step
     * @param {string} templateId - The TemplateId representing the content of the email for this step
     * @returns {DOMElement} - The created DOMElement to render
     */
    function CreateTaskDom(task, isPositiveStep, templateId) {
        var containerDom = CreateElement("div", campaignClassNames.TaskContainer, null, null);

        if (task) {
            containerDom.setAttribute(dataAttributes.CampaignTaskTypeId, GetTaskGuid(task));

            if (isPositiveStep === true) {
                containerDom.setAttribute(dataAttributes.PositiveDecision, "");
            } else if (isPositiveStep === false) {
                containerDom.setAttribute(dataAttributes.NegativeDecision, "");
            }

            containerDom.appendChild(CreateTaskContentSectionGmail(task.InterfaceToCall, templateId));
        }

        return containerDom;
    }

    /**
     * Creates the dropdown for the Multi-Recipient screen where you can choose a Salesforce object to get the list views for
     * @param {Object[]} salesforceObjects - The list of objects
     * @param {string} salesforceObjects.name - The value for the object
     * @param {string} salesforceObjects.label - The display value for the object
     */
    function PopulateMultiRecipientSalesforceObjectTypeDropDown (salesforceObjects) {
        if (salesforceObjects && salesforceObjects.length > 0) {

            const convertedValues = EbstaCustomDropdown.ConvertListOfValuesIntoValuesForCustomDropdown(salesforceObjects, {
                "Value": "name",
                "Text": "label"
            });

            const newDropDown = EbstaCustomDropdown.CreateCustomDropDown("sfEbSendCampaignMultiRecipientsSalesforceObjectType", convertedValues[0],
                convertedValues, function (selectedValueObject) {
                    GetListViewsForObject(selectedValueObject.value);
                });

            GetListViewsForObject(salesforceObjects[0].name);

            if (domElements.MultiRecipientSalesforceObjectDropdown) {
                ReplaceNodeWithNewNode(newDropDown, domElements.MultiRecipientSalesforceObjectDropdown);
                domElements.MultiRecipientSalesforceObjectDropdown = newDropDown;
            }
        } else {
            var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceObjectLoader');
            if (loaderDiv) {
                loaderDiv.innerHTML = 'No Salesforce objects found, please check your Salesforce access.';
            }
        }
    }

    function DisableMultiRecipientSalesforceListviewDropdown() {
        $(domElements.MultiRecipientSalesforceListviewDropdown).prop('disabled', true).trigger("chosen:updated");
    }

    function EnableMultiRecipientSalesforceListviewDropdown() {
        $(domElements.MultiRecipientSalesforceListviewDropdown).prop('disabled', false).trigger("chosen:updated");
    }

    /**
     * Creates the dropdown for the Multi-Recipient screen where you can choose a list view
     * @param {Object[]} salesforceListViews - The list of views
     * @param {string} salesforceObjects.id - The id for the list view
     * @param {string} salesforceObjects.label - The display value for the list view
     */
    function PopulateMultiRecipientSalesforceListViewsDropDown(salesforceListViews) {
        var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceListviewLoader');

        if (salesforceListViews && salesforceListViews.length > 0) {

            if (loaderDiv) {
                loaderDiv.innerHTML = '';
            }

            const convertedValues = EbstaCustomDropdown.ConvertListOfValuesIntoValuesForCustomDropdown(salesforceListViews, {
                "Value": "id",
                "Text": "label"
            });

            //Insert a blank value at the start of the list so that the chosen plugin show the placeholder
            convertedValues.unshift({
                value: null,
                text: null
            });

            const newDropDown = ebstaDomElementCreator.CreateDropDown("chosen-select", "sfEbSendCampaignMultiRecipientsSalesforceListView", convertedValues);

            // If this has already been setup with chosen ensure we destroy it before creating a new one as it will duplicate it
            if ($(domElements.MultiRecipientSalesforceListviewDropdown).data("chosen")){
                $(domElements.MultiRecipientSalesforceListviewDropdown).chosen("destroy");
            }

            ReplaceNodeWithNewNode(newDropDown, domElements.MultiRecipientSalesforceListviewDropdown);
            domElements.MultiRecipientSalesforceListviewDropdown = newDropDown;                

            $(domElements.MultiRecipientSalesforceListviewDropdown).chosen({
                width: "270px",
                search_contains: true,
                placeholder_text_single: "- Select a list view -"
            }).on("change", function () {
                GetPeopleFromListView(this.value, domElements.MultiRecipientSalesforceObjectDropdown.getAttribute("selectedValue"));
            });
            
        } else {
            if (loaderDiv) {
                loaderDiv.innerHTML = 'No List Views Found for salesforce object';
            }
        }
    }

    /**
     * Get salesforce records from a list view
     * @param {string} listviewId - The id of the list view
     * @param {string} objectName - The Salesforce object this list view is for
     */
    function GetPeopleFromListView(listviewId, objectName) {

        // disable list view selector and add loader
        if (domElements.MultiRecipientSalesforceListviewDropdown) {
            DisableMultiRecipientSalesforceListviewDropdown();
        }

        var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceListviewLoader');

        if (loaderDiv) {
            loaderDiv.appendChild(CreateSpinnerLoaderDiv());
        }

        var request = {
            MessageType: "GetPeopleFromListView",
            listViewId: listviewId,
            ObjectName: objectName
        };

        SendMessageToBackGround(request);
    }

    /**
     * Get salesforce list view for object
     * @param {string} objectName - The salesforce object that we want list views for
     */
    function GetListViewsForObject(objectName) {

        // disable list view selector and add loader
        if (domElements.MultiRecipientSalesforceListviewDropdown) {
            DisableMultiRecipientSalesforceListviewDropdown();
        }

        var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceListviewLoader');

        if (loaderDiv) {
            loaderDiv.appendChild(CreateSpinnerLoaderDiv());
        }

        var request = {
            MessageType: "GetListViewsForObject",
            ObjectName: objectName
        };

        SendMessageToBackGround(request);
    }

    /**
     * The mailbox you use to select the campaign from has a TokenType property attached to it
     * this function will map that number to a MailboxType number which we need to set for each stage
     * so it sends using the right engine
     * @param {int} tokenType - The int representing the token type
     * @returns {int} - The int which represents the type of mailbox we should be using to send this campaign with
     */
    function ConvertTokenTypeIntoMailboxType(tokenType) {
        switch (tokenType) {
        case 1:
            return 1;
        case 2:
            return 2;
        default:
            return 1;
        }
    }

    /** Represents the different match types we have for the conditional logic */
    let campaignMgrTaskExtraDecisionLogicMatchTypeEnum = {
        Matches: 1,
        DoesNotMatch: 2,
        Contains: 3,
        DoesNotContain: 4
    };

    /**
     * Creates a friendly name for the CampaignMgrTaskExtraDecisionLogicMatchTypeEnum so we can display back to a user
     * @param {int} value - The CampaignMgrTaskExtraDecisionLogicMatchTypeEnum value
     * @returns {string} - The friendly name for Match Type
     */
    function GetFriendlyNameForCampaignMgrTaskExtraDecisionLogicMatchTypeEnum(value) {
        switch (value) {
            case campaignMgrTaskExtraDecisionLogicMatchTypeEnum.Contains:
                return "Contains";
            case campaignMgrTaskExtraDecisionLogicMatchTypeEnum.DoesNotContain:
                return "Does Not Contain";
            case campaignMgrTaskExtraDecisionLogicMatchTypeEnum.Matches:
                return "Matches";
            case campaignMgrTaskExtraDecisionLogicMatchTypeEnum.DoesNotMatch:
                return "Does Not Match";
        }
    }

   /**
    * Creates text to display when the decision is "If Task is closed" set fo a stage
    * @param {int} timeFrameNumber - The amount of units of time
    * @param {string} timeBracket - The unit type of time i.e. days/weeks etc etc
    * @param {object[]} extraDecisionLogicArray - If there is extra logic set for this then pass them into here
    * @returns {string} - Returns the string to show
    */
    function GetIfTaskClosedDecisionText(timeFrameNumber, timeBracket, extraDecisionLogicArray) {
        let summaryText = "Once Task is completed";

        if (Array.isArray(extraDecisionLogicArray) && extraDecisionLogicArray.length > 0) {
            // We're only taking the first result currently, this will need to be adapted for multiple conditions
            let salesforceField = extraDecisionLogicArray[0].Field;
            let value = extraDecisionLogicArray[0].Value;
            let campaignMgrTaskExtraDecisionLogicMatchType = extraDecisionLogicArray[0].CampaignMgrTaskExtraDecisionLogicMatchType;

            summaryText += " and " + salesforceField
                + " " + GetFriendlyNameForCampaignMgrTaskExtraDecisionLogicMatchTypeEnum(campaignMgrTaskExtraDecisionLogicMatchType).toLowerCase()
                + " " + value;
        }

        summaryText += ", wait " + timeFrameNumber + " " + timeBracket;

        return summaryText;
    }

    /** The type of warning message that needs to be displayed */
    const duplicateWarningMessageType = {
        SingularContact: 1,
        MultipleContact: 2
    }

    /**
     * Fires a HTTP request to check if the selected recipient is currently live on another other campaign
     * @param {string} listOfEmailAddresses - The email address to search or
     * @param {function(boolean):void} continueCallBack - A callback to run whose only parameter is a boolean and should be given 
     * a true if we are good to continue after the dupe check, false if not
     */
    function checkIfSingleRecipientIsLiveOnACampaign(emailAddress, callBack) {
        // If features isn't enable just stop here
        if (enableCampaignDupeCheck !== true) {
            if (typeof callBack === "function") {
                callBack(true);
                return;
            }
        }   

        const request = {
            EmailAddresses: [emailAddress],
            SelectedCampaignId: lastLoadedCampaignId
        };

        SendCampaignRequest(CampaignRequestType.GetLiveCampaignInfoForContact, request, function (liveCampaignContacts) {
            if (liveCampaignContacts && liveCampaignContacts.length > 0) {
                //We should only really receive one back
                const liveCampaignContact = JSON.parse(liveCampaignContacts[0]);
                window.GLOBAL_EBSTA_MODAL.SetTitle("Recipients already in live cadences");  
                
                if (liveCampaignContact.IsLiveOnSelectedCampaign === true) {
                    window.GLOBAL_EBSTA_MODAL.SetOkButtonMode();
                    window.GLOBAL_EBSTA_MODAL.SetText("This recipient is already enrolled in this cadence. They will not be added to this cadence again.");
               
                    if (typeof callBack === "function") {
                        callBack(false);
                    }
                } else {
                    window.GLOBAL_EBSTA_MODAL.SetConfirmButtonMode();
                    window.GLOBAL_EBSTA_MODAL.SetConfirmButtonText("Yes");
                    window.GLOBAL_EBSTA_MODAL.SetCancelButtonText("No");
                    window.GLOBAL_EBSTA_MODAL.SetText("This recipient is in a live Cadence. Do you still want to add them to this cadence?");

                    window.GLOBAL_EBSTA_MODAL.HandleConfirmButton(function () {
                        if (typeof callBack === "function") {
                            callBack(true);
                        }
                    });

                    window.GLOBAL_EBSTA_MODAL.HandleCancelButton(function () {
                        if (typeof callBack === "function") {
                            callBack(false);
                        }
                    });
                }

                window.GLOBAL_EBSTA_MODAL.ShowModal();
                window.GLOBAL_EBSTA_MODAL.ShowDialog();    
            } else {
                if (typeof callBack === "function") {
                    callBack(true);
                }
            }
        });
    }

    /**
     * Fires a HTTP request to check if selected recipients are currently live on a campaign
     * @param {string[]} listOfEmailAddresses - List of email addresses to search for
     * @param {HTMLElement[]} tabSectionElements - Array of multirecipient window tab elements which this search relates 
     * @param {number} messageType - The type of warning to display
     * @param {function} callBack - A function to call when the request has completed
     * @param {boolean} isThisFinalWarning - Whether this is the final warning that a user should see when confirming the list
     */
    function checkIfRecipientsAreLiveOnACampaign(listOfEmailAddresses, tabSectionElements, messageType,
        callBack, isThisFinalWarning = false) {

        // If features isn't enable just stop here or there are no email addresses to query for
        if (enableCampaignDupeCheck !== true || !Array.isArray(listOfEmailAddresses) || listOfEmailAddresses.length == 0) {
            if (typeof callBack === "function") {
                callBack();
                return;
            }
        }
        
        // Wrap this up in a closure so it runs in owns scope
        (function (lListOfEmailAddresses, lTabSectionElements, lCallBack, lIsThisFinalWarning) {

            const uniqueListOfEmailAddresses =
                new Set(lListOfEmailAddresses.map(x => typeof x === 'string' ? x.toLowerCase() : x));

            const request = {
                EmailAddresses: [...uniqueListOfEmailAddresses],
                SelectedCampaignId: lastLoadedCampaignId
            };
        
            SendCampaignRequest(CampaignRequestType.GetLiveCampaignInfoForContact, request, function (liveCampaignContacts) {
                if (liveCampaignContacts && liveCampaignContacts.length > 0) {
                    SetDupeAlertIcon(liveCampaignContacts, lTabSectionElements);
                              
                    // Find the amount of recipients that are currently live on the selected campaign
                    const allRecipientsThatAreLiveOnSelectedCampaign =
                        liveCampaignContacts.filter(x => JSON.parse(x).IsLiveOnSelectedCampaign === true);
                
                    DeSelectRecipients(allRecipientsThatAreLiveOnSelectedCampaign, lTabSectionElements);

                    // Number of Recipients automatically excluded as they're already on this campaign   
                    const noOfRecipientAutomaticallyExcluded = allRecipientsThatAreLiveOnSelectedCampaign.length;

                    // If all the results back are flagged as all being selected on this campaign
                    const areAllFoundDuplicateContactsLiveOnSelectedCampaign = liveCampaignContacts.length == allRecipientsThatAreLiveOnSelectedCampaign.length;

                    if (areAllFoundDuplicateContactsLiveOnSelectedCampaign) {
                        ShowAllRecipientsAreLiveOnCurrentCampaignWarningMessage(noOfRecipientAutomaticallyExcluded, uniqueListOfEmailAddresses.size, messageType, lIsThisFinalWarning); 
                    } else {
                        ShowRecipientAlreadyLiveWarningMessage(liveCampaignContacts.length, allRecipientsThatAreLiveOnSelectedCampaign.length, messageType, function () {
                            DeSelectRecipients(liveCampaignContacts, lTabSectionElements);
                            if (lIsThisFinalWarning === true) {
                                lThisCampaignManagerInstance.SetMultipleRecipientsSelected();
                            }
                        }, function () {
                            if (lIsThisFinalWarning === true) {
                                lThisCampaignManagerInstance.SetMultipleRecipientsSelected();
                            }
                        });
                    }                   
                } else {   
                    if (lIsThisFinalWarning === true) {
                        lThisCampaignManagerInstance.SetMultipleRecipientsSelected();
                    }
                }

                if (typeof callBack === "function") {
                    lCallBack();
                }
            });
        })(listOfEmailAddresses, Array.from(tabSectionElements), callBack, isThisFinalWarning);
    }

    /**
     * Shows a warning to state that all of the recipients checked are live in the selected campaign
     * @param {number} noOfRecipientsExcluded - The number of recipients found to be live in the selected campaigns
     * @param {number} noOfSelectedRecipients - The number of recipients that are currently selected for the campaign
     * @param {number} messageType - The type of warning to display
     * @param {boolean} isThisFinalWarning - Whether this is the final warning that a user should see when confirming the list
    */
    function ShowAllRecipientsAreLiveOnCurrentCampaignWarningMessage(noOfRecipientsExcluded, noOfSelectedRecipients, messageType, IsThisFinalWarning) {
        window.GLOBAL_EBSTA_MODAL.SetTitle("Recipients already in live cadences");
        window.GLOBAL_EBSTA_MODAL.SetOkButtonMode();
        window.GLOBAL_EBSTA_MODAL.ShowModal();

        if (IsThisFinalWarning === true) {            
            // If all recipients have been excluded we need inform the user that they need to select someone who isn't
            if (noOfRecipientsExcluded === noOfSelectedRecipients) {
                window.GLOBAL_EBSTA_MODAL.SetText("All of your recipients are already enrolled on this cadence, " +
                    "please add recipients that are not already on this cadence.");
            } else {                
                window.GLOBAL_EBSTA_MODAL.SetText(getExcludedRecipientsDialogText(noOfRecipientsExcluded, messageType));

                window.GLOBAL_EBSTA_MODAL.HandleOkButton(() => {
                    lThisCampaignManagerInstance.SetMultipleRecipientsSelected();
                });
            }        
        } else {
            window.GLOBAL_EBSTA_MODAL.SetText(getExcludedRecipientsDialogText(noOfRecipientsExcluded, messageType));
        }
        window.GLOBAL_EBSTA_MODAL.ShowDialog();        
    }

    /**
     * Returns the text to render in the dialog for when we automatically exclude recipients from a campaign
     * @param {number} noOfRecipientsExcluded - The number of recipients that are live in the selected campaigns
     * @param {number} messageType - The type of warning to display
     */
    function getExcludedRecipientsDialogText(noOfRecipientsExcluded, messageType) {
        if (messageType === duplicateWarningMessageType.SingularContact) {
            return "This recipient is already enrolled in this cadence. They will be excluded from the list.";
        }

        return `${noOfRecipientsExcluded == 1
            ? "One of your recipients is"
            : noOfRecipientsExcluded + " of your recipients are"} already enrolled in this cadence. They will be excluded from the list.`;
    }

    /**
     * Shows a warning to say recipient(s) are currently live in a campaign and if they wish to deselect these
     * @param {number} noOfRecipientsInLiveCampaigns - The number of recipients that are live in other campaigns
     * @param {number} noOfRecipientsExcluded - The number of recipients that are live in the selected campaigns
     * @param {number} messageType - The type of warning to display
     * @param {Function} yesButtonCallback - The callback to happen if a user does select the "Yes" option
     * @param {Function} noButtonCallBack - The callback to happen if a user does select the "No" option
     */
    function ShowRecipientAlreadyLiveWarningMessage(noOfRecipientsInLiveCampaigns, noOfRecipientsExcluded, messageType,
        yesButtonCallback, noButtonCallBack) {
        window.GLOBAL_EBSTA_MODAL.SetTitle("Recipients already in live cadences");
        window.GLOBAL_EBSTA_MODAL.SetConfirmButtonMode();
        window.GLOBAL_EBSTA_MODAL.SetConfirmButtonText("Yes");
        window.GLOBAL_EBSTA_MODAL.SetCancelButtonText("No");
        window.GLOBAL_EBSTA_MODAL.HandleConfirmButton(yesButtonCallback);
        window.GLOBAL_EBSTA_MODAL.HandleCancelButton(noButtonCallBack);
        window.GLOBAL_EBSTA_MODAL.ShowModal();
        let text = "";

        if (messageType == duplicateWarningMessageType.SingularContact) {
            text = "This recipient is in a different live Cadence. Do you want to exclude them from the list?";
        } else {
            if (noOfRecipientsExcluded > 0) {
                text += getExcludedRecipientsDialogText(noOfRecipientsExcluded) + "\n\n"
            }
        
            const noOfRecipientsInDifferentCampaign = noOfRecipientsInLiveCampaigns - noOfRecipientsExcluded;
            if (noOfRecipientsInDifferentCampaign == 1) {
                text += `One of your recipients is in a different live cadence. Do you want to exclude them from the list?`;
            } else if(noOfRecipientsInDifferentCampaign > 1) {
                text += `${noOfRecipientsInLiveCampaigns - noOfRecipientsExcluded} of your recipients are already in different live cadences. Do you want to exclude them from the list?`;
            }
        }
        window.GLOBAL_EBSTA_MODAL.SetText(text);
        window.GLOBAL_EBSTA_MODAL.ShowDialog();
    }

    /**
     * Loops through all recipients which are found to be active in another campaign and enables the dupe icon so a user
     * can see which recipients are live in another campaign
     * @param {string[]} recipientsToDeSelect - List of recipients to deselect
     * @param {HTMLElement} tabSectionElements - The list of elements we should be setting duplicate icons for
     */
    function SetDupeAlertIcon(recipients, tabSectionElements) {
        tabSectionElements.forEach((tabSectionElement) => {
            const allRecipientEmails = tabSectionElement.getElementsByClassName("sfEbSendCampaignMultiRecipientsEmailCol");

            recipients.forEach(recipient => {
                recipient = JSON.parse(recipient);

                if (!recipient.EmailAddress) return;

                const matchedRows = Array.from(allRecipientEmails)
                    .filter(x => x.value.toUpperCase() === recipient.EmailAddress.toUpperCase());

                matchedRows.forEach(matchedRow => {
                    const dupeAlertCell = matchedRow
                        .closest(".sfEbSendCampaignMultiRecipientsTableContentRow")
                        .querySelector(".sfEbSendCampaignMultiRecipientsTableDupeAlert");

                    if (dupeAlertCell) {
                        AddClass(dupeAlertCell, "enabled");

                        const dupeAlertATag = dupeAlertCell.getElementsByTagName("a")[0];
                        if (dupeAlertATag) {
                            dupeAlertATag.target = "_blank";
                            dupeAlertATag.href = "https://console.ebsta.com/account/campaignmanager/contact.aspx?id=" + recipient.CampaignContactId;
                        }
                    }
                });
            });
        });
    }

    /**
     * Deselects a list of recipients from the list
     * @param {string[]} recipientsToDeSelect - List of recipients to deselect
     * @param {HTMLElement} tabSectionElement - The element of the tab we should be deselecting recipients from
     */
    function DeSelectRecipients(recipientsToDeSelect, tabSectionElements) {
        tabSectionElements.forEach((tabSectionElement) => {
            const allRecipientEmails = tabSectionElement.getElementsByClassName("sfEbSendCampaignMultiRecipientsEmailCol");

            recipientsToDeSelect.forEach(recipientToDeSelect => {
                recipientToDeSelect = JSON.parse(recipientToDeSelect);

                const matchedRows = Array.from(allRecipientEmails)
                    .filter(x => x.value.toUpperCase() == recipientToDeSelect.EmailAddress.toUpperCase());

                matchedRows.forEach(matchedRow => {
                    const checkBoxContainer = matchedRow
                        .closest(".sfEbSendCampaignMultiRecipientsTableContentRow")
                        .querySelector(".sfEbSendCampaignMultiRecipientsCheckBoxContainer");
                
                    const checkBox = checkBoxContainer.getElementsByTagName("input")[0];
                    if (checkBox.checked === true) {
                        checkBox.click();
                    }
                })
            });

            // CHeck if we have any checkboxes checked still, if we don't ensure we uncheck the select all box
            const allCheckboxes = document
                .querySelectorAll('.content-current .sfEbSendCampaignMultiRecipientsTableContent .sfEbSendCampaignMultiRecipientsCheckBoxContainer input:checked')
                .length;
    
            if (allCheckboxes === 0) {
                const currentTab = tabSectionElement.closest('.content-current');

                if (!currentTab) {
                    return
                }

                const selectAllCheckBox = currentTab
                    .closest('.content-current')
                    .querySelector(".sfEbSendCampaignMultiRecipientsTableHeader.sfEbSendCampaignMultiRecipientsCheckBoxContainer input")
        
                if (selectAllCheckBox) {
                    selectAllCheckBox.checked = false;;
                }
            }
        });
    }

    /**
     * Creates a loader type element to help display to the user we're running duplication checks
     * @param {bool} isTheCheckForMultipleRecipients - If the request is checking for multiple recipients, thie helps change the wording in the message
     */
    function createDuplicateLoaderElement(isTheCheckForMultipleRecipients) {
        const loadingMessageContainer = CreateElement("div", "loading");
        loadingMessageContainer.appendChild(CreateElement("div", icons.Spinner, null));

        let text = "Checking recipient's current status in other cadences...";
        if (isTheCheckForMultipleRecipients === true) {
            text = "Checking for recipients already in cadences...";
        }

        loadingMessageContainer.appendChild(CreateElement("span", null, text));        
        return loadingMessageContainer;
    }

    /**
     * Verifys the contacts details put into the form fields to see if they match what we need from them
     * @returns {bool} - True if contact details pass verification, false if not 
     */
    function VerifyContactDetails() {
        var processSelection = true;
    
        //get the email address.
        var emailAddress = "";
        var emailElement = document.getElementById('sfEbSendCampaignDetailsEmailValue');
        if (emailElement != null) {
            if (emailElement.nodeName === 'INPUT') {
                emailAddress = emailElement.value;
            }
            if (emailElement.nodeName === 'DIV') {
                emailAddress = emailElement.innerText;
            }
    
            if (!ValidateEmail(emailAddress.trim())) {
                processSelection = false;
                AddClass(emailElement, "ebError");
            } else {
                RemoveClass(emailElement, "ebError");
            }
        }
    
        //get the first name.
        var firstName = "";
        var firstNameElement = document.getElementById('sfEbSendCampaignDetailsFirstNameValue');
        if (firstNameElement != null) {
            if (firstNameElement.nodeName === 'INPUT') {
                firstName = firstNameElement.value;
            }
            if (firstNameElement.nodeName === 'DIV') {
                firstName = firstNameElement.innerText;
            }
    
            if (!ValidateName(firstName)) {
                processSelection = false;
                AddClass(firstNameElement, "ebError");
            } else {
                RemoveClass(firstNameElement, "ebError");
            }
        }
    
        //get the last name.
        var lastName = "";
        var lastNameElement = document.getElementById('sfEbSendCampaignDetailsLastNameValue');
        if (lastNameElement != null) {
            if (lastNameElement.nodeName === 'INPUT') {
                lastName = lastNameElement.value;
            }
            if (lastNameElement.nodeName === 'DIV') {
                lastName = lastNameElement.innerText;
            }
        }
    
        //get the company name input value.
        var companyName = "";
        var companyNameElement = document.getElementById('sfEbSendCampaignDetailsCompanyNameValue');
        if (companyNameElement != null) {
            if (companyNameElement.nodeName === 'INPUT') {
                companyName = companyNameElement.value;
            }
            if (companyNameElement.nodeName === 'DIV') {
                companyName = companyNameElement.innerText;
            }
        }
    
        if (processSelection) {
    
            // Check if these details are the same as what we already had pre-set, if so carry through the SalesforceId
            const newEmailAddress = emailAddress.trim();
            const newFirstName = firstName.trim();
            const newLastName = lastName.trim();
            const newCompanyName = companyName.trim();
    
            const isContactTheSame = newEmailAddress == contactObject.GetEmailAddress() &&
                newFirstName == contactObject.GetFirstName() &&
                newLastName == contactObject.GetLastName() &&
                newCompanyName == contactObject.GetCompany();
    
            const salesforceId = isContactTheSame ? contactObject.GetSalesforceId() : null;
                    
            contactObject = new Contact(emailAddress.trim(), firstName.trim(), lastName.trim(), companyName.trim(), null, salesforceId);
            return true;
        }
    
        return false;
    }
             
    /**
     * If this campaign is for a single recipient, we'll check we have enough details to send the campaign and attempt to send it. If this 
     * for a multiple recipients it'll send the campaign to the recipients that have been set already.
     */
    function CheckContactAndSendCampaign() {
        if (currentState && currentState.selectedCampaign) {

            if (VerifyContactDetails() || multipleContactSelectedBool) {

                if (!multipleContactSelectedBool) {
                    runSingleRecipientDuplicateCheck(contactObject, sendCampaign);
                } else {
                    // We already check multi-recipients in the multi-recipient screen so we can just go ahead and send the campaign
                    sendCampaign();
                }
            }
        }
    }

    /**
     * Runs a duplication check against the single contact set in the "Sending To" section
     * @param {object} contact - The contact 
     * @param {function} callback - The method to call if there was no duplication match or the user wishes to proceed anyways 
     */
    function runSingleRecipientDuplicateCheck(contact, callback) {
        const loadingMessageContainer = createDuplicateLoaderElement(false);

        const campaignModal = document.getElementById("sfEbSendCampaignMgrMainModel");
        if (campaignModal) {
            campaignModal.appendChild(loadingMessageContainer);
        }

        checkIfSingleRecipientIsLiveOnACampaign(contact.GetEmailAddress(), function (didCheckPass) {
            
            loadingMessageContainer.remove();
            // Did not pass the duplication check so do not continue with sending the campaign
            if (didCheckPass === false) {
                return;
            }

            callback();
        });
    }

    /**
     * Constructs the object containing details around the campaign that needs to be sent, and then sends it.
     */
    function sendCampaign() {
        // add task stages to the selected campaign object
        campaignDetailsSelected = {
            selectedCampaign: currentState.selectedCampaign,
            preSelectedSalesforceCampaign: GetRelatedSalesforceCampaignValue(), //Store the users choice for Related Salesforce Campaign so we can populate the dropdown with this value when reloading back up
            mailbox: selectedGoogleMailbox.details.Mailbox
        };

        var checkedValue = $('#ebsta-weekdaysonly-checkbox').is(':checked');
        campaignDetailsSelected.selectedCampaign.SendWeekdaysOnly = checkedValue;

        // add contact object(s) to the selected campaign object
        if (multipleContactSelectedBool) {
            if (multipleContactArray.length === 0) {
                //Show a dialog stating why they user can't confirm
                window.GLOBAL_EBSTA_MODAL.SetText("Please select at least 1 contact.");
                window.GLOBAL_EBSTA_MODAL.SetOkButtonMode();
                window.GLOBAL_EBSTA_MODAL.ShowModal();
                window.GLOBAL_EBSTA_MODAL.ShowDialog();
                return null;
            }
            campaignDetailsSelected['selectedContact'] = null;
            campaignDetailsSelected['selectedContacts'] = multipleContactArray;
            if (!firstEmailScheduledLater) {
                lThisCampaignManagerInstance.AddExtraDecisionStageAsFirstStage(campaignDetailsSelected.selectedCampaign, new Date());
            }
        } else {
            campaignDetailsSelected['selectedContact'] = contactObject.GetContact();
            campaignDetailsSelected['selectedContacts'] = null;
        }

        if (campaignComposeMode && !multipleContactSelectedBool && isFirstStepAnEmail) {
            //If this is for a single contact then put into the compose window
            lThisCampaignManagerInstance.SelectCampaignForContactComposeWindowActions();
        } else {

            //Set complex Add Values only if this is for a single recipient (multiples not currently supported)
            if (!multipleContactSelectedBool) {
                campaignDetailsSelected['ComplexAddValues'] = lComplexAddClosure.GetComplexAddValues();
            }

            //add a starter stage for send later if user has chosen to send the first email later and  
            if (firstEmailScheduledLater === true) {
                lThisCampaignManagerInstance.AddExtraStageForSendLater(currentState.selectedCampaign, firstEmailScheduledLaterFinalDate);
            }

            //If this is isn't within the compose window then send it straight off
            if (!campaignComposeMode) {
                lThisCampaignManagerInstance.SaveContactToCampaign(campaignDetailsSelected, IsCampaignTracked());
            }
            //If this is a multi recipient and in compose mode, close the compose window
            else if (campaignComposeMode && (multipleContactSelectedBool || !isFirstStepAnEmail)) {

                if (currentState.button != null) {
                    if (!GmailTools.IsThisAExternalGmailComposeScreen()) {
                        var trashIcon = GmailTools.Compose.GetTrashButtonFromComposeWindow(currentState.button);

                        if (trashIcon.length === 1) {
                            TriggerDiscardDraftComposeIcon(trashIcon[0]);
                        }

                        lThisCampaignManagerInstance.SaveContactToCampaign(campaignDetailsSelected, IsCampaignTracked());
                    } else {
                        GmailTools.Compose.InsertOverlay(GmailTools.Compose.GetWholeComposeWindow(currentState.button), "Saving Contacts to Campaign, please wait...");

                        var thisState = lThisCampaignManagerInstance;
                        //Wait a set time just to make sure we get the most up to date draft Id
                        setTimeout(function () {
                            campaignDetailsSelected.DraftId = GmailTools.Compose.GetDraftMessageIdValueFromCompose(currentState.button);
                            thisState.SaveContactToCampaign(campaignDetailsSelected, IsCampaignTracked());
                        }, 3000);

                        closeCampaignMgr();
                    }
                }
            }
        }
    }

    return {
        /**
         * Intilizes the closure
         */
        Initialize: function () {
            lThisCampaignManagerInstance = this;
            Initialize();
        },
        Open: Open,
        SetCloseCampaignFunction: SetCloseCampaignFunction,
        /**
         * Sets the campaign send limit. Default 250 per day.
         * @param {int} dailySendLimit.
         * @returns {void} 
         */
        SetCampaignSendLimit: function (dailySendLimit) {
            if (typeof dailySendLimit !== "undefined" && dailySendLimit !== null && dailySendLimit !== "undefined")
            {
                maximumMultiRecipients = dailySendLimit;
            }            
        },
        /**
         * Finds any modal currently active and removes it from the DOM
         * @returns {} 
         */
        RemoveCampaignManagerModal : function() {
            var modal = document.getElementById("sfEbSendCampaignMgrMainModel");
            if (modal != null) modal.remove();
        },
        /**
         * Sets click events on the buttons within the campaign modal
         * @returns {} 
         */
        SetClickListenersOnActionButtons: function() {
            // select button
            this.AttachEventListenerToCampaignModelSelectButton(document.getElementById("sfEbSendCampaignMgrMainModelSelect"));

            // Send Campaign Button for Single/Multi Recipients mode out of a compose window
            this.AttachEventListenerToCampaignModelSendButton(document.getElementById("sfEbSendCampaignMgrMainModelSend"));

            // close button
            this.AttachEventListenerToCampaignModelCloseButton(document.getElementById("sfEbSendCampaignMgrMainModelClose"));
        },
        /**
         * Response handler for a request sent through "SendMessageToBackGround"
         * @param {object} request - The request object 
         * @param {function} callback - A function can be used to call if anything extra needs to happen with out requets handling
         */
        SendMessageResponseHandlers: function (request, callback) {
            if (request.MessageType) {
                if (request.MessageType === "GetMostRecentlyUsedCampaigns") {
                    this.LoadRecentlyUsedCampaignsHandleResponse(request);
                } else if (request.MessageType === "SearchCampaigns") {
                    this.RunSearchForUserByNameHandleResponse(request);
                } else if (request.MessageType === "GetCampaignDetails") {
                    (function (campaignManagerInstance, request) {

                        //When our callback has returned true below, we can then being to render the details with this callback
                        var campaignDetailsCallBack = function () {
                            campaignManagerInstance.GetCampaignDetailsLoadResults(request);
                        };

                        // Make sure our callback returns true before attempting to render the details
                        // as we're relying on having that complete before rendering
                        if (callback() == true) {
                            campaignDetailsCallBack();
                        } else {
                            // If it doesn't return true first off above then put a retry process in
                            RetryWithCallback(60, 0.5, function () {
                                return callback() === true;
                            }, campaignDetailsCallBack, null);
                        }

                    })(lThisCampaignManagerInstance, request);

                } else if (request.MessageType === "GetTemplateById") {
                    if (request.InjectIntoComposeWindow === true) {
                        this.GetTemplateByIdAndInjectIntoComposeWindowHandleResponse(request);
                    } else {
                        this.GetTemplateByIdHandleResponse(request);
                    }
                } else if (request.MessageType === "MakeEditedCopyOfTemplate") {
                    if (request.SaveContactToCampaign === true) {
                        this.MakeEditedCopyOfTemplateToUseAsRootTemmplateHandleResponse(request);
                    } else {
                        this.MakeEditedCopyOfTemplateHandleResponse(request);
                    }
                } else if (request.MessageType === "SaveContactToCampaign" || request.MessageType === "SaveMultiRecipientsToCampaign") {
                    SaveTaskStagesForContactHandleResponse(request);
                } else if (request.MessageType === "GetCampaignSalesforceListViewObjects") {
                    this.GetCampaignSalesforceListViewObjectsHandleResponse(request);
                } else if (request.MessageType === "GetListViewsForObject") {
                    this.GetListViewsForObjectHandleResponse(request);
                } else if (request.MessageType === "GetPeopleFromListView") {
                    this.GetPeopleFromListViewHandleResponse(request);
                }
            }
        },
        GetIconsObj: function() {
            return icons;
        },
        SetLoadingSelectedCampaign : function(value) {
            loadingSelectedCampaign = value;
        },
        CreateContactFromRow: function (value) {
            return CreateContactFromRow(value);
        },
        /** Loads a campaign when it's previously been selected i.e the compose window */
        LoadAlreadySelectedCampaign: function() {
            loadingSelectedCampaign = true;
            this.ShowCampaignDetailsPage();
            ReplaceCampaignSendFromElement(CreateSendCampaignFromElement());
            this.LoadCampaignDetailsData(campaignDetailsSelected);
        },
        SetSendLaterDatesForSelectedCampaign: function(campaign) {
            if (campaign) {
                var tasks = campaign.ListOfTasks;
                if (tasks && tasks.length > 0) {
                    var i = 0;
                    for (i; i < tasks.length; i++) {
                        if (tasks[i].Live === true && tasks[i].ExecutionStatus === 2) {
                            firstEmailScheduledLater = true;
                            firstEmailScheduledLaterFinalDate = new Date(tasks[i].DecisionExecutionTime);

                        }
                    }
                }
            }
        },
        /**
         * Loads previously selected campaign data into the details view
         * @param {object} campaignDetails - The saved campaign object
         */
        LoadCampaignDetailsData: function(campaignDetails) {

            if (campaignDetails && campaignDetails.selectedCampaign && campaignDetails.selectedContact) {

                var campaignDetailsObject = campaignDetails.selectedCampaign;

                this.SetSendLaterDatesForSelectedCampaign(campaignDetails.selectedCampaign);

                if (firstEmailScheduledLater) {
                    this.ScheduleLaterButton(true);
                } else {
                    this.ScheduleLaterButton(false);
                }

                // save a copy of the selected campaign so we know what we are editing and to what campaign we are adding the contact
                if (currentState) {
                    currentState.selectedCampaign = campaignDetailsObject;
                }

                //Re-set the from mailbox to whatever was selected previously
                if (campaignDetails.mailbox) {
                    var mailboxElement = document.querySelector("#ddCampaignMailbox > ul > li > a[val=\"" + campaignDetails.mailbox + "\"]");
                    if (mailboxElement !== null) {
                        mailboxElement.click();
                    }
                }

                // sending to deatails
                PrePopulateContactsIntoCampaignModal();

                RenderCampaignMetaData(campaignDetailsObject, campaignDetails.preSelectedSalesforceCampaign);

                var sequenceSection = document.getElementById("sfEbSendCampaignSequence");
                if (sequenceSection) {

                    // render sequence in here
                    domElements.CampaignStepsDetailContainer = sequenceSection;
                    // create froala object for managing that in case user wants to edit an email template
                    this.froalaManager = FroalaClosure();
                    this.RenderSequence(campaignDetailsObject.ListOfTasks);
                    this.UpdateSubjectLines();
                }

                this.UpdateComplexAddState();
                this.CheckSendLaterHasBeenScheduledAlready();
                                
                ShouldWeShowSendCampaignButton();
            }
        },
        AttachListenerToScheduleLater: function(element, clickedFromFlowPage) {
            if (element) {
                element.addEventListener('click', function() {
                    clickedThroughToScheduleLaterFromFlowPage = clickedFromFlowPage;
                    lThisCampaignManagerInstance.SetupScheduleLaterPage();
                });
            }
        },
        ScheduleLaterPage: function() {
            this.ResetCampaignPage();
            currentPage.ScheduleLater = true;
            ShowCampaignSelectButton('Schedule');
            this.ChangeCancelButtonText('Cancel', false);
            HideCampaignSendButton();
        },
        SetupScheduleLaterPage: function() {
            this.HideMainModelDetailsDiv();
            this.ScheduleLaterPage();

            var scheduleLaterDiv = document.getElementById('sfEbstaCampaignMgrMainModelSchuleLater');
            if (scheduleLaterDiv) {
                // replace with new date picker and show

                var container = document.getElementById('sfEbstaCampaignMgrDatePickerContainer');
                if (container) {
                    var scheduler = this.GetOrCreateScheduler();
                    container.innerHTML = '';
                    container.appendChild(scheduler);
                }

                this.ShowElement(scheduleLaterDiv);
            } else {
                // create, add date picker and append
                scheduleLaterDiv = CreateElement('div', null, null, 'sfEbstaCampaignMgrMainModelSchuleLater');
                var datePickerContainer = CreateElement('div', null, null, 'sfEbstaCampaignMgrDatePickerContainer');

                var scheduler = this.GetOrCreateScheduler();

                datePickerContainer.appendChild(scheduler);
                scheduleLaterDiv.appendChild(datePickerContainer);

                if (mainCampaignMgrModel) {
                    this.InsertMainViewInModelBeforeButtonsSection(scheduleLaterDiv);
                }
            }
        },
        CreateMinuteDropDown: function(columnDiv) {
            minuteElement = CreateElement("select", "chosen-select", "", "");
            for (var i = 0; i < 60; i++) {
                var minuteText = i < 10 ? "0" + i : i;
                var opt = CreateOptionElement(minuteText, minuteText);
                minuteElement.appendChild(opt);
            }
            columnDiv.appendChild(minuteElement);
        },
        CreateCalendar: function(calendarContainerDiv) {
            calendarElement = new CibulCalendar(calendarContainerDiv, {
                range: false,
                onSelect: function(selected) {
                    lastDateSelected = selected;
                    lThisCampaignManagerInstance.SetLastSelectedDate();
                }
            });
        },
        SetLastSelectedDate: function() {
            this.SetTimeLastDateSelected();
            this.SetTimeAndDateEmailIsScheduledFor(lastDateSelected);
            this.CheckLastDateSelected();
        },
        CheckLastDateSelected: function() {
            if (lastDateSelected != null && lastDateSelected < new Date()) {
                timeSpan.innerText = "";
                timeSpan.appendChild(document.createTextNode("Date is not in the future."));
                timeSpan.classList.remove("ebsta-alert-info");
                timeSpan.classList.add("ebsta-alert-danger");
                HideCampaignSelectButton();
            } else {
                timeSpan.classList.add("ebsta-alert-info");
                timeSpan.classList.remove("ebsta-alert-danger");
                ShowCampaignSelectButton('Schedule');
            }
        },
        GetAmOrPmDropDownValue: function() {
            return $(amPmElement).val();
        },
        GetHourDropDownValue: function() {
            return $(hourElement).val();
        },
        SetLastDateSelected: function (date) {
            lastDateSelected = date;
        },
        SetTimeLastDateSelected: function() {
            var hour = parseInt(this.GetHourDropDownValue());
            var amOrPm = this.GetAmOrPmDropDownValue();
            if (amOrPm === "PM" && hour !== 12) hour += 12;
            else if (amOrPm === "AM" && hour === 12) hour = 0;
            lastDateSelected.setHours(hour);
            lastDateSelected.setMinutes(this.GetMinuteDropDownValue());
        },
        GetMinuteDropDownValue: function() {
            return $(minuteElement).val();
        },
        CreateTimeSpan: function(elementToAppendTo) {
            timeSpan = CreateElement("span", "ebsta-alert ebsta-alert-info sfEbScheduledTime", null, null);
            elementToAppendTo.appendChild(timeSpan);
        },
        SetTimeAndDateEmailIsScheduledFor: function(date) {
            timeSpan.innerText = "";
            timeSpan.appendChild(document.createTextNode(this.GetFriendlyNameForTimeAndDateIsScheduledFor(date)));
        },
        AppendOrdinalIndicatorOntoDay: function(number) {
            var j = number % 10,
                k = number % 100;
            if (j == 1 && k != 11) {
                return number + "st";
            }
            if (j == 2 && k != 12) {
                return number + "nd";
            }
            if (j == 3 && k != 13) {
                return number + "rd";
            }
            return number + "th";
        },
        GetFriendlyNameForTimeAndDateIsScheduledFor(date) {
            var day = this.AppendOrdinalIndicatorOntoDay(date.getDate());
            var month = monthNames[date.getMonth()];
            var time = this.GetFriendlyTimeFormat(date);

            return month + " " + day + " " + time;
        },
        Get12HourFormat: function(hour) {
            if (hour > 11) {
                return "PM";
            } else {
                return "AM";
            }
        },
        GetFriendlyHourFormat: function(hour) {
            return (hour + 24) % 12 || 12;
        },
        GetFriendlyMinuteFormat: function(minutes) {
            if (minutes < 10) minutes = "0" + minutes;
            return minutes;
        },
        GetFriendlyTimeFormat: function(date) {
            var amOrPm = this.Get12HourFormat(date.getHours());
            var hr = this.GetFriendlyHourFormat(date.getHours());
            var minutes = this.GetFriendlyMinuteFormat(date.getMinutes());

            return hr + ":" + minutes + " " + amOrPm;
        },
        CreateTimeDropDowns: function(columnDiv) {
            this.CreateHourDropDown(columnDiv);
            this.CreateMinuteDropDown(columnDiv);
            this.CreateAMPMDropDown(columnDiv);
        },
        CreateAMPMDropDown: function(columnDiv) {
            amPmElement = CreateElement("select", "chosen-select", "", "");
            var amOption = CreateOptionElement("AM", "AM");
            var pmOption = CreateOptionElement("PM", "PM");
            amPmElement.appendChild(amOption);
            amPmElement.appendChild(pmOption);

            columnDiv.appendChild(amPmElement);
        },
        CreateHourDropDown: function(columnDiv) {
            hourElement = CreateElement("select", "chosen-select", "", "");
            for (var i = 1; i < 13; i++) {
                var opt = CreateOptionElement(i, i);
                hourElement.appendChild(opt);
            }
            columnDiv.appendChild(hourElement);
        },
        SetHourDropDown: function(value) {
            hourElement.value = value;
        },
        SetMinuteDropDown: function(value) {
            minuteElement.value = value;
        },
        SetAmOrPmDropDown: function(value) {
            amPmElement.value = value;
        },
        SetTimeDateDropDownFieldsToDate: function(date) {
            var hour = date.getHours();
            this.SetHourDropDown(this.GetFriendlyHourFormat(hour));
            this.SetMinuteDropDown(this.GetFriendlyMinuteFormat(date.getMinutes()));
            this.SetAmOrPmDropDown(this.Get12HourFormat(hour));
            $(chosenFieldElements).trigger("chosen:updated");
        },
        SetDateOnCalendar: function(date) {
            calendarElement.setSelected({ begin: date, end: date });
        },
        GetOrCreateScheduler: function() {

            if (mainModelScheduler) {
                // clear time slected date and time and return 

                this.SetDateOnCalendar(lastDateSelected);
                this.SetTimeAndDateEmailIsScheduledFor(lastDateSelected);
                this.SetTimeDateDropDownFieldsToDate(lastDateSelected);
                this.CheckLastDateSelected();

                return mainModelScheduler;
            } else {
                //create

                var mainDiv = CreateElement("div", "", "", "sfEbCampaignMgrSendLaterSchedule");

                var firstRow = CreateBoostrapRow(null);

                //render the calendar on the left column
                var leftColumnDiv = CreateElement("div", "calendar-canvas col-xs-5", "", "sfEbstaCampaignMgrCalendarSchedule");
                this.CreateCalendar(leftColumnDiv);
                this.SetDateOnCalendar(lastDateSelected);
                firstRow.appendChild(leftColumnDiv);

                //Right column
                var rightColumnDiv = CreateElement("div", "col-xs-5", "", "sfEbScheduleRightCol");

                //Time drop downs
                var rightColumnFirstRow = CreateBoostrapRow(null);
                rightColumnDiv.appendChild(rightColumnFirstRow);
                firstRow.appendChild(rightColumnDiv);

                this.CreateTimeDropDowns(rightColumnFirstRow);
                this.SetTimeDateDropDownFieldsToDate(lastDateSelected);

                //Text details on when email will be sent
                var rightColumnSecondRow = CreateBoostrapRow("ebScheduleTextDetails");
                var scheduledTimeDiv = CreateElement("div", "sfEbScheduledTimeText", "Email will be sent at;", null);
                rightColumnSecondRow.appendChild(scheduledTimeDiv);
                rightColumnDiv.appendChild(rightColumnSecondRow);

                this.CreateTimeSpan(rightColumnSecondRow);
                this.SetTimeAndDateEmailIsScheduledFor(lastDateSelected);

                mainDiv.appendChild(firstRow);

                mainModelScheduler = mainDiv;
                chosenFieldElements = $(mainModelScheduler).find(".chosen-select");
                $(chosenFieldElements).chosen({ width: '30%' }).change(function() {
                    lThisCampaignManagerInstance.SetLastSelectedDate();
                });
                return mainModelScheduler;
            }
        },
        SetTimeAndDateEmailIsScheduledForNew: function() {
            this.SetTimeAndDateEmailIsScheduledFor(new Date());
        },
        ChooseAndExecuteCloseAction: function() {
            if (currentPage.SelectCampaign) {
                closeCampaignMgr();
            }
            if (currentPage.CampaignDetail) {
                HideCampaignSendButton();
                if (clickedFromMainModel) {
                    if (loadingSelectedCampaign === true) {
                        closeCampaignMgr();
                    } else {
                        this.HideMainModelDetailsDiv();
                        this.SHowMainModelContentDiv();
                        this.SelectCampaignPage();
                    }
                } else {
                    closeCampaignMgr();
                }
                firstEmailScheduledLater = false;
                firstEmailScheduledLaterFinalDate = null;
            }
            if (currentPage.EditEmailTemplate) {

                this.HideMainModelEmailEditor();
                this.ShowMainModelSequenceContainer();
                this.CampaignDetailsPage();
            }
            if (currentPage.ScheduleLater) {
                this.HideMainModelSendLaterScheduler();
                this.ShowMainModelDetailsDiv();
                if (clickedThroughToScheduleLaterFromFlowPage === true) {
                    this.CampaignDetailsPage();
                } else {
                    this.EditEmailTemplatePage();
                }
            }

            CampaignManagerResizing().ResizeCampaignManager();
        },
        ScheduleLaterButton: function(scheduled) {

            if (scheduled) {
                var scheuledLaterElement = document.getElementById('sfEbStaCampaignEmailScheduleLaterText');
                if (scheuledLaterElement) {
                    scheuledLaterElement.innerHTML = this.FormatSendLaterDate(firstEmailScheduledLaterFinalDate);
                    var calendarIcon = document.getElementById('sfEbStaCampaignEmailScheduleLaterIcon');
                    if (calendarIcon) {
                        calendarIcon.className = icons.CalendarScheduled;
                    }
                }
            }

            var scheuledLaterElementInFlowPage = document.getElementById('sfEbStaCampaignEmailScheduleLaterFromFlowPage');

            if (scheuledLaterElementInFlowPage) {
                if (scheduled) {
                    AddClass(scheuledLaterElementInFlowPage, "ebsta-btn-success");
                    scheuledLaterElementInFlowPage.title = "Email scheduler to be sent at " + this.FormatSendLaterDate(firstEmailScheduledLaterFinalDate);
                } else {
                    RemoveClass(scheuledLaterElementInFlowPage, "ebsta-btn-success");
                    scheuledLaterElementInFlowPage.title = "Click to Schedule email to be sent at a later date.";
                }
            }
        },
        FormatSendLaterDate: function(date) {
            if (date) {

                return date.getDate() + " " + monthNames[date.getMonth()] + " " + date.getFullYear() + " " + date.getHours() + ":" + this.GetFriendlyMinuteFormat(date.getMinutes());
            }
        },
        HideMainModelSendLaterScheduler: function() {
            var view = document.getElementById('sfEbstaCampaignMgrMainModelSchuleLater');
            if (view) {
                view.style.display = 'none';
            }
        },
        AddExtraStageForSendLater: function (campaign, sendLaterDate) {

            var processExtraStage = true;
            if (campaign) {
                // we are editing an already selected campaign
                var taskLists = campaign.ListOfTasks;

                if (taskLists && taskLists.length > 0) {
                    var i = 0;
                    for (i; i < taskLists.length; i++) {
                        if (taskLists[i].Live === true && taskLists[i].ExecutionStatus === 2) {
                            if (sendLaterDate !== null && typeof sendLaterDate === 'object') {
                                var sortedDate = this.CopyAndParseDateAndFixSeconds(sendLaterDate);
                                taskLists[i].DecisionExecutionTime = sortedDate.toISOString();
                                processExtraStage = false;
                                break;
                            }
                        }
                    }
                }
            }

            if (processExtraStage) {
                this.AddExtraDecisionStageAsFirstStage(campaign, sendLaterDate);
            }
        },
        /**
         * Adds an extra decision stage as the first stage so that it triggers the campaign to start/send at this point in time
         * @param {} campaign 
         * @param {} date 
         * @returns {} 
         */
        AddExtraDecisionStageAsFirstStage: function(campaign, startDate) {
            var taskLists = campaign.ListOfTasks;
            if (taskLists && taskLists.length > 0) {
                var i = 0;
                for (i; i < taskLists.length; i++) {
                    if (taskLists[i].RootTask === true) {
                        var decisionTaskTypeId = this.GetADesisionTaskTypeByTaskTypeAndDecisionTaskTypeInterFace(campaignMgrTaskTypeInterfaceToCallEnum.SendingAnEmail, campaignMgrDecisionTaskTypeInterfaceToCallEnum.AlwaysTrue);

                        if (startDate !== null && typeof startDate === 'object' && decisionTaskTypeId && decisionTaskTypeId.length > 0) {

                            var sortedDate = this.CopyAndParseDateAndFixSeconds(startDate);

                            var trigerredStage = {
                                CampaignMgrId: campaign.CampaignMgrId,
                                DecisionExecutionTime: sortedDate.toISOString(),
                                Live: true,
                                ExecutionStatus: 2,
                                CampaignMgrDecisionTaskTypeId: decisionTaskTypeId,
                                NextStagePositive: [taskLists[i].CampaignMgrTaskId]
                            }

                            taskLists[i].Live = false;
                            taskLists.push(trigerredStage);
                        }

                        break;
                    }
                }
            }
        },
        GetADesisionTaskTypeByTaskTypeAndDecisionTaskTypeInterFace: function(taskTypeInterface, decisionTaskTypeInterface) {
            var tasks = this.GetTasksWithDecisions();

            if (tasks && tasks.length > 0) {
                var i = 0;
                for (i; i < tasks.length; i++) {
                    if (tasks[i].CampaignManagerTask != null) {
                        if (tasks[i].CampaignManagerTask.InterfaceToCall != null) {
                            if (tasks[i].CampaignManagerTask.InterfaceToCall === taskTypeInterface) {
                                var associatedDecisions = tasks[i].AssociatedDecisions;
                                if (associatedDecisions && associatedDecisions.length > 0) {
                                    var n = 0;
                                    for (n; n < associatedDecisions.length; n++) {
                                        if (associatedDecisions[n].InterfaceToCall === decisionTaskTypeInterface) {
                                            return associatedDecisions[n].CampaignMgrDecisionTaskTypeId;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            return null;
        },
        /**
         * Actions to be fired at the end of the SelectCampaignForContact which updates the compose window elements 
         * to reflect what campaign the user has chosen
         * @param {} selectedCampaignObj 
         * @returns {} 
         */
        SelectCampaignForContactComposeWindowActions: function() {
            //Update button in compose window to reflect the selected state of the Campaign
            AddClass(domElements.ComposeWindowButton, "ebstaCampaignSelected");

            domElements.LoadingOverlay = GmailTools.Compose.InsertOverlay(GmailTools.Compose.GetWholeComposeWindow(currentState.button), "Loading first Email, please wait...");

            //Update compose window with email tracking selection
            lEbstaComposeButtonsClosure.UpdateEmailTrackingButtonWithState(IsCampaignTracked());

            //Update Send Later details for the Send Later button within the compose window
            if (firstEmailScheduledLater) {
                var sendLaterInstance = lEbstaComposeButtonsClosure.GetSendLaterClosure();
                sendLaterInstance.ScheduleButtonClick(lastDateSelected);
            }

            //Update Complex Add details
            lEbstaComposeButtonsClosure.SetComplexAddValues(lComplexAddClosure);
            lEbstaComposeButtonsClosure.UpdateSendEmailButton();

            //move the buttons tool bar up if we get a scroller
            MoveGmailComposeScreenIcons();

            if (contactObject.GetEmailAddress()) {
                // inject contacts into compose window if we need to
                this.InjectContactIntoRelativeComposeWindow(currentState.button, contactObject);
            }

            // get the template used by the root task and inject it into the compose window
            this.InjectRootTemplateIntoRelativeComposeWindow(campaignDetailsSelected.selectedCampaign);

            closeCampaignMgr();
        },
        InjectContactIntoRelativeComposeWindow: function(button, contactObj) {

            if (button && contactObj) {
                var outerTable = button.closest('.aoP');
                if (outerTable) {

                    var contactContainer = outerTable.getElementsByClassName('wO nr');
                    if (contactContainer && contactContainer.length > 0) {

                        var textArea = contactContainer[0].getElementsByClassName('vO');;
                        if (textArea && textArea.length === 1) {


                            var contactDiv = CreateElement('div', 'vR', null, null);
                            contactDiv.title = 'Please click the Ebsta Cadence button below to change contact';

                            var contactSpan = CreateElement('span', 'vN bfK a3q', null, null);
                            contactSpan.setAttribute('email', contactObj.GetEmailAddress());
                            var name = contactObj.GetEmailAddress();
                            if (contactObj.GetFullName()) {
                                name = contactObj.GetFullName();
                            }
                            contactSpan.appendChild(CreateElement('div', 'vT', name, null));

                            contactDiv.appendChild(contactSpan);
                            var input = CreateElement('input', null, null, null);
                            input.name = 'to';
                            input.type = 'hidden';
                            input.value = name + ' <' + contactObj.GetEmailAddress() + '>';

                            contactDiv.appendChild(input);

                            //get rid of what is currently in the contact window and add ours
                            this.RemoveContactsFromComposeWindow(contactContainer[0]);

                            contactContainer[0].insertBefore(contactDiv, textArea[0]);
                            textArea[0].style.display = 'none';

                            // insert the name of the contact into separate row which google show when focus is on the email body
                            this.InsertContactIntoComposeWindowPreviewRow(contactObj, outerTable);
                        }
                    }
                }
            }
        },
        // render contact information into a preview row in the compose window
        InsertContactIntoComposeWindowPreviewRow: function(contactObj, outerTable) {

            //Create a element replicating what gmail do when you put a new contact into a compose window
            var gmailContact = CreateElement('span', null, contactObj.GetFullName(), null);
            gmailContact.title = 'Please click the Ebsta Cadence button below to change contact';
            gmailContact.setAttribute('email', contactObj.GetEmailAddress());

            //Find if the "To" element within the compose window to append our own contact element
            var ebstaContactPreviewDiv = outerTable.getElementsByClassName('ebsta-ContactPreviewDiv');

            if (ebstaContactPreviewDiv && ebstaContactPreviewDiv.length > 0) {
                ebstaContactPreviewDiv[0].innerHTML = '';
                ebstaContactPreviewDiv[0].appendChild(gmailContact);
            } else {
                //Find if the "To" element within the compose window to append our own contact element
                var contactPreviewDiv = outerTable.getElementsByClassName('oL aDm');

                if (contactPreviewDiv && contactPreviewDiv.length === 1) {
                    var contactPreviewDivCopy = contactPreviewDiv[0].cloneNode(false);
                    contactPreviewDivCopy.appendChild(gmailContact);
                    contactPreviewDivCopy.classList.add('ebsta-ContactPreviewDiv');
                    contactPreviewDiv[0].parentNode.insertBefore(contactPreviewDivCopy, contactPreviewDiv[0].nextSibling);
                    contactPreviewDiv[0].style.display = 'none';
                }
            }
        },
        RemoveContactsFromComposeWindow: function(containerElement) {
            if (containerElement) {
                var contactsExisting = containerElement.getElementsByClassName('vR');

                while (contactsExisting[0]) {
                    contactsExisting[0].parentNode.removeChild(contactsExisting[0]);
                }
            }
        },
        AttachCampaignSendEventToSendButton: function(element, uniqueId) {
            if (element) {
                element.addEventListener('click', function() {
                    lThisCampaignManagerInstance.SaveContactToCampaignComposeMode(uniqueId);
                });
            }
        },
        /**
         * Saves a Contact to a campaign using the Compose Window mode
         * @param {} sendLaterDate - Date send later has been scheduled for (if it has)
         * @param {} trackEmail - Has email tracking been set for this
         * @param {} complexAddValues - Has complex add been set for this
         * @returns {} 
         */
        SaveContactToCampaignComposeMode: function(sendLaterDate, trackEmail, complexAddValues) {

            var saveObject = campaignDetailsSelected;

            if (saveObject && saveObject.selectedCampaign && saveObject.selectedContact && saveObject.rootTemplate) {

                var composeContainerDiv = GmailTools.Compose.GetWholeComposeWindow(domElements.ComposeWindowButton);

                if (composeContainerDiv) {

                    var editableDiv = composeContainerDiv.getElementsByClassName('editable');
                    var subjectBox = composeContainerDiv.getElementsByClassName('aoT');

                    if (editableDiv && editableDiv.length === 1) {

                        var emailContent = editableDiv[0].innerHTML;
                        var emailSubject = '';

                        if (subjectBox && subjectBox.length === 1 && subjectBox[0].name === 'subjectbox') {
                            emailSubject = subjectBox[0].value;
                        }

                        // replace the body and subject of the root template, make a copy and use that one
                        saveObject.rootTemplate.BodyTemplate = emailContent;

                        if (emailSubject.length > 0) {
                            saveObject.rootTemplate.EmailSubject = emailSubject;
                        }

                        //Get trash button so we can trigger the click event to close the draft window after saving the campaign
                        var trashButton = GmailTools.Compose.GetTrashButtonFromComposeWindow(composeContainerDiv);

                        if (trashButton.length > 0) {

                            //add over lay to the compose window while we save
                            GmailTools.Compose.InsertOverlay(composeContainerDiv, "Saving Contact to Campaign, please wait...");

                            // Has the email being selected as being tracked if so marked all stages to be tracked
                            if (trackEmail) {
                                this.FlagAllStagesToBeTracked(saveObject.selectedCampaign);
                            }

                            //Set Complex Add Values
                            if (complexAddValues != null) {
                                campaignDetailsSelected['ComplexAddValues'] = complexAddValues;
                            }

                            //Set Sent Later
                            //add a starter stage for send later if user has chosen to send the first email later and  
                            if (sendLaterDate != null) {
                                this.AddExtraStageForSendLater(campaignDetailsSelected.selectedCampaign, sendLaterDate);
                            }

                            // Only trigger the trash icon click when not in an external screen as triggering the 
                            // trash icon out of gmail will cause a javascript alert to appear which we can't handle
                            if (!GmailTools.IsThisAExternalGmailComposeScreen()) {
                                TriggerDiscardDraftComposeIcon(trashButton[0]);
                                lThisCampaignManagerInstance.MakeEditedCopyOfTemplateToUseAsRootTemmplate(saveObject.rootTemplate, saveObject, null);
                            } else {
                                // The new Gmail UI doesn't update the draft id on a compose anymore so we need to detect this ensure we don't
                                // wait for them to update the draft to get the latest one so we can delete it
                                var currentDraftId = GmailTools.Compose.GetDraftMessageIdValueFromCompose(composeContainerDiv);

                                if (!GmailTools.IsMessageIdForNewUi(currentDraftId)) {
                                    var draftElement = GmailTools.Compose.FindDraftIdElement(composeContainerDiv);
                                    if (draftElement) {
                                        //Trigger a change on the compose DOM so we always fire an event which causes Gmail to update the draft id
                                        GmailTools.Compose.TriggerChangeOnCompose(composeContainerDiv);

                                        //The callback for when we detect the change in the draft id for the compose window
                                        var mutationObserverCallback = function(mutations, mutationObserver) {
                                            lThisCampaignManagerInstance.MakeEditedCopyOfTemplateToUseAsRootTemmplate(saveObject.rootTemplate, saveObject,
                                                GmailTools.Compose.GetDraftMessageIdValueFromCompose(composeContainerDiv));

                                            //once we've detected one bunch of changes disconnect the obeserver as this should be enough currently.
                                            mutationObserver.disconnect();
                                        }

                                        // Attach a mutation obeserver onto the Draft element within the compose window so 
                                        // we can keep track of any draft id changes
                                        // TODO: MUTATION
                                        var mutationObserver = new MutationObserver(mutationObserverCallback);
                                        var config = { attributes: true, childList: true };
                                        mutationObserver.observe(draftElement, config);
                                    }
                                } else {
                                    //if we don't find the draft element we'll just go and save with the ID we can find right now
                                    lThisCampaignManagerInstance.MakeEditedCopyOfTemplateToUseAsRootTemmplate(saveObject.rootTemplate,
                                        saveObject, GmailTools.ConvertMessageIdIntoFriendlyId(currentDraftId));
                                }
                            }
                        }
                    }
                }
            }
        },
        /**
         * Saves a contact to a campaign not using the Compose Mode
         * @param {} uniqueId 
         * @returns {} 
         */
        SaveContactToCampaign: function (selectedCampaignObj, trackEmails) {

            if (selectedCampaignObj && selectedCampaignObj.selectedCampaign) {
                //replace the template id against the root task in selected campaign in SaveObject and save contact to campaign
                if (selectedCampaignObj.selectedCampaign.ListOfTasks && selectedCampaignObj.selectedCampaign.ListOfTasks.length > 0) {

                    var i = 0;
                    for (i; i < selectedCampaignObj.selectedCampaign.ListOfTasks.length; i++) {
                        //Have we set complex add information here if so update the stage
                        if ((selectedCampaignObj.ComplexAddValues != null
                            && (selectedCampaignObj.ComplexAddValues.ComplexAddWhoAndWhatIds.length > 0 || selectedCampaignObj.ComplexAddValues.ComplexAddAdditionalFields.length > 0)))
                        {
                            selectedCampaignObj.selectedCampaign.ListOfTasks[i].ComplexAddValues = selectedCampaignObj.ComplexAddValues;
                            selectedCampaignObj.selectedCampaign.ListOfTasks[i].AddToSalesforce = true;
                        }

                        selectedCampaignObj.selectedCampaign.ListOfTasks[i].IsEmailTracked = trackEmails;

                        //Remove the body from the task as we don't need it
                        selectedCampaignObj.selectedCampaign.ListOfTasks[i].BodyOfText = null;
                    }
                }

                if (selectedCampaignObj.selectedContact) {
                    // save single contact to campaign
                    this.SaveTaskStagesForContact(selectedCampaignObj, null);
                } else if (selectedCampaignObj.selectedContacts) {
                    //save multiple contacts
                    this.SaveTaskStagesForMultiRecipients(selectedCampaignObj);
                }
                closeCampaignMgr();
            }
        },
        FlagAllStagesToBeTracked: function(selectedCampaign) {
            var tasks = selectedCampaign.ListOfTasks;
            if (tasks && tasks.length > 0) {
                var i = 0;
                for (i; i < tasks.length; i++) {
                    tasks[i].IsEmailTracked = true;
                }
            }
        },
        HideGmailButton: function(button) {
            if (button) {
                var parent = button.closest('.n1tfz');

                if (typeof parent == "undefined" || parent == null || parent.length < 1) {
                    parent = button.closest('.btC');
                }

                if (parent) {
                    var gmailButton = parent.getElementsByClassName("Up");
                    if (gmailButton.length > 0) {
                        gmailButton[0].style.display = 'none';
                    }
                }
            }
        },
        InjectRootTemplateIntoRelativeComposeWindow: function(selectedCampaign) {
            if (selectedCampaign) {
                var tasks = selectedCampaign.ListOfTasks;
                if (tasks && tasks.length > 0) {
                    var i = 0;
                    for (i; i < tasks.length; i++) {
                        if (tasks[i].RootTask === true) {
                            this.GetTemplateByIdAndInjectIntoComposeWindow(tasks[i].CampaignTemplateId);
                            break;
                        }
                    }
                }
            }
        },
        ChooseAndExecuteSendAction: function() {
            if (currentPage.CampaignDetail) {
                CheckContactAndSendCampaign();
            }
        },
        SaveEmailTemplate : function() {
            this.froalaManager.DisableHtmlEditor();

            var taskEditObj = this.froalaManager.GetTaskEditObj();

            var htmlBody = this.froalaManager.GetHtmlBody();
            var emailSubject = this.froalaManager.GetSubject();

            if (htmlBody && htmlBody.length > 0 && emailSubject && emailSubject.length > 0) {
                var template = this.froalaManager.GetTemplate();

                // edit email body and subject
                template.BodyTemplate = htmlBody;
                template.EmailSubject = emailSubject;

                this.MakeEditedCopyOfTemplate(template, taskEditObj);
            }
        },
        ChooseAndExecuteSelectAction: function () {
            if (currentPage.EditEmailTemplate) {
                if (this.froalaManager) {
                    this.SaveEmailTemplate();
                }
            }
            if (currentPage.CampaignDetail) {
                CheckContactAndSendCampaign();
                ComplexAdd().CloseComplexAddScreen();
            }
            if (currentPage.ScheduleLater) {
                this.SetAppropriateSendLaterFlags(lastDateSelected);
                this.ScheduleLaterButton(true);
                this.HideMainModelSendLaterScheduler();
                this.ShowMainModelDetailsDiv();

                if (clickedThroughToScheduleLaterFromFlowPage === true) {
                    this.CampaignDetailsPage();
                } else {
                    this.EditEmailTemplatePage();
                    if (campaignComposeMode) {
                        ShowCampaignSelectButton('Use');
                    }
                }
            }
            CampaignManagerResizing().ResizeCampaignManager();
        },
        IsFirstEmailScheduledForLater: function() {
            return firstEmailScheduledLater;
        },
        GetFirstEmailScheduleDate: function() {

            return firstEmailScheduledLaterFinalDate;
        },
        SetAppropriateSendLaterFlags: function(date) {

            // to use as indicator whether we need to add an extra stage when adding contact to campaign
            firstEmailScheduledLater = true;
            firstEmailScheduledLaterFinalDate = date;

        },
        CopyAndParseDateAndFixSeconds: function(scheduledDate) {
            //var newDate = new Date(scheduledDate.getTime());
            var newDate = new Date(scheduledDate.getTime());
            newDate.setSeconds(0);
            return newDate;
        },
        ShowMainModelSequenceContainer: function() {
            var emailEditView = document.getElementById('sfEbSendCampaignSequenceContainer');
            if (emailEditView) {
                emailEditView.style.display = 'block';
            }
        },
        HideMainModelEmailEditor: function() {
            var emailEditView = document.getElementById('sfEbSendCampaignEmailEditorView');
            if (emailEditView) {
                emailEditView.style.display = 'none';
            }
        },
        UpdateTemplateEditIconElement: function (oldTemplateId, newTemplateId, newSubject) {

            var updatedTemplate = document.getElementById(oldTemplateId);
            if (updatedTemplate) {
                //update the edit icon.
                updatedTemplate.className = icons.PencilSquare;
                updatedTemplate.id = newTemplateId;

                //update the subject.
                $(updatedTemplate).parent().find(".emailSubject").text(newSubject);
                //update suject data attributes.
                $(updatedTemplate).parent().attr("data-campaignsubject", newSubject);
            }
        },
        // update temlate id of the task user has just edited against the selected campaign
        UpdateTemplateIdAgainstSelectedCampaign: function(campaignTemplateId, taskEditObj, newSubject) {


            if (currentState && currentState.selectedCampaign && campaignTemplateId && campaignTemplateId.length > 0 && taskEditObj) {
                if (currentState.selectedCampaign.CampaignMgrId === taskEditObj.campaignId) {
                    var taskList = currentState.selectedCampaign.ListOfTasks;
                    if (taskList && taskList.length > 0) {
                        var i = 0;
                        for (i; i < taskList.length; i++) {
                            if (taskList[i].CampaignMgrTaskId === taskEditObj.taskId) {

                                taskList[i].CampaignTemplateId = campaignTemplateId;

                                taskList[i].Subject = newSubject;
                                taskList[i].SubjectCore = newSubject;
                            }
                        }
                    }
                }
            }
        },
        MakeEditedCopyOfTemplateHandleResponse: function (response) {

            if (response && response.Success) {

                if (response.Values && response.Values.length > 0 && response.TaskEditObj) {

                    var newTemplate = JSON.parse(response.Values[0]);

                    var newTemplateId = newTemplate.CampaignTemplateId;
                    if (newTemplate && newTemplateId && newTemplateId.length > 0) {
                        // change the template id against the task user is updating, flag it as edited and get back to sequence view page
                        this.UpdateTemplateIdAgainstSelectedCampaign(newTemplateId, response.TaskEditObj, newTemplate.EmailSubject);

                        var oldTemplateId = response.OldTemplateId;
                        if (oldTemplateId && oldTemplateId.length > 0) {
                            //update the edit icon.
                            this.UpdateTemplateEditIconElement(oldTemplateId, newTemplateId, newTemplate.EmailSubject);

                            this.HideMainModelEmailEditor();
                            this.ShowMainModelSequenceContainer();
                            this.CampaignDetailsPage();
                        }
                    }
                }
            }
        },
        // edit template used by task
        MakeEditedCopyOfTemplate: function(template, taskEditObj) {

            if (template && taskEditObj) {

                var request = {
                    MessageType: "MakeEditedCopyOfTemplate",
                    Template: template,
                    TaskEditObj: taskEditObj,
                    SaveContactToCampaign: false
                };

                SendMessageToBackGround(request);
            }
        },
        MakeEditedCopyOfTemplateToUseAsRootTemmplateHandleResponse: function(response) {

            if (response && response.Success) {

                if (response.Values && response.Values.length > 0 && response.SaveObject) {
                    var newTemplate = JSON.parse(response.Values[0]);

                    var newTemplateId = newTemplate.CampaignTemplateId;
                    if (newTemplate && newTemplateId && newTemplateId.length > 0) {

                        var saveObject = response.SaveObject;


                        if (saveObject && saveObject.selectedCampaign && saveObject.selectedContact) {
                            //replace the template id against the root task in selected campaign in SaveObject and save contact to campaign
                            if (saveObject.selectedCampaign.ListOfTasks && saveObject.selectedCampaign.ListOfTasks.length > 0) {

                                var i = 0;
                                for (i; i < saveObject.selectedCampaign.ListOfTasks.length; i++) {
                                    //Have we set complex add information here if so update the stage
                                    if (saveObject.ComplexAddValues != null
                                        && (saveObject.ComplexAddValues.ComplexAddWhoAndWhatIds.length > 0 || saveObject.ComplexAddValues.ComplexAddAdditionalFields.length > 0)) {
                                        saveObject.selectedCampaign.ListOfTasks[i].AddToSalesforce = true;
                                        saveObject.selectedCampaign.ListOfTasks[i].ComplexAddValues = saveObject.ComplexAddValues;
                                    }

                                    if (saveObject.selectedCampaign.ListOfTasks[i].RootTask === true) {
                                        saveObject.selectedCampaign.ListOfTasks[i].CampaignTemplateId = newTemplateId;
                                    }
                                }
                            }
                        }

                        // save contact to campaign
                        this.SaveTaskStagesForContact(saveObject, response.DraftId);
                    }
                }
            }

            UpdateComplexAddButton(domElements.ComplexAddButton, false);
        },
        SaveTaskStagesForContact: function(saveObject, draftId) {

            if (saveObject && saveObject.selectedCampaign && saveObject.selectedCampaign.ListOfTasks && saveObject.selectedContact) {

                //apply global flags to all stages.
                saveObject = this.ApplyFlagsToTaskStages(saveObject);

                var request = {
                    MessageType: "SaveContactToCampaign",
                    Tasks: saveObject.selectedCampaign.ListOfTasks,
                    ContactObj: saveObject.selectedContact,
                    MailboxUsername: selectedGoogleMailbox.details.Mailbox,
                    DraftId: draftId,
                    CcRecipients: domElements.DetailsPage.CcTagsInput.GetValidValues(),
                    BccRecipients: domElements.DetailsPage.BccTagsInput.GetValidValues(),
                    RelatedSalesforceCampaignId: GetRelatedSalesforceCampaignValue(),
                    SelectedMailbox: { Mailbox: selectedGoogleMailbox.details.Mailbox, TokenType: selectedGoogleMailbox.details.TokenType }
                };

                SendMessageToBackGround(request);
            }
        },
        /**
         * Saves a mass campaign
         * @param {} saveObject 
         * @param {} draftId 
         * @returns {} 
         */
        SaveTaskStagesForMultiRecipients: function(saveObject) {

            if (saveObject && saveObject.selectedCampaign && saveObject.selectedCampaign.ListOfTasks && saveObject.selectedContacts) {

                //apply global flags to all stages.
                saveObject = this.ApplyFlagsToTaskStages(saveObject);

                var request = {
                    MessageType: "SaveMultiRecipientsToCampaign",
                    MultiRecipientCampaign: {
                        Tasks: saveObject.selectedCampaign.ListOfTasks,
                        Contacts: saveObject.selectedContacts,
                        MailboxUsername: selectedGoogleMailbox.details.Mailbox,
                        IanaTimezone: Intl.DateTimeFormat().resolvedOptions().timeZone,
                        DraftId: saveObject.DraftId,
                        AutoRelate: autoRelateMultiRecipientCampaign,
                        CcRecipients: domElements.DetailsPage.CcTagsInput.GetValidValues(),
                        BccRecipients: domElements.DetailsPage.BccTagsInput.GetValidValues(),
                        RelatedSalesforceCampaignId: GetRelatedSalesforceCampaignValue(),
                        SelectedMailbox: { Mailbox: selectedGoogleMailbox.details.Mailbox, TokenType: selectedGoogleMailbox.details.TokenType }
                    }
                };

                SendMessageToBackGround(request);
            }
        },
        //applies global flag variables to each task stage.
        ApplyFlagsToTaskStages: function(saveObject) {
            if (typeof saveObject != "undefined" && saveObject != null) {
                if (typeof saveObject.selectedCampaign != "undefined" && saveObject.selectedCampaign != null) {
                    if (typeof saveObject.selectedCampaign.ListOfTasks != "undefined" && saveObject.selectedCampaign.ListOfTasks != null) {
                        for (var i = 0; i < saveObject.selectedCampaign.ListOfTasks.length; i++) {
                            saveObject.selectedCampaign.ListOfTasks[i].SendOnWeekDaysOnly = saveObject.selectedCampaign.SendWeekdaysOnly;

                            //Set the mailbox type from the selected mailbox in the list
                            saveObject.selectedCampaign.ListOfTasks[i].MailboxType =
                                ConvertTokenTypeIntoMailboxType(selectedGoogleMailbox.details.TokenType);
                        }
                    }
                }
            }

            return saveObject;
        },
        // save the email in the compose window as a template and once we have the id back, save contact to campaign
        MakeEditedCopyOfTemplateToUseAsRootTemmplate: function(rootTemplate, saveObject, draftId) {

            if (rootTemplate && saveObject) {

                // get the root task template, replace the content and make an edited copy and save contact to campaign

                var request = {
                    MessageType: "MakeEditedCopyOfTemplate",
                    Template: rootTemplate,
                    SaveObject: saveObject,
                    SaveContactToCampaign: true,
                    DraftId: draftId
                };

                SendMessageToBackGround(request);
            }
        },
        GetCampaignDetailsLoadResults: function(response) {

            if (response.success) {

                if (response.values && response.values.length > 0) {

                    var campaignDetailsObject = JSON.parse(response.values[0]);

                    if (campaignDetailsObject) {

                        // save a copy of the selected campaign so we know what we are editing and to what campaign we are adding the contact
                        if (currentState) {
                            currentState.selectedCampaign = campaignDetailsObject;
                        }

                        RenderCampaignMetaData(campaignDetailsObject);

                        var sequenceSection = document.getElementById("sfEbSendCampaignSequence");
                        if (sequenceSection) {
                            // render sequence in here
                            domElements.CampaignStepsDetailContainer = sequenceSection;
                            // create froala object for managing that in case user wants to edit an email template
                            this.froalaManager = FroalaClosure();
                            
                            this.RenderSequence(campaignDetailsObject.ListOfTasks);
                            // we have loaded the stage stages, lets update subjects if we need to
                            this.UpdateSubjectLines();
                        }

                        ShouldWeShowSendCampaignButton();
                    }
                }
            }

            CampaignManagerResizing().ResizeCampaignManager();
        },
        /**
         * Goes and gets the compose windows current complex add values and updates the campaign to show these selected values
         * @returns {} 
         */
        UpdateComplexAddState: function () {
            if (lEbstaComposeButtonsClosure != null) {
                var complexAddClosure = lEbstaComposeButtonsClosure.GetComplexAddClosure();
                if (complexAddClosure != null) {
                    if (!complexAddClosure.IsComplexAddValuesEmpty()) {
                        lComplexAddClosure.SetComplexAddValues(complexAddClosure.GetComplexAddValues());
                        this.SetComplexAddButtonState(lComplexAddClosure);
                    } else {
                        lComplexAddClosure.SetComplexAddValues(null);
                    }
                }
            }
        },
        //Renders a message above the steps informing if the campaign is set to run on weekdays only
        RenderWeekDayMessage: function() {
            var weekdayAlert = CreateElement("div", "ebsta-sequence-checkbox");
            weekdayAlert.appendChild(CreateCustomCheckBoxV2("Send weekdays only", "ebsta-weekdaysonly-checkbox"));
            return weekdayAlert;
        },
        //Renders a checkbox for the campaign header indicating whether to send follow ups as replies or not.
        RenderSendAsRepliesCheckbox: function() {
            var sendAsReplyElement = CreateElement("div", "ebsta-sequence-checkbox");
            var followUpCheckBox = CreateCustomCheckBoxV2("Send follow ups as replies", "ebsta-follupsreplies-checkbox")
            this.AddFollowUpsCheckBoxChangeEvent(followUpCheckBox);
            sendAsReplyElement.appendChild(followUpCheckBox);
            return sendAsReplyElement;
        },
        /**
         * Render the whole campaign
         * @param {Array<object>} listOfTasks - A list of all the campaign tasks in a campaign
         */
        RenderSequence: function (listOfTasks) {

            isFirstStepAnEmail = false;

            if (listOfTasks != null) {
  
                var areAnyStepsASfPollingStep = false;

                //Find Root Task first so we can then find the other elements to go down the tree 
                var rootTask = null;
                for (var i = 0; i < listOfTasks.length; i++) {
                    const currentTask = listOfTasks[i];
                    

                    /*
                     * We need to find any if any of the tasks in this step have a decision which 
                     * relies on polling so we can change things further down the line
                     */
                    const taskTypeGuid = GetTaskGuid(currentTask);
                    const decisionGuid = this.GetDecisionGuid(currentTask);
                    const taskObject = this.GetTaskById(taskTypeGuid, this.GetTasksWithDecisions());
                    if (taskObject) {
                        const decisionObject = this.GetDecisionByIdFromTask(decisionGuid, taskObject);
                        if (decisionObject &&
                            decisionObject.InterfaceToCall ===
                            campaignMgrDecisionTaskTypeInterfaceToCallEnum.CheckIfTaskIsClosed) {
                            areAnyStepsASfPollingStep = true;
                        }
                    }

                    if (currentTask.RootTask === true) {
                        rootTask = listOfTasks[i];

                        if (taskObject && taskObject.CampaignManagerTask &&
                            taskObject.CampaignManagerTask.InterfaceToCall === campaignMgrTaskTypeInterfaceToCallEnum.SendingAnEmail) {
                            isFirstStepAnEmail = true;
                        }
                    }
                }

                if (rootTask != null) {
                    var taskTypeGuid = GetTaskGuid(rootTask);
                    var decisionGuid = this.GetDecisionGuid(rootTask);
                    var taskObject = this.GetTaskById(taskTypeGuid, this.GetTasksWithDecisions());

                    if (taskObject) {
                        var decisionObject = this.GetDecisionByIdFromTask(decisionGuid, taskObject);
                        domElements.CampaignStepsDetailContainer.innerHTML = '';
                        var currentTotalMinutes = rootTask.ExecutionTimeframeForDecision;
                        this.InsertNewTask(taskObject, decisionObject, null, rootTask, areAnyStepsASfPollingStep, 1);

                        if (areAnyStepsASfPollingStep === true) currentTotalMinutes = 1;

                        this.RenderStage(rootTask, listOfTasks, currentTotalMinutes, areAnyStepsASfPollingStep);

                    } else {
                        // render error
                        this.RemoveLoaderAndAddMessageToLoaderDiv('Error in Retrieving Cadence Sequence.');
                    }
                } else {
                    // render error
                    this.RemoveLoaderAndAddMessageToLoaderDiv('Error in Retrieving Cadence Sequence.');
                }
            } else {
                // render error
                this.RemoveLoaderAndAddMessageToLoaderDiv('Error in Retrieving Cadence Sequence.');
            }
        },
        UpdateSubjectLines: function() { //update subject lines if we are sending as replies.
 
            //see if the 'send as replies' checkbox is checked.
            var followUpsAsRepliesChecked = $('#ebsta-follupsreplies-checkbox').is(':checked');

            this.SetSendFollowUpsAsRepliesAgainstCampaign(followUpsAsRepliesChecked);

            //if check box is checked, append all the subject lines.
            if (followUpsAsRepliesChecked) {

                //get each stage section that's been rendered.
                $stageElements = $(".sfEbstaCampaignTaskEmail");

                if ($stageElements) {
                    //get id of the first task
                    var firstTaskId = $stageElements.first().data('campaigntaskid');
                    var firstTaskStage = this.GetTaskStageById(firstTaskId);
                    if (firstTaskStage) {

                        var newSubject = "RE: " + firstTaskStage.Subject;

                        //loop through each element.
                        $stageElements.each(function(i) {

                            //don't apply this to the first element.
                            if (i > 0) {

                                var taskStageId = $(this).data('campaigntaskid');
                                var taskStage = lThisCampaignManagerInstance.GetTaskStageById(taskStageId);
                                if (taskStage) {

                                    // change the subject of the template to new subject and save core subject against object (only if we are currently using core subject)
                                    if (!taskStage.SendFollowUpsAsReplies) {
                                        var oldSubject = taskStage.Subject;
                                        taskStage.SubjectCore = oldSubject;
                                        taskStage.SendFollowUpsAsReplies = true;
                                    }

                                    taskStage.Subject = newSubject;

                                    //set the text div.
                                    $(this).find(".emailSubject").text(newSubject);

                                    //reset the pencil icon for editing.
                                    $(this).find(".ebsta-campaign-edit-pencil").removeClass("fa-pencil-square").addClass("fa-pencil");
                                }
                            }
                        });
                    }
                }
            } else { //otherwise set them back to the original.
                $(".sfEbstaCampaignTaskEmail").each(function(i) {
                    //don't apply this to the first element.
                    if (i > 0) {

                        var taskStageId = $(this).data('campaigntaskid');
                        var taskStage = lThisCampaignManagerInstance.GetTaskStageById(taskStageId);

                        if (taskStage.SendFollowUpsAsReplies) {
                            // we want to use the templates core subject now
                            var originalSubject = taskStage.SubjectCore;
                            taskStage.Subject = originalSubject;
                            taskStage.SendFollowUpsAsReplies = false;

                            //set the text div.
                            $(this).find(".emailSubject").text(originalSubject);

                            //reset the pencil icon for editing.
                            $(this).find(".ebsta-campaign-edit-pencil").removeClass("fa-pencil-square").addClass("fa-pencil");
                        }

                        //apply the disabled class.
                        $(this).removeClass("ebsta-stage-disabled");
                    }
                });
            }
        },
        SetSendFollowUpsAsRepliesAgainstCampaign: function(val) {
            if (currentState) {
                var campaign = currentState.selectedCampaign;
                if (campaign) {
                    campaign.SendFollowUpsAsReplies = val;
                }
            }
        },
        GetTaskStageById: function(taskId) {
            if (currentState && currentState.selectedCampaign) {
                var listOfTasks = currentState.selectedCampaign.ListOfTasks;
                if (listOfTasks && listOfTasks.length > 0) {
                    var i = 0;
                    for (i; i < listOfTasks.length; i++) {
                        if (listOfTasks[i].CampaignMgrTaskId === taskId) {

                            return listOfTasks[i];
                        }
                    }
                }
            }
            return null;
        },
        GetDayOfExecutionFromTotalMinutes: function(totalMinutes) {
            var days = Math.floor(totalMinutes / 1440);
            days = days + 1;
            return days;
        },
        GetTasksWithDecisions: function() {
            return tasksWithDecisions;
        },
        /**
         * Renders a stage in the campaign view. It will recursively call once it has finished rendering all stages.
         * @param {object} task - The task we're looking for Positive and Negative stages for
         * @param {Array<object>} taskList - All the tasks in all the steps associated with a campaign.
         * @param {int} currentTotalMinutesOrStepCounter - The total number of minutes we're up to rendering our campaign or 
         *  if we're just rendering a simple step counter this will be keep to keep count of that
         * @param {boolean} showAsSteps - Whether we should be showing the Day/Step counter as Days or Steps
         */
        RenderStage: function (task, taskList, currentTotalMinutesOrStepCounter, showAsSteps) {
            var isThereAPositiveTask = false;
            var stageDayNumber;

            //If we're just showing steps then just increment the number
            if (showAsSteps === true) {
                currentTotalMinutesOrStepCounter += 1;
                stageDayNumber = currentTotalMinutesOrStepCounter;
            } else {
                stageDayNumber = this.GetDayOfExecutionFromTotalMinutes(currentTotalMinutesOrStepCounter);
            }
            var negativeTaskObject = this.GetNegativeStage(task, taskList);

            if (task.NextStagePositive != null) {
                for (var i = 0; i < task.NextStagePositive.length; i++) {
                    for (var j = 0; j < taskList.length; j++) {
                        if (taskList[j].CampaignMgrTaskId == task.NextStagePositive[i]) {
                            var selectedTask = taskList[j];
                            var taskTypeGuid = GetTaskGuid(selectedTask);
                            var decisionGuid = this.GetDecisionGuid(selectedTask);
                            var taskObject = this.GetTaskById(taskTypeGuid, this.GetTasksWithDecisions());
                            var decisionObject = this.GetDecisionByIdFromTask(decisionGuid, taskObject);
                           
                            this.InsertNewTask(taskObject, decisionObject, negativeTaskObject, selectedTask, showAsSteps, stageDayNumber);

                            if (showAsSteps === false) {
                                currentTotalMinutesOrStepCounter = currentTotalMinutesOrStepCounter + taskList[j].ExecutionTimeframeForDecision;
                            }

                            isThereAPositiveTask = true;
                            this.RenderStage(selectedTask, taskList, currentTotalMinutesOrStepCounter, showAsSteps);
                            break;
                        }
                    }
                }
            }

            //There could be a situation where this is only a negative Task so lets check if there was one
            //when there wasn't a positive task
            if (isThereAPositiveTask === false) {
                if (negativeTaskObject) {
                    this.InsertNewTask(null, null, negativeTaskObject, null, stageDayNumber);
                }
            }
        },
        UpdateTaskWithInfo: function(newTaskDom, campaignTask) {

            var taskContent = newTaskDom.getElementsByClassName(campaignClassNames.TaskContent);
            if (taskContent.length > 0) {
                var contentSection = taskContent[0];
                var subjectDiv = contentSection.getElementsByClassName(campaignClassNames.EmailSubject);
                if (subjectDiv.length > 0) {
                    var emailSubject = campaignTask.Subject;
                    if (campaignTask.SendFollowUpsAsReplies) {
                        emailSubject = campaignTask.SubjectCore;
                    }
                    subjectDiv[0].innerText = emailSubject;
                    contentSection.setAttribute(dataAttributes.SubjectName, emailSubject); //set the email subject attribute.

                }

                //Find Task info and set the subject as the title for the Salesforce task
                if (campaignTask.JsonExtraProperties != null) {
                    const addEntityObj = JSON.parse(campaignTask.JsonExtraProperties);
                    if (addEntityObj.Fields != null) {
                        const taskSubject = contentSection.getElementsByClassName(campaignClassNames.SfTaskSubject)[0];
                        if (taskSubject) {
                            let isThereASubjectProperty = false;

                            for (let i = 0; i < addEntityObj.Fields.length; i++) {
                                const currentField = addEntityObj.Fields[i];
                                if (currentField.Name != null) {
                                    if (currentField.Name == "Subject") {
                                        isThereASubjectProperty = true;
                                        const subject = addEntityObj.Fields[i].Value;
                                        taskSubject.innerText = subject;
                                        contentSection.setAttribute(dataAttributes.SubjectName, subject);
                                    }
                                }
                            }

                            if (!isThereASubjectProperty) {
                                taskSubject.innerText = "[NO SUBJECT]";
                            }
                        }
                    }
                }

                contentSection.setAttribute(dataAttributes.CampaignId, campaignTask.CampaignMgrId);
                contentSection.setAttribute(dataAttributes.TaskId, campaignTask.CampaignMgrTaskId);
            }
        },
        AddFileAttachList: function (template) {
            $( ".attachment-list-container" ).remove();
            var fileAttachments = template.FileAttachments;
            if (fileAttachments && fileAttachments.length > 0) {
                var attachmentsList = CreateElement("div", "attachment-list-container");
                var fileSize = 0;
                var fileLabel = "";

                for (var i = 0; i < fileAttachments.length; i++) {
                    fileSize = fileAttachments[i].FileSize / 1024 / 1024;
                    fileLabel = fileAttachments[i].FileName;
                    if (fileSize > 0) {
                        fileLabel += " (" + fileSize.toFixed(3) + "MB)";
                    }

                    var attachmentItemDiv = CreateElement('div', 'attachment-row', '', '');
                    var attachmentIcon = CreateElement('div', 'div-attachment-icon', '', '');
                    var attachmentItemText = CreateElement('div', 'div-attachment-text', fileLabel, '');

                    attachmentItemDiv.appendChild(attachmentIcon);
                    attachmentItemDiv.appendChild(attachmentItemText);

                    // add attachment container div to the attachment list widget.
                    attachmentsList.appendChild(attachmentItemDiv);
                }

                var toolbarRow = $('.fr-element');
                if (toolbarRow.length > 0 && toolbarRow[0]) {
                    insertAfter(attachmentsList, toolbarRow[0]);
                }
            }
        },
        GetTemplateByIdHandleResponse: function(response) {

            if (response && response.success) {
                if (response.values && response.values.length > 0 && response.templateEditObj) {
                    var template = JSON.parse(response.values[0]);

                    if (template && template.CampaignTemplateId && template.CampaignTemplateId.length > 0) {

                        var editorDiv = document.getElementById('sfEbStaCampaignEmailHtmlEditor');
                        if (editorDiv) {
                            editorDiv.innerHTML = '';
                            if (this.froalaManager != null) {

                                var isRootTask = this.IsRootTask(response.templateEditObj.taskId);

                                this.froalaManager.SetupFroala(false, '#sfEbStaCampaignEmailHtmlEditor', template);
                                this.froalaManager.InsertFroalaCustomHeader(response.templateEditObj, this.GetCustomHeader(isRootTask));
                                this.froalaManager.SetHtmlBody(template.BodyTemplate || "");
                                this.AddFileAttachList(template);

                                if (response.templateEditObj.subject) { //if we have a subject from the page use this.
                                    this.froalaManager.SetSubject(response.templateEditObj.subject || "", response.templateEditObj.isDisabled);
                                } else { //otherwise take the subject from the template.
                                    this.froalaManager.SetSubject(template.EmailSubject || "");
                                }
                            }
                        }
                    }
                }
            }
            CampaignManagerResizing().ResizeCampaignManager();
        },
        /**
         * Creates a custom header for the text editor
         * @param {} rootTask 
         * @returns {} 
         */
        GetCustomHeader : function(rootTask) {
            if (rootTask) {
                //schedule later
                var text = 'Schedule Later';
                var calendarIconClassName = this.GetIconsObj().Calendar;

                if (this.IsFirstEmailScheduledForLater() === true) {
                    // if user has scheduled the first email to go out later, lets render the date instead

                    var date = this.GetFirstEmailScheduleDate();

                    if (date !== null && typeof date !== 'undefined') {
                        text = this.FormatSendLaterDate(date);
                        calendarIconClassName = this.GetIconsObj().CalendarScheduled;
                    }
                }
                var scheduleLaterDiv = CreateElement("div", null, null, 'sfEbStaCampaignEmailScheduleLater');
                scheduleLaterDiv.appendChild(CreateElement('i', calendarIconClassName, null, 'sfEbStaCampaignEmailScheduleLaterIcon'));
                scheduleLaterDiv.appendChild(CreateElement('div', null, text, 'sfEbStaCampaignEmailScheduleLaterText'));

                this.AttachListenerToScheduleLater(scheduleLaterDiv, false);
                return scheduleLaterDiv;
            }
            return null;
        },
        IsRootTask(taskId) {
            if (currentState && currentState.selectedCampaign) {
                var tasks = currentState.selectedCampaign.ListOfTasks;
                if (tasks && tasks.length > 0) {
                    var i = 0;
                    for (i; i < tasks.length; i++) {
                        if (tasks[i].CampaignMgrTaskId === taskId) {
                            return tasks[i].RootTask;
                        }
                    }
                }
            }

            return false;
        },
        GetTemplateById: function(templateId, templateEditObj) {

            if (templateId && templateEditObj) {

                this.ShowEditEmailView(templateEditObj.subject);
                var request = {
                    MessageType: "GetTemplateById",
                    InjectIntoComposeWindow: false,
                    TemplateId: templateId,
                    TemplateEditObj: templateEditObj,
                    MailboxUsername: selectedGoogleMailbox.details.Mailbox,
                    ReplaceMergeFields: false
                };

                SendMessageToBackGround(request);
            }
        },
        GetTemplateByIdAndInjectIntoComposeWindowHandleResponse: function(response) {
            RemoveOverlay();
            if (response) {
                if (response.success) {
                    if (response.values && response.values.length > 0) {
                        var template = JSON.parse(response.values[0]);

                        if (template && template.CampaignTemplateId && template.CampaignTemplateId.length > 0) {

                            // inject email body in
                            var emailBody = domElements.ComposeWindowButton.closest('.iN');
                            if (emailBody) {

                                var editableDiv = emailBody.getElementsByClassName('editable');

                                if (editableDiv && editableDiv.length === 1) {
                                    // replace merge fields
                                    if (campaignDetailsSelected != null) {
                                        var emailBodyAFterMergeFieldsCompleted = this.ReplaceMergeFieldsWithContactInformation(template.BodyTemplate);
                                        editableDiv[0].innerHTML = emailBodyAFterMergeFieldsCompleted;
                                        // save the first template in the appropriate selected campaign so we can swap it out when we go to save the contact to the campign
                                        campaignDetailsSelected['rootTemplate'] = template;
                                    }

                                    GmailTools.Compose.TriggerChangeOnCompose(emailBody);
                                }
                            }

                            //inject email subject
                            var composeWindow = domElements.ComposeWindowButton.closest('.aoP');
                            if (composeWindow) {
                                var subjectBox = composeWindow.getElementsByClassName('aoT');
                                if (subjectBox && subjectBox.length > 0) {
                                    var n = 0;
                                    for (n; n < subjectBox.length; n++) {
                                        if (subjectBox[n].name === 'subjectbox') {
                                            var emailSubjectAFterMergeFieldsCompleted = this.ReplaceMergeFieldsWithContactInformation(template.EmailSubject);
                                            subjectBox[n].value = emailSubjectAFterMergeFieldsCompleted;
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        },
        ReplaceMergeFieldsWithContactInformation: function(body) {

            if (contactObject) {

                // replace email merge fields
                var emaiVal = contactObject.GetEmailAddress();
                if (emaiVal && emaiVal.length > 0) {
                    for (let mergefieldKey in mergeFields.EmailMergeFields) {
                        if (mergeFields.EmailMergeFields.hasOwnProperty(mergefieldKey)) {
                            body = body.replace(new RegExp(mergeFields.EmailMergeFields[mergefieldKey], 'g'), emaiVal);
                        }
                    }
                }

                // replace firstname merge fields
                var firstNameVal = contactObject.GetFirstName();
                if (firstNameVal && firstNameVal.length > 0) {
                    for (let mergefieldKey in mergeFields.FirstNameMergeFields) {
                        if (mergeFields.FirstNameMergeFields.hasOwnProperty(mergefieldKey)) {
                            body = body.replace(new RegExp(mergeFields.FirstNameMergeFields[mergefieldKey], 'g'), firstNameVal);
                        }
                    }
                }

                // replace second name merge fields
                var lastNameVal = contactObject.GetLastName();
                if (lastNameVal && lastNameVal.length > 0) {
                    for (let mergefieldKey in mergeFields.SecondNameMergeFields) {
                        if (mergeFields.SecondNameMergeFields.hasOwnProperty(mergefieldKey)) {
                            body = body.replace(new RegExp(mergeFields.SecondNameMergeFields[mergefieldKey], 'g'), lastNameVal);
                        }
                    }
                }

                //Full Name
                if (firstNameVal || lastNameVal) {
                    for (let mergefieldKey in mergeFields.FullNameMergeFields) {
                        if (mergeFields.FullNameMergeFields.hasOwnProperty(mergefieldKey)) {
                            body = body.replace(new RegExp(mergeFields.FullNameMergeFields[mergefieldKey], 'g'), firstNameVal + " " + lastNameVal);
                        }
                    }
                }

                // replace second name merge fields
                var companyNameVal = contactObject.GetCompany();
                if (companyNameVal && companyNameVal.length > 0) {
                    for (let mergefieldKey in mergeFields.CompanyMergeFields) {
                        if (mergeFields.CompanyMergeFields.hasOwnProperty(mergefieldKey)) {
                            body = body.replace(new RegExp(mergeFields.CompanyMergeFields[mergefieldKey], 'g'), companyNameVal);
                        }
                    }
                }

                //Replace all other merge fields that haven't been replace with blank
                body = body.replace(new RegExp("{!.*?}|{{{.*?}}}", "g"), "");
            }

            return body;
        },
        GetTemplateByIdAndInjectIntoComposeWindow: function(templateId) {

            if (templateId) {

                var request = {
                    MessageType: "GetTemplateById",
                    InjectIntoComposeWindow: true,
                    TemplateId: templateId,
                    MailboxUsername: selectedGoogleMailbox.details.Mailbox,
                    ReplaceMergeFields: true
                };

                SendMessageToBackGround(request);
            }
        },
        EditEmailTemplatePage: function() {
            this.ResetCampaignPage();
            currentPage.EditEmailTemplate = true;
            ShowCampaignSelectButton('Use');
            HideCampaignSendButton();
        },
        ShowEditEmailView: function(subject) {

            this.EditEmailTemplatePage();
            var mainPage = document.getElementById('sfEbSendCampaignMgrMainModelDetails');

            if (mainPage) {
                var sequenceView = document.getElementById('sfEbSendCampaignSequenceContainer');
                if (sequenceView) {

                    var emailEditView = document.getElementById('sfEbSendCampaignEmailEditorView');

                    if (emailEditView) {

                        var editorContainer = document.getElementById('sfEbStaCampaignEmailEditorContainer');

                        if (editorContainer) {
                            editorContainer.innerHTML = '';
                            var htmlEditor = CreateElement('div', '', "", 'sfEbStaCampaignEmailHtmlEditor');
                            htmlEditor.appendChild(CreateSpinnerLoaderDiv());
                            editorContainer.appendChild(htmlEditor);
                        }

                        sequenceView.style.display = "none";
                        this.ShowElement(emailEditView);
                    } else {
                        var text = "Edit Template";
                        if (subject && subject.length > 0) {
                            text = "Edit " + subject;
                        }
                        emailEditView = CreateElement('div', null, null, 'sfEbSendCampaignEmailEditorView');

                        //html editor contauiner
                        var editorContainer = CreateElement('div', '', "", 'sfEbStaCampaignEmailEditorContainer');

                        var htmlEditor = CreateElement('div', '', "", 'sfEbStaCampaignEmailHtmlEditor');
                        htmlEditor.appendChild(CreateSpinnerLoaderDiv());

                        editorContainer.appendChild(htmlEditor);

                        emailEditView.appendChild(editorContainer);

                        sequenceView.style.display = "none";
                        mainPage.appendChild(emailEditView);
                    }

                }
            }

        },
        GetNameForObject: function(obj) {
            return obj.Name;
        },
        /**
         * Creates the element which display when the next task will take place
         * @param {object} decision - The object containing information about the decision
         * @param {int} decisionTimeFrame - The amount of minutes the next Task would happen in
         * @param {object[]} extraDecisionLogicArray - If there has been additional logic set then display extra information
         * @returns {DOMElement} - The element to render
         */
        CreateDecisionDom: function (decision, decisionTimeFrame, extraDecisionLogicArray) {
            var containerDom = CreateElement("div", campaignClassNames.DecisionContainer);

            var decisionDom = CreateElement("div", campaignClassNames.Decision, null, null);
            decisionDom.setAttribute(dataAttributes.CampaignDecisionTaskTypeId, this.GetDecisionGuid(decision));
            decisionDom.setAttribute("data-interfacetocall", decision.InterfaceToCall);

            containerDom.appendChild(decisionDom);

            decisionDom.appendChild(CreateElement("i", icons.Clock, null, null));

            //Calculate the timeframe this step was due to happen. 
            //We store in total minutes so start from minutes and work the way up
            var timeBracket = "minute";
            var timeFrameNumber = decisionTimeFrame;

            if (decisionTimeFrame % minuteCalculations.week === 0) {
                timeBracket = "week";
                timeFrameNumber = decisionTimeFrame / minuteCalculations.week;
            } else if (decisionTimeFrame % minuteCalculations.day === 0) {
                timeBracket = "day";
                timeFrameNumber = decisionTimeFrame / minuteCalculations.day;
            } else if (decisionTimeFrame % minuteCalculations.hour === 0) {
                timeBracket = "hour";
                timeFrameNumber = decisionTimeFrame / minuteCalculations.hour;
            }

            if (timeFrameNumber > 1) {
                timeBracket += "s";
            }

            //Detect what type of decision this is and render the specified text
            var summaryText = "";
            if (decision.InterfaceToCall === campaignMgrDecisionTaskTypeInterfaceToCallEnum.CheckIfNoOneHasReplied) {
                summaryText = "If no reply in " + timeFrameNumber + " " + timeBracket;;
            } else if (decision.InterfaceToCall === campaignMgrDecisionTaskTypeInterfaceToCallEnum.AlwaysTrue) {
                summaryText = "In " + timeFrameNumber + " " + timeBracket;
            } else if (decision.InterfaceToCall === campaignMgrDecisionTaskTypeInterfaceToCallEnum.CheckIfTaskIsClosed) {
                summaryText = GetIfTaskClosedDecisionText(timeFrameNumber, timeBracket, extraDecisionLogicArray);
            }

            decisionDom.appendChild(CreateElement("span", null, summaryText));

            containerDom.appendChild(CreateElement('div', 'sfEbstaCampaignSideTaskLabel', GetNegativeDecisionTextLabel(decision.InterfaceToCall), null));

            return containerDom;
        },
        CreateExecutionDayDiv: function(labelName, days) {
            var div = CreateElement('div', campaignClassNames.TaskExecutionDay, null, null);
            div.appendChild(CreateElement('div', null, labelName, null));
            div.appendChild(CreateElement('div', campaignClassNames.TaskExecutionDayNumber, days, null));
            return div;
        },
        /**
         * Inserts a new step into the Campaign view
         * @param {object | null} positiveTask - An object containing details for the type of Task the 'Positive' Task of this step is.
         * @param {object | null} decisionObject - An object detailing the decision type related to this step, if there is one. If this is last 
         * step in a campaign this can be null so we won't display anything.
         * @param {object} negativeTask -  An object containing details for the 'Negative' Task within the step. This can be null
         * when there isn't a negative task for this step.
         * @param {object} wholeTask - The should contain all the details for this Task 
         * @param {boolean} showAsSteps - Whether we should be showing the Day/Step counter as Days or Steps
         * @param {int} campaignDayStepDuration - The day we should render in the timeline DOMEleshowAsStepsments where we show what day certain
         * events would happen within the campaign or if counting steps simply the step number we're on
         */
        InsertNewTask: function (positiveTask, decisionObject, negativeTask, wholeTask, showAsSteps, campaignDayStepDuration) {

            var newTaskDomContainer = CreateElement('div', campaignClassNames.Task, null, null);

            var newTaskDom = CreateElement("div", campaignClassNames.WholeTaskContent);

            //This will hold all the Tasks information, as well as the timeline info
            var mainRow = CreateElement("div", "taskRow");
            newTaskDom.appendChild(mainRow);

            var dayOfExecutionContainer = CreateElement('div', campaignClassNames.TaskExecutionDayContainer, null, null);
            dayOfExecutionContainer.appendChild(this.CreateExecutionDayDiv(showAsSteps === false ? "Day" : "Step", campaignDayStepDuration));
            mainRow.appendChild(dayOfExecutionContainer);

            //Will only contain the Ngative and Positive Task elements
            var tasksContainer = CreateElement("div", "tasks");
            mainRow.appendChild(tasksContainer);

            var positiveTaskDom = CreateTaskDom(positiveTask ? positiveTask.CampaignManagerTask : null, true,
                wholeTask ? wholeTask.CampaignTemplateId : null);
            this.UpdateTaskWithInfo(positiveTaskDom, wholeTask);
            tasksContainer.appendChild(positiveTaskDom);

            //Render the negative task right next to the positive task
            if (negativeTask) {
                var negativeTaskDom = this.CreateNegativeTask(negativeTask.CampaignTaskObject, negativeTask.TaskInfo);
                if (negativeTaskDom) {
                    tasksContainer.appendChild(negativeTaskDom);
                }
            }

            // If there are associated positive or negative decisions then we need to display the decision information related to this Task
            if (wholeTask) {
                if ((wholeTask.NextStageNegative && wholeTask.NextStageNegative.length > 0) 
                    || (wholeTask.NextStagePositive && wholeTask.NextStagePositive.length > 0))
                {
                    var newDecisionDom = this.CreateDecisionDom(decisionObject, wholeTask.ExecutionTimeframeForDecision,
                        wholeTask.ExtraDecisionLogic != null ? wholeTask.ExtraDecisionLogic : null);
                    newTaskDom.appendChild(newDecisionDom);
                }
            }
           
            newTaskDomContainer.appendChild(newTaskDom);
            domElements.CampaignStepsDetailContainer.appendChild(newTaskDomContainer);
        },
        /**
         * Creates the negative task associated with a step within a campaign
         * @param {object} task - An object containing details for the type of Task this step is
         * @param {object} taskInfo - An object containing details about the details of Task i.e. Email/Task Subject
         * @returns {DOMElement} - The element to render
         */
        CreateNegativeTask: function (task, taskInfo) {
            if (taskInfo != null) {
                var newTaskDom = CreateTaskDom(task.CampaignManagerTask, false, taskInfo.CampaignTemplateId);
                newTaskDom.classList.add(campaignClassNames.SideTask);
                this.UpdateTaskWithInfo(newTaskDom, taskInfo);
                return newTaskDom;
            }
            return null;
        },
        GetNegativeStage: function(task, taskList) {
            if (task.NextStageNegative != null) {
                for (var i = 0; i < task.NextStageNegative.length; i++) {
                    for (var j = 0; j < taskList.length; j++) {
                        if (taskList[j].CampaignMgrTaskId == task.NextStageNegative[i]) {
                            var selectedTask = taskList[j];
                            var taskTypeGuid = GetTaskGuid(selectedTask);
                            var taskObject = this.GetTaskById(taskTypeGuid, this.GetTasksWithDecisions());
                            return {
                                CampaignTaskObject: taskObject,
                                TaskInfo: selectedTask
                            };
                        }
                    }
                }
            }
            return null;
        },
        GetDecisionByIdFromTask: function(decisionId, task) {
            if (task.AssociatedDecisions != null) {
                for (var i = 0; i < task.AssociatedDecisions.length; i++) {
                    var currentDecision = task.AssociatedDecisions[i];
                    if (currentDecision.CampaignMgrDecisionTaskTypeId == decisionId) {
                        return currentDecision;
                    }
                }
            }
        },
        GetTaskById: function(taskTypeId, localTasksWithDecisions) {
            for (var i = 0; i < localTasksWithDecisions.length; i++) {
                var currentTask = localTasksWithDecisions[i];
                if (currentTask.CampaignManagerTask != null) {
                    if (currentTask.CampaignManagerTask.CampaignMgrTaskTypeId != null) {
                        if (currentTask.CampaignManagerTask.CampaignMgrTaskTypeId == taskTypeId) {
                            return currentTask;
                        }
                    }
                }
            }
        },
        GetDecisionGuid: function(decision) {
            return decision.CampaignMgrDecisionTaskTypeId;
        },
        //get details including tasks for campaign
        GetCampaignDetails: function (id, fromMainModel) {
            lastLoadedCampaignId = id;
            lastGetCampaignDetailsRequestKey = new Date().getTime() + id;

            //Reset Send Later elements and properties
            this.ResetSendLaterButton();
            this.ResetAllScheduleLaterVariables();
            lComplexAddClosure.SetComplexAddValues(null);

            clickedFromMainModel = fromMainModel;
            this.ShowCampaignDetailsPage();
            this.HideMainModelContentDiv();
            this.UpdateComplexAddState();
            this.CheckSendLaterHasBeenScheduledAlready();

            var request = {
                MessageType: "GetCampaignDetails",
                campaignId: id
            };

            var setupRequest = new GetSetupValues(true, function () {
                //Now we have all the setup values back we can now render the mailboxes a user can send the campaign from
                ReplaceCampaignSendFromElement(CreateSendCampaignFromElement());
                CheckGmailAccountIsAuthedBeforeSend();
            });
            
            // The callback method which should be fired before rendering the campaign details. 
            // This will ensure the details returned the match the last requested campaign
            var callback = (function (key, lSetupRequest) {
                return function () {
                    if (key === lastGetCampaignDetailsRequestKey && lSetupRequest.IsRequestActive() === false) {
                        //Set the value to null so everywhere else knows the request is finished
                        lastGetCampaignDetailsRequestKey = null;
                        ShouldWeShowSendCampaignButton();
                        return true;
                    }
                    return false;
                };
            })(lastGetCampaignDetailsRequestKey, setupRequest);

            SendMessageToBackGround(request, callback);
        },
        CampaignDetailsPage: function() {
            this.ResetCampaignPage();
            currentPage.CampaignDetail = true;
            this.ChangeCancelButtonText('Back', false);
            ShouldWeShowSendCampaignButton();
        },
        ResetAllScheduleLaterVariables: function() {
            firstEmailScheduledLater = false;
            lastDateSelected = new Date();
            lastDateSelected.setHours(lastDateSelected.getHours() + 1);
        },
        /** Sets and Creates the Campaign Details view up */
        ShowCampaignDetailsPage: function () {
            this.ShowMainModelDetailsDiv();
            this.HideMainModelSendLaterScheduler();
            this.ShowCampaignMgrMainModel();
            this.HideMainModelContentDiv();
            ShowContactContainer();
            HideMultiRecipientsText();
            this.CampaignDetailsPage();
            SetComplexAddButtonToBeEnabled();
            UpdateMultipleContactSelectedBool(false);

            if (mainCampaignMgrModel) {

                //Delete the DOM if it already exists as we'll go and recreate the whole DOM again
                var detailsView = document.getElementById("sfEbSendCampaignMgrMainModelDetails");
                if (detailsView) {
                    detailsView.remove();
                } 

                //Create the standard template DOM for the details paga of the Campaign
                this.CreateCampaignDetailsElement();
            }
            else {
                LogThis("Missing Ebsta Main Cadence"); 
            }
        },
        /** Creates the DOM for where we show the details of the campaign and where a user finalises a campaign being sending*/
        CreateCampaignDetailsElement() {
            var campaignDetailsDiv = CreateElement("div", "", "", "sfEbSendCampaignMgrMainModelDetails");

            campaignDetailsDiv.appendChild(CreateFromAndRecipientHeader());

            // container for campaign details
            var detailsDiv = CreateElement("div", "", "", "sfEbSendCampaignDetails");

            // sending to section
            var detailsSendingTo = CreateElement("div", "", "", "sfEbSendCampaignDetailsSendingTo");
            var sendingToHeaderSection = CreateElement("div", "sfEbSendCampaignDetailsHeader", null, "");
            sendingToHeaderSection.appendChild(document.createTextNode("Sending To"));
            sendingToHeaderSection.appendChild(this.CreateMultipleRecipientsButton());
            detailsSendingTo.appendChild(sendingToHeaderSection);

            //Contain the Contact details
            var contactDetailsContainer = CreateElement("div", "", "", "sfEbSendCampaignDetailsContactContainer");

            // email
            var detailsEmail = CreateElement("div", "sfEbSendCampaignDetailsRow", "", "");
            detailsEmail.appendChild(CreateElement("div", "", "Email", "sfEbSendCampaignDetailsEmailField"));
            contactDetailsContainer.appendChild(detailsEmail);

            // first name
            var detailsfirstName = CreateElement("div", "sfEbSendCampaignDetailsRow", "", "");
            detailsfirstName.appendChild(CreateElement("div", "", "First Name", "sfEbSendCampaignDetailsFirstNameField"));
            contactDetailsContainer.appendChild(detailsfirstName);

            // last name
            var detailsLastName = CreateElement("div", "sfEbSendCampaignDetailsRow", "", "");
            detailsLastName.appendChild(CreateElement("div", "", "Last Name", "sfEbSendCampaignDetailsLastNameField"));
            contactDetailsContainer.appendChild(detailsLastName);

            // company.
            var detailsCompanyName = CreateElement("div", "sfEbSendCampaignDetailsRow", "", "");
            detailsCompanyName.appendChild(CreateElement("div", "", "Company", "sfEbSendCampaignDetailsCompanyNameField"));
            contactDetailsContainer.appendChild(detailsCompanyName);

            detailsSendingTo.appendChild(contactDetailsContainer);

            //append all the divs to the main container.
            detailsDiv.appendChild(detailsSendingTo);

            // campaign section

            var campaign = CreateElement("div", "", "", "sfEbSendCampaignDetailsCampaign");
            campaign.appendChild(CreateElement("div", "sfEbSendCampaignDetailsHeader", "Cadence", ""));

            // name
            var detailsCampaignName = CreateElement("div", "sfEbSendCampaignDetailsRow", "", "");
            detailsCampaignName.appendChild(CreateElement("div", "", "Name", "sfEbSendCampaignDetailsCampaignNameField"));
            detailsCampaignName.appendChild(CreateElement("div", "", "", "sfEbSendCampaignDetailsCampaignNameValue"));
            campaign.appendChild(detailsCampaignName);

            // owner
            var detailsOwner = CreateElement("div", "sfEbSendCampaignDetailsRow", "", "");
            detailsOwner.appendChild(CreateElement("div", "", "Owner", "sfEbSendCampaignDetailsOwnerField"));
            detailsOwner.appendChild(CreateElement("div", "", "", "sfEbSendCampaignDetailsOwnerValue"));
            campaign.appendChild(detailsOwner);

            // recipients
            var detailsRecipients = CreateElement("div", "sfEbSendCampaignDetailsRow", "", "");
            detailsRecipients.appendChild(CreateElement("div", "", "Recipients", "sfEbSendCampaignDetailsRecipientsField"));
            detailsRecipients.appendChild(CreateElement("div", "", "", "sfEbSendCampaignDetailsRecipientsValue"));
            campaign.appendChild(detailsRecipients);

            //add a blank row to level up the divs.
            var detailsSpacer = CreateElement("div", "sfEbSendCampaignDetailsRow", "", "");
            detailsSpacer.appendChild(CreateElement("div", "", "Salesforce Campaign", "sfEbSendCampaignDetailsSalesforceRelationField"));
            detailsSpacer.appendChild(CreateElement("div", "", "N/A", "sfEbSendCampaignDetailsSalesforceRelationValue"));
            campaign.appendChild(detailsSpacer);

            detailsDiv.appendChild(campaign);

            campaignDetailsDiv.appendChild(detailsDiv);

            // container for sequence
            var sequenceContainer = CreateElement("div", "", "", "sfEbSendCampaignSequenceContainer");

            //Sequence container which contains the steps of the element
            var sequenceDiv = CreateElement("div", "", "", "sfEbSendCampaignSequence");
            sequenceDiv.appendChild(CreateSpinnerLoaderDiv());

            //create the container for the sequence header.
            var sequenceHeaderContainer = CreateElement("div", "sfEbSendCampaignDetailsHeader", "", "");

            //create the 'Sequence' header label.
            var sequenceHeaderLabel = CreateElement("span", "", "Sequence", "");

            //create the section for weekday only checkbox.
            var weekDayOnlyCheckboxDiv = this.RenderWeekDayMessage();

            //create the section for send follow ups as replies checkbox.
            var sendAsReplyCheckboxDiv = this.RenderSendAsRepliesCheckbox();

            //append the header label to the container.
            sequenceHeaderContainer.appendChild(sequenceHeaderLabel);

            //append the week day only checkbox to the container.
            sequenceHeaderContainer.appendChild(weekDayOnlyCheckboxDiv);

            //append the send follow ups as replies checkbox to the container.
            sequenceHeaderContainer.appendChild(sendAsReplyCheckboxDiv);

            //append the EmailTracking, AddToSalesforce and SendLater buttons
            sequenceHeaderContainer.appendChild(this.CreateSequenceHeaderButtons());

            sequenceContainer.appendChild(sequenceHeaderContainer);
            sequenceContainer.appendChild(sequenceDiv);

            campaignDetailsDiv.appendChild(sequenceContainer);

            this.InsertMainViewInModelBeforeButtonsSection(campaignDetailsDiv);

            PrePopulateContactsIntoCampaignModal();
        },
        /**
         * Creates the button which sits inside the campaign detaisl view when you're looking to add a contact(S) to a campaign
         * @returns {} 
         */
        CreateMultipleRecipientsButton: function() {
            var multiRecipientsButton = CreateElement("div", "sfEbSendCampaignMgrButton", "Add Recipients", "EbstaBtCampaignsAddRecipients");
            multiRecipientsButton.addEventListener("click", function() {
                CreateOrShowMultiRecipientModal();
            });
            return multiRecipientsButton;
        },
        /**
         * Sets the messaging/edit event for multiple recipients when selected.
         * @param {} firstLoad 
         * @returns {} 
         */
        SetMultipleRecipientsSelected: function()
        {
            if (multiRecipientRowSelectedCounter === 0) {
                ShowNoSelectedContactsWarning();
            } else if (multiRecipientRowSelectedCounter <= maximumMultiRecipients) {
                lThisCampaignManagerInstance.ResetMultipleContactArray();
                lThisCampaignManagerInstance.GetSelectedRecipientsFromMultiRecipientSection();

                //UPDATE THE CONTACTS DISPLAY TO SAY
                ReplaceContactDisplayWithMultiRecipientLink();
                 
                // only allow users to send from this point on
                HideElement(domElements.MultiRecipientModal);
                UpdateMultipleContactSelectedBool(true);
                SetComplexAddButtonForMultiRecipientMode();
            } else {
                //Show a dialog stating why they user can't confirm
                window.GLOBAL_EBSTA_MODAL.SetText("You've selected more contacts than the maximum allowed (" + maximumMultiRecipients + ").");
                window.GLOBAL_EBSTA_MODAL.SetOkButtonMode();
                window.GLOBAL_EBSTA_MODAL.ShowModal();
                window.GLOBAL_EBSTA_MODAL.ShowDialog();
            }
        },
        /**
         * gets list of selected recipients form the multi recipient section and add to the multipleContactArray variable
         * @returns {} 
         */
        GetSelectedRecipientsFromMultiRecipientSection: function () {
            multipleContactArray = new Array();
            multiRecipientRowSelectedCounter = 0;

            var multiRecipientModal = document.getElementById("sfEbSendCampaignMultiRecipientModal");
            if (multiRecipientModal) {
                var contactRecipientRowElements = multiRecipientModal.getElementsByClassName("sfEbSendCampaignMultiRecipientsTableContentRow sfEbSendCampaignMultiRecipientsSelected");
                if (contactRecipientRowElements && contactRecipientRowElements.length) {
                    for (i = 0; i < contactRecipientRowElements.length; i++) {  
                        var newContactObject = CreateContactFromRow(contactRecipientRowElements[i]);
                        if (newContactObject) {
                            multipleContactArray.push(newContactObject.GetContact());
                            multiRecipientRowSelectedCounter++;
                        }
                    }
                }
            }

            return multipleContactArray;
        },
        GetCampaignSalesforceListViewObjectsHandleResponse: function (response) {
            const loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceObjectLoader');
            if (response && response.success) {
                var values = response.values;
                if (values && values.length > 0) {

                    PopulateMultiRecipientSalesforceObjectTypeDropDown(JSON.parse(values));
                    if (loaderDiv) {
                        loaderDiv.innerHTML = "";
                    }
                } else {
                    if (loaderDiv) {
                        loaderDiv.innerHTML = 'No Salesforce objects found, please check your Salesforce access.';
                    }
                }
            } else {
                if (loaderDiv) {
                    loaderDiv.innerHTML = 'Error in retrieving your Salesforce object';
                }
            }
        },
        GetListViewsForObjectHandleResponse: function(response) {
            if (response && response.success) {
                var values = response.values;
                if (values && values.length > 0) {

                    PopulateMultiRecipientSalesforceListViewsDropDown(JSON.parse(values), response.objectName);

                } else {
                    var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceListviewLoader');
                    if (loaderDiv) {
                        loaderDiv.innerHTML = 'No List Views Found for salesforce object';
                    }
                }
            } else {
                var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceListviewLoader');
                if (loaderDiv) {
                    loaderDiv.innerHTML = 'Error in retrieving Salesforce list views for object';
                }
            }
        },
        /**
         * load people from salesforce list view into recipient table
         * @returns {} 
         */
        GetPeopleFromListViewHandleResponse: function (response) {

            if (response && response.success) {
                if (response.values) {
                    const allSalesforceRecords = JSON.parse(response.values);                    

                    const allEmailAddresses = allSalesforceRecords.map(x => x.EmailAddress).filter(x=>x);
                    const salesforceTabSectionTabContainer = document.querySelector("#sfEbSendCampaignMultiRecipientsSalesforceTabSection .sfEbSendCampaignMultiRecipientsTabSectionContainer");
                    
                    this.LoadPeopleIntoMultiTabSalesforceTable(allSalesforceRecords);

                    if (enableCampaignDupeCheck) {
                         // Show a loading element while we do the importing
                        const loadingMessageContainer = createDuplicateLoaderElement(true);
                        salesforceTabSectionTabContainer.appendChild(loadingMessageContainer);
                        
                        checkIfRecipientsAreLiveOnACampaign(allEmailAddresses, [salesforceTabSectionTabContainer], duplicateWarningMessageType.MultipleContact, function () {
                            loadingMessageContainer.remove();
                        });
                    }
                } else {
                    var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceListviewLoader');

                    if (loaderDiv) {
                        loaderDiv.innerHTML = 'Error in retrieving Salesforce records';
                    }
                }
            } else {
                var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceListviewLoader');

                if (loaderDiv) {
                    loaderDiv.innerHTML = 'Error in retrieving Salesforce records';
                }
            }
            UpdateMultiRecipientsTotalContactsSelectedHeaderText();
        },
        /**
         * laod peopel from salesforce list view into table
         * @returns {} 
         */
        LoadPeopleIntoMultiTabSalesforceTable: function (people) {
            if (domElements.MultiRecipientSalesforceTableData) {

                domElements.MultiRecipientSalesforceTableData.innerHTML = '';

                multiRecipientRowSelectedCounter = domElements.MultiRecipientModal.getElementsByClassName("sfEbSendCampaignMultiRecipientsSelected").length;
                if (people && people.length > 0) {

                    var i = 0;
                    for (i; i < people.length; i++) {
                        const newRecipientRow = CreateRecipientRow([people[i].Firstname, people[i].LastName, people[i].EmailAddress, people[i].Company], true);
                        newRecipientRow.setAttribute("salesforceid", people[i].SalesforceId);
                        domElements.MultiRecipientSalesforceTableData.appendChild(newRecipientRow);
                    }

                    people = null;
                } else {
                    const noRecipientsRow = CreateElement("tr", "sfEbSendCampaignMultiRecipientsTableNoRecipientsRow");
                    noRecipientsRow.appendChild(CreateElement("td", null, "No recipients found", null, { colspan: 100 }));
                    domElements.MultiRecipientSalesforceTableData.appendChild(noRecipientsRow);
                }

                var loaderDiv = document.getElementById('sfEbSendCampaignMultiRecipientsTabSalesforceListviewLoader');

                if (loaderDiv) {
                    loaderDiv.innerHTML = '';
                }

                if (domElements.MultiRecipientSalesforceListviewDropdown) {
                    EnableMultiRecipientSalesforceListviewDropdown();
                }
            }
        },
        /**
         * Creates the Salesforce List view tab section
         * @param {} sectionId 
         * @returns {} 
         */
        CreateSalesforceListViewSection: function(sectionId) {
            var recipientsSection = CreateElement("div", sectionId, "SF LIST VIEW SECTION");

            return recipientsSection;
        },
        /**
         * Clears the tables out for each tab in multi recipients
         * @returns {} 
         */
        ResetMultiRecipientTableData : function() {
            if (domElements.MultiRecipientCsvTableData != null) {
                ClearAndPopulateMultiRecipientTableData(domElements.MultiRecipientCsvTableData, null);
            }

            if (domElements.MultiRecipientTableData != null) {
                ClearAndPopulateMultiRecipientTableData(domElements.MultiRecipientTableData, null);
            }

            if (domElements.MultiRecipientSalesforceTableData != null) {
                ClearAndPopulateMultiRecipientTableData(domElements.MultiRecipientSalesforceTableData, null);
            }

            // clear inputs and drop downs from multi select model
            this.ClearMultiRecipientSelectors();

            multiRecipientRowSelectedCounter = 0;
            multiRecipientComposeErrorCounter = 0;
            UpdateMultiRecipientsTotalContactsSelectedHeaderText();
        },
        /**
         * clear all inputs and selectors in the multi recipient model
         * @returns {} 
         */
        ClearMultiRecipientSelectors: function() {
            if (domElements.MultiRecipientFirstNameInput) {
                domElements.MultiRecipientFirstNameInput.value = '';
            }

            if (domElements.MultiRecipientLastNameInput) {
                domElements.MultiRecipientLastNameInput.value = '';
            }

            if (domElements.MultiRecipientEmailInput) {
                domElements.MultiRecipientEmailInput.value = '';
            }

            if (domElements.MultiRecipientCompanyInput) {
                domElements.MultiRecipientCompanyInput.value = '';
            }

            if (domElements.MultiRecipientCsvInput) {
                domElements.MultiRecipientCsvInput.value = '';
            }

            if (domElements.MultiRecipientSalesforceObjectDropdown) {
                this.ClearCustomDropdown(domElements.MultiRecipientSalesforceObjectDropdown);
            }

            if (domElements.MultiRecipientSalesforceListviewDropdown) {
                this.ClearCustomDropdown(domElements.MultiRecipientSalesforceListviewDropdown);
                $(domElements.MultiRecipientSalesforceListviewDropdown).trigger("chosen:updated");
            }
        },
        /**
         * clear custom dropdown
         * @returns {} 
         */
        ClearCustomDropdown: function(dropdown) {
            var selectionText = dropdown.getElementsByClassName('ebsta-selectedtext')[0];
            var dropDown = dropdown.getElementsByClassName('ebsta-dropdown')[0];

            if (selectionText && dropDown) {
                selectionText.innerText = '';
                dropdown.setAttribute('selectedvalue', '');
                dropDown.innerHTML = '';
            }
        },
        /**
         * Clears all previous values from the multi-recipients popup, ie input boxes, selected csv file etc.
         */
        ClearRecipientsModal: function () {
            //input boxes in recipients tab.
            $(".sfEbSendCampaignMultiRecipientsTabSectionHeader input").val("");

            //csv selected file.
            $("#sfEbSendCampaignMultiRecipientsCsvInput input").val("");

            //csv selected label.
            $(".fileupload-text").text("Choose File");
        },
        AddFollowUpsCheckBoxChangeEvent: function (followUpCheckBox) {
            if (followUpCheckBox != null) {
                followUpCheckBox.addEventListener("change", function () {
                    lThisCampaignManagerInstance.UpdateSubjectLines();
                });
            }
        },
        //Creates the container which holds the buttons above the flow (Send Later, Email Tracking and Complex Add)
        CreateSequenceHeaderButtons: function() {
            var buttonsContainer = CreateElement("div", "", "", "sfEbSendCampaignSequenceHeaderButtonsContainer");

            //Add to salesforce button
            domElements.ComplexAddButton = CreateElement("div", "ebsta-button sfComplexAddSmallCompose sfEbCampaignAddToSalesforceButton ebsta-tooltip", "", "EbstaBtCampaignsAddToSalesforce");
            domElements.ComplexAddButton.title = "Add emails to Salesforce";
            buttonsContainer.appendChild(domElements.ComplexAddButton);
            AttachClickListenerToAddToSalesforceButton(domElements.ComplexAddButton, lComplexAddClosure);

            //Send Later button
            domElements.SendLaterButton = CreateElement("div", "sfEbSendLaterBtn ebsta-button", null, "sfEbStaCampaignEmailScheduleLaterFromFlowPage", "", "EbstaBtCampaignsSendLater");
            domElements.SendLaterButton.title = "Click to Schedule email to be sent at a later date.";
            buttonsContainer.appendChild(domElements.SendLaterButton);
            this.AttachListenerToScheduleLater(domElements.SendLaterButton, true);
            this.CheckSendLaterHasBeenScheduledAlready();

            //Email Tracking button
            domElements.EmailTrackingButton = CreateElement("div", "ebsta-button sfEbstaGmailEmailTracking sfEbstaGmailEmailTrackingNotSelected", "", "EbstaBtCampaignsEmailTracking");
            domElements.EmailTrackingButton.title = "Email Tracking Off";
            var imgSpn = CreateElement("span", "sfEbstaGmailEmailTrackingButton");
            domElements.EmailTrackingButton.appendChild(imgSpn);
            buttonsContainer.appendChild(domElements.EmailTrackingButton);
            AttachClickListenerToEmailTrackingButton(domElements.EmailTrackingButton, true);

            return buttonsContainer;
        },
        /**
         * Checks if Send later has been scheduled before setting the campaign if so sets the right properites to the date/time that was set
         * @returns {} 
         */
        CheckSendLaterHasBeenScheduledAlready: function () {
            if (lEbstaComposeButtonsClosure != null) {
                var sendLaterClosure = lEbstaComposeButtonsClosure.GetSendLaterClosure();
                if (sendLaterClosure != null) {
                    if (sendLaterClosure.IsScheduled()) {
                        firstEmailScheduledLaterFinalDate = sendLaterClosure.GetScheduledDate();
                        firstEmailScheduledLater = true;
                        this.ScheduleLaterButton(true);
                        this.SetLastDateSelected(firstEmailScheduledLaterFinalDate);
                    } else {
                        this.ScheduleLaterButton(false);
                        this.ResetAllScheduleLaterVariables();
                    }
                }
            }
        },
        UpdateEmailTrackingButtonState: function(selected) {
            emailTrackingButtonState = selected;
        },
        ResetSendLaterButton: function() {
            if (domElements.SendLaterButton != null) {
                domElements.SendLaterButton.title = "Click to Schedule email to be sent at a later date.";
                RemoveClass(domElements.SendLaterButton, "ebsta-btn-success");
            }
        },
        InsertMainViewInModelBeforeButtonsSection: function(view) {
            var buttonsContainer = document.getElementById('sfEbStaCampaignMgrMainModelButtonsContainer');
            if (buttonsContainer) {
                mainCampaignMgrModel.insertBefore(view, buttonsContainer);
            } else {
                mainCampaignMgrModel.appendChild(view);
            }
        },
        SetPagingArrowListener: function(pagingArrow, functionToCall) {
            pagingArrow.addEventListener("click", function() {
                functionToCall();
            });
        },
        GoBackOnePage: function() {
            if (pageNumber > 1) {
                pageNumber--;
                RunSearch();
            }
        },
        GoForwardOnePage: function() {
            if (pageNumber * itemsPerPage < maxResultsFound) {
                pageNumber++;
                RunSearch();
            }
        },
        /**
         * Click listener for paging numbers
         * @param {} element 
         * @returns {} 
         */
        AttachListenersForCampaignMgrPaging : function(element) {
            if (element != null) {
                element.addEventListener('click', function(e) {
                    lThisCampaignManagerInstance.UpdatePageNumberFromElementClicked(e);
                    RunSearch();
                    lThisCampaignManagerInstance.UpdatePagingNumbers();
                });
            }
        },
        //Creates the container which contains the paging numbers
        CreatePagingContainer: function() {
            var section = CreateElement("section", "center-text", null, "campaignPaging");
            HideElement(section);

            var leftChevron = CreateElement("i", "fa fa-chevron-left", null, "pagingBack");
            section.appendChild(leftChevron);
            this.SetPagingArrowListener(leftChevron, this.GoBackOnePage);

            var pageNumberSection = CreateElement("section", null, null, "campaignPageContainer");
            var firstNumber = CreateElement("a", "campaignPagingCircle circle-selected", 1, null);
            pageNumberSection.appendChild(firstNumber);
            this.AttachListenersForCampaignMgrPaging(firstNumber);

            var seperatorSpan = CreateElement("span", null, "...");
            pageNumberSection.appendChild(seperatorSpan);
            HideElement(seperatorSpan);

            for (var i = 2; i < 9; i++) {
                var nextNumber = CreateElement("a", "campaignPagingCircle", i, null);
                pageNumberSection.appendChild(nextNumber);
                this.AttachListenersForCampaignMgrPaging(nextNumber);
            }

            section.appendChild(pageNumberSection);

            var rightChevron = CreateElement("i", "fa fa-chevron-right", null, "pagingNext");
            section.appendChild(rightChevron);
            this.SetPagingArrowListener(rightChevron, this.GoForwardOnePage);

            domElements.PageNumbersContainer = section;

            return section;
        },
        UpdatePagingNumbers: function() {
            var pagesNeeded = Math.ceil(maxResultsFound / itemsPerPage);

            if (pagesNeeded > 1) {
                ShowElement(domElements.PageNumbersContainer);

                var pageNumberElements = domElements.PageNumbersContainer.getElementsByTagName("a");

                //Get the seperator icon so we can choose to either show or hide later on
                var seperatorSpan = domElements.PageNumbersContainer.getElementsByTagName("span")[0];

                //Helps detect when user presses 1 so we know to reset the state of the paging numbers
                var resetNumber = false;

                RemoveClass(domElements.PageNumbersContainer.getElementsByClassName(campaignClassNames.SelectedCircle)[0], campaignClassNames.SelectedCircle);

                if (pageNumber > 5 && pagesNeeded > 8) {
                    //Show the seperator to indicate hidden previous pages
                    ShowElement(seperatorSpan);

                    AddClass(pageNumberElements[4], campaignClassNames.SelectedCircle);

                    //Make sure any page over 5 that is pressed the number ends up in the middle of the set or the end
                    pageNumberElements[1].innerText = pageNumber - 3;
                    pageNumberElements[2].innerText = pageNumber - 2;
                    pageNumberElements[3].innerText = pageNumber - 1;
                    pageNumberElements[4].innerText = pageNumber;
                    pageNumberElements[5].innerText = pageNumber + 1;
                    pageNumberElements[6].innerText = pageNumber + 2;
                    pageNumberElements[7].innerText = pageNumber + 3;
                } else {
                    resetNumber = true;
                    //Reset back to 1
                    HideElement(seperatorSpan);
                    AddClass(pageNumberElements[pageNumber - 1], campaignClassNames.SelectedCircle);
                }

                for (var i = 0; i < pageNumberElements.length; i++) {
                    //User clicked 1 so reset numbers
                    if (resetNumber) {
                        pageNumberElements[i].innerText = i + 1;
                    }

                    //If theres enough items to fill this page number show it, if not hide
                    if (pagesNeeded >= pageNumberElements[i].innerText) {
                        ShowElement(pageNumberElements[i]);
                    } else {
                        HideElement(pageNumberElements[i]);
                    }
                }
            } else {
                //Hide entire paging if just one page
                HideElement(domElements.PageNumbersContainer);
            }
        },
        /**
         * Ties the campaign manager closure to the Ebsta buttons closure so we can update the buttons to 
         * reflect choices made within the campaign if one has been set
         * @param {} ebstaComposeButtonsClosure 
         * @returns {} 
         */
        SetEbstaComposeButtonsClosure: function (ebstaComposeButtonsClosure) {
            lEbstaComposeButtonsClosure = ebstaComposeButtonsClosure;
        },
        /**
         * Creates the campaign manager button for the compose window
         * @returns {} 
         */
        CreateCampaignManagerButton: function () {
            var td = Createtd("gU ebstaTD");

            var divButton = CreateElement("div", "sfEbstaCampaignMgrButton", null, null);
            divButton.title = 'Add to Cadence';
            if (buttonMode === 1) {
                divButton.appendChild(CreateElement('div', 'sfEbstaGmailCampaignMgr-style1', null, null));
            } else {
                var buttonOldStyle = CreateElement('div', 'sfebstacampaignmgrbuttonOldStyle', null, null);
                buttonOldStyle.appendChild(CreateElement('span', null, null, null));
                divButton.appendChild(buttonOldStyle);
            }

            td.appendChild(divButton);

            domElements.ComposeWindowButton = divButton;

            this.AttachEventListenerToCampaignMgrButton(td);
            return td;
        },
        /**
         * Attach event listener to campaign manager button
         * @param {} element 
         * @returns {} 
         */
        AttachEventListenerToCampaignMgrButton: function (element) {
            if (element != null) {
                element.addEventListener('click', function(e) {
                    lThisCampaignManagerInstance.SetCampaignComposeMode(true);
                    var target = e.target;
                    if (target && target !== e.currentTarget) {

                        lThisCampaignManagerInstance.UpdateEmailTrackingButtonState(lEbstaComposeButtonsClosure.GetEmailTrackingState());

                        if (lThisCampaignManagerInstance.HasCampaignBeenSelected()) {
                            lThisCampaignManagerInstance.LoadAlreadySelectedCampaign();
                        } else {
                            lThisCampaignManagerInstance.LoadRecentlyUsedCampaigns(element);
                        }
                    }

                    CampaignManagerResizing().ResizeCampaignManager();
                });
            }
        },
        /**
         * Returns information if a campaign has been selected
         * @returns {} 
         */
        HasCampaignBeenSelected : function() {
            return campaignDetailsSelected != null;
        },
        GetCampaignDetailsSelected : function() {
            return campaignDetailsSelected;
        },
        /**
         * Creates the button to put into the header in Gmail
         * @returns {} 
         */
        CreateGmailHeaderCampaignButton: function() {
            var divContainer = CreateElement("div", "ebsta-google-header-container");

            var campaignButton = CreateElement("button", "ebsta-btn ebsta-btn-primary ebsta-btn-lg ebsta-iconcontainer-campaignheader-gmail", "", "EbstaBtOpenCampaigns");
            campaignButton.title = "Email Cadences";
            var span = CreateElement("span", "fa fa-bullhorn");
            campaignButton.appendChild(span);

            divContainer.appendChild(campaignButton);

            this.RemoveCampaignManagerModal();
            this.AttachEventListenerToCampaignSelectButon(campaignButton, false);
            return campaignButton;
        },
        ResetMultipleContactArray: function () {
            multipleContactArray = new Array();
        },
        // load a starter popup with the most recently used campain and button to select one
        LoadRecentlyUsedCampaigns: function (element) {
            // lets wipe history here
            currentState.selectedCampaign = null;
            loadingSelectedCampaign = false;
            multipleContactSelectedBool = false,
            multipleContactArray = new Array();

            currentState.button = element;

            this.ShowCampaignMgrStarterPopup(element);

            PreSelectContactFromComposeWindowIntoCampaign(element);
            PreSelectCcAndBccContactsFromComposeWindowIntoCampaign(element);

            var request = {
                MessageType: "GetMostRecentlyUsedCampaigns"
            };

            SendMessageToBackGround(request);

            CreateRecentCampaignsLoader();
            ClearRecentCampaigns();
        },
        //Updates the button on the currentState object (this button represents the last Campaignmanager button clicked)
        UpdateCurrentStateButton : function(buttonElement) {
            currentState.button = buttonElement;
        },
        // remder result from LoadRecentlyUsedCampaigns
        LoadRecentlyUsedCampaignsHandleResponse: function (response) {
            HideElement(domElements.CampaignComposeRecentCampaignLoader);

            if (response.success) {
                if (response.values) {
                    //Reversing these values lets us show them in most recent used order 
                    response.values.reverse();
                    response.values.forEach(function(campaign) {
                        var jsonCampaign = JSON.parse(campaign);

                        if (jsonCampaign) {

                            var id = jsonCampaign.CampaignMgrId;
                            if (id && id.length > 0) {
                                var name = jsonCampaign.CampaignName || "";
                                var divCampaign = CreateElement("div", "sfEbComposeMenuOption sfEbstaCampaign ebsta-campaign-composemenu-item", null, "");
                                divCampaign.setAttribute("sfEbSendCampaignKey", id);

                                var iconDiv = CreateElement("div", null, null, null);
                                iconDiv.className = "sfEbSendCampaignMgrStarterPopupItemBullhornIcon " + icons.Bullhorn;
                                divCampaign.appendChild(iconDiv);
                                divCampaign.appendChild(CreateElement("div", null, name, null));

                                Prepend(divCampaign, domElements.DefaultCampaignMenuPopUp);
                                AttachClickThroughToCampaignFlowView(divCampaign, false);
                            }
                        }
                    });
                }
            } else {
                this.RenderStarterPopupRecentlyUsedCampaignsMessage('Error in reterieving recently used campaigns.');
            }
        },
        /**
         * Renders a message within the campaign menu within the compose window when no recent campaigns have been retrieved
         * @param {} message 
         * @returns {} 
         */
        RenderStarterPopupRecentlyUsedCampaignsMessage: function (message) {
            ShowElementBlock(domElements.CampaignComposeRecentCampaignLoader);
            if (domElements.CampaignComposeRecentCampaignLoader) {
                domElements.CampaignComposeRecentCampaignLoader.innerText = message;
            }
        },
        // call this when user clicks on page, run search or call to get all campaigns with paging
        UpdatePageNumberFromElementClicked: function (e) {
            var element = e.target;
            if (element) {
                var page = element.innerText;
                if (page !== null && typeof page !== 'undefined') {
                    pageNumber = parseInt(page);
                }
            }
        },
        // position and append recently used campaigns above the campaign manager starter popup
        PositionAndShowRecentlyUsedCampaigns: function(recentlyUsedCampaigns) {

            if (starterPopup) {
                var startupLoader = document.getElementById('sfEbSendCampaignMgrStarterPopupLoader');

                var loaderHeight = 0;

                if (startupLoader) {
                    loaderHeight = $(startupLoader).outerHeight(true);
                }

                var offset = starterPopup.getBoundingClientRect();
                recentlyUsedCampaigns.style.top = (offset.top - $(recentlyUsedCampaigns).outerHeight(true))  + loaderHeight + 2 + "px";
                recentlyUsedCampaigns.style.left = offset.left + "px";
                recentlyUsedCampaigns.style.display = "block";
            }
        },
        ClearSearchHistoryAndShowModelMainPage: function() {
            // starter popup exists, wipe search history and show
            var searchInput = document.getElementById("sfEbSendCampaignMgrSearch");
            if (searchInput) {
                searchInput.value = "";
            }

            AddLoaderToMainModelListItems();

            this.SHowMainModelContentDiv();
        },
        SelectCampaignPage: function () {

            this.ResetCampaignPage();
            currentPage.SelectCampaign = true;
            HideCampaignSelectButton();
            this.ChangeCancelButtonText("", true);
        },
        ChangeCancelButtonText: function(text, hide) {
            var closeButton = document.getElementById('sfEbSendCampaignMgrMainModelClose');
            if (closeButton) {
                closeButton.innerText = text;
                if (hide) {
                    HideElement(closeButton);
                } else {
                    ShowElement(closeButton);
                }
            }
        },
        ResetCampaignPage: function () {

            for (var key in currentPage) {
                if (currentPage.hasOwnProperty(key)) {
                    currentPage[key] = false;
                }
            }
        },
        SetCampaignComposeMode: function (value) {
            campaignComposeMode = value;
        },
        HideCampaignComposeMenu: function () {
            if(domElements.DefaultCampaignMenuPopUp != null)
            {
                HideElement(domElements.DefaultCampaignMenuPopUp);
            }
            if (recentlyUsedCampaignsPopup != null)
            {
                HideElement(recentlyUsedCampaignsPopup);
            }
        },
        CallFunctionOnEnterKey: function (inputElem, functionToCall) {
            inputElem.addEventListener("keypress", function (event) {
                var key = event.which || event.keyCode;
                if (key === 13) {
                    event.preventDefault();
                    pageNumber = 1;
                    functionToCall();
                }
            });
        },
        /**
         * Refresh button event for campaign modal
         * @param {} element 
         * @returns {} 
         */
        AttachEventListenerToCampaignRefreshButton : function(element) {
            if (element != null) {
                element.addEventListener('click', function (e) {
                    RunSearch();
                });
            }
        },
        /**
         * Click event for the "New Campaign" button
         * @param {} element 
         * @returns {} 
         */
        AttachCampaignCreationClickListener: function (element) {
            if (webUserId != null) {
                var uri = "https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=31";
                if (element != null) {
                    element.addEventListener('click', function () {
                        OpenNewTab(uri);
                        lThisCampaignManagerInstance.HideCampaignComposeMenu();
                    });
                }
            }
        },
        AttachTrackingAnalysisClickListener: function (element) {
            if (webUserId != null) {
                var uri = "https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=130";
                if (element != null) {
                    element.addEventListener('click', function () {
                        OpenNewTab(uri);
                        lThisCampaignManagerInstance.HideCampaignComposeMenu();
                    });
                }
            }
        },
        CreateMainModelContent : function() {
            var divCampaignContent = CreateElement("div", "", "", "sfEbSendCampaignMgrMainModelContent");

            // div for holding search elements
            var campaignSearchPageHeaderContainer = CreateElement("div", "", "", "sfEbSendCampaignMgrMainModelSearchContainer");

            var searchContainer = CreateElement("div", "", "", "sfEbSendCampaignMgrSearchContainer");

            campaignSearchPageHeaderContainer.appendChild(searchContainer);
            CreateUserDropdown(campaignSearchPageHeaderContainer);

            var searchInput = CreateElement("input", "sfEbStaCampaignMgrInput", "", "sfEbSendCampaignMgrSearch");
            searchInput.placeholder = "Cadence name";
            searchContainer.appendChild(searchInput);
            this.CallFunctionOnEnterKey(searchInput, function () { RunSearch(1) });

            var searchButton = CreateElement("span", icons.MagnifyingGlass, "", "sfEbSendCampaignMgrMainModelSearchButton");
            searchContainer.appendChild(searchButton);
            this.AttachEventListenerToCampaignSearchButton(searchButton);      

            var refreshButton = CreateElement("div", icons.Refresh, null, "sfEbSendCampaignMgrMainModelRefresh");
            this.AttachEventListenerToCampaignRefreshButton(refreshButton);
            campaignSearchPageHeaderContainer.appendChild(refreshButton);

            var newCampaignBtn = CreateElement("div", "sfEbSendCampaignMgrButton", "New Cadence", "EbstaBtNewCampaign");
            this.AttachCampaignCreationClickListener(newCampaignBtn);
            campaignSearchPageHeaderContainer.appendChild(newCampaignBtn);

            // put everything but the title in this div
            divCampaignContent.appendChild(campaignSearchPageHeaderContainer);

            // div for holding comapigns
            var divCampaignsList = CreateElement("div", "", "", "sfEbSendCampaignMgrMainModelCampaignsList");

            // campaign list headers
            var divHeaderContainer = CreateElement("div", "", "", "sfEbSendCampaignMgrMainModelListHeaders");

            // name header
            var divCampaignName = CreateElement("div", "sfEbSendCampaignMgrMainModeHeader sfEbSendCampaignMgrMainModeHeaderName", "Name", "");
            divHeaderContainer.appendChild(divCampaignName);

            // recipients 
            var divCampaignRecipients = CreateElement("div", "sfEbSendCampaignMgrMainModeHeader ebstacampaignrecipientscol", "Recipients", "");
            divHeaderContainer.appendChild(divCampaignRecipients);

            // date added
            var divCampaignDateAdded = CreateElement("div", "sfEbSendCampaignMgrMainModeHeader", "Date Added", "");
            divHeaderContainer.appendChild(divCampaignDateAdded);

            // open rate
            var openRate = CreateElement("div", "sfEbSendCampaignMgrMainModeHeader ebsta-campaigntrackingcell ebsta-campaigntrackingHeader ebsta-campaign-orderby", "Opened", "", { "data-sortfield": 1 });
            openRate.appendChild(CreateElement("span", "fa fa-sort"));
            divHeaderContainer.appendChild(openRate);

            // click rate
            var clickRate = CreateElement("div", "sfEbSendCampaignMgrMainModeHeader ebsta-campaigntrackingcell ebsta-campaigntrackingHeader ebsta-campaign-orderby", "Clicked", "", { "data-sortfield": 2 });
            clickRate.appendChild(CreateElement("span", "fa fa-sort"));
            divHeaderContainer.appendChild(clickRate);

            // reply rate
            if (renderReplyData + "" === "true") {
                var replyRate = CreateElement("div", "sfEbSendCampaignMgrMainModeHeader ebsta-campaigntrackingcell ebsta-campaigntrackingHeader ebsta-campaign-orderby", "Replied", "", { "data-sortfield": 3 });
                replyRate.appendChild(CreateElement("span", "fa fa-sort"));
                divHeaderContainer.appendChild(replyRate);
                this.AttachEventListenerToSortButton(replyRate);
            }

            this.AttachEventListenerToSortButton(openRate);
            this.AttachEventListenerToSortButton(clickRate);

            divCampaignsList.appendChild(divHeaderContainer);

            // campaign list items
            var divItemsContainer = CreateElement("div", "", "", "sfEbSendCampaignMgrMainModelListItems");

            AttachClickThroughToCampaignFlowView(divItemsContainer, true);

            //loader until we get campaigns
            var divLoader = CreateElement("div", "sfEbSendCampaignMgrMainModelSearchItem", "Loading", "");
            divItemsContainer.appendChild(divLoader);

            divCampaignsList.appendChild(divItemsContainer);

            divCampaignContent.appendChild(divCampaignsList);

            // div for paging
            var divCampaignsPaging = this.CreatePagingContainer();

            // we are gonna do paging lets setup listeners
            //AttachListenersForCampaignMgrPaging(divCampaignsPaging);

            divCampaignContent.appendChild(divCampaignsPaging);

            return divCampaignContent;
        },
        /**
         * attach listener function to sort button in campaign list table
         * @param {} element 
         * @returns {} 
         */
        AttachEventListenerToSortButton: function (element) {
            if (element != null) {
                element.addEventListener('click', function (evnt) {
                    var target = evnt.target;
                    if (target) {
                        var container = target.closest('.sfEbSendCampaignMgrMainModeHeader');
                        if (container) {
                            var sortField = container.dataset["sortfield"];
                            if (typeof sortField !== "undefined") {
                                if (sortField === campaignSortField) {
                                    campaignSortAsc = !campaignSortAsc;
                                } else {
                                    campaignSortField = sortField;
                                    campaignSortAsc = true;
                                }
                                RunSearch(1);
                            }
                        }
                    }
                });
            }
        },
        // function to attach event listener to campaign search button
        AttachEventListenerToCampaignSearchButton : function(element) {
            if (element != null) {
                element.addEventListener('click', function () {
                    RunSearch(1);
                });
            }
        },
        ShowMainModelContentPage: function() {
            this.HideMainModelDetailsDiv();
            this.HideMainModelSendLaterScheduler();
            this.ShowCampaignMgrMainModel();

            // select current page
            this.SelectCampaignPage();

            if (mainCampaignMgrModel) {

                var contentPage = document.getElementById('sfEbSendCampaignMgrMainModelContent');
                if (contentPage) {
                    this.ClearSearchHistoryAndShowModelMainPage();
                }
            }
        },
        // render result from LoadAllCampaignsForUser
        LoadAllCampaignsForUserHandleResponse: function (response, search) {

            //Request has been cancelled due to another search just do nothing
            if (response.RequestAborted === true) return;

            var campaignsContainer = document.getElementById('sfEbSendCampaignMgrMainModelListItems');

            if (campaignsContainer) {
                
                if (response.success) {

                    maxResultsFound = response.RowCounter;

                    this.UpdatePagingNumbers();
                    if (response.values) {
                        var foundValues = false;
                        response.values.forEach(function (campaign, index) {

                            if (index === 0) {
                                campaignsContainer.innerHTML = '';
                            }

                            var jsonCampaign = JSON.parse(campaign);

                            if (jsonCampaign) {
                                var id = jsonCampaign.CampaignMgrId;
                                if (id && id.length > 0) {

                                    var divCampaign = CreateElement("div", "sfEbSendCampaignMgrMainModelSearchItem sfEbstaCampaign ebsta-tooltip", "", "");

                                    // campaign name
                                    var name = jsonCampaign.CampaignName || "";

                                    var divName = CreateElement("div", "sfEbSendCampaignMgrMainModelItemColumn sfEbSendCampaignMgrCampaignName ebsta-ellipsis", name, "");
                                    divName.title = name;
                                    
                                    //Only set this attribute if we want this to be clickable
                                    divCampaign.setAttribute("sfEbSendCampaignKey", id);

                                    divCampaign.appendChild(divName);

                                    // campaign recipients
                                    var recipients = jsonCampaign.Recipients || "-";

                                    var divRecipients = CreateElement("div", "sfEbSendCampaignMgrMainModelItemColumn ebsta-ellipsis ebstacampaignrecipientscol", recipients, "");

                                    divCampaign.appendChild(divRecipients);

                                    // campaign date added
                                    var dateAdded = jsonCampaign.DateCreated || "-";
                                    var dateParts = dateAdded.split(/[\-T]+/);
                                    if (dateParts && dateParts.length > 2) {

                                        dateAdded = new Date(dateParts[0], dateParts[1] - 1, dateParts[2]).toDateString();
                                    }

                                    var divdateAdded = CreateElement("div", "sfEbSendCampaignMgrMainModelItemColumn ebsta-ellipsis", dateAdded, "");

                                    divCampaign.appendChild(divdateAdded);

                                    // campaign tracking open and click rate
                                    var openRate = jsonCampaign.NoTrackingStats && jsonCampaign.NoTrackingStats === true ? "-" : parseInt(jsonCampaign.EmailOpenedrate) + "%";
                                    var openrateContainer = CreateElement("div", "sfEbSendCampaignMgrMainModelItemColumn ebsta-campaigntrackingcell", openRate, "");

                                    var clickRate = jsonCampaign.NoTrackingStats && jsonCampaign.NoTrackingStats === true ? "-" : parseInt(jsonCampaign.ClickRate) + "%";
                                    var clickrateContainer = CreateElement("div", "sfEbSendCampaignMgrMainModelItemColumn ebsta-campaigntrackingcell", clickRate, "");

                                    divCampaign.appendChild(openrateContainer);
                                    divCampaign.appendChild(clickrateContainer);

                                    //render reply data.
                                    if (renderReplyData + "" === "true") {
                                        var replyRate = jsonCampaign.NoTrackingStats && jsonCampaign.NoTrackingStats === true ? "-" : parseInt(jsonCampaign.ReplyRate) + "%";
                                        var replyRateContainer = CreateElement("div", "sfEbSendCampaignMgrMainModelItemColumn ebsta-campaigntrackingcell", replyRate, "");
                                        divCampaign.appendChild(replyRateContainer);
                                    }                                    

                                    campaignsContainer.appendChild(divCampaign);
                                    foundValues = true;
                                }
                            }
                        });

                        if (!foundValues) {
                            var message = CreateNoCampaignMessage(search);
                            
                            this.RemoveLoaderAndAddMessageToLoaderDiv(message);
                        }
                    }
                } else {
                    this.RemoveLoaderAndAddMessageToLoaderDiv("error in retrieving used campaigns.");
                }
            }

            CampaignManagerResizing().ResizeCampaignManager();
        },
        RemoveLoaderAndAddMessageToLoaderDiv: function(message) {
            var loaderDiv = document.getElementsByClassName('ebsta-campaign-loaderDiv')[0];

            if (typeof message === "string")
            {
                message = document.createTextNode(message);
            }

            ReplaceNodeWithNewNode(message, loaderDiv);
        },
        ShowElement: function(element) {
            if (element) {
                element.style.display = "block";
            }
        },
        HideMainModelDetailsDiv: function () {
            var mainModelContentDiv = document.getElementById("sfEbSendCampaignMgrMainModelDetails");
            if (mainModelContentDiv) {
                mainModelContentDiv.style.display = "none";
            }
        },
        ShowMainModelDetailsDiv: function () {
            var mainModelContentDiv = document.getElementById("sfEbSendCampaignMgrMainModelDetails");
            if (mainModelContentDiv) {
                mainModelContentDiv.style.display = "block";
            }
        },
        SHowMainModelContentDiv: function() {
            var mainModelContentDiv = document.getElementById("sfEbSendCampaignMgrMainModelContent");
            if (mainModelContentDiv) {
                mainModelContentDiv.style.display = "block";
            }
        },
        HideMainModelContentDiv: function() {
            var mainModelContentDiv = document.getElementById("sfEbSendCampaignMgrMainModelContent");
            if (mainModelContentDiv) {
                mainModelContentDiv.style.display = "none";
            }
        },
        // handle response from search
        RunSearchForUserByNameHandleResponse: function (response) {

            this.LoadAllCampaignsForUserHandleResponse(response, true);
        },
        //assign postion to campaign manager starter popup
        PositionCampaignMgrStarterPopup: function (offset) {

            starterPopup.style.top = (offset.top - $(starterPopup).outerHeight(true)) + "px";
            starterPopup.style.left = offset.left + "px";
        },
        // create if needed and show campaign manager pop up 
        ShowCampaignMgrStarterPopup: function (button) {
            if (starterPopup !== null) {
                // starter popup exists
                this.ShowStarterPopupLoaderDiv();
                ShowElementBlock(starterPopup);
            }
            else
            {
                // starter popup does not exist. create and show
                starterPopup = this.CreateCampaignMgrStarterPopup();
                GmailTools.Compose.InsertMenu(button, starterPopup);
                ShowElementBlock(starterPopup);
            }
        },
        ShowStarterPopupLoaderDiv: function() {
            var loaderDiv = document.getElementById('sfEbSendCampaignMgrStarterPopupLoader');
            if (loaderDiv) {
                var loaderConatiner = document.getElementById('sfEbSendCampaignMgrStarterPopupLoaderDiv');
                if (loaderConatiner) {
                    loaderConatiner.innerHTML = '';
                    loaderConatiner.className = icons.Spinner;
                }
            }
        },
        // create if needed and show campaign manager pop up 
        ShowCampaignMgrMainModel: function () {

            $('#emailTemplateBox').remove();
            $('#ebstaBox').remove();
            if (mainCampaignMgrModel == null || !document.body.contains(mainCampaignMgrModel)) {
                this.RemoveCampaignManagerModal();
                mainCampaignMgrModel = this.CreateCampaignMgrMainModel();
                document.body.appendChild(mainCampaignMgrModel);
            }
            
            mainCampaignMgrModel.style.display = "table";

            this.SetClickListenersOnActionButtons();
        },
        //create campaign starter pop up
        CreateCampaignMgrStarterPopup: function() {
            domElements.DefaultCampaignMenuPopUp = CreateElement("div", "sfEbSendCampaignMgrStarterPopup sfEbComposeMenu", "", "");

            CreateRecentCampaignsLoader();

            // div for selecting other campaigns
            var divSelectCampaign = CreateElement("div", "sfEbComposeMenuOption", null, "sfEbSendCampaignMgrStarterPopupItemSelectCampign");
            divSelectCampaign.appendChild(CreateElement("div", icons.Download, null, null));
            divSelectCampaign.appendChild(CreateElement("div", null, "Select Cadence", null));
            domElements.DefaultCampaignMenuPopUp.appendChild(divSelectCampaign);

            // event listener on click for select button
            this.AttachEventListenerToCampaignSelectButon(divSelectCampaign, true);

            //div for new campaigns
            var divNew = CreateElement("div", "sfEbComposeMenuOption", null, "sfEbSendCampaignMgrStarterPopupItemNew");
            divNew.appendChild(CreateElement("div", icons.PlusSign, null, null));
            divNew.appendChild(CreateElement("div", null, "New Cadence", null));
            this.AttachCampaignCreationClickListener(divNew);
            domElements.DefaultCampaignMenuPopUp.appendChild(divNew);

            //div for link through to knowledge base
            var divKnowledgeBase = CreateElement("div", "sfEbComposeMenuOption", null, "EbstaAnchorSelectCampaignMenuHelp");
            divKnowledgeBase.appendChild(CreateElement("div", icons.QuestionMark, null, null));
            divKnowledgeBase.appendChild(CreateElement("div", null, "Help", null));
            divKnowledgeBase.title = 'Check out our knowledgebase article';
            this.AttachClickThroughListenerToElement(divKnowledgeBase, 'https://ebstalimited.zendesk.com/hc/en-us/articles/115002519188-Using-Email-Campaign');
            domElements.DefaultCampaignMenuPopUp.appendChild(divKnowledgeBase);

            return domElements.DefaultCampaignMenuPopUp;
        },
        AttachClickThroughListenerToElement: function(element, uri) {
            if (element) {
                element.addEventListener('click', function() {
                    OpenNewTab(uri);
                });
            } 
        },
        // function to attach event listener to campaign select button
        AttachEventListenerToCampaignSelectButon : function(element, composeMode) {
            if (element != null) {
                element.addEventListener('click', function () {
                    Open(this, composeMode);
                });
            }
        },
        HideMultiRecipientModal : function() {
            HideElement(domElements.MultiRecipientModal);
        },
        AttachEventListenerToCampaignModelSelectButton: function(element) {
            if (element) {
                element = CloneAndOverwriteNode(element);
                element.addEventListener('click', function() {
                    lThisCampaignManagerInstance.ChooseAndExecuteSelectAction();
                });
            }
        },
        //create campaign manager main model
        CreateCampaignMgrMainModel: function () {

            var model = CreateElement("div", null, null, "sfEbSendCampaignMgrMainModel");

            //div for holding the title and close button
            var titleContainer = CreateElement("div", "", "", "sfEbStaCampaignMgrMainModelHeaderAndTitle");

            var closeIcon = CreateElement("span", "sftitle-close");
            closeIcon.addEventListener("click", function () {
                closeCampaignMgr();
            });
            titleContainer.appendChild(closeIcon);

            // title
            var title = CreateElement("div", "sftitle-text", "Cadences", "sfEbSendCampaignMgrMainModelTitle");
            titleContainer.appendChild(title);

            var helpIcon = EbstaModalTools.CreateHelpIcon("ebsta-help-icon-right", null, "Visit Our Knowledge Base");
            helpIcon.id = "EbstaAnchorCampaignHelp";
            this.AttachClickLsitenerToKnowledgebaseArticle(helpIcon);
            titleContainer.appendChild(helpIcon);

            titleContainer.appendChild(CreateElement("a", "sfLink", "Manage Cadences", "EbstaAnchorManageCampaigns", {
                href: "https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=30",
                target: "_blank"
            }));
            if (renderTrackingAnalysis == 'true') {
                titleContainer.appendChild(CreateElement("a", "sfLink", "Tracking Analysis", "EbstaAnchorTrackingAnalysisCampaigns", {
                    href: "https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=130",
                    target: "_blank"
                }));
            }

            var buttonscontainer = CreateElement("div", "", "", "sfEbStaCampaignMgrMainModelButtonsContainer");

            // select button
            var selectButton = CreateElement("div", "sfEbSendCampaignMgrButton", "Select", "sfEbSendCampaignMgrMainModelSelect");
            selectButton.style.display = 'none';
            buttonscontainer.appendChild(selectButton);

            // Send Campaign Button for Single/Multi Recipients mode out of a compose window
            var sendButton = CreateElement("div", "sfEbSendCampaignMgrButton", "Send", "sfEbSendCampaignMgrMainModelSend");
            HideElement(sendButton);
            buttonscontainer.appendChild(sendButton);

            // close button
            var closeButton = CreateElement("div", "sfEbSendCampaignMgrButton", null, "sfEbSendCampaignMgrMainModelClose");
            HideElement(closeButton);
            buttonscontainer.appendChild(closeButton);

            model.appendChild(titleContainer);
            model.appendChild(buttonscontainer);
            var contentContainer = this.CreateMainModelContent();
            model.insertBefore(contentContainer, buttonscontainer);

            return model;
        },
        /**
         * Close button click event listener
         * @param {} closeButton 
         * @returns {} 
         */
        AttachEventListenerToCampaignModelCloseButton : function(closeButton) {
            if (closeButton != null) {
                closeButton = CloneAndOverwriteNode(closeButton);
                closeButton.addEventListener('click', function () {
                    lThisCampaignManagerInstance.ChooseAndExecuteCloseAction();
                    lThisCampaignManagerInstance.ClearRecipientsModal();
                });
            }
        },
        AttachEventListenerToCampaignModelSendButton: function (element) {
            if (element) {
                element = CloneAndOverwriteNode(element);
                element.addEventListener('click', function () {
                    lThisCampaignManagerInstance.ChooseAndExecuteSendAction();
                });
            }
        },
        ClickThroughToKnowledgebaseArticle: function () {
            var uri = 'https://ebstalimited.zendesk.com/hc/en-us/articles/115002519188-Using-Email-Campaign';
            if (currentPage.SelectCampaign) {
                OpenNewTab(uri + '#anchor1');
            }
            if (currentPage.CampaignDetail) {
                OpenNewTab(uri + '#anchor2');
            }
            if (currentPage.EditEmailTemplate) {
                OpenNewTab(uri + '#anchor5');
            }
            if (currentPage.ScheduleLater) {
                OpenNewTab(uri + '#anchor6');
            }
        },
        AttachClickLsitenerToKnowledgebaseArticle: function(element) {
            if (element) {
                element.addEventListener('click', this.ClickThroughToKnowledgebaseArticle);
            }
        },
        //Sets the complex add button to be selected
        SetComplexAddSelectedState : function() {
            UpdateComplexAddButton(domElements.ComplexAddButton, true);
        },
        SetComplexAddButtonState: function (complexAddClosure) {
            ResetClickListenerOnAddToSalesButton(complexAddClosure, true);
            var complexAddValues = complexAddClosure.GetComplexAddValues();
            if (complexAddValues != null &&
                (complexAddValues.ComplexAddWhoAndWhatIds.length > 0 || complexAddValues.ComplexAddAdditionalFields.length > 0)) {
                UpdateComplexAddButton(domElements.ComplexAddButton, true);
            } else {
                UpdateComplexAddButton(domElements.ComplexAddButton, false);
            }
        },
        ClearComplexAddState: function () {
            ResetClickListenerOnAddToSalesButton(null, true);
            UpdateComplexAddButton(domElements.ComplexAddButton, false);
        },
        ShowEmailTrackingOffWarning: function (confirmCallBack) {
            ShowEmailTrackingOffWarning(confirmCallBack);
        }
    }
}

/**
 * CSV Closure
 * @returns {} 
 */
function CreateCsvManagerObject(enableCsvMergeFields) {
    var targetElement = null;
    var rowsFailed = null;
    var elementRowProducer = null;

    /**
    * Checks all contents on a CSV line returning true if blank or false if it finds one value within the array
    * @param {} lineArrayData 
    * @returns {} 
    */
    function IsLineBlank(lineArrayData) {
        for (var i = 0; i < lineArrayData.length; i++) {
            if (lineArrayData[i] !== "") return false;
        }
        return true;
    }

    /**
     * Takes an entire CSV and returns an array line by line which contains another array for each column of data
     * Credit goes to https://stackoverflow.com/a/41563966/1005083 as is complies with
     * https://tools.ietf.org/html/rfc4180#page-2
     * @param {} text 
     * @returns {} 
     */
    function CsvToArray(text) {
        // MAC OSX Excel seems to save CSV with CR (\r) to divide a line which is not the standard, 
        // so replacing all CR ("\r") or CRLF ("\r\n") with LF ("\n') should help us get out of this
        text = text.replace(/[\r]|[\r\n]/g, '\n');
        
        var p = '', row = [''], ret = [row], i = 0, r = 0, s = !0, l;
        for (l in text) {
            l = text[l];
            if ('"' === l) {
                if (s && l === p) row[i] += l;
                s = !s;
            } else if (',' === l && s) l = row[++i] = '';
            else if ('\n' === l && s) {
                if ('\r' === p) row[i] = row[i].slice(0, -1);
                row = ret[++r] = [l = '']; i = 0;
            } else row[i] += l;
            p = l;
        }
        return ret;
    };

    /**
     * Loops through all lines of a CSV file and finds the max amount of columns used
     * @param {Array} allCsvLines - Array of lines of a CSV file
     * @returns {number} - The maxmium amount of columns used found
     */
    function getMaxNumberOfColumnsUsed(allCsvLines) {
        let maxColumns = 0;
        for (var n = 0; n < allCsvLines.length; n++) {
            const lineLength = allCsvLines[n].length;
            if (lineLength > maxColumns) {
                maxColumns = lineLength;
            }
        }
        return maxColumns;
    }

    function loadIntoTable(event, callBack) {
        if (targetElement !== null && typeof targetElement !== "undefined"
            && elementRowProducer !== null && typeof elementRowProducer !== "undefined") {
            var csv = event.target.result;
            var allTextLines = CsvToArray(csv);
            targetElement.innerHTML = '';

            var isCsvValid = true;
            var csvErrorMessage = '';
            var validRows = 0;

            for (var n = 0; n < allTextLines.length; n++) {

                if (n === 0) {
                    var headerLine = allTextLines[n];
                    // this is the first row and should be the header, if there are not four columns break and point user to the knowledge base
                    if (headerLine.length < 4) {
                        isCsvValid = false;
                        csvErrorMessage = 'Invalid CSV File.';
                        break;
                    } else if (headerLine.length >= 50) {
                        isCsvValid = false;
                        csvErrorMessage = 'Invalid CSV File : The maximum number of columns is 50.';
                        break;
                    } else {

                        if (enableCsvMergeFields) {
                            var csvTableHeaderRow = document.querySelector("#sfEbSendCampaignMultiRecipientsCsvTabSection thead tr");

                            if (csvTableHeaderRow) {

                                // clear any existing table headers before rebuilding them.
                                csvTableHeaders = [];

                                // clear existing header divs we may have added in a previous file load.
                                $('.csvbonusfieldheader').remove();

                                const numberOfColumnsUsed = getMaxNumberOfColumnsUsed(allTextLines);

                                for (var j = 0; j < numberOfColumnsUsed; j++) {

                                    var headerName = headerLine[j];
                                    if (!headerName) {
                                        headerName = "";
                                    }

                                    // add it to the bonus header list.
                                    csvTableHeaders.push(headerName);

                                    // we have bonus columns.
                                    if (j >= 4)
                                    {
                                        var newHeader = CreateElement("th", "sfEbSendCampaignMultiRecipientsTableHeader csvbonusfieldheader", headerName);

                                        // add header element to the table.
                                        csvTableHeaderRow.appendChild(newHeader);
                                    }
                                }
                            }
                        }
                    }

                    continue;
                };

                // render row of results
                if (!IsLineBlank(allTextLines[n])) {

                    validRows++;

                    var rowElement = elementRowProducer(allTextLines[n], true);
                    targetElement.appendChild(rowElement);
                }
            }

            // Check if we have any errors to render
            let errorMessageToRender = null;
            if (isCsvValid === false) {
                errorMessageToRender = csvErrorMessage;
            } else if (validRows === 0) {
                errorMessageToRender = "No valid rows found"
            };

            if(errorMessageToRender) {
                const tableBody = document.querySelector("#sfEbSendCampaignMultiRecipientsCsvTabSection tbody");
                if (tableBody) {
                    const row = CreateElement("tr", "sfEbSendCampaignMultiRecipientsTableNoRecipientsRow");
                    const cell = CreateElement("td", null, errorMessageToRender, null, { colspan: 100 });
                    row.appendChild(cell);
                    tableBody.appendChild(row);
                }
            }
        }

        callBack();            
    }

    return {
        InsertCsvIntoTable: function (file, target, rowProducer, callBack) {
            targetElement = target;
            elementRowProducer = rowProducer;
            if (window.FileReader) {
                // FileReader are supported.
                var reader = new FileReader();
                // Handle errors load
                reader.onload = function (e) {
                    loadIntoTable(e, callBack);
                }
                reader.onerror = this.errorHandler;
                // Read file into memory as UTF-8      
                reader.readAsText(file);
            } else {
                //console.log("File reader is not supported");
            }
        },
        errorHandler: function(evt) {
            if (evt.target.error.name === "NotReadableError") {
                alert("Unable to read file !");
            } else {
                //console.log(evt.target.error);
            }
        }
    }
}

/**
 * Resizes the campaign manager based on screen size
 * @returns {} 
 */
function CampaignManagerResizing() {
    var gMainModalElement = document.getElementById("sfEbSendCampaignMgrMainModel");
    var gResultsListContainer = document.getElementById("sfEbSendCampaignMgrMainModelListItems");

    /**
     * Takes an element within the campaign manager dom, along with a max and min size and resizes the element so that it fits the screen
     * @param {} element 
     * @param {} maxHeightOfElement 
     * @param {} minHeightOfElement 
     * @returns {} 
     */
    function ResizeElementToFit(element, maxHeightOfElement, minHeightOfElement) {
        //Gets the overall height of the element we want to change
        var currentHeightOfResizeableElement = element.offsetHeight;

        //Takes the overall window height and then the overall campaign manager DOM height
        var currentOffset = window.innerHeight - gMainModalElement.offsetHeight - 10;

        //If the offset is negative then the current state of the campaign window must be too big
        if (currentOffset < 0) {
            //Calculate a new smaller height for the element
            var newSmallerHeight = currentHeightOfResizeableElement + currentOffset;

            //If the height is less than the allowed min height that has been set then just set it to this
            if (newSmallerHeight < minHeightOfElement) {
                newSmallerHeight = minHeightOfElement;
            }

            element.style.height = newSmallerHeight + "px";
        }
        //If here is not offset and the element hasn't reached its max height then make the height larger
        else if (currentHeightOfResizeableElement < maxHeightOfElement) {
            //Calculate the new bigger height
            var newBiggerHeight = currentHeightOfResizeableElement + currentOffset;

            //If the new height is larger than what was calcualted then just set its max height
            if (newBiggerHeight > maxHeightOfElement) {
                element.style.height = maxHeightOfElement + "px";
            } else  {
                element.style.height = newBiggerHeight + "px";
            }
        }
    }

    /**
     * Resizes the Home screen on Campaigns
     * @returns {} 
     */
    function ResizeCampaignManagerHomeScreen() {
        ResizeElementToFit(gResultsListContainer, 350, 100);
    }

    /**
     * Resizes the Campaign Details Screen
     * @returns {} 
     */
    function ResizeCampaignDetailsScreen() {
        var campaignSequenceContainer = document.getElementById("sfEbSendCampaignSequence");
        if (IsVisible(campaignSequenceContainer)) {
            ResizeElementToFit(campaignSequenceContainer, 285, 50);
        }
    }

    /**
     * Resizes the Froala editor screen
     * @returns {} 
     */
    function ResizeFroalaEditor() {
        var froalaEditor = document.getElementsByClassName("fr-wrapper");
        if (froalaEditor.length > 0 && IsVisible(froalaEditor[0])) {
            ResizeElementToFit(froalaEditor[0], 200, 70);
        }
    }

    return {
        /**
         * Resizes campaign manager elements
         * @returns {} 
         */
        ResizeCampaignManager : function() {
            if (IsVisible(gMainModalElement)) {
                ResizeCampaignManagerHomeScreen();
                ResizeCampaignDetailsScreen();
                ResizeFroalaEditor();
            }
        }
    }
};
