﻿/// <reference path="BackgroundEngine.js" />
/// <reference path="Shared/domcreation.js" />
/// <reference path="Customisations/batteryventure.js"/>
/// <reference path="modules/googleaccount.js.js"/>

var whatProductApp = "10"; // Free
var version = "0";
var storeFirstRecipient = false;
var trackingPollObject;
var gmailAuthFailed = false;
var sfAuthRequestTabId; //Stores the ID of the tab that sent a request for SF auth
var googleAuthRequestTabId = null; //Stores the id of the tab that sent request for Google auth
var shouldWeShowCalendarSync = false; //Helps track when a user presses Google Calendar Sync button in the bubble that we show the calendar sync on load on either Gmail/Google Calendar
var dateCheckForPreviewEventRequest; //Helps keep a track of the last preview events request sent from bar.js so we only display the most recent one.

var shouldWeShowContactSync = false; //Helps track when a user presses Google Contact Sync button in the bubble that we show the contact sync on load
var dateCheckForPreviewContactRequest; //Helps keep a track of the last preview contact sync request sent from bar.js so we only display the most recent one.

/** Gmail Email Sending Global Variables **/
var global_AddGmailTaskSenderTabId = null; // Helps track the actual tab Id of a Save on Send action when used outside of the main Gmail Window

/** Gmail Email Sending  Global Variabkes **/

/** Gmail Complex Add Settings **/

    //Variable to store the complex add settings used in bar.js
    var globalComplexAddSettings = {
        customFieldInfo : null, // Custom fields to be inserted into Complex Add layout
        searchableObjects : null // Objects user has chosen to search for
    }

    //Variable to store the suggestion settings used in bar.js
    var globalSuggestionSearchSettings = {
        searchableObjects: null // Objects user has chosen to search for
    }

    UpdateComplexAddSettingsVariable();

    //If there were previous settings saved in localstorage then update the variable to keep the users previous settings
    if (localStorage.ComplexAddSearchableObjects) {
        globalComplexAddSettings.searchableObjects = JSON.parse(localStorage.ComplexAddSearchableObjects);
    }
    
    if (localStorage.ComplexAddSuggestionObjects) {
        //console.log("localStorage.ComplexAddSuggestionObjects: ");
        //console.log(localStorage.ComplexAddSuggestionObjects);
        globalSuggestionSearchSettings.searchableObjects = JSON.parse(localStorage.ComplexAddSuggestionObjects);
    }

/** Gmail Complex Add Settings **/
var globalGmailProperties;

var globalTaskTypes; // Global variable to hold the task type values for the Complex Add field

var globalAbTestingObject = null;

/** SendLater Variables **/
var sendLaterDraftStorageArray = new Array();

/**
 * If true then we currently have a layoutmanager request active, if false then not
 */
var globalIsThereALayoutManagerRequestActive = false;

// stops client logging to secure.
var disableLogging = false;

// check if we should be disabling logging.
CheckLogging();

function CheckLogging(){
    var companySettings = GetCompanySettings();

    if (typeof companySettings !== "undefined" && companySettings !== null) {
        // exlcude knowbe4/celarity/Unwindsoftware ltd
        if (companySettings.CompanyId === 'e2ff6a39-8492-418e-b867-6cff3c16027f' || companySettings.CompanyId === '803ba798-0e72-46a2-b9b9-5b1034258432'  || companySettings.CompanyId === 'ffefd474-c38c-4936-94c5-9f7549e41ced') {
            disableLogging = true;
        }
    }
}

/** Module to hold settings */
function EbstaSettings () {

    var settings = {
        campaignSettings: {
            lastSearchedCampaignUsers: null,
            lastSelectedMailbox: null,
        }
    };

    // If we previously saved settings from a session then pre-load them into the variable
    if (localStorage.EbstaSettings) {
        try {
            settings = JSON.parse(localStorage.EbstaSettings);
        } catch (error) {
            console.log("Ebsta settings Parseing error", error);
        }
    }

    /** 
     * Saves settings into Local Storage to persist user settings
     * during the extension updating or a new chrome instance being opened
     */
    function SaveSettingsIntoLocalStorage() {
        localStorage.EbstaSettings = JSON.stringify(settings);
    }

    /** Saves the value for whatever the user last selected in the owner dropdown when searching for a campaign */
    function SaveLastSearchedCampaignUsers(arrayOfIds) {
        if (arrayOfIds != null && arrayOfIds instanceof Array) {
            settings.campaignSettings.lastSearchedCampaignUsers = arrayOfIds;
            SaveSettingsIntoLocalStorage();
        }
    }

    /**
     * Keep track of which mailbox they selected last so we can prompt this for future selections
     * @param {string} mailbox - The name of the mailbox last chosen
     */
    function SaveLastSelectedMailbox(mailbox) {
        settings.campaignSettings.lastSelectedMailbox = mailbox;
        SaveSettingsIntoLocalStorage();
    }

    return {
        GetCampaignSettings: function () {
            return settings.campaignSettings;
        },
        SaveLastSearchedCampaignUsers: SaveLastSearchedCampaignUsers,        
        SaveLastSelectedMailbox: SaveLastSelectedMailbox
    }
}

var gEbstaSettings = new EbstaSettings();

/**
 * Helper object to contain all endpoints
 * @returns {} 
 */
function globalRequestEndPoints() {
    var connectorServiceEnpoint = localStorage.MainConnector + "/Connector.svc/";

    return {
        GetCalendarSyncRequestEndpoint : function() {
            return connectorServiceEnpoint + "CalendarSyncRequest";
        },
        GetLayoutManagerEndpoint : function() {
            return connectorServiceEnpoint + "GetLayoutManager";
        },
        /** CrmConnect.svc endpoint */
        GetCrmConnectEndpoint: function () {
            return localStorage.SelectUrl;
        }
    }
}

// Create Context Menus. 
var rightClickSearchOption = chrome.contextMenus.create({ "title": "Ebsta Search", "contexts": ['selection'], "onclick": SelectionSearch });

/** Sets us up for specific customisations */
function SetupCustomisations() {
    if (typeof gBatteryVentureCustomisation != "undefined") {
        gBatteryVentureCustomisation.Initialize(GetLayoutManagerFromLocalStorage(), rightClickSearchOption);
    }
}
SetupCustomisations();

/**
 * Returns a JSON string of the 3 core values you need for all requests
 */
function GetDefaultRequestParameters() {
    return '{"sessionId": "' + localStorage.SessionCode + '",' +
        '"userKey": "' + localStorage.WebUserId + '",' +
        '"crmId": "' + localStorage.CrmId + '"}';
}

/** 
 *  Module to hold all campaign manager values 
 *  @constructor
 */
function CampaignManagerValues(){
    var lLastRequestDate = null; // Last actual date we fired a request
    var lMS_PER_DAY = 1000 * 60 * 60 * 24; //Milliseconds per day

    /** Initial Setup Values needed for Campaigns */
    var lCampaignSetup = {
        TaskAndDecisions: null,
        CompanyUsers: null,
        EnableAutoRelateComplexAddForMultiRecipient: false,
        SendFromMailboxes: null, // List of mailboxes the user can choose to send the campaign from
        SalesforceCampaigns: null, //A list of Salesforce Campaigns
        EnableCsvMergeFields: false,
        EnableCampaignDupeCheck : false
    };

    var lCampaignSetupRequest = null, lTaskAndDecisionTypesRequest = null,
        lLastCampaignSearchRequest = null;

    /**
     * Calculates the difference in days between 2 different dates
     * @param {Date} a - first date to compare
     * @param {Date} b - second date to compare
     * @returns {int} - difference between two dates 
     */
    function DateDiffInDays(a, b) {
        // Discard the time and time-zone information.
        var utc1 = Date.UTC(a.getFullYear(), a.getMonth(), a.getDate());
        var utc2 = Date.UTC(b.getFullYear(), b.getMonth(), b.getDate());

        return Math.floor((utc2 - utc1) / lMS_PER_DAY);
    }

    /**
    * Works out the last time we run a request to get the information and if more than a day then go and do it again
    * @returns {boolean} - Returns true if we need to get the setup again 
    */
    function DoWeNeedToGoGetInformationAgain() {
        if (lCampaignSetup.TaskAndDecisions === null ||
            lCampaignSetup.CompanyUsers === null ||
            lLastRequestDate === null ||
            DateDiffInDays(lLastRequestDate, new Date()) > 0) {
            lLastRequestDate = new Date();
            return true;
        }
        return false;
    }

    /** 
     *  Creates an object which should be sent back to wherver the message has come from to signal it was succesful
     * @returns {object} - Object contain setup details needed for Campaigns
     **/
    function CreateSuccessfulCampaignSetupResponse() {
        var layoutManager = GetLayoutManagerFromLocalStorage();
        if (layoutManager && layoutManager.d && layoutManager.d.CompanySettings) {
            lCampaignSetup.EnableAutoRelateComplexAddForMultiRecipient = layoutManager.d.CompanySettings.CampaignAutoRelate == true;
        }

        //Find out if we have a last selected mailbox then see if exists in the list
        var lastSelectedMailbox = gEbstaSettings.GetCampaignSettings().lastSelectedMailbox;
        if (lastSelectedMailbox) {
            for (var i = 0; i < lCampaignSetup.SendFromMailboxes.length; i++) {
                var sendFromMailbox = lCampaignSetup.SendFromMailboxes[i];
                let lastSelectedMailboxUsername = null;
                let lastSelectedMailboxTokenType = null;

                // We transistioned this storage format from string to object so first we look for it by object value 
                // if not then we'll just presume it's a string like it was before
                if (lastSelectedMailbox.Mailbox != null) {
                    lastSelectedMailboxUsername = lastSelectedMailbox.Mailbox;
                } else {
                    lastSelectedMailboxUsername = lastSelectedMailbox;
                }

                if (lastSelectedMailbox.TokenType != null) {
                    lastSelectedMailboxTokenType = lastSelectedMailbox.TokenType;
                }

                if (sendFromMailbox.Mailbox != null && sendFromMailbox.Mailbox.toLowerCase() === lastSelectedMailboxUsername.toLowerCase() &&
                    (lastSelectedMailboxTokenType == null || sendFromMailbox.TokenType != null && sendFromMailbox.TokenType === lastSelectedMailboxTokenType)) {
                    lCampaignSetup.SendFromMailboxes[i].LastSelected = true;
                    break;
                }
            }
        }     

        return {
            MessageType: "GetCampaignSetup",
            success: true,
            values: lCampaignSetup
        };
    }

    /**
     * Get Setup values for Campaigns
     * @param {any} sendResponse - A chrome messaging response callback
     * @param {any} forceRetry - Bypasses any checks and fires a fresh request to get the values
     */
    function GetSetup(sendResponse, forceRetry) {

        if (DoWeNeedToGoGetInformationAgain() || forceRetry === true) {
            //Check there isn't a request already running through another message request we may have already received
            if (lCampaignSetupRequest === null) {
                var successCallBack = function (msg) {
                    if (msg && msg.d) {
                        if (msg.d.Users) {
                            lCampaignSetup.CompanyUsers = msg.d.Users;
                        }

                        if (msg.d.SalesforceCampaigns) {
                            lCampaignSetup.SalesforceCampaigns = msg.d.SalesforceCampaigns;
                        }

                        if (msg.d.SendFromMailboxes) {
                            lCampaignSetup.SendFromMailboxes = msg.d.SendFromMailboxes;
                        }

                        if (msg.d.EnableCsvMergeFields !== undefined) {
                            lCampaignSetup.EnableCsvMergeFields = msg.d.EnableCsvMergeFields;
                        }

                        if (msg.d.EnableCampaignDupeCheck !== undefined) {
                            lCampaignSetup.EnableCampaignDupeCheck = msg.d.EnableCampaignDupeCheck;
                        }
                    }
                    
                    lCampaignSetupRequest = null;
                };

                lCampaignSetupRequest = MakeAjaxCallToSecure("POST", globalRequestEndPoints().GetCrmConnectEndpoint() + "/GetCampaignSetup", GetDefaultRequestParameters(), "application/json; charset=utf-8", "json", successCallBack, successCallBack);
            }

            if (lTaskAndDecisionTypesRequest === null) {
                var taskAndDecisionsCallBack = function (msg) {
                    if (msg && msg.d && msg.d.Values) {
                        if (msg.d.Success === true) {
                            lCampaignSetup.TaskAndDecisions = msg.d.Values;
                        }
                    }

                    lTaskAndDecisionTypesRequest = null;
                };

                lTaskAndDecisionTypesRequest = MakeAjaxCallToSecure("POST", globalRequestEndPoints().GetCrmConnectEndpoint() + "/GetTaskAndDecisionTypes", GetDefaultRequestParameters(), "application/json; charset=utf-8", "json", taskAndDecisionsCallBack, taskAndDecisionsCallBack);
            }

            if (sendResponse) {
                //If there is a request running keep checking until it's finished/set back to null so we can send the response back
                RetryWithCallback(60, 2, function () {
                    return lCampaignSetupRequest === null && lTaskAndDecisionTypesRequest === null;
                }, function () {
                    (function (sendResponse) {
                        sendResponse(CreateSuccessfulCampaignSetupResponse());
                    })(sendResponse);
                }, null);
            }
        } else {
            sendResponse(CreateSuccessfulCampaignSetupResponse());
        }
    }

    /** Runs all functions to get all the needed values within this module */
    function GetAllValues() {
        GetSetup(null, true);
    }

    return {
        GetAllValues: GetAllValues,
        GetSetup: GetSetup,
        LastSearchCampaignsRequest: lLastCampaignSearchRequest
    };
}

/**
 * Global closure which holds information regarding campaign manager
 * @returns {} 
 */
var GLOBAL_EBSTA_CAMPAIGNMANAGERVALUES = new CampaignManagerValues();

/**
 * Module to help store Gmail specific content
 */
const gmailStorage = (function () {
    /**
     * Array of 'Save on Send' values which the user has chosen. We store them here waiting for a gmail request to complete, then
     * we'll attempt to match the request to one of these values store in the array so we can then add the email to whatever the
     * user had previously saved
     */
    const saveOnSendObjects = new Array();

    /** Array of Email Tracking objects which contains details around emails that had been requested to tracked */
    const emailTrackingObjects = new Array();

    /**
     * If there is data within the arrays we'll check they haven't been there for too long, if they have delete them
     * @param {Object[]} listOfObjects - A list of objects to tidy up
     * @param {int} listOfObjects[].DateInMilliseconds - The number of milliseconds using new Date().getTime(). This property
     * needs to exist for this to work.
     */
    function deleteOldObjects(listOfObjects) {
        for (let i = 0; i < listOfObjects.length; i++) {
            var item = listOfObjects[i];
            if (item.DateInMilliseconds) {

                //Larger than 5 minutes? delete!
                if ((Math.abs(item.DateInMilliseconds - new Date().getTime()) / 60000) > 5) {
                    listOfObjects.splice(i, 1);
                    i--;
                    continue;
                }
            }
        }
    }

    /**
     * Save the Save on Send request into storage
     * @param {object} objectToAdd - The object to store so we can add save on send objects when the request has completed
     */
    function addSaveOnSendObject(objectToAdd) {
        saveOnSendObjects.push(objectToAdd);
    }

    /**
     * Look for any save on send request which match by compose or immutableMessageId
     *
     * @param {string} composeId - The is the identifier for a compose window in old gmail (pre Nov 2018). Each compose
     * window is labeled with a number in the DOM attributes (look for 'composeid'). This is reset each time Gmail loads
     * it seems but increments for each compose window opened, including external ones. Purely here for old Gmail currently,
     * but needed.
     *
     * @param {string} immutableMessageId - This should be the id in the format of msg-a:r8356662329366068087.
     * You can find this within the DOM of a compose window by searching for "draft"
     *
     * @param {string} mailbox - The mailbox that this save on send object should match
     *
     * @returns {Object|null} - If there is a matching save on send object then this return, null if there isn't a match
     */
    function getSaveOnSendObject(composeId, immutableMessageId, mailbox) {
        deleteOldObjects(saveOnSendObjects);

        if (composeId || immutableMessageId) {
            for (let i = 0; i < saveOnSendObjects.length; i++) {
                var item = saveOnSendObjects[i];

                if (item) {
                    var complexAddClosureObject = item.Record;
                    var mailboxSavedFrom = item.Mailbox;

                    if (complexAddClosureObject != null && mailboxSavedFrom != null) {
                        // Make sure where the request has come from matches the store array as we could get ourselves 
                        // mixed up when multiple gmail windows are open
                        if (mailbox === mailboxSavedFrom) {
                            if (complexAddClosureObject.ComposeId === composeId ||
                                complexAddClosureObject.ComposeId === "#" + immutableMessageId) {
                                saveOnSendObjects.splice(i, 1);
                                return complexAddClosureObject;
                            }
                        }
                    }
                }
            }
        }

        return null;
    }

    /**
     * Keep storage of an EmailTracking request
     * @param {Object} objectToAdd - The object
     * @param {Object} objectToAdd.Record - The object contains details on the email tracking such as the unique trackinglogid, mailbox, recipients
     * @param {int} objectToAdd.DateInMilliseconds - The date this object was created
     */
    function addEmailTrackingObject(objectToAdd) {
        emailTrackingObjects.push(objectToAdd);
    }

    /**
     * Takes an array of tracking Id's found from parsing the email body and then tried to match
     * them with out stored array to see if we need to track this email
     *
     * @param {Array} trackingIdMatches - Regex.Exec result used to return array of tracking id's found in an email body,
     * where array[1] would be the first tracking id we'll use to match     *
     * @param {string} mailBox - The mailbox the request has come from
     * @returns {Object|null} - A stored object matching by trackingid or null if no match
     */
    function getEmailTrackingObject(trackingIdMatches, mailBox) {
        deleteOldObjects(emailTrackingObjects);

        if (Array.isArray(trackingIdMatches)) {

            //This would generally be an array returned from a regex.exec function call,
            //which always returns the parenthesized substring matches of a regex query from the 2nd element onwards. This is why it starts from 1.
            //https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/RegExp/exec
            for (let m = 1; m < trackingIdMatches.length; m++) {
                const trackingIdMatch = trackingIdMatches[m];

                for (let i = 0; i < emailTrackingObjects.length; i++) {
                    const obj = emailTrackingObjects[i];
                    if (obj) {
                        const record = obj.Record;
                        if (record && record.TrackingLogId === trackingIdMatch.toUpperCase() && record.Mailbox === mailBox) {
                            emailTrackingObjects.splice(i, 1);
                            return record;
                        }
                    }
                }
            }
        }
        return null;
    }
    
    return {
        AddSaveOnSendObject: addSaveOnSendObject,
        GetSaveOnSendObject: getSaveOnSendObject,
        AddEmailTrackingObject: addEmailTrackingObject,
        GetEmailTrackingObject: getEmailTrackingObject
    };
})();

function WhatVersion() {
    return whatProductApp;
}

// is it Google's image proxy for an ebsta email tracking gif
function FindEbstaEmailTrackingId(url) {

    var res = "";

    if (url && url.length > 0) {

        var webUserId = localStorage.WebUserId;

        if (webUserId && webUserId.length > 0) {
            
            //build regex to match ebsta's email tracking gif
            var striRegex = "https:\\/\\/console\\.ebsta\\.com\\/(" + webUserId + "-[A-Za-z0-9]+)\\/ebsta\\.gif\\b";
            var regex = new RegExp(striRegex, "g");
            var match = regex.exec(url);

            if (match && match.length === 2 && match[1] !== null) {

                res = match[1];
            }

        }
    }

    return res;
}

// send request up to our services to log an own tracked email read
function LogClientEmailOwnRead(trackingId) {
    
    var fieldList = '[';
    fieldList = fieldList + GenerateEntityField("TrackingId", trackingId, '', false, '');
    fieldList += "]";

    var jsonFieldList = JSON.stringify(fieldList);
    LogClientEvent("EmailOwnRead", jsonFieldList);
}

/**
 * The callback for the chrome.webRequest.onBeforeRequest.addListener we've set up.
 *
 * Currently it will block any request for tracking images that are for the user currently logged in
 * @param {object} requestDetails - Details of the request
 * @returns {object} - Returns an object deciding if we should push the request through or return a simple base64 data URI to imitate the image
 */
function chromeOnBeforeRequestCallback(requestDetails) {
    if (requestDetails) {

        const url = requestDetails.url;

        const trackingId = FindEbstaEmailTrackingId(url);

        if (trackingId && trackingId.length > 0) {

            LogClientEmailOwnRead(trackingId);

            return { redirectUrl: "data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAI4AAABcAQMAAACCxR9vAAAAAXNSR0IArs4c6QAAAARnQU1BAACxjwv8YQUAAAAJcEhZcwAAEnQAABJ0Ad5mH3gAAAADUExURf///6fEG8gAAAAWSURBVBgZY2AYBaNgFIyCUTAKhgcAAAbUAAHYfhw4AAAAAElFTkSuQmCC" };
        }
    }
    return { cancel: false };
}

/**
 * Catch requests from googleusercontent.com and console.ebsta.com so we can intercept the tracking images before they hit out servers
 */
chrome.webRequest.onBeforeRequest.addListener(chromeOnBeforeRequestCallback,
    { urls: ["*://*.googleusercontent.com/*", "*://console.ebsta.com/*"] },
    ["blocking"]);

//Finds Chrome version using the useragent
function GetChromeVersionFromUserAgent() {
    var match = window.navigator.userAgent.match(/Chrom(?:e|ium)\/([0-9\.]+)/);
    return match ? match[1] : null;
}

//Summarises the version number to just the first digit
function GetChromeVersionNumber() {
    var version = GetChromeVersionFromUserAgent();
    if (version) {
        return version.split('.')[0];
    } else {
        return 0;
    }
}

var engine = new BackgroundEngine();

if (!localStorage.first) {
    var urlLink = 'https://console.ebsta.com/post-install/';
 
    if (whatProductApp == "10") {
        urlLink = 'https://console.ebsta.com/post-install/';
    }
    if (whatProductApp == "20") {
        urlLink = 'https://console.ebsta.com/post-install/';
    }
    if (whatProductApp == "30") {
        urlLink = 'https://console.ebsta.com/post-install/';
    }
    
    chrome.tabs.create({
            url: urlLink
    });
    localStorage.first = true;
}

if (localStorage.GmailGo == null) {
    localStorage.GmailGo = true; 
}


if (localStorage.GmailGoLicence == null) {
    localStorage.GmailGoLicence = false; 
}

if (localStorage.ExpiredWizard == null) {
    localStorage.ExpiredWizard = false; 
}
if (localStorage.LinkedinWizard == null) {
    localStorage.LinkedinWizard = false;
}

if (localStorage.GmailWizard == null) {
    localStorage.GmailWizard = false;
}

if (localStorage.PremWizard == null) {
    localStorage.PremWizard = false;
}

if (localStorage.SwitchbarOffInGmail == null) {
    localStorage.SwitchbarOffInGmail = false;
}

if (localStorage.ABTester == null) {
    localStorage.ABTester = "1";
}


/**
 * Set the default values for the complex add suggestions if not already set
 * @returns {} 
 */
function SetComplexAddSuggestionValues() {
    if (localStorage.ComplexAddSuggestionObjects == null) {
        localStorage.ComplexAddSuggestionObjects = JSON.stringify([{ "Name": "Account", "ApiName": "Account", "ChosenAsObjectToSearch": true }, { "Name": "Case", "ApiName": "Case", "ChosenAsObjectToSearch": true }, { "Name": "Opportunity (Open)", "ApiName": "Opportunity-Open", "ChosenAsObjectToSearch": true }, { "Name": "Opportunity (Closed)", "ApiName": "Opportunity-Closed", "ChosenAsObjectToSearch": true }]);

        //update global variable so this doesn't kill us.
        globalSuggestionSearchSettings.searchableObjects = JSON.parse(localStorage.ComplexAddSuggestionObjects);
    }
}

SetComplexAddSuggestionValues();

// My Cache Layout Information Object. 
var cacheLayout = new Array();

function SetConnector() {

        var mainConnector = 'https://secure.ebsta.com';
        localStorage.MainConnector = mainConnector;
    
}

SetConnector();

var installId = '';

function s4() {
    return Math.floor((1 + Math.random()) * 0x10000)
               .toString(16)
               .substring(1);
}



function PollLicenceCheck() {

    
    var objectInstallId = localStorage.InstallId + ""; 
    var guid1 = s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4();

    if (objectInstallId == "undefined") {
        localStorage.InstallId = guid1;
    }

    if (localStorage.InstallId == null) {
        
        localStorage.InstallId = guid1;
    }

    installId = localStorage.InstallId;

    var webUserId = "";
    var clientId = "";
    var chromeVersion = "";

    if (localStorage.WebUserId != null) {
        webUserId = "" + localStorage.WebUserId;
    }

    var varData = {
        webuserId: webUserId,
        installId: installId,
        chromeVersion: chromeVersion,
        clientId: clientId
    }

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb
        url: localStorage.MainConnector + "/Connector.svc/LicenceCheckPoll", // Location of the service
        data: JSON.stringify(varData), //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        async: true,
        timeout: 30000,
        success: function (msg) {//On Successfull service call

        },
        error: function (xhr, status, error) {
            return false;
        }
    });

    setTimeout(PollLicenceCheck, 86400000);
}

PollLicenceCheck();



localStorage.CrmDBType = 2; 
// Lets just hard code in. 

var validation = true; 

// Security Starts. 
if (localStorage.SessionCode == null) {
    MovePopUpToLoginPage()
    validation = false; 
}
else {
    if (localStorage.SessionCode == '00000000-0000-0000-0000-000000000000') {
        MovePopUpToLoginPage();
        validation = false; 
    }

    if (localStorage.SessionCode == 'null') {
        MovePopUpToLoginPage();
        validation = false; 
    }
}

var runOnce = 0;

function CheckLicenceFlag() {
    if (runOnce == 0) {
        runOnce = 1;
        return 0;
    } else {
        return 1; 
    }
}

function GetVersion() {
    var manifest = chrome.runtime.getManifest();
    var current_version = manifest.version;
    return current_version;
}

function GetName() {
    var manifest = chrome.runtime.getManifest();
    var name = manifest.name;
    return name;
}

if (localStorage.CrmUrl != null) {
    
    if (localStorage.SearchUrl == null) {
        localStorage.SearchUrl = localStorage.CrmUrl;
    } else {
        if (localStorage.SearchUrl == 'undefined') {
            localStorage.SearchUrl = localStorage.CrmUrl;
        }
    }
    
    if (localStorage.AddUrl == null) {
        localStorage.AddUrl = localStorage.CrmUrl;
    } else {
        if (localStorage.AddUrl == 'undefined') {
            localStorage.AddUrl = localStorage.CrmUrl;
        }
    }

    if (localStorage.UpdateUrl == null) {
        localStorage.UpdateUrl = localStorage.CrmUrl;
    } else {
        if (localStorage.UpdateUrl == 'undefined') {
            localStorage.UpdateUrl = localStorage.CrmUrl;
        }
    }

    if (localStorage.SelectUrl == null) {
        localStorage.SelectUrl = localStorage.CrmUrl;
    } else {
        if (localStorage.SelectUrl == 'undefined') {
            localStorage.SelectUrl = localStorage.CrmUrl;
        }
    }
}

//Setting the LocalStorage and global variable object for Abtesting information
function SetAbTestingValues(abTesterVal) {
    if (abTesterVal != null) {
        localStorage.ABTester = abTesterVal;
        SetGlobalAbTestingObject(abTesterVal);
    }
}

//Sets the 'globalAbTestingObject' obj
function SetGlobalAbTestingObject(abTesterVal) {
    globalAbTestingObject = GetAbtestingObject(abTesterVal);
}

function ValidateSessionLicence(check,dontLoadWelcome,reloadWindow) {

    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","applicationType": "' + whatProductApp + '","versionId": "' + GetVersion() + '"}';

        $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url:  localStorage.MainConnector + "/Connector.svc/ValidateLicenceWithLog", // Location of the service
            data: varData, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            async: true,
            timeout: 30000,
            success: function (msg) {//On Successfull service call

                if (msg.d.ValidSession) {
                    //if (msg.d.EbstaSalesforce) {
                    //    localStorage.Mode = "true";
                    //  if (check != 1) {
                                       
                    var licenceFound = 'Free';

                    if (msg.d.EbstaSalesforcePro || msg.d.EbstaGrowth) {
                        licenceFound = 'Pro';
                        localStorage.GmailGoLicence = true; 
                    }
                    
                    if (msg.d.EbstaSalesforce) {
                        licenceFound = "Enterprise";
                    }
                    
                    localStorage.LicenceFound = licenceFound;
                    localStorage.TrialUsed = msg.d.TrialLicence;
                    localStorage.TrailLicenceFound = msg.d.TrailLicenceFound;
                    localStorage.LicenceTrialCurrent = msg.d.LicenceTrialCurrent;

                    localStorage.EbstaGrowthTrial = msg.d.EbstaGrowthTrial;
                    localStorage.GrowthTrialDaysRemaining = msg.d.GrowthTrialDaysRemaining;

                    localStorage.ProductName = msg.d.ProductFriendlyName;

                    SetAbTestingValues(msg.d.ABGroup);

                    localStorage.GmailGoLicence = msg.d.GmailPlus;
                    
                    if (localStorage.GmailGo == null) {
                        localStorage.GmailGo = true;
                    }
                    
                    
                    if (dontLoadWelcome != 0) {
                        var popupWindows = chrome.extension.getViews({ type: 'popup' });
                        if (popupWindows.length > 0) { // A popup has been found
                            // Reload the page correctly. 

                            if (localStorage.ChatterOnly != "1") {
                                popupWindows[0].MoveMe('../html/Main.html?action=Welcome');
                            } else {
                                popupWindows[0].MoveMe('../html/Main.html?action=WelcomeChatter');
                            }
                        }
                    }


                    if (reloadWindow == 1) {
                        var popupWindows = chrome.extension.getViews({ type: 'popup' });

                        if (popupWindows.length != 0) {
                            popupWindows[0].SetupTrial();
                        }
                    }
                }
                else {
                    if (check == 1) {                        
                        ClearCacheAndLogout();
                    }
                }

                return true;
            },
            error: function (xhr, status, error) {
                return false;
            }
        });
}

if (validation) {
    ValidateSessionLicence(1,1,0);

    if (localStorage.Rules == null) {
        GetRules(0);
    }
    
}
// Security Ends

/** Helps clear up any saved/cached objects in the JS layer to help clear session when users log out or switch users */
function ClearAllSavedBackgroundInfo() {
    GLOBAL_EBSTA_CAMPAIGNMANAGERVALUES = new CampaignManagerValues();
    gEbstaSettings = new EbstaSettings();
}

function openURL(uri) {
    chrome.tabs.create({ "url": uri });
}


function ClearCacheAndLogout() {
    LogoutofSession();
}

function ClearCacheAfterLogoutSession() {
    ClearCache();
    ClearAllSavedBackgroundInfo();

    cacheEmailAdress = new Array();
    localStorage.SessionCode = null;
    localStorage.Firstname = '';
    localStorage.Surname = '';
    localStorage.WebUserId = '';
    localStorage.HelpMessage = '';
    localStorage.CrmId = '';
    localStorage.removeItem("LayoutManager");
    localStorage.GmailAccountActive = '';
    localStorage.GmailAccounts = '';
    var rules = localStorage.Rules;
    var GmailWizard = localStorage.GmailWizard;
    var PremWizard = localStorage.PremWizard;
    var LinkedinWizard = localStorage.LinkedinWizard;
    var ExpiredWizard = localStorage.ExpiredWizard;

    var gmailGo = localStorage.GmailGo;
    var switchbarOffInGmail = localStorage.SwitchbarOffInGmail;
    var emailTrackingSettings = localStorage.EmailTrackingSettings;

    var hideInboxIcons = localStorage.HideInboxIcons;
    var trackingAnalysis = localStorage.TrackingAnalysis;
    var dailyCampaignLimit = localStorage.DailyCampaignLimit;

    var enableMutationObservers = localStorage.EnableMutationObservers;

    var installId = localStorage.InstallId;
    var abFlagStore = localStorage.ABTester;
    localStorage.clear();
    localStorage.InstallId = installId;
    localStorage.Rules = rules;
    localStorage.GmailWizard = GmailWizard;
    localStorage.PremWizard = PremWizard;
    localStorage.LinkedinWizard = LinkedinWizard;
    localStorage.ExpiredWizard = ExpiredWizard;
    //Save ABFlag when logging out - not doing so can cause wrong actions (like old bar drop - login bar) when logged out.
    localStorage.ABTester = abFlagStore;

    localStorage.HideInboxIcons = hideInboxIcons;
    localStorage.TrackingAnalysis = trackingAnalysis;
    localStorage.DailyCampaignLimit = dailyCampaignLimit;

    localStorage.EnableMutationObservers = enableMutationObservers;

    localStorage.GmailGo = gmailGo;
    localStorage.SwitchbarOffInGmail = switchbarOffInGmail;
    if (emailTrackingSettings != null) {
        localStorage.EmailTrackingSettings = emailTrackingSettings;
    }

    SetConnector();
    localStorage.first = true;

    //Need to reset the check layout manager flag now a potential new login is happening
    checkedLayoutMgr = 0;

    MovePopUpToLoginPage();

    TellGmailUserIsNowLoggedOut();
}

//Sends a message to all tabs currently in gmail that user is now logged out and hides buttons as well as updating variables
function TellGmailUserIsNowLoggedOut() {
    chrome.tabs.getAllInWindow(null, function (tabs) {
        for (var i = 0; i < tabs.length; i++) {
            if (tabs[i].url.indexOf("mail.google.com") != -1 || tabs[i].url.indexOf("google.calendar.com") != -1
                || tabs[i].url.indexOf(".google.com/contacts") != -1 || tabs[i].url.indexOf("contacts.google.com") != -1) {
                SendMessageToHideGmailButtonsAndLoguserOut(tabs[i].id, true);
            }
        }
    });
}

function LogoutofSession() {

    console.log("Logging out"); 
    if (localStorage.SessionCode !== null) {

        var varData = '{"sessionId": "' +
            localStorage.SessionCode +
            '","userKey": "' +
            localStorage.WebUserId +
            '","crmId": "' +
            localStorage.CrmId +
            '","applicationId": "1"}';

        $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url: "https://secure.ebsta.com/Connector.svc/Logout", // Location of the service
            data: varData, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            async: true,
            timeout: 10000,
            success: function(msg) { //On Successfull service call
                ClearCacheAfterLogoutSession();
                console.log("Log out successfully sent");
            },
            error: function(xhr, status, error) {

                ClearCacheAfterLogoutSession();
                console.log(xhr);
                console.log(status);
                console.log(error);
            }
        });
    } else {
        console.log("Missing Session Key on Logout");
        ClearCacheAfterLogoutSession();
    }

}

// History Stuff store by tabid
var historyTab = new Array();

function ClearCache() {
    historyTab.length = 0;
    cacheLayout.length = 0;
    historyTab = new Array();
    cacheLayout = new Array();
    cacheEmailAdress = new Array();
}

function HistoryOfTab(location,tabId) {
    this.location = location;  //  Array of History.
    this.tabId = tabId; // TabId . 
}

function HistoryAdd(newUrl, tabId) {

    newUrl = '' + newUrl; 
    var histPos = GetHistoryPosition(tabId);

    if (histPos == -1) {
        // Lets add a new history.

        var historyTabLocal = [];
        if (newUrl.indexOf('Main.html?chk=1') != -1) {
            if (localStorage.ChatterOnly != "1") {
                newUrl = 'Main.html?action=Welcome';
            } else {
                newUrl = 'Main.html?action=WelcomeChatter';
            }
        }

        var urlToAdd = '' + newUrl;
        //urlToAdd = urlToAdd.replace('?chk=1','');

        if (newUrl.indexOf('action=LinkJumper') == -1) {
            historyTabLocal.push(urlToAdd);
        }

        var historyObj = new HistoryOfTab(historyTabLocal, tabId); 

        historyTab.push(historyObj);       
        
    }
    else {
        // We already have a location.
         
        if (newUrl.indexOf('Main.html?chk=1') == -1 && newUrl.indexOf('action=LinkJumper') == -1) {

            var historyObj = historyTab[histPos];
            var lastUrl = historyObj.location[historyObj.location.length -1];

            if (lastUrl + '' != '' + newUrl) {
                var arr = historyObj;

                arr.location.push('' + newUrl);

                historyTab.splice(histPos, 1, arr);
            }
        }     
    }
}

function HistoryGoBack(tabId) {

    var url = '';

    var histPos = GetHistoryPosition(tabId);

    var historyObj = historyTab[histPos];
    if (historyObj.location.length != 1) {
        url = historyObj.location[historyObj.location.length - 2];
        historyObj.location.splice(historyObj.location.length - 1, 1);
    }
    else {
        url = historyObj.location[0];
    }

    return url;
}

function HistoryGoBack2(tabId) {

    var url = '';

    var histPos = GetHistoryPosition(tabId);

    var historyObj = historyTab[histPos];

    if (historyObj.location.length != 1) {
        url = historyObj.location[historyObj.location.length - 2];
        historyObj.location.splice(historyObj.location.length - 2, 1);
    }
    else {
        url = historyObj.location[0];
    }

    return url;
}

function GetHistoryPosition(tabId) {

    var pos = -1;

    for (var i = 0; i < historyTab.length; i++) {

       if (historyTab[i].tabId + '' == '' + tabId) {
            return i; 
        }
    }

    return pos; 
}

var recentObject = new Array();

function RecentTab(html, id, entityType, title, typeObject) {
    this.html = html;  //  Array of History.
    this.id = id; // TabId . 
    this.entityType = entityType;
    this.title = title;
    this.typeObject = typeObject; 
}

function AddToRecentList(html,id,entityType,title,type) {

    if (title != null) {
        if (title.trim().length > 0) {
            var newObject = new RecentTab(html, id, entityType, title, type);


            var pos = -1;

            for (var i = 0; i < recentObject.length; i++) {
                if (recentObject[i].id == id) {
                    pos = i;
                    break;
                }
            }

            if (pos != -1) {
                recentObject.splice(pos, 1);
            }
            // Put it back on top;
            recentObject.splice(0, 0, newObject);
        }
    }

}

/**
 * Sends a message to main.js to show the recently viewed records list
 * @returns {} 
 */
function GetRecentlist() {
    setTimeout(function () {
        chrome.extension.sendMessage({ Action: "ShowRecentlyViewedList", RecentObject: recentObject });
    }, 100);
}

// The Cache object in Storage. 
function CacheLayout(searchName, formsData, actionName, tabId,tabUrl) {
    this.searchName = searchName;
    this.formsData = formsData;
    this.actionName = actionName;
    this.tabId = tabId;
    this.tabUrl = tabUrl;
}

// Add Update Cache. 
function AddUpdateCache(searchName, formsData, action, tabId,tabUrl) {

 
    
    if (localStorage.WebUserId != null) {
        var cacheObj = new CacheLayout(searchName, formsData, action, tabId, tabUrl);
   
        var foundPos = GetCachePosition(tabId);


        if (foundPos == -1) {
            // Add New Cache
          
            cacheLayout.push(cacheObj);
        }
        else {
            // Update the Cache
            cacheLayout[foundPos] = cacheObj;
        }
    }
}

function GetPageCache(tabId) {
    
    var foundPos = GetCachePosition(tabId);

    if (foundPos == -1) {
        // Add New Cache
        return null;
    }
    else {
        // Update the Cache
       return cacheLayout[foundPos];
    }

}

// Gets Cache Position. 
function GetCachePosition(tabId) {
    var position = -2;  

    for (var i = 0; i < cacheLayout.length; i++) {
        if (cacheLayout[i].tabId + '' == '' + tabId) {
            position = i;
            break;
        }
        //Do something
    }

    if (position != -2) {
        return position; 
    }
    else {
        return -1;
    }
}
// Cache Layout Ends Here. 

function CheckTrial() {


    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '"}';

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb
        url: localStorage.MainConnector + "/Connector.svc/StartAFreeTrial", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        async: false,
        timeout: 30000,
        success: function (msg) {//On Successfull service call

            if (msg.d) {
                ValidateSessionLicence(0,0,1);

            } else {

                var popupWindows = chrome.extension.getViews({ type: 'popup' });

                if (popupWindows.length != 0) {
                    popupWindows[0].AlreadyUsedTrial();
                }
            }

        },
        error: function (xhr, status, error) {


        }
    });

}

//this function is in place to facilitate the logging of the correct events depending on wizard / start trial workflow taken by the client.  This is to facilitate correct pardot list additions
function CheckTrial2(origin) {

    var varData = '{"origin": "' + origin + '","sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '"}';

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb
        url: localStorage.MainConnector + "/Connector.svc/StartAFreeTrial2", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        async: false,
        timeout: 30000,
        success: function (msg) {//On Successfull service call

            if (msg.d) {
                ValidateSessionLicence(0, 0, 1);

            } else {

                var popupWindows = chrome.extension.getViews({ type: 'popup' });

                if (popupWindows.length != 0) {
                    popupWindows[0].AlreadyUsedTrial();
                }
            }

        },
        error: function (xhr, status, error) {


        }
    });

}

function CheckLicence() {


    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","applicationType": "' + whatProductApp + '","versionId": "' + GetVersion() + '"}';

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb
        url: localStorage.MainConnector + "/Connector.svc/ValidateLicenceWithLog", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        async: true,
        timeout: 30000,
        success: function (msg) {//On Successfull service call

            if (msg.d.ValidSession) {

                var licenceFound = 'Free';

                if (msg.d.EbstaSalesforcePro || msg.d.EbstaGrowth) {
                    licenceFound = 'Pro';
                    localStorage.GmailGoLicence = true;
                }

                if (msg.d.EbstaSalesforce) {
                    licenceFound = "Enterprise";
                }

                localStorage.LicenceFound = licenceFound;
                localStorage.TrialUsed = msg.d.TrialLicence;
                localStorage.TrailLicenceFound = msg.d.TrailLicenceFound;
                localStorage.LicenceTrialCurrent = msg.d.LicenceTrialCurrent;

                localStorage.EbstaGrowthTrial = msg.d.EbstaGrowthTrial;
                localStorage.GrowthTrialDaysRemaining = msg.d.GrowthTrialDaysRemaining;

                localStorage.ProductName = msg.d.ProductFriendlyName;
                
                SetAbTestingValues(msg.d.ABGroup); 

                localStorage.GmailGoLicence = msg.d.GmailPlus;

                if (localStorage.GmailGo == null) {
                    localStorage.GmailGo = true;
                }
            }


        },
        error: function (xhr, status, error) {
            

        }
    });

}

//Checks the last time we've checked for a Layout Manager and updates the global flag (checkedLayoutMgr)
function CheckLastLayoutManagerCheck()
{
    if (localStorage.LayoutManagerLastCheck != null)
    {
        var lastCheckDate = localStorage.LayoutManagerLastCheck;
        if (lastCheckDate > 0)
        {
            var nowDate = new Date();
            var timeDiff = Math.abs(nowDate.getTime() - lastCheckDate);

            //Difference in days between now and last time we checked
            var diffDays = Math.floor(timeDiff / (24 * 60 * 60 * 1000));
            if(diffDays > 0)
            {
                LogEventShort("24hr Layout Manager Check");
                //Set flag back to 0 so CheckLayoutManager() will check if there is an updated CRMConnector
                checkedLayoutMgr = 0;
            }
        }
    }
}

var checkedLayoutMgr = 0; 

function CheckLayoutManager() {

    if (ValidateTheBasics()) {
        //Lets check to see if we need to go and look for an updated CRMConnector
        if (checkedLayoutMgr == 1) {
            CheckLastLayoutManagerCheck();
        }

        if (checkedLayoutMgr == 0) {
            checkedLayoutMgr = 1;
            var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '"}';

            localStorage.LayoutManagerLastCheck = new Date().getTime();

            $.ajax({
                type: "POST", //GET or POST or PUT or DELETE verb
                url: localStorage.MainConnector + "/Connector.svc/CheckLayoutManager", // Location of the service
                data: varData, //Data sent to server
                contentType: "application/json; charset=utf-8", // content type sent to server
                dataType: "json", //Expected data format from server
                processdata: true, //True or False
                timeout: 30000,
                success: function(msg) { //On Successfull service call
                    // We need to work out how to get this going. 

                    var versionCurrent = localStorage.LayoutManagerVersion;

                    if (versionCurrent != null) {

                        if (versionCurrent != msg.d) {
                            // Get Layout. 
                            LogEventShort("New CRM Connector Found");
                            GetFormLayout();
                        } else {
                            CheckLayoutVersion();
                        }

                    } else {
                        // Get New Layout
                        LogEventShort("Get Form Layout-4");
                        GetFormLayout();
                    }

                },
                error: function(xhr, status, error) {

                }
            });
        }
    }
}

//Triggers an event everytime a new window is opened
chrome.windows.onCreated.addListener(function () {
    //Run a layoutmanager check on Chrome open to make sure we update the layoutmanager at least once a day for the user
    CheckLayoutManager();
});

// Gets a New Forms Layout. 
function GetRules(firstLoad) {
    //Log time we get rules;
    var date = new Date();
    var currentTicks = ((date.getTime() - date.getTimezoneOffset() * 60000) * 10000) + 621355968000000000;
    localStorage.rulesVersionLastDownloaded = parseInt(currentTicks);


    var service = localStorage.MainConnector + '/Connector.svc/GetRuleListSalesforce';
    if (localStorage.Debug != null) {
        if (localStorage.Debug == "true") {
            service = localStorage.MainConnector + '/Connector.svc/GetRuleList_Debug';
        }
    }
  

    var rulesDomain = "";
    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","domain": "' + rulesDomain + '"}';
  
    $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url: service, // Location of the service
            data: varData, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            timeout: 30000,
            success: function (msg) {//On Successfull service call
             

                // We need to work out how to get this going. 
                var objJson = JSON.stringify(msg);

                if (msg.d != null) {
                    localStorage.Rules = objJson;
                }

                if (firstLoad == 1) {
                    ReRunSearchAgainstTab();
                }
            },
            error: function (xhr, status, error) {
            
            }
        });
    
    
    
}

function ValidateTheBasics() {
    var bl_validate = true; 
    
    if (!localStorage.SessionCode || localStorage.SessionCode == '') {
        bl_validate = false;
    }

    if (!localStorage.WebUserId || localStorage.WebUserId == '') {
        bl_validate = false;
    }

    if (!localStorage.CrmId || localStorage.CrmId == '') {
        bl_validate = false;
    }

    return bl_validate; 

}
// Gets a New Forms Layout. 
function GetFormLayout() {
    if (ValidateTheBasics()) {
        LogEventShort("Get Form Layout");
        GetLayoutManager([function() {GetRules(0);}, TriggerMainRenderPage]);
    } else {
        MovePopUpToLoginPage();
    }
}

/**
 * Moves the pop up to the login page using
 * @returns {} 
 */
function MovePopUpToLoginPage() {
    chrome.extension.sendMessage({ Action: "MovePageToUrl", LoginPage: true });
    chrome.browserAction.setPopup({ popup: '../html/Login.html' });
}

/**
 * Moves the pop up to the Loading page
 * @returns {} 
 */
function MovePopUpToLoadingPage() {
    chrome.extension.sendMessage({ Action: "MovePageToUrl", LoadingPage: true });
    chrome.browserAction.setPopup({ popup: '../html/Loading.html' });
}

/**
 * Moves the pop up the the home page
 * @returns {} 
 */
function MovePopUpToMainPage() {
    chrome.extension.sendMessage({ Action: "MovePageToUrl", MainPage: true });
    chrome.browserAction.setPopup({ popup: '../html/Main.html?chk=1' });
}

/**
 * Sends a message to main.js to trigger the RenderPage function();
 * @returns {} 
 */
function TriggerMainRenderPage() {
    chrome.extension.sendMessage({ Action: "TriggerMainRenderPage"});
}

/**
 * Handles an errors from GetLayoutManager requests
 * @returns {} 
 */
function HandleGetLayoutManagerRequestError() {
    ClearCacheAndLogout();
    localStorage.removeItem("LayoutManager");
    MovePopUpToLoginPage();
}

// Check if rules are up to date
var checkedRules = false;

function CheckRulesVersion() {

    if (!checkedRules) {
        checkedRules = true;

        var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '"}';

        $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url: localStorage.MainConnector + "/Connector.svc/CheckRulesVersion", // Location of the service
            data: varData, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            timeout: 30000,
            success: function (msg) {//On Successfull service call
         
                var rulesVersionLastUpdated = msg.d;
                if (localStorage.rulesVersionLastDownloaded != null) {
                    if (localStorage.rulesVersionLastDownloaded < rulesVersionLastUpdated) {
                        GetRules(0);
                    }
                }
                else {
                    GetRules(0);
                }

            },
            error: function (xhr, status, error) {
            }
        });
     }

}
// End rules check

// get most reently used campaigns by user
function GetMostRecentlyUsedCampaigns(sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    var varDataSelect = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '"';

    var callBack = function (msg) {

        var response = { MessageType: "GetMostRecentlyUsedCampaigns", success: false };
        if (msg && msg.d) {

            response.success = msg.d.Success;
            response.errorMessage = msg.d.ErrorMessage;
            response.values = msg.d.Values;
        }

        sendResponse(response);
    }

    MakeAjaxCallToSecure("POST", selectUrl + "/GetMostRecentlyUsedCampaigns", varDataSelect, "application/json; charset=utf-8", "json", callBack, callBack);
}

// search all of user's campaigns by campaign name
function SearchCampaigns(request, sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    if (request && request.PageNumber !== null && typeof request.PageNumber !== 'undefined' && request.ItemsPerPage !== null && typeof request.ItemsPerPage !== 'undefined') {

        //Abort the last request if we still have one running
        const lastRequest = GLOBAL_EBSTA_CAMPAIGNMANAGERVALUES.LastSearchCampaignsRequest;
        if (lastRequest != null) {
            lastRequest.abort();
        }

        var varDataSelect = {
            "sessionId": localStorage.SessionCode,
            "userKey": localStorage.WebUserId,
            "crmId": localStorage.CrmId,
            searchRequest: {
                SearchTerm: request.SearchTerm,
                PageNumber: request.PageNumber,
                ItemsPerPage: request.ItemsPerPage,
                OrderBy: request.OrderField || 0,
                OrderDescending: typeof request.SortAsc === "boolean" ? request.SortAsc : false,
                WebuserIds: request.WebuserIds
            }
        }

        var callBack = function (msg) {

            var response = { MessageType: "SearchCampaigns", success: false };
            if (msg && msg.d) {

                response.success = msg.d.Success;
                response.errorMessage = msg.d.ErrorMessage;
                response.values = msg.d.Values;
                response.pageNumber = msg.d.PageNumber;
                response.pageCount = msg.d.PageCount;
                response.RowCounter = msg.d.RowCounter;
            } else if (msg.statusText === "abort") {
                response.RequestAborted = true;
            }

            sendResponse(response);
            GLOBAL_EBSTA_CAMPAIGNMANAGERVALUES.LastSearchCampaignsRequest = null;
        };

        GLOBAL_EBSTA_CAMPAIGNMANAGERVALUES.LastSearchCampaignsRequest = MakeAjaxCallToSecure("POST", selectUrl + "/SearchCampaigns", JSON.stringify(varDataSelect), "application/json; charset=utf-8", "json", callBack, callBack);
    } else {
        sendResponse({ MessageType: "SearchCampaigns", success: false, errorMessage: "null paging values" });
    }
}

// search details for a specific campign by user
function GetCampaignDetails(request, sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    if (request && request.campaignId !== null && typeof request.campaignId !== 'undefined') {

        var varDataSelect = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '", "campaignId": "' + request.campaignId + '"}';

        var callBack = function (msg) {

            var response = { MessageType: "GetCampaignDetails", success: false };
            if (msg && msg.d) {

                response.success = msg.d.Success;
                response.errorMessage = msg.d.ErrorMessage;
                response.values = msg.d.Values;
            }

            sendResponse(response);
        }

        MakeAjaxCallToSecure("POST", selectUrl + "/GetCampaignDetails", varDataSelect, "application/json; charset=utf-8", "json", callBack, callBack);
    } else {
        sendResponse({ MessageType: "GetCampaignDetails", success: false, errorMessage: "null id" });
    }
}

// search details for a specific campign by user
function GetTemplateById(request, sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    if (request && request.TemplateId !== null && typeof request.TemplateId !== 'undefined') {

        var varDataSelect = {
            sessionId: localStorage.SessionCode,
            userKey: localStorage.WebUserId,
            crmId: localStorage.CrmId,
            mailboxUsername: request.MailboxUsername,
            templateId: request.TemplateId,
            replaceMergeFields: request.ReplaceMergeFields
        }

        var callBack = function (msg) {

            var response = { MessageType: "GetTemplateById", templateEditObj: request.TemplateEditObj, InjectIntoComposeWindow: request.InjectIntoComposeWindow, success: false };

            if (msg && msg.d) {

                response.success = msg.d.Success;
                response.errorMessage = msg.d.ErrorMessage;
                response.values = msg.d.Values;
            }

            sendResponse(response);
        }

        MakeAjaxCallToSecure("POST", selectUrl + "/GetTemplateById", JSON.stringify(varDataSelect), "application/json; charset=utf-8", "json", callBack, callBack);
    } else {
        sendResponse({ MessageType: "GetTemplateById", success: false, errorMessage: "null id or edit object" , InjectIntoComposeWindow: request.InjectIntoComposeWindow});
    }
}

// search details for a specific campign by user
function MakeEditedCopyOfTemplate(request, sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    //console.log("MakeEditedCopyOfTemplate");
    //console.log(request);

    if (request && request.Template !== null && typeof request.Template !== 'undefined') {

        var varDataSelect = {
            sessionId: localStorage.SessionCode,
            userKey: localStorage.WebUserId,
            crmId: localStorage.CrmId,
            templateObj:  JSON.stringify(request.Template)
        };

        var callBack = function (msg) {
            var response = {
                MessageType: "MakeEditedCopyOfTemplate",
                Success: false,
                SaveContactToCampaign: request.SaveContactToCampaign,
                SaveObject: request.SaveObject,
                DraftId: request.DraftId,
                OldTemplateId: request.Template.CampaignTemplateId,
                TaskEditObj: request.TaskEditObj
            };
            if (msg && msg.d) {

                response.Success = msg.d.Success;
                response.ErrorMessage = msg.d.ErrorMessage;
                response.Values = msg.d.Values;
            }

            sendResponse(response);
        }

        MakeAjaxCallToSecure("POST", selectUrl + "/MakeEditedCopyOfTemplate", JSON.stringify(varDataSelect), "application/json; charset=utf-8", "json", callBack, callBack);
    } else {
        sendResponse({ MessageType: "MakeEditedCopyOfTemplate", Success: false, ErrorMessage: "null objects" });
    }
}

// save contact to campaign
function SaveContactToCampaign(request, sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    if (request && request.Tasks !== null && typeof request.Tasks !== 'undefined' && request.ContactObj !== null && typeof request.ContactObj !== 'undefined') {

        var varDataSelect = {
            saveContactToCampaignRequest: {
                SessionId: localStorage.SessionCode,
                Webuserid: localStorage.WebUserId,
                CrmId: localStorage.CrmId,
                CampaignContact: request.ContactObj,
                MailboxUsername: request.MailboxUsername,
                IanaTimezone: Intl.DateTimeFormat().resolvedOptions().timeZone,
                DraftId: request.DraftId,
                CcRecipients: request.CcRecipients,
                BccRecipients: request.BccRecipients,
                RelatedSalesforceCampaignId: request.RelatedSalesforceCampaignId
            },
            stages: JSON.stringify(request.Tasks)
        };

        gEbstaSettings.SaveLastSelectedMailbox(request.SelectedMailbox);

        var callBack = function (msg) {

            var response = { MessageType: "SaveContactToCampaign", success: false, draftId: request.DraftId || ""};
            if (msg && msg.d) {

                response.success = msg.d.Success;
                response.errorMessage = msg.d.ErrorMessage;
                response.values = msg.d.Values;
            }

            sendResponse(response);
        }

        MakeAjaxCallToSecure("POST", selectUrl + "/SaveCampaign", JSON.stringify(varDataSelect), "application/json; charset=utf-8", "json", callBack, callBack);
    } else {
        sendResponse({ MessageType: "SaveContactToCampaign", success: false, errorMessage: "null objects" });
    }
}

/**
 * Saves multi recipients to campaign
 * @param {} tabId 
 * @param {} request 
 * @returns {} 
 */
function SaveMultiRecipientsToCampaign(request, sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    if (request && request.MultiRecipientCampaign != null) {
        var varDataSelect = {
            sessionId: localStorage.SessionCode,
            userKey: localStorage.WebUserId,
            crmId: localStorage.CrmId,
            multiRecipientCampaign: JSON.stringify(request.MultiRecipientCampaign),
            CcRecipients: request.CcRecipients,
            BccRecipients: request.BccRecipients,
            RelatedSalesforceCampaignId: request.RelatedSalesforceCampaignId
        };

        gEbstaSettings.SaveLastSelectedMailbox(request.MultiRecipientCampaign.SelectedMailbox);

        var callBack = function(msg) {
            var response = { MessageType: "SaveMultiRecipientsToCampaign", success: false };
            if (msg && msg.d) {

                response.success = msg.d.Success;
                response.errorMessage = msg.d.ErrorMessage;
                response.values = msg.d.Values;
            }

            sendResponse(response);
        }

        MakeAjaxCallToSecure("POST", selectUrl + "/SaveMultiRecipientsToCampaign", JSON.stringify(varDataSelect), "application/json; charset=utf-8", "json", callBack, callBack);
    } else {
        sendResponse({ MessageType: "SaveMultiRecipientsToCampaign", success: false, errorMessage: "null objects" });
    }
}

/**
 * get people from salesforce list view to use in multi recipient campaign
 * @param {} tabId 
 * @param {} request 
 * @returns {} 
 */
function GetPeopleFromListView(request, sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    if (request && request.ObjectName != null && request.listViewId) {
        var varDataSelect = {
            sessionId: localStorage.SessionCode,
            userKey: localStorage.WebUserId,
            crmId: localStorage.CrmId,
            objectName: request.ObjectName,
            listViewId: request.listViewId
        };

        var callBack = function (msg) {
            var response = { MessageType: "GetPeopleFromListView", success: false };
            if (msg && msg.d) {

                response.success = msg.d.Success;
                response.errorMessage = msg.d.ErrorMessage;
                response.values = msg.d.Values;
            }

            sendResponse(response);
        }

        MakeAjaxCallToSecure("POST", selectUrl + "/GetPeopleFromListView", JSON.stringify(varDataSelect), "application/json; charset=utf-8", "json", callBack, callBack);
    } else {
        sendResponse({ MessageType: "GetPeopleFromListView", success: false, errorMessage: "null objects" });
    }
}

/**
 * get all list views by object
 * @param {} tabId 
 * @param {} request 
 * @returns {} 
 */
function GetListViewsForObject(request, sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    if (request && request.ObjectName != null) {
        var varDataSelect = {
            sessionId: localStorage.SessionCode,
            userKey: localStorage.WebUserId,
            crmId: localStorage.CrmId,
            objectName: request.ObjectName
        };

        var callBack = function (msg) {
            var response = { MessageType: "GetListViewsForObject", objectName: request.ObjectName, success: false };
            if (msg && msg.d) {

                response.success = msg.d.Success;
                response.errorMessage = msg.d.ErrorMessage;
                response.values = msg.d.Values;
            }

            sendResponse(response);
        }

        MakeAjaxCallToSecure("POST", selectUrl + "/GetListViewsForObject", JSON.stringify(varDataSelect), "application/json; charset=utf-8", "json", callBack, callBack);
    } else {
        sendResponse({ MessageType: "GetListViewsForObject", success: false, errorMessage: "null objects" });
    }
}

/**
 * get objects user has access for list view multi recipient campaigns 
 * @param {} tabId 
 * @param {} request 
 * @returns {} 
 */
function GetCampaignSalesforceListViewObjects(sendResponse) {

    var selectUrl = localStorage.SelectUrl;

    var varDataSelect = {
        sessionId: localStorage.SessionCode,
        userKey: localStorage.WebUserId,
        crmId: localStorage.CrmId
    };

        var callBack = function(msg) {
        var response = { MessageType: "GetCampaignSalesforceListViewObjects", success: false };
            if (msg && msg.d) {

                response.success = msg.d.Success;
                response.errorMessage = msg.d.ErrorMessage;
                response.values = msg.d.Values;
            }

            sendResponse(response);
    }

    MakeAjaxCallToSecure("POST", selectUrl + "/GetCampaignSalesforceListViewObjects", JSON.stringify(varDataSelect), "application/json; charset=utf-8", "json", callBack, callBack);
}

/**
 * Use this to send generic Campaign requests
 * @param {any} request
 * @param {any} sendResponse
 */
function SendCampaignRequest(request, sendResponse) {
    var response = { MessageType: "CampaignRequest", success: false };

    var selectUrl = localStorage.SelectUrl;

    if (request) {

        //The structure of the requests the endpoint is expecting
        var objectRequest = {
            "sessionId": localStorage.SessionCode,
            "userKey": localStorage.WebUserId,
            "crmId": localStorage.CrmId,
            campaignRequest: {
                CampaignRequestType: request.CampaignRequestType,
                RequestBody: JSON.stringify(request.RequestBody)
            }
        };

        //Callback to be actioned at the end of the request
        var callBack = function (msg) {
                        
            if (msg && msg.d) {

                response.success = msg.d.Success;
                response.errorMessage = msg.d.ErrorMessage;
                response.values = msg.d.Values;
            }

            if (sendResponse) {
                sendResponse(response);
            }
        };

        MakeAjaxCallToSecure("POST", selectUrl + "/CampaignRequest", JSON.stringify(objectRequest), "application/json; charset=utf-8", "json", callBack, callBack);
    } else {
        sendResponse(response);
    }
}

// centralised method for making ajax call
function MakeAjaxCallToSecure(type, url, data, contentType, dataType, onSuccess, onError) {

    return $.ajax({
        type: type,
        url: url, // Location of the service
        data: data, //Data sent to server
        contentType: contentType, // content type sent to server
        dataType: dataType, //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: onSuccess,
        error: onError
    });
}


/**
 * Fires a SelectEntity http request
 * @param {string} entityname - The type of search to run
 * @param {any} fields - The list of fields to be sent up with the request
 * @param {any} keyfields - The key fields to be sent with the request
 * @param {int} tabId - The id of the tab the request originated from
 * @param {any} extraInformation - This is a general field which can be used for anything
 * @param {function} successCallBack - A one parameter function which will be called if the request is success
 */
function SelectEntity(entityname, fields, keyfields, tabId, extraInformation, successCallBack) {

    //console.log("run select: " + entityname);

    if (fields.length == 0) {
        fields = '"' + fields + '"';
    }

    var selectUrl = localStorage.SelectUrl;

    if (entityname == "awesomeebstatrackingpoll")
    {
        selectUrl = "https://secure.ebsta-webservices.com/CrmConnect.svc";
    }
    
    var varDataSelect = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","entityName": "' + entityname + '","fieldsExpected": ' + fields + ',"ids": ' + keyfields + '}';

    var popupWindows = chrome.extension.getViews({ type: 'popup' });
    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb
        url: selectUrl + "/SelectEntity", // Location of the service
        data: varDataSelect, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call
            var objJson = JSON.stringify(msg);
            if (msg.d != null) {
                if (msg.d.ErrorMessage != null) {
                    if (msg.d.ErrorMessage.indexOf('REAUTH:') != -1) {

                        if (tabId != null && typeof tabId != 'undefined') {
                            SendMissingAuth(parseInt(tabId));
                        }
                        
                        ClearCacheAndLogout();
                        authFail = true;
                    }

                    if (msg.d.ErrorMessage.indexOf('Unauthorized') != -1) {

                        if (tabId != null && typeof tabId != 'undefined') {
                            SendMissingAuth(parseInt(tabId));
                        }

                        ClearCacheAndLogout();
                        authFail = true;

                    }
                }
            }
            
            if (entityname == "usertasksbarcount") {
                LoadTaskBar(objJson);
            }
            else {
                localStorage.OpenFromTaskBar = -1;
            }

            // If we have a callback, use this over trying to use the chrome.extension.getViews({ type: 'popup' }) function 
            // as this is a much cleaner process and enables us to test the extension through Selenium type tests
            if (typeof successCallBack === "function") {
                successCallBack(objJson);
            } else {
                try {
                    if (popupWindows.length > 0 && entityname != null && objJson != null) { // A popup has been found
                        popupWindows[0].PassData(entityname, objJson, 0);
                    }
                } catch (err) {
                    //ignore
                }
            }

            // Section Number is passed through within the fields variable
            if (entityname.toLowerCase() == "cbcustombarsection") {
                PostCustomBarInfo(objJson, entityname, tabId, fields);
            }

            if (entityname.toLowerCase() == "cbcontactbyid" || entityname.toLowerCase() == "cballopportunities" || entityname.toLowerCase() == "cbtaskeventinfo" || entityname.toLowerCase() == "cbaccountinfo" || entityname.toLowerCase() == "cbaccountcontacts" || entityname.toLowerCase() == "cbleadinfo" || entityname.toLowerCase() == "cbleadactivityinfo") {
                PostBarInfo(objJson, entityname, tabId);
            }

            if (entityname.toLowerCase() == "awesomeebstatrackingpoll") {
                try{                
                    DisplayTrackingOpenAlert(objJson);
                }catch(error)
                {

                }
            }

            if (entityname.toLowerCase() == "getmeetingscheduler") {

                chrome.tabs.executeScript(tabId, { code: "RenderMeetingSchedulerLink('" + entityname + "'," + objJson + ")" });
            }

            if (entityname.toLowerCase() == "gettemplatefolders") {
                var lastUsedEmailFolder;
                if (localStorage.LastUsedEmailFolder != null) {

                    lastUsedEmailFolder = localStorage.LastUsedEmailFolder;
                }
                chrome.tabs.executeScript(tabId, { code: "RenderTemplateFoldersResult('" + entityname + "'," + objJson + ",'" + lastUsedEmailFolder + "')" });
            }
            
            if (entityname.toLowerCase() == "getemailtemplatebyid") {
                chrome.tabs.executeScript(tabId, { code: "EmailTemplateSelectResult('" + entityname + "'," + objJson + ")" });
            }
            
            if (entityname.toLowerCase() == "gettemplatesforfolderbyid") {
                chrome.tabs.executeScript(tabId, { code: "FolderTemplatesResult('" + entityname + "'," + objJson + ")" });
            }

            // sending back the result, items
            if (entityname.toLowerCase() == "awesomeebstagetemailtrackingitems") {
                chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "LoadTrackItems", generalfields: objJson });
            }
			
			if (entityname.toLowerCase() == "gmailauth") {
			    if (!gmailAuthFailed) {
			        var parsedValues = JSON.parse(msg.d.Values);
			        localStorage.GmailAuth = parsedValues.AuthResult;
			    }                
			}

			if (entityname.toLowerCase() == "awesomeebstacheckcontacthastrackingevents") {
			    var parsedValues = JSON.parse(msg.d.Values);
			    chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "ProccessContactHasTrackingEvents", HasTrackingEvents: parsedValues.ContactHasTrackingEvents });
			}

            switch (entityname.toLowerCase()) {
                case "awesomeebstagetemailtrackingheadersforopenedmails":
                case "awesomeebstagetemailtrackingheadersforcontact":
                case "awesomeebstagetemailtrackingheadersallevents":
                case "contactsyncsetup_getlistviews":
                    if (extraInformation !== null && typeof extraInformation === "function") {
                        extraInformation(msg);
                    }
                    return true;
            }

			if (entityname.toLowerCase() == "getcompactlayoutdetailsforrecord") {
                if (extraInformation.SendResponse) {
                    extraInformation.SendResponse({
                        ObjJson: objJson,
                        InstanceUrl: localStorage.InstanceUrl
                    });
                }
			}
			
            //The request to check for fields on an event object within the Onboarding process
			if (entityname.toLowerCase() == "calsyncsetup_checkcalendarsyncsettings") {
			    chrome.tabs.executeScript(tabId, { code: "HandleCalSyncSetupCheckResponse(" + objJson + ")" });
			}

            //The request to get all calendar sync and google calendar details for a user to select their options within the onboarding process
			if (entityname.toLowerCase() == "calsyncsetup_getgooglecalendarsandcalendarsyncdetails") {
			    chrome.tabs.executeScript(tabId, { code: "HandleCalSyncSetupSettingsResponse(" + objJson + ")" });
			}

            //Tells bar.js if the user is fully auth or not so we can action
            if (entityname.toLowerCase() == "checkgmailauthformailbox") {

                var needsGAuth = false;
                var parsedValues;

                if (msg.d.Values != null) {
                    if (msg.d.Values.length > 0) {
                        parsedValues = JSON.parse(msg.d.Values);

                        if (parsedValues.NeedsGAuth == true) {
                            needsGAuth = true;
                        }

                    }
                }

                chrome.tabs.executeScript(tabId, { code: "HandleGmailAuthForMailboxResponse(" + needsGAuth + "," + shouldWeShowCalendarSync + "," + shouldWeShowContactSync + ")" });
                //Reset the flags now that we've used it for the next load event
                shouldWeShowCalendarSync = false;
                shouldWeShowContactSync = false;
            }

			if (entityname.toLowerCase() == "selectgoogleemailids") {
			    if (msg.d.Values != null) {
			        if (msg.d.Values.length > 0) {
                        //Send results back to client so we can update the buttons to reflect the results returned
			            chrome.tabs.executeScript(tabId, { code: "UpdateEbstaButtonsWithGmailToReflectEmailBeingAdded(" + objJson + ")" });
                        //Update local storage with our new findings
			            InsertGmailIdsIntoLocalStorage(msg.d.Values);
			        }
			    }
			}

            //Calls HandleCallBackFromGetScheduleDetailRequest which will determine if the compose window in question is scheduled
			if (entityname === "selectsendlaterschedule") {
                if (msg != null && msg.d != null) {
                    var values = msg.d.Values;
                    if (values != null && values.length > 0) {
                        //Update array
                        if (sendLaterDraftStorageArray != null && sendLaterDraftStorageArray.indexOf(extraInformation) === -1) {
                            sendLaterDraftStorageArray.push(extraInformation);
                        }

                        //Send response call back to where it come from so we can action it
                        extraInformation(values);
                    }
                }
            }

            localStorage.DoubleTap = '';
        },
        error: function (xhr, status, error) {
            localStorage.DoubleTap = '';
            if (popupWindows.length) { // A popup has been found

             
                popupWindows[0].RecoverOutofError();

                popupWindows[0].SelectError(entityname);
            }

            if (extraInformation !== null && typeof extraInformation === "function") {
                extraInformation({
                    d: {
                        xhr: xhr,
                        status: status,
                        error: error
                    }
                });
            }
        }
    });
}



function replyBtnClick() {
	//Write function to respond to user action.

    chrome.tabs.executeScript(parseInt(localStorage.LastTabId), { code: "ShowEmailTrackingSummaryBar()" });
}

//Returns the 'globalAbTestingObject' object or sets it initially if it's not been set yet
function GetGlobalAbTestingObject() {
    //Extra check to make sure we go and get the obj if it doesn't exist
    if (globalAbTestingObject == null) {
        globalAbTestingObject = GetAbtestingObject(localStorage.ABTester);
    }

    return globalAbTestingObject;
}

function DisplayTrackingOpenAlert(objectData) {

    var db = JSON.parse(objectData);

    if (db != null && typeof db != "undefined") {
        var values = db.d.Values;

        var recipientString = '';

        if (values != null && typeof values != "undefined") {
            var resultsLength = values.length;
            if (resultsLength > 0) {

                var firstValue = null;

                //Check the users has access to Link Tracking alerts, if not only show Email Tracking alert info
                if (GetGlobalAbTestingObject().EnableLinkTracking !== 1)
                {
                    //Reset length as we're not sure how many link tracking alerts we have
                    resultsLength = 0;

                    //Find the first and second Email Tracking alert within the return list, if any
                    //Set the first found Email tracking alert as the new firstValue so it can be user later on to display information
                    for (var i = 0; i < values.length; i++) {
                        var localVal = JSON.parse(values[i]);
                        if (!localVal.IsTrackingLink) {
                            resultsLength++;
                            if (resultsLength > 1) break;
                            firstValue = localVal;
                        }
                    }

                    //Stop here if we didn't find an email alerts as no need to keep on
                    if (resultsLength === 0) return;

                } else {
                    //Set the first value to be the main object we use to display the alert
                    firstValue = JSON.parse(values[0]);
                }

                if (firstValue != null)
                {
                    //default display - email open event by subject
                    var actionDisplay = "";
                    if (!firstValue.IsTrackingLink) {
                        if (firstValue.EmailSubject != null) {
                            actionDisplay = "Viewed " + firstValue.EmailSubject;
                        }
                    } else { // - link clicked event by uri display
                        if (firstValue.TrackingLinkUri != null) {
                            actionDisplay = "Clicked " + CleanTrackedUriForDisplay(firstValue.TrackingLinkUri);
                        }
                    }

                    var totalRecipients = firstValue.EmailRecipients.split(',');
                    if (totalRecipients.length > 1) {
                        recipientString = 'Someone';
                    } else if (totalRecipients.length == 1) {
                        recipientString = totalRecipients[0];
                    } else {
                        recipientString = 'Unknown';
                    }

                    //Append more text when we have more the 1 alert returned back
                    if (resultsLength > 1) {
                        actionDisplay += " + other notifications";
                    }

                    //Create object for the notification alert
                    var opt = {
                        type: "basic",
                        title: "" + recipientString,
                        message: actionDisplay,
                        iconUrl: "../images/Ebsta-128.png",
                        isClickable: true
                    }

                    //Create the notification alert
                    chrome.notifications.create("", opt, creationCallback);
                }
            }
        }
    }
}

function creationCallback(callBack) {
  
}

chrome.notifications.onClicked.addListener(replyBtnClick);

/**
 * Runs a search for many parts within the extension
 * @param {string} entityname1 - Unique name for type of search
 * @param {string} search - Search term
 * @param {any} tabId - Id of the tab
 * @param {any} amountOnPage - Amount of results
 * @param {any} page - Page Number
 * @param {any} orderBy - Order by
 * @param {any} filters - filters
 * @param {any} addToRecent - ???
 * @param {function} successCallback - A function to call when the search request is successful
 * @param {function} errorCallback - A function to call when the search request is unsuccessful
 */
function Search(entityname1, search, tabId, amountOnPage, page, orderBy, filters, addToRecent, successCallback, errorCallback) {

    var entityname = entityname1;

    if (entityname1.indexOf("lookupemail-") != -1) {
        entityname = entityname1.replace('lookupemail-', 'lookup-');
    }

    var storeSearchMade = entityname + '-SearchMade';
    localStorage[storeSearchMade] = search;

    if (typeof amountOnPage === 'undefined' || amountOnPage == '' || amountOnPage == null) { amountOnPage = 7; } //default row count

    if (search.trim().length < 2) {
        if (typeof errorCallback === 'function') {
            errorCallback({ errormessage: "Please enter at least 2 characters in the search box." });
        }
        return;
    } else {      

        if (typeof amountOnPage === 'undefined' || amountOnPage == '') {
            amountOnPage = 7;
        } //default row count

      
        if (localStorage.LastSearchWord != null) {

            if (localStorage.LastSearchWord == search && localStorage.LastTabId == tabId && localStorage.LastSearchAmountOnPage == amountOnPage) {

                var store = entityname + '-Search';

                //If we've cached he search result then try and find it to save running a fresh search
                if (localStorage[store] != null) {
                    var objJson = localStorage[store];

                    if (typeof successCallback === 'function') {
                        //We need to delay the callback slightly as the method which calls to here actually returns a loading gif straight after, 
                        //if we didn't delay the call then all the user would see would be a loading gif
                        (function (lEntityname, lObjJson, lsearchUsed) {
                            setTimeout(function () {
                                successCallback(lEntityname, lObjJson, lsearchUsed);
                            }, 500);
                        })(entityname, objJson, 1);
                    }
                    return;
                }
            }
        }

        var searchFields = "[" + GenerateEntityField('SearchType', 'manual', '', 'false');
        searchFields = searchFields + ',' + GenerateEntityField('SessionKey', '' + localStorage.SessionKey, '', 'false');
        searchFields = searchFields + ',' + GenerateEntityField('EBActionTaken', '' + localStorage.Action, '', 'false');
        searchFields = searchFields + ',' + GenerateEntityField('SiteURL', '' + localStorage.LastPageUrl, '', 'false') + "]";
        var strFields = JSON.stringify(searchFields);

        var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","searchName": ' + JSON.stringify(search) + ',"entityToSearch": "' + entityname + '","orderBy": "0","pageNumber": 1,"amountonPage": ' + amountOnPage + ',"filters": ' + JSON.stringify(localStorage.BlockObject) + ',"searchFields": ' + strFields + '}';

        $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url: localStorage.SearchUrl + "/SearchV2", // Location of the service
            data: varData, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            timeout: 30000,
            success: function (msg) { //On Successfull service call

                var objJson = JSON.stringify(msg);
                var lastSearch = localStorage[storeSearchMade];

                
                try {
                    lastSearch = decodeURI(lastSearch);
                } catch(err) {
                    
                }
                
                var lastSearchCheck = msg.d.SearchTermUsed;
                try {
                    lastSearchCheck = decodeURI(lastSearchCheck);
                } catch(errr) {

                }

                lastSearchCheck = $(document.createElement('div')).html(lastSearchCheck).text();
                
                if (lastSearch == lastSearchCheck) {

                    var store = entityname + '-Search';
                    if (addToRecent == 1) {
                        localStorage.RecentSearch = '<a href="Main.html?action=Search&search=' + search + '">' + search + '</a>';

                        localStorage.LastSearchWord = search;
                        localStorage.LastSearchAmountOnPage = amountOnPage;
                        localStorage.LastTabId = tabId;
                        localStorage[store] = objJson;
                    }

                    if (typeof successCallback === 'function') {
                        successCallback(entityname, objJson, 1);
                    }
                }
            },
            error: function (xhr, status, error) {
                if (typeof errorCallback === 'function') {
                    errorCallback({
                        xhr: xhr
                    });
                };
            }
        });
    }
}

function SearchChatterUser(searchname) {
    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","searchName": ' + JSON.stringify(searchname) + ',"entityToSearch": "ChatterUser","orderBy": "0","pageNumber": 1,"amountonPage": 7,"filters": ' + JSON.stringify(localStorage.BlockObject) + '}';
 
    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb
        url: localStorage.SearchUrl + "/Search", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) { //On Successfull service call
           
            var objJson = JSON.stringify(msg);

            var popupWindows = chrome.extension.getViews({ type: 'popup' });
                if (popupWindows.length) { // A popup has been found

                    popupWindows[0].ChatterResult(objJson);
                }
            
           

        },
        error: function (xhr, status, error) {
            
        }
    });
}

function SearchChatterTopics(searchname) {
    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","searchName": ' + JSON.stringify(searchname) + ',"entityToSearch": "ChatterTopic","orderBy": "0","pageNumber": 1,"amountonPage": 7,"filters": ' + JSON.stringify(localStorage.BlockObject) + '}';

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb
        url: localStorage.SearchUrl + "/Search", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) { //On Successfull service call

            var objJson = JSON.stringify(msg);

            var popupWindows = chrome.extension.getViews({ type: 'popup' });
            if (popupWindows.length) { // A popup has been found

                popupWindows[0].ChatterResult(objJson);
            }



        },
        error: function (xhr, status, error) {

        }
    });
}
//Remove an entry from the EmailTrackingLog table if Gmail send has been undone
function DeleteGmailTrackingRecords(entityName, trackingLogId)
{
    var _trackingLogId = "";
    if (trackingLogId != null) {
        _trackingLogId = "\"" + trackingLogId + "\"";
        var varDataUpdate = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","entityName": "' + entityName + '","trackingLogId": ' + _trackingLogId + '}';

        $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url: localStorage.UpdateUrl + "/DeleteEmailTrackingEntry", // Location of the service
            data: varDataUpdate, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            timeout: 30000,
            success: function (msg) {//On Successfull service call

                var objJson = JSON.stringify(msg);

            },
            error: function (xhr, status, error) {
                localStorage.DoubleTap = '';

            }
        });
    }
}


/**
 * Updates an entity
 * @param {string} entityname
 * @param {string} fields
 * @param {any} tabId
 * @param {function} successCallBack - A one parameter function which will be called with the returned data from the request if successful
 */
function UpdateEntity(entityname, fields, tabId, successCallBack) {
    
    if (localStorage.DoubleTap != null) {   
        var test = entityname + fields + tabId;
        if (localStorage.DoubleTap == test) {
            //Double tap. 
            return;
        }
    }

                  
    localStorage.DoubleTap = entityname + fields + tabId;
    
    var entityNameSend = '';

    if (entityname.indexOf("-Edit") != -1) {
        entityNameSend = entityname.substring(0, entityname.indexOf("-Edit"));
    } else {
        entityNameSend = entityname;
    }


    var varDataUpdate = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","entityName": "' + entityNameSend + '","fields": ' + fields + '}';
    var popupWindows = chrome.extension.getViews({ type: 'popup' });

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb
        url: localStorage.UpdateUrl + "/UpdateEntity", // Location of the service
        data: varDataUpdate, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call
            //            chrome.tabs.sendMessage(parseInt(tabId), { entityName: "candidate", result: "" + msg.d.Values[0] }, function (response) {

            //            });


            var objJson = JSON.stringify(msg);           
            //if we return an error here, clear the date boxes otherwise when the page is resubmitted, the times will be added again, and the format will not be accepted
            
            // If we have a callback, use this over trying to use the chrome.extension.getViews({ type: 'popup' }) function 
            // as this is a much cleaner process and enables us to test the extension through Selenium type tests
            if (typeof successCallBack === "function") {
                successCallBack(objJson);
            } else if (popupWindows.length) {
                // A popup has been found
                popupWindows[0].SaveComplete(entityname, objJson, 'Updated');
            }

        },
        error: function (xhr, status, error) {
            localStorage.DoubleTap = '';
            if (popupWindows.length) { // A popup has been found

                popupWindows[0].RecoverOutofError(); 
                popupWindows[0].SaveComplete(entityname, 'Error : ' + status, 'Updated');
                SelectEntity("Event", "", GenerateBHKeyField(id), localStorage.LastTabId);           
            }
        }
    });
}

var cacheEmailAdress = new Array();

function CacheEmailObj(_entityname, _crmId, _emailAddress,_name) {
    this.Entityname = _entityname;
    this.CrmId = _crmId;
    this.EmailAddress = _emailAddress;
    this.Name = _name; 
}

function AddEntity(entityname, fields, tabId) {
    AddEntityNew(entityname, fields, tabId, "");
}

/**
 * Fires an AddEntity request
 * @param {string} entityname - The type of search to run
 * @param {any} fields - The list of fields to be sent up with the request
 * @param {int} tabId - The id of the tab the request originated from
 * @param {any} extraInformation - This is a general field which can be used for anything
 * @param {function} successCallBack - A one parameter function which will be called if the request is success
 */
function AddEntityNew(entityname, fields, tabId, extraInformation, successCallBack) {

    var emailAdd = false;

    if (entityname == "TaskEmail") {
        entityname = "Task";
        emailAdd = true;
        localStorage.DoubleTap = "";
    }

    if (entityname == "GmailTask") {
        emailAdd = true;
        localStorage.DoubleTap = "";

        DisplayGoogleInboxMessage("Adding Email to Salesforce", tabId,"");
    }

    if (entityname == "CalSyncSetup_AddPreviewSettingsGoogleCalendarSync" || entityname == "Onboarding-SyncCalendarNow" || entityname == "ContactSyncSetup_SavePreviewSettings") {
        localStorage.DoubleTap = "";
    }

    if (localStorage.DoubleTap != null) {
        var test = entityname + fields + tabId; 
        if (localStorage.DoubleTap == test) {
            //Double tap. 
            return;
        }
    }
    
    localStorage.DoubleTap = entityname + fields + tabId;
    
    var varDataAdd = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","entityName": "' + entityname + '","fields": ' + fields + '}';

    var popupWindows = chrome.extension.getViews({ type: 'popup' });

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb
        url: localStorage.AddUrl + "/AddEntity", // Location of the service
        data: varDataAdd, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call
            //            chrome.tabs.sendMessage(parseInt(tabId), { entityName: "candidate", result: "" + msg.d.Values[0] }, function (response) {
  
            //            });
            //ClearCache();
            localStorage.RecentSearch = '';

            var objJson = JSON.stringify(msg);
            //if we return an error here, clear the date boxes otherwise when the page is resubmitted, the times will be added again, and the format will not be accepted

  //          localStorage.LastEmailSearch0 = '';
//            localStorage.LastEmailSearch1 = '';

            var result = "" + msg.d.Success;

            // As we're sending up an email address in this request it'll get caught within the caching of email objects below
            // but we don't want to store this in the Cached Emails as there is no relevant details to use from this response so just finish here
            if (entityname) {
                if (entityname.toLowerCase() == "updategoogleaccountdontauthnow") {
                    //Get updated list of accounts
                    GetGmailAccounts(tabId, false);
                    return true;
                } else if (entityname === "emailtracking_mutenotification") {
                    chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "UpdateEmailTrackingAlertIcon", ExtraInfo: extraInformation });
                    return true;
                }
            }

            if (result.toLowerCase() == "true") {
                var crmId = msg.d.CrmId;
                var emailAddress = msg.d.EmailAddress;
                var name1 = msg.d.Name; 
                var cacheemobj = new CacheEmailObj(entityname, crmId, emailAddress,name1);
                if (emailAddress != null) {
                    if (crmId != null) {
                        cacheEmailAdress.push(cacheemobj);
                    }
                }
            }
            
            if (emailAdd)
            {
                // Inject back on to the page.
               GiveBackEmailResult(msg, tabId,extraInformation);
               if (entityname.toLowerCase() == 'gmailtask') {
                   //if auth with google fails, reset local storage flag
                   if (msg.d.Message == 'GMAILREAUTH:') {
                       localStorage.GmailAuth = 0;
                       gmailAuthFailed = true;

                       HandleGmailReAuth();
                   }
               }

            }
            else {

                // If we have a callback, use this over trying to use the chrome.extension.getViews({ type: 'popup' }) function 
                // as this is a much cleaner process and enables us to test the extension through Selenium type tests
                if (typeof successCallBack === "function") {
                    successCallBack(objJson);
                } else {
                    try {
                        if (popupWindows.length) { // A popup has been found  

                            ////console.log('fire save complete...');

                            popupWindows[0].SaveComplete(entityname, objJson, 'Added');
                        }
                    } catch (error) {
                        // Error
                    }
                }

                if (entityname.toLowerCase() == "setcalendarstatetoken") {
                    popupWindows[0].CalendarStateTokenResult(msg.d.EmailAddress);
                }


                if (entityname == "Acccount" ^ entityname == "Contact") {
                    // Don't run asearch.

                    ////console.log('add entity success...');
                    //Clear cache for Gmail as we may of found null results earlier and this will give us null results each time we add
                    if (localStorage.LastPageUrl != null) {
                        if (localStorage.LastPageUrl.indexOf("mail.google.com/mail") != -1) {
                            localStorage.LastEmailSearch0 = "";
                            localStorage.LastEmailSearchResult0 = "";
                            localStorage.LastEmailSearch1 = "";
                            localStorage.LastEmailSearchResult1 = "";
                        }
                        else {

                            RunBackgroundSearch(localStorage.LastSearchWord, tabId, "coresearch", 1, localStorage.LastPageUrl, 0);
                            localStorage.LastSearchWord = null;
                        }
                    } 
                }

                if (entityname.toLowerCase == 'gmailtask') {
                    //if auth with google fails, reset local storage flag
                    if (msg.d.Message == 'GMAILREAUTH:') {
                        localStorage.GmailAuth = 0;
                        gmailAuthFailed = true;

                        HandleGmailReAuth();
                    }
                }

                //Automatically get the preview events once we know the settings were saved correctly
                if (entityname == "CalSyncSetup_AddPreviewSettingsGoogleCalendarSync") {
                    //console.timeEnd("CalSyncSavePreviewSettings");
                    GetPreviewEventsForCalendarSync(tabId, extraInformation);
                }

                //Automatically get the preview contacts once we know the settings were saved correctly
                if (entityname == "ContactSyncSetup_SavePreviewSettings")
                {
                    if (msg.d.CrmLink) {

                        //Report back to client first incase user just wants to get on with the sync rather than wait for settings
                        chrome.tabs.executeScript(tabId, { code: "SetContactSyncRuleIdInPreviewTable(" + msg.d.CrmLink + ")" });

                        GetPreviewContactsForContactSync(tabId, msg.d.CrmLink);
                    }
                }

                if (entityname === "sendlatersaveschedule") {
                    chrome.tabs.executeScript(tabId, { code: "global_SendLaterObject.HandleCallBackFromSavingDraft(" + msg.d.Success + ")" });
                }

            }
        },
        error: function (xhr, status, error) {

            if (emailAdd) {
                DisplayGoogleInboxMessage('Failed to add email to Salesforce.', tabId,"failure");
            } else {

                localStorage.DoubleTap = '';
                if (popupWindows.length) { // A popup has been found

                    popupWindows[0].RecoverOutofError();
                    popupWindows[0].SaveComplete(entityname, 'Error : ' + status, 'Added');
                }
            }

        }
    });
}

function HandleGmailReAuth()
{
    //Auth has failed - redirect to authorisation process.
    var authUrl = 'https://console.ebsta.com/Google/ActivateGoogle.aspx?wb=' + localStorage.WebUserId;
    window.open(authUrl, '_blank');
}

function ClearUpJson(value) {

    return value.replace(/"/g, '&quot;').replace(/\\/g, '\\\\');
}

function GenerateEntityField(name, value, typeofField, key,controlName) {
    return '{ "__type": "EntityField:#Culuru.Elemento.Common", "Name": "' + name + '", "Value": ' + JSON.stringify(value) + ', "TypeOfField": "' + typeofField + '", "IsKey": "' + key + '","ControlName": "' + controlName + '" }';
}

function GenerateEntityFieldNoFilter(name, value, typeofField, key, controlName) {
    return '{ "__type": "EntityField:#Culuru.Elemento.Common", "Name": "' + name + '", "Value": ' + JSON.stringify(value) + ', "TypeOfField": "' + typeofField + '", "IsKey": "' + key + '","ControlName": "' + controlName + '" }';
}

function GenerateBHKeyField(value) {

    
    var fields = '[';
    fields = fields + GenerateEntityField("userID", value, '', 'true', '');
    fields = fields + ',' + GenerateEntityField("SiteURL", '' + localStorage.LastPageUrl, '', 'false', '');
    fields = fields + ',' + GenerateEntityField('SessionKey', '' + localStorage.SessionKey, '', 'false');
    fields = fields + ',' + GenerateEntityField("EBActionTaken", '' + localStorage.Action, '', 'false', '');
    fields = fields + ']';

    var fieldsStr = JSON.stringify(fields);

    return fieldsStr; 

}

/**
 * Creates a Entity Field used for Custom Search which we pass up to services
 * @param {} emailvalue 
 * @param {} namevalue 
 * @returns {} 
 */
function GenerateCustomBarKeyfield(emailvalue, namevalue, id) {
    var fields = '[';
    fields = fields + GenerateEntityField("emailAddress", emailvalue, '', 'true', '');
    fields = fields + ',' + GenerateEntityField("personName", namevalue, '', 'true', '');

    if (id) {
        fields = fields + ',' + GenerateEntityField("salesforceid", id, '', 'true', '');
    }

    fields = fields + ']';

    var fieldsStr = JSON.stringify(fields);

    return fieldsStr;

}

function GenerateBHKeyFieldWithRecordTypeId(value, recordTypeId,action) {

    
    
    var fields = '[';
    fields = fields + GenerateEntityField("userID", value, '', 'true', '');
    fields = fields + "," + GenerateEntityField("RecordTypeId", recordTypeId, '', 'false', '');
    fields = fields + ',' + GenerateEntityField("SiteURL", '' + localStorage.LastPageUrl, '', 'false', '');
    fields = fields + ',' + GenerateEntityField("EBActionTaken", '' + localStorage.Action, '', 'false', '');
    fields = fields + ',' + GenerateEntityField('SessionKey', '' + localStorage.SessionKey, '', 'false');
    fields = fields + ']';

    var fieldsStr = JSON.stringify(fields);

    return fieldsStr;

}

function SelectionSearch(info, tab) {
    var url = tab.url.toLowerCase();

    var selection = info.selectionText;

    selection = selection.replace("'", "\\'");
    //console.log("Ebsta Search: Selection Search: ");
    //console.log(selection);
    chrome.tabs.executeScript(tab.id, { code: "proc.RightClickSearchLoadBar('" + selection + "')" });
  
}

/**
 * Fires a request to get layout manager
 * @param {} getRulesParameter 
 * @param {} arrayOfCallBacks 
 * @returns {} 
 */
function GetLayoutManager(arrayOfSuccessCallBacks) {
    globalIsThereALayoutManagerRequestActive = true;
    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '"}';

    $.ajax({
        type: "POST", 
        url: globalRequestEndPoints().GetLayoutManagerEndpoint(),
        data: varData,
        contentType: "application/json; charset=utf-8",
        dataType: "json",
        processdata: true,
        timeout: 1020000,
        success: function(msg) {
            if (msg != null && msg.d != null) {
                if (msg.d.ErrorMessage != null) {
                    if (msg.d.ErrorMessage.length === 0) {
                        var objJson = JSON.stringify(msg);

                        localStorage.LayoutManager = objJson;
                        localStorage.LayoutManagerVersion = msg.d.Version;
                        localStorage.CrmUrl = msg.d.CrmUrl; // Allows for change in URL Automatically to clients. 
                        localStorage.SearchUrl = msg.d.SearchUrl;
                        localStorage.AddUrl = msg.d.AddUrl;
                        localStorage.UpdateUrl = msg.d.UpdateUrl;
                        localStorage.SelectUrl = msg.d.SelectUrl;
                        localStorage.CrmDBType = msg.d.DatabaseTypeId;
                        localStorage.GmailCalendarAuth = msg.d.GoogleCalendarSyncId;
                        localStorage.GmailCalendarSync = msg.d.GoogleCalendarSync;
                        localStorage.GmailAuth = msg.d.GmailAuthType;
                        localStorage.EmailBlacklist = msg.d.EmailBlacklist;
                        if (msg.d.OpportunityStageProbabilities) {
                            localStorage.OpportunityStageList = JSON.stringify(msg.d.OpportunityStageProbabilities);
                        }
                        //Added by Amar: 27/01/2015 
                        //Store the gmail accounts list in a property named d, so that the json format matches json returned / stored by getgmailaccounts functions.
                        if (msg.d.GmailAccounts != null) {
                            var localStorageObj = { d: msg.d.GmailAccounts };
                            localStorage.GmailAccounts = JSON.stringify(localStorageObj);
                        }

                        localStorage.Username = msg.d.SalesforceUsername;
                        localStorage.UserId = msg.d.SaleforceUserId;

                        gmailAuthFailed = false;

                        // hides icons in main inbox view.
                        localStorage.HideInboxIcons = msg.d.HideInboxIcons;

                        // renders NEW tracking analysis dashboard
                        localStorage.TrackingAnalysis = msg.d.TrackingAnalysis;

                        //daily limit on campaign sending.
                        if (msg.d.DailyCampaignLimit != null) {
                            localStorage.DailyCampaignLimit = msg.d.DailyCampaignLimit;
                        } else {
                            localStorage.DailyCampaignLimit = 250;
                        }

                        //enable use of mutation observers.
                        localStorage.EnableMutationObservers = msg.d.EnableMutationObservers;

                        //render reply data.
                        localStorage.RenderReplyData = msg.d.ShowReplyData;

                        localStorage.InstanceUrl = msg.d.DataUrl;
                        localStorage.UseDefaultBarSearch = msg.d.UseDefaultBarSearch;
                        engine.SendMessageToGmailAndLinkedinTabs({ MessageType: "UpdateDefaultBarSearch", Value: msg.d.UseDefaultBarSearch });
                        GetTaskTypes(msg);
                        UpdateGlobalParsedLayoutManagerVariable();
                        UpdateComplexAddSettingsVariable();

                        // check if we should be disabling logging.
                        CheckLogging();

                        //Special callbacks needed to be action at the end of success request
                        if (arrayOfSuccessCallBacks) {
                            for (var i = 0; i < arrayOfSuccessCallBacks.length; i++) {
                                arrayOfSuccessCallBacks[i]();
                            }
                        }

                        SetupCustomisations();
                    } else {
                        if (msg.d.ErrorMessage.indexOf('REAUTH:') !== -1) {
                            ClearCacheAndLogout();
                            MovePopUpToLoginPage();
                        }
                    }
                }
            }
        },
        error: function (xhr, status, error) {
            console.log("xhr", xhr);
            console.log("status", status);
            console.log("error", error);
            HandleGetLayoutManagerRequestError();
        }
    }).always(function () {
        globalIsThereALayoutManagerRequestActive = false;
    });
}

/**
 * The inital get layout manager call when a user successfully auths in
 * @returns {} 
 */
function GetFormLayoutWithRedirect() {
    LogEventShort("GetFormLayoutWithRedirect");
    CheckLicence();
    MovePopUpToLoadingPage();
    GetLayoutManager([function () { GetRules(1); }, MovePopUpToMainPage]);
}

function OpenPaymentPage() {
    var pricingpage = "https://console.ebsta.com/LoginSalesforce.aspx?j=3";
    chrome.tabs.create({ "url": pricingpage });
}

if (localStorage.GmailAccountActive == "reload") {
    localStorage.GmailAccountActive = "true";
}

//Sets the event for a user on uninstall
function SetUninstallEvent(webUserId) {
    if (GetChromeVersionNumber() > 40) {
        //Try/catch is here because of a 'just in case this fails' as this would bring BSOD
        try
        {
            chrome.runtime.setUninstallURL("https://console.ebsta.com/post-uninstall/uninstall.aspx?wu=" + webUserId);
        }
        catch (error)
        {
            
        }
    }
}

var localRules = '';

chrome.extension.onMessage.addListener(
  function (request, sender, sendResponse) {

      if (request.MessageType == "VALIDATE") {
          try {
              //console.log("call to check licence");
              CheckLicence();
          } catch(error) {

          }
      }

      if (request.MessageType == "Auth") {
          localStorage.SessionCode = request.SessionCode;
          localStorage.WebUserId = request.WebUserId;

          //On un-install event
          SetUninstallEvent(request.WebUserId);

          localStorage.CrmId = request.CrmId;

          localStorage.Firstname = request.Firstname;
          localStorage.Surname = request.Surname;
          localStorage.CrmUrl = request.CrmUrl;
          localStorage.SearchUrl = request.SearchUrl;
          localStorage.Locale = request.Locale;
          localStorage.ChatterOnly = request.ChatterOnly;
        
          localStorage.AddUrl = request.AddUrl;
          localStorage.UpdateUrl = request.UpdateUrl;
          localStorage.SelectUrl = request.SelectUrl;
          
          localStorage.removeItem("LayoutManager"); // Reset Layout Manager. 
          localStorage.Mode = true;//request.EbstaLicence;

          localStorage.CrmDBType = request.DatabaseTypeId; //this will break for jobscience and talentrover
          
          GetFormLayoutWithRedirect();
          
          //Reload all Gmail tabs when auth'ing in to enable all features within gmail correctly
          engine.ReloadTabs();

          Start_Or_Stop_Tracking_Poll();

          ClearAllSavedBackgroundInfo();
          GLOBAL_EBSTA_CAMPAIGNMANAGERVALUES.GetAllValues();
      }

      if (request.MessageType == "OpenGoogleAuthTabURI") {

          //Open the url - and store the sender tab id. So that we can go back to the tab afterwards.
          googleAuthRequestTabId = sender.tab.id;
          chrome.tabs.create({ "url": request.URI });
      }

      if (request.MessageType == "OpenSFAuthTabURI") {
          sfAuthRequestTabId = sender.tab.id;
          //Open URI in new tab
          chrome.tabs.create({ "url": request.URI });
      }

      //fixes issue of previous details parsing to next add
      if (request.MessageType == "ClearParseInformation")
      {
          localStorage.ParseInformation = "";
      }

      if (request.MessageType == "ProGetStarted") {
          LogEventShort("ProGetStarted");
          var origin = "premsitewizard";

          CheckTrial2(origin);
          chrome.tabs.executeScript(sender.tab.id, { code: "CloseWizard()" });
      }
      
      if (request.MessageType == "PaymentPageOpen") {
          LogEventShort("BarPaymentOpen");
          OpenPaymentPage();
      }
      
      if (request.MessageType == "ExpiredGetStarted") {
          LogEventShort("ExpiredGetStarted");
          OpenPaymentPage();
          chrome.tabs.executeScript(sender.tab.id, { code: "CloseWizard()" });
      }
      
      if (request.MessageType == "GmailGetStarted") {

          //console.log('GmailGetStarted');

          LogEventShort("GmailGetStarted");
          var origin = "gmailwizard";
          //remove trial start - gmail is free...
          //CheckTrial2(origin);
          chrome.tabs.executeScript(sender.tab.id, { code: "CloseWizard()" });
          if (localStorage.GmailGo == "true" && localStorage.GmailGoLicence == "true") {
              setTimeout(chrome.tabs.executeScript(sender.tab.id, { code: "SetMsgInbox()" }), 2000);
              setTimeout(chrome.tabs.executeScript(sender.tab.id, { code: "LoadComplete()" }), 2000);
          }
      }
      
      if (request.MessageType == "LinkedinGetStarted") {
          LogEventShort("LinkedinGetStarted");
          //  CheckTrial(); 
          chrome.tabs.executeScript(sender.tab.id, { code: "CloseWizard()" });
       
      }
      
      if (request.MessageType == "SeePricing") {
          LogEventShort("See Pricing");
          OpenPaymentPage();
          chrome.tabs.executeScript(sender.tab.id, { code: "CloseWizard()" });
      }
      
      if (request.MessageType == "NoThanks") {
          LogEventShort("NoThanks");
          chrome.tabs.executeScript(sender.tab.id, { code: "CloseWizard()" });
      }

      if (request.MessageType === "UpdateNotificationOption") {
          LogEventShort("UpdateNotificationOption");
          if (request.TurnOffNotifications != null) {
              Update_EmailTracking_Poll(request.TurnOffNotifications);
          }
      }

      if (request.MessageType === "Get_LocalStorage_Item_And_Return_JSON") {
          if (request.LocalStorageKey != null) {
              sendResponse({ LocalStorageItem: Get_LocalStorage_Item_And_Return_JSON(request.LocalStorageKey) }); 
          }
      }

      if (request.MessageType == "LicenceInformation") {
          var barLicence = "Pro";

          if (localStorage.LicenceFound == "Free") {

              var trialUsedStorage = 0;

              if (localStorage.TrialUsed != null && typeof localStorage.TrialUsed != "undefined") {
                  trialUsedStorage = parseInt(localStorage.TrialUsed);
              }

              if (localStorage.TrailLicenceFound == "true" && trialUsedStorage > 0) {
                  
                  var d = new Date();

                  var day = "" + d.getDay();

                  if (localStorage.UpgradeDay == day) {
                      barLicence = "Expired";
                  } else {
                      barLicence = "NoShow";
                  }

              }
              
              if (localStorage.TrailLicenceFound == "false") {
                  barLicence = "NoTrialYet";
              }
          }

          if (sender.tab.url.indexOf("linkedin.com") != -1) {
              barLicence = "Pro";
          }
          
          sendResponse({ LicenceType: barLicence });
          
      }

      if (request.MessageType == "SetUpgradeDay") {
          var d = new Date();
          var day = "" + d.getDay();
          localStorage.UpgradeDay = "" + d.getDay();

      }
      //sets local storage values immediately after authorisation
      if (request.MessageType == "PostGmailAuth") {
          localStorage.GmailCalendarAuth = request.AuthId;
          localStorage.GmailCalendarSync = true;
      }

      //Fires when the user lands on the consoles full auth response page from a google auth
      if (request.MessageType == "PostGmailFullAuth") {
          localStorage.GmailAuth = request.Auth;
          gmailAuthFailed = false;
          //Move back to the tab after auth
          if (googleAuthRequestTabId != null && typeof googleAuthRequestTabId != 'undefined') {
              chrome.tabs.update(googleAuthRequestTabId, { active: true });
              chrome.tabs.executeScript(googleAuthRequestTabId, { code: "HandleGoogleAuthCompleted()" });
              googleAuthRequestTabId = null;

              //Close the tab we came from
              var currentTab = sender.tab.id;
              if (currentTab != null && typeof currentTab != 'undefined') {
                  chrome.tabs.remove(currentTab);
              }
          }
      }

      if (request.MessageType == "GoogleSendButton") {

          //console.log('GoogleSendButton');

          setTimeout(function() {
              chrome.tabs.getAllInWindow(null, function(tabs) {

                  for (var i = 0; i < tabs.length; i++) {
                      if (tabs[i].url.indexOf("https://mail.google.com/mail/") != -1) {
                          chrome.tabs.executeScript(tabs[i].id, { code: "AddGoogleSendButtons()" });
                      }
                  }
              });
          }, 500);
      }

      if (request.MessageType === "BARSETUP") {

          if (localStorage.LicenceFound == "Free" || localStorage.LicenceFound == "Pro" || localStorage.LicenceFound == "Enterprise") {
              localStorage.GmailGoLicence = true;
          }

          if (ValidateTheBasics()) {
              // As layout manager may take longer than a page load we need implement a retry action so when we send information back for this request it 
              // actually has the properties from the LayoutManager to send back
              RetryWithCallback(120, 1,
                  function () {
                      // As we always get a new layoutmanager on updating the extension this will enable the client 
                      // to have the most up-to-date settings as it will have to wait for the new Layoutmnager object 
                      // to return before sending values back
                      return localStorage.LayoutManager != null && globalIsThereALayoutManagerRequestActive === false;
                  },
                  function() {
                      var strAbRules = GetAbTestingRules(localStorage.ABTester);

                      //get the instance url
                      var instanceUrl = localStorage.InstanceUrl;

                      //Bar Customisation. Check the default bar search flag to see if we are doing a custom bar search or not. Custom = false.
                      var defaultBarSearch = true;
                      var defaultLocalStorage = localStorage.UseDefaultBarSearch;

                      if (defaultLocalStorage + "" === "false") {
                          defaultBarSearch = false;
                      }

                      var whoWhatFieldEnabledObj = IsWhoOrWhatFieldsEnabledOnTask();

                      var dailyCampaignLimitCount = 250;
                      //daily limit on campaign sending.
                      if (localStorage.DailyCampaignLimit != null) {
                          dailyCampaignLimitCount = localStorage.DailyCampaignLimit;
                      }

                      sendResponse({
                          GmailGo: "" + localStorage.GmailGo,
                          GmailGoLicence: "" + localStorage.GmailGoLicence,
                          LinkedinWizard: "" + localStorage.LinkedinWizard,
                          GmailWizard: "" + localStorage.GmailWizard,
                          PremWizard: "" + localStorage.PremWizard,
                          LoggedIn: true,
                          WhatVersion: whatProductApp,
                          TrailLicenceFound: "" + localStorage.TrailLicenceFound,
                          LicenceFound: "" + localStorage.LicenceFound,
                          ExpiredWizard: "" + localStorage.ExpiredWizard,
                          GmailViewLogged: "" + localStorage.GmailViewLogged,
                          GmailBar: "" + localStorage.SwitchbarOffInGmail,
                          AbRules: "" + strAbRules,
                          WebUserID: "" + localStorage.WebUserId,
                          InstanceUrl: "" + instanceUrl,
                          TrackingChoice: localStorage.TrackingChoice,
                          WhoWhatEnabledObject: whoWhatFieldEnabledObj,
                          EbstaGrowthTrial: localStorage.EbstaGrowthTrial,
                          GrowthTrialDaysRemaining: localStorage.GrowthTrialDaysRemaining,
                          useDefaultBarSearch: defaultBarSearch,
                          trackingAnalysis: localStorage.TrackingAnalysis,
                          enableMutationObservers: localStorage.EnableMutationObservers,
                          dailyCampaignLimit: dailyCampaignLimitCount,
                          RenderReplyData: localStorage.RenderReplyData,
                          GmailAccounts: localStorage.GmailAccounts,
                          CompanySettings: GetCompanySettings()
                      });
                  });
          } else {
              //Just send a basic response back to say the user isn't logged in so they can handle additional logic around that
              sendResponse({
                  LoggedIn: false
              });
          }

          return true;
      }
      
      if (request.MessageType == "PremWizardOff") {
          // LogEventShort("PremWizardShown");
          localStorage.PremWizard = true;

          var origin = "premsitewizard";
          CheckTrial2(origin);
      }
      
      if (request.MessageType == "LinkedinWizardOff") {
          //  LogEventShort("LinkedinWizardShown");
          localStorage.LinkedinWizard = true; 
      }
     
      if (request.MessageType == "ExpiredWizardOff") {
          //LogEventShort("GmailExpiredWizardShown");
          localStorage.ExpiredWizard = true;
      }
      
      if (request.MessageType == "GmailWizardOff") {
          // LogEventShort("GmailWizardShown");
          localStorage.GmailWizard = true;
      }

      if (request.MessageType == "LogGmailView") {

          if (localStorage.GmailViewLogged + '' != 'true') {

              LogEventShort("GmailViewLogged");
          }

          localStorage.GmailViewLogged = true;
      }

      //Log the type of gmail the user is using
      if (request.MessageType == "LogGmailType") {
          LogEventShort(request.LogType);
      }

      if (request.MessageType == "LogUsersGmailAddress") {
          //LogEventShort(request.LogType);
          if (request.EmailAddress != null && typeof request.EmailAddress != 'undefined') {
              //If account is not in local storage - log it and and get updated accounts from db. - Also set the local storage active flag to match that of the account
              if (!IsAccountRegisteredAndCheckActiveStatus(request.EmailAddress)) {
                  InsertGmailAddressForUser(request.EmailAddress);
              }
          }
      }
      
      
      if (request.MessageType == "GetAccountActive") {
          var responseObj = GoogleAccount.CheckGmailAccountEnabled(request.EmailAddress);
          sendResponse({
              AccountActive: "" + responseObj.AccountActive,
              DropBarWithNoAuth: responseObj.DropBarWithNoAuth,
              GoogleCalendarSyncBubbleClick: shouldWeShowCalendarSync,
              GoogleContactSyncBubbleClick: shouldWeShowContactSync
          });
      }
      
      if (request.MessageType == "GetGmailAccounts") {

          //Are we logged in? - if not dont check for accounts
          if (!localStorage.SessionCode || !localStorage.WebUserId || !localStorage.CrmId) {
              return;
          }

          if (typeof localStorage.GmailAccountActive != 'undefined' && localStorage.GmailAccountActive != 'reload') {
              if (request.CurrentEmailAccount != null && typeof request.CurrentEmailAccount != 'undefined') {
                  if (request.CurrentEmailAccount.length > 0) {
                      //Is the current gmail account in the local storage - Also set the local storage active flag to match that of the account
                      if (!IsAccountRegisteredAndCheckActiveStatus(request.CurrentEmailAccount)) {
                          //Set the local storage gmail accounts to active by default
                          localStorage.GmailAccountActive = 'true';
                          //If account is not in local storage - log it and and get updated accounts from db.
                          InsertGmailAddressForUser(request.CurrentEmailAccount, sendResponse);
                          return;
                      }
                  }
              }
          }

          if (typeof localStorage.GmailAccountActive === 'undefined') {
              //Set the local storage gmail accounts to active by default
              localStorage.GmailAccountActive = 'true';
              if (request.CurrentEmailAccount != null && typeof request.CurrentEmailAccount != 'undefined') {
                  if (request.CurrentEmailAccount.length > 0) {
                      //If we the gmail account active flag is not set (ie. if we have just logged in)
                      //Go check if this account exsists in local storage / and check its active / deactive status if its not there try to inser it and get new list of accounts
                      if (!IsAccountRegisteredAndCheckActiveStatus(request.CurrentEmailAccount)) {
                          InsertGmailAddressForUser(request.CurrentEmailAccount, sendResponse);
                          return;
                      }
                  }
              }
              //if we cant find the current email in local storage - go get a list of gmail accounts
              GetGmailAccounts(sender.tab.id, false, sendResponse);
              return true;
          }

          if (localStorage.GmailAccountActive == 'reload') {
              GetGmailAccounts(sender.tab.id, false, sendResponse);
              return true;
          }

          sendResponse();
      }

      if (request.MessageType == "CheckCalendarSyncSetup") {
          LoadCalendarStatus(1, sender.tab.id);
      }
      
      if (request.MessageType == "BarDrop") {
          
          var barLicence = "Pro";

          if (localStorage.LicenceFound == "Free") {
              if (localStorage.TrailLicenceFound == "true") {
                  var d = new Date();

                  var day = "" + d.getDay();

                  if (localStorage.UpgradeDay != day) {
                      barLicence = "Expired";
                  } else {
                      barLicence = "NoShow";
                  }

              }

              if (localStorage.TrailLicenceFound == "false") {
                  barLicence = "NoTrialYet";
              }
          }
          
          if (sender.tab.url.indexOf("linkedin.com") != -1) {
              barLicence = "Pro";
          }

          var alwaysDropBar = typeof gBatteryVentureCustomisation != undefined && gBatteryVentureCustomisation.IsCustomBarEnabled();

          sendResponse({
              Rules: "" + localStorage.Rules,
              ChatterOnly: "" + localStorage.ChatterOnly,
              LicenceType: barLicence,
              AlwaysDropBar: alwaysDropBar
          });
      }
      
      if (request.MessageType == "InjectGoogle") {

          sendResponse({ result: "injected" });
          chrome.tabs.executeScript(sender.tab.id, { file: 'js/jquery-3.5.1.min.js' });
         
      }

      if (request.MessageType == "GoogleInboxSearch") {

          //console.log('GoogleInboxSearch');

          var searchArry = JSON.parse(request.search);

          sendResponse({ result: "Processing" });
          RunBackgroundEmailSearch(searchArry, sender.tab.id, 0, request.Url,false,"normal");          
      }

      if (request.MessageType == "LogEventToDB") {
          LogEventShort(request.logEvent);
      }

      //log an email sent from gmail with tracking and template information if available.
      if (request.MessageType == "LogEmailSent") {
          var trackingLogId = '';
          var templateId = '';

          if (request.TrackingLogId != null && typeof request.TrackingLogId != "undefined") {
              trackingLogId = request.TrackingLogId;
          }

          if (request.TemplateId != null && typeof request.TemplateId != "undefined") {
              templateId = request.TemplateId;
          }

          LogEventShort('EmailSent - TrackingId: ' + trackingLogId + ', TemplateId: ' + templateId);
      }
      //remove EmailTrackingLog entry if gmail send has been undone
      if (request.MessageType == "DeleteEmailTrackingEntry")
      {
          if (localStorage.LastEmailTrackingLog != null && typeof localStorage.LastEmailTrackingLog != "undefined") {
              DeleteGmailTrackingRecords("DeleteEmailTrackingEntry", localStorage.LastEmailTrackingLog);
          }
      }

      if (request.MessageType == "AddTaskEmailComplex") {
          LogEventShort("TaskAdd-Complex");
          sendResponse({ result: "Processing" });
          AddTaskEmail(request, sender.tab.id);
      }

      if (request.MessageType == "AddTaskEmailComplexNew") {
          sendResponse({ result: "Processing" });
          AddTaskEmail(request, sender.tab.id);
      }

      if (request.MessageType === "GetComplexAddSettings") {

          if (globalComplexAddSettings) {
              if (!globalComplexAddSettings.searchableObjects) {
                  LogEventShort("SearchableObjects-NotFound-Retry");

                  UpdateComplexAddSettingsVariable();
              }
          } else {
              LogEventShort("globalComplexAddSettings-NotFound");
          }

          if (IsValidJSON(localStorage.LayoutManager)) {
              var lgm = JSON.parse(localStorage.LayoutManager);

              var resultList = GetTaskTypes(lgm);

              //Check we have the searchableObjects to make sure we have something always
              SetComplexAddSuggestionValues();

              sendResponse({
                  result: resultList,
                  CustomComplexAddSetup: globalComplexAddSettings.customFieldInfo,
                  ComplexAddSearchableObjects: globalComplexAddSettings.searchableObjects,
                  ComplexAddSuggestions: globalSuggestionSearchSettings.searchableObjects
              });
          }
      }

      if (request.MessageType == "AddTaskEmail") {

          LogEventShort("TaskAdd-Quick");
          sendResponse({ result: "Processing" });
          AddTaskEmail(request, sender.tab.id);
      }

      if (request.MessageType == "AddGmailTask") {
          sendResponse({ result: "Processing" });
          AddGmailTask(request, sender.tab.id);
      }

      if (request.MessageType == "SetFirstRecipientFlag") {
          storeFirstRecipient = request.SetFirstRecord;
      }

      if (request.MessageType == "RetrieveFirstRecipientStorage") {

          var firstRecipientStorage = localStorage.FirstRecipientResult;

          sendResponse({ json: firstRecipientStorage });
      }

      if (request.MessageType == "GetMeetingSchedulerUrl") {

          SelectEntity("getmeetingscheduler", "", GenerateBHKeyField(localStorage.WebUserId), sender.tab.id);

          //sendResponse({ json: firstRecipientStorage });
      }

      //Gets the tracked Related To local storage information and sends back the data
      if (request.MessageType == "GetLocalStorageRelatedToSelections")
      {
          var msgcache = "";
          if (localStorage.RelatedToSelections) {
              if (localStorage.RelatedToSelections.length != 0) {
                  msgcache = localStorage.RelatedToSelections;
              }
          }

          sendResponse({ relatedToHistory: msgcache });
      }

      if (request.MessageType == "GetLocalStorageEmailSearchResult") {
         
          if (localStorage.LastEmailSearchResult0 != null && localStorage.LastEmailSearchResult0.length != 0) {
              var msgcache = localStorage.LastEmailSearchResult0;
              sendResponse({ json: msgcache });
          } else if (localStorage.LastEmailSearchResult1 != null && localStorage.LastEmailSearchResult1.length != 0) {
              var msgcache = localStorage.LastEmailSearchResult1;
              sendResponse({ json: msgcache });
          }
      }

      //Clears local storage for cached Gmail searches and results
      if (request.MessageType == "ClearLocalStorageEmailSearchResults") {

          if (localStorage.LastEmailSearch0) {
              localStorage.LastEmailSearch0 = "";
          }
          if (localStorage.LastEmailSearchResult0) {
              localStorage.LastEmailSearchResult0 = "";
          }
          if (localStorage.LastEmailSearch1) {
              localStorage.LastEmailSearch1 = "";
          }
          if (localStorage.LastEmailSearchResult1) {
              localStorage.LastEmailSearchResult1 = "";
          }
      }

      if (request.MessageType == "GetLocalStorageLastSearchWord") {
          var lastSearch = "";
          if (localStorage.LastSearchWord) {
              if (localStorage.LastSearchWord.length != 0) {
                  lastSearch = localStorage.LastSearchWord;

                  //if we are using custom bar searches, send back the last custom bar search, not the last main search.
                  if (localStorage.UseDefaultBarSearch + "" === "false")
                  {
                      if (sender && sender.tab && sender.tab.url) {
                          //in gmail, we want to used to email as the last search term.
                          if (sender.tab.url.indexOf("mail.google.com") !== -1) {
                              lastSearch = localStorage.LastCustomSearchEmail;
                          } else { //otherwise on jobboards/linkedin we want to use the name.
                              lastSearch = localStorage.LastCustomSearchName;
                          }
                      }
                  }
              }
          }

          sendResponse({ result: lastSearch ? lastSearch : "nolastwordsearch" });
      }
     

      if (request.MessageType == "EmailLookup") {
          sendResponse({ result: "Processing" });

          RunBackgroundSearch(request.SearchTerm, sender.tab.id, "elookup-" + request.Entity, 0, sender.tab.url, 0);
          
      }

      if (request.MessageType == "SearchComplexAdd-Who") {
          RunBackgroundSearch(request.SearchTerm, sender.tab.id, "SearchComplexAdd-Who", 2, sender.tab.url, 0, sendResponse);
          return true;
      }

      if (request.MessageType == "SearchComplexAdd-What") {
          if (request.ObjectsToSearch) {
              UpdateSuggestionOrSearchObjectsVariable(request.ObjectsToSearch, globalComplexAddSettings, "ComplexAddSearchableObjects");
          }
          RunBackgroundSearch(request.SearchTerm, sender.tab.id, "SearchComplexAdd-What", 2, sender.tab.url, 0, sendResponse);
          return true;
      }

      if (request.MessageType == "SaveRelatedAndSuggestedSearchObjects") {
          if (request.RelatedObjectsToSearch) {
              UpdateSuggestionOrSearchObjectsVariable(request.RelatedObjectsToSearch, globalComplexAddSettings, "ComplexAddSearchableObjects");
          }
          if (request.SuggestedObjectsToSearch) {
              UpdateSuggestionOrSearchObjectsVariable(request.SuggestedObjectsToSearch, globalSuggestionSearchSettings, "ComplexAddSuggestionObjects")
          }
      }

      //Runs a search for an email adsress, firstly checking local storage then if finding nothing fires off a fresh email request
      if (request.MessageType == "RunEmailSearch") {
          var result = SearchLocalStorageForSingleEmail(request.EmailAddress);
          if (result) {
              sendResponse(result);
          } else {
              FireEmailSearchRequest(request.EmailAddress,
                  function SuccessCallBack(httpResponseData) {
                      //Success Callback will simply return the first result currently
                      if (httpResponseData != null && httpResponseData.d != null) {
                          var responseValues = httpResponseData.d.Values;
                          if (responseValues != null && responseValues.length > 0) {
                              sendResponse(JSON.parse(responseValues[0]));
                          }
                      }

                      sendResponse(null);
                  },
                  function FailureCallback() {
                      sendResponse(null);
                  });
          }
          return true;
      }

      if (request.MessageType == "EmailRelatedObjectLookup") {
          RunBackgroundSearch(request.SearchTerm, sender.tab.id, "RelatedObjectLookup", 0, sender.tab.url, 0, sendResponse);
          return true;
      }
      
      if (request.MessageType == "GoogleMessageSearch") {

          sendResponse({ result: "Processing" });
          var searchArry = JSON.parse(request.search);
          RunBackgroundEmailSearch(searchArry, sender.tab.id, 1, request.Url, request.barrun,"normal");
          SearchForGoogleMessageIds(request.GoogleMessageIds, sender.tab.id);
      }

      if (request.MessageType == "EmailTrack") {

          sendResponse({ result: "Processing" });

          var fieldList = '[';
          fieldList = fieldList + GenerateEntityField("TrackingLogId", request.TrackingLogId, '', false, '');
          fieldList = fieldList + ',' + GenerateEntityField("EmailSubject", request.EmailSubject, '', false, '');
          fieldList = fieldList + ',' + GenerateEntityField("Recipients", request.Recipients, '', false, '');
          fieldList = fieldList + ',' + GenerateEntityField("RecipientCount", request.RecipientCount, '', false, '');
          fieldList = fieldList + ',' + GenerateEntityField("TemplateId", request.TemplateId, '', false, '');
          fieldList = fieldList + ',' + GenerateEntityField("MailId", request.MailId, '', false, '');
          if (request.Base10MessageId) fieldList += ',' + GenerateEntityField("gmailBase10MessageId", request.Base10MessageId, '', false, '');
          if (request.ImmutableMessageId) fieldList += ',' + GenerateEntityField("immutableMessageId", request.ImmutableMessageId, '', false, '');
          fieldList = fieldList + ',' + GenerateEntityField("Mailbox", request.Mailbox, '', false, '');
          if (request.linktrack) {
              fieldList = fieldList + ',' + GenerateEntityField("linktrack", request.linktrack, '', false, '');
          }
          fieldList += "]";

          var jsonFieldList = JSON.stringify(fieldList);
          localStorage.LastEmailTrackingLog = request.TrackingLogId;
          LogEmailTrackingLogEvent("emailtrackinglogevent", jsonFieldList);

      }

      if (request.MessageType == "LoadWhoWhat") {

          var lgm = JSON.parse(localStorage.LayoutManager);

          var whoId = "";
          var whatId = "";
          
          for (var d = 0; d < lgm.d.CrmEntities.length; d++) {


              if (lgm.d.CrmEntities[d].Name == "Task") {
                  for (var l = 0; l < lgm.d.CrmEntities[d].Fields.length; l++) {
                      var field = lgm.d.CrmEntities[d].Fields[l];
                      
                      if (field.Name == "Name") {
                          whoId = field.DataSource; 
                      }
                      
                      if (field.ValueField == "WhatId") {
                          whatId = field.DataSource; 
                      }
                  }
              }
          }

          sendResponse({ whoId: whoId, whatId : whatId });
      }
      
      if (request.MessageType == "BarSearch" && localStorage.ChatterOnly != "1") {

          if (localStorage.CrmId == null) {
              sendResponse({ result: "notloggedin" });
              localStorage.RunSearchAgain = request.SearchTerm;
              localStorage.RunSearchTabId = sender.tab.id;
              localStorage.LastPageUrl = sender.tab.url;
              //if (sender.tab.url.indexOf(".linkedin.")) {
              //    chrome.tabs.executeScript(sender.tab.id, { code: "proc.DisplayFrame()" });
              //}


          } else {
              var mainUrl = sender.tab.url;
              var searchTerm = request.SearchTerm;
              var barSearchType = request.BarSearchType;

              if (mainUrl != null && typeof mainUrl != "undefined") {
                  if (mainUrl.indexOf("cwjobs.co.uk") != 1 || document.URL.indexOf("recruiter.totaljobs.com/Recruitment/CandidateSearch") != -1) {
                      var split = searchTerm.split(":");
                      if (split.length > 0) {
                          searchTerm = split[0].trim();
                      }
                  }

                  //Check to see if this a battery venture if so do our own custom request for them
                  if (typeof gBatteryVentureCustomisation != "undefined" && gBatteryVentureCustomisation.IsCustomBarEnabled()) {
                      gBatteryVentureCustomisation.RunAccountBarSearch(sender.tab.url, sendResponse, searchTerm, barSearchType);
                      return true;
                  } else {
                      RunBackgroundSearch(searchTerm, sender.tab.id, "coresearch", 1, sender.tab.url, request.ChangeView);
                  }
              }
          }
      }
          if (request.MessageType == "BarCompanySearch" && localStorage.ChatterOnly != "1") {

              sendResponse({ result: "running" });
              if (localStorage.CrmDBType == 2 || localStorage.CrmDBType >= 20) {

                  RunBackgroundSearch(request.SearchTerm, sender.tab.id, "coresearch", 1, sender.tab.url, request.ChangeView);
              }
          }

          if (request.MessageType == "ValidateSearch") {

              if (localStorage.WebUserId != null) {
                  sendResponse({ result: "1" });
              }
              else {
                  sendResponse({ result: "0" });
              }
          }

          if (request.MessageType == "CloseBar") {
              chrome.tabs.executeScript(sender.tab.id, { code: "proc.HideFrame()" });
          }

          if (request.MessageType == "LoadSingleContactSegments") {

              var objectToSearchId = request.ObjectId;
              if (sender != null && objectToSearchId != null) {
                  if (sender.tab != null && sender.tab.id != null) {
                        SelectEntity("cbcontactbyid", "", GenerateBHKeyField(objectToSearchId), sender.tab.id);
                        SelectEntity("cbtaskeventinfo", "", GenerateBHKeyField(objectToSearchId), sender.tab.id);

                        //If we found an Account associated with this record fire this off for our opportunity search
                        if (request.AccountId != null) {
                            objectToSearchId = request.AccountId;
                        }

                        SelectEntity("cballopportunities", "", GenerateBHKeyField(objectToSearchId), sender.tab.id);
                  }
              }             
          }

      //Custom Bar Search
          if (request.MessageType == "LoadCustomBarSegments") {

              //set up our parameters to pass through. We will search on both the name and the email address.
              var emailList = request.Email;
              var personName = request.NameSearchTerm;
              var salesforceId = request.SalesforceId;

              //Loop through the sections we need to run for this request
              if (request.SectionsToLoad) {
                  for (var i = 0; i < request.SectionsToLoad.length; i++) {
                      SelectEntity("cbcustombarsection", request.SectionsToLoad[i], GenerateCustomBarKeyfield(emailList, personName, salesforceId), sender.tab.id);
                  }
              }
              
              //update search history for custom bar search/selection.
              UpdateLastCustomSearch(emailList, personName, sender.tab.id, sender.tab.url);
          }

          if (request.MessageType == "LoadSingleLeadSegments") {
              var leadId = request.LeadId;

              SelectEntity("cbleadinfo", "", GenerateBHKeyField(leadId), sender.tab.id);
              SelectEntity("cbtaskeventinfo", "", GenerateBHKeyField(leadId), sender.tab.id);
          }

          if (request.MessageType == "LoadSingleAccountSegments") {

              var accountId = request.AccountId;

              SelectEntity("cbaccountinfo", "", GenerateBHKeyField(accountId), sender.tab.id);
              SelectEntity("cballopportunities", "", GenerateBHKeyField(accountId), sender.tab.id);
              SelectEntity("cbaccountcontacts", "", GenerateBHKeyField(accountId), sender.tab.id);

              var fields = '[';
              fields = fields + GenerateEntityField("ContactId", '', '', 'false', '');
              fields = fields + ']';

              var fieldsStr = JSON.stringify(fields);

              SelectEntity("cbtaskeventinfo", fieldsStr, GenerateBHKeyField(accountId), sender.tab.id);
          }

          if (request.MessageType == "LogBarClick") {
              LogEventShort(request.Element);
          }

          if (request.MessageType == "RestyleBar") {

              chrome.tabs.sendMessage(sender.tab.id, { MessageType: "RestyleBar", Url: localStorage.LastPageUrl, ZIndexValue: '' + request.ZIndexValue }, function (response) {

              });
          }
            
          if (request.MessageType == "GetTemplateFolders") {
              SelectEntity("gettemplatefolders", "", GenerateBHKeyField(localStorage.WebUserId), sender.tab.id);
          }
      
          if (request.MessageType == "GetEmailTemplateById") {

              //retrieve the first seached email result.
              var recipientResultObject = localStorage.FirstRecipientResult;

              var whoRecordId = '';
              if (request.WhoId) {
                  whoRecordId = request.WhoId;
              }

              var whoObjectType = '';
              if (request.WhoRecordType) {
                  whoObjectType = request.WhoRecordType;
              }

              var whatRecordId = '';
              if (request.WhatId) {
                  whatRecordId = request.WhatId;
              }

              var whatObjectType = '';
              if (request.WhatRecordType) {
                  whatObjectType = request.WhatRecordType;
              }

              var fields = '[';
              fields = fields + GenerateEntityField("WhoRecordId", whoRecordId, '', 'false', '');
              fields = fields + ',' + GenerateEntityField("WhoObjectType", whoObjectType, '', 'false', '');
              fields = fields + ',' + GenerateEntityField("WhatRecordId", whatRecordId, '', 'false', '');
              fields = fields + ',' + GenerateEntityField("WhatObjectType", whatObjectType, '', 'false', '');
              fields = fields + ']';

              var fieldsStr = JSON.stringify(fields);
              //fieldsStr = "";
              SelectEntity("getemailtemplatebyid", fieldsStr, GenerateBHKeyField(request.TemplateId), sender.tab.id);
          }
     
          if (request.MessageType == "GetTemplatesForFolderById") {

              var folderId = '';

              if (request.FolderId != null && typeof request.FolderId != 'undefined') {
                  localStorage.LastUsedEmailFolder = request.FolderId;
                  folderId = '' + request.FolderId;
              }
              SelectEntity("gettemplatesforfolderbyid", "", GenerateBHKeyField(folderId), sender.tab.id);
          }

          if (request.MessageType == "LogOpenButtonClicked") {

              LogEventShort("Email_views_Checked");
          }

          // email tracking - data request items
          // param: trackingLogId
          if (request.MessageType == "GetEmailTrackingItems") {
              var itemfields = '[' + GenerateEntityField("trackId", request.trackId, '', 'false', '') + ']';
              var itemStr = JSON.stringify(itemfields);
              SelectEntity("awesomeebstagetemailtrackingitems", "", itemStr, sender.tab.id);
          }

          // tracking bar
          if (request.MessageType == "OpenCloseTrackingPanelMessage") {
              chrome.tabs.sendMessage(parseInt(sender.tab.id), { MessageType: "OpenCloseTrackingPanelMessage", generalfields: "" });
          }

          //Check if a contact (by email) has tracking events
          if (request.MessageType == "CheckContactHasTrackingEvents") {
              CheckContactHasTrackingEventsRequest(request, sender.tab.id, request.ContactEmail);
          }

          //Updates local storage to keep track of the last tracking choice by the user
          if (request.MessageType == "LogTrackingChoice") {
              localStorage.TrackingChoice = request.TrackingChoice;
          }

          if (request.MessageType == "GetEmailTrackingHeadersAllEvents") {

              var pagefields = '[' + GenerateEntityField("page", request.Page, '', 'false', '') + ', ';
              pagefields += GenerateEntityField("mailstatus", request.MailStatus, '', 'false', '') + ']';

              var pageStr = JSON.stringify(pagefields);
              SelectEntity("awesomeebstagetemailtrackingheadersallevents", "", pageStr, sender.tab.id, sendResponse);
              return true;
          }

          if (request.MessageType == "GetEmailTrackingHeadersForContact") {

              var pagefields = '[' + GenerateEntityField("page", request.Page, '', 'false', '') + ', ';
              pagefields += GenerateEntityField("filteredbycontact", request.ContactEmail, '', 'false', '') + ', ';
              pagefields += GenerateEntityField("mailstatus", request.MailStatus, '', 'false', '') + ']';

              var pageStr = JSON.stringify(pagefields);
              SelectEntity("awesomeebstagetemailtrackingheadersforcontact", "", pageStr, sender.tab.id, sendResponse);
              return true;
          }

          if (request.MessageType == "GetEmailTrackingHeadersForOpenedMails") {

              var pagefields = '[' + GenerateEntityField("page", request.Page, '', 'false', '') + '] ';
              var pageStr = JSON.stringify(pagefields);

              SelectEntity("awesomeebstagetemailtrackingheadersforopenedmails", "", pageStr, sender.tab.id, sendResponse);
              return true;
          }

          if (request.MessageType == "FireHoverOverRequest") {
              var pagefields = '[' + GenerateEntityField("recordId", request.RecordId, '', 'false', '') + ', ';
              pagefields += GenerateEntityField("recordType", request.RecordType, '', 'false', '') + '] ';
              var pageStr = JSON.stringify(pagefields);
              var extraInformation = {
                  SendResponse: sendResponse
              }
              SelectEntity("getCompactLayoutDetailsForRecord", "", pageStr, sender, extraInformation);
              return true;
          }
        
          //Checks for fields needed to begin setup of Calendar Sync
          if (request.MessageType == "CalSyncSetupCheckSettings") {
              //request check of calendar settings
                var pagefields = '[' + GenerateEntityField("RefreshCache", request.RefreshCahe, '', 'false', '') + ']';

                var pageStr = JSON.stringify(pagefields);

                SelectEntity("CalSyncSetup_CheckCalendarSyncSettings", "", pageStr, sender.tab.id);
          }
            
          //Returns a list of user calendars and also their global settings such as syncing private events and recurring events
          if (request.MessageType === "SendCalendarSyncRequest") {
              SendCalendarSyncRequest(request, sendResponse);
              return true;
          }
          
          //Saves the preview state of a user on 'Preview Settings' being clicked
          if (request.MessageType == "CalSyncSavePreviewSettings") {
              //console.time("CalSyncSavePreviewSettings");
              var pagefields = '[';

              //This is to help track the last request which was made by the user to stop multiple calls rendering back in the client
              if (request.DateCheck != null)
              {
                  dateCheckForPreviewEventRequest = request.DateCheck;
              }

              if (localStorage.WebUserId)
              {
                  pagefields += GenerateEntityField("Id", localStorage.WebUserId, '', 'true', '');

                  if (request.GmailAccount != null) {
                      pagefields += "," + GenerateEntityField("GmailAccount", request.GmailAccount, '', 'false', '');
                  }

                  if (request.PrivateEvents != null) {
                      pagefields += "," + GenerateEntityField("DontSyncPrivateEvents", request.PrivateEvents, '', 'false', '');
                  }

                  if (request.RecurringEvents != null) {
                      pagefields += "," + GenerateEntityField("DontSyncRecurringEvents", request.RecurringEvents, '', 'false', '');
                  }

                  if (request.ColourEvents != null) {
                      pagefields += "," + GenerateEntityField("SyncOnlyColourEvents", request.ColourEvents, '', 'false' , '');
                  }

                  if (request.Calendars) {
                      var calendarItems = "[";
                      for (var i = 0; i < request.Calendars.length; i++) {
                          var newItem = '{';
                          newItem = newItem + '"id": ' + JSON.stringify(request.Calendars[i].calendarId) + ',';
                          newItem = newItem + '"isPrimary": ' + JSON.stringify(request.Calendars[i].isPrimary) + ',';
                          newItem = newItem + '"isSyncingToSalesforce": ' + JSON.stringify(request.Calendars[i].syncToSf) + ',';
                          newItem = newItem + '"isSyncingToGoogle": ' + JSON.stringify(request.Calendars[i].syncToGoogle) + ',';
                          newItem = newItem + '"accessRole": ' + JSON.stringify(request.Calendars[i].accessRole) + ',';
                          newItem = newItem + '"calendarTitle": ' + JSON.stringify(request.Calendars[i].calendarTitle);
                          newItem = newItem + '},';
                          calendarItems += newItem;
                      }
                      calendarItems += "]";

                      pagefields += "," + GenerateEntityField("CalendarItems", calendarItems, '', 'false', '');
                  }

                  if (request.IsOldAuthTokenBeingUsed) {
                      pagefields += "," + GenerateEntityField("IsOldAuthTokenBeingUsed", request.IsOldAuthTokenBeingUsed, '', 'false', '');

                      //Resetting the GmailAccount value as we're using the old auth token so we don't need to pass this up
                      request.GmailAccount = '';
                  }

                  pagefields += ']';

                  var pageStr = JSON.stringify(pagefields);

                  AddEntityNew("CalSyncSetup_AddPreviewSettingsGoogleCalendarSync", pageStr, sender.tab.id, request.GmailAccount);
              }
          }
            
          //Confirms the preview settings previously set are the ones the user has confirmed
          if (request.MessageType == "CalSyncConfirmSettings") {
              if (localStorage.WebUserId) {
                    var pagefields = '[';
                    pagefields += GenerateEntityField("webuserid", localStorage.WebUserId, '', 'true', '');

                    if (request.GmailAccount) {
                        pagefields += "," + GenerateEntityField("gmailaccount", request.GmailAccount, '', 'false', '');
                    }

                    if (request.IsOldAuthTokenBeingUsed) {
                        pagefields += "," + GenerateEntityField("IsOldAuthTokenBeingUsed", request.IsOldAuthTokenBeingUsed, '', 'false', '');
                    }

                    pagefields += ']';

                    var pageStr = JSON.stringify(pagefields);

                    AddEntity("CalSyncSetup_UpdateCalendarSyncPreviewSettingToBeLive", pageStr, sender.tab.id);
              }
          }
            
            // Get List views for Contact Sync Setup
            if (request.MessageType === "ContactSyncSetupListViews") {
                let fields = '[';
                fields = fields + GenerateEntityField("WebUserId", '' + localStorage.WebUserId, '', 'false', '');
                fields += "," + GenerateEntityField("GmailAccount", '' + request.GmailAccount, '', 'false', '');
                fields += "," + GenerateEntityField("SearchString", '' + request.SearchString, '', 'false', '');
                fields = fields + ']';

                SelectEntity("ContactSyncSetup_GetListViews", "", JSON.stringify(fields), sender.tab.id, sendResponse);
                return true;
            }
            
            //Send request to Stop a particular Contact Sync
            if (request.MessageType == "ContactSyncStopListSync") {
                var fields = '[';
                if (request.ContactSyncRuleId) {
                    fields = fields + GenerateEntityField("ContactSyncRuleId", '' + request.ContactSyncRuleId, '', 'false', '');
                }
                fields = fields + ']';

                var fieldsStr = JSON.stringify(fields);

                AddEntity("ContactSyncSetup_StopListSync", fieldsStr, sender.tab.id);
            }
            
            //Send request to restart Contact Sync (not supported currently RJW - 10/09/2015)
            if (request.MessageType == "ContactSyncRestartListSync") {
                var fields = '[';
                if (request.ContactSyncRuleId) {
                    fields = fields + GenerateEntityField("ContactSyncRuleId", '' + request.ContactSyncRuleId, '', 'false', '');
                }
                fields = fields + ']';

                var fieldsStr = JSON.stringify(fields);

                AddEntity("ContactSyncSetup_RestartListSync", fieldsStr, sender.tab.id);
            }
            
            //Send request to Save the user's preview settings which in turn fires off the request to get the preview contacts
            if (request.MessageType == "ContactSyncPreviewSettings") {
                var fields = '[';

                //This is to help track the last request which was made by the user to stop multiple calls rendering back in the client
                if (request.DateCheck != null) {
                    dateCheckForPreviewContactRequest = request.DateCheck;
                }

                if (request.ListViewId && request.ListViewName && request.FieldMappings) {
                    //List View Id
                    fields += GenerateEntityField("ListViewId", '' + request.ListViewId, '', 'false', '');

                    //List View Name
                    fields += "," + GenerateEntityField("ListViewName", '' + request.ListViewName, '', 'false', '');

                    //Gmail account
                    fields += "," + GenerateEntityField("GmailAccount", '' + request.GmailAccount, '', 'false', '');

                    //Field mappings
                    var fieldMappingsArray = "[";
                    for (var i = 0; i < request.FieldMappings.length; i++) {
                        var newItem = '{';
                        newItem = newItem + '"googleField": ' + JSON.stringify(request.FieldMappings[i].googleField) + ',';
                        newItem = newItem + '"salesforceField": ' + JSON.stringify(request.FieldMappings[i].salesforceField) + ',';
                        newItem = newItem + '},';
                        fieldMappingsArray += newItem;
                    }
                    fieldMappingsArray += "]";

                    fields += "," + GenerateEntityField("FieldMappings", fieldMappingsArray, '', 'false', '');
                }

                fields = fields + ']';

                var fieldsStr = JSON.stringify(fields);

                AddEntity("ContactSyncSetup_SavePreviewSettings", fieldsStr, sender.tab.id);
            }
            
            //Confirm Settings for COntact Sync
            if (request.MessageType == "ContactSyncConfirmSettings") {
                var fields = '[';
                if (request.ContactSyncRuleId) {
                    fields = fields + GenerateEntityField("ContactSyncRuleId", '' + request.ContactSyncRuleId, '', 'false', '');
                }

                fields += ']';

                var fieldsStr = JSON.stringify(fields);

                AddEntity("ContactSyncSetup_ConfirmSettings", fieldsStr, sender.tab.id);
            }

            //Mute/Unmute Notification Request
            if (request.MessageType === "Update_Tracking_Mute_Request") {
                var fields = '[';
                if (request.Mute != null) {
                    fields += GenerateEntityField("Mute", '' + request.Mute, '', 'false', '');
                    if (request.TrackingLogId != null) {
                        fields += "," + GenerateEntityField("TrackingLogId", '' + request.TrackingLogId, '', 'false', '');
                    }
                }

                fields = fields + ']';

                var fieldsStr = JSON.stringify(fields);
                AddEntityNew("emailtracking_mutenotification", fieldsStr, sender.tab.id, request);
                return true;
            }
            
          //This is fired on Gmail or Calendar page load and tells us if the user needs to be authed or not so we fire them into the auth process for the onboarding process
          if (request.MessageType === "CheckGmailAuthForMailbox") {
              let pagefields = '[' + GenerateEntityField("mailboxId", request.MailboxAddress, '', 'false', '') + ', ';
              pagefields += GenerateEntityField("webUserId", localStorage.WebUserId, '', 'false', '') + '] ';
              let pageStr = JSON.stringify(pagefields);

              SelectEntity("checkgmailauthformailbox", "", pageStr, sender.tab.id, null);

              return true;
        }

          if (request.MessageType === "CampaignMailboxAuthCheck") {
              let pageFields = '[' + GenerateEntityField("MailboxAddress", request.MailboxAddress, '', 'false', '') + ', ';

              if (request.IsSharedMailbox === true) {
                  pageFields += GenerateEntityField("IsSharedMailbox", "true", '', 'false', '') + ', ';
              }

              if (request.SharedMailboxUserId !== null) {
                  pageFields += GenerateEntityField("SharedMailboxUserId", "" + request.SharedMailboxUserId, '', 'false', '') + ', ';
              }

              pageFields += GenerateEntityField("TokenType", "" + request.TokenType, '', 'false', '') + ', ';
              pageFields += GenerateEntityField("webUserId", localStorage.WebUserId, '', 'false', '') + '] ';

              SelectEntity("campaignmailboxauthcheck", "", JSON.stringify(pageFields), sender.tab.id, null,
                  (function (lSendResponse) {
                      return function(responseData) {
                          lSendResponse(JSON.parse(responseData));
                      };
                  })(sendResponse));

              return true;
          }

          if (request.MessageType == "UpdateLocalGmail") {
              GoogleAccount.UpdateLocalStorage(request.GoogleAccountEmail, request.isActive);

              var fieldList = '[';
              fieldList = fieldList + GenerateEntityField("Id", localStorage.WebUserId, '', true, '');
              var emailList = '[';
              var newItem = '{';
              newItem = newItem + '"googleAccountEmail": "' + request.GoogleAccountEmail + '",';
              newItem = newItem + '"isActive": "' + request.isActive + '",';
              newItem = newItem + '}';
              emailList = emailList + newItem;
              emailList = emailList + ']';

              fieldList = fieldList + ',' + GenerateEntityField('GoogleAccountEmailList', emailList, '', 'false');
              fieldList = fieldList + ',' + GenerateEntityField('SiteURL', '' + localStorage.LastPageUrl, '', 'false');
              fieldList = fieldList + ',' + GenerateEntityField('SessionKey', '' + localStorage.SessionKey, '', 'false');
              fieldList = fieldList + ',' + GenerateEntityField('EBActionTaken', '' + localStorage.Action, '', 'false');
              fieldList = fieldList + ']';

              var strFields = JSON.stringify(fieldList);

              AddEntity('updategoogleaccounts', strFields, sender.tab.id);

              SendMessageToHideGmailButtonsAndLoguserOut(sender.tab.id, false);
          }


          if (request.MessageType == "UpdateGoogleAccountDontAuthNow") {
              //Updates ths the auth now setting for a mailbox
              var fieldList = '[';
              fieldList = fieldList + GenerateEntityField("googleAccountEmail", request.GoogleAccountEmail, '', 'false', '');
              fieldList = fieldList + ',' + GenerateEntityField('googleDontAuthNow', request.GoogleDontAuthNow, '', 'false', '');
              fieldList = fieldList + ']';

              var fieldsObj = JSON.stringify(fieldList);

              AddEntityNew("updategoogleaccountdontauthnow", fieldsObj, sender.tab.id, "");
          }

          if (request.MessageType == "BarOpenWizardModals") {
              //send message to open wizard modal
              chrome.tabs.executeScript(sender.tab.id, { code: "HandleBarOpenWizardModals()" });
          }
            

          //Turn Calendar Sync off using the 'Stop Calendar Sync' button within the onboarding process
          if (request.MessageType == "Onboarding-StopCalendarSync") {
              GoogleCalendarSyncOnOff("CalendarSyncOff", sender.tab.id);
          }
            
          //User has request to sync their calendars now, fire a request to make this happe
          if (request.MessageType == "Onboarding-SyncCalendarNow") {
              var fieldList = '[';
              fieldList += GenerateEntityField("", "", "", "false", "");
              fieldList += ']';

              var fieldsObj = JSON.stringify(fieldList);

              AddEntityNew("CalSyncSetup_SyncCalendarNow", fieldsObj, sender.tab.id, "");
          }
          
          //Find the current active tab and window and sends back a URL for the wizard button within the bar
          if (request.MessageType == "GetPageUrl") {             
              chrome.tabs.query({ "active": true, windowId: chrome.windows.WINDOW_ID_CURRENT }, function (tabs) {
                  sendResponse({ url: tabs[0].url });
              });

              // Returning true lets us send the response back to the intial message sender
              // http://stackoverflow.com/questions/20077487/chrome-extension-message-passing-response-not-sent/20077854#20077854
              return true; 
          }

      /*
       * Keeps a record of a user requesting a 'Save On Send' process, so when the Gmail
       * request is complete we can come pick it up later and then add to the users salesforce
       */
      if (request.MessageType == "AddSaveOnSendObject") {
          gmailStorage.AddSaveOnSendObject(request);

          //We need to keep track of the senders id so we know where to display the message when trying to save the object
          global_AddGmailTaskSenderTabId = sender.tab.id;
      }

      //Returns the save on send object matching the request its looking for
      if (request.MessageType === "GetSaveOnSendObject") {
          sendResponse({
              SaveOnSendObject: gmailStorage.GetSaveOnSendObject(request.ComposeId, request.ImmutableMessageId, request.Mailbox)
          });
      }

      if (request.MessageType === "AddEmailTrackingObject") {
          gmailStorage.AddEmailTrackingObject(request);
      }

      if (request.MessageType === "GetEmailTrackingObject") {
          sendResponse({ FoundObject: gmailStorage.GetEmailTrackingObject(request.TrackingIdMatches, request.Mailbox) });
      }

          //Logs a user out and updates the bar to show logged out state
          if (request.MessageType === "LogUserOut") {
              ClearCacheAndLogout();
              if (sender != null) {
                  if (sender.tab != null) {
                      if (sender.tab.id != null) {
                          SendMissingAuth(sender.tab.id);
                      }
                  }
              }
              return true;
          }

        //Saves a scheduled email once user presses Send
        if (request.MessageType === "sendlatersaveschedule") {
            var fieldList = '[';

            if (request.MessageId != null) {
                sendLaterDraftStorageArray.push(request.MessageId);
                fieldList += GenerateEntityField("emailid", request.MessageId, "", "false", "");
            }

            fieldList += "," + GenerateEntityField("gmailusername", request.GmailUsername, "", "false", "");
            fieldList += "," + GenerateEntityField("senddatetime", request.ScheduledDate, "", "false", "");
            //If we have a value for this draft already in the table lets just update the row with the new draft and time
            if (request.CurrentSendLaterId != null) {
                fieldList += "," + GenerateEntityField("currentsendlaterid", request.CurrentSendLaterId, "", "false", "");
            }

            if (request.IsEmailTracked) {
                fieldList += "," + GenerateEntityField("isemailtracked", request.IsEmailTracked, "", "false", "");
            }

            if (request.ComplexAddValues) {
                fieldList += "," + GenerateEntityField("complexaddvalues", JSON.stringify(request.ComplexAddValues), "", "false", "");
            }

            if (request.SfTemplateId) {
                fieldList += "," + GenerateEntityField("sftemplateid", request.SfTemplateId, "", "false", "");
            }

            fieldList += ']';

            AddEntityNew("sendlatersaveschedule", JSON.stringify(fieldList), sender.tab.id, "");
        }
            
        //Cancels a Scheduled email once user press Cancel Schedule
        if (request.MessageType === "sendlatercancelschedule") {
            AddEntityNew("sendlatercancelschedule", JSON.stringify("[" + GenerateEntityField("sendlaterid", request.SendLaterId, "", "true", "") + "]"), sender.tab.id, "");
        }

        //Searches to see if an email has been scheduled
        if (request.MessageType === "selectsendlaterschedule") {
            SelectEntity("selectsendlaterschedule", "", JSON.stringify("[" + GenerateEntityField("emailId", request.MessageId, "", "true", "") + "]"), sender.tab.id, sendResponse);
            return true;
        }

        //Send Later - Update Draft Id
        if (request.MessageType === "updatedraftid") {
            if (request.OldDraftId != null && request.NewDraftId) {
                UpdateSendLaterDraftArray(request.OldDraftId, request.NewDraftId);
            }
        }

        // campaign manager get most recently used campaigns
        if (request.MessageType === "GetMostRecentlyUsedCampaigns") {       
            GetMostRecentlyUsedCampaigns(sendResponse);
            return true;
        }

        // campaign manager serch campaigns for user by campaign name
        if (request.MessageType === "SearchCampaigns") {
            SearchCampaigns(request, sendResponse);
            return true;
        }


        // campaign manager get details for specific campaign
        if (request.MessageType === "SendCampaignRequest") {
            SendCampaignRequest(request, sendResponse);
            return true;
        }

        // campaign manager get details for specific campaign
        if (request.MessageType === "GetCampaignDetails") {
            GetCampaignDetails(request, sendResponse);
            return true;
        }

        if (request.MessageType === "GetCampaignSetup") {
            GLOBAL_EBSTA_CAMPAIGNMANAGERVALUES.GetSetup(sendResponse, request.ForceHttpRequestRetry);
            return true;
        }
        
        // get campaign task template by id
        if (request.MessageType === "GetTemplateById") {
            GetTemplateById(request, sendResponse);
            return true;
        }

        // make edited copy of template 
        if (request.MessageType === "MakeEditedCopyOfTemplate") {
            MakeEditedCopyOfTemplate(request, sendResponse);
            return true;
        }

        // save contact to campaign
        if (request.MessageType === "SaveContactToCampaign") {
            SaveContactToCampaign(request, sendResponse);
            return true;
        }

        //Saves a mass campaign
        if (request.MessageType === "SaveMultiRecipientsToCampaign") {
            SaveMultiRecipientsToCampaign(request, sendResponse);
            return true;
        }

        //get salesforce objects for multi recipient campaigns
        if (request.MessageType === "GetCampaignSalesforceListViewObjects") {
            GetCampaignSalesforceListViewObjects(sendResponse);
            return true;
        }

        //get list views for an object
        if (request.MessageType === "GetListViewsForObject") {
            GetListViewsForObject(request, sendResponse);
            return true;
        }

        //get people from a salesforce list view
        if (request.MessageType === "GetPeopleFromListView") {
            GetPeopleFromListView(request, sendResponse);
            return true;
        }

        if (request.MessageType === "ShouldWeShowBar" && request.Url) {
            sendResponse(ShouldWeShowBar(request.Url));
            return true;
        }

      if (request.MessageType === "SaveLastSearchedCampaignUsers" && request.SavedIds) {
          gEbstaSettings.SaveLastSearchedCampaignUsers(request.SavedIds);
          sendResponse();
          return true;
      }

      if (request.MessageType === "GetLastSearchedCampaignUsers") {
          sendResponse(gEbstaSettings.GetCampaignSettings().lastSearchedCampaignUsers);
          return true;
      }
  });


/*
 * Updates local storage and ubble paging cache when running custom bar search. 
 */
function UpdateLastCustomSearch(emailList, personName, tabId, tabUrl)
{
    var searchTermForBubble = personName;

    //set the recent search term. email if google, name if jobboard/linkedin.
    if (tabUrl.indexOf("mail.google.com") !== -1) {
        searchTermForBubble = emailList;
        localStorage.LastSearchWord = emailList;
    }

    //set some local storage for last search term.
    localStorage.LastCustomSearchEmail = emailList;
    localStorage.LastCustomSearchName = personName;

    //update the recent search.
    localStorage.RecentSearch = '<a href="Main.html?action=Search&search=' + searchTermForBubble + '">' + searchTermForBubble + '</a>';

    //update the page history.
    AddUpdateCache(searchTermForBubble, "", "Main.html?action=Search&search=" + searchTermForBubble, tabId, tabUrl);
}

//CheckForEmailTrackingFilterRequest
function CheckContactHasTrackingEventsRequest(request, tabId, contactEmail) {
    if (trackingPollObject != null) {
        var paramfields = '['

        if (contactEmail) {

            if (contactEmail.length !== 0) {
                paramfields += GenerateEntityField("filteredByContact", contactEmail, '', 'false', '');
            }
        }

        paramfields += ']';
        var paramfieldsStr = JSON.stringify(paramfields);
        SelectEntity("awesomeebstacheckcontacthastrackingevents", "", paramfieldsStr, tabId);
    }
}

function ReRunSearchAgainstTab() {
    if (localStorage.RunSearchAgain != null) {
        if (localStorage.RunSearchAgain != '') {

            var tabId1 = parseInt(localStorage.RunSearchTabId);

            RunBackgroundSearch(localStorage.RunSearchAgain, tabId1, "coresearch", 1, localStorage.LastPageUrl, 1);

            localStorage.RunSearchAgain = "";
           localStorage.RunSearchTabId = "";

        }
    }
}

var runBGSearchTimer = new Date();

function RunBackgroundSearch(searchTerm,tabId,searchType,showBar,tabUrl,changeView,sendResponse) {

    if (localStorage.CrmDBType != null) {
       
            localStorage.removeItem('coresearch-Search');

        if (localStorage.SessionCode != null) {
            if (localStorage.SessionCode != '00000000-0000-0000-0000-000000000000') {
                if (searchTerm != null) {
                    if (tabUrl.indexOf("mail.google.com") != -1) {

                        if (searchTerm.indexOf("@") != -1) {

                            var newArry = new Array();
                            newArry.push(searchTerm);

                            var runnormal = true;

                            if (localStorage.GmailGo + '' == 'true' && localStorage.GmailAccountActive + '' == 'true') {
                                if (localStorage.GmailGoLicence + '' == 'true') {
                                    runnormal = false;

                                    if (searchType == null) {
                                        searchType = "";
                                    }

                                    if (searchType.indexOf("RelatedObjectLookup") != -1 || searchType.indexOf("SearchComplexAdd") != -1)
                                    {
                                        runnormal = true;
                                    }

                                    //Bypass the RunBackgroundEmailSearch and go straight to Searchbackground to search in live for Complex Add and Related Object search
                                    if (!runnormal)
                                    {
                                        if (searchType.indexOf("-ComplexAdd") != -1) {
                                            RunBackgroundEmailSearch(newArry, tabId, 1, tabUrl, true, "ComplexAdd");
                                        } else {
                                            if (searchType.indexOf("slookup") != -1) {
                                                RunBackgroundEmailSearch(newArry, tabId, 1, tabUrl, false, searchType);
                                            } else {
                                                RunBackgroundEmailSearch(newArry, tabId, 1, tabUrl, true, "normal");
                                            }
                                        }
                                    }
                                }
                            }

                            if (runnormal) {
                                SearchBackground(searchType, searchTerm, tabId, showBar, tabUrl, changeView, sendResponse);
                            }
                        } else {

                            SearchBackground(searchType, searchTerm, tabId, showBar, tabUrl, changeView, sendResponse);
                        }

                    } else {
                        SearchBackground(searchType, searchTerm, tabId, showBar, tabUrl, changeView, sendResponse);
                    }
                }
            } else {
                SendMissingAuth(tabId);
            }
        } else {
            SendMissingAuth(tabId);
        }
    }
}

function SendMissingAuth(tabId) {

    chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "BarSearchResults", SearchTerm: "", Result: "0", SearchType: "MissingAuth", CrmDBType: localStorage.CrmDBType, Url: localStorage.LastPageUrl }, function (response) {
    });
    SendMessageToHideGmailButtonsAndLoguserOut(tabId, true);
}



function LogEventShort(eventName) {

    // switch off logging.
    if (disableLogging) {
        return;
    }

    var searchFieldsLog = "[" + GenerateEntityField('SessionKey', '' + localStorage.SessionKey, '', 'false');
    searchFieldsLog = searchFieldsLog + ',' + GenerateEntityField('EBActionTaken', '' + localStorage.Action, '', 'false');
    searchFieldsLog = searchFieldsLog + ',' + GenerateEntityField('SiteURL', '' + localStorage.LastPageUrl, '', 'false') + ',' + GenerateEntityField('SessionCode', '' + localStorage.SessionCode, '', 'false') + ',' + GenerateEntityField('CrmDBType', '' + localStorage.CrmDBType, '', 'false') + "]";

    var strFieldsLog = JSON.stringify(searchFieldsLog);

    var varDataLog = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","eventName": "' + eventName + '","fields": ' + strFieldsLog + '}';

    if (localStorage.AddUrl != null) {
        $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url: localStorage.AddUrl + "/LogEventShortSalesforce", // Location of the service
            data: varDataLog, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            timeout: 30000,
            success: function () { //On Successfull service call

            },
            error: function () {

            }
        });
    }
}

function LogClientEvent(eventName, fields) {

    // switch off logging.
    if (disableLogging) {
        return;
    }

    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","entityName": "' + eventName + '","fields": ' + fields + '}';

    if (localStorage.AddUrl != null) {
        $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url: localStorage.AddUrl + "/LogClientEvent", // Location of the service
            data: varData, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            timeout: 30000,
            success: function () { //On Successfull service call

            },
            error: function () {

            }
        });
    }
}


function LogEmailTrackingLogEvent(eventName, fields) {

    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","eventName": "' + eventName + '","fields": ' + fields + '}';

    if (localStorage.AddUrl != null) {
        $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url: localStorage.AddUrl + "/LogEmailTrackingLogEvent", // Location of the service
            data: varData, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            timeout: 30000,
            success: function () { //On Successfull service call

            },
            error: function () {

            }
        });
    }
}

function SearchBackground(entityname1, search, tabId, showbar, tabUrl, changeView, sendResponse) {


     //var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","searchName": ' + JSON.stringify(search) + ',"entityToSearch": "' + entityname + '"}';
    localStorage.LastPageUrl = tabUrl;
    var entityname = entityname1;
    var elookupSearch = false;
    var slookupSearch = false;
    var relatedLookupSearch = false;
    var complexAddSearch = false;
    var complexAddSearchOld = false;
    
    if (entityname1.indexOf("elookup-") != -1) {
        entityname = entityname.replace("elookup-", "lookup-");
        elookupSearch = true; 
    }

    if (entityname1.indexOf("slookup-") != -1) {
        entityname = "emailsearch";
        elookupSearch = true;
        slookupSearch = true;
    }

    var searchFields = "[" + GenerateEntityField('SearchType', 'auto', '', 'false');


    if (entityname1.indexOf("RelatedObjectLookup") != -1) {
        elookupSearch = true;
        relatedLookupSearch = true;
        var objectString = GetUserChosenSearchableObjectsForSuggestedOrRelatedSearches(globalSuggestionSearchSettings);

        if (objectString.length > 0) {
            searchFields += "," + GenerateEntityField('SuggestedSearchObjects', objectString, '', 'false');
        } else {
            //If no objects have been selected then lets just stop
            sendResponse(null);
            return false;
        }
    }

    if (entityname1.indexOf("-ComplexAdd") != -1) {
        elookupSearch = true;
        complexAddSearchOld = true;
    }
    
    var filterString = JSON.stringify(localStorage.BlockObject);

    if (typeof filterString === 'undefined') {
        filterString = '""';
    }

    //Detect if this is a search for Complex Add and get the objects chosen to search for
    if (entityname1.indexOf("SearchComplexAdd") >= 0) {
        elookupSearch = true;
        complexAddSearch = true;
        if (entityname1 === "SearchComplexAdd-What") {
            var objectString = GetUserChosenSearchableObjectsForSuggestedOrRelatedSearches(globalComplexAddSettings);
            if (objectString.length > 0) {
                searchFields += "," + GenerateEntityField('ComplexAddSearchObjects', objectString, '', 'false');
            }
        }
        //Trigger new search type by inserting new request name 'SearchObjectComplexAdd' 
        entityname = entityname1.replace("SearchComplexAdd", "SearchObjectComplexAdd");
    }

    searchFields = searchFields + ',' + GenerateEntityField('SessionKey', '' + localStorage.SessionKey, '', 'false');
    searchFields = searchFields + ',' + GenerateEntityField('EBActionTaken', '' + localStorage.Action, '', 'false');
     searchFields = searchFields + ',' + GenerateEntityField('SiteURL', '' + localStorage.LastPageUrl, '', 'false') + "]";

    var strFields = JSON.stringify(searchFields);

    var varData = '{"sessionId": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","searchName": ' + JSON.stringify(search) + ',"entityToSearch": "' + entityname + '","orderBy": "0","pageNumber": 1,"amountonPage":  10 ,"filters": ' + filterString + ',"searchFields": ' + strFields + '}';

    var popupWindows = chrome.extension.getViews({ type: 'popup' });
    if (search.trim().length < 3 && !complexAddSearch && !complexAddSearchOld) {
        if (popupWindows.length) {
            popupWindows[0].ShowSearchError('Please enter at least 3 characters in the search box.');
        }

    } else {

        var storeSearchMade = entityname + '-SearchMade';
        localStorage[storeSearchMade] = search;

        $.ajax({
            type: "POST", //GET or POST or PUT or DELETE verb
            url: localStorage.CrmUrl + "/SearchV2", // Location of the service
            data: varData, //Data sent to server
            contentType: "application/json; charset=utf-8", // content type sent to server
            dataType: "json", //Expected data format from server
            processdata: true, //True or False
            timeout: 30000,
            success: function (msg) { //On Successfull service call
                if (msg.d != null) {
                    if (msg.d.ErrorMessage != null) {
                        if (msg.d.ErrorMessage.indexOf('REAUTH:') != -1 || msg.d.ErrorMessage.indexOf('Unauthorized') != -1) {
                            ClearCacheAndLogout();
                            chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "BarSearchResults", SearchTerm: "", Result: "0", SearchType: "MissingAuth", CrmDBType: localStorage.CrmDBType, Url: localStorage.LastPageUrl }, function (response) { });
                           
                        }
                    }
                }
                var lastSearch = localStorage[storeSearchMade];

                var searchTermOffJson = $(document.createElement('div')).html(msg.d.SearchTermUsed).text();
                
                if (lastSearch == searchTermOffJson || relatedLookupSearch) {

                    var objJson = JSON.stringify(msg);
                    if (!elookupSearch) {
 
                        var store = entityname + '-Search';

                        localStorage.LastSearchWord = search;
                        localStorage.LastTabId = tabId;
                        localStorage[store] = objJson;

                        localStorage.RecentAutoSearch = '<a href="Main.html?action=Search&search=' + search + '">' + search + '</a>';
                        var lk = false;
                        if (tabUrl != null) {
                            tabUrl = '' + tabUrl;
                        }

                        var resultCount = msg.d.Values.length;
                        localStorage.HelpMessage = 'You can add a candidate or a contact by using the icons below';
                        if (changeView != "0") {
                            if (lk) {

                                AddUpdateCache(search, "", "Main.html?action=Search&msgLk=1&search=" + search, tabId, tabUrl);
                            } else {
                                if (resultCount == 0) {
                                    AddUpdateCache(search, "", "Main.html?action=Picker", tabId, tabUrl);
                                } else {
                                    AddUpdateCache(search, "", "Main.html?action=Search&search=" + search, tabId, tabUrl);
                                }
                            }
                        }
                        
                        if (showbar == 1) {
                            var sendBar = true;
                            if (lk) {
                                if (resultCount == 0) {
                                    resultCount = -2;
                                    sendBar = false;
                                }
                            }
                            if (sendBar) {
                                if (lk) {
                                    if (resultCount > 0) {
                                        chrome.tabs.executeScript(parseInt(tabId), { code: "proc.DisplayFrame()" });
                                    }
                                }
                                chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "BarSearchResults", SearchTerm: "" + search, Result: "" + resultCount, SearchType: "" + entityname, CrmDBType: localStorage.CrmDBType, Url: localStorage.LastPageUrl, DataObject: objJson }, function (response) {

                                    // This should be results.. ?
                                    LogEventShort("Bar-Dropped");
                                });
                               
                            } else {
                                if (resultCount == -2) {
                                    chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "BarSearchResults", SearchTerm: "" + search, Result: "" + resultCount, SearchType: "" + entityname, CrmDBType: localStorage.CrmDBType, Url: localStorage.LastPageUrl, DataObject: objJson }, function (response) {
                                        // This should be results.. ?
                                        LogEventShort("Bar-Dropped");
                                    });
                                }
                            }
                        }
                        
                    } else {
                        if (slookupSearch)
                        {
                            ////console.log("slookup Back");
                            chrome.tabs.executeScript(tabId, { code: "PassBackSLookup('" + entityname1 + "'," + objJson + ",'" + msg.d.SearchTermUsed + "')" });
                        }
                        else if (relatedLookupSearch || complexAddSearch)
                        {
                            sendResponse(objJson);
                        }
                        else {
                            ////console.log("Elookup Back");
                            // Elookup. 
                            chrome.tabs.executeScript(tabId, { code: "PassDataBack('" + entityname1 + "'," + objJson + ",'" + msg.d.SearchTermUsed + "')" });
                        }

                    }
                }
            },
            error: function(xhr, status, error) {
              
            }
        });
    }
}

function DisplayGoogleInboxMessage(message, tabId,msgCss) {

    var intTabId = parseInt(tabId);

    chrome.tabs.sendMessage(intTabId, { MessageType: "DisplayGmailMessage", MessageContent: "" + message, MsgCss: msgCss }, function (response) {
        // This should be results.. ?
    });
}

function ReRunTabSearch() {

    //chrome.tabs.getSelected(null, function (tab) {
    //    chrome.tabs.executeScript(parseInt(tab.id), { code: "proc.ReRunSearch()" });
    //});

    chrome.tabs.query({
        active: true               // Select active tabs
    }, function (array_of_Tabs){
        if (array_of_Tabs.length > 0)
        {
            var tabID = -1;
            for (var i = 0; i < array_of_Tabs.length ; i++)
            {
                if (array_of_Tabs[i].url.indexOf("chrome-devtools") == -1)
                {
                    tabID = array_of_Tabs[i].id;
                    break;
                }
            }

            if (tabID != -1)
            {

                chrome.tabs.executeScript(parseInt(tabID), { code: "proc.ReRunSearch()" });
            }
        }
    });
    
}

function UpdateTabId(tabId) {
    /// Remeber what tabs we have in storage so we can wipe if need and check. 

}




function SetLastActivity() {
    localStorage.LastActivity = new Date();
}

function LoadTaskBar(objJson) {
    var openTasks = JSON.parse(objJson);
    var tabId = 0;
    var tabUrl = '';
     
    var len = parseInt(openTasks.d.Values[0]);
    localStorage.NewOpenTasks = len;

    chrome.tabs.getSelected(null, function (tab) {
        tabId = tab.id;
        tabUrl = tab.url;


        if (len > 0) {
            chrome.tabs.executeScript(parseInt(tabId), { code: "proc.LoadFrame('usertasksbar', 26)" });
            localStorage.OpenFromTaskBar = 1;
        }
        AddUpdateCache("", "", "Main.html?action=Welcome", tabId, tabUrl);
    });

    SetLastActivity();
}

function GetExtraInformation(typeExtraInformation, keyId, assignment,tabId) {

        
    if (typeExtraInformation == "ContactAccount") {
        SelectEntity("contactaccount", "", GenerateBHKeyField(keyId), tabId);
    }

    if (typeExtraInformation == "SearchAccountAuto") {
        Search("SearchAccountAuto", keyId, tabId, 1, null, null, null, 0);
    }
}

function CheckLayoutVersion() {

    // Check Version number. 
    if (IsValidJSON(localStorage.LayoutManager)) {
        var lgmObj = JSON.parse(localStorage.LayoutManager);

        try {
            // Version number. Check. 
            if (lgmObj.d.VersionNumber + "" != "2") {
                GetFormLayout();
            }
        } catch (err) {
            // If we don't have the object lets go and pull.  
            GetFormLayout();
        }
    }
}

CheckLayoutVersion();

function CheckForLinkedinStatus() {
    localStorage.LinkedInStatus = false;

    if (localStorage.ParseInformation != null) {


        var parseVal = JSON.parse(localStorage.ParseInformation);

        if (parseVal != null) {

            if (parseVal.Url.indexOf(".linkedin.") != -1) {
                localStorage.LinkedInStatus = true;
            }
        }
    }
}

function utf8_to_b64(str) {
    return window.btoa(unescape(encodeURIComponent(str)));
}

function isResumeField(fieldTotTest) {

    var isResField = false;

    if (fieldTotTest == "ts2__Text_Resume__c" || fieldTotTest == "TR1__TextResume__c" || fieldTotTest == "AVTRRT__Resume__c" /* Remove Click to Cloud CV Parsing|| fieldTotTest == "PeopleCloud1__Resume__c" */) {
        isResField = true;
    }

    return isResField;
}


function GoogleCalendarSyncOnOff(entityname, tabId) {    
    var isActive = "";
    if (entityname == "CalendarSyncOn")
    {
        isActive = "1";
    }

    if (entityname == "CalendarSyncOff") {
        isActive = "0";
    }

    var varData = '{"sessionCode": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","isActive": "' + isActive + '"}';
    var popupWindows = chrome.extension.getViews({ type: 'popup' });

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb        
        url: localStorage.MainConnector + "/Connector.svc/GoogleCalendarSyncOnOff", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call
            //            chrome.tabs.sendMessage(parseInt(tabId), { entityName: "candidate", result: "" + msg.d.Values[0] }, function (response) {

            //            });
            //ClearCache();

            var objJson = JSON.stringify(msg);
            if (popupWindows.length) { // A popup has been found              
                popupWindows[0].GmailCalendarSyncUpdateComplete(isActive);
            }

        },
        error: function (xhr, status, error) {

                DisplayGoogleInboxMessage('Failed to add sync with Gmail Calendar Salesforce.', tabId,"failure");
        }
    });
    
}

function GoogleCalendarSyncOnOffFullAuth(entityname, tabId) {
    var isActive = "";
    if (entityname == "CalendarSyncOn") {
        isActive = "1";
    }

    if (entityname == "CalendarSyncOff") {
        isActive = "0";
    }

    var varData = '{"sessionCode": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","isActive": "' + isActive + '"}';
    var popupWindows = chrome.extension.getViews({ type: 'popup' });

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb        
        url: localStorage.MainConnector + "/Connector.svc/GoogleCalendarSyncOnOffFullAuth", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call
            //            chrome.tabs.sendMessage(parseInt(tabId), { entityName: "candidate", result: "" + msg.d.Values[0] }, function (response) {

            //            });
            //ClearCache();

            var objJson = JSON.stringify(msg);
            if (popupWindows.length) { // A popup has been found              
                popupWindows[0].GmailCalendarSyncUpdateComplete(isActive);
            }

        },
        error: function (xhr, status, error) {

            DisplayGoogleInboxMessage('Failed to add sync with Gmail Calendar Salesforce.', tabId, "failure");
        }
    });

}

function PostBarInfo(responseData, entity, tabId) {
    //console.log("PostBarInfo");
    if (localStorage.LayoutManager != null)
    {
        chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "ProcessBarSections", DataObject: responseData, EntityType: entity, InstanceUrl: '' + localStorage.InstanceUrl, Url: '' + localStorage.LastPageUrl });
    }
}

//Bar Customisation. Custom Bar Search. Send off our message for each section
function PostCustomBarInfo(responseData, entity, tabId, sectionNumber) {
    if (localStorage.LayoutManager != null) {
        chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "ProcessCustomBarSections", DataObject: responseData, EntityType: entity, InstanceUrl: '' + localStorage.InstanceUrl, Url: '' + localStorage.LastPageUrl, SectionNumber: sectionNumber });
    }
}


//Load the status of a user's calendar sync object.
function LoadCalendarStatus(checkupType, tabId) {
    var varData = '{"sessionCode": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '"}';
    var popupWindows = chrome.extension.getViews({ type: 'popup' });

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb        
        url: localStorage.MainConnector + "/Connector.svc/GetCalendarSyncStatus", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call

            var objJson = JSON.stringify(msg);

            if (checkupType == 1) {
                chrome.tabs.sendMessage(tabId, { MessageType: "InjectCalendarSyncStatus", CalendarSyncObject: objJson, LoggedInWebUserId: localStorage.WebUserId }, function (response) {

                });
            } else {
                if (popupWindows.length) { // A popup has been found              
                    popupWindows[0].ShowCalendarSettingsStatus(objJson);
                }
            }
        },
        error: function (xhr, status, error) {

        }
    });
}

//Looks up to see if the users Google account has been deactivted if so set some flags to stop Ebsta doing it's thang
function HasThisGmailAccountBeenDeactivated(emailAddress)
{
    if (localStorage.GmailAccounts != null && localStorage.GmailAccounts != '')
    {
        var json = JSON.parse(localStorage.GmailAccounts);

        if (json.d != null)
        {
            var parsedJson = json.d;
            if (parsedJson != null)
            {
                for (var i = 0; i < parsedJson.length; i++)
                {
                    if (parsedJson[i]["GoogleAccountEmail"] != null && parsedJson[i]["IsActive"] != null)
                    {
                        if (parsedJson[i]["GoogleAccountEmail"] == emailAddress)
                        {
                            if (parsedJson[i]["IsActive"] == 0) {
                                localStorage.GmailAccountActive = "false";
                                break;
                            }
                            else {
                                localStorage.GmailAccountActive = "true";
                                break;
                            }
                        }
                    }
                }
            }
        }
    }
}

//Change - Amar 18.01.2015: Checks local storage for email before calling this function. Now inserts email to db and returns updated list of emails
function InsertGmailAddressForUser(emailAddress, callBack) {

    var varData = '{"sessionCode": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","emailAddress": "' + emailAddress + '"}';

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb        
        url: localStorage.MainConnector + "/Connector.svc/InsertGoogleAccountReturnList", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call

            //If server does not return null
            if (msg.d != null) {
                var objJson = JSON.stringify(msg);
                localStorage.GmailAccounts = objJson;
            }

            if (callBack) {
                callBack();
            }
            //Else - Server returned null - dont update local storage list
        },
        error: function (xhr, status, error) {
            if (callBack) {
                callBack();
            }
        }
    });
}

//Gets the users Gmail accounts for them to select in the settings creen
function GetGmailAccounts(tabid, renderPage, callBack)
{
    var varData = '{"sessionCode": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '"}';
    var popupWindows = chrome.extension.getViews({ type: 'popup' });

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb        
        url: localStorage.MainConnector + "/Connector.svc/GetGmailAccounts", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call

            var objJson = JSON.stringify(msg);

            localStorage.GmailAccounts = objJson;
            if (!renderPage)
            {
                chrome.tabs.executeScript(tabid, { code: "LogUsersGmailAddress()" });
            }
            else if (popupWindows.length > 0 && renderPage)
            {           
                popupWindows[0].ShowGmailAccounts(objJson);
            }

            if (callBack) {
                callBack();
            }
        },
        error: function (xhr, status, error) {
            if (callBack) {
                callBack();
            }
        }
    });
}
/**
 * Sends a message to hide gmail buttons and also log the user ou
 * @param {} tabId 
 * @param {} loguserOut 
 * @returns {} 
 */
function SendMessageToHideGmailButtonsAndLoguserOut(tabId, loguserOut) {
    chrome.tabs.sendMessage(parseInt(tabId), { MessageType: "HideGmailButtons", LogUserOut: loguserOut });
}

//Updates local storage to keep a record of what a user had selected as a 'Related To' object against a Who (Contact/lead)
function UpsertSelectedRelatedToObject(whoId, whatId, whatRecordType, whatName)
{
    if (whoId != null && whoId.length > 0)
    {
        //Get the Local Storage data
        var localStorageParsedJson = localStorage.RelatedToSelections;

        if (localStorageParsedJson != null) {
            localStorageParsedJson = JSON.parse(localStorageParsedJson);

            var relatedObjectArray = localStorageParsedJson["relatedToObjects"];

            if (relatedObjectArray != null) {

                // Remove the last entry for the whoid as the user may have just added to a contact and 
                // we don't want to keep suggesting a previous object they may have chosen
                for (var i = 0; i < relatedObjectArray.length; i++) {
                    var localStorageWhoId = relatedObjectArray[i].whoId;
                    if (localStorageWhoId != null) {
                        if (localStorageWhoId == whoId) {
                            relatedObjectArray.splice(i, 1);
                            break;
                        }
                    }
                }

                //We only want to save if there is actually info for what's
                if (whatId != null && whatId !== -1 && whatRecordType != null && whatName != null &&
                        whatId.length > 0 && whatRecordType.length > 0 && whatName.length > 0) {
                    //Insert into the start of the array so we can the most recent activty at the top
                    relatedObjectArray.unshift(JSON.parse(CreateRelatedObjectJson(whoId, whatId, whatRecordType, whatName)));
                }

                //If we're storing too many objects then lets remove the oldest one
                var maxStorageAmount = 50;
                if (relatedObjectArray.length >= maxStorageAmount) {
                    relatedObjectArray.splice(maxStorageAmount - 1, 1);
                }

                //Update Local Storage
                localStorageParsedJson["relatedToObjects"] = relatedObjectArray;
                localStorage.RelatedToSelections = JSON.stringify(localStorageParsedJson);
            }
        }
        else if (whatId != null && whatId !== -1 && whatRecordType != null && whatName != null &&
                    whatId.length > 0 && whatRecordType.length > 0 && whatName.length > 0)
        {
            var newJson = "{\"relatedToObjects\":[";
            newJson += CreateRelatedObjectJson(whoId, whatId, whatRecordType, whatName);
            newJson += "]}";

            newJson = JSON.parse(newJson);

            localStorage.RelatedToSelections = JSON.stringify(newJson);
        }
    }
}

//Creates a new object to store in localstorage to keep track of user selected items in complex add
function CreateRelatedObjectJson(whoId, whatId, whatRecordType, whatName) {

    whoId = EscapeCharactersForJsonParsing(whoId);
    whatId = EscapeCharactersForJsonParsing(whatId);
    whatRecordType = EscapeCharactersForJsonParsing(whatRecordType);
    whatName = EscapeCharactersForJsonParsing(whatName);

    var newJson = "{\"whoId\":\"" + whoId + "\",";
        newJson += "\"whatId\":\"" + whatId + "\",";
        newJson += "\"whatRecordType\":\"" + whatRecordType + "\",";
        newJson += "\"whatName\": \"" + whatName + "\"";
        newJson += "}";

    return newJson;
}


function AddGmailTask(request, tabId) {


    localStorage.TagAssociatedEmail = '' + request.EmailText;
    var fields = '[';
    fields = fields + GenerateEntityFieldNoFilter("gmailUserId", request.GmailUserId, '', 'false', '');
    if (request.AddType) fields = fields + "," + GenerateEntityFieldNoFilter("addType", request.AddType, '', 'false', '');
    if (request.RequestedFromUrl) fields = fields + "," + GenerateEntityFieldNoFilter("requestedFromUrl", request.RequestedFromUrl, '', 'false', '');
    if (request.GmailMessageId) fields = fields + "," + GenerateEntityFieldNoFilter("gmailMessageId", request.GmailMessageId, '', 'false', '');
    if (request.GmailBase10MessageId) fields = fields + "," + GenerateEntityFieldNoFilter("gmailBase10MessageId", request.GmailBase10MessageId, '', 'false', '');
    if (request.GmailImmutableId) fields = fields + "," + GenerateEntityFieldNoFilter("gmailImmutableId", request.GmailImmutableId, '', 'false', '');
    fields = fields + "," + GenerateEntityFieldNoFilter("gmailAttachmentIds", request.GmailAttachmentIds, '', 'false', '');
    fields = fields + "," + GenerateEntityFieldNoFilter("Subject", request.Subject, '', 'false', '');

        var whoId = -1;
        var whatId = -1;

        if (request.Email == "NEW") {
            whoId = request.WhoId;
            whatId = request.WhatId;
        } else {
            var useWhoId = false;

            if (request.SFType == "Contact") {
                useWhoId = true;
                whatId = GetWhatId(request.Email);
            }

            if (request.SFType == "Lead") {
                useWhoId = true;
            }

            if (useWhoId) {
                whoId = request.Email;
            } else {
                whatId = request.Email;
            }

        }

    //Update Local Storage to retain users last selected 'Related To' object for a certain Contact/Lead
    var whatRecordName = "", whatRecordType = "";

    if (whatId != -1)
    {
        if (whatId)
        {
            if (whatId.length > 0)
            {
                if (request.WhatRecordName) {
                    whatRecordName = request.WhatRecordName;
                }

                if (request.WhatRecordType) {
                    whatRecordType = request.WhatRecordType
                }                
            }
        }
    }

    //Variables to catch so we can prefill some values with our own if the user hasn't manually set these
    var isUserSettingTaskStatus = false, 
        isUserSettingTaskPriority = false,
        isUserSettingTaskType = false;

    if (typeof request.CustomFields != "undefined") {
        if (request.CustomFields != null) {
            for (var i = 0; i < request.CustomFields.length; i++) {
                if (request.CustomFields[i].FieldName != null && request.CustomFields[i].FieldValue != null && request.CustomFields[i].FieldType != null) {

                    switch (request.CustomFields[i].FieldName) {
                        case "Status":
                            isUserSettingTaskStatus = true;
                            break;
                        case "Priority":
                            isUserSettingTaskPriority = true;
                            break;
                        case "Type":
                            isUserSettingTaskType = true;
                            break;
                        default:
                            break;
                    }
                    fields = fields + "," + GenerateEntityFieldNoFilter(request.CustomFields[i].FieldName, request.CustomFields[i].FieldValue, request.CustomFields[i].FieldType, 'false', '');
                }
            }
        }
    }

    //Only if the user hasn't manually set the type in the custom field section of complex add should we get into here
    if (!isUserSettingTaskType) {
        //Set the task type to what the user has defined
        if (typeof request.TaskType != "undefined" && request.TaskType != null && request.TaskType !== "") {
            //if the user has selected No Type then send nothing over
            if (request.TaskType != "No Type") {
                fields = fields + "," + GenerateEntityFieldNoFilter("Type", '' + request.TaskType, '', 'false', '');
            }
        }
        //Finds if a user hasn't selected any extra fields in Complex Add but has 'Email' as a type on task, so sets the field to Email
        else if (GetLookupValue("Task-Types", "Email") == "Email") {
            fields = fields + "," + GenerateEntityFieldNoFilter("Type", 'Email', '', 'false', '');
        }
    }

    if (!isUserSettingTaskStatus) {
        fields = fields + "," + GenerateEntityFieldNoFilter("Status", 'EbstaCompleted', '', 'false', '');
    }

    if (!isUserSettingTaskPriority) {
        fields = fields + "," + GenerateEntityFieldNoFilter("Priority", 'Normal', '', 'false', '');
    }

    if (whatId != null) fields = fields + "," + GenerateEntityFieldNoFilter("WhatId", whatId, '', 'false', '');
    if (whoId != null) fields = fields + "," + GenerateEntityFieldNoFilter("WhoId", whoId, '', 'false', '');

    fields = fields + ',' + GenerateEntityField('SiteURL', '' + localStorage.LastPageUrl, '', 'false');
    fields = fields + ',' + GenerateEntityField('SessionKey', '' + localStorage.SessionKey, '', 'false');
    fields = fields + ',' + GenerateEntityField('EBActionTaken', '' + localStorage.Action, '', 'false');
    fields = fields + "]";

    var strFields = JSON.stringify(fields);

    // If this flag has been specified that means the user is current sending this email on a different tab 
    // so this will tell AddEntity to direct the success message to the right tab/window
    if (request.UseDifferentTabId) {
        tabId = global_AddGmailTaskSenderTabId;
    }

    //Keep track of the added ID so we can show this in the client
    InsertGmailIdsIntoLocalStorage([request.GmailMessageId]);

    AddEntityNew("GmailTask", strFields, tabId, request.Name);

    //Keep track of users selection
    UpsertSelectedRelatedToObject(whoId, whatId, whatRecordType, whatRecordName);
}

// we need to communicate with bar.js in order to show the bar
function ShowEmailTrackingSummaryBackground(tabId) {

    chrome.tabs.query({
        active: true               
    }, function () {

        if (tabId != -1) {
            chrome.tabs.executeScript(parseInt(tabId), { code: "ShowEmailTrackingSummaryBar()" });
        }
    });
}

function checkGmailAuth(tabId) {

    var gmailAuth = localStorage.GmailAuth;

    var fields = '[';
    fields = fields + GenerateEntityField("WebUserId", localStorage.WebUserId, '', 'false', '');
    fields = fields + ']';

    var fieldsStr = JSON.stringify(fields);

    if (typeof gmailAuth == 'undefined' || gmailAuth < 2) {
        SelectEntity("gmailauth", "", fieldsStr, tabId);
        return false;
    }

    return true;
}

function RefreshTrackingPoll() {

    var crmId = localStorage.CrmId;
    var webUserId = localStorage.WebUserId;
    var sessionCode = localStorage.SessionCode;

    //don't poll if we are logged out and/or we are going to fail at validation level anyway.
    if (crmId != null && typeof crmId != "undefined" && webUserId != null && typeof webUserId != "undefined" && sessionCode != null && typeof sessionCode != "undefined")
    {
        var fields = '[';
        fields = fields + GenerateEntityField("WebUserId", '' + localStorage.WebUserId, '', 'false', '');
        fields = fields + ']';

        var fieldsStr = JSON.stringify(fields);

        SelectEntity("awesomeebstatrackingpoll", "", fieldsStr, localStorage.LastTabId);
    }
    clearTimeout(trackingPollObject);
    trackingPollObject = setTimeout(RefreshTrackingPoll, 60000); //poll every minute.
}

//Grabs a localstorage value and returns it back a JSON Object
function Get_LocalStorage_Item_And_Return_JSON(localStorageKey) {
    if (localStorage.getItem(localStorageKey)) {
        return JSON.parse(localStorage.getItem(localStorageKey));
    }
    return new Object();
}

//Stops tracking poll from firing
function Stop_Tracking_Poll() {
    clearTimeout(trackingPollObject);
    trackingPollObject = null;
}

//Checks for local storage flag and starts or stops the trackingpoll 
function Start_Or_Stop_Tracking_Poll() {
    var emailSettings = Get_LocalStorage_Item_And_Return_JSON("EmailTrackingSettings");
    if (emailSettings != null) {
        if (emailSettings.TurnOffNotifications) {
            Stop_Tracking_Poll();
            return true;
        }
    }

    var abRules = GetAbtestingObject(localStorage.ABTester);
    if (abRules != null) {
        if (abRules.EnableGmailEmailTracking + "" !== "1") {
            Stop_Tracking_Poll();
            return true;
        }
    }

    clearTimeout(trackingPollObject);
    trackingPollObject = setTimeout(RefreshTrackingPoll, 60000);
}

Start_Or_Stop_Tracking_Poll();

function IsAccountRegisteredAndCheckActiveStatus(gmailAccount) {
    var accountRegistered = false;

    if (gmailAccount != null && typeof gmailAccount != "undefined") {
        if (localStorage.GmailAccounts != null && typeof localStorage.GmailAccounts != "undefined") {
            var gmailAccounts = JSON.parse(localStorage.GmailAccounts);
            if (gmailAccounts.d != null) {
                for (var i = 0; i < gmailAccounts.d.length; ++i) {
                    if (gmailAccounts.d[i].GoogleAccountEmail == gmailAccount) {
                        accountRegistered = true;
                        
                        //Set the local storage active flag to that of the current accounts active value
                        if (gmailAccounts.d[i].IsActive == '1') {
                            localStorage.GmailAccountActive = "true";
                        } else if (gmailAccounts.d[i].IsActive == '0') {
                            localStorage.GmailAccountActive = "false";
                        }
                    }
                }
            }
        }
    }

    return accountRegistered;
}

//Object associated with IsWhoOrWhatFieldsEnabledOnTask()
function WhoOrWhatEnabledObject(_whoEnabled, _whatEnabled) {
    this.whoEnabled = _whoEnabled;
    this.whatEnabled = _whatEnabled;
}

//Searchs LayoutManager to see if the Who and What Fields are enabled on the task object
function IsWhoOrWhatFieldsEnabledOnTask() {

    var rtnObject = new WhoOrWhatEnabledObject(true, true);

    if (globalGmailProperties != null) {
        if (globalGmailProperties.isTaskWhoEnabled != null) {
            if (!globalGmailProperties.isTaskWhoEnabled) {
                rtnObject.whoEnabled = false;
            }
        }

        if (globalGmailProperties.isTaskWhatEnabled != null) {
            if (!globalGmailProperties.isTaskWhatEnabled) {
                rtnObject.whatEnabled = false;
            }
        }
    }

    return rtnObject;
}

//Fires the web service request to get back preview events to show on last page of calendar sync setup
function GetPreviewEventsForCalendarSync(tabid, gmailAccount) {
    //console.time("GetPreviewEventsForCalendarSync");
    var varData = '{"sessionCode": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","gmailAccount": "' + gmailAccount + '"}';

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb        
        url: localStorage.MainConnector + "/Connector.svc/GetCalendarSyncPreviewEvents", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call
            //console.timeEnd("GetPreviewEventsForCalendarSync");
            var objJson = JSON.stringify(msg);

            chrome.tabs.executeScript(tabid, { code: "HandlePreviewEventsForCalSync(" + objJson + ", " + dateCheckForPreviewEventRequest + ")" });

        },
        error: function (xhr, status, error) {

        }
    });
}

//Fires the web service request to get back preview contact to show on last page of contact sync setup
function GetPreviewContactsForContactSync(tabid, contactSyncRuleId) {
    //console.timeEnd("GetPreviewContactsForContactSync");
    var varData = '{"sessionCode": "' + localStorage.SessionCode + '","userKey": "' + localStorage.WebUserId + '","crmId": "' + localStorage.CrmId + '","contactSyncRuleId": ' + contactSyncRuleId + '}';

    $.ajax({
        type: "POST", //GET or POST or PUT or DELETE verb        
        url: localStorage.MainConnector + "/Connector.svc/GetContactSyncPreview", // Location of the service
        data: varData, //Data sent to server
        contentType: "application/json; charset=utf-8", // content type sent to server
        dataType: "json", //Expected data format from server
        processdata: true, //True or False
        timeout: 30000,
        success: function (msg) {//On Successfull service call
            //console.timeEnd("GetPreviewContactsForContactSync");
            var objJson = JSON.stringify(msg);

            chrome.tabs.executeScript(tabid, { code: "CreateContactSyncPreviewSettings(" + objJson + ", " + dateCheckForPreviewContactRequest + "," + contactSyncRuleId + ")" });

        },
        error: function (xhr, status, error) {

        }
    });
}

// Fetches the task types valiable from local storage and updates the global variable if it's not been set already
function GetTaskTypes(lgm)
{
    if (!globalTaskTypes)
    {
        globalTaskTypes = GetLookupMultiDetailsForDropDown("Task-Types", lgm);
    }

    return globalTaskTypes;
}

// Updates the global variable 'globalParsedLayoutManager' to save parsing this each and every time something needs it
function UpdateGlobalParsedLayoutManagerVariable() {

    if (IsValidJSON(localStorage.LayoutManager)) {
        var layoutManager = JSON.parse(localStorage.LayoutManager);
        if (layoutManager != null && layoutManager.d != null) {
            globalGmailProperties = layoutManager.d.GmailProperties;
        }
    }
}

// Gets data from LocalStorage and updates the global variable 'globalComplexAddSettings' 
// which contains the settings for custom fields and objects searchable within complex add
function UpdateComplexAddSettingsVariable() {

    if (localStorage.LayoutManager != null) {
        if (!globalGmailProperties) {
            UpdateGlobalParsedLayoutManagerVariable();
        }

        if (globalGmailProperties != null) {
            if (globalGmailProperties.ComplexAddSettings) {
                globalComplexAddSettings.customFieldInfo = globalGmailProperties.ComplexAddSettings;
            } else {
                //Clear the variable if user has no settings
                globalComplexAddSettings.customFieldInfo = null;
            }

            if (globalGmailProperties.SearchableObjects) {
                if (globalComplexAddSettings.searchableObjects) {
                    globalComplexAddSettings.searchableObjects = UpdateSearchableObjectsForComplexAdd(globalGmailProperties.SearchableObjects);
                } else {
                    globalComplexAddSettings.searchableObjects = GetSearchableObjectsForComplexAdd(globalGmailProperties.SearchableObjects);
                }
            } else {
                //Clear the variable if user has no settings
                globalComplexAddSettings.searchableObjects = null;
            }
        }
    }
}

/**
* Updates the global variable for the objects to be used for the suggestion searches
* Takes in the selections made by the user adds them to the global variable and in to local storage
*/
function UpdateSuggestionOrSearchObjectsVariable(objectsSelectedInClient, optionsObject, optionsObjectName) {

    if (optionsObject) {
        if (optionsObject.searchableObjects) {
            for (var i = 0; i < optionsObject.searchableObjects.length; i++) {
                var object = optionsObject.searchableObjects[i];
                if (object.ApiName) {
                    if (objectsSelectedInClient.indexOf(object.ApiName) > -1) {
                        optionsObject.searchableObjects[i].ChosenAsObjectToSearch = true;
                    } else {
                        optionsObject.searchableObjects[i].ChosenAsObjectToSearch = false;
                    }
                }
            }
            localStorage[optionsObjectName] = JSON.stringify(optionsObject.searchableObjects);
        }
    }
}

//Helps update the searchable objects to keep the list up to date but not wiping the users last saved preferences
function UpdateSearchableObjectsForComplexAdd(objectsVar) {
    var newArray = [];

    for (var i = 0; i < objectsVar.length; i++) {
        if (objectsVar[i].FriendlyName && objectsVar[i].Name) {
            var foundObject = false;

            //Find the object within the saved list so we can keep the users state
            for (var x = 0; x < globalComplexAddSettings.searchableObjects.length; x++) {
                var savedObj = globalComplexAddSettings.searchableObjects[x];
                if (savedObj.ApiName == objectsVar[i].Name) {
                    foundObject = true;
                    newArray.push(savedObj);
                    break;
                }
            }

            //If the object was not found in the original list then create a new object for this one
            if (!foundObject) {
                var newObj = new ComplexAddSearchObject(objectsVar[i].FriendlyName, objectsVar[i].Name, true);
                newArray.push(newObj);
            }
        }
    }

    return newArray;
}

//An object creator for the choice of objects to search within Complex Add
function ComplexAddSearchObject(name, apiName, chosenAsObjectToSearch) {
    this.Name = name;
    this.ApiName = apiName;
    this.ChosenAsObjectToSearch = chosenAsObjectToSearch;
}

//Returns an array of objects that can be searched along with
function GetSearchableObjectsForComplexAdd(objectsVar) {

    var arrayOfObjects = [];
    for (var i = 0; i < objectsVar.length; i++) {
        if (objectsVar[i].FriendlyName && objectsVar[i].Name) {
            var newObj = new ComplexAddSearchObject(objectsVar[i].FriendlyName, objectsVar[i].Name, true);
            arrayOfObjects.push(newObj);
        }
    }
    return arrayOfObjects;
}

/**
//Builds a comma separated string with a list of objects that we should be using for suggested or related searches
*/

function GetUserChosenSearchableObjectsForSuggestedOrRelatedSearches(optionsObject) {
    var rtnString = "";
    if (optionsObject.searchableObjects) {
        for (var i = 0; i < optionsObject.searchableObjects.length; i++) {
            var object = optionsObject.searchableObjects[i];
            if (object.ChosenAsObjectToSearch) {

                if (rtnString.length > 0)
                {
                    rtnString += ",";
                }

                rtnString += object.ApiName;
            }
        }
    }

    return rtnString;
}



//Reloads the Gmail Tabs on Oninstall and Update. 
chrome.runtime.onInstalled.addListener(function () {
    LogEventShort("Extension Installed/Updated: " + GetVersion());
    LogEventShort("Chrome Version: " + GetChromeVersionFromUserAgent());
    engine.ReloadTabs();

    //Set the uninstall event to direct user to uninstall page
    if (localStorage.WebUserId) {
        SetUninstallEvent(localStorage.WebUserId);
    }

    LogEventShort("Get Form Layout-2");
    //Lets get a new layout on extension update
    GetFormLayout();
    GLOBAL_EBSTA_CAMPAIGNMANAGERVALUES.GetAllValues();
});

//Tidier version of firing the 'ContentGet' script
function Fire_Content_Get_Script(tab) {
    if (tab) {
        if (tab.id || tab.id === 0) {
            if (!isNaN(tab.id)) {
                chrome.tabs.executeScript(tab.id, { file: 'js/ContentGet.js' });
            }
        }
    }
}

//Updates the email tracking poll depending on user selection
function Update_EmailTracking_Poll(turnNotificationsOff) {
    var localStorageTrackingItem = Get_LocalStorage_Item_And_Return_JSON("EmailTrackingSettings");
    if (localStorageTrackingItem != null) {
        if (turnNotificationsOff) {
            localStorageTrackingItem["TurnOffNotifications"] = true;
            Stop_Tracking_Poll();
        } else {
            localStorageTrackingItem["TurnOffNotifications"] = false;
            RefreshTrackingPoll();
        }
    }
    Stringify_Value_To_JSON_and_Set_LocalStorage_Value("EmailTrackingSettings", localStorageTrackingItem);
}

//Stringifys a value and stores it into localstorage with given key
function Stringify_Value_To_JSON_and_Set_LocalStorage_Value(localStorageKey, value) {
    localStorage.setItem(localStorageKey, JSON.stringify(value));
}

//Clean up the tracked uri for display 
function CleanTrackedUriForDisplay(url) {

    var cleanUrl = url + "";
    if (cleanUrl.length > 40) {
        cleanUrl = cleanUrl.substring(0, 40) + "..."
    }

    return cleanUrl;
}

// Takes a 2 different drafts ids. Finds if the old ID exists in our array, if so 
// will remove it and fire a request up to the server to update our records. 
// It will then store the new Daft id for future reference
function UpdateSendLaterDraftArray(oldId, newId) {
    if (sendLaterDraftStorageArray != null) {
        var index = sendLaterDraftStorageArray.indexOf(oldId);
        if (index >= 0) {
            //Lets send up new request to update the oldid to the new id
            SendDraftUpdateIdRequest(oldId, newId);
            sendLaterDraftStorageArray.splice(index, 1);

            //Now update the array with the newId
            sendLaterDraftStorageArray.push(newId);
        }
    }
}

//Sends a request up to the server to update a sendlater email
function SendDraftUpdateIdRequest(oldId, newId) {
    var fieldList = '[';
    fieldList += GenerateEntityField("olddraftid", oldId, "", "false", "");
    fieldList += "," + GenerateEntityField("newdraftid", newId, "", "false", "");
    fieldList += ']';

    AddEntityNew("sendlaterupdatedraftid", JSON.stringify(fieldList), "", "");
}

/**
 * Creates a Calendar Sync Request object to send up to the server
 * @param {} requestname 
 * @param {} emailaddress 
 * @returns {} 
 */
function CreateCalendarSyncRequest(requestname, emailaddress) {
    return JSON.stringify({
        calendarSyncRequest: {
            UserId: localStorage.WebUserId,
            SessionCode: localStorage.SessionCode,
            CrmId: localStorage.CrmId,
            RequestName: requestname,
            EmailAddress: emailaddress
        }
    });
}

/**
 * Fires a request to the generic endpoint for all Calendar Sync Request
 * @param {Object} request - The details for the request
 * @param {string} request.RequestName - The name of the request
 * @param {string} request.EmailAddress - Email address for user
 * @param {Function} sendResponse - The method to call when we received a response
 */
function SendCalendarSyncRequest(request, sendResponse) {
    var callBack = function (msg) {

        var response = { success: false };
        if (msg && msg.d) {

            response.success = msg.d.Success;
            response.errorMessage = msg.d.ErrorMessage;
            response.values = msg;
        }

        sendResponse(response);
    }

    MakeAjaxCallToSecure("POST", globalRequestEndPoints().GetCalendarSyncRequestEndpoint(),
        CreateCalendarSyncRequest(request.RequestName, request.EmailAddress),
                  "application/json; charset=utf-8", "json", callBack, callBack);
}

/**
 * Returns a LayoutManager if it exists within LocalStorage
 * @returns {} 
 */
function GetLayoutManagerFromLocalStorage() {
    if (IsValidJSON(localStorage.LayoutManager)) {
        return JSON.parse(localStorage.LayoutManager);
    }
    return null;
}

/**
 * Looks through the layout manager and retrieves the company settings
 * @returns {object} - The company setting properties
 */
function GetCompanySettings() {
    var parsedLayoutManager = GetLayoutManagerFromLocalStorage();
    if (parsedLayoutManager && parsedLayoutManager.d && parsedLayoutManager.d.CompanySettings) {
        return parsedLayoutManager.d.CompanySettings;
    } 
    return null;
}

/**
 * Takes a URL and checks the settings to see if we have a match for that website
 * @param {string} url - The URL to check to see if its blacklisted
 * @returns {boolean} - True if the URL isn't blacklisted 
 */
function ShouldWeShowBar(url) {
    if (url) {
        if (localStorage.SwitchOffBarWebsites) {
            var parsedIntoArray = JSON.parse(localStorage.SwitchOffBarWebsites);
            if (Array.isArray(parsedIntoArray)) {
                if (parsedIntoArray.indexOf("ALL_WEBSITES_WILL_BE_BLOCKED") !== -1) return false;
                return parsedIntoArray.findIndex(x => url.toLocaleUpperCase().indexOf(x.toLocaleUpperCase()) > -1) === -1;
            }
        }
    }

    return true;
}