﻿/// <reference path="complexadd.js" />
/// <reference path="Shared/domcreation.js" />
/// <reference path="gmailtools.js" />

var GmailGo = false;
var GmailGoLicence = false;
var GmailWizard = false;
var LinkedinWizard = false;
var GmailWizard = false;
var PremWizard = false;
var ExpiredWizard = false; 
var LicenceFound = "Free";
var LicenceType = "";
var LoggedIn = false;
var WhatVersion = 10;
var trialLicenceFound = false;
var gmailViewLogged = false;
var gmailBar = false;
var abRules;
var calendarSyncLoaded = false;
var complexAddFeature = false;
var webUserId = "";
var emailTemplateFeature = true;
var showEmailTemplateBox = false;
var lastTemplateSelected = "";
var lastTrackingIdInjected = "";
var instanceUrl = "";
var lastTrackingChoice = "false";
var whoWhatEnabledObject = null;
var gmailAuthFlag = 0;
var renderTrackingAnalysis = null;
var globalCampaignLimit = 250; //max campaign limit, default 250.

/** Represents certain settings for the company */
var globalCompanySettings = {};

//enable the use of mutation observers.
var enableMutationObservers = false;

//display stats for reply data.
var renderReplyData = false;

var growthTrialEndDate;
var renderTrialCountdown = false;
var daysRemainingOnTrial = 0;

var buttonMode = 0; //BUTTONSTYLE: sets which style of gmail buttons we should render.

var currentlyInGrowthTrial = false;
var useDefaultBarSearch = true;

//Onboarding Global Variables
var gmailMailboxAuthChecked = false; // Helps tell us if we've checked for the Auth already
var isAccountAuthorised = false; // Is this users account authorised
var isCalendarSyncLive = false; //Does this user currently have a live calendar sync 

//Global variables which we'll need to use when user has proceeded from preview settings to confirming settings
var global_syncPrivateEvents;
var global_syncRecurringEvents;
var global_syncColourEvents;
var global_calendarArray;
var global_dateCheckForPreviewEventRequestArray = new Array(); //This helps us track the last time a user had clicked preview settings so we match the right request when its returned
var global_dateCheckForPreviewContactRequestArray = new Array(); //This helps us track the last time a user had clicked preview settings in Contact Sync so we match the right request when its returned
var global_usingOldAuthToken = false; //Helps us track if this user is currently using the old auth token to set up their calendar sync
var global_doesUserHavePrivateEventField = true; // Keeps track to see if a user has access to IsPrivate field in SF

var listOfRelatedToHistory = null; // Global variable to keep the list of Related To objects that have been used recently (gets stored in localstorage.RelatedToSelections and assigns to this variable on every complex add click)

var port = chrome.runtime.connect();

//Standard z-index for the bar in gmail, this helps us appear over a compose window and the Google title & search bar
var EBSTA_GMAIL_BAR_ZINDEX = 7;

// object for encoding html characters
var HTMLEncode = (function() {
    var encodeHTMLmap = {
        "&": "&amp;",
        "'": "'",
        '"': "&quot;",
        "<": "&lt;",
        ">": "&gt;"
    };

    return {
        encodeHTMLmapper: function(match) {
            return encodeHTMLmap[match];
        }
    }
})();

// Allows us to switch on or off logging. 
function LogThis(msg) {
   // console.log(msg);
}

/**
 * Handles the data return from the proxy regarding an drafts being saved
 * @param {} responseData 
 * @returns {} 
 */
function HandleUpdatedDraftResponse(responseData) {

    //Takes a string and finds the draftid within this string so we can reference this as the old draft id which has now been updated
    // e.g. =&=&=&to=&cc=&bcc=&subjectbox=&composeid=1476889432826&from=robebsta%40gmail.com&subject=&draft=157dd78731699d77&bwd=&rm=&cans
    function GetDraftIdFromBody(bodyContent) {
        var regex = /&draft=(.*?)&/g;
        var regexMatch = regex.exec(bodyContent);
        if (regexMatch != null && regexMatch.length > 1) {
            return regexMatch[1];
        }
        return null;
    }

    if (responseData.ResponseText != null && responseData.ParsedParams != null && responseData.BodyContent != null) {
        // Within the response back from a save draft request there is a piece of text
        // containining "Your Message Has been saved" and directly after this you should see the new draft id
        var draftMessageRegex = /message has been saved.\",\[\"(.*?)\",/g;

        var oldDraftId = GetDraftIdFromBody(responseData.BodyContent);
        if (oldDraftId != null) {
            var newDraftIdMatch = draftMessageRegex.exec(responseData.ResponseText);

            if (newDraftIdMatch != null && newDraftIdMatch.length > 1) {
                var newDraftId = newDraftIdMatch[1];

                //Send message to background to check if this ID needs to be updated on our records
                chrome.extension.sendMessage({
                    MessageType: "updatedraftid",
                    OldDraftId: oldDraftId,
                    NewDraftId: newDraftId
                });
            }
        }
    }
}

/**
 * Checks to see if we've stored an object matching the tracking id of the id's we've found within an email.
 * If so send a request to track this up to our servers.
 * @param {string} messageId - The id of the email in the old format i.e. "166c78e70fe78f07"
 * @param {string} urlIdMatches - The tracking link urls found within the email
 * @param {string} base10MessageId - The messageId but in base10 format (may not be used anymore)
 * @param {string} immutableMessageId - The id of the message which never seems to change i.e. msg-a:r6941005600118229215
 */
function UpdateLogTrackingArrayBeforeTracking(messageId, urlIdMatches, base10MessageId, immutableMessageId) {

    chrome.extension.sendMessage({
            MessageType: "GetEmailTrackingObject",
            TrackingIdMatches: urlIdMatches,
            Mailbox: GmailTools.GetUserName()
        },
        function(response) {
            if (response && response.FoundObject) {

                //Update the object with the ID data relating to this message
                response.FoundObject.MailId = messageId;
                response.FoundObject.Base10MessageId = base10MessageId;
                response.FoundObject.ImmutableMessageId = immutableMessageId;

                chrome.extension.sendMessage(response.FoundObject);
            }
        });
}

/**
 * Checks if we have a matching 'Save on Send' object for this email we've picked up, if so action the process of adding emails to Salesforce
 * @param {string} messageId - The id of the email in the old format i.e. "166c78e70fe78f07"
 * @param {string} composeId - The id of the compose window that was used to send this email
 * @param {string} immutableMessageId - The id of the message which never seems to change i.e. msg-a:r6941005600118229215
 * @param {string} base10MessageId - The messageId but in base10 format (may not be used anymore)
 */
function UpdateComplexAddArrayBeforeSaveOnSend(messageId, composeId, immutableMessageId, base10MessageId)
{
    //Due to external Gmail compose windows, we send all the objects to the background so we need to go fetch the saved object from there
    chrome.extension.sendMessage({
        MessageType: "GetSaveOnSendObject",
        ComposeId: composeId,
        ImmutableMessageId: immutableMessageId,
        Mailbox: GmailTools.GetUserName()
    }, function (response) {
        if (response != null && response.SaveOnSendObject != null) {
            SaveOnSendWithMessageId(response.SaveOnSendObject, messageId, immutableMessageId, base10MessageId);
        }
    });
}

/**
 * After picking up from gmailProxy.js that an email as been sent, lets check to see if the user wanted this added to SF also
 * @param {Object} complexAddClosureObject - The Stored 'Save on Send' object which contains details on how we should add this email
 * @param {string} gmailMessageId - The id of the email in the old format i.e. "166c78e70fe78f07"
 * @param {string} immutableMessageId - The id of the message which never seems to change i.e. msg-a:r6941005600118229215
 * @param {string} base10MessageId - The messageId but in base10 format (may not be used anymore)
 */
function SaveOnSendWithMessageId(complexAddClosureObject, gmailMessageId, immutableMessageId, base10MessageId) {

    for (var complexAddRowCount = 0;
        complexAddRowCount < complexAddClosureObject.ComplexAddWhoAndWhatIds.length;
        complexAddRowCount++) {
        var obj = complexAddClosureObject.ComplexAddWhoAndWhatIds[complexAddRowCount];

        var whatId = null,
            whoId = null,
            nameToUseWhenAdding = null;

        if (obj != null) {
            whatId = obj.WhatId;
            whoId = obj.WhoId;
            nameToUseWhenAdding = obj.WhoName != null ? obj.WhoName : obj.WhatRecordName;
        }

        if (whoId == undefined || whoId == "undefined" || whoId == null) {
            whoId = -1;
        }

        if (whatId == undefined || whatId == "undefined" || whoId == null) {
            whatId = -1;
        }

        chrome.extension.sendMessage({
            MessageType: "AddGmailTask",
            Email: "NEW",
            EmailText: obj != null ? obj.EmailText : "",
            WhoId: "" + whoId,
            WhatId: "" + whatId,
            GmailUserId: "" + GoogleTools.GetEmailAddress(),
            GmailMessageId: gmailMessageId,
            GmailBase10MessageId: base10MessageId,
            GmailImmutableId: immutableMessageId,
            GmailOriginalSubject: complexAddClosureObject.OriginalSubject,
            GmailAttachmentIds: obj != null ? obj.GmailAttachmentIds : "",
            SaveOnSend: true,
            Subject: complexAddClosureObject.Subject,
            TaskType: obj != null ? obj.selectedTaskTypeValue : "",
            Name: nameToUseWhenAdding != null && nameToUseWhenAdding ? nameToUseWhenAdding : "Salesforce",
            WhatRecordType: obj != null ? obj.whatRecordType : "",
            WhatRecordName: obj != null ? obj.whatRecordName : "",
            UseDifferentTabId: obj != null ? obj.UseDifferentTabId : "",
            CustomFields: complexAddClosureObject.ComplexAddAdditionalFields,
            AddType: "ComposeView-SaveOnSend",
            RequestedFromUrl: complexAddClosureObject.RequestedFromUrl //What the url was when then save on send request was made
        });

        if (obj != null && obj.currentTracking) {
            //Log the event
            chrome.extension.sendMessage({
                    MessageType: "LogEmailSent",
                    TrackingLogId: "" + obj.lastTrackingIdInjected,
                    TemplateId: "" + obj.currentTemplateId
                    }, function (response) {

                });
        }
    }

    //Log event
    if (complexAddClosureObject.LogEvent) {
        LogEvent(complexAddClosureObject.LogEvent);
    }
}

if ((document.URL.toLowerCase().indexOf("mail.google.com/mail") != -1 || document.URL.toLowerCase().indexOf(".google.com/calendar") != -1
    || document.URL.toLowerCase().indexOf("contacts.google.com") != -1 || document.URL.toLowerCase().indexOf(".google.com/contacts") !== -1)
    && document.URL.toLowerCase().indexOf(".google.com/calendar/printevent") == -1) {

    var link = document.createElement("link");
    link.href = chrome.extension.getURL("css/ebstagoogle.css");
    link.type = "text/css";
    link.rel = "stylesheet";
    link.id = "ebstagoogleId";
    (document.head || document.documentElement).appendChild(link);
}

var eventListenerSetup = false;
var observer = null;
var observerPresentation = null;
var eventMsgSet = false;
var jsonPerson = {};

//Helps determine
function ReceiveMessage(event) {

    //LogThis('message recieved in main page...');
    //LogThis(event);
    // we must trust the source of the message coming back!
    if (event.origin !== "https://console.ebsta.com") {
        return;
    } else {
        //LogThis('we trust the inbound url...');
        //LogThis("ReceiveMessage()");
        //LogThis(event);

        var parsedData = JSON.parse(event.data);
        var authResult = 0;
        var userCode = "";

        //get the user code returned
        if (parsedData.userCode != null && parsedData.userCode != 'null') {
            userCode = parsedData.userCode;
        }

        //response webuserid from authpage does not match current webuser - fail auth.
        if (userCode != webUserId) {
            HandleGoogleAuthFailed();
            return;
        }

        if (parsedData != null) {
            if (parsedData.authResult != null) {
                authResult = parsedData.authResult;
            }
        }

        if (authResult == 1) {
            HandleGoogleAuthCompleted();
        } else {
            HandleGoogleAuthFailed();
        }
       
    }
}

var barEventSetup = false; 

/** 
 *Adds a listener to the current window ready to receive messages from any other windows trying to communicate with it
 *Force it only to ever do this once. 
*/ 
function SetupAuthMessageListener() {
    if (barEventSetup === false) {
        //listen for inbound browser messages.   
        if (typeof window.addEventListener != 'undefined') {
            window.addEventListener("message", ReceiveMessage, false);
        }
        else if (typeof window.attachEvent != 'undefined') { //for ie.
            window.attachEvent('onmessage', ReceiveMessage);
        }
        barEventSetup = true;
    }
}


function PassJson(emailAddress, fullName) {

    //console.time('PassJson');
    var firstName = '';
    var lastName = '';

    if (typeof fullName != "undefined" && fullName != "undefined" && fullName != null) {
        var nameStr = fullName.split(' ');

        firstName = nameStr[0];

        if (nameStr.length > 1) {
            lastName = nameStr[nameStr.length - 1];
        }
    }   

    jsonPerson["Url"] = document.URL;
    jsonPerson["Firstname"] = "" + firstName;
    jsonPerson["Surname"] = "" + lastName;
    jsonPerson["Email"] = emailAddress;
    jsonPerson["Mobile"] = "";
    jsonPerson["HomePhone"] = "";
    jsonPerson["WorkPhone"] = "";
    jsonPerson["Phone"] = "";
    jsonPerson["Fax"] = "";
    jsonPerson["Company"] = "";
    jsonPerson["Title"] = "";
    jsonPerson["Address"] = "";
    jsonPerson["Postcode"] = "";
    jsonPerson["City"] = "";
    jsonPerson["State"] = "";
    jsonPerson["Country"] = "";
    jsonPerson["Twitter"] = "";
    jsonPerson["WebAddress"] = "";
    jsonPerson["PublicProfileURL"] = "";
    jsonPerson["Skills"] = "";
    jsonPerson["Source"] = "Gmail";
    jsonPerson["Description"] = "";
    jsonPerson["Street"] = "";

    //Custom fields
    jsonPerson["DealDescription"] = "";
    jsonPerson["DealPrice"] = "";
    jsonPerson["DealDiscount"] = "";
    jsonPerson["DealSaving"] = "";
    //Currency is £ for now - link to locale
    jsonPerson["DealRRP"] = "";
    jsonPerson["DealQuantity"] = "";
    //DealLink is the same as URL
    jsonPerson["DealLink"] = document.URL;
    jsonPerson["CompetitorName"] = "";
    //LightIron
    jsonPerson["Gender"] = "";
    jsonPerson["Aka"] = "";
    jsonPerson["LastRelease"] = "";
    jsonPerson["NextRelease"] = "";
    jsonPerson["ProjectType"] = "";
    jsonPerson["ProjectName"] = "";
    jsonPerson["ProjectStatus"] = "";
    jsonPerson["ProjectGenre"] = "";
    jsonPerson["ProjectBudget"] = "";
    jsonPerson["ProjectDevNotes"] = "";
    jsonPerson["ProjectDiStartDate"] = "";
    jsonPerson["ProjectLogLine"] = "";
    jsonPerson["ProjectReleaseDate"] = "";
    jsonPerson["ProjectStartDate"] = "";
    jsonPerson["ProjectWrapDate"] = "";
    jsonPerson["ProjectDirector"] = "";
    jsonPerson["ProjectProductionCompany"] = "";
    jsonPerson["ProjectWriters"] = "";
    jsonPerson["ProjectCast"] = "";
    jsonPerson["ProjectDistributor"] = "";
    jsonPerson["ProjectProducers"] = "";
    jsonPerson["ProjectEditors"] = "";
    jsonPerson["ProjectProductionManagers"] = "";

    jsonPerson["LinkedInPro"] = '1';

    //channel navigator...
    jsonPerson["LegalName"] = "";
    jsonPerson["MailingAddress"] = "";
    //jsonPerson["Description"] = "";
    jsonPerson["LinkedInUrl"] = "";
    jsonPerson["AccountSource"] = "";
    jsonPerson["AccountSourceDetails"] = "";
    jsonPerson["StateIncorporated"] = "";
    jsonPerson["StartYear"] = "";
    jsonPerson["FiscalEndYear"] = "";
    jsonPerson["Ownership"] = "";
    jsonPerson["ServiceOverview"] = "";
    jsonPerson["Employees"] = "";
    jsonPerson["BillableEmployees"] = "";
    jsonPerson["AnnualRevenue"] = "";
    jsonPerson["OtherServices"] = "";
    jsonPerson["RunRate"] = "";
    jsonPerson["PartnersAndCertifications"] = "";
    jsonPerson["OtherCustomerFocus"] = "";
    jsonPerson["ActiveCustomers"] = "";
    jsonPerson["CustomerBaseSize"] = "";
    jsonPerson["Department"] = "";
    jsonPerson["Type"] = "";

    jsonPerson["FullBillingAddress"] = "";
    jsonPerson["SST"] = "";
    jsonPerson["AwardsRankings"] = "";

    jsonPerson["VerticalMarkets"] = "";
    jsonPerson["VendorPartnerships"] = "";
    jsonPerson["RevenueRange"] = "";
    jsonPerson["CommunicationNotes"] = "";

    //console.timeEnd('PassJson');
    //console.time('PassJson1');
    if (typeof abRules !== "undefined" && abRules !== null) {
        if (abRules.ParseGmailContent + "" == "1") {
            if (document.URL != null && document.URL != "undefined") {
                if (document.URL.indexOf("mail.google.com") != -1) {

                    var emailBodyContent = GetGmailBodyContent();

                    if (emailBodyContent != null && typeof emailBodyContent != "undefined") {
                        if (emailBodyContent.length > 0) {
                            ParseDataFromEmailContent(jsonPerson, emailBodyContent);
                        }
                    }

                    //Set variables for Gmail Content
                    jsonPerson['gmailLastEmailInThreadContent'] = "";
                    jsonPerson['gmailLastEmailInThreadWhoId'] = "";
                    jsonPerson['gmailLastEmailInThreadWhoName'] = "";
                    jsonPerson['gmailLastEmailInThreadWhatId'] = "";
                    jsonPerson['gmailLastEmailInThreadWhatName'] = "";
                    jsonPerson['gmailLastEmailInThreadWhoType'] = "";

                    //Get Subject from Gmail
                    jsonPerson['gmailEmailSubject'] = BarJsGetValueByXpath('//*[@class="hP"]');

                    //Get email content for the last email
                    jsonPerson['gmailLastEmailInThreadContent'] = GmailTools.MessageView.GetLastEmailInThread();

                    //Get Contact/Lead details along with a related Account is applicable
                    var gmailLastEmailInThreadAssociatedRecord = document.evaluate('//span[contains(@class,"sfComplexAdd")][last()]', document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                    if (gmailLastEmailInThreadAssociatedRecord) {
                        //Get id of the Who
                        if (gmailLastEmailInThreadAssociatedRecord.getAttribute("sfemailaddress")) {
                            jsonPerson['gmailLastEmailInThreadWhoId'] = gmailLastEmailInThreadAssociatedRecord.getAttribute("sfemailaddress");
                        }

                        //Get Name of the Who
                        if (gmailLastEmailInThreadAssociatedRecord.getAttribute("sfname")) {
                            jsonPerson['gmailLastEmailInThreadWhoName'] = gmailLastEmailInThreadAssociatedRecord.getAttribute("sfname");
                        }

                        //Get entity type of the Who
                        if (gmailLastEmailInThreadAssociatedRecord.getAttribute("sftype")) {
                            jsonPerson['gmailLastEmailInThreadWhoType'] = gmailLastEmailInThreadAssociatedRecord.getAttribute("sftype");
                        }

                        //Get id type of the what
                        if (gmailLastEmailInThreadAssociatedRecord.getAttribute("whatid")) {
                            jsonPerson['gmailLastEmailInThreadWhatId'] = gmailLastEmailInThreadAssociatedRecord.getAttribute("whatid");
                        }

                        //Get name of the what
                        if (gmailLastEmailInThreadAssociatedRecord.getAttribute("whatname")) {
                            jsonPerson['gmailLastEmailInThreadWhatName'] = gmailLastEmailInThreadAssociatedRecord.getAttribute("whatname");
                        }
                    }
                }
            }
        }
    }
    //console.timeEnd('PassJson1');
}

function ParseDataFromEmailContent(jsonPerson, emailBodyContent) {
    
    //console.time('ParseDataFromEmailContent');

    var regExRuleList = GetRegexRuleList();

    for (var i = 0; i < regExRuleList.length; i++) {
        var ruleObject = regExRuleList[i];

        var fieldName = ruleObject.ParseField;
        var regExString = ruleObject.Regex;
        var regexFlags = "" + ruleObject.Flags;

        var path = new RegExp(regExString, regexFlags);

        var regExReturnValue = emailBodyContent.match(path);

        if (regExReturnValue != null) {

            var selectedValue = regExReturnValue[0];

            //trim the value for unwanted spaces.
            if (selectedValue != null) {
                selectedValue = selectedValue.trim();

                //Strip out line breaks here
                selectedValue = selectedValue.replace(/(\r\n|\n|\r)|<br \/>|<br\/>|<br>/gim, "");
            }

            var parseMessage = '';

            if (fieldName == "Mobile") {
                if (jsonPerson["Mobile"] == '') {
                    jsonPerson["Mobile"] = selectedValue;
                    parseMessage = 'ParseMobilePhone: ' + selectedValue;
                }
            }

            if (fieldName == "HomePhone") {
                if (jsonPerson["HomePhone"] == '') {
                    jsonPerson["HomePhone"] = selectedValue;
                    parseMessage = 'ParseHomePhone: ' + selectedValue;
                }
            }

            if (fieldName == "WorkPhone") {
                if (jsonPerson["WorkPhone"] == '') {
                    jsonPerson["WorkPhone"] = selectedValue;
                    parseMessage = 'ParseWorkPhone: ' + selectedValue;
                }
            }

            if (fieldName == "Phone") {
                if (jsonPerson["Phone"] == '') {
                    jsonPerson["Phone"] = selectedValue;
                    parseMessage = 'ParsePhone: ' + selectedValue;
                }
            }
        }
    }

    //console.timeEnd('ParseDataFromEmailContent');
}

function GetRegexRuleList() {

    //we can make this variable at a later stage.

    var regExRuleList = new Array();

    //NOTE: we are character escaping here!!!

    //phone
    regExRuleList.push({ RuleId: "1", ParseField: "Phone", Regex: "(?:(?:\\(?(?:0(?:0|11)\\)?[\\s-]?\\(?|\\+)44\\)?[\\s-]?(?:\\(?0\\)?[\\s-]?)?)|(?:\\(?0))(?:(?:\\d{5}\\)?[\\s-]?\\d{4,5})|(?:\\d{4}\\)?[\\s-]?(?:\\d{5}|\\d{3}[\\s-]?\\d{3}))|(?:\\d{3}\\)?[\\s-]?\\d{3}[\\s-]?\\d{3,4})|(?:\\d{2}\\)?[\\s-]?\\d{4}[\\s-]?\\d{4}))(?:[\\s-]?(?:x|ext\\.?|\\#)\\d{3,4})?$", Flags: "gim" });
    regExRuleList.push({ RuleId: "2", ParseField: "Phone", Regex: "\\D?(\\d{4})\\D?\\D? (\\d{3})\\D?(\\d{4})", Flags: "gim" }); //us
    regExRuleList.push({ RuleId: "3", ParseField: "Phone", Regex: "[2-9]\\d{2}-\\d{3}-\\d{4}", Flags: "gim" }); //us
    regExRuleList.push({ RuleId: "4", ParseField: "Phone", Regex: "\\s*\\(?(020[78]\\)? ?[1-9][0-9]{2} ?[0-9]{4})\\s*", Flags: "gim" }); //uk
    regExRuleList.push({ RuleId: "5", ParseField: "Phone", Regex: "(^\\D?(\\d{3})\\D?\\D?(\\d{3})\\D?(\\d{4})$)|(\\s\\D?(\\d{3})\\D?\\D?(\\d{3})\\D?(\\d{4})$)", Flags: "gim" }); //uk
    //regExRuleList.push({ RuleId: "6", ParseField: "Phone", Regex: "(^(\\(?\\+?[0-9]*\\)?)?[0-9_\\- \\(\\)]*$)|(\\s(\\(?\\+?[0-9]*\\)?)?[0-9_\\- \\(\\)]*$)", Flags: "gim" }); //uk


    //work phone
    regExRuleList.push({ RuleId: "7", ParseField: "WorkPhone", Regex: "(?:(?:\\(?(?:0(?:0|11)\\)?[\\s-]?\\(?|\\+)44\\)?[\\s-]?(?:\\(?0\\)?[\\s-]?)?)|(?:\\(?0))(?:(?:\\d{5}\\)?[\\s-]?\\d{4,5})|(?:\\d{4}\\)?[\\s-]?(?:\\d{5}|\\d{3}[\\s-]?\\d{3}))|(?:\\d{3}\\)?[\\s-]?\\d{3}[\\s-]?\\d{3,4})|(?:\\d{2}\\)?[\\s-]?\\d{4}[\\s-]?\\d{4}))(?:[\\s-]?(?:x|ext\\.?|\\#)\\d{3,4})?$", Flags: "gim" });
    regExRuleList.push({ RuleId: "8", ParseField: "WorkPhone", Regex: "\\D?(\\d{4})\\D?\\D? (\\d{3})\\D?(\\d{4})", Flags: "gim" }); //us
    regExRuleList.push({ RuleId: "9", ParseField: "WorkPhone", Regex: "[2-9]\\d{2}-\\d{3}-\\d{4}", Flags: "gim" }); //us
    regExRuleList.push({ RuleId: "10", ParseField: "WorkPhone", Regex: "\\s*\\(?(020[78]\\)? ?[1-9][0-9]{2} ?[0-9]{4})\\s*", Flags: "gim" }); //uk
    regExRuleList.push({ RuleId: "11", ParseField: "WorkPhone", Regex: "(^\\D?(\\d{3})\\D?\\D?(\\d{3})\\D?(\\d{4})$)|(\\s\\D?(\\d{3})\\D?\\D?(\\d{3})\\D?(\\d{4})$)", Flags: "gim" }); //uk
    //regExRuleList.push({ RuleId: "12", ParseField: "WorkPhone", Regex: "(^(\\(?\\+?[0-9]*\\)?)?[0-9_\\- \\(\\)]*$)|(\\s(\\(?\\+?[0-9]*\\)?)?[0-9_\\- \\(\\)]*$)", Flags: "gim" }); //uk

    //mobile
    regExRuleList.push({ RuleId: "13", ParseField: "Mobile", Regex: "07([\\d]{3})[(\\D\\s)]?[\\d]{3}[(\\D\\s)]?[\\d]{3}", Flags: "gim" }); //uk
    regExRuleList.push({ RuleId: "14", ParseField: "Mobile", Regex: "\\s[2-9]\\d{2}.\\d{3}.\\d{4}", Flags: "gim" }); //us
    regExRuleList.push({ RuleId: "15", ParseField: "Mobile", Regex: "\\s[2-9]\\d{2} \\d{3} \\d{4}", Flags: "gim" }); //us
    regExRuleList.push({ RuleId: "16", ParseField: "Mobile", Regex: "\\s[2-9]\\d{2}-\\d{3}-\\d{4}", Flags: "gim" }); //us

    //home phone

    return regExRuleList;
}

function EscapeCharacters(text) {
    return text.replace("\\", "\\\\");
}

function GetGmailBodyContent() {
    var emailContent = '';

    var quickAddButton = $('.gs');

    if (quickAddButton != null && typeof quickAddButton != "undefined") {
        if (quickAddButton.length > 0) {
            emailContent = quickAddButton[0].innerText;
        }
    }

    return emailContent;
}

function SetMsgInbox() {
    if (!eventMsgSet) {

        eventMsgSet = NewEmailEventSetup();

    }
}

function LoadComplete() {
    var list = $(".Cp");
  
    if (list.is(':visible')) {
        CheckInbox(true);
    }
}

var gmailMutationObserver, emailContainer, gmailMutationObserver2;
var runSearch, runSearchInboxMsg;
var tabIDForPanelView;

function DecisionObj(_runSearch, _runMessageBox, _runComposeSearch, _runReplyForwardSearch, _barDrop, _number, _EmailTrack, _DelayTimer, _CheckPlainText, _GoogleButtonsCreate, _UndoGmailSend) {
    this.RunSearchInbox = _runSearch;
    this.RunMessageBox = _runMessageBox;
    this.RunComposeSearch = _runComposeSearch;
    this.RunReplyForwardSearch = _runReplyForwardSearch;
    this.BarDrop = _barDrop;
    this.Number = _number;
    this.EmailTrack = _EmailTrack;
    this.DelayTimer = _DelayTimer;
    this.CheckPlainText = _CheckPlainText; //Helps check for changes to an email type to update the gmailtracking button
    this.GoogleButtonsCreate = _GoogleButtonsCreate;
    this.UndoGmailSend = _UndoGmailSend;
}

function AddButtonDelay() {
    if (LoggedIn && isAccountAuthorised) {
      
        //now set on an interval rather than a timeout as Ebsta buttons weren't displaying
        var intervalIdentifier = setInterval(function() {
            AddGoogleSendButtons(intervalIdentifier);
        }, 500);
        setTimeout(() => { clearInterval(intervalIdentifier); }, 10000);
    }
}

// decorator
function DecisionOnClick(event) {

   // console.time("DecisionOnClick");
    var decision = DecisionOnClickWorker(event);
   // console.timeEnd("DecisionOnClick");
    return decision;
}

function DecisionOnClickWorker(event) {

    var decision = new DecisionObj(false, false, false, false, false, 0, 0, false, false, false, false);

    //esnure that we have a classname, and that it is a string
    //for example svgs can have a classname that is a instance of SVGAnimatedString
    //and se would cause exception when running indexOf - we are expecting only a string
    if (event.target.className != null
        && typeof (event.target.className) === "string") {

        //hide send later div if user isn't clickin on it
        if (event.target.closest(".sfEbSendLaterSchedule") == null) {
            var hideSendLaterDialogues = document.getElementsByClassName("sfEbSendLaterSchedule");
            for (var i = 0; i < hideSendLaterDialogues.length; i++) {
                HideElement(hideSendLaterDialogues[i]);
            }
        }

        //hide campaign manager div if user isn't clicking on it
        if (event.target.closest(".sfEbSendCampaignMgrStarterPopup") == null) {
            var campaignManagerStarterPopups = document.getElementsByClassName("sfEbSendCampaignMgrStarterPopup");

            if (campaignManagerStarterPopups && campaignManagerStarterPopups.length > 0) {
                for (var i = 0; i < campaignManagerStarterPopups.length; i++) {
                    campaignManagerStarterPopups[i].style.display = "none";
                }
            }
        }

        //Catch opening draft email clicks
        if (event.target.closest(".yO") != null) {
            decision.GoogleButtonsCreate = true;
        }

        if (event.target.className.indexOf("T-I J-J5-Ji T-I-Js-IF ar7 ns T-I-ax7 L3 T-I-Kq") != -1) {
            decision.Number = 101;
            return decision;
        }
        
        if (event.target.className.indexOf("J-Ke n0") != -1) {

            //LogThis('fire 1');

            decision.RunMessageBox = true;
            decision.RunSearchInbox = true;
            //decision.BarDrop = true;
            decision.Number = 30;
            return decision;
        }

        if (event.target.className.indexOf("HF Hr") != -1) {

            //LogThis('fire 2');

            FireButtonBackground();
            decision.Number = 31;
            return decision;
        }

        if (event.target.className.indexOf("amn") != -1) {

            //LogThis('fire 3');

            decision.GoogleButtonsCreate = true;
            decision.Number = 1;
            return decision;
        }

        if (event.target.className.indexOf("ams amt") != -1) {
            decision.Number = 102;
            decision.GoogleButtonsCreate = true;
            return decision;
        }

        if (event.target.className.indexOf("abG af9 d5") != -1) {
            decision.Number = 104;
            decision.GoogleButtonsCreate = true;
            return decision;
        }

        if (event.target.className.indexOf("J-J5-Ji T-I-J3 Nz NS") != -1) {
            decision.Number = 105;
            decision.GoogleButtonsCreate = true;
            return decision;
        }


        if (event.target.className.indexOf("anj.T-I-J3.ank") != -1) {
            decision.Number = 107;
            decision.GoogleButtonsCreate = true;
            return decision;
        }

        if (event.target.className.indexOf("T-I J-J5-Ji T-I-Js-IF aaq T-I-ax7 L3") != -1
            || event.target.className.indexOf("hB T-I-J3") != -1
            || event.target.className.indexOf("cj") != -1
            || event.target.className.indexOf("ams amt") != -1
            || event.target.className.indexOf("T-I T-I-KE L3") != -1
            || event.target.className.indexOf("Yh akV") != -1) {

            AddButtonDelay();
            AttachKeyDownListenerOnGmailReply();
            decision.Number = 1;
            return decision;
        }


        if (event.target.className.indexOf("editable LW-avf") != -1 || (event.target.className.indexOf("gmail_quote") != -1)) {

           
            //LogThis('hit reply/forward...');
            AddButtonDelay();

            //Find if this is a reply/forward message or compose message
            if ($(event.toElement).closest(".adB").length) {
                if ($(event.toElement).closest(".adB").length > 0) {
                    decision.RunReplyForwardSearch = true;
                }
            }
                // Compose message
            else if ($(event.toElement).closest(".aaZ").length) {
                if ($(event.toElement).closest(".aaZ").length > 0) {
                    decision.RunComposeSearch = true;
                }
            }
            decision.Number = 2;
            decision.GoogleButtonsCreate = true;
            return decision;
        }

        //Refresh Button
        if ($(event.target).find('.asf').length > 0 || $(event.target).closest('.nu').length > 0) {

            //LogThis('fire 4');

            decision.Number = 3;
            decision.RunSearchInbox = true;
            decision.BarDrop = true;
            return decision;
        }

        if (event.target.className.indexOf("zF") != -1) {

            //LogThis('fire 5');

            //runSearch = true;
            decision.Number = 4;
            decision.RunMessageBox = true;
            decision.BarDrop = true;
            return decision;
        }

        //Clicks on emails

        var lcClassName = "";

        if (event.toElement != null) {
            if (event.toElement.className != null) {
                lcClassName = "" + event.toElement.className;

                lcClassName = lcClassName.toLowerCase();

                //Detect user selecting the panel preview button
                if (lcClassName.indexOf("aph t-i-j3 j-j5-ji") != -1
                        || lcClassName.indexOf("asa") != -1
                        || lcClassName.indexOf("t-i j-j5-ji apf") != -1) {
                    //runSearchInboxMsg = true;
                    emailContainer = null;
                    decision.Number = 5;
                    decision.RunSearchInbox = true;
                    decision.DelayTimer = 250;
                    return decision;
                }

                //Back and forward arrows
                if ($(event.toElement).closest('.h0').length > 0 && !($(event.toElement).hasClass('.T-I-JE') || $(event.toElement).closest('.T-I-JE').length > 0)) {
                    decision.Number = 6;
                    //inbox link = https://mail.google.com/mail/u/0/#inbox
                    //message link = https://mail.google.com/mail/u/0/#inbox/149525f4b4697eb8
                    if (document.URL.split("/").length <= 7) {
                        //Main Inbox - we're not calling the search for the main inbox as we're running a mutation observer for more consistency
                        decision.RunSearchInbox = true;
                    } else {

                        //LogThis('fire 6');

                        //Message
                        decision.RunMessageBox = true;
                        decision.BarDrop = true;
                    }
                    return decision;
                }



                //Back and forward arrows
                if (event.toElement.localName.indexOf("img") != -1 && (event.toElement.className.indexOf("adj") != -1 || event.toElement.className.indexOf("adk") != -1)) {
                    decision.Number = 7;
                    //decision.RunMessageBox = true;
                    return decision;
                }
            }
        }


        //Clicks on emails in a conversation
        if (event.target.className.indexOf("gE hI") != -1) {
            //runSearch = true;
            //barDrop = false;
            decision.Number = 8;
            decision.RunMessageBox = true;
            decision.BarDrop = false;
            return decision;
        }

        if (event.target.className.indexOf("iA g6") != -1) {
            decision.Number = 9;
            decision.RunMessageBox = true;
            decision.BarDrop = false;
            return decision;
        }

        if (event.target.className == "gD" || event.target.className == "ajn") {
            decision.Number = 10;
            decision.RunMessageBox = true;
            decision.BarDrop = false;
            return decision;
        }

        //if (event.target.className.indexOf("G3 G2") != -1) {
        //    runSearch = false;
        //}

        if (event.target.className.indexOf("gF gK") != -1) {
            //runSearch = true;
            //runSearchInboxMsg = false;
            //barDrop = false;
            decision.Number = 11;
            decision.RunMessageBox = true;
            decision.RunSearchInbox = false;
            decision.BarDrop = false;
            return decision;
        }

        //back to folder button divs.
        if (event.target.className.indexOf(" lS ") != -1 || $(event.target).closest(".lS").length) {

            ////LogThis('mark the search flag...');

            //runSearch = true;
            //runSearchInboxMsg = false;
            //barDrop = false;
            decision.Number = 12;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = false;
            decision.BarDrop = false;
            return decision;
        }

        //back to folder button divs.
        if (event.target.className.indexOf("T-I-ax7") != -1) {
            //runSearch = true;
            //runSearchInboxMsg = false;
            //barDrop = false;
            decision.Number = 13;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = false;
            decision.BarDrop = false;
            return decision;
        }

        //next/prev 50 email button.
        if (event.target.className.indexOf("amF") != -1) {
            //runSearch = true;
            //runSearchInboxMsg = false;
            //barDrop = false;
            decision.Number = 14;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = false;
            decision.BarDrop = false;
            return decision;
        }

        //next/prev 50 email button.
        if (event.target.className.indexOf("T-I-J3") != -1 && event.target.className.indexOf("hB mK") == -1) {
            decision.Number = 14;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = false;
            decision.BarDrop = false;
            decision.DelayTimer = 350;

            return decision;
        }

        //next/prev 50 email button.
        if (event.target.className.indexOf("amd") != -1 || $(event.target).closest(".amd").length) {
            //runSearch = true;
            //runSearchInboxMsg = false;
            //barDrop = false;
            decision.Number = 15;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = false;
            decision.BarDrop = false;
            return decision;
        }

        if (event.target.className.indexOf("aKw") != -1) {
            //runSearch = false;
            //runSearchInboxMsg = true;
            decision.Number = 16;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = false;
            return decision;
        }

        if (event.target.className.indexOf("aKv") != -1) {
            //runSearch = false;
            //runSearchInboxMsg = true;
            decision.Number = 17;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = false;
            return decision;
        }


        if (event.target.className.indexOf("aKy") != -1) {
            //runSearch = false;
            //runSearchInboxMsg = true;
            decision.Number = 18;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = false;
            return decision;
        }

        if (event.target.className.indexOf("aKz") != -1) {
            //runSearch = false;
            //runSearchInboxMsg = true;
            decision.Number = 19;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = false;
            return decision;
        }

        if (event.target.className.indexOf("yW") != -1) {
            decision.Number = 20;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = true;
            decision.BarDrop = true; // Helps catch email clicks when you click just right of the name
            return decision;
        }

        if (event.target.className.indexOf("amJ T-I-J3") != -1) {
            //runSearch = true;
            //runSearchInboxMsg = true;
            decision.Number = 21;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = true;
            return decision;
        }

        //Refresh Button
        if (event.target.className.indexOf("asf T-I J-J5-Ji") != -1 || event.target.className.indexOf("T-I J-J5-Ji nu T-I-ax7") != -1) {
            //runSearchInboxMsg = true;
            // decision.RunSearchInbox = true;

            //LogThis('fire 7');

            decision.Number = 22;
            decision.RunMessageBox = true;
            decision.BarDrop = true;
            return decision;
        }

        //'chosen-container' helps stop the bar firing when pressing the drop down arrow on a chosen combo box
        if (event.target.localName == 'b' && $(event.target).closest('.chosen-container').length == 0) {
            //LogThis('fire 8');

            //    runSearch = true;
            decision.Number = 23;
            decision.RunSearchInbox = true;
            decision.RunMessageBox = true;
            decision.BarDrop = true;
            return decision;
        }

        //Check for click on the menu options
        if (event.target.innerText.trim().toLowerCase() == 'mark all as read' || event.target.innerText.trim().toLowerCase() == 'mark as unread' || event.target.innerText.trim().toLowerCase() == 'mark as read' || event.target.innerText.trim().toLowerCase() == 'add star') {

            //LogThis('fire 9');

            //runSearchInboxMsg = true;
            decision.Number = 24;
            decision.RunMessageBox = true;
            return decision;
        }

        //Check for star button clicks
        if (event.target.title.trim().toLowerCase() == 'starred') {

            //LogThis('fire 10');

            decision.Number = 25;
            runSearchInboxMsg = true;
            decision.RunMessageBox = true;
            decision.BarDrop = true;
            return decision;
        }
        //Prevent inboxmessage calles whend back button is pressed
        //if (event.toElement.className.indexOf("asa") != -1) {
        //    runSearch = false;
        //}
        //if (event.toElement.className.indexOf("G-Ni") != -1) {
        //    runSearch = false;
        //}
        //if (event.toElement.className.indexOf("aos") != -1) {
        //    runSearch = false;
        //}
        //if (event.toElement.className.indexOf("T-I J-J5-Ji ash T-I-ax7 L3") != -1) {
        //    //LogThis('set inbox search flag to false...');

        //    runSearch = false;
        //}

        //Compose email button
        if (event.target.className.indexOf("T-I J-J5-Ji T-I-KE L3") != -1) {

            //LogThis('fire 11');

            SetSaveOnSend();
            AttachKeyDownListenerOnGmailCompose();
            //LogThis('compose button fired...');
            //LogThis("Resize Gmail 1");
            setTimeout(function () { SetupResizeWindowEventsForGmailCompose(); }, 500);
            decision.Number = 26;
            decision.RunMessageBox = false;
            decision.BarDrop = true;
            return decision;

        }

        // This is Really expensive. 
        // //LogThis("Looking here.");
        if (event.toElement != null) {
            //if(event.toElement.localName
            ////LogThis("HEre");

            if ((event.toElement.localName.toLowerCase().indexOf("span") != -1 && event.toElement.className.toLowerCase().indexOf("y2") != -1)
                || (event.toElement.id.indexOf(":") == 0 && event.toElement.className.toLowerCase().indexOf("xy a4w") != -1)
                || event.toElement.className.toLowerCase().indexOf("yx xy") != -1
                || (event.toElement.localName.toLowerCase().indexOf("td") != -1 && event.toElement.className.toLowerCase().indexOf("xy") != -1)) {

                //LogThis('fire 12');

                decision.Number = 27;
                decision.RunMessageBox = true;
                decision.BarDrop = true;
                return decision;
            }

            if (event.target != null) {
                if (event.toElement.localName.toLowerCase().indexOf("span") != -1 && (event.target.id.indexOf(":") == 0 || event.target.outerHTML.toLowerCase().indexOf("yp") != -1)) {
                    //Catches Ebsta icons
                    if (event.target.outerHTML.toLowerCase().indexOf("sftype") == -1 && event.toElement.className.indexOf("sfEbstaGmailEmailTracking") == -1 &&
                        event.toElement.className.indexOf("ebsta-glyphicon") == -1) {
                        //Catches CC and BCC clicks
                        if (event.toElement.className.indexOf("aB") == -1 && event.toElement.className.indexOf("aQY") == -1 && event.toElement.className.indexOf("ebsta-glyphicon") == -1) {

                            //LogThis('fire 13');

                            decision.Number = 28;
                            decision.RunMessageBox = true;
                            decision.BarDrop = true;
                            decision.GoogleButtonsCreate = true;
                            return decision;
                        }
                    }
                }
            }

            var previewPane = document.getElementsByClassName('nH if');

            /*
            * Preview Pane
            */
            if (previewPane != null) {
                //Look for Preview Pane email clicks
                var lowClassName = event.toElement.className.toLowerCase();
                if (lowClassName.indexOf("y6") != -1
                    || lowClassName.indexOf("apb") != -1
                    || lowClassName.indexOf("yw") != -1
                    || lowClassName.indexOf("xy") != -1
                    || lowClassName.indexOf("y2 apz") != -1) {

                    //LogThis('fire 14');

                    decision.Number = 29;
                    decision.RunSearchInbox = true;
                    decision.RunMessageBox = true;
                    decision.BarDrop = true;
                    return decision;

                    //                    runSearch = true;
                    //                  runSearchInboxMsg = true;
                }

                //Detect Star clicks when in previewpane mode and prevent calls from happening
                //if (event.toElement.className.toLowerCase().indexOf("apu apc") != -1 || event.toElement.className.toLowerCase().indexOf("axw t-kt") != -1 || event.toElement.className.toLowerCase().indexOf("t-kt t-kt-jp") != -1) {
                //    // decision.RunSearchInbox = true;
                //    // decision.RunMessageBox = true;
                //    return decision;
                //    // return true;
                //}

                ////Detect Forward and Reply clicks in quick reply box
                //if (event.toElement.className.toLowerCase().indexOf("ams amt") != -1) {
                //    return decision;
                //    //return true;
                //}

            }

            //Pick up any type of email click when in the main screen
            //Each email row in Gmail is associated with a zA class
            //The googleButtonStyle condition makes sure we don't fire any more events on the Ebsta button clicks when in split pane mode
            if ($(event.toElement).closest(".zA").length > 0 && event.toElement.className.indexOf("googleButtonStyle") == -1) {

                //LogThis('fire 15');

                decision.Number = 32;
                decision.RunMessageBox = true;
                decision.BarDrop = true;
                return decision;
            }

            //Forward and Back Arrows
            if ($(event.toElement).closest(".h0").length > 0) {

                //LogThis('fire 16');

                decision.Number = 33;
                decision.RunMessageBox = true;
                decision.BarDrop = true;
                return decision;
            }

            //More Tab Catching
            if ($(event.toElement).closest(".aRz").length > 0) {
                decision.Number = 34;
                decision.RunSearchInbox = true;
                return decision;
            }

            //More Inbox/Starred/Important etc catching
            if ($(event.toElement).closest(".TK").length > 0) {
                decision.Number = 35;
                decision.RunSearchInbox = true;

                return decision;
            }
            //Is the user using the compose box
            if ($(event.toElement).closest(".aaZ").length) {
                if ($(event.toElement).closest(".aaZ").length > 0) {
                    if (event.toElement.name != null && event.toElement.name == "subjectbox") {
                        decision.RunComposeSearch = true;
                        decision.Number = 36;
                        return decision;
                    }
                }
            }

            //Email Tracking on the Send email button but not on the "Send with Cadence" button or our own Send Button
            if (event.toElement.className.indexOf("aoO") != -1 && !HasClass(event.toElement, "sfEbstaSendWithCampaignButton")
                && !HasClass(event.toElement, "sfEbSendBtn")) {
                decision.Number = 37;
                decision.RunMessageBox = true; // When hitting reply to a message and you have conversation mode setup the new email will append to your conversation but buttons won't appear
                decision.EmailTrack = true;
                //Set all current send buttons to false
                $("[current-send*='true'").attr("current-send", "false");
                //Set this compose screens send button as the current send button
                $(event.target).attr("current-send", "true");
                return decision;
            }

            //Looks for click events on the minimise and maximise of compose screens
            if (event.toElement) {
                if ($(event.toElement).closest('.aXJ').length) {
                    //LogThis('minimize or maximize compose window hit...');
                    //LogThis("Resize Gmail 2");
                    setTimeout(function () { SetupResizeWindowEventsForGmailCompose(); }, 500);
                    decision.GoogleButtonsCreate = true;
                    decision.Number = 38;
                    return decision;
                }
            }

            //Archive Buttons
            if (event.toElement.className.indexOf("lR") != -1 || ($(event.toElement).closest('.lR').length)) {
                decision.Number = 39;
                decision.RunMessageBox = true;
                return decision;
            }

            //Report as Spam Buttons
            if ((event.toElement.className.indexOf("nN") != -1 || $(event.toElement).closest('.nN').length)
                && event.toElement.closest(".sfEbSendCampaignMgrMainModelSearchItem") == null) //Stop firing on campaign manager search results
            {
                //LogThis('fire 17');

                decision.Number = 40;
                decision.RunMessageBox = true;
                decision.BarDrop = true;
                return decision;
            }

            //Delete Buttons
            if (event.toElement.className.indexOf("nX") != -1 || ($(event.toElement).closest('.nX').length)) {

                //LogThis('fire 18');

                decision.Number = 41;
                decision.RunMessageBox = true;
                decision.BarDrop = true;
                return decision;
            }

            //Gmail compose screen menu options and 'Edit-Subject' within reply/forward screen
            if (event.target.className.indexOf("J-N-Jz") != -1) {

                if ($(event.target).closest('.aki').length == 0) {
                    //LogThis('fire 19');

                    AddButtonDelay();
                    AttachKeyDownListenerOnGmailReply();
                    decision.CheckPlainText = true;
                    decision.GoogleButtonsCreate = true;
                    decision.Number = 42;
                } else {
                    decision.RunSearchInbox = true;
                    decision.RunMessageBox = true;
                    decision.Number = 48;
                }

                return decision;
            }
            
            if (event.target.className.indexOf("J-N-JT") != -1) {

                //LogThis('fire 19');
                AddButtonDelay();
                AttachKeyDownListenerOnGmailReply();
                decision.CheckPlainText = true;
                decision.GoogleButtonsCreate = true;
                decision.Number = 420;
                return decision;
            }

            //Helps catch the plain text mode selection
            if (event.target.className.indexOf('J-N-Jo') != -1 && $(event.target).closest('.J-N-Jz').text() == "Plain text mode" ||
                event.target.className.indexOf('J-N J-Ks') != -1 && $(event.target).text() == "Plain text mode") {

                //LogThis('fire 20');
                
                decision.CheckPlainText = true;
                decision.Number = 43;
                return decision;
            }

            //reply/forward icon click from dropdown.
            if (event.target.className.indexOf("f4 J-N-JX") != -1) {
                decision.GoogleButtonsCreate = true;
                decision.Number = 44;
                return decision;
            }

            //Images on the right hand side (recent photos)
            if (event.target.className.indexOf("am1") != -1) {
                decision.RunMessageBox = true;
                decision.BarDrop = true;
                decision.Number = 45;
                return decision;
            }

            // When hitting reply to a message and you have conversation mode 
            // setup the new email will append to your conversation but buttons won't appear
            if (event.target.className.indexOf("sfBtSaveCompose") != -1) {
                decision.RunMessageBox = true;
                decision.Number = 46;
                return decision;
            }

            // ReplyAll Button within an email when it's set as the default reply method!
            if (event.target.className.indexOf("T-I J-J5-Ji T-I-Js-IF aaq T-I-ax7 L3") != -1 || event.target.className.indexOf("hB mK T-I-J3") != -1) {
                decision.GoogleButtonsCreate = true;
                decision.Number = 47;
                return decision;
            }

            // "View Message" link which appears after sending an email
            if ($(event.target).closest('.b8.UC').length > 0) {

                decision.RunMessageBox = true;
                decision.Number = 49;

                //Undo on email send
                if (event.target.id.indexOf("link_undo") != -1) {
                    decision.UndoGmailSend = true;
                }

                return decision;
            }
        }

        return decision;
    }

    //if there is not class name for us to run by, or 
    //for other reasons we dont find a decision above 
    //returns the decision with all false
    return decision;
}

//Detects the choice of the plain text option chosen by the user and updates the buttons to reflect this
function DetectPlainTextModeChoice(event) {

    //console.log("detect plain text emails");
    //console.log(abRules);

    //only do this if we have email tracking enabled.
    if (abRules.EnableGmailEmailTracking + "" == "1") {
        ////LogThis('detect plain text text...');
        if ($(event.target).text() == "Plain text mode" || $(event.target).closest('.J-N').text() == "Plain text mode") {

            var plainTextSelected = "true";

            //Delay the call as the attributes don't change instantly
            setTimeout(function () {
                if ($(event.target).attr("aria-checked")) {
                    plainTextSelected = $(event.target).attr("aria-checked");
                }
                else if ($(event.target).closest('.J-Ks').attr("aria-checked")) {
                    plainTextSelected = $(event.target).closest('.J-Ks').attr("aria-checked");
                }

                //Convert the found attribute string to bool
                if (plainTextSelected == "true") {
                    plainTextSelected = true;
                }
                else if (plainTextSelected == "false") {
                    plainTextSelected = false;
                }

                UpdateGmailTrackingButtonForPlainTextEmails($(event.target), plainTextSelected);
            }, 350);
        }
    }
}

// save caret position
function SaveCaretPosition() {
    meetingSchedulerLinkObject.SetCaret();
}

function NewEmailEventSetup() {

    //console.time('NewEmailEventSetup');

    //LogThis('NewEmailEventSetup()');

    var returnVal = false;
    if (document.URL.toLowerCase().indexOf("mail.google.com/mail") != -1) {
      //  LogThis("NewEmailEventSetup()");
        AddGoogleSendButtons();

        //View=btop helps catch the smaller externale gmail windows
        if (document.URL.indexOf("view=btop") !== -1) {
            setTimeout(function() { CheckInboxMsg(true); }, 1750);
            AttachKeyDownListenerOnGmailReply();
        } else {
            RetryWithCallback(60, 0.5,
                function() {
                    return GmailTools.MessageView.IsThisAMessageView();
                }, function() {
                    CheckInboxMsg(true);
                    AttachKeyDownListenerOnGmailReply();
                });
        }

        //Attach click event listener on compose message if it's displayed

        GmailTools.SetClickDecisionHandler();

        //LogThis("Resize Gmail 4");
        //Lets call it initially to pick up any small compose screens on load
        SetupResizeWindowEventsForGmailCompose();

        // Lets setup Presentation. 
        return true;

    }

    //console.timeEnd('NewEmailEventSetup');

}

//Setups a window resize listener on Google Calendar so we can make sure the buttons are displaying correctly
function SetupWindowResizeListenerForGoogleCalendar()
{
    $(window).resize(function ()
    {
        //console.time('ResizeWindowGCalendar');
        ChangeOnboardingButtonOnResize();
        //console.timeEnd('ResizeWindowGCalendar');
    });
}

//Changes the outer div on the onboarding button so that it doesn't overlap any gmail elements when the screen is smaller
function ChangeOnboardingButtonOnResize() {
    //console.log("ChangeOnboardingButtonOnResize resizing screen");
    if (window.innerWidth < 1200)
    {
        if ($('.eb-authwizard-restart').length == 1) {

            var standardContainerWidth = 0;

            //need to apply a starting offset due to other icons Google have
            if (document.URL.indexOf("mail.google.com") > -1) {
                standardContainerWidth = 150;
            }

            //Dyanmically build the width going by the width of whatever buttons exists in the header
            var getAllButtonContainers = document.getElementsByClassName("ebsta-google-header-container");
            for (var i = 0; i < getAllButtonContainers.length; i++) {
                standardContainerWidth += getAllButtonContainers[i].offsetWidth;
            }
            standardContainerWidth = standardContainerWidth+ "px";

            $('.eb-authwizard-restart').parent().parent().css("min-width", standardContainerWidth);
        }
    }
}

//Resizes the buttons in the compose screen when screen is too small to display these
function SetupResizeWindowEventsForGmailCompose() {
    //console.time('SetupResizeGmailCompose');

    //Only modifiy the div if we've set the user to be enabled on Tracking or Templates
    //if (abRules.ShowGmailTemplates + "" == "1" || abRules.EnableGmailEmailTracking + "" == "1") {
       
    //    $('.aSt .aVN:visible,.AD, .iN').each(function () {

            
    //        //Find lengths of email tracking and template buttons to help tell us when we need to shrink the send + add to sf button
    //        //var totalEbstaDivLength = 0;
    //        //$(this).find('.sfGetEmailTemplates,.sfEbstaGmailEmailTracking').closest('.ebstaTD').each(function () {
    //        //    totalEbstaDivLength += $(this).outerWidth();
    //        //});

    //        //Get row width for the icons element in compose screen
    //        var iconRowWidth = "" + $(this).find('.n1tfz').outerWidth();

    //        // iconRowWidth = 1618 
    //        // 528
    //        //LogThis('icon row width: ' + iconRowWidth);

    //        if (parseInt(iconRowWidth) < 700) {
    //            //LogThis('Here');
    //            $(this).find('.sfComplexAddSend').each(function () {
    //                //LogThis('SF Complex');
    //                $(this).removeClass('sfComplexAddSend');
    //                $(this).addClass('sfComplexAddSmallCompose');
    //                $(this).addClass('googleButtonStyleLargeComposeScreen');
    //                $(this).val("");
    //            });
    //        } else {
    //            $(this).find('.sfComplexAddSmallCompose, .googleButtonStyleLargeComposeScreen').each(function () {
    //                //LogThis('SF Complex2');
    //                $(this).addClass('sfComplexAddSend');
    //                $(this).removeClass('sfComplexAddSmallCompose');
    //                $(this).removeClass('googleButtonStyleLargeComposeScreen');
    //                $(this).val("Send + Add Salesforce");
    //            });
    //        }
    //    });
    //}

    MoveGmailComposeScreenIcons();

    //console.timeEnd('SetupResizeGmailCompose');
}

//Moves the icons at the bottom of Gmail Compose if an overflow is evident
function MoveGmailComposeScreenIcons() {
    //console.time('MoveGmailCompose');
    $('.aSt .aVN:visible,.AD, .iN').each(function () {

  
        var composeDivWidth = $(this).outerWidth();

        var $rowElement = $(this).find('.n1tfz');

        if (typeof $rowElement == "undefined" || $rowElement == null || $rowElement.length < 1) {
            //console.log("get second el");
            $rowElement = $(this).find('.btC');
        }

        var iconRowWidth = $rowElement.outerWidth();

        //console.log("icon width:");
        //console.log(iconRowWidth);
        //console.log($rowElement);

        if (iconRowWidth > composeDivWidth)
        {
            $(this).find('.aDh').css('padding-bottom', '5px');
        }
        else
        {
            $(this).find('.aDh').css('padding-bottom', '0px');
            //$(this).find('.I5').find('.editable').focus(); //Put focus on email body so Gmail updates the compose screen to remove the overflow bar
        }
    });
    //console.time('MoveGmailComposeEnd');

}


//Updates local storage to retain the last user choice of the tracking option
function LogTrackingChoiceInLocalStorage(isBoxSelected) {
    chrome.extension.sendMessage({ MessageType: "LogTrackingChoice", TrackingChoice: isBoxSelected });

    //Lets also update local variable
    lastTrackingChoice = isBoxSelected;
}

//Logs a user out of Ebsta
function Log_User_Out() {
    chrome.extension.sendMessage({
        MessageType: "LogUserOut"
    });
}

// create tracking link for links found in tracked email
function CreateTrackedLink(trackingId, linkTrackedId, oldHref) {
    //encodeURIComponent on oldHref: escape characters that would disrupt query string
    return "https://console.ebsta.com/linktracking/track.aspx?trackid=" + trackingId + "&linktrackingid=" + linkTrackedId + "&linkuri=" + encodeURIComponent(oldHref);
}

// html content regex for parsing email content
function GetHtmlRegexForParsingHtmlTagNames() {
    var tagRegex = /[0-9a-zA-Z][0-9a-zA-Z:]*/,
        attributeNameRegex = /[^\s"'>\/=\x00-\x1F\x7F]+/,
        attributeValRegex = /(?:"[^"]*?"|'[^']*?'|[^'"=<>`\s]+)/,
        nameValRegex = attributeNameRegex.source + '(?:\\s*=\\s*' + attributeValRegex.source + ')?';

    return new RegExp([

        // All HTML tags
        '(?:',
        '</?', // start of tag name < or </
        '(?:',
        tagRegex.source,
        '(?:',
        '(?:\\s+|\\b)', // any number of whitespace chars before an attribute
        nameValRegex, // attr="value" (with optional ="value" part)
        ')*',
        '\\s*/?', // any trailing spaces and optional '/' before the closing '>'
        ')',
        '>',
        ')'
    ].join(""), 'gi');
}

// encodes html characters 
function HtmlEncode(text) {
    return text.replace(/[&"'<>]/g, HTMLEncode.encodeHTMLmapper);
}

// builds anchor tags and replaces tets with it
function ReplaceMatchedTextLinkWithAnchorTag(match) {

    if (match && match.length > 0) {

        //get rid of trailing .
        var trailingDotFound = false;
        if (match[match.length - 1] === '.') {
            trailingDotFound = true;
            match = match.slice(0, match.length - 1);
        }

        // we want to take the uri up to the first occurance of speech mark to replicate Google
        var uriSeparated = match.split(/([\"\,])(.+)?/);

        // append the trailing text to the end of the anchor tag
        var trailingText = "";
        if (uriSeparated && uriSeparated.length > 2) {
            match = uriSeparated[0];
            trailingText = trailingText + uriSeparated[1];
            var i = 2;
            for (i; i < uriSeparated.length; i++) {
                if (uriSeparated[i] && uriSeparated[i].length > 0) {
                    trailingText = trailingText + uriSeparated[i];
                }
            }
        }

        var hrefLink = match;
        if (match.toLowerCase().indexOf('http') === -1) {
            hrefLink = 'http://' + hrefLink;
        }

        var anchorTag = document.createElement('a');
        anchorTag.setAttribute('href', hrefLink);
        anchorTag.setAttribute('target', '_blank');
        anchorTag.appendChild(document.createTextNode(match));

        var txt = document.createElement("textarea");
        txt.innerHTML = anchorTag.outerHTML;

        var result = txt.value;

        txt = null;

        result = result + trailingText;

        if (trailingDotFound) {
            result = result + '.';
        }

        return result;
    }

    return match;
}

// replace all plain text emails with anchor tags
function ReplacePlainTextLinks(targetEmail) {
    var emailContent = targetEmail.html();

    //run regex on the html content of the email
    var htmlRegexExp = GetHtmlRegexForParsingHtmlTagNames();

    // regex for parsing text in between elements to text links
    var textLinkRegex = '\\b((?:www\\.|https?:\\/\\/)[\\S]+)';
    var linkRegExp = new RegExp(textLinkRegex, 'gi');
    
    var htmlResult;
    var index = 0;
    var offset = 0;
    var textToParse;

    var newEmailContent = "";
    var foundHtmlContent = false;

    var inAnchorTag = false;
    while ((htmlResult = htmlRegexExp.exec(emailContent)) !== null) {

        foundHtmlContent = true;
        var tagText = htmlResult[0];
        var isClosingAnchorTag = tagText === '</a>';
        offset = htmlResult.index;
        // text between the last element tag and this one
        textToParse = emailContent.substring(index, offset);
        // move the index to after the tag name for the next set
        index = offset + tagText.length;

        if (textToParse) {
            // if we are currently in an anchor tag, inAnchorTag will be true and isClosingTag will be true unless we have other elements in there like <wbr>
            // either way we will not parse

            if (!inAnchorTag && !isClosingAnchorTag) {
                // text in between element tags that isnt in an anchor tag already
                // replcae with anchor tag

                textToParse = textToParse.replace(linkRegExp, ReplaceMatchedTextLinkWithAnchorTag);
            }
        }

        newEmailContent = newEmailContent + (textToParse + tagText);

        // if we are in reply compose and we get to the previous correspondence section, stop parsing
        if (tagText === '<div class="gmail_quote">') {
            var restOfTheEmail = emailContent.substring(index, emailContent.length);
            newEmailContent = newEmailContent + restOfTheEmail;
            break;
        }

        //if we have just got to an opening anchor tag lets set inAnchorTag to true so that we ignore everything until we get to a closing anchor tag
        // where we set inAnchorTag to false and contine to parse
        if (tagText === '<a>' || tagText.indexOf('<a ') !== -1) {
            inAnchorTag = true;
        }
        if (isClosingAnchorTag) {
            inAnchorTag = false;
        }
    }
    // make sure we don't replace the email with an empty string if regex doesnt find anything
    if (foundHtmlContent) {

        targetEmail.html(newEmailContent);
    }
}

// decide whether link should be tracked going by href
function LinkToBeTracked(href) {

    if (!href) {
        return false;
    }

    if (href.length > 0) {
        var lowerCaseHref = href.toLowerCase();
        if (lowerCaseHref.indexOf('mailto:') !== -1 ||
            lowerCaseHref.indexOf('tel:') !== -1 ||
            lowerCaseHref.indexOf('sms:') !== -1 ||
            lowerCaseHref.indexOf('mms:') !== -1 ||
            lowerCaseHref.indexOf('skype:') !== -1 ||
            lowerCaseHref.indexOf('callto:') !== -1)
        {
            return false;
        }
    } else {
        return false;
    }

    return true;
}

// replaces all links in email with tracking links and return object of tracked links
function ReplaceLinksForLinkTracking(targetEmail, trackingId) {
    var trackedLinks = '[';
    var linkTrackedId = 0;

    if (abRules.EnablePlainTextlinkTracking === 1) {
        // lets replace all plain text links in email with anchor tags before tracking them
        ReplacePlainTextLinks(targetEmail);
    }

    if (trackingId && trackingId.length > 0) {

        // find all links that are not in the extra section 
        targetEmail.find('a:not(.gmail_quote *)').each(function () {

            var oldHref = $(this).attr('href');

            // make sure we have a href and it is not a mailto link
            if (LinkToBeTracked(oldHref)) {

                oldHref = GetPreviousTrackedLinkInfo(oldHref);

                // send array of tracked links down to web services
                if (trackedLinks.length > 1) {
                    trackedLinks = trackedLinks + ',';
                }
                trackedLinks = trackedLinks + '{ "LinkTrackingId" : "' + linkTrackedId + '", "LinkUri" : "' + oldHref + '" }';        

                var newHref = CreateTrackedLink(trackingId, linkTrackedId, oldHref);

                // replace link
                $(this).attr('href', newHref);
                
                linkTrackedId++;
            }
        });
    }

    trackedLinks = trackedLinks + ']';

    return trackedLinks;
}

/**
 * Checks if the url has already been modified with an Ebsta URI, 
 * if so then just returns the original URI so we can recreate another link with a new trackingId
 * @param {} href 
 * @returns {} 
 */
function GetPreviousTrackedLinkInfo(href) {
    if (href != null && href.toLowerCase().indexOf('https://console.ebsta.com/linktracking/track.aspx') > -1) {
        var oldHref = getParameterByNameFromUriString("linkuri", href);
        return oldHref;
    }
    return href;
}

/**
 * Sets up an email to be tracked by inserting an image into the body
 * @param {DOMElement} tickBoxDiv - The tracking icon clicked
 * @param {string} emailSubject - The Subject of the email
 * @param {boolean} shouldWeLog - Denotes whether we should be logging this email being tracked
 * @return {boolean} - It will return false if it finds the user isn't logged in
 */
function InsertImgForGmailEmailTracking(tickBoxDiv, emailSubject, shouldWeLog = true) {

    //If we've lost the state of the webuserid than don't insert an image and kick the user off
    if (webUserId == null || webUserId === "undefined") {
        Log_User_Out();
        return false;
    }

    var closestTrackingBtn = $(tickBoxDiv).closest('.aoP').find('.sfEbstaGmailEmailTracking');

    //If the tracking box has been selected then do it
    if (closestTrackingBtn.hasClass('sfEbstaGmailEmailTrackingSelected')) {

        //Find compose box and insert image
        $(tickBoxDiv).closest('.aoP').find('.editable').each(function () {

            //Remove any previous email tracking gifs
            var divElements = $(this)[0].getElementsByTagName("div");
            for (var i = 0; i < divElements.length; i++) {
                var divElem = divElements[i];
                if (divElem != null && divElem.className != null) {
                    if (divElem.className.indexOf("ebstaET") > -1
                        && $(divElem).closest(".gmail_quote").length === 0) //Only remove links that weren't apart of previous emails within the conversation thread
                    {
                        divElem.remove();
                    }
                }
            }

            //Create unique url for the tracking id's
            var uniqueIdForUrl = webUserId + "-" + GetDateInMillisecs();

            var div = document.createElement("div");
            div.className = "ebstaET";

            var img = document.createElement('img');
            //img.src = "https://console.ebsta.com/EmailTracking/EmailTrack.aspx?trackid=" + uniqueIdForUrl;
            img.src = "https://console.ebsta.com/" + uniqueIdForUrl + "/ebsta.gif";

            img.width = "1";
            img.height = "1";
            $(img).attr('style', 'visibility: hidden;');
            $(div).append(img);

            // When forwarding emails it includes a block div with the previous email contents so we need to find this div and 
            // insert the tracking before it just in case the forward contains any more ebsta tracking info as we'll detect an 
            // old tracking id rather than the new one
            var gmailQuoteDiv = $(this).find('.gmail_quote');
            if (gmailQuoteDiv.length > 0) {
                $(gmailQuoteDiv).each(function () {
                    //Make sure we only append the image once
                    $(this).before(div);
                    return false;
                });
            } else {
                $(this).append(div);
            }

            var composeOrReply = IsTheEmailMessageAComposeOrReply(tickBoxDiv);

            lastTrackingIdInjected = uniqueIdForUrl;

            //Lets track the email in our DB
            var emailTrackingLogObj = TrackGmailEmailTracking(uniqueIdForUrl, composeOrReply, tickBoxDiv, emailSubject);

            // track links?
            if (abRules.EnableLinkTracking === 1 && uniqueIdForUrl !== "" && emailTrackingLogObj != null) {

                var trackedLinks = ReplaceLinksForLinkTracking($(this), uniqueIdForUrl);

                if (trackedLinks && trackedLinks.length > 2) {

                    emailTrackingLogObj.linktrack = trackedLinks;
                }
            }

            // Was this a send with campaign button click? as we don't want to keep this in the 
            // array for later as we now handle this logging on the server
            var wasTheSendWithCampaignButtonClicked = false;
            if (tickBoxDiv != null) {
                wasTheSendWithCampaignButtonClicked = tickBoxDiv.closest(".sfEbstaCampaignMgrSend") != null;
            }

            if (!wasTheSendWithCampaignButtonClicked && emailTrackingLogObj != null) {

                // When opening emails and compose screens using CTRL-Click or Shift-Click they open in a different window
                // but are still related to the main instance of Gmail. When you send an email through these screens our proxy
                // which catches the request and all important messageid we need to get the message information is picked up in main Gmail window.
                // So we're now sending the tracking email information to background.js and we'll pick it up once gmailproxy.js finds the request we need and sends us back the info,
                // once we find that ID we'll fetch the object we just sent to background.js and follow the same procedure we use to add a complex add object

                if (shouldWeLog) {
                    chrome.extension.sendMessage({
                        MessageType: "AddEmailTrackingObject",
                        Record: emailTrackingLogObj,
                        DateInMilliseconds: new Date().getTime()
                    });
                }
            }
        });

        LogTrackingChoiceInLocalStorage(true);
    }
    else if (closestTrackingBtn.hasClass("sfEbstaGmailEmailTrackingNotSelected")) //only if button found and has non selected class
    {
        LogTrackingChoiceInLocalStorage(false);
    }
}

//Returns the date in milliseconds
function GetDateInMillisecs() {
    return new Date().getTime();
}

//Track information of an email that has been selected to track by user in our DB
function TrackGmailEmailTracking(uniqueIdForUrl, composeOrReply, tickBoxDiv, emailSubject) {

    var emailAddresses = GetAllEmailsInGmailComposeOrReplyWindow(composeOrReply, tickBoxDiv);

    if (emailAddresses != null && emailAddresses.length === 0) return null;

    //Get TemplateId if there is one
    var templatedID = "";
    if ($(tickBoxDiv).closest('.I5').find('.ebstaTemplateObject').attr('ebstatemplateid')) {
        templatedID = $('.ebstaTemplateObject').attr('ebstatemplateid');
    }

    var rtrObject = {
        MessageType: "EmailTrack",
        TrackingLogId: uniqueIdForUrl.toUpperCase(),
        WebUserId: webUserId.toUpperCase(),
        EmailSubject: emailSubject,
        Recipients: emailAddresses,
        RecipientCount: 0,
        TemplateId: templatedID,
        MailId: "",
        Mailbox: GetGmailUsersEmailAddress()
    };

    return rtrObject;
}

/**
 * Takes an element and finds the compose window it sits in to find the subject
 * @param {} composeOrReply 
 * @param {} elementWithinComposeWindow 
 * @returns {} 
 */
function GetSubjectLineInGmailComposeOrReplyWindow(elementWithinComposeWindow)
{
    var subject = "";

    var composeWindow = GetComposeWindowFromElement(elementWithinComposeWindow);
    if (composeWindow != null) {
        var subjectContainer = composeWindow.getElementsByClassName("aoT")[0];
        if (subjectContainer != null) {
            subject = subjectContainer.value;
        }
    }

    return subject;
}

function log(message) {

    //LogThis(message);
}

//Finds all email address that an email is being sent to
function GetAllEmailsInGmailComposeOrReplyWindow(composeOrReply, tickBoxDiv)
{
    var msgBodyToSearchFor = $("[email]");

    var emailAddressToSearch = new Array();

    if (composeOrReply == "compose")
    {
        $(tickBoxDiv).closest(".aaZ").find('.eV').find(msgBodyToSearchFor).each(function ()
        {
            emailAddressToSearch.push($(this).attr("email"));
        });
    }
    else if (composeOrReply == "reply")
    {
        $(tickBoxDiv).closest(".HM").find('.eV').find(msgBodyToSearchFor).each(function ()
        {
            emailAddressToSearch.push($(this).attr("email"));
        });
    } else if (composeOrReply == "sfcomposegmail") {
        //We are in salesforce compose in gmail
        $(tickBoxDiv).closest(".I5").find('.eV').find(msgBodyToSearchFor).each(function () {
            emailAddressToSearch.push($(this).attr("email"));
        });
    }
    // I5 looks to be a generic class that belongs to each of the compose 
    // windows we've encountered so lets use this for potential compose windows we haven't yet come across
    else
    {
        $(tickBoxDiv).closest(".I5").find('.eV').find(msgBodyToSearchFor).each(function () {
            emailAddressToSearch.push($(this).attr("email"));
        });
    }

    //Lets make a comma delimited string
    var emailaddress = "";

    for (var i = 0; i < emailAddressToSearch.length ; i++)
    {
        if (i == 0)
        {
            emailaddress += emailAddressToSearch[i];
        }
        //Don't add duplicate emails
        else if (emailaddress.indexOf(emailAddressToSearch[i]) == -1)
        {
            emailaddress += "," + emailAddressToSearch[i];
        }
    }

    //alert(emailaddress);

    return emailaddress;
}

//Helps identify which window the email was being sent from REPLY/COMPOSE
function IsTheEmailMessageAComposeOrReply(event) {

    //Salesforce -> compose in gmail
    if (window.location.href.indexOf("view=cm") != -1) {
        return "sfcomposegmail";
    }

    //Find if this is a reply/forward message or compose message
    if ($(event).closest(".adB").length) {
        if ($(event).closest(".adB").length > 0) {
            return "reply";
        }
    }
    // Compose message
    else if ($(event).closest(".aaZ").length) {
        if ($(event).closest(".aaZ").length > 0) {
            return "compose";
        }
    }

    return "";
}

//
function GetTrackHeaders(page) {
    chrome.extension.sendMessage({
        MessageType: "GetEmailTrackingHeader", Page: page
    }, function (response) {

    });
}



//Attaches a key down event listener on compose box
function AttachKeyDownListenerOnGmailCompose() {


    setTimeout(function () {
        //console.time('AttachKeyDown');

        $('.aaZ').each(function () {

            ////LogThis('set keydown event...');

            $(this).undelegate("keydown");
            $(this).on("keydown", function (event) {

                ////LogThis('keydown fired...');

                if (event.keyCode == 9) {
                    RunCheckComposeAndReplySendee(true, false);
                }
            });
        });
        //console.timeEnd('AttachKeyDown');

    }, 250);

}

//Attachs listener to reply box witih Gmail
function AttachKeyDownListenerOnGmailReply() {

    //console.time('AttachKeyDownListner');
    ////LogThis('AttachKeyDownListenerOnGmailReply()');

    setTimeout(function () {
        $('.adB').each(function () {
            //console.time('AttachKeyDownListner1');
            $(this).undelegate("keydown");
            $(this).on("keydown", function (event) {
                if (event.keyCode == 9) {
                    RunCheckComposeAndReplySendee(false, true);
                }
            });
        });
        //console.timeEnd('AttachKeyDownListner1');
    }, 250);
    //console.timeEnd('AttachKeyDownListner');
}


function StopLooking() {

    try {
        observerPresentation.disconnect();
    } catch (error) {

    }
    try {
        observer.disconnect();
    }
    catch (erro) {
    }


}

function FireButtonBackground() {

    //LogThis('FireButtonBackground');

    chrome.extension.sendMessage({
            MessageType: "GoogleSendButton"
        }, function(response) {

        });
}

function CheckGmailAuthStorage() {
    chrome.extension.sendMessage({
        MessageType: "CheckGmailAuthStorage"
    }, function (response) {
        gmailAuthFlag = response.GmailAuth;
    });
}

/**
 * Gets values from background to setup variables needed in order to setup Ebsta within a page
 * @returns {} 
 */
function Setup() {
    chrome.extension.sendMessage({
        MessageType: "BARSETUP"
    }, function(response) {
        GmailGo = response.GmailGo;
        GmailGoLicence = response.GmailGoLicence;
        LinkedinWizard = response.LinkedinWizard;
        GmailWizard = response.GmailWizard;
        PremWizard = response.PremWizard;
        LicenceFound = response.LicenceFound;
        ExpiredWizard = response.ExpiredWizard;
        LoggedIn = response.LoggedIn;
        WhatVersion = response.WhatVersion;
        trialLicenceFound = response.TrailLicenceFound;
        gmailViewLogged = response.GmailViewLogged;
        gmailBar = response.GmailBar;
        abRules = response.AbRules != null ? JSON.parse(response.AbRules) : {};
        webUserId = response.WebUserID;
        instanceUrl = response.InstanceUrl;
        lastTrackingChoice = response.TrackingChoice;
        whoWhatEnabledObject = response.WhoWhatEnabledObject; //Object to inform us if they have either What/Who from Task disabled and to reflect in complex add

        currentlyInGrowthTrial = response.EbstaGrowthTrial;
        daysRemainingOnTrial = response.GrowthTrialDaysRemaining;
        useDefaultBarSearch = response.useDefaultBarSearch;
        renderTrackingAnalysis = response.trackingAnalysis;
        enableMutationObservers = response.enableMutationObservers;
        renderReplyData = response.RenderReplyData;

        globalCompanySettings = response.CompanySettings;

        if (typeof response.dailyCampaignLimit !== "undefined" && response.dailyCampaignLimit !== null && response.dailyCampaignLimit !== "undefined") {
            globalCampaignLimit = response.dailyCampaignLimit;
        }

        CheckTrialStatus();

        if (document.URL.toLowerCase().indexOf("mail.google.com/mail") == -1 && document.URL.toLowerCase().indexOf(".google.com/calendar") == -1
            && document.URL.toLowerCase().indexOf("contacts.google.com") == -1 && document.URL.toLowerCase().indexOf(".google.com/contacts") == -1) {
            dropBarUsingRulesEngine();
        } else {
            if ((document.URL.toLowerCase().indexOf("contacts.google.com") !== -1 || document.URL.toLowerCase().indexOf(".google.com/contacts") !== -1)
                && (abRules != null && abRules.EnableContactSync + "" === "0")) {
                return false;
            }
            GoogleTools.Initialize();
        }
                
        if (document.URL.toLowerCase().indexOf(".force.com") > -1 || document.URL.toLowerCase().indexOf(".salesforce.com") > -1) {
            if (globalCompanySettings
                && globalCompanySettings.DisableCampaigns !== true
                && globalCompanySettings.CampaignsInSalesforce === true) {
                if (global_SalesforceHelper) {
                    global_SalesforceHelper.Initialize(webUserId);
                }
            }
        }
    });
}

function CheckTrialStatus() {
    //console.log('CheckTrialStatus()');

    //console.log('ebstaGrowthLicence: ' + ebstaGrowthLicence);
    //console.log(ebstaGrowthLicence);

    //console.log('currentlyInTrial: ' + currentlyInTrial);
    //console.log(currentlyInTrial);

    //if we are and ebsta growth user.
    if (currentlyInGrowthTrial + "" === "true") {
        //console.log('WE ARE IN GROWTH LICENCE');

        renderTrialCountdown = true;

        //calculate the remaining days on the trial from the expiry date.
        //GetRemainingDays();

    }else {
        //no trial, don't show any trial countdown messaging.

        renderTrialCountdown = false;
    }
}

function GetRemainingDays() {
    ////check we have an expiry date before converting anything.
    //if(growthTrialEndDate != null && typeof growthTrialEndDate != "undefined") {
    //    //var newExpiryDate = new Date(growthTrialEndDate).getFullYear();
    //    var newExpiryDate = new Date(growthTrialEndDate);

    //    var oneDay = 24*60*60*1000; // hours*minutes*seconds*milliseconds
    //    var todaysDate = new Date();

    //    console.log('growthTrialEndDate: ' + growthTrialEndDate);
    //    console.log('newExpiryDate: ' + newExpiryDate);
    //    console.log('todaysDate: ' + todaysDate);
    //    console.log('oneDay: ' + oneDay);

    //    daysRemainingOnTrial = Math.round((todaysDate - newExpiryDate) / (oneDay));

    //    renderTrialCountdown = true;

    //    console.log('daysRemainingOnTrial: ' + daysRemainingOnTrial);
    //}
}

var currentMailBoxActive = false;

// Fires the message request for Step 1 of Calendar Sync setup page where we check the user has access to certain fields on the event object
// Refresh Cache - 
function SendMessageToCheckFieldsForCalendarSync(refreshCache)
{
    $('.eb-continuebtn').hide();
    $('.ebstaStep:visible').find('.ebNewLoader').show();
    $('.ebstaStep:visible').find('.alert, .table').hide();
    //console.time("CalSyncSetupCheckSettings");
    chrome.extension.sendMessage({ MessageType: "CalSyncSetupCheckSettings", RefreshCahe: refreshCache });
}

//Fires the message to get the google calendar and calendar sync details to be displayed in the onboarding screen
function SendMessageToGetCalendarSyncSettings() {
    const continueButton = $('.eb-continuebtn');
    continueButton.hide();

    ResetCalendarSettingsChooseSettingsStep();

    chrome.extension.sendMessage({
        MessageType: "SendCalendarSyncRequest",
        RequestName: "CalSyncCheckAdminMode",
        EmailAddress: GetGmailUsersEmailAddress()
    }, function (response) {
        if (response && response.values && response.values.d && Array.isArray(response.values.d.Values)) {
            const values = response.values.d.Values;
            for (var i = 0; i < values.length; i++) {

                var responseValue = values[i];
                if (responseValue) {
                    var responseValueParsed = JSON.parse(responseValue);

                    if (responseValueParsed.AdminModeMessage) {
                        $('.ebstaStep5').children().hide();

                        // Show and setup the continue button to skip to the next step
                        continueButton.text("Continue").show();
                        continueButton.off();
                        continueButton.on("click", () => {
                            ShowEbstaStep(8);
                        });

                        $('#eb-onboarding-calendarsyncadminmodemessage').empty().text(responseValueParsed.AdminModeMessage).show();
                        return;
                    }
                }
            }
        }

        //We may reset the continue button above to handle the admin message so re-set up the continue button
        SetListenerUpOnContinueButton(false, true, false, false);
        continueButton.text("Preview Settings");

        // If we don't find an AdminModeMessage then go a fetch the calendars ync details for this users Calendar Sync
        chrome.extension.sendMessage({
            MessageType: "SendCalendarSyncRequest",
            RequestName: "CalSyncGetCalendarsAndSyncDetails",
            EmailAddress: GetGmailUsersEmailAddress()
        }, function (response) {
            HandleCalSyncSetupSettingsResponse(response.values);
        });
    });
}

//The Callback request when the preview events request has returned
function HandlePreviewEventsForCalSync(objJson, dateCheckForPreviewEventRequest) {
    RenderPreviewEventsCalendarSyncSetup(objJson, dateCheckForPreviewEventRequest);
}

//Handles the reponse for when we look for the fields we need on an event object during the onboarding process
function HandleCalSyncSetupCheckResponse(responseObj) {
    //console.timeEnd("CalSyncSetupCheckSettings");
    //console.time("HandleCalSyncSetupCheckResponse");
    var hasErrors = false;

    if (responseObj != null && responseObj != 'undefined') {
        if (responseObj.d) {
            if (responseObj.d.Values) {
                if (responseObj.d.Values.length > 0)
                {
                    hasErrors = false;
                    global_doesUserHavePrivateEventField = true;
                    $('.eb-fieldErrorTableCell').hide();

                    //Loop through result values to find the fields that we found to have problems with
                    for (var i = 0; responseObj.d.Values.length > i; i++)
                    {
                        var valueObj = JSON.parse(responseObj.d.Values[i]);

                        if (valueObj) {
                            if (!valueObj.IsAvailable) {
                                //If this is the IsPrivate field just change the global variable for later use don't show the error page
                                if (valueObj.FieldName == "IsPrivate")
                                {
                                    global_doesUserHavePrivateEventField = false;
                                }
                                else
                                {
                                    hasErrors = true;
                                    $('.eb-fieldErrorTableCell[sffieldname="' + valueObj.FieldName + '"]').show();
                                }
                            }
                        }
                    }

                    //Show error table
                    if (hasErrors) {
                        $('.eb-fieldErrorTable').show();

                        $('.eb-continuebtn').text("Refresh Settings").show(); //Need to change the text of the button as the user will need to change the object in their SF before continuing
                        $('.ebstaStep:visible').find('.ebNewLoader').hide(); //Hide normal messages now
                        $('.ebstaStep:visible').find('.alert-danger, .alert-info').show(); //Show error messages
                        SetListenerUpOnContinueButton(true, false, false, false); //Re listen the listener on the continue button as we now need the button to recheck the object for the fields 
                    }
                }
            }
        }
    }

    //No Error were found to proceed to next step
    if (!hasErrors)
    {
        SetListenerUpOnContinueButton(false, false, false, false);
        $('.eb-continuebtn').text("Continue").trigger("click");
    }

    //console.timeEnd("HandleCalSyncSetupCheckResponse");
}

//Changes the status of the Sync Now Button in onbaording process
function UpdateSyncNowButton(activateButton) {
    $('#eb-wizard-onboarding .eb-syncnowbtn').each(function () {
        if (activateButton) {
            $(this).text('Sync Now').prop('disabled', false);
        } else {
            $(this).text('Currently Syncing').prop('disabled', true);;
        }
    });
}

//The callback when we retrieve all details on a users Current calendar sync details and their google calendars
function HandleCalSyncSetupSettingsResponse(responseObj)
{
    //console.timeEnd("CalSyncGetCalendarsAndSyncDetails");
    //console.time("HandleCalSyncSetupSettingsResponse");
    //Lets hide the loader and empty the calendar list if exists now that we have new data
    $('.ebstaStep5 .ebNewLoader').hide();
    $('.ebstaStep5 .eb-calsync-calendarlist').empty();
    $('#eb-onboarding-calendarsyncadminmodemessage').hide();

    global_usingOldAuthToken = false;

    if (responseObj) {
        if (responseObj.d) {
            if (responseObj.d.Values) {
                if (responseObj.d.Values.length > 0) {
                    for (var i = 0; i < responseObj.d.Values.length; i++) {

                        var fld = responseObj.d.Values[i];
                        if (fld) {
                            var noteList = JSON.parse(fld);

                            if (noteList) {
                                $('.eb-continuebtn').show();
                                //Create a calendar entry in the onboarding screen for each calendar record
                                if (noteList.id != null) {
                                    CreateCalendarEntryInCalendarSyncSetup(noteList);

                                    // We'd like to log what the primary calendar was at the time so we can have a general check to 
                                    // see if their current account matches the primary calendar that they're going to sync, this helps
                                    // us check if users would be setting up the calendar sync within another account
                                    if (noteList.primary) {
                                        LogEvent("Primary Calendar: " + noteList.id + " - Current Gmail: " + GetGmailUsersEmailAddress());
                                    }

                                    isCalendarSyncLive = false;
                                }
                                else {
                                    //If a record from the results didn't have an id this means a user currently has a live calendar sync as we only return live calendar syncs back
                                    isCalendarSyncLive = true;

                                    //Update the checkboxes if we've found the preview settings from before
                                    if (noteList.DontSyncPrivateEvents != null) {
                                        $('#eb-wizard-onboarding .eb-onboarding-syncprivateeventsoption input').prop("checked", noteList.DontSyncPrivateEvents);
                                    }

                                    if (noteList.DontSyncRecurringEvents != null) {
                                        $('#eb-wizard-onboarding .eb-onboarding-syncrecurringeventsoption input').prop("checked", noteList.DontSyncRecurringEvents);
                                    }

                                    //Update the sync now button depending on current status
                                    if (noteList.CalendarSyncStatus != null) {
                                        if (noteList.CalendarSyncStatus == 0) {
                                            UpdateSyncNowButton(false);
                                        } else {
                                            UpdateSyncNowButton(true);
                                        }
                                    }

                                    if (noteList.SyncOnlyColourEvents) {
                                        //We're not doing anything with coloured events just yet
                                    }

                                    //If we're still using the old Calendar Sync token we need to pass this back up to make sure we don't go looking for token associated with the current users gmail page]
                                    if (noteList.UsingOldCalendarSyncToken) {
                                        global_usingOldAuthToken = true;
                                    }

                                    //Details of a users last calendar sync
                                    if (noteList.LastSync) {
                                        var lastSyncText = "Not Synced Yet";

                                        //Format date into readable form
                                        var date = new Date(noteList.LastSync);

                                        if (date && date.getFullYear() > 2000) {
                                            var g_days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
                                            var g_months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];

                                            var mins = date.getMinutes();
                                            if (mins < 10) {
                                                mins = "0" + date.getMinutes();
                                            }
                                            lastSyncText = date.getDate() + " " + g_months[date.getMonth()] + ' ' + date.getFullYear() + " " + ReturnDateForPreviewEvents(date);
                                        }

                                        //Update last sync text
                                        $('.eb-onboarding-lastsynctext').text(lastSyncText);
                                    }
                                }                                
                            }
                        }
                    }
                }
                else
                {
                    //If this process dies by any chance we'll direct them to the Auth process to try re-authing themselves
                    OpenOnboardingModal(1);
                    isAccountAuthorised = false;
                }
            }
        }
    }

    UpdateCalendarSettingsPage(isCalendarSyncLive);

    //console.timeEnd("HandleCalSyncSetupSettingsResponse");
}

/**
 * Sends a request of to get the List Views for Contact Sync
 * @param {string} searchString - A string to filter the results of the list views by name
 * @param {function} callback - A function to call when we received a response back from gathering list view information
 * @param {DOMElement} resultsTable - The table to render the results in
 */
function GetContactSyncListViews(searchString, callback, resultsTable) {

    chrome.extension.sendMessage({
        MessageType: "ContactSyncSetupListViews",
        GmailAccount: GetGmailUsersEmailAddress(),
        SearchString: searchString
    }, function (response) {
        callback();
        RenderContactSyncSetupListViews(response, resultsTable);
    });
}

/**
 * Displays the results from a Contact List View Search
 * @param {object} resultsObject - The results of the search
 * @param {any} resultsTable - The table the results need to be appended too
 */
function RenderContactSyncSetupListViews(resultsObject, resultsTable) {

    if (resultsObject && resultsObject.d) {
        var array = resultsObject.d.Values;

        if (array && array.length > 0) {

            var tableBody = resultsTable.getElementsByTagName("tbody")[0];

            if (tableBody) {

                EmptyElement(tableBody);

                for (var i = 0; i < array.length; i++) {
                    var parsedJSON = JSON.parse(array[i]);

                    var listViewRow = CreateElement("tr", null, null, null);

                    var tableCellElement = CreateElement("td", null, null, null);

                    //Create List View Names with hyperlinks to the actual list in Salesforce
                    var hyperLink = "#";
                    if (parsedJSON.InstanceURL) {
                        //Set URL for list view (We need to take the last 3 characters off the id as the URL doesn't work)
                        hyperLink = parsedJSON.InstanceURL + "/003?fcf=" + parsedJSON.id.substring(0, parsedJSON.id.length - 3);

                        //Update the Create New List URL
                        $('.eb-confirmcontactsync-tableheader-right').attr('href', parsedJSON.InstanceURL + "/ui/list/FilterEditPage?ftype=c")
                    }
                    var listViewNameHyperlink = CreateHyperlink("", hyperLink, parsedJSON.label);
                    listViewNameHyperlink.addEventListener("click", function () { LogEvent("Onboarding Click - Edit List View"); });
                    tableCellElement.appendChild(listViewNameHyperlink);

                    $(listViewRow).append(tableCellElement).hide().fadeIn(100);

                    var detailedColumn = CreateElement("td", null, "", null);

                    //Create the associated button for user to action the list view sync
                    var buttonText = "", buttonClass = "";

                    switch (parsedJSON.ListStatus) {
                        case 'restartNeeded':
                            buttonText = "Restart Sync";
                            buttonClass = "ebsta-btn ebsta-btn-lg ebsta-btn-warning eb-contactsync-restartbtn";
                            break;
                        case 'off':
                            buttonText = "Start Sync";
                            buttonClass = "ebsta-btn ebsta-btn-lg ebsta-btn-success eb-contactsync-startbtn";
                            break;
                        case 'on':
                            buttonText = "Stop Sync";
                            buttonClass = "ebsta-btn ebsta-btn-lg ebsta-btn-danger eb-contactsync-stopbtn";
                            break;
                    }

                    var selectListButton = CreateButton("eb-contactsync-listViewSelect" + parsedJSON.id, buttonText, buttonClass);
                    selectListButton.setAttribute('listViewId', parsedJSON.id);
                    selectListButton.setAttribute('listViewName', parsedJSON.label);
                    if (parsedJSON.ContactSyncRuleId) {
                        selectListButton.setAttribute('contactsyncruleid', parsedJSON.ContactSyncRuleId);
                    }
                    AttachListenerToContactSyncSyncButtons(selectListButton, parsedJSON.FieldMappings);

                    detailedColumn.appendChild(selectListButton);
                    detailedColumn.appendChild(CreateBr());

                    //Display Last Sync text
                    if (parsedJSON.LastSync) {
                        var lastSyncTextNode = CreateDiv("", "eb-confirmcontactsync-lastsynctext");
                        lastSyncTextNode.appendChild(CreateTextNode("Last Sync: "));

                        var lastSyncText = "Not Synced Yet";
                        //Format date into readable form
                        var date = new Date(parsedJSON.LastSync);
                        if (date && date.getFullYear() > 2000) {
                            var g_days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
                            var g_months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];

                            var mins = date.getMinutes();
                            if (mins < 10) {
                                mins = "0" + date.getMinutes();
                            }
                            lastSyncText = date.getDate() + " " + g_months[date.getMonth()] + ' ' + date.getFullYear() + " " + ReturnDateForPreviewEvents(date);
                        }

                        lastSyncTextNode.appendChild(CreateElement("b", null, lastSyncText, null));
                        detailedColumn.appendChild(lastSyncTextNode);
                    }

                    $(listViewRow).append(detailedColumn).hide().fadeIn(100);

                    $(tableBody).append(listViewRow);

                    resultsTable.style.display = "";
                }
            }
        } else {
            var errorText = CreateDiv("", "alert alert-danger text-center");

            let errorMessageContainer = CreateElement("p", "eb-textcontent", "You currently have no Contact List Views to choose from.");
            errorText.appendChild(errorMessageContainer);

            if (resultsObject.d.status === "timeout") {
                errorMessageContainer.textContent = "You're search term was too broad and took too long, please narrow your search criteria and try again.";
            } else {
                var secondLineText = CreateElement("p", "eb-textcontent", "");
                secondLineText.appendChild(CreateTextNode("Please click "));
                secondLineText.appendChild(CreateHyperlink("alert-link", "https://emea.salesforce.com/ui/list/FilterEditPage?ftype=c", "click here"));
                secondLineText.appendChild(CreateTextNode(" to create a new list."));

                errorText.appendChild(secondLineText);
            }

            $('.eb-contactsync-selectlistviews').append(errorText);
        }
    }
}

//Attaches a listener to the buttons assocaited with Starting and Stopping a list view sync
function AttachListenerToContactSyncSyncButtons(buttonDiv, fieldMappings) {
    buttonDiv.addEventListener("click", function () {

        var listViewId = $(buttonDiv).attr('listViewId');
        var listViewName = $(buttonDiv).attr('listViewName');
        var contactSyncRuleId = $(buttonDiv).attr('contactsyncruleid');

        var logEventText = "";

        //Start Contact Sync
        if ($(buttonDiv).hasClass('eb-contactsync-startbtn')) {
            //Fire request off move to next stage
            ShowEbstaStep(10);
            $('.ebstaStep10 .ebNewLoader').show();
            $('.ebstaStep10 .eb-contactsync-fieldmappings').hide();  
            RenderListViewColumnsForFieldMapping(fieldMappings, listViewId, listViewName);
            $('.ebstaStep10 .ebNewLoader').hide();
            logEventText = "Onboarding Click - Start Sync (" + listViewId + ")";
        }
        //Stop Calendar Sync
        else if ($(buttonDiv).hasClass('eb-contactsync-stopbtn')) {
            $(buttonDiv).removeClass('ebsta-btn-danger').addClass('ebsta-btn-success');
            $(buttonDiv).removeClass('eb-contactsync-stopbtn').addClass('eb-contactsync-startbtn').text('Start Sync');
            FireRequestToStopOrRestartContactSync("stop", contactSyncRuleId);
            $(buttonDiv).closest('td').find('.eb-confirmcontactsync-lastsynctext').remove();
            logEventText = "Onboarding Click - Stop Sync (" + listViewId + ")";
        }
        //Restart Calendar Sync
        else if ($(buttonDiv).hasClass('eb-contactsync-restartbtn')) {
            $(buttonDiv).removeClass('ebsta-btn-warning').addClass('ebsta-btn-danger');
            $(buttonDiv).removeClass('eb-contactsync-restartbtn').addClass('eb-contactsync-stopbtn').text('Stop Sync');
            FireRequestToStopOrRestartContactSync("restart", contactSyncRuleId);
            logEventText = "Onboarding Click - Restart Sync (" + listViewId + ")";
        }

        LogEvent(logEventText);

        $(this).blur();
    });
}

//Fires the associated request to either Stop or Restart a contact sync
function FireRequestToStopOrRestartContactSync(stopOrStart, contactSyncRuleId) {
    if (stopOrStart.toLowerCase() == "stop") 
    {
        chrome.extension.sendMessage({ MessageType: "ContactSyncStopListSync", ContactSyncRuleId: contactSyncRuleId });
    } 
    else if(stopOrStart.toLowerCase() == "restart")
    {
        chrome.extension.sendMessage({ MessageType: "ContactSyncRestartListSync", ContactSyncRuleId: contactSyncRuleId });
    }
}

//Displays the field mapping data tables
function RenderListViewColumnsForFieldMapping(fieldMappings, listViewId, listViewName) {
    $('#eb-wizard-onboarding .eb-contactsync-fieldmappingtablesparentdiv table > tbody').empty();

    $('#eb-wizard-onboarding .eb-contactsync-fieldmappingtablesparentdiv').attr('listViewId', listViewId);
    $('#eb-wizard-onboarding .eb-contactsync-fieldmappingtablesparentdiv').attr('listViewName', listViewName);

    if (fieldMappings.length > 0) {
        var leftTableBody = $('#eb-wizard-onboarding .eb-contactsyncfieldmapping-table-left > tbody');
        var rightTableBody = $('#eb-wizard-onboarding .eb-contactsyncfieldmapping-table-right > tbody');
        for (var i = 0; i < fieldMappings.length; i++) {
            var listViewRow = CreateElement("tr", null, null, null);
            
            var tableCellElement = CreateElement("td", null, fieldMappings[i].GoogleField, null);
            $(listViewRow).append(tableCellElement);

            var rightTableCellElement = CreateElement("td", null, null, null);

            if (fieldMappings[i].GoogleField) {

                var dropDownDiv = CreateFieldMappingDropDown("sfContactSyncField chosen-select", fieldMappings[i].SalesforceFields);

                rightTableCellElement.appendChild(dropDownDiv);
            }          

            $(listViewRow).append(rightTableCellElement);

            if (i < 6) {
                $(leftTableBody).append(listViewRow);
            } else {
                $(rightTableBody).append(listViewRow);
            }
        }

        $('#eb-wizard-onboarding .eb-contactsync-fieldmappings,#eb-wizard-onboarding .eb-contactsync-fieldmappingtablesparentdiv').show();

        //Use the included Chosen jquery script to create our custom drop down box and include width param
        $(".chosen-select").chosen({ width: '210px'});

        SetDefaultMappingValues();
    }
}

//Goes through all dropdowns and sets some values we feel should be chosen to map (if those fields exist)
function SetDefaultMappingValues(){
    $('#eb-wizard-onboarding .eb-contactsync-fieldmappingtablesparentdiv table > tbody > tr').each(function () {
        var googleField = $(this).find('td:first-child').text();
        var defaultVal = "";
        if (googleField) {
            switch (googleField.toLowerCase()) {
                case "email":
                    defaultVal = "Email"
                    break;
                case "name":
                    defaultVal = "Name"
                    break;
                case "salutation":
                    defaultVal = "Salutation"
                    break;
                case "phone":
                    defaultVal = "HomePhone"
                    break;
                case "mobile":
                    defaultVal = "MobilePhone"
                    break;
                case "address":
                    defaultVal = "MailingStreet"
                    break;
                case "city":
                    defaultVal = "MailingCity"
                    break;
                case "state":
                    defaultVal = "MailingState"
                    break;
                case "zip/postal code":
                    defaultVal = "MailingPostalCode"
                    break;
                case "country":
                    defaultVal = "MailingCountry"
                    break;
                case "company":
                    defaultVal = "Account.Name"
                    break;
                case "job title":
                    defaultVal = "Title"
                    break;
            }
        }
        $(this).find('td:last-child > select').val(defaultVal).trigger("chosen:updated");
    });
}

//Creates the select drop down div for Field Mapping in Contact Sync onboarding
function CreateFieldMappingDropDown(className, salesforceFields) {
    var dropDown = document.createElement('select');
    dropDown.setAttribute("class", className);

    //Loop through array and append options for drop down
    if (salesforceFields.length > 0) {
        for (var i = 0; i < salesforceFields.length; i++) {
            if (salesforceFields[i].fieldNameOrPath) {
                if (salesforceFields[i].label) {
                    var optionForDropDown = document.createElement('option');
                    optionForDropDown.setAttribute("value", salesforceFields[i].fieldNameOrPath);
                    optionForDropDown.innerText = salesforceFields[i].label;
                    dropDown.appendChild(optionForDropDown);
                }
            }
        }
    } else {
        var optionForDropDown = document.createElement('option');
        optionForDropDown.setAttribute("value", "nosync");
        optionForDropDown.innerText = "No Fields To Sync";
        dropDown.appendChild(optionForDropDown);
    }

    return dropDown;
}

//Creates the basic table for the list view pages of Contact Sync
function CreateTableForListViewContactSync() {
    var tableDiv = CreateElement("table", "table table-striped table-bordered table-condensed eb-confirmcontactsync-table text-left", null, null);

    //Table Header
    var tableHeadDiv = CreateElement("thead", null, null, null);
    var headerRow = CreateElement("tr", null, null, null);
    
    var headerCell = CreateElement("th", null, "Contact List Name", null);
    var createListText = CreateHyperlink("eb-confirmcontactsync-tableheader-right", "#", "+ Create New List")
    createListText.addEventListener("click", function () { LogEvent("Onboarding Click - Create New List"); });
    headerCell.appendChild(createListText);
    headerRow.appendChild(headerCell);

    headerRow.appendChild(CreateElement("th", null, "Sync Status", null));
    tableHeadDiv.appendChild(headerRow);
    tableDiv.appendChild(tableHeadDiv);

    tableDiv.appendChild(document.createElement("tbody"));
    tableDiv.style.display = "none";
    return tableDiv;
}

//Create default tables for the field mapping section of Contact Sync
function CreateTablesForFieldMappingsContactSync() {
    var parentTableDiv = CreateDiv("", "eb-contactsync-fieldmappingtablesparentdiv");

    //First (left) Table
    var tableDiv = CreateElement("table", "table table-striped table-bordered table-condensed eb-contactsyncfieldmapping-table-left text-left", null, null);

    var tableHeadDiv = CreateElement("thead", null, null, null);
    var headerRow = CreateElement("tr", null, null, null);

    var headerCell = CreateElement("th", null, "Google Field", null);
    headerRow.appendChild(headerCell);

    headerRow.appendChild(CreateElement("th", null, "Salesforce Field", null));
    tableHeadDiv.appendChild(headerRow);
    tableDiv.appendChild(tableHeadDiv);

    tableDiv.appendChild(document.createElement("tbody"));

    parentTableDiv.appendChild(tableDiv);

    //Second (Right) Table
    var tableDiv2 = CreateElement("table", "table table-striped table-bordered table-condensed eb-contactsyncfieldmapping-table-right text-left", null, null);

    var tableHeadDiv2 = CreateElement("thead", null, null, null);
    var headerRow2 = CreateElement("tr", null, null, null);

    var headerCell2 = CreateElement("th", null, "Google Field", null);
    headerRow2.appendChild(headerCell2);

    headerRow2.appendChild(CreateElement("th", null, "Salesforce Field", null));
    tableHeadDiv2.appendChild(headerRow2);
    tableDiv2.appendChild(tableHeadDiv2);

    tableDiv2.appendChild(document.createElement("tbody"));

    //hide container div initially
    parentTableDiv.appendChild(tableDiv2);
    parentTableDiv.style.display = "none";

    return parentTableDiv;
}

//Creates basic table for Preview Contacts
function CreateTableForContactSyncPreviewSettings() {
    var tableDiv = CreateElement("table", "table table-striped table-bordered table-condensed eb-contactsyncpreviewcontacts-table text-left", null, null);

    //Table Header
    var tableHeadDiv = CreateElement("thead", null, null, null);
    var headerRow = CreateElement("tr", null, null, null);

    var headerCell = CreateElement("th", null, "Name", null);
    headerRow.appendChild(headerCell);

    headerRow.appendChild(CreateElement("th", null, "Company", null));

    headerRow.appendChild(CreateElement("th", null, "Email", null));

    headerRow.appendChild(CreateElement("th", null, "Phone", null));
    tableHeadDiv.appendChild(headerRow);

    tableDiv.appendChild(tableHeadDiv);

    tableDiv.appendChild(document.createElement("tbody"));
    tableDiv.style.display = "none";
    return tableDiv;
}

//Gets the field mapping values that the user has chosen
function GetSelectedFieldMappings() {
    var FieldMapping = function (googleField, salesforceField) {
        this.googleField = googleField;
        this.salesforceField = salesforceField;
    };

    var fieldMappingArray = new Array();

    //Find field mapping values the user has chosen
    $('#eb-wizard-onboarding .eb-contactsync-fieldmappingtablesparentdiv table > tbody > tr').each(function () {
        var googleField = $(this).find('td:first-child').text();
        var salesforceField = $(this).find('td:last-child > .chosen-select').chosen().val();
        fieldMappingArray.push(new FieldMapping(googleField, salesforceField));
    });
    return fieldMappingArray;
}

//Gets the users chosen Field Mappping values and sends them up to the DB so we can then get a list of preview contacts to display
function GetContactsForPreviewSettingsScreen(listViewName) {
    var fieldMappings = GetSelectedFieldMappings();

    var listViewId = $('#eb-wizard-onboarding .ebstaStep10 .eb-contactsync-fieldmappingtablesparentdiv').attr('listviewid');
    
    // Keeps track of the last request time so we can make sure the actual request that comes back matches this request
    // This helps catch a user pressing preview settings multiple times quickly
    var nowTime = new Date().getTime();
    global_dateCheckForPreviewContactRequestArray.unshift(nowTime);

    chrome.extension.sendMessage({
        MessageType: "ContactSyncPreviewSettings",
        FieldMappings: fieldMappings,
        ListViewId: listViewId,
        ListViewName: listViewName,
        GmailAccount: GetGmailUsersEmailAddress(),
        DateCheck: nowTime
    });

    $('#eb-wizard-onboarding .eb-contactsyncpreviewcontacts-table, #eb-wizard-onboarding .ebstaStep11 .alert').hide();
    $('.ebstaStep11 .ebNewLoader').show();
}

//Display the preview of contacts we'll sync
function CreateContactSyncPreviewSettings(objJson, dateCheck, contactSyncRuleId) {
    $('#eb-wizard-onboarding .eb-contactsyncpreviewcontacts-table > tbody').empty();

    var displayNoContactsMessage = true;

    if (objJson) {
        if (objJson.d) {
            if (objJson.d.ListOfContacts) {
                var fieldsArray = objJson.d.ListOfContacts;
                if (dateCheck == global_dateCheckForPreviewContactRequestArray[0]) {
                    if (fieldsArray.length > 0) {
                        var tableBody = $('#eb-wizard-onboarding .eb-contactsyncpreviewcontacts-table > tbody');
                        for (var i = 0; i < fieldsArray.length; i++) {
                            var listViewRow = CreateElement("tr", null, null, null);

                            var contactEmail = "";
                            if (fieldsArray[i].EmailSyncField) {
                                contactEmail = fieldsArray[i].EmailSyncField;
                            }

                            //Skip contacts without email addresses
                            if (contactEmail.length == 0) {
                                continue;
                            }

                            //Insert Name details
                            var contactName = "";
                            if (fieldsArray[i].NameField) {
                                contactName = fieldsArray[i].NameField;
                            }

                            var tableCellElement = CreateElement("td", null, contactName, null);
                            listViewRow.appendChild(tableCellElement);

                            //Insert Company Details
                            var companyDetails = "";
                            if (fieldsArray[i].AccountNameField) {
                                companyDetails = fieldsArray[i].AccountNameField;
                            }

                            var tableCellElement = CreateElement("td", null, companyDetails, null);
                            listViewRow.appendChild(tableCellElement);

                            //Insert Email Address
                            var tableCellElement = CreateElement("td", null, null, null);
                            tableCellElement.appendChild(CreateTextNode(contactEmail.toLowerCase()));

                            listViewRow.appendChild(tableCellElement);

                            //Insert Phone details
                            var contactPhone = "";
                            if (fieldsArray[i].PhoneField) {
                                contactPhone = fieldsArray[i].PhoneField;
                            } else if (fieldsArray[i].MobilePhoneField) {
                                contactPhone = fieldsArray[i].MobilePhoneField;
                            }

                            var tableCellElement = CreateElement("td", null, null, null);

                            var mainPhoneDiv = CreateDiv("", "");
                            var textPhoneDiv = CreateDiv(contactPhone, "eb-contactsyncpreview-phonenumber");
                            mainPhoneDiv.appendChild(textPhoneDiv);
                            //Insert 'View More' Link to display the details row
                            var textDiv = CreateDiv("View More", "");
                            var viewMoreHyperLink = CreateHyperlink("ebsta-float-right eb-contactsyncpreview-viewmore", "#", "")
                            viewMoreHyperLink.setAttribute("onClick", "return false;");
                            viewMoreHyperLink.appendChild(textDiv);
                            mainPhoneDiv.appendChild(viewMoreHyperLink);
                            tableCellElement.appendChild(mainPhoneDiv);

                            //Click Listener for 'View More' hyperlink which will display the details row and change the text
                            textDiv.addEventListener("click", function () {
                                var detailsRow = $(this).closest('tr').next();
                                if ($(detailsRow).is(':visible')) {
                                    $(detailsRow).hide();
                                    $(this).text('View More');
                                } else {
                                    $(detailsRow).show();
                                    $(this).text('View Less');
                                }
                            });

                            listViewRow.appendChild(tableCellElement);

                            //Create the Contact Details Row
                            var detailedViewRow = CreateElement("tr", "eb-contactsyncpreviewcontacts-table-contactdetails", null, null);
                            var tableCellElement = CreateElement("td", null, null, null);
                            tableCellElement.setAttribute("colspan", 4);
                            tableCellElement.appendChild(CreateDetailedTableForContactPreviewDetails(fieldsArray[i]));

                            detailedViewRow.appendChild(tableCellElement);
                            $(detailedViewRow).hide();
                            $(tableBody).append(listViewRow);
                            $(tableBody).append(detailedViewRow);
                        }
                    }
                }
            }
        }

        if (objJson.d.TotalCountOfContactsForSyncing) {
            if (objJson.d.TotalCountOfContactsForSyncing > 0) {
                displayNoContactsMessage = false;
                var displayText = "Ebsta will sync " + objJson.d.TotalCountOfContactsForSyncing;

                if (objJson.d.TotalCountOfContactsForSyncing > 1) {
                    displayText += " contacts to Google Contacts."
                } else {
                    displayText += " contact to Google Contacts."
                }

                $('#eb-wizard-onboarding .ebstaStep11 .alert-warning').show().text(displayText);
            }
        }
    }

    $('.ebstaStep11 .ebNewLoader').hide();

    //Display an error message if this list has no contacts returned
    if (displayNoContactsMessage) {
        $('#eb-wizard-onboarding .ebstaStep11 .alert-danger').show();
    } else {
        $('#eb-wizard-onboarding .eb-contactsyncpreviewcontacts-table').show();
    }
}

//Sets the ContactSyncRuleId so we can use this value to update the table
function SetContactSyncRuleIdInPreviewTable(contactSyncRuleId) {
    $('#eb-wizard-onboarding .eb-contactsyncpreviewcontacts-table').attr('contactSyncRuleId', contactSyncRuleId);
    $('#eb-wizard-onboarding .eb-continuebtn').show();
}

//Takes a preview contact record and creates the detailed view for a user to see
function CreateDetailedTableForContactPreviewDetails(record) {
    var tableDiv = CreateElement("table",null, null, null);

    var headerRow = CreateElement("tr", null, null, null);
    var cellCount = 0;
    for (var key in record) {
        if (record.hasOwnProperty(key)) {
            var keyValue = record[key];
            if (keyValue) {
                var headerCell = CreateElement("td", null, null, null);

                key = ConvertKeyToDisplayValue(key);

                if (key.length > 0) {
                    var textContent = CreateElement("ebsmall", null, key + ": " + keyValue, null);
                    headerCell.appendChild(textContent);
                    headerRow.appendChild(headerCell);
                    cellCount++;

                    //Only go 2 columns wide when showing details so create a new row after appending to 2 columns
                    if (cellCount === 2) {
                        tableDiv.appendChild(headerRow);
                        headerRow = CreateElement("tr", null, null, null);
                        cellCount = 0;
                    }
                }
            }
        }
    }

    //If the last row has just a single cell then append it onto the div to display 
    if (cellCount === 1) {
        tableDiv.appendChild(headerRow);
    }

    return tableDiv;
}
    
    //Converts a ContactSyncEntity property into a readable value for the user
    function ConvertKeyToDisplayValue(key) {
        switch (key) {
            case "AccountNameField":
                return "Company";
            case "JobTitleField":
                return "Job Title";
            case "MobilePhoneField":
                return "Mobile";
            case "PostalCodeField":
                return "Zip/Postal Code";
            case "EmailSyncField":
                return "Email";
            default:
                if (key.indexOf("Field") != -1) {
                    return key.replace("Field", "");
                } else {
                    return "";
                }
        }
    }

    //Sends a request to confirm the settings for a Contact Sync list view
    function ConfirmContactSyncSettings(contactSyncRuleId) {
        chrome.extension.sendMessage({ MessageType: "ContactSyncConfirmSettings", ContactSyncRuleId: contactSyncRuleId });
    }

    //Updates the calendar settings page removing and adding elemnts which should be there
    function UpdateCalendarSettingsPage(isCalendarSyncLive)
    {
    if (isCalendarSyncLive && $('.ebstaStep:visible').hasClass('ebstaStep5'))
    {
        $('.eb-onboarding-currentstatustext').text("Active").addClass("text-success").removeClass("text-danger");
        $('.eb-onboarding-lastsynctext, .eb-onboarding-lastsyncheader, .eb-stopcalendarsyncbtn, .eb-syncnowbtn').css('visibility', '').fadeIn();
    }
    else
    {
        $('.eb-onboarding-currentstatustext').text("Awaiting Setup").removeClass("text-success").addClass("text-danger");
        $('.eb-onboarding-lastsynctext, .eb-onboarding-lastsyncheader, .eb-stopcalendarsyncbtn, .eb-syncnowbtn').css('visibility', 'hidden');
    }
    }

    function CloseWizard() {
    var wizard = document.getElementById("ebsta-wizard");
    if (wizard != null) {
        wizard.style.display = "none";
    }

    var wizard = document.getElementById("ebsta-opaque");

    if (wizard != null) {
        wizard.style.display = "none";
    }
    }




    function ShowOpaque() {

    this._div = document.createElement("div");
    this._div.style.zIndex = "100000";
    this._div.id = "ebsta-opaque";
    this._div.style.position = "absolute";
    this._div.style.width = "100%";
    this._div.style.height = "100%";
    this._div.style.background = "rgba(107, 107, 107, .5)";
    this._div.style.display = "none";
    //document.body.insertBefore(this._div, document.body.firstChild);
    this._div.style.display = "inline";
    }

    //Takes the ComplexAdd settings object from LayoutManager and creates the fields within the complex add screen
    function CreateResultsDivForCustomFieldSearch(db) {
    var mainResultsDiv = CreateDiv("", "resultsWindow");
    
    if (db != null) {
        if (db.d != null) {
            if (db.d.Values != null) {
                if (db.d.Values.length > 0) {
                    for (var i = 0; i < db.d.Values.length; i++) {
                        var result = JSON.parse(db.d.Values[i]);

                        var id = result.Id;
                        var name = result.Name;
                        var type = result.attributes.type;
                        var addIcon = CreateAddIcon();

                        //As Case does not have a name we need to show the CaseNumber
                        if(type == "Case"){
                            name = result.CaseNumber;
                        }

                        var resultDiv = CreateComplexAddObjectDiv(CheckStringLength(name, 37),
                            type,
                            "result CABoxStyle sfSuggest" + type + " sfSuggestGrey",
                            addIcon);

                        resultDiv.setAttribute("recordId", id);
                        resultDiv.setAttribute("recordType", type);
                        resultDiv.setAttribute("recordName", name);

                        SetClickListenerOnAddingResult(resultDiv);

                        mainResultsDiv.appendChild(resultDiv);
                    }
                } else {
                    var resultDiv = CreateDiv("No results found. Please search again.", "ebNoResult");
                    mainResultsDiv.appendChild(resultDiv);
                }
            }
        }
    }

    return mainResultsDiv;
    }

    //Adds a listener to the results returned back from reference fields for the custom fields in Complex Add
    function SetClickListenerOnAddingResult(div) {
    if (div != null) {
        div.addEventListener("click", function ()
        {
            var recordType = div.getAttribute('recordtype');
            var recordId = div.getAttribute('recordid');
            var recordName = div.getAttribute('recordname');

            var crossIcon = CreateCrossIcon();

            if (recordType.toLowerCase() == "opportunity") {
                recordType = "Opp";
            }

            var newlySelectedDiv = CreateComplexAddObjectDiv(CheckStringLength(recordName, 37), recordType, "CABoxStyle sfSuggest" + recordType, crossIcon);
            newlySelectedDiv.setAttribute("recordId", recordId);
            newlySelectedDiv.setAttribute("recordType", recordType);
            newlySelectedDiv.setAttribute("recordName", recordName);

            SetClickListenerOnRemoveResult(crossIcon);

            //If a selection has already been chosen then trigger the remove click on that div so it follows the same function of being appended back onto the results list
            $(this).closest('.ebsta-form-group').find('.ebCustomFieldSearchContainer').find('.CACrossIcon').each(function () {
                $(this)[0].click();
            });

            //Insert select result into input box
            $(this).closest('.ebsta-form-group').find('.ebCustomFieldSearchContainer').append(newlySelectedDiv);

            //Hide the input elements from the user disabling them from search until they remove the selected object
            $(this).closest('.ebsta-form-group').find('.ebCustomFieldSearchContainer').find('input, span').hide();

            //Now we've got everything and rendered the selectec will remove it from the results list
            $(this).remove();
        });
    }
    }

    //Adds a listener to the select result on reference fields for custom fields in Complex Add
    function SetClickListenerOnRemoveResult(div)
    {
    if (div != null) {
        div.addEventListener("click", function ()
        {
            //Hide the hover over box once something has been selected
            var hoverOverDiv = document.getElementsByClassName("sfRecordHoverDiv userSelecting");
            if (hoverOverDiv.length === 1) {
                HideElement(hoverOverDiv[0]);
                RemoveClass(hoverOverDiv[0], "userSelecting");
            }

            var recordType = div.parentNode.getAttribute('recordtype');
            var recordId = div.parentNode.getAttribute('recordid');
            var recordName = div.parentNode.getAttribute('recordname');

            var addIcon = CreateAddIcon();

            if (recordType.toLowerCase() == "opportunity") {
                recordType = "Opp";
            }

            var newlySelectedDiv = CreateComplexAddObjectDiv(CheckStringLength(recordName, 37), recordType, "CABoxStyle sfSuggest" + recordType + " sfSuggestGrey", addIcon);
            newlySelectedDiv.setAttribute("recordid", recordId);
            newlySelectedDiv.setAttribute("recordType", recordType);
            newlySelectedDiv.setAttribute("recordName", recordName);

            SetClickListenerOnAddingResult(newlySelectedDiv);

            //Put object back into results window
            $(this).closest('.ebsta-form-group').find('.resultsWindow').append(newlySelectedDiv);

            //Show the input elements from the user now that they've deselected the chosen object
            $(this).closest('.ebsta-form-group').find('.ebCustomFieldSearchContainer').find('input, span').show();

            //Remove the selection
            $(this).parent().remove();
        });
    }
    }

    function PassDataBack(entityType, db, searchUsed) {

    //Catch the Custom Field Search for fields with Complex Add that are references
    if (entityType.indexOf("-CustomFieldSearch") != -1) {
        var resultsDiv = CreateResultsDivForCustomFieldSearch(db);

        //Find the field the user was searching from so we can append the results to the correct field
        //e.g. Account-ComplexAdd-CustomFieldSearch-ApiName=TestField__c
        var regex = /ApiName=(.*)/g;
        var regexMatch = regex.exec(entityType);
        if (regexMatch) {
            $('div[apiname=' + regexMatch[1] + "]").parent().find('.resultsWindow,.resultsLoader').remove();
            $('div[apiname=' + regexMatch[1] + "]").after(resultsDiv);
        }
    }

    if (entityType.indexOf("elookup-") != -1) {

       // var entity = entityType.replace("elookup-", "");
        
        var listOl = document.createElement('ol');
        listOl.id = "allLogins1";
        listOl.setAttribute("class", "itemlist");


        for (var dt = 0; dt < db.d.Values.length; dt++) {
            var strJson = db.d.Values[dt];
            var res = JSON.parse(strJson);

       
            //for (var dt = 0; dt < db.d.Values.length; dt++) {

            //    var res = JSON.parse(db.d.Values[dt]);

                if (res.Id + '' != 'null') {
                    var nameField = GetNameFieldValue(res);

                    var objectType = '';

                    if (res.attributes != null) {
                        objectType = res.attributes.type;
                    } else if (res.type != null) {
                        objectType = res.type;
                    }

                    var relatedText = '';

                    if (objectType == "Contact") {
                        if (res.Account != null) {
                            relatedText = res.Account.Name;
                        } else if (res.Email != null) {
                            relatedText = res.Email;
                        }
                    }

                    if (objectType == "Lead") {
                        if (res.Company != null) {
                            relatedText = res.Company;
                        } else if (res.Email != null) {
                            relatedText = res.Email;
                        }
                    }

                    if (objectType == "Account") {
                        if (res.BillingCity != null && res.BillingCity != "") {
                            relatedText = res.BillingCity;
                        } else if (res.ShippingCity != null && res.ShippingCity != "") {
                            relatedText = res.ShippingCity;
                        }
                    }

                    if (objectType == "Opportunity") {
                        if (res.StageName != null) {
                            relatedText = res.StageName;
                        }
                    }

                    if (objectType == "Case"){
                        if (res.CaseNumber != null) {
                            relatedText = res.CaseNumber;
                        }
                    }

                    if (objectType == "Solution") {
                        if (res.SolutionNumber != null) {
                            relatedText = res.SolutionNumber;
                        }
                    }

                    if (objectType == "Product2") {
                        if (res.ProductCode != null) {
                            relatedText = res.ProductCode;
                        }
                    }

                    var liItem = document.createElement('li');
                    liItem.setAttribute("role", "listitem");
                    liItem.setAttribute("class", "lookup pointer");
                    liItem.setAttribute("id", res.Id);
                    liItem.setAttribute("name", nameField);
                    liItem.innerText = nameField + ' (' + relatedText + ')';
                    listOl.appendChild(liItem);
                    // htmlStr = htmlStr + '<li role="listitem" class="lookup pointer" id="' + res.Id + '" name="' + GetNameFieldValue(res) + '" >' + GetNameFieldValue(res) + '</li>';
                }
           // }
        }

        if (db.d.Values.length == 0) {
            var liItemEmpty = document.createElement('li');
            liItemEmpty.className = "";
            liItemEmpty.innerText = "No matches. Please search again";
            listOl.appendChild(liItemEmpty);

            //                htmlStr += '<li>No matches. Please search again</li>';
        }
     
        var searchResplaceHolder = document.getElementById('entityResults');
        
        if (searchResplaceHolder != null) {
            searchResplaceHolder.appendChild(CreateDiv(entityType + " Results", "entitySearchLabelResults"));
            searchResplaceHolder.innerHTML = '';
            searchResplaceHolder.appendChild(listOl);
        }

        var textHidden = $("body").find(".lookup");
        
        textHidden.each(function () {
            $(this).click(function () {

                SelectEntityLookup($(this).attr("id"), $(this).attr("name"), $("#entityResults").attr("sfType"));
              
            });
        });
    }

    }

    function GetNameFieldValue(objResult) {
    var result = "";
    var counter = 0;

    for (var key in objResult) {

        if (counter == 2) {
            result = "" + objResult[key];
            break;
        }
        counter++;
    }

    return result;
    }

    function SelectEntityLookup(id, name, control) {

    // We just need to set these back in the box and lets 


    var elmTextName = "#txtWhoName";
    var elmBt = "#btLookupWhoId";
    
    if (control == "WhatId") {
        elmTextName = "#txtWhatName";
        elmBt = "#btLookupWhatId";
    }
    
    $(elmTextName).attr("Value", name);
    $(elmTextName).attr("sfid", id);
    $(elmTextName).attr("class", "sfTextBoxOld readonly");
    $(elmTextName).attr("readonly", "");
    
    $(elmBt).text("Change");

    CloseLookupWindow(); 
    }

    function CloseLookupWindow() {

    $('#sfTxtSearch').die();

    $('#sfbtSearch').unbind('click');
    
    showLookup = false;
    $('.sfBoxLookup').remove();
    }

    function ShowAdvancedAddBox() {
    $(".acX").addClass("fixed");
    var divBox = document.createElement('div');
    divBox.id = "ebstaBox";
    divBox.setAttribute("class", "sfBox Kj-JD aJ7");
    divBox.setAttribute("style", "left: 653px; top: 302.5px; opacity: 1;");

    (document.body || document.documentElement).appendChild(divBox); 

    }

    function CreateDivWithId(Name, className,id) {
    var spanHeader = document.createElement('Div');
    spanHeader.setAttribute("class", className);
    spanHeader.innerText = Name;
    spanHeader.setAttribute("id", id);


    return spanHeader;
    }

    function CreateDiv(Name,className) {
    var spanHeader = document.createElement('Div');
    spanHeader.setAttribute("class", className);
    spanHeader.innerText = Name;


    return spanHeader;
    }

    function CreateSpan(name, className) {
    var spanHeader = document.createElement('span');
    spanHeader.setAttribute("class", className);
    spanHeader.innerText = "" + name;

    return spanHeader; 
    }

    function CreateTxtbox(name, className, idname) {
    var spanHeader = document.createElement('input');
    spanHeader.setAttribute("class", className);
   // spanHeader.innerText = "" + name;
    spanHeader.setAttribute("type", "textbox");
    spanHeader.setAttribute("id", idname);


    return spanHeader;
    }

    function OptionBox(name, className, idname) {
    var spanHeader = document.createElement('select');
    spanHeader.setAttribute("class", className);
    spanHeader.innerText = "" + name;
    spanHeader.setAttribute("id", idname);
    

    return spanHeader; 
    }

    function CreateBr() {
    var spanHeader = document.createElement('br');
    return spanHeader;
    }

    function CreateTextNode(text){
    return text = document.createTextNode(text);
    }

    function CreateLabel(text, className) {
    var labelDiv = document.createElement('label');
    labelDiv.setAttribute("class", className);
    labelDiv.appendChild(CreateTextNode(text));
    return labelDiv;
    }

    function CreateCheckbox(className) {
    var checkBoxDiv = document.createElement('input');
    checkBoxDiv.setAttribute("class", className);
    checkBoxDiv.setAttribute("type", "checkbox");
    return checkBoxDiv;
    }

    function CreateImg(src,alt,className) {
        var spanHeader = document.createElement('img');
        spanHeader.setAttribute("src", src);
        spanHeader.setAttribute("alt", alt);

        if (className) {
            spanHeader.setAttribute("class", className);
        }
        return spanHeader;
    }

    function CreateBt(name, className, idname) {
    var spanHeader = document.createElement('button');
    spanHeader.setAttribute("class", className);
    spanHeader.innerText = "" + name;
    spanHeader.setAttribute("id", idname);


    return spanHeader;
    }

    function CreateRow(className) {
    var spanHeader = document.createElement('div');
    spanHeader.setAttribute("class", className);
    return spanHeader;
    }

    function Createtd(className) {
    var spanHeader = document.createElement('td');
    spanHeader.setAttribute("class", className);
    return spanHeader;
    }

    function CreateHyperlink(className, href, text) {
    var linkElement = CreateElement("a", className, text, null);
    linkElement.setAttribute("href", href);
    return linkElement;
    }

    function CreateDropDown(className, options) {
    var dropDown = document.createElement('select');
    dropDown.setAttribute("class", className);

    //Loop through array and append options for drop down
    for (var i = 0; i < options.length; i++)
    {
        var optionForDropDown = document.createElement('option');
        optionForDropDown.setAttribute("value", options[i]);
        optionForDropDown.innerText = options[i];
        dropDown.appendChild(optionForDropDown);
    }

    return dropDown;
    }


    function LoadWhoWhatIds(SFType,Id,ObjectName) {
    chrome.extension.sendMessage({
        MessageType: "LoadWhoWhat"
    }, function (response) {

        var whatId = response.whatId;
        SplitUpIntoOptions(whatId, "WhatId",SFType,Id,ObjectName);
        var whoId = response.whoId;
        SplitUpIntoOptions(whoId, "WhoId", SFType, Id, ObjectName);

        //$('#sfWhoId').trigger('change');
        BlockLeadValue(0);
        CheckLeadStatus();

        $("body").delegate("#sfWhoId", "change", function () {
            BlockLeadValue(1);
        });
    });
    }

    function SplitUpIntoOptions(list, typeList,SFType, Id, ObjectName) {

    var lst = list.split('^');
    
    var elm = document.getElementById("sfWhoId");
    var elmTextName = "#txtWhoName";
    var elmBt = "#btLookupWhoId";

    if (typeList == "WhatId") {
        $("#sfWhatId").empty();
        elm = document.getElementById("sfWhatId");
        elmTextName = "#txtWhatName";
        elmBt = "#btLookupWhatId";
    } else {
        $("#sfWhoId").empty();
    }


    for (var d = 0; d < lst.length; d++) {

        var opt = document.createElement('option');
        opt.setAttribute("value", lst[d]);
        opt.innerText = lst[d];

        if (SFType == lst[d]) {
            opt.setAttribute("selected", "");
      

            $(elmTextName).attr("Value", ObjectName);
            $(elmTextName).attr("sfid", Id);
            $(elmTextName).attr("class", "sfTextBoxOld readonly");
            $(elmTextName).attr("readonly", "");
            $(elmBt).text("Change");
            

        }
        elm.appendChild(opt);
    }

    }

    function ApplySlookup_ConvertObject(obj) {
    ApplySlookup(obj.LookEmailAddress, obj.Id, obj.Type, obj.Server, obj.FoundMultipleRecords, obj.Url, obj.Name)
    }


    function ApplySlookup(emailAddress, id, type, server, multipleRecords, url, name) {
    LoadWhoWhatIds(type, id, name);
    }

    function AddEmailTemplatesBox(e) {

    $("#emailTemplateBox").remove();
    //Create the email template box (modal / pop out)
    var mainBox = document.createElement('div');
    mainBox.id = "emailTemplateBox";
    mainBox.setAttribute("class", "sfBox front");
    
    var emailTemplateBox = document.createElement('div');
    emailTemplateBox.setAttribute("class", "sfEmailTemplateOptions");

        // LK - append 

     

    //Create a content row
    var divContentRow = CreateRow("sfRowContent");
    divContentRow.setAttribute("id", "sfRowContent");
    emailTemplateBox.appendChild(divContentRow);
    
    //Create a templates list row
    var divListtRow = CreateRow("sfTemplatesList");
    divListtRow.setAttribute("id", "sfTemplatesList");
    divListtRow.style.display = "none";
    emailTemplateBox.appendChild(divListtRow);

    //Find if this is a compose or reply/forward screen
    if (e != null)
    {
        var target = null;

        if (e.toElement != null) {
            target = e.toElement
        }
        else if (e.target != null)  
        {
            target = e.target
        }
        
        if (target != null)
        {
            var composeOrReply = IsTheEmailMessageAComposeOrReply(target);
            //Find all email address associated with this email window
            var listOfEmailAddresses = GetAllEmailsInGmailComposeOrReplyWindow(composeOrReply, target);            
            //Put list into div
            emailTemplateBox.setAttribute("emailaddresses", listOfEmailAddresses);
        }
    }

    chrome.extension.sendMessage({
        MessageType: "GetTemplateFolders"
    });
    
    mainBox.appendChild(emailTemplateBox);
    
    var loadingResults = CreateDiv("Loading templates... ", "sfLoadingResults");
    loadingResults.className = 'resultsLoader';
    loadingResults.setAttribute("style", "display:inline-block;margin-left:10px;margin-top:15px;font-size:14px");
    var loadingGif = CreateImg(chrome.extension.getURL("/images/ajax-loader.gif"), "Loading templates...");
    loadingResults.appendChild(loadingGif);

    emailTemplateBox.appendChild(loadingResults);

    //Append the template
    (document.body || document.documentElement).appendChild(mainBox);
    
    LoadContactOrLeadSuggestionsBoxForEmailTemplates();

    //Create a title and close button for the box
    var templateBoxTitle = CreateSpan("Select a Salesforce Template", "sftitle-text");
    $(mainBox).prepend(templateBoxTitle);

    // Create a container for all the header items in Salesforce template to allow for a border to be created
    var headerContainer = document.createElement("div");
    headerContainer.className = 'ebsta-header-container';

    // prepend the headcontainer
    mainBox.prepend(headerContainer);
    
    // append the title span into the headerContainer
    headerContainer.appendChild(templateBoxTitle);

        var templateHelpIcon = EbstaModalTools.CreateHelpIcon("ebsta-help-icon-right",
            "https://ebstalimited.zendesk.com/hc/en-us/articles/231603647-Using-Salesforce-Templates",
            "Visit Our Knowledge Base");

        templateHelpIcon.id = "EbstaAnchorTemplatesHelp﻿";

    // Append helpIcon to the headerContainer
        headerContainer.appendChild(templateHelpIcon);

    var manageTemplatesLink = document.createElement('a');
    manageTemplatesLink.target = '_blank';
    manageTemplatesLink.href = instanceUrl + '/00X?setupid=CommunicationTemplatesEmail';
    manageTemplatesLink.appendChild(document.createTextNode("Manage Templates"));
        manageTemplatesLink.className = "manageTemplatesLink sfLink";
        manageTemplatesLink.id = "EbstaAnchorManageTemplates";

    headerContainer.appendChild(manageTemplatesLink);

    mainBox.appendChild(CreateSpan("", "sftitle-close-templates sftitle-close"));

    $('#emailTemplateBox .sfBtCancel').click(function () {
        showEmailTemplateBox = false;
        $('#emailTemplateBox').remove();
        $('.sfEBComplexAddSearchObjectChoiceDiv').hide();
    });


    $('#emailTemplateBox .sftitle-close-templates').click(function () {
        showEmailTemplateBox = false;
        $('#emailTemplateBox').remove();
        $('.sfEBComplexAddSearchObjectChoiceDiv').hide();
    });

    }

/**
 * Renders already selected complex add objects into the view, if none chosen then runs the default suggestions procedure
 * @param {} obj 
 * @returns {} 
 */
function RenderSelectedObjectsForComplexAdd(obj, subjectTextBox) {
    if (obj.ComplexAddClosure != null) {
        var complexAddValues = obj.ComplexAddClosure.GetComplexAddValues();
        if (complexAddValues != null && complexAddValues.ComplexAddWhoAndWhatIds != null && complexAddValues.ComplexAddWhoAndWhatIds.length > 0) {
            complexAddValues.ComplexAddWhoAndWhatIds.forEach(function(element) {
                if (element != null && (element.WhoId != null || element.WhatId != null)) {
                    CreateNewComplexAddRow(element.WhoId, element.WhoName, element.WhoRecordType, element.WhoEmail, element.WhatId, element.WhatRecordName, element.WhatRecordType, true, element.GmailAttachmentIds);
                }
            });

            if (!obj.ComplexAddClosure.IsComplexAddValuesEmpty()) {
                subjectTextBox.value = obj.ComplexAddClosure.GetSubject();
            }
        } else {
            RunSuggestionsOnComplexAddLoad(obj);
        }
    } else {
        RunSuggestionsOnComplexAddLoad(obj);
    }

    CreateNewComplexAddRow();
}

/**
 * Checks local storage to see if we have any results matching the email addresses passed in, 
 * if so appends them onto the complex add suggestions
 * @param {} obj 
 * @returns {} 
 */
function RunSuggestionsOnComplexAddLoad(obj) {
    CreateNewComplexAddRow();

    chrome.extension.sendMessage({
        MessageType: "GetLocalStorageEmailSearchResult"
    }, function (response) {
        var wasPrimaryEmailFound = false;

        if (response && IsValidJSON(response.json)) {
            var json = JSON.parse(response.json);

            for (var j = 0; j < json.d.Values.length; j++) {
                var parsedJson = JSON.parse(json.d.Values[j]);

                var emailFromJson = "";
                if (parsedJson["Email-0"] != null) {
                    emailFromJson = parsedJson["Email-0"];
                }

                //Only continue if the email address found is not blank as blanks will match and results will appear in complex add which would be wrong.
                if (emailFromJson.length == 0) {
                    continue;
                }

                var objectType = parsedJson.Type;
                var name = parsedJson.Name;
                var whoId = parsedJson.Id;

                var emailaddress = obj.Id;

                //Check that we have an email first - may open add before entering recipient.
                if (emailaddress != null && typeof emailaddress != 'undefined') {
                    //email entered can be mixed case - make sure we match either way
                    if (emailaddress.toLowerCase() == emailFromJson.toLowerCase()) {
                        wasPrimaryEmailFound = true;

                        var whatRecordId = null, whatRecordName = null, whatRecordType = null;

                        //If we've got some history of a 'Related To' object for this record then lets pre-populate the Related To box instantly
                        var result = LookForRelatedToObject(whoId);

                        if (result) {
                            if (result.WhatRecordId) {
                                whatRecordId = result.WhatRecordId;
                            }

                            if (result.WhatRecordName) {
                                whatRecordName = result.WhatRecordName;
                            }

                            if (result.WhatRecordType) {
                                whatRecordType = result.WhatRecordType;
                            }
                        }

                        //If we didn't find any previous selection for this object we'll default it to the associated Account (if there is any)
                        if (whatRecordId == null) {
                            //Find Account Details from search results
                            if (parsedJson["Account"] != null) {
                                if (parsedJson["Account"].Id != null) {
                                    whatRecordId = parsedJson["Account"].Id;
                                }
                                if (parsedJson["Account"].Name != null) {
                                    whatRecordName = parsedJson["Account"].Name;
                                }

                                whatRecordType = "Account";
                            }
                        }

                        CreateNewComplexAddRow(whoId, name, objectType, emailaddress, whatRecordId, whatRecordName, whatRecordType);
                        CreateNewComplexAddRow();
                        break;
                    }
                }
            }
        }

        if (!wasPrimaryEmailFound) {
            CreateNewComplexAddRow();
            obj.SecondaryEmailSearch.push(obj.EmailText);
        }

        new ComplexAdd().CheckForLeads();

        CreateContactSuggestions(obj);

    });
}

    //Takes a whoid and finds if there is a record matching in the listOfRelatedToHistory variable which is the RelatedToSelections localstorage then returns an object with associated values
    function LookForRelatedToObject(whoId)
    {
    var whatRecordId = null, whatRecordName = null, whatRecordType = null;

    if (listOfRelatedToHistory)
    {
        var parseJsonRelatedToHistory = JSON.parse(listOfRelatedToHistory);

        for (var i = 0; i < parseJsonRelatedToHistory["relatedToObjects"].length; i++) {
            if (parseJsonRelatedToHistory["relatedToObjects"][i].whoId != null) {
                if (parseJsonRelatedToHistory["relatedToObjects"][i].whoId == whoId) {
                    if (parseJsonRelatedToHistory["relatedToObjects"][i].whatId) {
                        whatRecordId = parseJsonRelatedToHistory["relatedToObjects"][i].whatId;
                    }

                    if (parseJsonRelatedToHistory["relatedToObjects"][i].whatName) {
                        whatRecordName = parseJsonRelatedToHistory["relatedToObjects"][i].whatName;
                    }

                    if (parseJsonRelatedToHistory["relatedToObjects"][i].whatRecordType) {
                        whatRecordType = parseJsonRelatedToHistory["relatedToObjects"][i].whatRecordType;
                    }

                    break;
                }
            }
        }
    }

    return { WhatRecordId: whatRecordId, WhatRecordName: whatRecordName, WhatRecordType: whatRecordType };
    }


    //generate a random id for an email. //currently parked and unused.
    function GenerateEmailId()
    {
    function s4() {
        return Math.floor((1 + Math.random()) * 0x10000)
          .toString(16)
          .substring(1);
    }

    return 'id-' + s4() + s4();
    }

    ////REPLACE FOR MESSAGE ID SOLUTION.
    //grabs the id of a sent email. currently parked and unused.
    function GetSentEmailId()
    {
    var sentEmailId = '';
    
    var linkContainer = $("#link_vsm");

    //LogThis(linkContainer);

    if(linkContainer != null && typeof linkContainer != "undefined")
    {
        if (linkContainer.attr("param") != null && typeof linkContainer.attr("param") != "undefined") {
            sentEmailId = linkContainer.attr("param");
        }
    }

    //LogThis('sent email id: ' + sentEmailId);

    return sentEmailId;
    }

    // called when gmail send button has been clicked (not the "Send+Add to SF" btn)
    function LoggingEmailSend() {

    //console.time('LoggingEmailSend');
    //Check if tracking is selected in the current add compose screen
    var currentTracking = false;
    if ($("*[current-send*='true'").closest('.iN').find('.sfEbstaGmailEmailTrackingSelected').length > 0) {
        currentTracking = true;
    }

    //If it is log tracking
    if (currentTracking) {
        //Get the template id for the current add compose screen
        var currentTemplateId = "";
        if ($("*[current-send*='true'").closest('.iN').find('.ebstaTemplateObject').attr('ebstatemplateid')) {
            currentTemplateId = $("*[current-send*='true'").closest('.iN').find('.ebstaTemplateObject').attr('ebstatemplateid');
        }
        //Log the event
        chrome.extension.sendMessage({
            MessageType: "LogEmailSent", TrackingLogId: "" + lastTrackingIdInjected, TemplateId: "" + currentTemplateId
        }, function (response) {

        });
    }

    //console.timeEnd('LoggingEmailSend');
    }

    function RemoveGmailTrackingLog ()
    {
        //Only do this if EmailTracking is enabled and if the user currently has it switched on
        if (abRules.EnableGmailEmailTracking + "" == "1" && lastTrackingChoice) {
            if (lastTrackingIdInjected != null && lastTrackingIdInjected != "") {
                    chrome.extension.sendMessage({
                        MessageType: "DeleteEmailTrackingEntry", TrackingLogId: "" + lastTrackingIdInjected
                    }, function (response) {

                    });
                }
            }
    }
    
/**
 * Sets a fields value for the additional fields we display in the top section of Complex Add
 * @param {} inputElement 
 * @param {} savedField 
 * @returns {} 
 */
function ProcessComplexAddFieldValue(inputElement, savedField) {
    var fieldType = savedField.FieldType;
    var fieldValue = savedField.FieldValue;

    if (fieldType != null && fieldValue != null) {
        if (fieldType == "multipicklist") {
            $(inputElement).val(fieldValue.split(';'));
            $(inputElement).trigger('chosen:updated');
        }

        if (fieldType == "picklist") {
            inputElement.value = fieldValue;
            $(inputElement).trigger('chosen:updated');
        }

        if (fieldType == "date" || fieldType == "datetime") {
            var splitDateTime = fieldValue.split("-");
            if (splitDateTime.length > 2) {

                //Reformat date back so it calibrates with the date calendar plugin
                var dateParsed = new Date();
                dateParsed.setFullYear(splitDateTime[0]);
                dateParsed.setMonth(splitDateTime[1]);
                dateParsed.setDate(splitDateTime[2].substring(0,2));

                if (fieldType == "date") {
                    inputElement.value = dateParsed.getDate() + "/" + dateParsed.getMonth() + "/" + dateParsed.getFullYear();
                } else if (fieldType == "datetime" && splitDateTime.length === 4) {
                    inputElement.value = dateParsed.getDate() + "/" + dateParsed.getMonth() + "/" + dateParsed.getFullYear() + " " + splitDateTime[3];
                }
            }
        }

        if (fieldType == "reference") {
            var recordType = savedField.ReferenceRecordType;
            var recordId = fieldValue;
            var recordName = savedField.ReferenceRecordName;

            var crossIcon = CreateCrossIcon();

            if (recordType.toLowerCase() == "opportunity") {
                recordType = "Opp";
            }

            var newlySelectedDiv = CreateComplexAddObjectDiv(CheckStringLength(recordName, 37), recordType, "CABoxStyle sfSuggest" + recordType, crossIcon);
            newlySelectedDiv.setAttribute("recordId", recordId);
            newlySelectedDiv.setAttribute("recordType", recordType);
            newlySelectedDiv.setAttribute("recordName", recordName);

            SetClickListenerOnRemoveResult(crossIcon);

            //Hide Elements
            inputElement.childNodes.forEach(function(element) {
                HideElement(element);
            });

            inputElement.appendChild(newlySelectedDiv);
        }

        if (fieldType == "boolean") {
            inputElement.checked = savedField.FieldValue == "true";
        }
    }
}

/**
 * Finds all extra complex add fields and checks if a value is already set previously, if so then sets the field to have those values
 * @param {} complexAddObject 
 * @param {} complexAddDiv 
 * @returns {} 
 */
function SetPreSelectedValuesInComplexAddCustomFields(complexAddObject, complexAddDiv) {
    if (complexAddObject != null && complexAddObject.ComplexAddClosure != null) {
        var complexAddValues = complexAddObject.ComplexAddClosure.GetComplexAddValues();
        if (complexAddValues != null && complexAddValues.ComplexAddAdditionalFields != null) {

            //Find all the input fields
            var extraFieldInputs = [];
            extraFieldInputs = Array.prototype.concat.apply(extraFieldInputs, complexAddDiv.getElementsByTagName("input"));
            //Add the extra chosen dropdown elements
            extraFieldInputs = Array.prototype.concat.apply(extraFieldInputs, complexAddDiv.getElementsByClassName("chosen-select"));
            //Add search controllers too
            extraFieldInputs = Array.prototype.concat.apply(extraFieldInputs, complexAddDiv.getElementsByClassName("ebCustomFieldSearchContainer"));

            if (extraFieldInputs.length > 0) {
                complexAddValues.ComplexAddAdditionalFields.forEach(function (savedField) {
                    Array.prototype.forEach.call(extraFieldInputs, function (inputElement) {
                        if (inputElement.getAttribute("apiname") == savedField.FieldName && savedField.FieldName != null) {
                            inputElement.value = savedField.FieldValue;
                            ProcessComplexAddFieldValue(inputElement, savedField);
                            return true;
                        }
                    });
                });
            }
        }
    }
}

//Detects if a user has Task Types and includes a field so they cant set this value
function GetComplexAddSettings(complexAddObject, complexAddDiv) {
    chrome.extension.sendMessage({
        MessageType: "GetComplexAddSettings"
    }, function (response) {

        //console.log("GetComplexAddSettings()");
        //console.log(response);

        //If there are custom fields for the complex add screen then create the fields, if not create a default one for task type
        if (response.CustomComplexAddSetup != null)
        {
            CreateCustomFieldsForComplexAdd(response.CustomComplexAddSetup);
        }
        else
        {
            if (response.result.length > 0) {
                CreateDefaultTaskTypeBoxForComplexAdd(response.result);
            }
        }

        SetPreSelectedValuesInComplexAddCustomFields(complexAddObject, complexAddDiv);

        if (response.ComplexAddSearchableObjects && response.ComplexAddSuggestions) {
            CreateComplexAddSearchableObjectsMenu(response.ComplexAddSearchableObjects, response.ComplexAddSuggestions);
        }

    });
    }

    //Create the default Task Type dropdown for un-customised Complex Add screens
    function CreateDefaultTaskTypeBoxForComplexAdd(result)
    {
    var arrayOfTaskTypes = [];

    arrayOfTaskTypes.push("");

    //loop through result and append onto new array
    for (var i = 0; i < result.length; i++) {
        arrayOfTaskTypes.push(result[i].value);
    }

    //Create drop down box
        var dropDownDiv = CreateDropDown("sfTaskTypeDropDown chosen-select", arrayOfTaskTypes);
        dropDownDiv.id = "EbstaDdComplexAddTaskType";
        dropDownDiv.setAttribute("apiname", "Type");

    var row = CreateRow("sfComplexAddExtraFields");
    var txtDiv = CreateDiv("Type", "sfTaskType");
    row.appendChild(txtDiv);
    row.appendChild(dropDownDiv);

    //Insert after the subject row
    $('.sfSubjectRow').after(row);

    SetComplexAddCustomFieldsValueToEmail();

    //Use the included Chosen jquery scropt to create our custom drop down box and include width param and the ability to clear selection
    $(".chosen-select").chosen({ width: "40%" , allow_single_deselect: true});
    }

//Creates the custom fields for Complex Add Screen if the user has chosen to customise
function CreateCustomFieldsForComplexAdd(customComplexAddObject) {
    var row = CreateRow("sfComplexAddExtraFields");

    //Error Message when required fields haven't been selected
    var errorMessageSpan = CreateElement("span", "ebsta-alert ebsta-alert-danger sfComplexAddExtraFieldsInputError", "Please select values for fields marked red");
    HideElement(errorMessageSpan);
    row.appendChild(errorMessageSpan);

    if (customComplexAddObject.CustomFields != null) {
        var columnDiv = CreateDiv("", "col-xs-6");
        var containerRow = CreateRow("sfComplexAddExtraFieldsRow");

        for (var i = 0; i < customComplexAddObject.CustomFields.length; i++) {
            var formGroupDiv = CreateDiv("", "ebsta-form-group");

            //Create Name Label
            if (customComplexAddObject.CustomFields[i].Name != null) {
                var textLabel = CreateLabel(customComplexAddObject.CustomFields[i].Name, "ebsta-label");
                formGroupDiv.appendChild(textLabel);
            }

            //Create Input/Select elements
            if (customComplexAddObject.CustomFields[i].TypeOfField != null) {
                //console.time("Processfield" + customComplexAddObject.CustomFields[i].TypeOfField);
                var processedDiv = ProcessCustomComplexAddField(customComplexAddObject.CustomFields[i], customComplexAddObject.DropDownValues);
                //console.timeEnd("Processfield" + customComplexAddObject.CustomFields[i].TypeOfField);
                formGroupDiv.appendChild(processedDiv);
            }

            columnDiv.appendChild(formGroupDiv);
            containerRow.appendChild(columnDiv);
            row.appendChild(containerRow);

            //Create a new row if this happens to be the 2nd element of the row
            if (i % 2 == 1) {
                containerRow = CreateRow("sfComplexAddExtraFieldsRow");
            } else {
                //Set Left Column class
                columnDiv.setAttribute("class", "col-xs-6 left-col");
            }

            var columnDiv = CreateDiv("", "col-xs-6");
        }
    }

    $('.sfSubjectRow').after(row);

    SetComplexAddCustomFieldsValueToEmail();

    //Use the included Chosen jquery scropt to create our custom drop down box and include width param and the ability to clear selection
    //console.time("SetChosenFields");
    $(".chosen-select").chosen({ width: "97%", allow_single_deselect: true });
    //console.timeEnd("SetChosenFields");

    //Set Calendars Up using our the CibulCalendar plugin
    //console.time("SetDateTimeFields");
    $('input[sffieldtype=datetime],input[sffieldtype=date]').each(function() {
        setCibulCalendar($(this).attr('id'), { range: false });
    });
    //console.timeEnd("SetDateTimeFields");
}

//For each drop downlist in the complex add screen if there is a value which is some type of word relevant to 'Email, E-mail'
    function SetComplexAddCustomFieldsValueToEmail()
    {
    //If there is a value with email within it
    $('.chosen-select').on('chosen:ready', function (evt, params)
    {
        //Find default value for this field if there is one
        var defaultVal = $(this).attr('defaultval');

        $(this).find('option').each(function ()
        {
            var value = $(this).val();

            if (value != null)
            {
                //If this value is the value being set for this then select it
                if (defaultVal) {
                    if (value == defaultVal) {
                        $(this).attr('selected', 'selected');
                        $(".chosen-select").trigger("chosen:updated");
                        return false;
                    }
                }

                //Replace non letters
                var replacedValued = value.replace(/\W/g, '').toLowerCase();

                //Choose the first value that contains 'email'
                if (value.replace(/\W/g, '').toLowerCase() == "email")
                {
                    $(this).attr('selected', 'selected');
                    $(".chosen-select").trigger("chosen:updated");
                    return false;
                }
            }
        });
    });
    }

/**
 * Verifys the inputs of the custom fields within Complex Add
 * @returns {} 
 */
function VerifyCustomFieldInputs(saveComplexAddButton) {
    var returnValue = true;    
    if (saveComplexAddButton != null) {
        var complexAddDiv = saveComplexAddButton.closest("#ebstaBox");
        if (complexAddDiv != null) {

            //Find all custom fields in complex add
            var extraFieldInputs = [];
            extraFieldInputs = Array.prototype.concat.apply(extraFieldInputs, complexAddDiv.getElementsByTagName("input"));
            //Add the extra chosen dropdown elements
            extraFieldInputs = Array.prototype.concat.apply(extraFieldInputs, complexAddDiv.getElementsByClassName("chosen-select"));
            //Add search controllers too
            extraFieldInputs = Array.prototype.concat.apply(extraFieldInputs, complexAddDiv.getElementsByClassName("ebCustomFieldSearchContainer"));

            if (extraFieldInputs.length > 0) {
                Array.prototype.forEach.call(extraFieldInputs, function (element) {
                    if (element.hasAttribute("required")) {
                        if (!CheckCustomFieldHasValue(element)) {
                            returnValue = false;
                        }
                    }
                });
            }

            //Show Error Message
            if (!returnValue) {
                var errorSpan = complexAddDiv.getElementsByClassName("sfComplexAddExtraFieldsInputError");
                ShowElement(errorSpan[0]);
            }
        }
    }

    return returnValue;
}

/**
 * Checks that the custom field has a value if not returns false
 * @param {} element 
 * @returns {} 
 */
function CheckCustomFieldHasValue(element) {
    if (element.tagName != null && element.tagName.toLowerCase() === "input") {
        if (element.value === "") {
            CreateErrorBorderAndErrorCrossForInputDiv(element);
            return false;
        } else {
            RemoveErrorBorderAndErrorCrossForInputDiv(element);
            return true;
        }
    }

    //Check Picklists and MultiPicklists
    if (HasClass(element, "chosen-select"))
    {
        if (!$(element).val()) {
            AddClass(element.nextSibling, "chosen-ebsta-error");
            return false;
        } else {
            RemoveClass(element.nextSibling, "chosen-ebsta-error");
            return true;
        }
    }

    //Lookup Fields
    if (HasClass(element, "ebCustomFieldSearchContainer")) {
        var findChosenObject = element.getElementsByClassName("CABoxStyle");
        if (findChosenObject.length == 0) {
            AddClass(element, "ebsta-error");
            return false;
        } else {
            RemoveClass(element, "ebsta-error");
            return true;
        }
    }

    return true;
}

//Takes an object (with its values if applicable) and creates the div depending on its type for custom field in complex add
function ProcessCustomComplexAddField(fieldObject, dropDownValues) {
    var fieldType = fieldObject.TypeOfField;
    var inputDiv;

    if (fieldType == "multipicklist") {
        for (var i = 0; i < dropDownValues.length; i++) {
            if (dropDownValues[i].ApiName != null) {
                if (dropDownValues[i].ApiName == fieldObject.ValueField) {
                    inputDiv = CreatePicklistForm(dropDownValues[i].Values, true, "");
                    inputDiv.value = "";
                }
            }
        }
    }

    if (fieldType == "picklist") {
        for (var i = 0; i < dropDownValues.length; i++) {
            if (dropDownValues[i].ApiName != null) {
                if (dropDownValues[i].ApiName == fieldObject.ValueField) {
                    var defaultValue = "";

                    //Find the default value for this dropdown if one exists
                    if (dropDownValues[i].DefaultValue) {
                        defaultValue = dropDownValues[i].DefaultValue;
                    }

                    inputDiv = CreatePicklistForm(dropDownValues[i].Values, false, defaultValue);
                }
            }
        }
    }

    if (fieldType == "date" || fieldType == "datetime") {
        inputDiv = CreateTxtbox("", "ebsta-form-control", "sf" + fieldType + fieldObject.ValueField);
    }

    if (fieldType == "reference") {
        var dataSource = fieldObject.DataSource;
        inputDiv = CreateTextBoxForCustomFieldComplexAddLookup(dataSource);
    }

    if (fieldType == "boolean") {
        inputDiv = CreateCheckbox("ebsta-checkbox");
    }

    if (inputDiv == null) {
        inputDiv = CreateTxtbox("", "ebsta-form-control", "");
    }

    inputDiv.setAttribute('sfFieldType', fieldType);
    inputDiv.setAttribute('apiname', fieldObject.ValueField);
    //Set require attribute on field so we can verify these exists before letting a user add it
    if (fieldObject.Required === true) {
        inputDiv.setAttribute("required","");
    }

    CreatePopOverHelperError();
    AttachChangeListenerOnComplexAddCustomFields(inputDiv, fieldType);
    AttachHoverOverListenerOnComplexAddCustomFields(inputDiv, fieldType);

    return inputDiv;
}

//Creates an Ebsta pop over helper
    function CreatePopOverHelperError()
    {
    if ($('.ebsta-popover').length == 0) {
        var popOverDiv = CreateDiv("", "ebsta-popover bottom");

        var arrowDiv = CreateDiv("", "ebsta-arrow");
        popOverDiv.appendChild(arrowDiv);

        var contentDiv = CreateDiv("", "ebsta-popover-content");
        popOverDiv.appendChild(contentDiv);

        $('body').append(popOverDiv);
    }
    }

    //Applies a hover over for all custom fields in complex add and displays an error message associated with fields that have bad input
    function AttachHoverOverListenerOnComplexAddCustomFields(inputDiv, fieldType) {

    $(inputDiv).hover(function (handlerIn) {
        if ($(inputDiv).closest('.ebsta-has-error').length > 0) {
            var errorMsg = GetErrorMessageForFieldType(fieldType);
            $('.ebsta-popover-content').text(errorMsg);

            //Find current div's position so we can position the hover over correctly
            var currentPos = $(inputDiv).offset();

            $('.ebsta-popover').css({
                "top": currentPos.top + $(inputDiv).outerHeight(),
                "left": currentPos.left
            });

            $('.ebsta-popover').show();
        }
    },
    function (handlerOut) {
        $('.ebsta-popover').hide();
    });
    }

//Takes a div and message and displays the popover div when a user hover overs the div
function AttachHoverOverOnDivToShowEbstaPopUpMessage(div, displayText) {

    CreatePopOverHelperError();

    $(div).hover(function (handlerIn) {
        var thisObj = $(this);
        $('.ebsta-popover-content').text(displayText);
        $('.ebsta-popover').show();

        //Show it  off screen so we can pick up the width
        $('.ebsta-popover').css({
            "top": -1000,
            "left": -1000
        });

        //Find out if margin or padding exsits on these div to help offset the location of the popover
        var marginPaddingWidths = parseInt(thisObj.css('margin-left')) + parseInt(thisObj.css('padding-left'));
        var marginPaddingHeights = parseInt(thisObj.css('margin-top')) + parseInt(thisObj.css('padding-top'));
            
        //Find current div's position so we can position the hover over correctly
        var currentPos = $(div).offset();

        $('.ebsta-popover').css({
            "top": currentPos.top + marginPaddingHeights + thisObj.outerHeight(),
            "left": currentPos.left - ($('.ebsta-popover-content').outerWidth() / 2) + (thisObj.width() / 2) + marginPaddingWidths
        });
    },
    function (handlerOut) {
        $('.ebsta-popover').hide();
    });
}

//Returns an error message relating to field type for Complex Add custom fields
function GetErrorMessageForFieldType(fieldType) {
    switch (fieldType) {
        case "double":
        case "currency":
            return "Please enter a valid number i.e. 10.2/7.0";

        case "email":
            return "Please enter a valid email address i.e. joebloogs@gmail.com";

        case "date":
            return "Please enter a valid Date (dd/mm/yyyy) i.e 30/07/2015";

        case "datetime":
            return "Please enter a valid Date and Time (dd/mm/yyyy hh:mm) i.e 30/07/2015 12:15";
        
        default:
            return "There is an error with this field. Please input an acceptable value for this " + fieldType + " field";
    }
    }

    //Attach a value change listener to the custom fields within complex add so we can detect any issues or change data where needs be
    function AttachChangeListenerOnComplexAddCustomFields(inputDiv, fieldType)
    {
    inputDiv.addEventListener("change", function () {

        var addErrorBorder = false;
        if (this.value != null)
        {
            //Append the time onto the date if not already there as out date picker does not currently give us a time back
            if (fieldType == "datetime") {
                var regex = /^[0-9]{2}\/[0-9]{2}\/[0-9]{4}$/g;
                regexMatch = regex.exec(this.value);
                if (regexMatch) {
                    this.value = this.value + " 00:00";
                }
            }

            //Check if this value entered is good or bad
            if (this.value.length > 0) {
                //console.time("ValidateCustomFieldInput");
                if (!ValidateCustomFieldInput(fieldType, this.value)) {
                    addErrorBorder = true;
                }
                //console.timeEnd("ValidateCustomFieldInput");
            }
        }        

        //If the value is bad then put a red border with a red cross to say there's an issue with the field
        if (addErrorBorder)
        {
            CreateErrorBorderAndErrorCrossForInputDiv(inputDiv);
            return true;
        }

        RemoveErrorBorderAndErrorCrossForInputDiv(inputDiv);
    });
    }

    //Creates the error border for the custom fields for when bad values are apparent
    function CreateErrorBorderAndErrorCrossForInputDiv(inputDiv)
    {
    $(inputDiv).parent().addClass("ebsta-has-feedback").addClass("ebsta-has-error");

    //if the glyphyicon is already there just show it rather than recreating it
    if ($(inputDiv).parent().find('.ebsta-glyphicon').length > 0)
    {
        $(inputDiv).parent().find('.ebsta-glyphicon').show();
    }
    else
    {
        var crossIcon = CreateSpan("", "ebsta-glyphicon ebsta-glyphicon-crossnocircle ebsta-form-control-feedback");
        $(inputDiv).after(crossIcon);
    }
    }

    //Removes the error border for the custom fields
    function RemoveErrorBorderAndErrorCrossForInputDiv(inputDiv) {
    $(inputDiv).parent().removeClass("ebsta-has-feedback").removeClass("ebsta-has-error");
    $(inputDiv).parent().find('.ebsta-glyphicon').hide();
    }

    //Validates the values in custom fields for complex add
    function ValidateCustomFieldInput(fieldType, value)
    {
    var regexMatch;

    switch (fieldType) {
        case "double":
        case "currency":
            var regex = /^\d{0,2}(\.\d*){0,1}$/g;
            regexMatch = regex.exec(value);
            break;
        
        case "email":
            return ValidateEmail(value);

        case "date":
            var regex = /^[0-9]{2}\/[0-9]{2}\/[0-9]{4}$/g;
            regexMatch = regex.exec(value);
            break;

        case "datetime":
            var regex = /^[0-9]{2}\/[0-9]{2}\/[0-9]{4}\s[0-9]{2}:[0-9]{2}$/g;
            regexMatch = regex.exec(value);
            break;

        default:
            return true;
    }

    if (regexMatch != null) {
        if (regexMatch.length > 0) {
            return true;
        }
    }

    return false;
    }

    //Creates the input box for reference fields so a user can search for the object they need
    function CreateTextBoxForCustomFieldComplexAddLookup(dataSource)
    {
    var customInputDiv = CreateDiv("", "ebCustomFieldSearchContainer");

    var textBox = CreateTxtbox("", "ebsta-form-control");
    textBox.setAttribute("placeholder", "Search " + dataSource);
    customInputDiv.appendChild(textBox);

    var searchButton = CreateSpan("", "magnifySearchIcon", "");

    TriggerEventOnEnter(textBox);

    //Set Click event on magnify icon
    searchButton.addEventListener("click", function () {
        //Get Value from field
        var textBoxVal = textBox.value;

        //Only fire off search for 2 characters or more
        if ($(this).hasClass('magnifySearchIcon')) {
            if (textBoxVal != null) {
                if (textBoxVal.length > 1) {
                    var apiName = customInputDiv.getAttribute('apiname');
                    RunSearchLookup(dataSource + '-ComplexAdd-CustomFieldSearch-ApiName=' + apiName, textBoxVal);

                    $(this).closest('.ebsta-form-group').find('.resultsLoader, .ebNoResult').remove();

                    var loadingResults = CreateLoadingGifDiv()
                    $(customInputDiv).after(loadingResults);

                    $(this).removeClass('magnifySearchIcon').addClass('crossSearchIcon');
                } else {
                    textBox.setAttribute("placeholder", "Please enter at least 2 characters");
                    textBox.value = "";
                }
            }
        } else {
            textBox.value = "";
            $(this).closest('.ebsta-form-group').find('.resultsWindow').slideUp();
            $(this).removeClass('crossSearchIcon').addClass('magnifySearchIcon');
        }
    });

    customInputDiv.appendChild(searchButton);

    return customInputDiv;
    }

    //Creates a loading gif
    function CreateLoadingGifDiv() {
    var loadingResults = CreateDiv("Loading results... ", "sfLoadingResults");
    loadingResults.className = 'resultsLoader';
    var loadingGif = CreateImg(chrome.extension.getURL("/images/ajax-loader.gif"), "Loading...");
    loadingResults.appendChild(loadingGif);
    return loadingResults;
    }

    //Creates a dropdown for both single and multiple selections
    function CreatePicklistForm(arrayOfValues, isThisAMultiPicklist, defaultValue)
    {
    var selectDiv = CreateElement('select', 'chosen-select', "", "");

    if (arrayOfValues != null)
    {
        //Insert a blank value so the users doesn't have to select a value
        if (!isThisAMultiPicklist) {
            arrayOfValues.unshift({ Name: "" });
        }

        for (var i = 0; i < arrayOfValues.length ; i++)
        {
            if (arrayOfValues[i].Name != null) 
            {
                var optionDiv = CreateElement("option", "", arrayOfValues[i].Name, "");
                selectDiv.appendChild(optionDiv);
            }
        }
    }

    if (isThisAMultiPicklist) {
        selectDiv.setAttribute("multiple", "");
    }

    selectDiv.setAttribute("defaultval", defaultValue);

    return selectDiv;
    }

    //Tracks user clicks for the rows
    function AddListenerForComplexAddRow(obj, isThisForTemplates) {
    obj.addEventListener("click", function (event)
    {
        //Text Box Clicks
        if ($(this).attr('class').indexOf("CAObjectContainer") != -1)
        {
            if (!$(this).closest('.sfAddComplexRow').hasClass("sfCurrentSelectedRow"))
            {
                //Reset current row
                $(this).closest('#sfComplexAddRowsContainer').find('.sfAddComplexRow').each(function ()
                {
                    $(this).attr("currentSelectedRow", "false");

                    $(this).removeClass("sfCurrentSelectedRow");

                    //Toggle between hide/show
                    $(this).find('.sfRowWhatSuggest').each(function () {
                        $(this).slideUp();
                    });                    

                    //Toggle between hide/show
                    $(this).find('.resultsWindow').each(function () {
                        $(this).slideUp();
                    });

                    //Attachment Row
                    $(this).find('.sfAttachRow').each(function () {
                        $(this).slideUp();
                    });

                    //Reset Arrow
                    $(this).find('.sfArrowDown').each(function () {
                        $(this).removeClass("sfArrowDown"); //remove the down arrow.
                        $(this).addClass("sfArrowRight");
                    });
                });

                //Set Arrow to down for current row selected row
                if ($(this).closest('.sfAddComplexRow').find('.resultsWindow, .sfRowWhatSuggestRight').find('.CABoxStyle, .sfNoSuggestCaseFound').length > 0) {
                    $(this).closest('.sfAddComplexRow').find('.sfArrowRight').removeClass("sfArrowRight").addClass("sfArrowDown");
                }

                //Set the new current selected row
                $(this).closest('.sfAddComplexRow').attr("currentSelectedRow", "true").addClass("sfCurrentSelectedRow");

                var callScrollerAction = true;

                //Toggle between hide/show
                $(this).closest('.sfAddComplexRow').find('.sfRowWhatSuggest').each(function () {
                    callScrollerAction = false;
                    $(this).slideToggle("1000", function () {
                        ScrollToPosition($(this));
                        
                    });
                });

                //Results Windows
                $(this).closest('.sfAddComplexRow').find('.resultsWindow').each(function () {
                    callScrollerAction = false;
                    $(this).slideToggle("1000", function () {
                        ScrollToPosition($(this));
                        
                    });
                });

                //Attachment Row
                $(this).closest('.sfAddComplexRow').find('.sfAttachRow').each(function () {
                    callScrollerAction = false;
                    $(this).slideToggle("1000", function () {
                        ScrollToPosition($(this));
                    });
                });

                if (callScrollerAction) {
                    ScrollToPosition($(this).closest('.sfAddComplexRow').find('.SALeftSegment'));
                }

            }          

            $(this).closest('#sfComplexAddRowsContainer').find('.sfAddComplexRow').each(function () {
                $(this).find('.sfWhoSearch,.sfWhatSearch').each(function () 
                {
                    $(this).removeClass('sfGreyFocusBox');
                });
            });

            //Only if it's not a lead
            if (!$(this).hasClass("sfGreyOutBox"))
            {
                $(this).addClass('sfGreyFocusBox');
            }

            //Remove Grey Focus Box if this is for templates
            if ($(this).hasClass("sfGreyFocusBox") && $(this).closest(".sfEmailTemplateSearch").length) {
                $(this).removeClass("sfGreyFocusBox");
            }
        }

        //Arrow clicks
        if ($(this).attr('class').indexOf("sfArrow") != -1)
        {
            if (!$(this).closest('.sfAddComplexRow').hasClass("sfCurrentSelectedRow"))
            {
                //Reset all rows
                $(this).closest('#sfComplexAddRowsContainer').find('.sfAddComplexRow').each(function ()
                {
                    $(this).attr("currentSelectedRow", "false");

                    $(this).removeClass("sfCurrentSelectedRow");

                    //Toggle between hide/show
                    $(this).find('.sfRowWhatSuggest').each(function () {
                        $(this).slideUp();
                    });

                    //Results Window
                    $(this).find('.resultsWindow').each(function () {
                        $(this).slideUp();
                    });

                    //Attachment Row
                    $(this).find('.sfAttachRow').each(function () {
                        $(this).slideUp();
                    });

                    $(this).find('.sfArrowDown').each(function () {
                        if ($(this).attr("class").indexOf("sfArrowDown") != -1)  //it is already a down arrow.
                        {
                            $(this).removeClass("sfArrowDown"); //remove the down arrow.
                            $(this).addClass("sfArrowRight");
                        }
                    });
                });

                //If there are suggestions set the arrow down if not leave as right
                if ($(this).closest('.sfAddComplexRow').find('.resultsWindow, .sfRowWhatSuggestRight').find('.CABoxStyle, .sfNoSuggestCaseFound').length > 0) {
                    $(this).closest('.sfAddComplexRow').find('.sfArrowRight').removeClass("sfArrowRight").addClass("sfArrowDown");
                }
                
                //Set the new current selected row
                $(this).closest('.sfAddComplexRow').attr("currentSelectedRow", "true").addClass("sfCurrentSelectedRow");

                var callScrollerAction = true;

                //Toggle between hide/show
                $(this).closest('.sfAddComplexRow').find('.sfRowWhatSuggest').each(function () {
                    callScrollerAction = false;
                    $(this).slideToggle("1000", function () {
                        ScrollToPosition($(this));

                    });
                });

                //Results Windows
                $(this).closest('.sfAddComplexRow').find('.resultsWindow').each(function () {
                    callScrollerAction = false;
                    $(this).slideToggle("1000", function () {
                        ScrollToPosition($(this));

                    });
                });

                //Attachment Row
                $(this).closest('.sfAddComplexRow').find('.sfAttachRow').each(function () {
                    callScrollerAction = false;
                    $(this).slideToggle("1000", function () {
                        ScrollToPosition($(this));
                    });
                });

                if (callScrollerAction) {
                    ScrollToPosition($(this).closest('.sfAddComplexRow').find('.SALeftSegment'));
                }
                               
            }
            //When a user minimises the box
            else if ($(this).closest('.sfAddComplexRow').hasClass("sfCurrentSelectedRow"))
            {
                //Set Arrow to right for current row selected row
                //$(this).closest('.sfAddComplexRow').find('.sfArrowDown').removeClass("sfArrowDown").addClass("sfArrowRight");

                if ($(this).closest('.sfAddComplexRow').find('.resultsWindow, .sfRowWhatSuggestRight').find('.CABoxStyle, .sfNoSuggestCaseFound').length > 0)
                {
                    if ($(this).closest('.sfAddComplexRow').find('.sfArrow').hasClass('sfArrowDown'))
                    {
                        $(this).closest('.sfAddComplexRow').find('.sfArrowDown').removeClass("sfArrowDown").addClass("sfArrowRight");
                    }
                    else
                    {
                        $(this).closest('.sfAddComplexRow').find('.sfArrowRight').removeClass("sfArrowRight").addClass("sfArrowDown");
                    }
                }
                else
                {
                    $(this).closest('.sfAddComplexRow').find('.sfArrowDown').removeClass("sfArrowDown").addClass("sfArrowRight");
                }

                //Set the new current selected row
                $(this).closest('.sfAddComplexRow').attr("currentSelectedRow", "true").addClass("sfCurrentSelectedRow");

                var callScrollerAction = true;

                //Toggle between hide/show
                $(this).closest('.sfAddComplexRow').find('.sfRowWhatSuggest').each(function () {
                    callScrollerAction = false;
                    $(this).slideToggle("1000", function () {
                        ScrollToPosition($(this));

                    });
                });

                //Results Windows
                $(this).closest('.sfAddComplexRow').find('.resultsWindow').each(function () {
                    callScrollerAction = false;
                    $(this).slideToggle("1000", function () {
                        ScrollToPosition($(this));
                    });
                });

                //Results Windows
                $(this).closest('.sfAddComplexRow').find('.sfAttachRow').each(function () {
                    callScrollerAction = false;
                    $(this).slideToggle("1000", function () {
                        ScrollToPosition($(this));
                    });
                });

                if (callScrollerAction) {
                    ScrollToPosition($(this).closest('.sfAddComplexRow').find('.SALeftSegment'));
                }
            }
        }

        new ComplexAdd().CheckForLeads();
    });
    }


    var lastScrollTime = new Date();

    function ScrollToPosition(elem)
    {
    var fixBox = false;

    var dateNow = new Date();

    if (Math.abs(dateNow - lastScrollTime) > 1000) {
  
        lastScrollTime = new Date();

        if ($(elem).closest('.sfAddComplexRow').offset() != null && $(elem).closest('.sfAddComplexRow').parent() != null)
        {
            $('#ebstaBox').find('#sfComplexAddRowsContainer').animate({
                scrollTop: $(elem).closest('.sfAddComplexRow').parent().scrollTop() + $(elem).closest('.sfAddComplexRow').offset().top - $(elem).closest('.sfAddComplexRow').parent().offset().top
            },
            {
                duration: 500,
            });
        }

    }
    }

/**
 * Triggered from the Complex Add when a users presses Save/Add on Send
 * @param {object} obj - Contains details around the values chosen in the complex add window 
 */
function SaveComplexAddNew(obj) {
    
    //Get the string we'll use to log this out
    var eventLogString = GetComplexAddLoggingDetailsForRow($('#ebstaBox'), obj.ComplexAddClosure != null);

    //Lets get details for custom fields if there are any
    var customFields = GetCustomFieldValuesFromComplexAdd();

    var title = "";
    if (document.getElementById('txtSubject') != null) {
        title = document.getElementById('txtSubject').value;
    }

    // If we have a closure at this point we need to treat this as saving the 
    // complex add state rather than saving and adding
    if (obj.ComplexAddClosure) {
        obj.ComplexAddClosure.InitialiseComplexAddValues();
        obj.ComplexAddClosure.SetComplexAddAdditionalFields(customFields);
        obj.ComplexAddClosure.UpdateComplexAddButton(true);
        obj.ComplexAddClosure.SetComposeId(obj.ComposeId);
        obj.ComplexAddClosure.SetSubject(title);
        obj.ComplexAddClosure.SetLogEventString(eventLogString);
    } else {
        LogEvent(eventLogString);
    }
    
    //Look for all all visible rows
    $('#ebstaBox').find('.sfAddComplexRow').each(function(index) {

        //console.time('anon fn: #ebstaBox).find(.sfAddComplexRow).each(');
        var whatIds = "";
        var whoId = "";

        //New for gmail api
        var gmailAttachments = new Array();
        var gmailOrigialSubject = "";
        var gmailUserId = "";
        
        var name = "", whoEmailAddress = "";
        var whoRecordType = "";
        var whatRecordType = "", whatRecordName = "";

        //Find the WhoId
        if ($(this).attr("sfaddcomplexrowwhoid") != null) {
            whoId = $(this).attr("sfaddcomplexrowwhoid");
        }

        //Find the WhatId's
        if ($(this).attr("sfaddcomplexrowwhatids") != null) {
            var val = $(this).attr("sfaddcomplexrowwhatids");
            whatIds = val.split(",");
        }

        $(this).find('.sfWhoSearch').find('.sfSuggestContact,.sfSuggestLead').each(function() {
            name = $(this).attr("recordname");
            whoRecordType = $(this).attr("recordtype");
            whoEmailAddress = $(this).attr("email");
            return false;
        });

        //Looks for the Related To information added so it can be added to the RelatedToSelections sectionin LocalStorage later
        $(this).find('.sfWhatSearch').find('.CABoxStyle').each(function() {
            if ($(this).attr("recordtype")) {
                whatRecordType = $(this).attr("recordtype");
            }

            if ($(this).attr("recordname")) {
                whatRecordName = $(this).attr("recordname");
            }
        });

        //Find the AttachmentId's for checked attachments
        $(this).find('.SAAttachments').find('.sfAttachmentSelected .sfAttachmentCheckBox').each(function() {
            //Convert the client side attachment id ie. 0.1, 0.2 to id equvalent for API ie. 1,2
            var attachmentId = $(this).val();

            attachmentId = attachmentId.replace('0.', '');
            gmailAttachments.push("" + attachmentId);
        });

        gmailAttachments = gmailAttachments.join();

        //Find the gmail message id;
        const gmailMessageId = $(this).closest('.sfComplexAddContainer').attr('gmailmessageid');
        const immutableMessageId = $(this).closest(".sfComplexAddContainer").attr("newformatgmailmessageid");
        
        gmailOrigialSubject = obj.OriginalSubject;

        //Find the gmailUserId
        gmailUserId = GetGmailUsersEmailAddress();

        if (name == "") {
            $(this).find('.sfWhatSearch').find('.CABoxStyle').each(function() {
                name = $(this).attr("recordname");
                return false;
            });
        }

        //Find if a user has specified a task type
        var selectedTaskTypeValue = "";

        if ($('.sfTaskTypeDropDown').find(":selected").text().length > 0) {
            selectedTaskTypeValue = $('.sfTaskTypeDropDown').find(":selected").text();
        }

        //Remove emptys from whatids
        var whatidscopy = new Array();
        for (var i = 0; i < whatIds.length; i++) {
            if (whatIds[i] != "") {
                whatidscopy.push(whatIds[i]);
            }
        }

        //Stop here if we're just saving state of the complex add rather than straight off adding to salesforce
        if (obj.ComplexAddClosure != null) {
            var complexAddRowObject = obj.ComplexAddClosure.CreateComplexWhoAndWhatId(whoId, name, whoRecordType, whoEmailAddress, whatidscopy[0], whatRecordName, whatRecordType, gmailAttachments);
            obj.ComplexAddClosure.AddComplexAddRow(complexAddRowObject);

            //If their just using the standard type field make sure we pass this value up too!
            if (selectedTaskTypeValue && index < 1 && customFields.length === 0) {
                obj.ComplexAddClosure.SetComplexAddAdditionalFields(new Array({
                    FieldName: "Type",
                    FieldType: "picklist",
                    FieldValue: selectedTaskTypeValue
                }));
            }

            return;
        }

        //The core part of the message to send up
        var coreMessageObj = {
            MessageType: "AddGmailTask",
            Email: "NEW",
            EmailText: obj.EmailText,
            GmailUserId: GoogleTools.GetEmailAddress(),
            GmailMessageId: gmailMessageId,
            GmailImmutableId: immutableMessageId,
            GmailOriginalSubject: obj.OriginalSubject,
            GmailAttachmentIds: gmailAttachments,
            SaveOnSend: obj.SaveOnSend,
            Subject: title,
            TaskType: selectedTaskTypeValue,
            Name: name,
            WhatRecordType: whatRecordType,
            WhatRecordName: whatRecordName,
            CustomFields: customFields,
            AddType: "MessageView-ComplexAdd",
            RequestedFromUrl: document.location.href
        };

        //Loop through all whatids, if there are any, and add a record for each
        if (whatidscopy.length > 0) {
            for (var i = 0; i < whatidscopy.length; i++) {
                let whatId = whatidscopy[i];

                if (whoId == undefined || whoId == "undefined") {
                    whoId = -1;
                }

                if (whatId != "") {
                    coreMessageObj.WhoId = whoId;
                    coreMessageObj.WhatId = whatId;

                    chrome.extension.sendMessage(coreMessageObj);
                }
            }
        }
        else if (whoId != "" && whoId != undefined && whoId != "undefined") {
            coreMessageObj.WhoId = whoId;
            chrome.extension.sendMessage(coreMessageObj);
        }

        //Check if tracking is selected in the current complex add compose screen
        if (index == 0) {

            var currentTracking = false;
            if ($("*[current-complexadd*='true'").closest('.iN').find('.sfEbstaGmailEmailTrackingSelected').length > 0) {
                currentTracking = true;
            }

            //If it is log tracking
            if (currentTracking) {
                //Get the template id for the current complex add compose screen
                var currentTemplateId = "";
                if ($("*[current-complexadd*='true'").closest('.iN').find('.ebstaTemplateObject').attr('ebstatemplateid')) {
                    currentTemplateId = $("*[current-complexadd*='true'").closest('.iN').find('.ebstaTemplateObject').attr('ebstatemplateid');
                }
                //Log the event
                chrome.extension.sendMessage({
                    MessageType: "LogEmailSent",
                    TrackingLogId: "" + lastTrackingIdInjected,
                    TemplateId: "" + currentTemplateId
                }, function(response) {

                });
            }
        }
    });

    //Calls the function to update the send button within the compose window
    if (obj.FunctionToUpdateSendButtons) {
        obj.FunctionToUpdateSendButtons();
    }
}

//Gets details for all custom fields selected within Complex Add
function GetCustomFieldValuesFromComplexAdd() {
    //Create a custom object to tidy up fields we need to send when adding these fields
    var CustomFieldEntity = function (fieldName, fieldType, fieldValue) {
        this.FieldName = fieldName;
        this.FieldType = fieldType;
        this.FieldValue = fieldValue;
    };

    var arrayOfCustomFieldEntities = new Array();

    $('#ebstaBox > .sfComplexAddExtraFields .ebsta-form-group').find('select[sffieldtype], input[sffieldtype], div[sffieldtype]').each(function () {
        var fieldType = $(this).attr('sffieldtype');
        var fieldApiName = $(this).attr('apiname');
        var fieldValue = $(this).val();

        //Extra values needed for reference type fields
        var referenceRecordType = "", referenceRecordName = "";

        if (fieldType && fieldApiName) {
            if (fieldType == "picklist" || fieldType == "multipicklist") {
                fieldValue = $(this).chosen().val();
                //if this is a multipicklist then an array will be returned which we'll then need a send a list up like option 1;option3;option4
                if (fieldValue instanceof Array) {
                    var newfieldValue = fieldValue[0];
                    for (var i = 1 ; i < fieldValue.length; i++) {
                        newfieldValue += ";" + fieldValue[i];
                    }
                    fieldValue = newfieldValue;
                }
            }
            else if (fieldType == "date") {
                //Convert dd/mm/yyyy to yyyy/mm/dd for Salesforce purposes
                fieldValueSplit = fieldValue.split("/");
                if (fieldValueSplit.length > 2) {
                    fieldValue = fieldValueSplit[2] + "-" + fieldValueSplit[1] + "-" + fieldValueSplit[0];
                }
            }
            else if (fieldType == "datetime") {
                //Convert dd/mm/yyyy hh:mm to yyyy/mm/dd hh:mm for Salesforce purposes
                var time = fieldValue.match(/\s[0-9]{2}:[0-9]{2}/g);
                fieldValueSplit = fieldValue.replace(time, "").trim().split("/");
                if (fieldValueSplit.length > 2) {
                    fieldValue = fieldValueSplit[2] + "-" + fieldValueSplit[1] + "-" + fieldValueSplit[0] + time;
                }
            }
            else if (fieldType == "boolean") {
                //This is mainly for checkboxes
                if ($(this).is(':checked')) {
                    fieldValue = "true";
                }
                else {
                    fieldValue = "false";
                }
            }
            else if (fieldType == "reference") {
                fieldValue = $(this).find('.CABoxStyle').attr('recordid');
                //Look for addition values for the selected record (Currently used to help pre-selected save values)
                if ($(this)[0] != null) {
                    var referenceFieldRecord = $(this)[0].getElementsByClassName("CABoxStyle");
                    if (referenceFieldRecord != null && referenceFieldRecord.length > 0) {
                        var selectedRecordDiv = referenceFieldRecord[0];
                        referenceRecordType = selectedRecordDiv.getAttribute("recordtype");
                        referenceRecordName = selectedRecordDiv.getAttribute("recordname");;
                    }
                }
            }
        }

        if (fieldValue) {
            if (fieldValue.length > 0) {
                var newCustomFieldEntity = new CustomFieldEntity(fieldApiName, fieldType, fieldValue);
                //Check and add additional valus should they exists
                if (referenceRecordType != null) newCustomFieldEntity.ReferenceRecordType = referenceRecordType;
                if (referenceRecordName != null) newCustomFieldEntity.ReferenceRecordName = referenceRecordName;
                arrayOfCustomFieldEntities.push(newCustomFieldEntity);
            }
        }
    });
    return arrayOfCustomFieldEntities;
}

/**
 * Gets the log event string we'll use to report on this complex add row
 * @param {} ebstaBox 
 * @param {} saveOnSend 
 * @param {} noOfEmailsToAdd 
 * @returns {} 
 */
function GetComplexAddLoggingDetailsForRow(ebstaBox, saveOnSend, whatIdsSplit) {

    var map = {
        'RAT': 0, // Records Added Total
        'CN': 0, // Contacts 
        'LD': 0, // Leads
        'WHOS': 0, // How many of the whoid's were suggestions
        'AC': 0, // Accounts
        'CS': 0, // Cases
        'OP': 0, // Opportunities
        'AS': 0, // Assets
        'OR': 0, // Orders
        'SL': 0, // Solutions
        'CM': 0, // Campaigns
        'CT': 0, // Contracts
        'WHATS': 0, // How many of the whatid's were suggestions
        'ATT': 0 // How many attachments did the user choose to attach
    };

    var valueArray = ["RAT", "CN", "LD", "WHOS", "AC", "CS", "OP", "AS", "OR", "SL", "CM", "CT", "WHATS", "ATT"];
    var noOfEmailsBeingAdded = -1;

    //Get all the rows and find what objects are being addedd
    $(ebstaBox).find('.sfAddComplexRow').each(function() {
        //Remove emptys from whatids
        var whatidscopy = new Array();
        if (whatIdsSplit != null) {
            for (var i = 0; i < whatIdsSplit.length; i++) {
                if (whatIdsSplit[i] != "") {
                    whatidscopy.push(whatIdsSplit[i]);
                }
            }
        }

        var noOfWhats = 1;

        if (whatidscopy.length > 0) {
            noOfWhats = 0;
            for (var i = 0; i < whatidscopy.length; i++) {
                var whatId = whatidscopy[i];
                if (whatId != -1) {
                    noOfWhats++;
                }
            }
        }

        noOfEmailsBeingAdded += noOfWhats;

        map["RAT"] = noOfEmailsBeingAdded;

        $(this).find('.sfWhoSearch, .sfWhatSearch').find('.CABoxStyle').each(function() {
            var recordType = $(this).attr("recordtype");

            //Was this an initial who suggestion
            if ($(this).hasClass("sfInitSuggestion")) {
                map['WHOS'] = parseInt(map['WHOS']) + 1;
            }

            //Was this an initial what suggestion
            $(this).closest('.sfWhatSearch').find('.CABoxStyle').each(function() {
                if (!$(this).hasClass("result")) {
                    map['WHATS'] = parseInt(map['WHATS']) + 1;
                }
            });

            if (recordType.toLowerCase() == "contact") {
                map['CN'] = parseInt(map['CN']) + 1;
            } else if (recordType.toLowerCase() == "lead") {
                map['LD'] = parseInt(map['LD']) + 1;
            } else if (recordType.toLowerCase() == "account") {
                map['AC'] = parseInt(map['AC']) + 1;
            } else if (recordType.toLowerCase() == "case") {
                map['CS'] = parseInt(map['CS']) + 1;
            } else if (recordType.toLowerCase() == "opp") {
                map['OP'] = parseInt(map['OP']) + 1;
            } else if (recordType.toLowerCase() == "asset") {
                map['AS'] = parseInt(map['AS']) + 1;
            } else if (recordType.toLowerCase() == "order") {
                map['OR'] = parseInt(map['OR']) + 1;
            } else if (recordType.toLowerCase() == "solution") {
                map['SL'] = parseInt(map['SL']) + 1;
            } else if (recordType.toLowerCase() == "campaign") {
                map['CM'] = parseInt(map['CM']) + 1;
            } else if (recordType.toLowerCase() == "contract") {
                map['CT'] = parseInt(map['CT']) + 1;
            }
        });

        //Find Attachments and
        $(this).find('.sfAttachRow .sfAttachmentSelected').each(function() {
            map['ATT'] = parseInt(map['ATT']) + 1;
        });
    });

    var eventString = "Task-Add-Complex-";

    //Saveonsend or added a received email
    if (saveOnSend == 0)
    {
        eventString += "Received,";
    }
    else if (saveOnSend == 1)
    {
        eventString += "Sent,";
    }

    //Create String
    for (var i = 0; i < valueArray.length; ++i) {
        eventString += valueArray[i] + "=" + map[valueArray[i]] + ",";
    }

    return eventString;
}

//Logs an event to logger
function LogEvent(message) {
    chrome.extension.sendMessage({
        MessageType: "LogEventToDB",
        logEvent: message
    }, function(response) {

    });
}

//Creates a new suggestions for any emails found on salesforce
    function CreateComplexAddEmailSuggestions(name, emailaddress, recordType, whoId, whatId, whatName, whatRecordType)
    {
    if (emailaddress == null || typeof emailaddress == undefined) {
        emailaddress = "";
    }

    var emailDiv = CreateDiv("", "sfEmailSuggest CABoxStyleContainer");

    var crossIcon = CreateAddIcon();
    $(crossIcon).removeClass("AddSuggestionIcon");
    $(crossIcon).addClass("AddSuggestionIconWhite");

    var boxTextLabelContent = CheckStringLength(name, 37);

    if(emailaddress != null)
    {
        if (emailaddress.length > 0)
        {
            boxTextLabelContent = CheckStringLength(name + " (" + emailaddress + ")", 37);
        }
    }

    var style = "sfSuggest" + recordType + " CABoxStyle ebsta-complexadd-suggestion-people";

    var emailSpan = CreateComplexAddObjectDiv(boxTextLabelContent, recordType, style, crossIcon);

    emailDiv.setAttribute("WhoId", whoId);
    emailDiv.setAttribute("recordType", recordType);
    emailDiv.setAttribute("sfEmailAdd", emailaddress);
    emailDiv.setAttribute("sfName", name);

    //Include WhatId details if passed through so we can find this details to append into the Related To field later on
    if (whatId) {
        emailDiv.setAttribute("relatedWhatId", whatId);
        if (whatName) {
            emailDiv.setAttribute("relatedWhatName", whatName);
        }

        if (whatRecordType) {
            emailDiv.setAttribute("relatedWhatRecordType", whatRecordType);
        }
    }
    emailDiv.appendChild(emailSpan);

    AddEventListenerToComplexAddWhoObject(emailDiv);

    return emailDiv;
    }

function CreateContactSuggestions(obj)
{
    //Checks if the user actually has access to the whoid field of a task object before getting suggestions
    if (whoWhatEnabledObject != null)
    {
        if (!whoWhatEnabledObject.whoEnabled)
        {
            return false;
        }
    }

    chrome.extension.sendMessage({
        MessageType: "GetLocalStorageEmailSearchResult"
    }, function(response) {
        var json = null;

        if (response && IsValidJSON(response.json)) {
            json = JSON.parse(response.json);
        }

        var divSuggestRow3 = CreateRow("sfRowWhoSuggest");
        var divSuggestRowLeft = CreateDivWithId("", "sfRowWhoSuggestLeft", "");

        var header1 = document.createElement('h3');
        header1.setAttribute("class", "sfPeopleSuggestionsHeader");
        header1.appendChild(document.createTextNode('People Suggestions'));

        divSuggestRow3.appendChild(header1);

        //Get Name Suggestions from other contacts within the email
        if (obj.SecondaryEmailSearch != null && obj.SecondaryEmailSearch.length > 0) {
            var doWeNeedToSearchForNewEmails = false;
            var newEmailSearchArray = new Array();

            for (var i = 0; i < obj.SecondaryEmailSearch.length; i++) {
                var emailaddress = obj.SecondaryEmailSearch[i];

                var foundEmailInLocalStorage = false;

                if (json) {
                    for (var j = 0; j < json.d.Values.length; j++) {
                        var parsedJson = JSON.parse(json.d.Values[j]);
                        var emailFromJson = "";
                        if (parsedJson["Email-0"] != null) {
                            emailFromJson = parsedJson["Email-0"];
                        }
                        var recordType = parsedJson.Type;
                        var name = parsedJson.Name;
                        var whoId = parsedJson.Id;

                        if (recordType != null && typeof recordType != "undefined") {
                            if (recordType.toLowerCase() == 'contact' || recordType.toLowerCase() == 'lead') { //process people only!!!

                                //We don't want to show leads that have been converted
                                if (recordType.toLowerCase() == 'lead') {
                                    if (parsedJson["IsLeadConverted"]) {
                                        if (parsedJson["IsLeadConverted"] == "True") {
                                            continue;
                                        }
                                    }
                                }

                                var renderSuggestionInlist = false;

                                //Email could be mixed case
                                if (emailaddress != null && typeof emailaddress != 'undefined') {
                                    if (emailaddress.toLowerCase() == emailFromJson.toLowerCase()) {

                                        foundEmailInLocalStorage = true;
                                        renderSuggestionInlist = true;
                                    }
                                }
                                //Helps catch more cases of the original email address associated with the complex add button
                                //as we were only picking up the first instance of an email in cache when there could've potentially be 2+
                                else if (obj.EmailText && obj.Id) {
                                    if (emailFromJson.toLowerCase() == obj.EmailText.toLowerCase() && obj.Id != whoId) {
                                        foundEmailInLocalStorage = true;
                                        renderSuggestionInlist = true;
                                    }
                                }

                                if (renderSuggestionInlist) {
                                    var whatId = null, whatName = null, whatRecordType = null;
                                    //If we've got some history of a 'Related To' object for this record then lets get those results so we can pre-populate the the Related To box
                                    var result = LookForRelatedToObject(whoId);

                                    if (result) {
                                        if (result.WhatRecordId) {
                                            whatId = result.WhatRecordId;
                                        }

                                        if (result.WhatRecordName) {
                                            whatName = result.WhatRecordName;
                                        }

                                        if (result.WhatRecordType) {
                                            whatRecordType = result.WhatRecordType;
                                        }
                                    }

                                    //If we didn't find any previous selection for this object we'll default it to the associated Account (if there is any)
                                    if (whatId == null) {
                                        //Find Account Details from search results
                                        if (parsedJson["Account"] != null) {
                                            if (parsedJson["Account"].Id != null) {
                                                whatId = parsedJson["Account"].Id;
                                            }
                                            if (parsedJson["Account"].Name != null) {
                                                whatName = parsedJson["Account"].Name;
                                            }

                                            whatRecordType = "Account";
                                        }
                                    }

                                    //Make sure we haven't made a record for this already within the complex add screen
                                    if ($('div.sfAddComplexRow[sfaddcomplexrowwhoid="' + whoId + '"').length == 0 && $('div.sfEmailSuggest[whoid="' + whoId + '"').length == 0) {
                                        var emailDiv = CreateComplexAddEmailSuggestions(name, emailFromJson, recordType, whoId, whatId, whatName, whatRecordType);
                                        divSuggestRowLeft.appendChild(emailDiv);
                                    }
                                }
                            }
                        }
                    }
                }

                if (!foundEmailInLocalStorage) {
                    doWeNeedToSearchForNewEmails = true;
                    newEmailSearchArray.push(emailaddress);
                }
            }

            if (doWeNeedToSearchForNewEmails) {

                var searchTerm = "";

                for (var i = 0; i < newEmailSearchArray.length; i++) {
                    searchTerm += newEmailSearchArray[i] + " OR ";
                }

                searchTerm = searchTerm.substring(0, searchTerm.length - 4).trim();

                var loadingResults = CreateDiv("Loading other potential matches... ", "sfLoadingResults");
                loadingResults.className = 'resultsLoader';
                loadingResults.setAttribute("style", "display:inline-block;margin-left:10px;margin-top:0px;");
                var loadingGif = CreateImg(chrome.extension.getURL("/images/ajax-loader.gif"), "Loading other potential matches...");

                loadingResults.appendChild(loadingGif);

                header1.appendChild(loadingResults);

                //Run Search
                chrome.extension.sendMessage({
                    MessageType: "SearchComplexAdd-Who",
                    SearchTerm: searchTerm
                }, function (response) {
                    DisplaySearchResults("", response, searchTerm)
                });
            }

            divSuggestRow3.appendChild(divSuggestRowLeft);
            var divBox = document.getElementsByClassName('sfComplexAddContainer')[0];
            var saveRow = document.getElementById('sfRowComplete');
            divBox.appendChild(divSuggestRow3);
        } else {
            divSuggestRow3.appendChild(divSuggestRowLeft);
            var divBox = document.getElementsByClassName('sfComplexAddContainer')[0];
            var saveRow = document.getElementById('sfRowComplete');
            divBox.appendChild(divSuggestRow3);

            $('#ebstaBox').find(".sfPeopleSuggestionsHeader").css("display", "none");
        }
    });
}

    function CreateComplexAddObjectDiv(text, recordType, classname, crossOrAdd) {

    var surroundinggif = CreateDiv("", classname);
    
    var svgurl = "https://console.ebsta.com/svg/bar-icons.svg";

    var icon = null;

    if (recordType.toLowerCase() == "opp" || recordType.toLowerCase() == "opportunity"
        || recordType.toLowerCase() == "account"
        || recordType.toLowerCase() == "lead"
        || recordType.toLowerCase() == "contact"
        || recordType.toLowerCase() == "case") {
        icon = CreateDiv("", "mainIcon " + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length) + "Icon");
        icon.style.backgroundImage = svgurl;
        icon.style.backgroundRepeat = false;
    }
    else
    {
        icon = CreateDiv("", "mainIcon CustomIcon");
        icon.style.backgroundImage = svgurl;
        icon.style.backgroundRepeat = false;

        //Remove class if this is an object we don't have a set icon for and generalise it under a custom object icon
        if (classname.indexOf("sfSuggest") != -1)
        {
            $(surroundinggif).removeClass("sfSuggest" + recordType);
            $(surroundinggif).addClass("sfSuggestCustom");
        }
    }

    surroundinggif.appendChild(icon);

    var nameText = CreateDiv(text, "sfSearchResultNameText sf"+recordType + "NameTxt");

    surroundinggif.appendChild(nameText);

    surroundinggif.appendChild(crossOrAdd);

    //Create hover over event for complex add objects
    CreateHoverOverEvent(surroundinggif, HoverOverStates.ComplexAdd);

    return surroundinggif;
    }

    function CreateCrossIcon() {
    var cross = document.createElement('a');
    cross.setAttribute("class", "CACrossIcon");

    return cross;
    }

    function CreateAddIcon() {
    var cross = document.createElement('span');
    cross.setAttribute("class", "AddSuggestionIcon");

    return cross;
    }

    function AddListenerToDropDownArrowComplexAdd(obj)
    {
    obj.addEventListener("click", function (event) {

        //we may want to stop this if we're not opening another section.
        if ($(this).attr("class").indexOf("sfArrowDown") != -1) { //it is already a down arrow.
            $(this).removeClass("sfArrowDown"); //remove the down arrow.
            $(this).addClass("sfArrowRight");//add the right arrow.
        } else { //it is not a down arrow.
            $(this).removeClass("sfArrowRight"); //remove the right arrow.
            $(this).addClass("sfArrowDown");//add the down arrow.
        }

        $(this).closest('.sfAddComplexRow').each(function ()
        {

            $(this).find('.sfRowWhatSuggest').each(function ()
            {
                if ($(this).css("display") == "none")
                {
                    $(this).fadeIn();
                }
                else
                {
                    $(this).fadeOut();
                }
            });
        });

    });
    }

    function CheckStringLength(inputString, trimIndex) {
    if (inputString != null) {
        if (inputString.length > trimIndex) {
            inputString = inputString.substring(0, trimIndex) + '...';

            return inputString;
        } else {
            return inputString;
        }
    }

    return inputString;
    }

    //Adds a new row to complex add
    function CreateNewComplexAddRow(whoId, whoName, recordType, email, whatId, whatName, whatRecordType, stopSuggestionSearch, preSelectedAttachmentIds) {
    //console.time("CreateNewComplexAddRow");

    //If we're creating an empty row, lets see if one already exists before creating it
    if (!(whoId && typeof whoId !== 'undefined'))
    {
        var shouldWeKillThisFunction = false;
        $('#ebstaBox').find('.sfAddComplexRow').last().each(function () {
            var whoAttributes = $(this).attr('sfaddcomplexrowwhoid');
            var whatAttributes = $(this).attr('sfaddcomplexrowwhatids');

            if ((whoAttributes == "" || whoAttributes == null || whoAttributes == undefined || whoAttributes == "undefined") &&
                (whatAttributes == "" || whatAttributes == null || whatAttributes == undefined || whatAttributes == "undefined"))
            {
                shouldWeKillThisFunction = true;
                return false;
            }
        });

        if (shouldWeKillThisFunction) {
            return false;
        }
    }

    //Remove the empty rows on add
    $('#ebstaBox').find('.sfAddComplexRow').each(function () {
        var lastComplexRow = $(this);

        if ((lastComplexRow.attr('sfaddcomplexrowwhoid') == "" || lastComplexRow.attr('sfaddcomplexrowwhoid') == null || lastComplexRow.attr('sfaddcomplexrowwhoid') == undefined || lastComplexRow.attr('sfaddcomplexrowwhoid') == "undefined") &&
                (lastComplexRow.attr('sfaddcomplexrowwhatids') == "" || lastComplexRow.attr('sfaddcomplexrowwhatids') == null || lastComplexRow.attr('sfaddcomplexrowwhatids') == undefined || lastComplexRow.attr('sfaddcomplexrowwhatids') == "undefined")) {
            $(this).remove();
            //return false;
        }

        //Set Arrow to down for current row selected row
        if (whoId != undefined)
        {
            $(this).closest('.sfAddComplexRow').find('.sfArrowDown').removeClass("sfArrowDown").addClass("sfArrowRight");
            $(this).attr("currentselectedrow", "false");
            $(this).removeClass("sfCurrentSelectedRow");
            $(this).find('.sfRowWhatSuggest,.resultsWindow,.sfAttachRow').slideUp();
        }
    });

    var arrowDirectionClass = "sfArrowRight";

    if (whoId != null) {
        arrowDirectionClass = "sfArrowDown";
    }

    var divBox = document.getElementById('sfComplexAddRowsContainer');

    var mainDivRow = CreateRow("sfAddComplexRow");

    var divrow2 = CreateRow("sfSearchWhoWhatRow");

    divrow2.className = "newRowForAttachments";

    var arrowDiv = CreateDiv("", "sfArrow " + arrowDirectionClass);
    AddListenerForComplexAddRow(arrowDiv);
    //AddListenerToDropDownArrowComplexAdd(arrowDiv);
    divrow2.appendChild(arrowDiv);

    var customInputDiv = CreateDiv("", "sfWhoSearch CAObjectContainer");
    AddListenerForComplexAddRow(customInputDiv);

    //LogThis("WhoId: " + whoId);

    //LogThis("record type: " + recordType);

    if (whoId == undefined || whoId == "" ) {
        var whoIdElem = CreateTxtbox("", "sfSearchWhoTextBox CATextInput");
        whoIdElem.setAttribute("placeholder", "Search Name");
        TriggerEventOnEnter(whoIdElem);

        var searchButton = CreateSpan("", "sfWhoSearchBtn magnifySearchIcon", "");
        customInputDiv.appendChild(whoIdElem);
        customInputDiv.appendChild(searchButton);
        AddListenerForWhoSearchButton(searchButton);
    }
    else 
    {
        mainDivRow.setAttribute("class", "sfAddComplexRow sfCurrentSelectedRow");

        var crossIcon = CreateCrossIcon();
        crossIcon.className = crossIcon.className + " ebsta-complexadd-remove-object";

        var boxTextLabelContent = CheckStringLength(whoName, 37);
        if (email != null && typeof email != undefined)
        {
            if (email.length > 0)
            {
                boxTextLabelContent = CheckStringLength(whoName + " (" + email + ")", 37);
            }
        }

        //Only pre populate the Name field if this object being passed in was a Contact/Lead
        if (recordType.toLowerCase() == "contact" || recordType.toLowerCase() == "lead") {

            var whoIdButton = CreateComplexAddObjectDiv(boxTextLabelContent, recordType, "CABoxStyle sfInitSuggestion sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length), crossIcon);
            whoIdButton.setAttribute("sfWhoId", whoId);
            whoIdButton.setAttribute("recordType", recordType);
            whoIdButton.setAttribute("recordName", whoName);
            whoIdButton.setAttribute("email", email);
            AddListenerToWhoButtonToRemove(crossIcon);

            customInputDiv.appendChild(whoIdButton);

            mainDivRow.setAttribute("sfAddComplexRowWhoId", whoId);

        }
        else
        {
            var whoIdElem = CreateTxtbox("", "sfSearchWhoTextBox CATextInput");
            whoIdElem.setAttribute("placeholder", "Search Name");
            TriggerEventOnEnter(whoIdElem);

            var searchButton = CreateSpan("", "sfWhoSearchBtn magnifySearchIcon", "");
            customInputDiv.appendChild(whoIdElem);
            customInputDiv.appendChild(searchButton);
            AddListenerForWhoSearchButton(searchButton);

            // If there was a record passed into this function then its not a Who object so we need 
            // to re-associate it with the What Object so it can be pre-rendered into the complex add box
            if (whatRecordType == null) {
                whatRecordType = recordType;
                whatId = whoId;
                whatName = whoName;
            }
        }
    }

    var customInputDiv2 = CreateDiv("", "sfWhatSearch CAObjectContainer");
    AddListenerForComplexAddRow(customInputDiv2);

    var whatIdElem = CreateTxtbox("", "sfSearchWhatTextBox CATextInput", "sfWhatId");
    whatIdElem.setAttribute("placeholder", "Search Related To");

    var searchButton = CreateSpan("", "sfWhatSearchBtn magnifySearchIcon", "");
    AddListenerForWhatSearchButton(searchButton);

        var chooseObjectsToSearchButton = CreateObjectSearchMenu();
        chooseObjectsToSearchButton.className = chooseObjectsToSearchButton.className + " ebsta-complexadd-filter-object";

    //If a what object has been specified then we must populate the Related To box for this row
    if (whatRecordType)
    {
        var whatCrossIcon = CreateCrossIcon();
        whatCrossIcon.className = whatCrossIcon.className + " ebsta-complexadd-remove-object";

        if (whatRecordType.toLowerCase() == "opportunity") {
            whatRecordType = "Opp";
        }

        relatedToObject = CreateComplexAddObjectDiv(CheckStringLength(whatName, 37), whatRecordType, "CABoxStyle sfSuggest" + whatRecordType, whatCrossIcon);
        relatedToObject.setAttribute("whatId", whatId);
        relatedToObject.setAttribute("recordType", whatRecordType);
        relatedToObject.setAttribute("recordName", whatName);

        //Hide text search box and search button
        whatIdElem.style.display = "none";
        searchButton.style.display = "none";
        chooseObjectsToSearchButton.style.display = "none";

        AddEventListenerToComplexRemoveObject(whatCrossIcon);

        customInputDiv2.appendChild(relatedToObject);
        mainDivRow.setAttribute("sfaddcomplexrowwhatids", whatId);
    }

    customInputDiv2.appendChild(searchButton);
    customInputDiv2.appendChild(chooseObjectsToSearchButton);

    TriggerEventOnEnter(whatIdElem);

    customInputDiv2.appendChild(whatIdElem);
    customInputDiv2.appendChild(CreateBr());

    var SALeftSegmentContainer = document.createElement('div');
    SALeftSegmentContainer.className = 'SALeftSegment CASegment';
    SALeftSegmentContainer.appendChild(customInputDiv);

    var SARightSegmentContainer = document.createElement('div');
    SARightSegmentContainer.className = 'SARightSegment CASegment';
    SARightSegmentContainer.appendChild(customInputDiv2);

    divrow2.appendChild(SALeftSegmentContainer);
    divrow2.appendChild(SARightSegmentContainer);

    var lastRow = null;
    $('#ebstaBox').find('.sfAddComplexRow').each(function () {
        lastRow = $(this);
    });

    var clsBtn = CreateSpan("", "CADeleteRow", "");
    clsBtn.setAttribute("title", "Delete");
    AddListenerForCloseButton(clsBtn);

    divrow2.appendChild(clsBtn);

    //Add Row To List
    mainDivRow.appendChild(divrow2);
    //Append Attach Row if it's been created

    //delay the attachments row to minimize latency when obtaining Ids.
    setTimeout(function () {
        var attachRow = AddAttachmentRow(preSelectedAttachmentIds);

        if (attachRow != null) {
            SALeftSegmentContainer.appendChild(attachRow);

            if ($(mainDivRow).hasClass("sfCurrentSelectedRow")) {

            } else {
                $(attachRow).css("display", "none");
            }
        }
    }, 400);

    divBox.appendChild(mainDivRow);

    $(divrow2).hide().fadeIn();
    //$(divrow2).find('.sfArrow').trigger("click");

    var suggestRow = CreateRow("sfGetSuggestRow");
    var divSuggestRowRight = CreateDivWithId("", "sfRowGetSuggestRight", "sfRowGetSuggestRight");
    var loadingSuggestionsText = CreateDiv("Loading Suggestions", "sfLoadingSuggestions");
    var loadingGif = CreateImg(chrome.extension.getURL("/images/ajax-loader.gif"), "Loading Suggestions");

    divSuggestRowRight.appendChild(loadingSuggestionsText);
    divSuggestRowRight.appendChild(loadingGif);
    suggestRow.appendChild(divSuggestRowRight);
    //suggestRow.appendChild(CreateBr());
    divrow2.appendChild(suggestRow);

    var hideSuggestionLoader = true;

    if (recordType == "Contact" && !stopSuggestionSearch)
    {
        if (whoWhatEnabledObject != null)
        {
            if (whoWhatEnabledObject.whoEnabled && whoWhatEnabledObject.whatEnabled)
            {
                RequestEmailRelatedObjectLookup(whoId);
                hideSuggestionLoader = false;
            }
        }
    }

    if(hideSuggestionLoader)
    {
        loadingGif.style.visibility = "hidden";
        loadingSuggestionsText.style.visibility = "hidden";
        $(suggestRow).hide();
        $(arrowDiv).addClass("sfArrowRight").removeClass("sfArrowDown");
    }

        new ComplexAdd().CheckForLeads();

    var complexRows = document.getElementsByClassName("sfAddComplexRow");
    //if (complexRows.length > 0) {
    //    $(complexRows[complexRows.length - 2]).attr("currentselectedrow", "true");
    //    $(complexRows[complexRows.length - 2]).addClass("sfCurrentSelectedRow");
    //    $(complexRows[complexRows.length - 2]).find(".sfArrow").removeClass("sfArrowRight").addClass("sfArrowDown");
    //}
    
    for (var i = 1; i <= complexRows.length; i++)
    {
        var elem = complexRows[i - 1];
        //Check if this is the last row if so remove close icon
        $(elem).find('.CADeleteRow').each(function ()
        {
            if (i == complexRows.length)
            {
                $(this)[0].style.visibility = "hidden";
            }
            else
            {
                $(this)[0].style.visibility = "visible";
            }
        });

        //Only display suggestions box of last selected box
        //if (i != complexRows.length - 1)
        //{
        //    $(elem).find('.sfRowWhatSuggest').each(function () {
        //        $(this).remove();
        //    });
        //}
    }

    //Scrolls to position of the inserted row
    //ScrollToPosition($(SALeftSegmentContainer));

   // console.time("CheckIfWhoAndWhatFieldsAreEnabled");
    CheckIfWhoAndWhatFieldsAreEnabled(divrow2);
   // console.timeEnd("CheckIfWhoAndWhatFieldsAreEnabled");

    return divrow2;

    ////}, 500);
    }

//Creates the object search menu used in Complex Add and Templates where a user can search for a specific object
function CreateObjectSearchMenu() {
    var chooseObjectsToSearchButton = CreateSpan("", "ebsta-glyphicon ebsta-glyphicon-option-after", "");
    ShowComplexAddSearchableObjectsMenu(chooseObjectsToSearchButton);
    AttachHoverOverOnDivToShowEbstaPopUpMessage(chooseObjectsToSearchButton, "Click to choose which objects to search");
    return chooseObjectsToSearchButton;
}

//Creates a menu attached to complex add where a user can select which objects they'll like to search
function CreateComplexAddSearchableObjectsMenu(arrayOfObjects, arrayOfSuggestions) {

    //console.log("CreateComplexAddSearchableObjectsMenu");

    var mainDiv = $('body > .sfEBComplexAddSearchObjectChoiceDiv');

    var divCount = $(mainDiv).length;
    if (divCount === 0) {
        mainDiv = CreateDiv("", "sfEBComplexAddSearchObjectChoiceDiv");
    } else {
        mainDiv = $(mainDiv)[0];
        $(mainDiv).empty();
    }

    //Header Div
    var headerDiv = CreateDiv("", "sfEBComplexAddSearchObjectChoiceDivHeader");
    headerDiv.appendChild(CreateDiv("Choose Objects to Search", "sfEBComplexAddSearchObjectChoiceTitle"));

    var closeIconDiv = CreateDiv("", "ebsta-glyphicon ebsta-glyphicon-close-icon");
    $(closeIconDiv).on("click", function () {
        $(mainDiv).slideUp();
    });
    headerDiv.appendChild(closeIconDiv);
    mainDiv.appendChild(headerDiv);

    //Select and De-Select all Div
    mainDiv.appendChild(CreateSelectAndDeSelectContainer());

    mainDiv.appendChild(AddSearchOptionsIntoDivs(arrayOfObjects, "sfEBComplexAddSearchObjectChoiceDivOptionContainer"));
    //Additional section allowing selection of objects for Suggestions
    var suggestHeaderDiv = CreateDiv("", "sfEBComplexAddSearchObjectChoiceDivHeader");
    suggestHeaderDiv.appendChild(CreateDiv("Choose Objects for Suggested Search", "sfEBComplexAddSearchObjectChoiceTitle"));
    mainDiv.appendChild(suggestHeaderDiv);
    
    mainDiv.appendChild(AddSearchOptionsIntoDivs(arrayOfSuggestions, "sfEBComplexAddSearchObjectChoiceDivOptionContainerSuggested"));

    var confirmDiv = CreateDiv("Done", "sfEBComplexAddSearchObjectChoiceDivConfirm center-text ebsta-btn ebsta-btn-success");
    $(confirmDiv).on("click", function () {
        $(mainDiv).slideUp();
    });
    AddListenerForDoneButton(confirmDiv);
    mainDiv.appendChild(confirmDiv);

    $(mainDiv).hide();

    if (divCount === 0) {
        document.body.appendChild(mainDiv);
    }
}
/**
Takes in an array of either our related search or suggestion terms and builds out our divs
*/
function AddSearchOptionsIntoDivs(arrayofSelections, divName)
{
    var optionSearchContainer = CreateDiv("", divName);

    for (var i = 0; i < arrayofSelections.length; i++) {
        var optionDiv = CreateDiv("", "sfEBComplexAddSearchObjectChoiceDivOption");
        optionDiv.title = arrayofSelections[i].Name;

        var circleTickCrossGlyphicon = CreateDiv("", "ebsta-glyphicon");

        //console.log(arrayofSelections[i]);

        if (arrayofSelections[i].ChosenAsObjectToSearch) {
            $(circleTickCrossGlyphicon).addClass('ebsta-glyphicon-circletick');
        } else {
            $(circleTickCrossGlyphicon).addClass('ebsta-glyphicon-circlecross');
        }

        circleTickCrossGlyphicon.setAttribute("apiname", arrayofSelections[i].ApiName);
        AttachClickListenerOnGlyphiconForComplexAddObjectChoice(optionDiv);

        var text = CreateSpan(arrayofSelections[i].Name);

        optionDiv.appendChild(circleTickCrossGlyphicon);
        optionDiv.appendChild(text);
        optionSearchContainer.appendChild(optionDiv);
    }

    if (divName == "sfEBComplexAddSearchObjectChoiceDivOptionContainer")
    {
        if (arrayofSelections.length % 2 == 1) {
            optionSearchContainer.appendChild(CreateDiv("", "sfEBComplexAddSearchObjectChoiceDivOption"));
        }
    }

    optionSearchContainer.appendChild(optionDiv);
    return optionSearchContainer;
}

//Creates the header to either select or de-select all within the Complex Add object search menu
function CreateSelectAndDeSelectContainer() {
    var selectAllContainer = CreateDiv("", "sfEBComplexAddSearchObjectChoiceDivSelectContainer");

    var selectAllDiv = CreateDiv("", "sfEBComplexAddSearchObjectChoiceDivSelectAll");
    selectAllDiv.appendChild(CreateDiv("", "ebsta-glyphicon ebsta-glyphicon-circletick"));
    selectAllDiv.appendChild(CreateSpan("Select All", ""));

    AttachClickListenerToSelectAndDeSelect(selectAllDiv, true);

    var deSelectAllDiv = CreateDiv("", "sfEBComplexAddSearchObjectChoiceDivDeSelectAll");
    deSelectAllDiv.appendChild(CreateDiv("", "ebsta-glyphicon ebsta-glyphicon-circlecross"));
    deSelectAllDiv.appendChild(CreateSpan("De-select All", ""));

    AttachClickListenerToSelectAndDeSelect(deSelectAllDiv, false);

    selectAllContainer.appendChild(selectAllDiv);
    selectAllContainer.appendChild(deSelectAllDiv);

    return selectAllContainer;
}

//Attachs a click listener to the select and de-select options with the complex add object search div
function AttachClickListenerToSelectAndDeSelect(div, selectingAll) {
    div.addEventListener("click", function() {
        if (selectingAll) {
            $('body > .sfEBComplexAddSearchObjectChoiceDiv > .sfEBComplexAddSearchObjectChoiceDivOptionContainer > ' +
                '.sfEBComplexAddSearchObjectChoiceDivOption > .ebsta-glyphicon-circlecross').removeClass('ebsta-glyphicon-circlecross').addClass('ebsta-glyphicon-circletick');
        } else {
            $('body > .sfEBComplexAddSearchObjectChoiceDiv > .sfEBComplexAddSearchObjectChoiceDivOptionContainer > ' +
                '.sfEBComplexAddSearchObjectChoiceDivOption > .ebsta-glyphicon-circletick').removeClass('ebsta-glyphicon-circletick').addClass('ebsta-glyphicon-circlecross');
        }
    });
}

//Attachs a click listener onto each object withn the complex add object div
function AttachClickListenerOnGlyphiconForComplexAddObjectChoice(div) {
    div.addEventListener("click", function () {
        var glyphIconDiv = $(div).find('.ebsta-glyphicon');
        if ($(glyphIconDiv).hasClass('ebsta-glyphicon-circletick')) {
            $(glyphIconDiv).addClass('ebsta-glyphicon-circlecross').removeClass('ebsta-glyphicon-circletick');
        } else if ($(glyphIconDiv).hasClass('ebsta-glyphicon-circlecross')) {
            $(glyphIconDiv).addClass('ebsta-glyphicon-circletick').removeClass('ebsta-glyphicon-circlecross');
        }
    });
}

function ShowComplexAddSearchableObjectsMenu(clickedMenu) {
    clickedMenu.addEventListener("click", function() {
        //Get position of click menu div so we can render the menu right besides it
        var offset = $(this).closest('.sfBox').offset();
        var menuDiv = $('body > .sfEBComplexAddSearchObjectChoiceDiv');
        if ($(menuDiv).length > 0) {
            $(menuDiv).slideUp();
            if (!$(menuDiv).is(':visible')) {
                $(menuDiv).css('top', offset.top + ($(this).closest('.sfBox').outerHeight() / 2) - ($(menuDiv).outerHeight() / 2));
                $(menuDiv).css('left', offset.left + ($(this).closest('.sfBox').outerWidth() / 2) - ($(menuDiv).outerWidth() / 2));
                $(menuDiv).slideDown();
            }
        }

        $('.ebsta-popover').hide();
    });
}

//Checks to see if there is access to either the Who and/or What fields and updates complex add and templates search boxes to reflect this
function CheckIfWhoAndWhatFieldsAreEnabled(divrow) {
    if (whoWhatEnabledObject != null)
    {
        if (!whoWhatEnabledObject.whoEnabled)
        {
            //We only want to change complex add's who search field as you can still search for a Contact/lead for an Email in SF
            if (!$(divrow).hasClass('sfEmailTemplateSearch'))
            {
                var textNode = CreateTxtbox("", "CATextInput");
                textNode.value = "This field is not active for your Task object";
                textNode.disabled = "disabled";

                //Remove what ever is there firstly
                //Removes anything in the who search box
                $(divrow).find('.sfWhoSearch .CABoxStyle, .sfWhoSearchBtn, .sfSearchWhoTextBox').remove();

                //Removes suggestions
                $(divrow).closest('.sfComplexAddContainer').find('.sfRowWhoSuggest').remove();//Complex Add

                //Remove attributes if we've set any
                $(divrow).closest('.sfAddComplexRow').attr("sfaddcomplexrowwhoid", ""); //Complex add and templates

                //Render the text to show user they're unable to add objects to whoid
                $(textNode).addClass("sfSearchWhoTextBox");
                $(divrow).find('.sfWhoSearch').addClass('sfGreyOutBox').append(textNode);
            }
        }

        if (!whoWhatEnabledObject.whatEnabled)
        {
            var textNode = CreateTxtbox("", "CATextInput");
            textNode.value = "This field is not active for your Task object";
            textNode.disabled = "disabled";

            //Remove what ever is there firstly
            //Removes anything in the what box
            $(divrow).find('.sfWhatSearch .CABoxStyle, .sfWhatSearchBtn, .sfSearchWhatTextBox, .sfRowWhatSuggest, br, .ebsta-glyphicon-option-after').remove();

            //Render the text to show user they're unable to add objects to whoid
            $(textNode).addClass("sfSearchWhatTextBox");
            $(divrow).find('.sfWhatSearch').addClass('sfGreyOutBox').append(textNode);
        }
    }
    }

/**
 * Checks to see if the attachment was pre-selected before in a complex add window
 * and updates the state of the attachment to reflect this
 * @param {} preSelectedAttachmentIdsSplit 
 * @param {} attachmentId 
 * @param {} attachmentContainer 
 * @returns {} 
 */
function UpdatePreSelectedAttachments(preSelectedAttachmentIdsSplit, attachmentId, attachmentContainer) {
    if (preSelectedAttachmentIdsSplit != null && attachmentId != null) {
        var attachmentIdSplit = attachmentId.split(".");

        if (attachmentIdSplit.length > 1) {
            if (preSelectedAttachmentIdsSplit.indexOf(attachmentIdSplit[1]) != -1) {
                RemoveClass(attachmentContainer, "sfAttachmentUnselected");
                AddClass(attachmentContainer, "sfAttachmentSelected");
            }
        }
    }
}

    //Add the attachment row to complex add
function AddAttachmentRow(preSelectedAttachmentIds) {
    //Get attachments
    var messageAttachments = createMessageObject($("*[current-emailadd*='true'"));

    if (messageAttachments.Attachments) {

        if (messageAttachments.Attachments.length > 0) {
            var attachRow = CreateRow("sfAttachRow");

            var SAattachmentContainer = document.createElement('div');
            SAattachmentContainer.className = 'SAAttachments';

            //Create attachment container

            attachRow.appendChild(CreateDiv("Select Attachments", "sfAttachmentHeader"));

            var preSelectedAttachmentIdsSplit = null;
            if (preSelectedAttachmentIds != null) {
                preSelectedAttachmentIdsSplit = preSelectedAttachmentIds.split(",");
            }

            for (var i = 0; i < messageAttachments.Attachments.length; i++) {
                var attachmentContainer = document.createElement('div');
                attachmentContainer.className = "sfAttachment sfAttachmentUnselected CABoxStyle";

                var attachmentId = messageAttachments.Attachments[i].Id;

                UpdatePreSelectedAttachments(preSelectedAttachmentIdsSplit, attachmentId, attachmentContainer);

                var uniqueId = $.guid++;

                var checkbox = document.createElement('input');
                checkbox.type = "checkbox";
                checkbox.className = "sfAttachmentCheckBox";
                checkbox.name = attachmentId;
                checkbox.value = attachmentId;
                checkbox.id = uniqueId;

                var label = document.createElement('label');
                label.htmlFor = uniqueId;
                attachmentContainer.appendChild(CreateDiv(CheckStringLength(messageAttachments.Attachments[i].Name, 25), "sfAttachmentNameTxt"));

                attachmentContainer.appendChild(checkbox);
                attachmentContainer.appendChild(label);
                attachmentContainer.appendChild(CreateAddIcon());

                SAattachmentContainer.appendChild(attachmentContainer);

                $(attachmentContainer).on("click", function() {
                    if ($(this).hasClass('sfAttachmentUnselected')) {
                        $(this).removeClass('sfAttachmentUnselected').addClass('sfAttachmentSelected');
                    } else if ($(this).hasClass('sfAttachmentSelected')) {
                        $(this).removeClass('sfAttachmentSelected').addClass('sfAttachmentUnselected');
                    }
                });


            }

            attachRow.appendChild(SAattachmentContainer);

            return attachRow;
        }
    }
}

//Creates the suggestions for a given row
    function CreateDivForSuggestions(rowWhoId) {

    var rowElems = document.getElementsByClassName("sfAddComplexRow");

    //Finds the corresponding row for the selected user
    var rowElem = null;
    for (var i = 0; i < rowElems.length; i++) {
        var whoId = "";
        if (rowElems[i].attributes["sfAddComplexRowWhoId"] != null) {
            whoId = rowElems[i].attributes["sfAddComplexRowWhoId"].value;
        }

        if (rowWhoId == whoId) {
            rowElem = rowElems[i];
        }
    }

    if (rowElem != null)
    {
        //Create Div for suggestions
        var divSuggestRow2 = CreateRow("sfRowWhatSuggest");

        var divSuggestRowRight = CreateDivWithId("", "sfRowWhatSuggestRight CASuggestRight", "sfRowWhatSuggestRight");

        var divSuggestRowRightSuggestions = CreateDiv("", "sfSuggestRowRightSuggestions");

        var header2 = document.createElement('span');
        header2.className = 'suggestionListTitle';
        header2.appendChild(document.createTextNode('Other suggestions'));

        divSuggestRow2.appendChild(header2);

        divSuggestRowRight.appendChild(divSuggestRowRightSuggestions);
        divSuggestRow2.appendChild(divSuggestRowRight);

        var findRightSegment = rowElem.getElementsByClassName("SARightSegment");

        if (findRightSegment != null && typeof findRightSegment != "undefined") {
            if (findRightSegment.length > 0) {
                findRightSegment[0].appendChild(divSuggestRow2);
            }
        }

        ////rowElem.appendChild(divSuggestRow2);
    }

    }

    //Removes the button from the list
    function AddListenerToWhoButtonToRemove(obj) {
    obj.addEventListener("click", function (event) {

        //Hide the hover over box once something has been selected
        $('.sfRecordHoverDiv').removeClass('userSelecting').hide();

        var thisBt = $(this)[0];

        //If this was an initial suggestion then put it back with the suggestions
        if ($(this).closest('.sfInitSuggestion').hasClass('sfInitSuggestion'))
        {
            $('#ebstaBox, #emailTemplateBox').find(".sfPeopleSuggestionsHeader").css("display", "block");

            var whoId = $(this).closest('.sfInitSuggestion').attr("sfwhoid");
            var recordType = $(this).closest('.sfInitSuggestion').attr("recordtype");
            var name = $(this).closest('.sfInitSuggestion').attr("recordname");
            var emailaddress = $(this).closest('.sfInitSuggestion').attr("email");
            var style = "sfSuggest" + recordType + " CABoxStyle";

            var emailDiv = CreateDiv("", "sfEmailSuggest CABoxStyleContainer");

            var crossIcon = CreateAddIcon();
            $(crossIcon).removeClass("AddSuggestionIcon");
            $(crossIcon).addClass("AddSuggestionIconWhite");

            if (emailaddress == null || typeof emailaddress == undefined)
            {
                emailaddress = "";
            }

            var boxTextLabelContent = CheckStringLength(name, 37);

            if (emailaddress != null)
            {
                if (emailaddress.length > 0)
                {
                    boxTextLabelContent = CheckStringLength(name + " (" + emailaddress + ")", 37);
                }
            }

            var emailSpan = CreateComplexAddObjectDiv(boxTextLabelContent, recordType, style, crossIcon);

            emailDiv.setAttribute("WhoId", whoId);
            emailDiv.setAttribute("recordType", recordType);
            emailDiv.setAttribute("sfEmailAdd", emailaddress);
            emailDiv.setAttribute("sfName", name);
            emailDiv.appendChild(emailSpan);

            //Add listener on object
            AddEventListenerToComplexAddWhoObject(emailDiv);

            //Complex Add
            if (document.getElementsByClassName("sfRowWhoSuggestLeft").length) {
                if (document.getElementsByClassName("sfRowWhoSuggestLeft").length > 0) {
                    document.getElementsByClassName("sfRowWhoSuggestLeft")[0].appendChild(emailDiv);
                }
            }

            //Email Template appending
            if (document.getElementsByClassName("sfEmailTemplateRowWhoSuggestLeft").length) {
                if (document.getElementsByClassName("sfEmailTemplateRowWhoSuggestLeft").length > 0) {
                    document.getElementsByClassName("sfEmailTemplateRowWhoSuggestLeft")[0].appendChild(emailDiv);
                }
            }
        }

        $(this).closest('.sfComplexAddContainer').find('.sfGreyFocusBox').removeClass('sfGreyFocusBox');

        $(this).closest('.sfAddComplexRow, .sfEmailTemplateSearch').each(function ()
        {
            $(this)[0].setAttribute("sfaddcomplexrowwhoid", "");

            if (($(this).attr('sfaddcomplexrowwhatids') == ""
                || $(this).attr('sfaddcomplexrowwhatids') == null
                || $(this).attr('sfaddcomplexrowwhatids') == undefined
                || $(this).attr('sfaddcomplexrowwhatids') == "undefined"))
            {
                //Remove row if this is in complex add screen
                if ($(this).hasClass('sfAddComplexRow'))
                {
                    $(this).remove();
                }
            }

            //Create new search box
            $(this).find('.sfWhoSearch').each(function () {
                var whoIdElem = CreateTxtbox("", "sfSearchWhoTextBox CATextInput");
                whoIdElem.setAttribute("placeholder", "Search Name");
                TriggerEventOnEnter(whoIdElem);
                $(this).find('.magnifySearchIcon').remove();
                var searchButton = CreateSpan("", "sfWhoSearchBtn magnifySearchIcon");
                AddListenerForWhoSearchButton(searchButton);

                $(this)[0].appendChild(whoIdElem);
                $(this)[0].appendChild(searchButton);
            });

            //Remove Suggestions Box
            $(this).find('.sfRowWhatSuggest').each(function () {
                $(this).slideUp("500", function () { $(this).remove(); });
            });

            //Remove Suggestions Box
            $(this).find('.sfSuggestLead, .sfSuggestContact').each(function () {
                $(this).remove();
            });
            
        });

        thisBt.remove();

    });
    }

    //A queue type system which holds previous search requests until the on before has been completed
    function RequestEmailRelatedObjectLookup(whoId)
    {
    chrome.extension.sendMessage({
            MessageType: "EmailRelatedObjectLookup",
            SearchTerm: whoId,
    }, function (response) {
            ShowRelatedToObjects("", response, whoId);
        }
    );
    }

    //Removes a row
    function AddListenerForCloseButton(obj) {
    obj.addEventListener("click", function (event) {

        $(this).closest('.sfAddComplexRow').each(function () {
            $(this).remove();
        });

    });

    return obj;
    }

    //Adds a row
    function AddListenerForAddButton(obj) {
    obj.addEventListener("click", function (event) {
        //CreateNewComplexAddRow();
    });

    return obj;
    }

    //updates the objects chosen for related to and suggested searches when we hit the 'Done button'
    function AddListenerForDoneButton(obj) {
        obj.addEventListener("click", function (event) {
            var whatObjectsToSearch = GetWhatObjectsToSearch();
            var whatObjectsToSuggest = GetWhatSuggestionsToSearch();
            chrome.extension.sendMessage({
                MessageType: "SaveRelatedAndSuggestedSearchObjects", RelatedObjectsToSearch: whatObjectsToSearch, SuggestedObjectsToSearch: whatObjectsToSuggest
            }, function (response) {
                DisplaySearchResults("", response, "")
            });

        });

        return obj;
    }

    //Adds a listener to each search button
    function AddListenerForWhatSearchButton(obj)
    {
    obj.addEventListener("click", function (event) {
        var searchVal = "";

        if ($(this).hasClass('magnifySearchIcon'))
        {
            $(this).parent().find('.sfSearchWhatTextBox').each(function () {
                searchVal = $(this)[0].value;

                //Run Search 
                if (searchVal.length > 1) {

                    $(this).attr('placeholder', "Search Related To");

                    var loadingResults = CreateDiv("Loading results... ", "sfLoadingResults");
                    loadingResults.className = 'resultsLoader';
                    loadingResults.setAttribute("style", "display:inline-block;margin-left:10px;margin-top:10px;");
                    var loadingGif = CreateImg("https://console.ebsta.com/images/google/ajax-loader.gif", "Loading results... ");

                    loadingResults.appendChild(loadingGif);

                    var parentObject = $(this).parent();

                    //If there is a loader there already don't add another
                    if ($(parentObject).parent().find('.resultsLoader').length == 0)
                    {
                        parentObject.after(loadingResults);
                    }
                    
                    //$('.sfNoSuggestCaseFound').hide();

                    //Clear last search
                    $('#ebstaBox, #emailTemplateBox').find('span[lastsearch="true"]').each(function () {
                        $(this).attr("lastsearch", "false");
                    });

                    $(this).parent().find('.sfWhatSearchBtn').each(function () {
                        $(this).attr("lastsearch", "true");
                    });

                    var whatObjectsToSearch = GetWhatObjectsToSearch();

                    chrome.extension.sendMessage({
                        MessageType: "SearchComplexAdd-What", SearchTerm: searchVal, ObjectsToSearch: whatObjectsToSearch
                    }, function (response) {
                        DisplaySearchResults("", response, searchVal)
                    });
                }
                else if (searchVal.length < 2) {
                    $(this).val("");
                    $(this).attr('placeholder', "Please enter at least 2 characters");
                }

                return false;
            });
        }
        else
        {
            $(this).parent().find('.CATextInput').val("");
            $(this).parent().find('.CATextInput').focus();
            $(this).parent().find('.sfWhatSearchBtn').removeClass('crossSearchIcon').addClass('magnifySearchIcon');
            $(this).closest('.CASegment').find('.resultsWindow').slideUp(function () { $(this).remove(); });
        }

    });

    return obj;
    }

//Gets the list of chosen 'What' objects to search for within Complex Add
function GetWhatObjectsToSearch() {
    var objectsToSearch = [];
    $('body > .sfEBComplexAddSearchObjectChoiceDiv > .sfEBComplexAddSearchObjectChoiceDivOptionContainer .sfEBComplexAddSearchObjectChoiceDivOption .ebsta-glyphicon-circletick').each(function () {
        if ($(this).attr('apiname')) {
            objectsToSearch.push($(this).attr('apiname'));
        }
        });
    return objectsToSearch;
}


//Gets the list of chosen objects to search for within suggested search
function GetWhatSuggestionsToSearch() {
    var objectsToSearch = [];
    $('body > .sfEBComplexAddSearchObjectChoiceDiv > .sfEBComplexAddSearchObjectChoiceDivOptionContainerSuggested .sfEBComplexAddSearchObjectChoiceDivOption .ebsta-glyphicon-circletick').each(function () {
        if ($(this).attr('apiname')) {
            objectsToSearch.push($(this).attr('apiname'));
        }
    });
    return objectsToSearch;
}



//Triggers the click event when a user presses enter on the keyboard
function TriggerEventOnEnter(obj) {
    obj.addEventListener("keyup", function (event) {
        //Enter
        if (event.keyCode == 13)
        {
            $(this).parent().find('.crossSearchIcon').removeClass('crossSearchIcon').addClass('magnifySearchIcon');

            $(this).parent().find('.magnifySearchIcon').each(function () {
                $(this).trigger("click");
            });
        }
        //Backspace
        else if (event.keyCode == 8)
        {
            $(this).parent().find('.crossSearchIcon').removeClass('crossSearchIcon').addClass('magnifySearchIcon');
        }
    });
    }

    //Adds a listener to each search button
    function AddListenerForWhoSearchButton(obj) {
    obj.addEventListener("click", function (event) {

        var searchVal = "";

        if ($(this).hasClass('magnifySearchIcon'))
        {
            $(this).parent().find('.sfSearchWhoTextBox').each(function ()
            {
                searchVal = $(this)[0].value;

                //Run Search 
                if (searchVal.length > 1) {

                    $(this).parent().find('.sfLoadingResults,img').each(function () {
                        $(this).remove();
                    });

                    var loadingResults = CreateDiv("Loading results... ", "sfLoadingResults");
                    loadingResults.className = 'resultsLoader';
                    loadingResults.setAttribute("style", "display:inline-block;margin-left:10px;margin-top:10px;");
                    var loadingGif = CreateImg("https://console.ebsta.com/images/google/ajax-loader.gif", "Loading results..");

                    loadingResults.appendChild(loadingGif);

                    var parentObject = $(this).parent();

                    //If there is a loader there already don't add another
                    if ($(parentObject).parent().find('.resultsLoader').length == 0) {
                        parentObject.after(loadingResults);
                    }

                    //$('.sfNoSuggestCaseFound').hide();

                    //Clear last search
                    $('#ebstaBox, #emailTemplateBox').find('span[lastsearch="true"]').each(function () {
                        $(this).attr("lastSearch", "false");
                    });

                    //Set this button for the last search so the results can find the right box
                    $(this).closest('.sfWhoSearch').find('.sfWhoSearchBtn').attr("lastSearch", "true");

                    //Run Search
                    chrome.extension.sendMessage({
                        MessageType: "SearchComplexAdd-Who", SearchTerm: searchVal
                    }, function (response) {
                        DisplaySearchResults("", response, searchVal)
                    });

                }
                else if (searchVal.length < 2) {
                    $(this).val("");
                    $(this).attr('placeholder', "Please enter at least 2 characters");
                }
            });
        }
        else
        {
            $(this).parent().find('.CATextInput').val("");
            $(this).parent().find('.CATextInput').focus();
            $(this).parent().find('.sfWhoSearchBtn').removeClass('crossSearchIcon').addClass('magnifySearchIcon');
            $(this).closest('.CASegment').find('.resultsWindow').slideUp(function () { $(this).remove(); });
        }

    });
    }

    //Add event listeners to recommended Contacts and Leads
    function AddEventListenerToComplexAddWhoObject(obj) {

    obj.addEventListener("click", function (event) {

        //Hide the hover over box once something has been selected
        $('.sfRecordHoverDiv').removeClass('userSelecting').hide();

        var whoId = "";
        var recordType = "";
        var email = "";
        var whoName = "";
        var elem = null;
        var elemToDelete = null;
        var whatId = null; var whatName = null; var whatRecordType = null;

        $(this).closest('.sfEmailSuggest').each(function ()
        {
            whoId = $(this).attr("whoid");
            recordType = $(this).attr("recordtype");
            email = $(this).attr("sfEmailAdd");
            whoName = $(this).attr("sfName");

            //Find if there is any related information so we can pre-populate the Related To field on selection
            if ($(this).attr("relatedWhatId")) {
                whatId = $(this).attr("relatedWhatId");

                if ($(this).attr("relatedwhatname")) {
                    whatName = $(this).attr("relatedwhatname");
                }
                if ($(this).attr("relatedWhatRecordType")) {
                    whatRecordType = $(this).attr("relatedWhatRecordType");
                }
            }

            elemToDelete = $(this);
        });

        var isThisForComplexAdd = false;
        var isThisForEmailTemplate = false;

        $(this).closest('#ebstaBox').find('.sfAddComplexRow').each(function () {
            elem = $(this);
            isThisForComplexAdd = true;
        });
        $(this).closest('.sfEmailTemplateSuggestions').find('.sfEmailTemplateSearch').each(function () {
            elem = $(this);
            isThisForEmailTemplate = true;
        });
        if (isThisForComplexAdd)
        {
            CreateNewComplexAddRow(whoId, whoName, recordType, email, whatId, whatName, whatRecordType);
            CreateNewComplexAddRow();
        }
        else if (isThisForEmailTemplate)
        {
            var crossIcon = CreateCrossIcon();
            crossIcon.className = crossIcon.className + " ebsta-complexadd-remove-object";

            AddListenerToWhoButtonToRemove(crossIcon);

            var boxTextLabelContent = CheckStringLength(whoName, 37);

            if (email != null || typeof email != undefined)
            {
                if (email.length > 0)
                {
                    boxTextLabelContent = CheckStringLength(whoName + " (" + email + ")", 37);
                }
            }

            var whoIdButton = CreateComplexAddObjectDiv(boxTextLabelContent, recordType, "CABoxStyle sfInitSuggestion sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length), crossIcon);
            whoIdButton.setAttribute("sfWhoId", whoId);
            whoIdButton.setAttribute("recordType", recordType);
            whoIdButton.setAttribute("recordName", whoName);
            whoIdButton.setAttribute("email", email);

            //Remove text box and divs if already there
            $(elem).find('.sfWhoSearch').find('.CABoxStyle,.magnifySearchIcon').each(function () {
                //Put results back onto suggestions list
                if ($(this).hasClass('sfInitSuggestion')) {
                    AddSuggestionBackToEmailTemplateSuggestionsList($(this));
                }
                $(this).remove();
            });
            $(elem).find('.sfSearchWhoTextBox').remove();
            $(elem).find('.sfWhoSearch').append(whoIdButton);

            new ComplexAdd().CheckForLeads();
        }

        var newRow = $("div[sfaddcomplexrowwhoid='" + whoId + "']");

        newRow.find('#sfWhoId').each(function () {
            var btn = CreateBt(whoName, "sfSearchWhoBtn", whoId);
            $(this).after(btn);

            //$(this).attr("value", whoName);
            //$(this).attr("whoid", whoId);
        });

        $(elemToDelete).remove();

        //Remove people suggestions header if there are non left
        var noOfSuggestDivs = $(".sfEmailSuggest").length;
        if(noOfSuggestDivs == 0)
        {
            $('#ebstaBox, #emailTemplateBox').find(".sfPeopleSuggestionsHeader").css("display", "none");
        }
        
    });

        new ComplexAdd().CheckForLeads();

    return obj;
    }

    //Adds a suggestions back to the list if another suggestion had been picked
    function AddSuggestionBackToEmailTemplateSuggestionsList(div)
    {
    var whoId = $(div).attr("sfwhoid");
    var recordType = $(div).attr("recordtype");
    var name = $(div).attr("recordname");
    var emailaddress = $(div).attr("email");
    var style = "sfSuggest" + recordType + " CABoxStyle";

    var emailDiv = CreateDiv("", "sfEmailSuggest CABoxStyleContainer");

    var crossIcon = CreateAddIcon();
    $(crossIcon).removeClass("AddSuggestionIcon");
    $(crossIcon).addClass("AddSuggestionIconWhite");

    var boxTextLabelContent = CheckStringLength(name + " (" + emailaddress + ")", 37);

    var emailSpan = CreateComplexAddObjectDiv(boxTextLabelContent, recordType, style, crossIcon);

    emailDiv.setAttribute("WhoId", whoId);
    emailDiv.setAttribute("recordType", recordType);
    emailDiv.setAttribute("sfEmailAdd", emailaddress);
    emailDiv.setAttribute("sfName", name);
    emailDiv.appendChild(emailSpan);

    //Add listener on object
    AddEventListenerToComplexAddWhoObject(emailDiv);

    //Email Template appending
    if (document.getElementsByClassName("sfEmailTemplateRowWhoSuggestLeft").length) {
        if (document.getElementsByClassName("sfEmailTemplateRowWhoSuggestLeft").length > 0) {
            document.getElementsByClassName("sfEmailTemplateRowWhoSuggestLeft")[0].appendChild(emailDiv);
        }
    }
    }

    //Add event listener onto suggested option and add into box
    function AddEventListenerToComplexAddedObject(obj) {

    obj.addEventListener("click", function (event) {

        //Hide the hover over box once something has been selected
        $('.sfRecordHoverDiv').removeClass('userSelecting').hide();

        var whatId = "";
        var recordType = "";
        var whatName = "";


        var elemToRemove = null;

        $(this).closest('.CABoxStyle').each(function ()
        {
            whatId = $(this).attr("whatid");
            recordType = $(this).attr("recordType");
            whatName = $(this).attr("recordName");
            elemToRemove = $(this);
        });

        var elem = $(this).closest('.sfAddComplexRow,.sfEmailTemplateSearch')[0];
        var sfaddcomplexrowwhatids = $(this).closest('.sfAddComplexRow').attr("sfaddcomplexrowwhatids");

        /****NOT MULTIPLE WHATIDS***/
        //if (sfaddcomplexrowwhatids != null) {
         //   $(this).closest('.sfAddComplexRow').attr("sfaddcomplexrowwhatids", sfaddcomplexrowwhatids + "," + whatId);
        //} else {
         $(this).closest('.sfAddComplexRow').attr("sfaddcomplexrowwhatids", whatId);
        //}

         $(elem).find('.sfWhatSearch').each(function ()
         {
             var crossIcon = CreateCrossIcon();
             crossIcon.className = crossIcon.className + " ebsta-complexadd-remove-object";

            AddEventListenerToComplexRemoveObject(crossIcon);

            var btn = CreateComplexAddObjectDiv(CheckStringLength(whatName, 37), recordType, "CABoxStyle sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length), crossIcon);
            $(btn).find('.mainIcon').removeClass(recordType + "Icon");
            $(btn).find('.mainIcon').removeClass("CustomIcon");
            //$(btn).find('.mainIcon').addClass("selected" + recordType + "Icon");
            ChangeIconToReflectSelectedObject(btn, recordType);
            btn.setAttribute("whatid", whatId);
            btn.setAttribute("recordType", recordType);
            btn.setAttribute("recordName", whatName);

            if($(elemToRemove).hasClass('result')){
                $(btn).addClass('result');
            }
            
            /****NOT MULTIPLE WHATIDS***/
            //Remove element if its already in the list
            $(this).find('.CABoxStyle').each(function ()
            {
                var whatId = $(this).attr("whatid");
                var recordType = $(this).attr("recordType");
                var whatName = $(this).attr("recordName");
                var addIcon = CreateAddIcon();

                var btn2 = CreateComplexAddObjectDiv(CheckStringLength(whatName, 37), recordType, "CABoxStyle sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length)+" sfSuggestGrey", addIcon);
                btn2.setAttribute("whatid", whatId);
                btn2.setAttribute("recordType", recordType);
                btn2.setAttribute("recordName", whatName);

                AddEventListenerToComplexAddedObject(btn2);

                //Add removed object to suggestion list or results list depending on what the initial object was
                if ($(this).hasClass('result'))
                {
                    $(btn2).addClass('result');
                    $(this).closest('.sfAddComplexRow, .sfEmailTemplateSearch .SARightSegment').find('.resultsWindow').each(function () {
                        $(this).append(btn2);
                    });
                }
                else
                {
                    $(this).closest('.sfAddComplexRow').find('.sfSuggestRowRightSuggestions').each(function () {
                        $(this).after(btn2);
                    });
                }

                $(this).remove();
            });

            /****NOT MULTIPLE WHATIDS***/
            //Hides the search fields
            $(this).find('.sfSearchWhatTextBox,.sfWhatSearchBtn,br, .ebsta-glyphicon-option-after').each(function () {
                $(this)[0].style.display = "none";
            });

            $(this).append(btn);
        });

        elemToRemove.remove();

        new ComplexAdd().CheckForLeads();

    });

    return obj;
    }

    //Remove object from the box and appends back onto original list
    function AddEventListenerToComplexRemoveObject(obj) {
    obj.addEventListener("click", function (event)
    {
        //Hide the hover over box once something has been selected
        $('.sfRecordHoverDiv').removeClass('userSelecting').hide();

        var whatId = "";
        var recordType = "";
        var whatName = "";

        var elemToRemove = null;

        $(this).closest('.CABoxStyle').each(function ()
        {
            whatId = $(this).attr("whatid");
            recordType = $(this).attr("recordType");
            whatName = $(this).attr("recordName");

            //Remove the item from the searchox
            elemToRemove = $(this);
        });

        //Remove row if empty
        if (($(this).closest('.sfAddComplexRow').attr('sfaddcomplexrowwhoid') == ""
                || $(this).closest('.sfAddComplexRow').attr('sfaddcomplexrowwhoid') == null
                || $(this).closest('.sfAddComplexRow').attr('sfaddcomplexrowwhoid') == undefined
                || $(this).closest('.sfAddComplexRow').attr('sfaddcomplexrowwhoid') == "undefined")) {
            $(this).closest('.sfAddComplexRow').remove();
        }

        //sfAddComplexRow = Complex add ---- sfEmailTemplateSearch = Template Search
        var elem = $(this).closest('.sfAddComplexRow,#emailTemplateBox')[0];
        //Create new box from removed element
        $(elem).find('.sfRowWhatSuggestRight, .sfEmailTemplateSearch').each(function () {
            var newDiv = null;
            var addIcon = CreateAddIcon();
            //AddEventListenerToComplexAddedObject(addIcon);

            if (recordType.toLowerCase() == "opp") {
                newDiv = CreateComplexAddObjectDiv(CheckStringLength(whatName, 37), recordType, "CABoxStyle sfSuggestOpp sfSuggestGrey", addIcon);
            }
            else
            {
                newDiv = CreateComplexAddObjectDiv(CheckStringLength(whatName, 37), recordType, "CABoxStyle sfSuggest" + recordType + " sfSuggestGrey", addIcon);
            }

            newDiv.setAttribute("WhatId", whatId);
            newDiv.setAttribute("recordtype", recordType);
            newDiv.setAttribute("recordName", whatName);

            AddEventListenerToComplexAddedObject(newDiv);

            //If the initial object was a search result put back in result list
            if ($(elemToRemove).hasClass('result'))
            {
                $(newDiv).addClass('result');
                if($(this).hasClass('sfEmailTemplateSearch'))
                {
                    $(this).find('.SARightSegment .resultsWindow').append(newDiv);
                }
                else
                {
                    $(this).closest('.SARightSegment').find('.resultsWindow').append(newDiv);
                }
            }
            else
            {
                $(this).find('.sfSuggestRowRightSuggestions').after(newDiv);
            }

        });

        //Remove whatid from the attributes of sfaddcomplexrowwhatids
        var whatIdArray = $(this).closest('.sfAddComplexRow').attr('sfaddcomplexrowwhatids');
        if (whatIdArray != null && whatIdArray != undefined) {
            whatIdArray = whatIdArray.trim().split(",");
        
            var newWhatIdArray = "";
            for (var i = 0; i < whatIdArray.length; i++)
            {
                if (whatId != whatIdArray[i] && whatIdArray[i].length > 0)
                {
                    newWhatIdArray += whatIdArray[i] + ",";
                }
            }
            whatIdArray = newWhatIdArray;
            $(this).closest('.sfAddComplexRow').attr('sfaddcomplexrowwhatids', whatIdArray);
        }

        /****NOT MULTIPLE WHATIDS***/
        //Find if box containcs element already
        var noOfElements = 0;

        $(this).closest('.sfWhatSearch').find('.CABoxStyle').each(function () {
            noOfElements++;
        });

        //Hides the search fields
        if (noOfElements>0)
        {
            $(this).closest('.sfWhatSearch').find('.sfSearchWhatTextBox,.sfWhatSearchBtn,br, .ebsta-glyphicon-option-after').each(function ()
            {
                $(this)[0].style.display = "inline-block";
            });
        }

        /**End **/

        elemToRemove.remove();

    });

    return obj;
    }

    //Adds a click listener to each results in the results box
    function AddListenerToResultsList(obj) {
    obj.addEventListener("click", function (event) {

        //Hide the hover over box once something has been selected
        $('.sfRecordHoverDiv').removeClass('userSelecting').hide();

        var item = $(this).closest('.result')[0];

        var isWhoSearch = false, isWhatSearch = false;
        var recordType = "";

        //Find if this is a who or what search
        if ($(this).closest('.result').attr("whatid") != null) {
            isWhatSearch = true;
        }
        else if ($(this).closest('.result').attr("whoid") != null) {
            isWhoSearch = true;
        }

        if ($(this).closest('.result').attr("recordtype") != null) {
            recordType = $(this).closest('.result').attr("recordtype");
        }

        var innerText = $(this).closest('.result').find('.sfSearchResultNameText')[0].innerText;

        var row = null;

        var isThisForTemplates = false;

        //Find the div
        $(this).closest('.sfAddComplexRow, .sfEmailTemplateSearch').each(function () {
            row = $(this)[0];
            //Lets pick up if this search is in the templates screen so we can prevent other functionalilty from happening.
            if ($(this).hasClass('sfEmailTemplateSearch')) {
                isThisForTemplates = true;
            }
        });

        if (isWhoSearch) {

            //Remove the textbox
            $(row).find('.sfSearchWhoTextBox').each(function () {
                $(this).remove();
            });

            //Remove the Search btn
            $(row).find('.sfWhoSearchBtn').each(function () {
                $(this).remove();
            });

            //Add New button to box
            $(row).find('.sfWhoSearch').each(function () {

                var crossIcon = CreateCrossIcon();
                AddListenerToWhoButtonToRemove(crossIcon);

                var btn = CreateComplexAddObjectDiv(CheckStringLength(innerText, 37), recordType, "CABoxStyle sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length), crossIcon);
                $(btn).find('.mainIcon').removeClass(recordType + "Icon");
                $(btn).find('.mainIcon').removeClass("CustomIcon");
                //$(btn).find('.mainIcon').addClass("selected" + recordType + "Icon");
                ChangeIconToReflectSelectedObject(btn, recordType);
                btn.setAttribute("recordType", recordType);
                btn.setAttribute("sfwhoid", item.getAttribute("whoid"));
                btn.setAttribute("recordName", innerText);

                $(this)[0].appendChild(btn);
            });

            //if (recordType == "Contact") {
            //    RequestEmailRelatedObjectLookup(item.getAttribute("whoid"));
            //}

            row.setAttribute("sfaddcomplexrowwhoid", item.getAttribute("whoid"));

            if (recordType.toLowerCase() == "contact") {
                $(row).find('.sfGetSuggestRow').each(function () {
                    $(this)[0].style.visibility = "visible";
                    $(this)[0].style.display = "block";
                });

                $(row).find('.sfRowGetSuggestRight').find('img').each(function() {
                    $(this)[0].style.visibility = "visible";
                });

                $(row).find('.sfRowGetSuggestRight').find('.sfLoadingSuggestions').each(function() {
                    $(this)[0].style.visibility = "visible";
                });

                if (!isThisForTemplates) {
                    if (whoWhatEnabledObject != null) {
                        if (whoWhatEnabledObject.whatEnabled) {

                            RequestEmailRelatedObjectLookup(item.getAttribute("whoid"));

                            //If there isn't already anything chosen in the Related To field then go find if there is a related to object to use
                            if ($(row).find('.sfWhatSearch > .CABoxStyle').length == 0)
                            {
                                //If there is an associated account with this record chosen then pre-populate this into the Related To textbox
                                if (item) {
                                    if (item.getAttribute("relatedwhatid")) {
                                        var whatId = item.getAttribute("relatedwhatid");
                                        var whatName = item.getAttribute("relatedwhatname");
                                        var recordType = item.getAttribute("relatedwhatrecordtype");

                                        //Create Related To Object to pre-populate
                                        var whatCrossIcon = CreateCrossIcon();
                                        whatCrossIcon.className = whatCrossIcon.className + " ebsta-complexadd-remove-object";

                                        whatIdElem = CreateComplexAddObjectDiv(CheckStringLength(whatName, 37), recordType, "CABoxStyle sfSuggest" + recordType, whatCrossIcon);
                                        whatIdElem.setAttribute("whatId", whatId);
                                        whatIdElem.setAttribute("recordType", recordType);
                                        whatIdElem.setAttribute("recordName", whatName);
                                        AddEventListenerToComplexRemoveObject(whatCrossIcon);

                                        //Hide certain elements now that we're going to pre-populate the Related To field
                                        $(row).find('.sfWhatSearchBtn, .sfSearchWhatTextBox, br, .ebsta-glyphicon-option-after').hide();

                                        //Update Row attributes
                                        $(row).attr("sfaddcomplexrowwhatids", whatId);

                                        //Append Account Object into Related To Field
                                        $(row).find('.sfWhatSearch').append(whatIdElem);
                                    }
                                }
                            }
                        } else {
                            $(row).find('.sfGetSuggestRow, .sfLoadingSuggestions, img').hide();
                            $(row).find('.sfArrow').removeClass('sfArrowDown').addClass('sfArrowRight');
                        }
                    }
                }

            }

            //Remove results Window
            $(row).find('.resultsWindow').each(function () {
                $(this).remove();
            });

        }
        else if (isWhatSearch) {
            //Add New button to box
            $(row).find('.sfWhatSearch').each(function () {

                var isAResult = false;

                if ($(item).hasClass('result')) {
                    isAResult = true;
                }

                /****NOT MULTIPLE WHATIDS***/
                //Remove element if its already in the list
                //Remove element if its already in the list
                $(this).find('.CABoxStyle').each(function () {
                    var whatId = $(this).attr("whatid");
                    var recordType = $(this).attr("recordType");
                    var whatName = $(this).attr("recordName");
                    var addIcon = CreateAddIcon();

                    var btn2 = CreateComplexAddObjectDiv(CheckStringLength(whatName, 37), recordType, "CABoxStyle sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length) + " sfSuggestGrey", addIcon);
                    btn2.setAttribute("whatid", whatId);
                    btn2.setAttribute("recordType", recordType);
                    btn2.setAttribute("recordName", whatName);

                    AddEventListenerToComplexAddedObject(btn2);

                    //Add removed object to suggestion list or results list depending on what the initial object was
                    if ($(this).hasClass('result')) {
                        $(btn2).addClass('result');
                        $(this).closest('.sfAddComplexRow, .sfEmailTemplateSearch .SARightSegment').find('.resultsWindow').each(function () {
                            $(this).append(btn2);
                        });
                    }
                    else {
                        $(this).closest('.sfAddComplexRow').find('.sfSuggestRowRightSuggestions').each(function () {
                            $(this).after(btn2);
                        });
                    }

                    $(this).remove();
                });

                /****NOT MULTIPLE WHATIDS***/
                //Hides the search fields
                $(this).find('.sfSearchWhatTextBox,.sfWhatSearchBtn,br, .ebsta-glyphicon-option-after').each(function () {
                    $(this)[0].style.display = "none";
                });

                //var btn = CreateBt(item.innerHTML, "sfWhatBtn", item.getAttribute("whatid"));
                var crossIcon = CreateCrossIcon();
                crossIcon.className = crossIcon.className + " ebsta-complexadd-remove-object";

                AddEventListenerToComplexRemoveObject(crossIcon);

                var btn = CreateComplexAddObjectDiv(CheckStringLength(innerText, 37), recordType, "CABoxStyle sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length), crossIcon);
                $(btn).find('.mainIcon').removeClass(recordType + "Icon");
                $(btn).find('.mainIcon').removeClass("CustomIcon");
                //$(btn).find('.mainIcon').addClass("selected" + recordType + "Icon");
                ChangeIconToReflectSelectedObject(btn, recordType);
                btn.setAttribute("whatid", item.getAttribute("whatid"));
                btn.setAttribute("recordType", recordType);
                btn.setAttribute("recordName", innerText);

                if (isAResult) {
                    $(btn).addClass('result');
                }

                //AddEventListenerToComplexRemoveObject(btn);
                $(this)[0].appendChild(btn);
            });

            /****NOT MULTIPLE WHATIDS***/
            //Append new whatids
            //if (row.getAttribute("sfaddcomplexrowwhatids") != null)
            //{
            //    row.setAttribute("sfaddcomplexrowwhatids", row.getAttribute("sfaddcomplexrowwhatids") + "," + item.getAttribute("whatid"));
            //} else {
            row.setAttribute("sfaddcomplexrowwhatids", item.getAttribute("whatid"));
            //}     

        }

        $(this).remove();

        if (!isThisForTemplates) {
            CreateNewComplexAddRow();
        }

        new ComplexAdd().CheckForLeads();
    });
    }

    //Changes the icon to match the object
    function ChangeIconToReflectSelectedObject(btn, recordType)
    {
    if (recordType.toLowerCase() == "opp"
        || recordType.toLowerCase() == "account"
        || recordType.toLowerCase() == "lead"
        || recordType.toLowerCase() == "contact"
        || recordType.toLowerCase() == "case") {
        $(btn).find('.mainIcon').addClass("selected" + recordType + "Icon");
    }
    else
    {
        $(btn).find('.mainIcon').addClass("selectedCustomIcon");
    }
    }

    //Displays the search results
    function DisplaySearchResults(entityName, json, search) {

    var parsedJson = "";
    try {
        parsedJson = JSON.parse(JSON.parse(json).d.Values[0]);
    }
    catch (error) {

    }

    var currentSearchBox = null;

    var whoSearch = false;
    var whatSearch = false;
    var complexAddSearch = false, templateSearch = false;

    //Tracks to see if this is an initial open window search for emails
    var firstWindowInstance = true;
    //Find box where search come from
    $('#ebstaBox, #emailTemplateBox').find('span[lastsearch="true"]').each(function ()
    {
        firstWindowInstance = false;

        var lastSearch = $(this).attr("lastsearch");

        if (lastSearch.toLowerCase() == "true") {
            //Find if it was a Who or What search
            $(this).parent().find('.sfSearchWhoTextBox').each(function () {
                currentSearchBox = $(this)[0];
                whoSearch = true;
                $(this).parent().find('.sfWhoSearchBtn,.sfWhatSearchBtn').removeClass('magnifySearchIcon').addClass('crossSearchIcon');
            });

            $(this).parent().find('.sfSearchWhatTextBox').each(function () {
                currentSearchBox = $(this)[0];
                whatSearch = true;
                $(this).parent().find('.sfWhoSearchBtn,.sfWhatSearchBtn').removeClass('magnifySearchIcon').addClass('crossSearchIcon');
            });
        }

        $(this).closest('.CASegment').find('.resultsLoader, .resultsWindow').each(function() {
            $(this).remove();
        });

        $(this).parent().find('.sfLoadingResults').each(function() {
            $(this).slideUp();
        });

        $(this).parent().find('img').each(function() {
            $(this).remove();
        });

        if ($(this).closest('#ebstaBox').length > 0) {
            complexAddSearch = true;
        } else if ($(this).closest('#emailTemplateBox').length > 0) {
            templateSearch = true;
        }
        //$('.resultsLoader').hide();

        //return false;
        });

    //Remove already visible results window
    if (whoSearch) {
        $('#ebstaBox').find('.SALeftSegment ').find('.resultsLoader').each(function () {
            if ($(this).css("display") != "none") {
                $(this).parent().find('.resultsWindow').remove();
                $(this).remove();
            }
        });
    }
    else if (whatSearch)
    {
        $('#ebstaBox').find('.SARightSegment').find('.resultsLoader').each(function () {
            if ($(this).css("display") != "none") {
                $(this).parent().find('.resultsWindow').remove();
                $(this).remove();
            }
        });
    }

    //If this is the first open window instance display potential results which weren't cached
    if (firstWindowInstance) {
        var complexAddSuggestionBox = document.getElementsByClassName('sfRowWhoSuggestLeft');
        var templateSuggestionBox = document.getElementsByClassName('sfEmailTemplateRowWhoSuggestLeft');

        //Find if this is for complex add or templates
        var rowSuggestLeft, isThisComplexAdd = false;
        if (complexAddSuggestionBox.length) {
            rowSuggestLeft = document.getElementsByClassName('sfRowWhoSuggestLeft')[0];
            isThisComplexAdd = true;
        } 
        
        if (templateSuggestionBox.length) {
            rowSuggestLeft = document.getElementsByClassName('sfEmailTemplateRowWhoSuggestLeft')[0];
        }

        if (rowSuggestLeft) {
            if (parsedJson.length > 0) {
                for (var i = 0; i < parsedJson.length; i++) {
                    var recordType = parsedJson[i]["attributes"]["type"];
                    var id = parsedJson[i]["Id"];
                    var name = parsedJson[i]["Name"];
                    var emailAddress = parsedJson[i]["Email"];

                    var whatId = null, whatName = null, whatRecordType = null;
                    //If we've got some history of a 'Related To' object for this record then lets get those results so we can pre-populate the the Related To box
                    var result = LookForRelatedToObject(id);

                    if (result) {
                        if (result.WhatRecordId) {
                            whatId = result.WhatRecordId;
                        }

                        if (result.WhatRecordName) {
                            whatName = result.WhatRecordName;
                        }

                        if (result.WhatRecordType) {
                            whatRecordType = result.WhatRecordType;
                        }
                    }

                    //If we didn't find any previous selection for this object we'll default it to the associated Account (if there is any)
                    if (whatId == null) {
                        //Find Account Details from search results
                        if (parsedJson[i]["Account"] != null) {
                            if (parsedJson[i]["Account"].Id != null) {
                                whatId = parsedJson[i]["Account"].Id;
                            }
                            if (parsedJson[i]["Account"].Name != null) {
                                whatName = parsedJson[i]["Account"].Name;
                            }

                            whatRecordType = "Account";
                        }
                    }

                    if (recordType.indexOf("Contact") != -1 || recordType.indexOf("Lead") != -1) {
                        //has this lead been converted already if so that means they can't add/update anything to it
                        if (recordType.indexOf("Lead") != -1 && parsedJson[i]["IsConverted"] && complexAddSearch) {
                            if (parsedJson[i]["IsConverted"] == true) {
                                continue;
                            }
                        }

                        if (isThisComplexAdd) {
                            //If we haven't pre-populated the Complex add box with any results lets put the first result from here into there
                            if ($('.sfAddComplexRow').length == 1) {
                                CreateNewComplexAddRow(id, name, recordType, emailAddress, whatId, whatName, whatRecordType);
                                CreateNewComplexAddRow();
                            } else {
                                //Make sure we haven't made a record for this already within the complex add screen
                                if ($('div.sfAddComplexRow[sfaddcomplexrowwhoid="' + id + '"').length == 0 && $('div.sfEmailSuggest[whoid="' + id + '"').length == 0) {
                                    var emailDiv = CreateComplexAddEmailSuggestions(name, emailAddress, recordType, id, whatId, whatName, whatRecordType);
                                    rowSuggestLeft.appendChild(emailDiv);
                                }
                            }
                        } else {
                            var emailDiv = CreateComplexAddEmailSuggestions(name, emailAddress, recordType, id, whatId, whatName, whatRecordType);
                            rowSuggestLeft.appendChild(emailDiv);
                        }
                    }
                } 
            }
        }

        //In templates if we've found suggestions but nothing is currently pre-populated then put the first suggestion in
        if (!isThisComplexAdd) {
            $('#emailTemplateBox').find('.sfEmailTemplateRowWhoSuggestLeft').fadeIn();
            if ($('#emailTemplateBox').find('.sfWhoSearch > .CABoxStyle').length == 0) {
                $('#emailTemplateBox').find('.sfEmailTemplateRowWhoSuggestLeft > .sfEmailSuggest').each(function () {
                    $(this).trigger("click");
                    return true;
                });
            }
        }

        $('.sfRowWhoSuggest, .sfPeopleSuggestionsHeader').find('.resultsLoader').remove();

        //Remove People Suggestions if there are no suggestions
        if ($(rowSuggestLeft).find('.sfEmailSuggest').length == 0) {
            $(rowSuggestLeft).parent().find('.sfPeopleSuggestionsHeader').hide();
        }
    }
    else if (currentSearchBox != null)
    {

        var resultsWindows = CreateDiv("", "resultsWindow");

        var whoObjectsFound = false;
        var whatObjectsFound = false;

        if (parsedJson.length > 0)
        {
            for (var i = 0; i < parsedJson.length; i++)
            {
                var recordType = parsedJson[i]["attributes"]["type"];

                if (whoSearch)
                {
                    if (recordType.toLowerCase() == ("contact") || recordType.toLowerCase() == ("lead"))
                    {
                        //Has this lead been converted already if so that means they can't add/update anything to it
                        if (recordType.indexOf("Lead") != -1 && parsedJson[i]["IsConverted"] && complexAddSearch) {
                            if (parsedJson[i]["IsConverted"] == true) {
                                continue;
                            }
                        }

                        whoObjectsFound = true;

                        var id = parsedJson[i]["Id"];
                        var name = parsedJson[i]["Name"];
                        var emailAddress = parsedJson[i]["Email"];
                        
                        var whatId = null, whatName = null, whatRecordType = null;
                        //If we've got some history of a 'Related To' object for this record then lets get those results so we can pre-populate the the Related To box
                        var result = LookForRelatedToObject(id);

                        if (result) {
                            if (result.WhatRecordId) {
                                whatId = result.WhatRecordId;
                            }

                            if (result.WhatRecordName) {
                                whatName = result.WhatRecordName;
                            }

                            if (result.WhatRecordType) {
                                whatRecordType = result.WhatRecordType;
                            }
                        }

                        //Lets make sure we haven't chosen this particular match already within the complex add screen
                        var isSelectionAlreadyThere = false;
                        var isAccountAlreadySelected = false; //Lets also make sure the Account object hasn't been selected also so we can prevent it from pre-populating again
                        $('.sfAddComplexRow').each(function () {
                            if ($(this).attr('sfaddcomplexrowwhoid') == id && $(this).attr('sfaddcomplexrowwhatids'))
                            {
                                isSelectionAlreadyThere = true;
                                if ($(this).find('.sfWhatSearch > .sfSuggestAccount').length > 0)
                                {
                                    isAccountAlreadySelected = true;
                                }
                            }
                        });

                        //Find Account Details from search results if we didn't find anything from previous history or we've already chosen the combination of WhoId and WhatId
                        if (whatId == null || (isSelectionAlreadyThere && !isAccountAlreadySelected)) {
                            if (parsedJson[i]["Account"] != null) {
                                if (parsedJson[i]["Account"].Id != null) {
                                    whatId = parsedJson[i]["Account"].Id;
                                }
                                if (parsedJson[i]["Account"].Name != null) {
                                    whatName = parsedJson[i]["Account"].Name;
                                }

                                whatRecordType = "Account";
                            }
                        }

                        //Reset values to null as we've already pre-populated this selection
                        if (isSelectionAlreadyThere && isAccountAlreadySelected) {
                            whatId = null;
                            whatName = null;
                            whatRecordType = null;
                        }

                        var addIcon = CreateAddIcon();

                        var combinedTxt = name;
                        if (emailAddress != null) {
                            combinedTxt +=" (" + emailAddress + ")";
                        }

                        var resultDiv = CreateComplexAddObjectDiv(CheckStringLength(combinedTxt, 37), recordType, "result CABoxStyle sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length) + " sfSuggestGrey ebsta-template-lookup-whoid", addIcon);
                        resultDiv.setAttribute("whoid", id);
                        resultDiv.setAttribute("recordType", recordType);
                        resultDiv.setAttribute("recordName", name);

                        //Append Related To object Details onto the search result so we can use them to create the Related To object once a user has selected it
                        if (whatId)
                        {
                            resultDiv.setAttribute("relatedWhatId", whatId);

                            if (whatName) {
                                resultDiv.setAttribute("relatedWhatName", whatName);
                            }

                            if (whatRecordType) {
                                resultDiv.setAttribute("relatedWhatRecordType", whatRecordType);
                            }
                        }

                        AddListenerToResultsList(resultDiv);
                        resultsWindows.appendChild(resultDiv);
                    }
                }
                else if (whatSearch)
                {
                    if (recordType.toLowerCase() != ("contact") && recordType.toLowerCase() != ("lead"))
                    {
                        whatObjectsFound = true;

                        var id = parsedJson[i]["Id"];
                        var name = "";
                        if (parsedJson[i]["Name"] != null)
                        {
                            name = parsedJson[i]["Name"];
                        }
                        else if (parsedJson[i]["SolutionName"] != null && parsedJson[i]["SolutionNumber"] != null)
                        {
                            name = parsedJson[i]["SolutionName"];
                            name = name + " (" + parsedJson[i]["SolutionNumber"] + ")";
                        }
                        else if (parsedJson[i]["Subject"] != null)
                        {
                            name = parsedJson[i]["Subject"];
                        }
                        else if (parsedJson[i]["OrderNumber"] != null && parsedJson[i]["Account"]["Name"] != null)
                        {
                            name = parsedJson[i]["OrderNumber"];
                            name = name + " (" + parsedJson[i]["Account"]["Name"] + ")";
                        }
                        else if (parsedJson[i]["ContractNumber"] != null && parsedJson[i]["Account"]["Name"] != null)
                        {
                            name = parsedJson[i]["ContractNumber"];
                            name = name + " (" + parsedJson[i]["Account"]["Name"] + ")";
                        }

                        if (recordType.toLowerCase() == "opportunity" && parsedJson[i]["StageName"] != null)
                        {
                            //If this is a closed op don't show
                            //if (parsedJson[i]["StageName"].toLowerCase().indexOf("closed") != -1)
                            //{
                            //    continue;
                            //}

                            recordType = "Opp";
                            name = name + " (" + parsedJson[i]["StageName"] + ")";
                        }

                        if (recordType.toLowerCase() == "case" && parsedJson[i]["CaseNumber"] != null)
                        {
                            var caseDivText = "";
                            var caseNumber = parsedJson[i]["CaseNumber"];

                            if (caseNumber.length == 0) {
                                if (name.length != 0) {
                                    caseDivText = name;
                                } else {
                                    caseDivText = id;
                                }
                            } else {
                                if (name.length != 0) {
                                    caseDivText = caseNumber + " - " + name;
                                } else {
                                    caseDivText = caseNumber;
                                }
                            }

                            name = caseDivText;
                        }

                        var addIcon = CreateAddIcon();  

                        var resultDiv = CreateComplexAddObjectDiv(CheckStringLength(name, 33), recordType, "result CABoxStyle sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length) + " sfSuggestGrey ebsta-template-lookup-whatid", addIcon);

                        resultDiv.setAttribute("whatid", id);
                        resultDiv.setAttribute("recordType", recordType);
                        resultDiv.setAttribute("recordName", name);

                        AddListenerToResultsList(resultDiv);
                        resultsWindows.appendChild(resultDiv);
                    }
                }
            }
        }

        if(!(whoSearch && whoObjectsFound) && !(whatSearch && whatObjectsFound))
        {
            var resultDiv = CreateDiv("No results found. Please search again.", "result");
            resultsWindows.appendChild(resultDiv);
        }

        $(currentSearchBox).closest('.sfAddComplexRow').find('.sfArrowRight').removeClass("sfArrowRight").addClass("sfArrowDown");

        var parentNode = currentSearchBox.parentNode;

        $(parentNode).after(resultsWindows);
        //$(resultsWindows).after(CreateBr());
        $(resultsWindows).hide().slideDown("500", function () {
            ScrollToPosition($(currentSearchBox));
        });

        //currentSearchBox.parentNode.insertBefore(resultsWindows, currentSearchBox.nextSibling);
    }

    }

    //Shows Related Objects returned for selected Contact Id
    function ShowRelatedToObjects(entityName, json, search)
    {
        CreateDivForSuggestions(search);

        var sfRowSuggestRight = null;
        var selectedWhatId = "";

        $("div[sfaddcomplexrowwhoid='" + search + "']").find('.sfRowWhatSuggestRight').each(function () {
            sfRowSuggestRight = $(this)[0];

            //Hide loader gif
            $(this).closest('.sfAddComplexRow').find('img').each(function () {
                $(this)[0].style.visibility = "hidden";
            });

            //Hide loader gif
            $(this).closest('.sfAddComplexRow').find('.sfGetSuggestRow').each(function () {
                $(this)[0].style.display = "none";
            });

            selectedWhatId = ''
            //Find if anything is already been selected the Related To field so we don't create dupes when Related To objects are returned
            if ($(this).closest('.sfAddComplexRow').attr('sfaddcomplexrowwhatids')) {
                selectedWhatId = $(this).closest('.sfAddComplexRow').attr('sfaddcomplexrowwhatids');
            }
        });

        var noOfRelatedObjectsCreated = 0;

        //Show results
        if (json) {
            var json = JSON.parse(json);
            if (json != null) {
                if (json.d != null) {
                    if (json.d.Values != null) {
                        var values = json.d.Values;
                        for (var x = 0; x < values.length; x++) {
                            var parsedObject = JSON.parse(values[x]);
                            if (parsedObject != null) {
                                //Loop through the parsed object in the order the results were returned
                                for (var key in parsedObject) {
                                    var objectType = key;

                                    if (parsedObject.hasOwnProperty(key)) {
                                        //Get list of results for the object type
                                        var arrayOfResults = parsedObject[key];
                                        for (var i = 0; i < arrayOfResults.length; i++) {
                                            var addIcon = CreateAddIcon();
                                            var parsedResultObject = arrayOfResults[i];

                                            var name = parsedResultObject["Name"];
                                            var id = parsedResultObject["Id"];

                                            if (id != null && name != null) {
                                                //Skip over if this already exists in the Related To field
                                                if (id == selectedWhatId) {
                                                    continue;
                                                }

                                                var resultSuggestionDiv = null;

                                                if (objectType.toLowerCase() === "account") {
                                                    resultSuggestionDiv = CreateComplexAddObjectDiv(CheckStringLength(name, 37), "Account", "CABoxStyle sfSuggestAccount sfSuggestGrey ebsta-complexadd-suggestion-object", addIcon);
                                                    resultSuggestionDiv.setAttribute("WhatId", id);
                                                    resultSuggestionDiv.setAttribute("recordType", "Account");
                                                    resultSuggestionDiv.setAttribute("recordName", name);
                                                } else if (objectType.toLowerCase() === "case") {
                                                    var caseNumber = parsedResultObject["CaseNumber"];

                                                    if (caseNumber != null) {
                                                        var caseDivText = "";
                                                        if (caseNumber.length === 0) {
                                                            if (name != null && name.length !== 0) {
                                                                caseDivText = name;
                                                            } else {
                                                                caseDivText = id;
                                                            }
                                                        } else {
                                                            if (name != null && name.length !== 0) {
                                                                caseDivText = caseNumber + " - " + name;
                                                            } else {
                                                                caseDivText = caseNumber;
                                                            }
                                                        }

                                                        var labelContent = CheckStringLength(caseDivText, 37);

                                                        resultSuggestionDiv = CreateComplexAddObjectDiv(labelContent, "Case", "CABoxStyle sfSuggestCase sfSuggestGrey ebsta-complexadd-suggestion-object", addIcon);
                                                        resultSuggestionDiv.setAttribute("WhatId", id);
                                                        resultSuggestionDiv.setAttribute("recordType", "Case");
                                                        resultSuggestionDiv.setAttribute("recordName", labelContent);
                                                        resultSuggestionDiv.setAttribute("title", "Case");
                                                    }
                                                } else if (objectType.toLowerCase() === "opportunity") {
                                                    var stage = parsedResultObject["Stage"];
                                                    var oppLabel = CheckStringLength(name + " (" + stage + ")", 37);

                                                    resultSuggestionDiv = CreateComplexAddObjectDiv(oppLabel, "Opp", "CABoxStyle sfSuggestOpp sfSuggestGrey ebsta-complexadd-suggestion-object", addIcon);
                                                    resultSuggestionDiv.setAttribute("WhatId", id);
                                                    resultSuggestionDiv.setAttribute("recordType", "Opp");
                                                    resultSuggestionDiv.setAttribute("recordName", name + " (" + stage + ")");
                                                }

                                                AddEventListenerToComplexAddedObject(resultSuggestionDiv);

                                                if (sfRowSuggestRight != null) {
                                                    sfRowSuggestRight.appendChild(resultSuggestionDiv);
                                                    noOfRelatedObjectsCreated++;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        $(sfRowSuggestRight).hide().slideDown(function () {
            ScrollToPosition($(this).closest('.sfAddComplexRow').find('.SALeftSegment'));
        });
    
        if(noOfRelatedObjectsCreated == 0)
        {
            if (sfRowSuggestRight != null) {
                //No Suggestions found
                var noSuggest = CreateDiv("No suggestions found", "sfNoSuggestCaseFound");

                sfRowSuggestRight.style.overflowY = "hidden";
                sfRowSuggestRight.parentNode.insertBefore(noSuggest, sfRowSuggestRight.parentNode.firstChild);

                $(noSuggest).hide().fadeIn("500", function () {
                    ScrollToPosition($(this));
                });

                //Remove the Other Suggestions div if nothing to show
                $(sfRowSuggestRight).closest('.sfRowWhatSuggest').find('.suggestionListTitle').each(function () {
                    $(this).remove();
                });
            }
        }

    }

    function BlockLeadValue(val) {

    var whoIdResult = $('#sfWhoId').val();

    if (val == 1) {
        if (whoIdResult != "Lead") {
            $("#txtWhoName").removeAttr("readonly", "");
            $("#txtWhoName").attr("sfid", "-1");
            $("#txtWhoName").val("");
            $("#txtWhoName").removeClass("readonly");
            $("#btLookupWhoId").text("Lookup");
        }
    }
    }

    function CheckLeadStatus() {
        var whoIdResult = $('#sfWhoId').val();

        if (whoIdResult == "Lead") {

            $("#txtWhatName").attr("sfid", "-1");
            $("#txtWhatName").attr("readonly", "");
            $("#txtWhatName").val("");
            $("#sfWhatId").prop("disabled", true);
            $("#btLookupWhatId").prop("disabled", true);
        } else {
            $("#sfWhatId").prop("disabled", false);
            $("#btLookupWhatId").prop("disabled", false);
            $("#txtWhatName").removeAttr("readonly", "");
            $("#txtWhatName").removeClass("readonly");        
        }
    }

    function SaveComplexAdd(obj) {
        var title = $('#txtSubject').val();

    
        var valueOfWhoId = $('#txtWhoName').attr('sfId');
        var valueOfWhatId = $('#txtWhatName').attr('sfId');

        var whoName = "" + $('#txtWhoName').val();
        whoName = whoName.trim();
    
        var whatName = "" + $('#txtWhatName').val();
        whatName = whatName.trim();

        if (valueOfWhatId == -1) {
            name = whoName;
        } else {
            name = whatName;

            if (valueOfWhoId != -1) {
                name = name + " / " + whoName; 
            }
        }

        chrome.extension.sendMessage({
            MessageType: "AddTaskEmailComplex", Email: "NEW", EmailText: obj.EmailText, Time: obj.Time, Title: title, Msg: obj.Msg, SFType: "NEW", WhoId: valueOfWhoId, WhatId: valueOfWhatId, Name: name, FoundDate: obj.FoundDate
        }, function (response) {
            // Response back show box or do something amzing.
        }
            );

    }

    function RunSearchLookup(objectType, objectSearch) {

        //var results = document.getElementById("entityResults");

        //if (results != null) {
        //    results.appendChild(CreateImg("https://console.ebsta.com/images/google/ajax-loader.gif", "loading"));
            chrome.extension.sendMessage({
                MessageType: "EmailLookup",
                SearchTerm: objectSearch,
                Entity: objectType
            }, function (response) {
                // Response back show box or do something amzing.
            }
            );
        //}
    }

    var showLookup = false;

    function ShowSearchLookup(lookupType,objectType,objectSearch) {
        if (!showLookup) {
            showLookup = true;

            var divBox = document.createElement('div');
            divBox.id = "ebstaBoxLookup";
            divBox.setAttribute("class", "sfBoxLookup");
            divBox.appendChild(CreateSpan("Please search for the " + objectType, "sftitle-textOld"));
            divBox.appendChild(CreateSpan("", "sf-close"));
            var divRow = CreateRow("row");
        
            divRow.appendChild(CreateSpan("Search", "sfSearchLookup"));
            var box = CreateTxtbox("sfText", "sfText", "sfTxtSearch");
            box.setAttribute("value", objectSearch);
            divRow.appendChild(box);
            divRow.appendChild(CreateBt("Search", "sfbt", "sfbtSearch"));

            divBox.appendChild(divRow);        
        
            var results = CreateDivWithId("", "", "entityResults");
            results.setAttribute("sfType", lookupType);
        
            divBox.appendChild(results);

            (document.body || document.documentElement).appendChild(divBox);

            RunSearchLookup(objectType + '-ComplexAdd', objectSearch);

            $('#sfbtSearch').click(function () {

                objectSearch = $('#sfTxtSearch').val();
            
                RunSearchLookup(objectType + '-ComplexAdd', objectSearch);
    
            });

            $('#sfTxtSearch').live("keypress", function (e) {
                /* ENTER PRESSED*/
                if (e.keyCode == 13) {
                    objectSearch = $('#sfTxtSearch').val();

                    RunSearchLookup(objectType + '-ComplexAdd', objectSearch);
          
                }
            });
        

            $('.ebsta-button-darkCloseLookup').click(function () {
                CloseLookupWindow();
            });
        
            $('.sf-close').click(function () {
                CloseLookupWindow();
            });
        }

    }

    function TaskEmailFailed(msg) {
        $(".sfQuickAddFired").removeClass("sfQuickAdd").removeClass("sfQuickAddFired").addClass("sfQuickAddFail");

        alert("Failed to add Email to Salesforce " + msg);
    }

    function TaskEmailAdded() {
        //$(".sfQuickAdd").removeClass("sfQuickAdd").addClass("sfQuickAdded");
        //$(".sfComplexAdd").removeClass("sfComplexAdd").addClass("sfQuickAdded");
    
        // $(".sfQuickAddFired").removeClass("sfQuickAdd").removeClass("sfQuickAddFired");
        
        $(".sfComplexAddFired").removeClass("sfComplexAdd").removeClass("sfComplexAddFail").removeClass("sfComplexAddFired").addClass("sfComplexAdded");
        $(".sfQuickAddFired").removeClass("sfQuickAdd").removeClass("sfQuickAddFail").removeClass("sfQuickAddFired").addClass("sfQuickAdded");
    }



    function ApplyStyleEmailStyle_ConvertObject(obj) {
        ApplyStyleEmailStyle(obj.LookEmailAddress, obj.Id, obj.Type, obj.Server, obj.FoundMultipleRecords, obj.Url, obj.Name);
    }


function ApplyStyleEmailStyle(emailAddress, id, type, server, multipleRecords, url, name) {
        var multiRecordSpan = '';
        var multiRecordFlag = '0';
        var multiRecordClass = '';

        ////LogThis('apply email style...');

        if (multipleRecords) {

            //multiRecordSpan = '<span class="multiRecordSpan">+</span>';
            multiRecordFlag = '1';
            multiRecordClass = 'multiContact';
        } else {
            multiRecordSpan = '';
        }

        
        if (url == document.URL) {

            //console.time('ApplyStyleEmailStyle2');

            var searchEmailAddress = emailAddress.replace("'", "\\'");

            var msgBody = $("[email='" + searchEmailAddress + "']:visible");

            //console.timeEnd('ApplyStyleEmailStyle2');

            //console.time('ApplyStyleEmailStyle3');

            var inboxSearchClass = '.UI';

            if ($(".apP").length > 0) {
                inboxSearchClass = '.apP';
            }
            //console.timeEnd('ApplyStyleEmailStyle3');

            //if ($(".aDP").length > 0) {
            //    inboxSearchClass = '.aDP';
            //}
        
        

            if (msgBody != null) {
                //console.time('ApplyStyleEmailStyle4');

                $(inboxSearchClass).find(msgBody).each(function() {

                    var foundus = false;

                    $(this).parent().parent().children().each(function() {

                        if ($(this).attr("sf") != null) {
                            try {
                                //for multi-recipient feeds we should only render an icon for the first/most recent email in the list.
                                var renderIcon = true;

                                if ($(this).next().children() != null && typeof $(this).next().children() != "undefined") {
                                    if ($(this).next().children().length > 1) {

                                        var firstElement = $(this).next().children()[0];

                                        if (firstElement.getAttribute("email") != emailAddress) { //if the email doesn't match the first in the list, we don't want to render an icon.
                                            renderIcon = false;
                                        }
                                    }
                                }

                                //renderIcon = true; 
                                if (renderIcon) {

                                    var spnQuickAdd = CreateSpan("", 'sf' + type.toLowerCase() + ' sfOpen googleButtonStyle ' + multiRecordClass);
                                    spnQuickAdd.setAttribute("sf", "1");
                                    spnQuickAdd.setAttribute("email", emailAddress);
                                    spnQuickAdd.setAttribute("sfId", server + id);
                                    spnQuickAdd.setAttribute("instance", server);
                                    spnQuickAdd.setAttribute("mR", multiRecordFlag);
                                    spnQuickAdd.setAttribute("title", 'Open ' + name);
                                    spnQuickAdd.setAttribute("sfname", name);
                                    // spnQuickAdd.appendChild(multiRecordSpan);
                                    $(this).html(spnQuickAdd);


                                }
                                foundus = true;
                            } catch(error) {
                                //LogThis(error);
                            }
                        }

                    });

                    if (!foundus) {
                        var counter = 0;
                        $(this).parent().each(function() {

                            //for multi-recipient feeds we should only render an icon for the first/most recent email in the list.
                            var renderIcon = true;

                            if ($(this).next().children() != null && typeof $(this).next().children() != "undefined") {
                                if ($(this).next().children().length > 1) {

                                    var firstElement = $(this).next().children()[0];

                                    if (firstElement.getAttribute("email") != emailAddress) { //if the email doesn't match the first in the list, we don't want to render an icon.
                                        renderIcon = false;
                                    }
                                }
                            }

                            if ($(this).attr("id") != ":qy" && renderIcon) {
                                var divSF = CreateDiv("", "");
                                divSF.setAttribute("sf", "1");
                                
                                var spnQuick1 = CreateSpan("", 'sf' + type.toLowerCase() + ' sfOpen googleButtonStyle ' + multiRecordClass);
                                spnQuick1.setAttribute("email", emailAddress);
                                spnQuick1.setAttribute("sfId", server + id);
                                spnQuick1.setAttribute("instance", server);

                                spnQuick1.setAttribute("mR", multiRecordFlag);
                                spnQuick1.setAttribute("sf", "1");
                             
                                
                                spnQuick1.setAttribute("title", 'Open ' + name);
                                spnQuick1.setAttribute("sfname", name);

                                divSF.appendChild(spnQuick1); 
                                $(this).before(divSF);
                                
                                //$(this).before('<div sf="1">' +
                                //    '<span email="' + emailAddress + '"  sfId="' + server + id + '" instance="' + server + '" mR="' + multiRecordFlag + '" class="sf' + type.toLowerCase() + ' sfOpen googleButtonStyle ' + multiRecordClass + '" 
                                //title="Open ' + name + ' " sfname="' + name + '">' + multiRecordSpan + '</span></div>');

                            }
                                                  });
                    }

                });
                //console.timeEnd('ApplyStyleEmailStyle4');

            }
        }
        //console.timeEnd('ApplyStyleEmailStyle');

    }

    function ApplyStyleEmailStyleSpacing(emailAddress, url) {
        //console.time('ApplyStyleEmailStyleSpacing');

        if (url == document.URL) {
            $("*[email*='" + emailAddress + "'").each(function () {
                var foundus = false;


                if ($(this).attr("id") != ":qy") {
                    $(this).parent().parent().children().each(function () {
                        if ($(this).attr("sf") != null) {
                            //$(this).html('<span sfId="" class="sfNoMatch"></span>');
                            foundus = true;
                        }
                    });

                    if (!foundus) {
                        var counter = 0;
                        $(this).parent().each(function () {
                            $(this).before('<div sf="0"><span sfId="" class="sfNoMatch"></span></div>');
                        });
                    }
                }
            });
        }
        //console.timeEnd('ApplyStyleEmailStyleSpacing');

    }
    
    // If the request is returned after the buttons have rendered then this will look for any emails associated with the found ID
    // and change the buttons to show that this email has already been added
    function UpdateEbstaButtonsWithGmailToReflectEmailBeingAdded(listOfGoogleIds) {
        if (listOfGoogleIds) {
            if (listOfGoogleIds.d) {
                if (listOfGoogleIds.d.Values) {
                    if (listOfGoogleIds.d.Values.length > 0) {
                        $(".gH").find(".gK").find(".sfQuickAdd, .sfComplexAdd").each(function () {
                            if (GmailTools.MessageView.HasEmailBeenAddedAlready($(this), listOfGoogleIds.d.Values)) {
                                if ($(this).hasClass('sfQuickAdd')) {
                                    $(this).removeClass('sfQuickAdd').addClass('sfQuickAdded');
                                }

                                //Only display the added symbol to Complex Add button if the quick add button doesn't exist
                                if ($(this).hasClass('sfComplexAdd') && $(this).prev().hasClass('sfnoemailadd')) {
                                    $(this).removeClass('sfComplexAdd').addClass('sfComplexAdded');
                                }
                            }
                        });
                    }
                }
            }
        }
    }

    /**
     * Function gets called from connector.js as a result of a search in ProcessResults.
     * Calls GmailTools.ApplyStyleEmailStyleMsgConvertObject(obj), which makes decisions on what
     * to do in terms of adding relevant buttons to the email that is open, e.g complex add, quick add etc
     * @param {any} obj - Result object of search
     */
    function ApplyStyleEmailStyleMsg_ConvertObject(obj) {
        GmailTools.ApplyStyleEmailStyleMsgConvertObject(obj);
    }

    var setGoogleEvents = false;
    
    function SetGoogleClickEvent() {

        ////LogThis('set google click event...');
        
        $(".sfOpen").undelegate("click");
        $(".sfOpen").click(function(e) {

            e.stopImmediatePropagation();

            var email = $(this).attr("email");

            var fullName = $(this).attr("sfname");

            emailAddressToSearchJsonParse = email;
            fullnameToSearchJsonParse = fullName;
            var nameandemail = email + ",,," + fullName;

            if (useDefaultBarSearch)
            {
                proc.LoadFrame(email, 1);
            }
            else
            {
                proc.LoadFrame(nameandemail, 4);
            }
        });
        
            $(".sfnocontactLarge").undelegate("click");
            $(".sfOpenLarge").undelegate("click");
            $(".sfQuickAdd,.sfQuickAdded").undelegate("click");
            $(".sfComplexAdd, .sfComplexAdded").undelegate("click");

            //$(document).on("click", ".sfnocontactLarge", function (e) {
            $(".sfnocontactLarge").click(function(e) {
              
                e.stopImmediatePropagation();
                var url = $(this).attr("sfId");
                var email = $(this).attr("sfemailaddress");

                var fullName = "";

                $("[email='" + email + "']:visible").each(function() {
                    //title = "Email: " + $(this).text();
                    //console.log("Name Text: " + $(this).text());

                    fullName = $(this).text();

                    if(fullName.length > 0)
                    {
                        return false;
                    }

                    // outerText: "Cabrera, Stacey"
                }
                );

                emailAddressToSearchJsonParse = email;
                fullnameToSearchJsonParse = fullName;
                var nameandemail = email + ",,," + fullName;

                //console.log("search 6");
                if (useDefaultBarSearch)
                {
                    proc.LoadFrame(email, 1);
                }
                else
                {
                    proc.LoadFrame(nameandemail, 4);
                }
            });

          //  $(document).on("click", ".sfOpenLarge", function (e) {
           $(".sfOpenLarge").click(function(e) {

                e.stopImmediatePropagation();
                var url = $(this).attr("sfId");
                var email = $(this).attr("sfemailaddress");

                var fullName = $(this).attr("sfname");

                emailAddressToSearchJsonParse = email;
                fullnameToSearchJsonParse = fullName;

                //Custom Bar Search. Check if we are doing a default or custom bar search. Custom Bar Search is type 4
                var nameandemail = email + ",,," + fullName;

                if (useDefaultBarSearch)
                {
                    proc.LoadFrame(email, 1);
                }
                else
                {
                    proc.LoadFrame(nameandemail, 4);
                }
            });


         //  $(document).on("click", ".sfQuickAdd", function (e) {
        $(".sfQuickAdd, .sfQuickAdded").click(function(e) {

            // LogThis('quick add clicked...');

            //checkGmailAuth();
            e.stopImmediatePropagation();
            var email = $(this).attr("sfEmailAddress");
            var type = $(this).attr("sfType");
            var name = $(this).attr("sfname");
            var emailText = $(this).attr("sfEmailAddressText");

            //New for gmail api
            var gmailAttachments = new Array();
            var gmailOrigialSubject = "";
            var gmailUserId = "";
            var saveOnSend = 0;

            //Set all complex add buttons to current = falsle
            $("[current-emailAdd*='true'").attr("current-emailAdd", "false");
            //Set the clicked complex add button as the current complex add button

            $(this).attr("current-emailAdd", "true");

            const gmailMessageId = getMessageId($("*[current-emailadd*='true'"));
            const gmailImmutableMessageId = GmailTools.MessageView.GetNewFormatMessageId($(this)[0]);

            var lasttime = "";
            var title = "";
            var message = "Added by Ebsta\n\n";
            message += "Link to original email: " + document.URL + "\n\n";

            $(this).addClass("sfQuickAddFired");

            $(".g3").each(function() {
                    lasttime = $(this).attr("title");
                }
            );

            var patt = '\"' + lasttime + '\"\,([0-9]*),';

            var reg = new RegExp(patt);

            var res = reg.exec(document.body.outerHTML);
            var foundDate = false;

            gmailAttachments = gmailAttachments.join();

            //Find the gmail message id;
            // gmailMessageId = $(this).closest('.sfComplexAddContainer').attr('gmailmessageid');

            //Find the gmailUserId (email)
            gmailUserId = GetGmailUsersEmailAddress();


            if (res != null) {
                if (res.length != 0) {
                    var timeStamp = parseInt(res[1]);
                    var dt = new Date(timeStamp / 1000);
                    var dateFormatted = dateToYMD(dt);
                    lasttime = dateFormatted;
                    foundDate = true;
                }
            }

            title = "Email Received: ";

            //If the user sent this email change the subject title
            $(this).closest('.acZ').find('span').each(function() {
                var usersEmailAddess = GetGmailUsersEmailAddress();
                if ($(this).attr('email')) {
                    if ($(this).attr('email').toLowerCase() == usersEmailAddess.toLowerCase()) {
                        title = "Email Sent: ";
                    }
                }
            });

            //console.time("QuickAddFindSubject");
            var titleFound = false;
            //Find the closest compose / reply container
            $(this).closest("div.aeF, div.nH.if").find('.hP:visible').each(function() {
                //If we allready have a subject - dont get another one.
                if (!titleFound) {
                    //If we dont have a subject - get it and set title and original subject.
                    gmailOrigialSubject = $(this).text();
                    title += gmailOrigialSubject;
                    titleFound = true;
                }
            });
            //console.timeEnd("QuickAddFindSubject");

            $(this).closest('.gs').find(".adO").each(function() {
                    //title = "Email: " + $(this).text();

                    var mailContent = $(this).clone();
                    mailContent.find("br").replaceWith("\n\n");
                    var mailTextContent = mailContent.text();

                    if (mailTextContent != null && typeof mailTextContent != "undefined") {
                        mailTextContent = mailTextContent.replace(/ \s+/g, '');
                    }

                    message += mailTextContent;
                }
            );

            chrome.extension.sendMessage({
                MessageType: "AddGmailTask",
                EmailText: emailText,
                Email: email,
                GmailAttachmentIds: "",
                GmailUserId: GoogleTools.GetEmailAddress(),
                GmailMessageId: gmailMessageId,
                GmailImmutableId: gmailImmutableMessageId,
                GmailOriginalSubject: gmailOrigialSubject,
                SaveOnSend: saveOnSend,
                Subject: title,
                Name: name,
                SFType: type,
                AddType: "QuickAdd",
                RequestedFromUrl: document.location.href
            });
        });

          //  $(document).on("click", ".sfComplexAdd", function (e) {
           $(".sfComplexAdd, .sfComplexAdded").click(function (e) {

               $('#emailTemplateBox').remove();


               ////LogThis('complex add clicked...');
                 
                //Set all complex add buttons to current = falsle
                $("[current-emailAdd*='true'").attr("current-emailAdd", "false");
                //Set the clicked complex add button as the current complex add button
                $(this).attr("current-emailAdd", "true");


                //if you pop complex add when the page is resized, you get native gmail buttons overlapping our box, so we set the zIndex of these lower than that of the complex add box
                if (document.getElementsByClassName("gE iv gt").length != null) {
                    if (document.getElementsByClassName("gE iv gt").length > 0) {
                        document.getElementsByClassName("gE iv gt")[0].style.zIndex = 0;
                    }
                }

                if (document.getElementsByClassName("G-atb D E").length != null) {
                    if (document.getElementsByClassName("G-atb D E").length > 0) {
                        document.getElementsByClassName("G-atb D E")[0].style.zIndex = 0;
                    }
                }

                e.stopImmediatePropagation();

                var type = $(this).attr("sfType");
                var sfname = $(this).attr("sfname");
                var emailText = $(this).attr("sfEmailAddressText");

                var email = $(this).attr("sfEmailAddress");

                var lasttime = "";
                var title = "";
                var message = "Added by Ebsta\n\n";
                message += "Link to original email: " + document.URL + "\n\n";

                $(this).addClass("sfComplexAddFired");

                $(".g3").each(function () {
                    lasttime = $(this).attr("title");

                }
                );
                var patt = '\"' + lasttime + '\"\,([0-9]*),';
            
                //title="24 November 2014 at 11:15"
                var reg = new RegExp(patt);

                var res = reg.exec(document.body.outerHTML);
                var foundDate = false;

                if (res != null) {
                    if (res.length != 0) {
                        var timeStamp = parseInt(res[1]);
                        var dt = new Date(timeStamp / 1000);
                        var dateFormatted = dateToYMD(dt);
                        lasttime = dateFormatted;
                        foundDate = true;
                    }
                }

                var originalSubject = "";
                title = "Email Received: ";

               //If the user sent this email change the subject title
                $(this).closest('.acZ').find('span[email]').each(function ()
                {
                    var usersEmailAddess = GetGmailUsersEmailAddress();
                    if ($(this).attr('email'))
                    {
                        if ($(this).attr('email').toLowerCase() == usersEmailAddess.toLowerCase())
                        {
                            title = "Email Sent: ";
                        }
                    }
                });

                //console.time("ComplexAddFindSubject");
                var subjectFound = false;
               //Find the closest compose / reply container
                $(this).closest("div.aeF, div.nH.if").find('.hP:visible').each(function () {
                    //If we allready have a subject - dont get another one.
                    if (!subjectFound) {
                        //If we dont have a subject - get it and set title and original subject.
                        originalSubject = $(this).text();
                        title += originalSubject;

                        //Set that we have a found a subject - subjects can be empty strings
                        subjectFound = true;
                    }
                });
                //console.timeEnd("ComplexAddFindSubject");

                $(this).closest('.gs').find(".adO").each(function () {
                    //title = "Email: " + $(this).text();

                    var mailContent = $(this).clone();
                    mailContent.find("br").replaceWith("\n\n");
                    var mailTextContent = mailContent.text();

                    if (mailTextContent != null && typeof mailTextContent != "undefined") {
                        mailTextContent = mailTextContent.replace(/ \s+/g, '');
                    }
                    message += mailTextContent;
                }
               );

                var secondaryEmailSearchArray = [];

                //Finds all email address associated with the current email
                $(this).closest('.gs').find("span.g2").each(function () {
                    secondaryEmailSearchArray.push($(this).attr("email"));
                });

                //Remove succesffully added message
                $("#sfgmailId").remove();

                //Find any related to objects to pre-populate the fields with
                var whatId = null, whatName = null, whatRecordType = null;

                if ($(this).attr("whatid")) {
                    whatId = $(this).attr("whatid");

                    if ($(this).attr("whatname")) {
                        whatName = $(this).attr("whatname");
                    }

                    if ($(this).attr("whatRecordType")) {
                        whatRecordType = $(this).attr("whatRecordType");
                    }
                }

                //We'll first go and check Local Storage for any 'Related To' objects previous chosen for the chosen WhoId
                chrome.extension.sendMessage({
                    MessageType: "GetLocalStorageRelatedToSelections"
                }, function (response) {

                    

                    //If we've got some history of a 'Related To' object for this record then lets pre-populate the Related To box instantly
                    if (response.relatedToHistory) {
                        listOfRelatedToHistory = response.relatedToHistory;
                        var result = LookForRelatedToObject(email);

                        if (result) {
                            if (result.WhatRecordId) {
                                whatId = result.WhatRecordId;
                            }

                            if (result.WhatRecordName) {
                                whatName = result.WhatRecordName;
                            }

                            if (result.WhatRecordType) {
                                whatRecordType = result.WhatRecordType;
                            }
                        }
                    }

                    new ComplexAdd().AddComplexAdd({
                        MessageType: "AddTaskEmail",
                        Id: email,
                        EmailText: emailText,
                        SecondaryEmailSearch: secondaryEmailSearchArray,
                        Time: lasttime,
                        Title: title,
                        Msg: message,
                        SFType: type,
                        ObjectName: sfname,
                        SaveOnSend: 0,
                        SaveOnSendObj: 0,
                        FoundDate: foundDate,
                        OriginalSubject: originalSubject,
                        WhatId: whatId,
                        WhatName: whatName,
                        WhatRecordType: whatRecordType,
                        AddType: "MessageView-ComplexAdd",
                        Url: document.location.href
                    });
                });
                
            });
        //}
    }

    function dateToYMD(date) {
        var d = date.getDate();
        var m = date.getMonth() + 1;
        var y = date.getFullYear();
        return '' + y + '-' + (m <= 9 ? '0' + m : m) + '-' + (d <= 9 ? '0' + d : d);
    }


    var gmailSetup = false;


    var fullName = '';
    var emailAddressFound = 0;
    var emailAddressRegex = '';
    
    function CheckInboxMsg(barload) {
        //console.log("bar.js: start of CheckInboxMsg");
        var d = new Date();
    
        var googleArry = new Array(), googleArry2 = new Array();
        var googleCounter = 0, googleCounter2 = 0;


        var checkButtons = true;
        //console.time('CheckInboxMsgEditable');
        $("div[g_editable='true']").each(function (index, element) {
            if (checkButtons === true) {
                chrome.extension.sendMessage({ MessageType: "GetAccountActive", EmailAddress: GetGmailUsersEmailAddress() }, function (response) {
                    if (response != null) {
                        if (response.AccountActive + "" === "true") {
                            AddButtonDelay();
                        }
                    }
                });
                checkButtons = false;
            }
        }); 

       // console.timeEnd('CheckInboxMsgEditable');

        var logRead = "0";

        //We want to search differently for Sent Items as we want to search for the sendee not the sender
        if (document.URL.toLowerCase().indexOf("sent") == -1)
        {
            $(".gD:visible").each(function (index, element) {
                googleArry2[googleCounter2] = {};
                googleArry2[googleCounter2]["emailAddress"] = $(this).attr("email");
                googleArry2[googleCounter2]["fullName"] = $(this).attr("name");

                if (googleArry.indexOf($(this).attr("email")) == -1) {
                    googleArry[googleCounter] = $(this).attr("email");

                    if (googleCounter2 == index && barload + "" == "true") {
                        fullName = $(this).attr("name");

                        if (logRead == "0") {
                            logRead = "1";
                        }
                        
                    }

                    googleCounter++;
                }

                googleCounter2++;

            });

            //console.timeEnd('CheckInboxMsg2');

        } else {
            //Lets look for the sendee
            //console.time('CheckInboxMsg3');
            $(".h7:visible:last, .kv:visible:last").find(".g2:visible:first").each(function (index, element) {
                if (googleArry2.length == 0) {
                    googleArry2[googleCounter2] = {};
                    googleArry2[googleCounter2]["emailAddress"] = $(this).attr("email");
                    googleArry2[googleCounter2]["fullName"] = $(this).attr("name");
                }
                if (logRead == "0") {
                    logRead = "1";
                }

                fullName = $(this).attr("name");
            });
            //console.timeEnd('CheckInboxMsg3');

            //console.time('CheckInboxMsg4');

            //We still need to find all the emails within this email for more buttons to appears
            $(".g2").each(function (index, element) {
                if (googleArry.indexOf($(this).attr("email")) == -1) {
                    googleArry[googleCounter] = $(this).attr("email");
                    googleCounter++;
                }
            });
            //console.timeEnd('CheckInboxMsg4');

        }

        if (emailAddressFound == 0) {
            //console.time('CheckInboxMsg5');

            emailAddressRegex = GetGmailUsersEmailAddress();
            emailAddressFound = 1;
            //console.timeEnd('CheckInboxMsg5');

        }
        
        var currentUserEmailAddress = emailAddressRegex;
        var emailAddressToSearch = "";


        //console.time('CheckInboxMsg6');
        //If there is only 1 email address in the list then use it
        if (googleArry2.length == 1)
        {
            emailAddressToSearch = googleArry2[0]["emailAddress"];
        }
        //If there is more than one email address try and find another email address to search rather than the users own email
        else if (googleArry2.length > 1)
        {
            emailAddressToSearch = googleArry2[googleArry2.length - 1]["emailAddress"];

            if (emailAddressToSearch == currentUserEmailAddress)
            {
                for (var i = googleArry2.length-1; i > -1; i--)
                {
                    if (googleArry2[i]["emailAddress"] != currentUserEmailAddress)
                    {
                        emailAddressToSearch = googleArry2[i]["emailAddress"];
                        fullName = googleArry2[i]["fullName"];
                        break;
                    }
                }
            }
        }
        
        //Lets find all the ids avaliable on the screen for where buttons do appear to tell user if this email has already been added by them
        var messageIds = GmailTools.MessageView.FindMessageIdsOnGmailPage();

        //Only load bar for the last message in an email conversation
        if (barload) {
            if (googleArry2.length > 0) {
                ////LogThis('Test 123');
                emailAddressToSearchJsonParse = emailAddressToSearch;
                fullnameToSearchJsonParse = fullName;

                // for Custom Bar Search we will pass through the name as well as the email address. searchtype = 4.
                var nameandemail = emailAddressToSearch + ",,," + fullName;

                if (useDefaultBarSearch)
                {
                    proc.LoadFrame(emailAddressToSearch, 1);
                }
                else
                {
                    proc.LoadFrame(nameandemail, 4);
                }
            }
        } 

        if (LoggedIn) {
            RunMsgSearch(googleArry, messageIds);
        }
    }

var emailAddressToSearchJsonParse = '';
var fullnameToSearchJsonParse = '';

/**
 * Custom data parsing for Gmail which is used to send back to bar.js
 * @param {} rules 
 * @returns {} 
 */
function SendDetailForParsingBack(rules) {
    PassJson(emailAddressToSearchJsonParse, fullnameToSearchJsonParse);

    return {
        ParseInformation: jsonPerson,
        Url: document.URL,
        CustomGmailOptFound: Does_Email_Have_Custom_Rules_Associated_With_It(rules)
    }
}

/**
 * Returns the users Gmail address they're logged into
 * @returns {} 
 */
function GetGmailUsersEmailAddress() {
    return GoogleTools.GetEmailAddress();
}
            
    //Runs a search for when a user is in the Compose or Reply boxes and runs a search for the first sendee within the email
    function RunCheckComposeAndReplySendee(composeBox, replyForwardBox) {
        //console.time('RunCheckCompose');

        ////LogThis('run check compose and reply sendee...');

        var msgBodyToSearchFor = $("[email]:visible");
        var emailAddressToSearch = "";

        var recipientCounter = 0;

        if (composeBox)
        {
            $(".aaZ").find(msgBodyToSearchFor).each(function () {

                ////LogThis('found compose box of class: aaZ');

                emailAddressToSearch = $(this).attr("email");

                var firstRecord = true;

                if (recipientCounter == 0) { //this is the first email in the recipient list.
                    firstRecord = true;
                } else {
                    firstRecord = false;
                }

                chrome.extension.sendMessage({
                    MessageType: "SetFirstRecipientFlag",
                    SearchTerm: "" + emailAddressToSearch,
                    SetFirstRecord: firstRecord
                }, function (response) {

                });

                recipientCounter++;

                //return false;
            });
        }
        else if (replyForwardBox)
        {
            $(".Iy").find(msgBodyToSearchFor).each(function () {
                emailAddressToSearch = $(this).attr("email");

                var firstRecord = true;

                if (recipientCounter == 0) { //this is the first email in the recipient list.
                    firstRecord = true;
                } else {
                    firstRecord = false;
                }

                chrome.extension.sendMessage({
                    MessageType: "SetFirstRecipientFlag",
                    SearchTerm: "" + emailAddressToSearch,
                    SetFirstRecord: firstRecord
                }, function (response) {

                });

                recipientCounter++;

                //return false;
            });
        }

        ////LogThis('get localstorage last search word...');

        //Lets find what our last search was to see if we should fire the bar off again
        chrome.extension.sendMessage({
            MessageType: "GetLocalStorageLastSearchWord"
        }, function (response) {
            var runSearch = true;

            if (response.result != null)
            {
                var lastSearchedEmailAddress = response.result;

                //If this email address is blank or equals the last search we done then don't run the search
                if (!emailAddressToSearch || emailAddressToSearch === lastSearchedEmailAddress) {
                    runSearch = false;
                }
            }

            if (runSearch)
            {
                emailAddressToSearchJsonParse = emailAddressToSearch;
                fullnameToSearchJsonParse = '';

                if (useDefaultBarSearch) {
                    proc.LoadFrame(emailAddressToSearch, 1);
                }
                else {
                    proc.LoadFrame(emailAddressToSearch + ",,," + fullName, 4);
                }
            }
        });

        //console.timeEnd('RunCheckCompose');
    }

    var startInbox = false; 
    function CheckInbox(msgFound) {
      //  console.time('CheckInbox');
        
       // console.time('CheckInbox1');

        var googleArry1 = new Array();
        var googleCounter = 0;
       

        var msgBody1 = $("[email]:visible");

        $(".aeF").find(msgBody1).each(function () {
            ////LogThis(this);
            if (googleArry1.indexOf($(this).attr("email")) == -1) {

                googleArry1[googleCounter] = $(this).attr("email");
                googleCounter++;
            }
              
        });

        LogThis(googleArry1);
    //    console.timeEnd('CheckInbox1');
    
       // console.time('CheckInbox2');
        if (LoggedIn) {
            RunInboxSearch(googleArry1);
            if (msgBody1.length > 0) {
                GmailTools.SetClassToStateEmailsHaveBeenChecked(msgBody1[0]);
            }
        }
     //   console.timeEnd('CheckInbox2');

     //   console.timeEnd('CheckInbox');
    }


    function RunInboxSearch(googleArry) {
        //console.time('RunInboxSearch');
        try {
       
            var rw = JSON.stringify(googleArry);
            chrome.extension.sendMessage({
                MessageType: "GoogleInboxSearch",
                Url: "" + document.URL,
                search: rw,
                barrun: false
            }, function(response) {

            });
        } catch(error) {
            LogThis("Errors in Run inbox Search" + error);
        }
        //console.timeEnd('RunInboxSearch');
    }

    function RunMsgSearch(googleArry, messageIds) {

        //LogThis('RunMsgSearch');
        try {
            var rw = JSON.stringify(googleArry);
        
            chrome.extension.sendMessage({
                MessageType: "GoogleMessageSearch",
                Url: "" + document.URL,
                search: rw,
                barrun: false,
                GoogleMessageIds: messageIds
            }, function (response) {

            });
        } catch(error) {

        }
        //console.timeEnd('RunMsgSearch');

    }

    var listening = false;
    var barLoadedCount = 0;
    var lastSearchRunWhenDomModified = '';
    var hideBar = false;
    var foundCirus = false;

    var isChatterOnly = '' + 0;

    function isSearchTypeTwo() {

        var searchTypeTwos = [
            'members.ocpinfo.com/search/search.aspx',
            'linkedin.com/company',
            'linkedin.com/sales/accounts/',
            'linkedin.com/sales/company/',
            'linkedin.com/recruiter/company',
            'linkedin.com/company-beta/',
            'zoominfo.com/zoominfo/#!search/profile/company?',
            'zoominfo.com/s/#!search/profile/company?',
            'connect.data.com/company/',
            'angel.co/',
            'lead411.com/company',
            'channel-navigator.com',
            'subscriber.hoovers.com/h/company',
            '.boardex.com/boards/summary/',
            '.termscout.com/',
            '.yelp.com/biz/',
            '.yelp.co.uk/biz/',
            'xing.com/comp',
            'crunchbase.com/acquisition/',
            'crunchbase.com/funding_round/'
        ];

        for (var i = 0; i < searchTypeTwos.length; i++) {
            if (document.URL.toLowerCase().indexOf(searchTypeTwos[i]) > -1) {
                if (document.URL.indexOf(".ocpinfo.") > -1) {
                    var pageTest = BarJsGetValueByXpath('//*[@id="Table1"]/tbody/tr/td[1]/b');
                    if (pageTest == "PHARMACY DETAIL") {
                        return true;
                    }
                    return false;
                }

                if (document.URL.toLowerCase().indexOf("subscriber.hoovers.com") != -1 && document.URL.toLowerCase().indexOf('/people') != -1) {
                    return false;
                }

                return true;
            }
        }

        //The page doesn't navigate when showing the company search section so we need to look for class
        if (document.URL.toLowerCase().indexOf('go.zoominfo.com/#/app/search-core') > -1) {
            return document.getElementsByClassName("company_card_left").length > 0;
        }

        return false;
    };

    //run bar drop decision with a retry up to n times, or until it drops bar
    var decideIfBarShouldDropWithRetry = function (pageRules, searchType) {

        //bar not dropped - set retry limit and track count, retry every n secons
        var barDropped = false;
        var retryCount = 0;
        var retryLimit = 5;
        var waitBetweenTry = 1000;

        //try to drop the bar - if proc.LoadFrame does not run returns false
        barDropped = decideIfBarShouldDrop(pageRules, searchType);

        //start retry - up to n times each n seconds, or until bar drops
        var retryInterval = setInterval(function () {
            if (retryCount < retryLimit && !barDropped) {
                retryCount++;
                barDropped = decideIfBarShouldDrop(pageRules, searchType);
            } else {
                //success or give up, clear interval and stop
                clearInterval(retryInterval);
            }

        }, waitBetweenTry);

}

/**
 * Loops through a number of XPaths to find any elements on the DOM which match these expressions and returns the first value it finds
 * @param {object} pageRules - An object with properties that contains rules for person and company Names
 * @param {int} searchType - 1 if a person search, 2 if a Company search
 * @returns {string} - The first found value from the XPaths
 */
function GetFirstFoundValueFromListOfRules(pageRules, searchType) {

    var barDetails = '';

    if (document.URL.indexOf("google.com") == -1) {
        if (pageRules.fullName.length) {
            //Get full name
            for (var i = 0; i < pageRules.fullName.length; i++) {

                if (BarJsGetValueByXpath(pageRules.fullName[i]) !== "") {
                    barDetails = BarJsGetValueByXpath(pageRules.fullName[i]);

                    //Tidy up the Person name for BoardEx.com
                    if (document.URL.indexOf(".boardex.com") != -1 && barDetails.length > 0) {
                        barDetails = TidyPersonNameBoardEx(barDetails);
                    }

                    //Tidy up the Person name for BoardEx.com
                    if (document.URL.indexOf("jobsite.co.uk/cvs-bin/cvdisplay_text?") != -1 && barDetails.length > 0) {
                        barDetails = TidyPersonNameJobSite(barDetails);
                    }
                    break;
                }
            }
        }

        if (barDetails == '' && pageRules.firstName.length && pageRules.surname.length) {

            var firstName = '';
            var surname = '';

            //Get first name
            for (var i = 0; i < pageRules.firstName.length; i++) {
                if (BarJsGetValueByXpath(pageRules.firstName[i]) !== "") {
                    firstName = BarJsGetValueByXpath(pageRules.firstName[i]);
                    break;
                }
            }
            //Get surname
            for (var i = 0; i < pageRules.surname.length; i++) {
                if (BarJsGetValueByXpath(pageRules.surname[i]) !== "") {
                    surname = BarJsGetValueByXpath(pageRules.surname[i]);
                    break;
                }
            }

            if (firstName !== '' && surname !== '') {
                barDetails = firstName + " " + surname;
            }
        }

        if (barDetails == '' && pageRules.company.length && searchType == 2 ||
            (document.URL.toLowerCase().indexOf('subscriber.hoovers.com/h/company') != -1 && searchType == 2) ||
            (document.URL.indexOf(".termscout.com/") !== -1 && searchType == 2)) {
            for (var i = 0; i < pageRules.company.length; i++) {
                if (BarJsGetValueByXpath(pageRules.company[i]) != "") {

                    barDetails = BarJsGetValueByXpath(pageRules.company[i]);

                    //Tidy up the company name for BoardEx.com as it includes a lot of gunk
                    if (document.URL.indexOf(".boardex.com") != -1 && barDetails.length > 0) {
                        barDetails = TidyCompanyNameBoardEx(barDetails);
                    }
                    break;
                }
            }
        }
    }

    return barDetails;
}

    /**
     * Checks a number of xpaths to find some text to search against, if we find text we'll drop the bar
     * @param {object} pageRules - An object with properties that contains rules for person and company Names
     * @param {int} searchType - 1 if a person search, 2 if a Company search
     * @returns {boolean} - Returns try if we dropped the bar
     */
    function decideIfBarShouldDrop(pageRules, searchType) {

        var barDetails = GetFirstFoundValueFromListOfRules(pageRules, searchType);
        
        var dropBar = true;

        if ((document.URL.indexOf("gsahq.org") > -1 || document.URL.indexOf("gesa.memberclicks.net") > -1) && BarJsGetValueByXpath('//*[@id="cProfileWrapper"]/div/div/div[4]/div/a[1]') == "Edit Profile") {
            dropBar = false;
        }
        if ((document.URL.indexOf("xing.com/xtm") > -1 ||
            document.URL.indexOf(".indeed.") !== -1)
            && barDetails == lastSearchRunWhenDomModified) {
            dropBar = false;
        }

        //if (localStorage.ChatterOnly != "1") {
        //    dropBar = false;
        //}

        if (barDetails !== '' && dropBar == true && document.URL.toLowerCase().indexOf("mail.google.com") == -1) {
            lastSearchRunWhenDomModified = barDetails;

            //Custom Bar Search. Check if we are doing a default or custom bar search. Custom Bar Search is type 4.
            if (useDefaultBarSearch) {
                proc.LoadFrame(barDetails, searchType);
            }
            else {
                var nameandemail = "" + ",,," + barDetails;
                proc.LoadFrame(nameandemail, 4);
            }

            return true;
        }

        return false;
    };
    
    //The name appears within text that is hard to extract straight off using xpath so we'll need to tidy it up here
    function TidyPersonNameJobSite(name) {
        //Example formats;
        // Jobsite CV for	Enda O'Flaherty	27-Aug-2015 15:16  
        // telecomsjobsource.co.uk CV for	Tejeshwari Injaganeri	27-Aug-2015 12:59 

        var regexMatch = new RegExp("CV for(.*?)\\s+\\d");
        var match = regexMatch.exec(name.trim());
        if (match) {
            return match[1];
        }
    }

    //Tidy's Contact names in BoardEx.com as some have 1 initial middle names which effect the search, this helps it make it be a little more smarter
    function TidyPersonNameBoardEx(name)
    {
        //Remove any bracketted details
        name = name.replace(/\(.+\)/g, '');

        //Remove some titles that appear
        name = name.replace(/Professor.? /g, '');
        name = name.replace(/Dr.? /g, '');
        name = name.replace(/Doctor.? /g, '');
        name = name.replace(/The Hon.? /g, '');
        name = name.replace(/Lady.? /g, '');
        name = name.replace(/Dame.? /g, '');
        name = name.replace(/Major.? /g, '');
        name = name.replace(/General.? /g, '');
        name = name.replace(/Brigadier.? /g, '');
        name = name.replace(/Sir.? /g, '');
        name = name.replace(/Jr/g, '');
        name = name.replace(/Sr/g, '');

        //Lets just take the 1st and last word found after the split excluding any potential middle names
        var nameSplit = name.trim().split(" ");

        if (nameSplit.length > 2) {
            name = nameSplit[0] + " " + nameSplit[nameSplit.length - 1];
        }

        name = name.trim();
        return name;
    }

    //Fixes Company names in BoardEx.com as some are displayed like PROLOGIS (Prologis Trust prior to 06/2002) (De-listed 06/2011) (Quoted)
    function TidyCompanyNameBoardEx(name)
    {
        //Remove any bracketted details e.g. (QUOTED)
        name = name.replace(/\(.+\)/g, '');

        //Lets looks for the last tag of a company and remove any titled information i.e. INC, CORP, LTD
        var companyNameSplit = name.trim().split(" ");
        if (companyNameSplit.length > 1) {
            var companyTags = ['ltd', 'corp', 'inc', 'plc', 'lp', 'sa', 'limited'];

            var lastWord = companyNameSplit[companyNameSplit.length - 1].toLowerCase();

            //If it's matched our list then rebuild the string up
            if (companyTags.indexOf(lastWord) != -1) {
                name = '';
                for (var i = 0; i < companyNameSplit.length - 1; i++) {
                    name += companyNameSplit[i] + " ";
                }
            }
        }

        name = name.trim();
        return name;
    }

    function dropBarUsingRulesEngine(docURL) {
        var thisURL = (function () {
            if (docURL) {
                return docURL.toLowerCase();
            }
            return document.URL.toLowerCase();
        })();

        if (thisURL.indexOf("mail.google.com") != -1 || thisURL.indexOf(".linkedin.") != -1 || LicenceFound == "Pro" || LicenceFound == "Enterprise") {

            if (!isInBlackList(thisURL)) {
                var thisPagesRules = {
                    valid: false,
                    fullName: new Array(),
                    firstName: new Array(),
                    surname: new Array(),
                    company: new Array(),
                    email: new Array
                };

                //Drop bar based on RulesEngine
                chrome.extension.sendMessage({
                    MessageType: "BarDrop"
                }, function (response) {
                    if (response) {

                        //Step 1: Get Rules into Bar.js
                        if (response.LicenceType == "Pro" || response.LicenceType == "NoTrialYet" || response.LicenceType == "Expired") {

                            LicenceType = response.LicenceType;
                            
                            var rules = JSON.parse(response.Rules).d;
                            
                            //Step 2: Loop through and find rules which match current domain
                            for (var i = 0; i < rules.length; i++) {
                                //Step 3: if document's URL is found in Rule's URL


                                if (thisURL.indexOf(rules[i].URL.toLowerCase()) > -1) {
                                    //At this point, we'll need one of three things: Full Name, Company Name, or First Name + Last Name
                                    //So we'll quickly make a list of the rules we have, and pass them into a testing function

                                    var regexFound = false;

                                    try {
                                        regexFound = rules[i].Regex;

                                    } catch (err) {
                                        // Error.
                                    }

                                    if (!regexFound) {
                                        if (rules[i].Field == "Full Name") {
                                            thisPagesRules.fullName.push(rules[i].Xpath);
                                            thisPagesRules.valid = true;
                                            continue;
                                        }
                                        if (rules[i].Field == "First Name") {
                                            thisPagesRules.firstName.push(rules[i].Xpath);
                                            thisPagesRules.valid = true;
                                            continue;
                                        }
                                        if (rules[i].Field == "Surname") {
                                            thisPagesRules.surname.push(rules[i].Xpath);
                                            thisPagesRules.valid = true;
                                            continue;
                                        }

                                        //This one's for company pages
                                        if (rules[i].Field == "Employer") {
                                            thisPagesRules.company.push(rules[i].Xpath);
                                            thisPagesRules.valid = true;
                                            continue;
                                        }
                                        //This one's for film pages
                                        if (rules[i].Field == "ProjectName") {
                                            thisPagesRules.company.push(rules[i].Xpath);
                                            thisPagesRules.valid = true;
                                            continue;
                                        }
                                        //This one's for Gmail
                                        if (rules[i].Field == "Email") {
                                            thisPagesRules.email.push(rules[i].Xpath);
                                            thisPagesRules.valid = true;
                                            continue;
                                        }
                                    }
                                }
                            }

                            var searchType = 1;
                            if (thisPagesRules.fullName.length == 0 && thisPagesRules.firstName.length == 0 && thisPagesRules.surname.length == 0 && thisPagesRules.company.length > 0) {
                                searchType = 2;
                            }
                            if (isSearchTypeTwo()) {
                                searchType = 2;
                            }

                            if (response.AlwaysDropBar === true) {
                                let barSearchType = BarSearchTypes.CustomBarAccount.URL;
                                const searchValue = GetFirstFoundValueFromListOfRules(thisPagesRules, searchType);
                                //We do different types of search when it comes to linkedin, this check helps ensure we set the right type of search
                                if (thisURL.indexOf(".linkedin.") !== -1) {

                                    //If we don't have a valid value to search with, it'll end up searching "null" so to prevent 
                                    //this from happening just exit out to prevent us running a search
                                    if (!searchValue) return true;

                                    if (searchType === 2) {
                                        barSearchType = BarSearchTypes.CustomBarAccount.LINKEDINCOMPANY;
                                    } else {
                                        barSearchType = BarSearchTypes.CustomBarAccount.LINKEDINUSER;
                                    }
                                }

                                proc.LoadFrame(GetFirstFoundValueFromListOfRules(thisPagesRules, searchType), barSearchType);
                                return true;
                            }

                            //Loop is done - now evaluate xpaths and drop bar

                            //We don't want to run searches for chatter only users...
                            //var isChatterOnly = response.ChatterOnly;
                            isChatterOnly = response.ChatterOnly;

                            if (thisPagesRules.valid) {
                                //If there's no name Xpath, searchType = 2

                             

                                ////removes the grey bar drop...
                                ////if (response.LicenceType == "Expired") {
                                ////    if (document.URL.indexOf("linkedin.com") == -1) {
                                ////        searchType = "50";

                                ////    }
                                ////}

                                if (document.URL.indexOf("dice.com") > -1 || document.URL.indexOf("indeed.com") > -1
                                    || document.URL.indexOf("universobit.com.ar/empresas") > -1) {
                                    //|| document.URL.indexOf("linkedin.com/sales/accounts") > -1 || document.URL.indexOf("linkedin.com/recruiter/") > -1) {
                                    setTimeout(function () {
                                        if (document.URL.indexOf("dice.com") > -1) {
                                            if (BarJsGetValueByXpath('//*[@id="tmCandidateName"]/h1').length > 0) {


                                                decideIfBarShouldDrop(thisPagesRules, searchType);
                                            } else {
                                                setTimeout(function () {

                                                    decideIfBarShouldDrop(thisPagesRules, searchType);
                                                }, 2000);
                                            }
                                        } else {
                                            if (BarJsGetValueByXpath('//h3[contains(concat(" ",@class," ")," name ")]//span[contains(concat(" ",@class," ")," pull-left ")]') == lastSearchRunWhenDomModified) {
                                                setTimeout(function () {
                                                    decideIfBarShouldDrop(thisPagesRules, searchType);
                                                }, 2000);
                                            } else {
                                                decideIfBarShouldDrop(thisPagesRules, searchType);
                                            }
                                        }
                                    }, 1000);
                                } else {
                                    if (DoesWebsiteNeedRetryProcess()) {
                                        setTimeout(function () {
                                            decideIfBarShouldDropWithRetry(thisPagesRules, searchType);
                                        }, 500);
                                    } else {
                                        decideIfBarShouldDrop(thisPagesRules, searchType);
                                    }
                                }
                            }

                            // Die quietly
                        }
                    } // Respionse.

                });
            }
        }
    };

/**
 * Determines if a website needs to have our retry logic in order for the Ebsta bar to drop
 * @returns {} 
 */
function DoesWebsiteNeedRetryProcess() {
    var websiteArray = [
        "linkedin.com/recruiter/",
        "linkedin.com/sales/",
        "linkedin.com/in/",
        "linkedin.com/talent/",
        ".linkedin.com/company-beta/",
        ".termscout.com/",
        "xing.com",
        "totaljobs.com",
        "entelo.com",
        ".crunchbase.com",
        "zoominfo.com",
        ".amazinghiring.com",
        ".careerbuilder.com"
    ];

    //findIndex returns the index of the element that matches condition or -1 if no elements match
    return websiteArray.findIndex(x => document.URL.toLowerCase().indexOf(x) > -1) > -1;
}

    function WizardsLoad() {
        if (LoggedIn + "" == "true") {

            var loadWizards = true;

            ////if (trialLicenceFound + "" == "true" && LicenceFound == "Free") {
            ////    loadWizards = false;
            ////}

            if (document.URL.toLowerCase().indexOf("logout") == -1) {
                if (document.URL.toLowerCase().indexOf(".linkedin.com") != -1) {
                    if (LinkedinWizard + '' == 'false') {
                        chrome.extension.sendMessage({
                            MessageType: "LinkedinWizardOff"
                        }, function (response) {
                        });

                        if (GmailWizard + '' != 'true') {
                            if (loadWizards) {

                                if (loadWizards) {
                                    var showNewBar = false;

                                    ////LogThis('AB Rules');
                                    ////LogThis(abRules);

                                    if (abRules.ShowNewBar + "" == "1") {
                                        showNewBar = true;
                                    }

                                    if (abRules.ShowNewBarEveryWhere + "" == "1") {
                                        showNewBar = true;
                                    }
                                }
                            }
                        }
                    }
                }
            }

            if (document.URL.toLowerCase().indexOf("mail.google.com/mail") != -1 && document.URL.toLowerCase().indexOf("accounts.google.com") == -1) {

                if (trialLicenceFound + '' == 'true' && LicenceFound + '' == "Free" && ExpiredWizard + '' == 'false') {

                    //ShowExpiredWizard();

                    //chrome.extension.sendMessage({
                    //    MessageType: "ExpiredWizardOff"
                    //}, function (response) {
                    //});

                    chrome.extension.sendMessage({
                        MessageType: "GmailWizardOff"
                    }, function (response) {
                    });
                }

                if (GmailWizard + '' == 'false') {
                    if (loadWizards) {
                        var showNewBar = false;

                        if (abRules.ShowNewBar + "" == "1") {
                            showNewBar = true;
                        }

                        if (abRules.ShowNewBarEveryWhere + "" == "1") {
                            showNewBar = true;
                        }
                   
                    }

                    chrome.extension.sendMessage({
                        MessageType: "GmailWizardOff"
                    }, function(response) {
                    });
                }
            }
        }
    }

    var eventSet = true;
    var dlastCheck = new Date();

    function AssignMonsterHeaderEvent() {
        var containerId = "detailHeader";
        var container = document.getElementById(containerId);

        if (container != null) {
            container.addEventListener("DOMSubtreeModified", function () {
                var name = '' + BarJsGetValueByXpath('//*[@id="ctl00_ctl00_ContentPlaceHolderBase_ContentPlaceHolderRight_lld_Detail_controlDetailTop_lbName"]');

                if (name == '') {
                    name = '' + BarJsGetValueByXpath('//*[@id="CandidateDetailTop_lbName"]');
                }

                if (name != '' && name != lastSearchRunWhenDomModified) {
                    //console.log("search 9");
                    proc.LoadFrame(name, 1);
                    lastSearchRunWhenDomModified = name;
                }
            });
        }
    }

    var headerEventAssigned = 0;

    function ListenForPageEventsIndeed() {

        var urlCheck = document.URL.toLowerCase();
        if (urlCheck.indexOf('indeed.com') != -1) {

            window.addEventListener("hashchange", function () {
                var hash = '' + document.location.hash;
                hash = hash.toLowerCase();

                if (hash.indexOf("/view?id=") != -1) {
                    dropBarUsingRulesEngine();
                }
                else {
                    proc.HideFrame();
                    lastSearchRunWhenDomModified = '';
                }
            });
        }

        if (urlCheck.indexOf("resumes.indeed.com/search") !== -1) {
            resumesIndeedHelper.SetupResumesIndeedObservers();
        }
    }

    function ListenForPageEventsMonster() {

        var urlCheck = document.URL.toLowerCase();
        if (urlCheck.indexOf('/jcm/resumesearch/searchresults.aspx') != -1 ^ urlCheck.indexOf('/jcm/candidates') > -1 ^ urlCheck.indexOf('hiring.monster.ca/jcm/candidates') != -1) {

            window.addEventListener("hashchange", function() {

                chrome.extension.sendMessage({ MessageType: "ClearParseInformation" });

                var hash = '' + document.location.hash;
                hash = hash.toLowerCase();

                if (hash.indexOf("s=detail") != -1 ^ hash.indexOf("s=pagedetailstate") != -1) {
                    // I have a Detail. 
                    if (headerEventAssigned == 0) {
                        AssignMonsterHeaderEvent();
                        headerEventAssigned = 1;
                    }
                } else {
                    proc.HideFrame();
                    lastSearchRunWhenDomModified = '';
                }
            });
        }

        if (urlCheck.indexOf("hiring.monster.com/supersearch") !== -1) {

            // TODO: MUTATION
            var monsterObserver = new MutationObserver(function (mutations) {
                mutations.forEach(function (mutation) {
                    if (mutation.addedNodes != null && mutation.addedNodes.length > 0) {
                        for (var i = 0; i < mutation.addedNodes.length; i++) {
                            if (mutation.addedNodes[i].classList.contains("profile")) {
                                dropBarUsingRulesEngine();
                            }
                        }
                    }
                });
            });
            monsterObserver.observe(document.body, {
                childList: true
            });
        }
    }

    //Sets clicks events to track when people go into candidate records
    function ListenForPageEventsUniversoBit() {
        if (document.URL.indexOf(".universobit.com.ar/empresas") != -1)
        {
            window.addEventListener("mousedown", function (event) {
                if (event.toElement != null)
                {
                    //Intial candidate search results
                    if (event.toElement.title != null && event.toElement.title.toLowerCase().indexOf("candidato") != -1)
                    {
                        dropBarUsingRulesEngine();
                    }
                    //Next and Previous buttons
                    var className = event.toElement.className;
                    if (className != null && (className.toLowerCase().indexOf("postulante") != -1 || className.toLowerCase().indexOf("bullet") != -1))
                    {
                        dropBarUsingRulesEngine();
                    }
                    //More next and previous buttons
                    var outerTxt = event.target.outerText;
                    if (outerTxt != null && (outerTxt.toLowerCase().indexOf("siguiente") != -1 || outerTxt.toLowerCase().indexOf("anterior") != -1))
                    {
                        dropBarUsingRulesEngine();
                    }
                }
            });
        }
    }

    function attrModified(mutation) {
        if (hideBar == false) {
            proc.HideFrame();
            hideBar = true;
        }
        if (hideBar == true) {
            dropBarUsingRulesEngine();
            hideBar = false;
        }
    }

    function ListenForPageEventsBroadbean() {
        var urlCheck = document.URL.toLowerCase();
        if (urlCheck.indexOf('adcourier.com') > -1) {
            window.addEventListener('DOMSubtreeModified', function () {
                dropBarUsingRulesEngine();
                var prevButton = document.evaluate('//div[contains(concat(" ",@class," ")," BeanBoxActive ")]/div[2]/a[3]', document, null, 9, null).singleNodeValue;
                var nextButton = document.evaluate('//div[contains(concat(" ",@class," ")," BeanBoxActive ")]/div[2]/a[2]', document, null, 9, null).singleNodeValue;
                var closButton = document.evaluate('//div[contains(concat(" ",@class," ")," BeanBoxActive ")]/div[2]/a[1]', document, null, 9, null).singleNodeValue;
                var BeanBoxOverlay = document.getElementById("BeanBoxOverlay");
                prevButton.addEventListener("click", function () {
                    dropBarUsingRulesEngine();
                });
                nextButton.addEventListener("click", function () {
                    dropBarUsingRulesEngine();
                });
                closButton.addEventListener("click", function () {
                    proc.HideFrame();
                });
                if (BeanBoxOverlay != null) {
                    // TODO: MUTATION
                    var observer = new WebKitMutationObserver(function (mutations) {
                        mutations.forEach(attrModified);
                    });
                    observer.observe(BeanBoxOverlay, {
                        attributes: true
                    });
                }
            });
        }
    }

    function triggerCompanySearch() {
        if (document.URL.indexOf('www.crunchbase.com/company/') > -1) {
            var name = BarJsGetValueByXpath('//*[@id="breadcrumbs"]/span[2]//text()');
            //console.log("search 10");
            proc.LoadFrame(name, 2);
        }


    }

    function ListenForPageEventsGoogle() {
        var urlCheck = document.URL.toLowerCase();
        if (urlCheck.indexOf('mail.google.com/mail') != -1) {
            window.addEventListener("hashchange", function () {
                var hash = '' + document.location.hash;
                hash = hash.toLowerCase();
           
                var useNormal = true;

                if (GmailGo + '' == 'true') {
                    if (GmailGoLicence + '' == 'true') {
                        useNormal = false; 
                        //CheckInbox(false);
                    }
                }

                if (useNormal) {
                    if (hash.indexOf("#inbox/") != -1 ^ hash.indexOf("#imp/") != -1 ^ hash.indexOf("#starred/") != -1 ^ hash.indexOf("#sent/") != -1 ^ hash.indexOf("#drafts/") != -1) {

                        setTimeout(function() {
                            dropBarUsingRulesEngine();
                        }, 1000);
                    } else {

                        //console.log("search 4");

                        proc.LoadFrame('', 33);
                    }
                }
            });

            //setTimeout(function () {

            //    var urlCheck1 = document.URL.toLowerCase();
            //    dropBarUsingRulesEngine(urlCheck1);
            //}, 2000);
        }
}
    
/** 
 * A closure to help with getting window location details (helps with being able to mock this in tests) 
 */
const ebstaDocumentLocationHelper = (() => {
    return {
        /**
         * Wrapper for document.location.host
         * @returns The location host of the current document
         */
        getHost: () => {
            return document.location.host;
        }
    }
})();

const zoomInfoEvents = (function () {
    let areEventsSet = false;

    /**
     * Mutation observer for app.zoominfo.com website
     */
    let appDotZoomInfoMutationObserver = null;

    /**
     * All events used within V2 of ZoomInfo
     * @returns {} 
     */
    function SetupZoomInfoV2Events() {

        if (areEventsSet === true) {
            return;
        }

        //Keep this in its own closure so we can keep track of what elements we've added to
        /**
         * Last Element clicked
         */
        var lastClickedElement = null;

        /**
         * Custom class we'll append to the click element so we can gather information from the right record
         */
        var ebstaParserClassName = "Ebsta_Parser";

        /**
         * Removes the custom class from the previously chosen element, if there was one
         * @returns {} 
         */
        function RemoveEbstaParserClass() {
            if (lastClickedElement) {
                lastClickedElement.classList.remove(ebstaParserClassName);
            }
        }

        /**
         * Adds our custom parser class to an element given
         * @param {} element 
         * @returns {} 
         */
        function AddCustomClassToElement(element) {
            element.classList.add(ebstaParserClassName);
        }

        /**
         * The user is able to internally naviagate which automatically shows profiles, 
         * as we're appending our own class to clicked profiles we need 
         * to automatically append our class on the profile loaded at DOM time so the bar drops.
         * @returns {} 
         */
        function DetectProfileIsOpenOnLoad() {
            //Set parameters for the interval timer
            var retryCount = 0, retryLimit = 5, waitBetweenTry = 1000;

            var urlCheck = document.URL.toLowerCase();
            //The UI update no longer allows a profile container in the search list to be open on load 
            //We don't need this running repeatedly on the new layout with URL of 'app.zoominfo.com' 
            if (urlCheck.indexOf('go.zoominfo.com') > -1) {
                var retryInterval = setInterval(function () {
                    if (retryCount < retryLimit) {
                        retryCount++;
                        // Find the main contact card first which should then have its details below, if so then apply our 
                        // custom class to this element and the bar should drop                                                                
                        var contactElement = document.getElementsByTagName("zi-person-row")[0];
                        if (contactElement) {
                            var isContactProfileFullyLoaded = document.getElementById("primaryContact");

                            if (isContactProfileFullyLoaded) {
                                AddCustomClassToElement(contactElement);
                                dropBarUsingRulesEngine();
                                clearInterval(retryInterval);
                            }
                        }
                    } else {
                        //success or give up, clear interval and stop
                        clearInterval(retryInterval);
                    }

                }, waitBetweenTry);
            } else if (urlCheck.indexOf('app.zoominfo.com/#/apps/profile') > -1) {
                dropBarUsingRulesEngine();
            }
        }

        // Sets up a mutation observer rather than a click event to detect url changes 
        // as way of indicating a user is looking at different profiles
        if (ebstaDocumentLocationHelper.getHost() === "app.zoominfo.com") {

            if (appDotZoomInfoMutationObserver === null) {
                let oldHref = document.location.href;

                appDotZoomInfoMutationObserver = new MutationObserver(function (mutations) {
                    mutations.forEach(function (mutation) {
                        if (oldHref != document.location.href) {
                            oldHref = document.location.href;
                            if (document.location.href.indexOf("apps/profile/person") > 0) {
                                dropBarUsingRulesEngine();
                            }
                        }
                    });
                });

                var config = {
                    childList: true,
                    subtree: true
                };

                appDotZoomInfoMutationObserver.observe(document.body, config);
            }
        }

        /**
        * Add Click listener to entire DOM as its an SPA we can't attach it to specific 
        * elements as this will be removed and added over and over agian
        */
        document.addEventListener("mouseup", function (event) {
            if (event.target && event.button === 0) {
                //Find the Contact container to ensure it was a click event on a contact
                var profileContainer = event.target.closest("zi-person-row, .result-row");
                //Find the details container so we can prevent the bar from loading if a user clicks in this area                    
                const contactProfileDetailContainer = event.target.closest(".profile-container");
                //Don't do this on individual profile pages
                const areWeOnSingleProfilePage = document.URL.toLowerCase().indexOf('app.zoominfo.com/#/apps/profile') > 0;
                if (profileContainer && !contactProfileDetailContainer && !areWeOnSingleProfilePage) {
                    RemoveEbstaParserClass();
                    lastClickedElement = profileContainer;
                    AddCustomClassToElement(profileContainer);
                    dropBarUsingRulesEngine();
                    chrome.extension.sendMessage({ MessageType: "ClearParseInformation" });
                }
            }
        });


        //Adds a listener to detect URL changes so we can catch internal url redirecting
        window.addEventListener("hashchange", function () {
            DetectProfileIsOpenOnLoad();
        });

        DetectProfileIsOpenOnLoad();
        
        areEventsSet = true;
    }

    /** Disconnect the mutation observer we may have setup for the app.zoominfo.com website */
    function disconnectAppDotZoomInfoMutationObserver() {
        if (appDotZoomInfoMutationObserver) {
            appDotZoomInfoMutationObserver.disconnect();
        }
    }

    return {
        SetupZoomInfoV2Events: SetupZoomInfoV2Events,
        DisconnectAppDotZoomInfoMutationObserver: disconnectAppDotZoomInfoMutationObserver
    }
})();

    /**
     * Sets events for particular websites
     * @returns {} 
     */
    function listenForPageEvents() {
        ListenForPageEventsMonster();
        ListenForPageEventsIndeed();

        triggerCompanySearch();

        var urlCheck = document.URL.toLowerCase();

        if (urlCheck.indexOf('www.oilandgasjobsearch.com/candidate') != -1
            ^ (urlCheck.indexOf('zoominfo.com') != -1 && urlCheck.indexOf("go.zoominfo.com") === -1)
            ^ urlCheck.indexOf('recruit.identified.com') > -1
            ^ urlCheck.indexOf("xing.com/xtm") > -1) {
            //Backup method - listen for all DOM changes
            if (eventSet) {
                eventSet = false;
                window.addEventListener("DOMSubtreeModified", function () {
                    if (urlCheck.indexOf("www.oilandgasjobsearch.com/candidate") > -1 ^ urlCheck.indexOf('zoominfo.com') != -1 ^ urlCheck.indexOf('arbeitgeber.monster.de/') != -1 ^ urlCheck.indexOf('recruit.identified.com') > -1 ^ urlCheck.indexOf("xing.com/xtm") > -1) {
                        var dt1 = new Date();

                        var diff = Math.abs(dlastCheck - dt1);

                        if (diff > 70) {
                            OnDomSubtreeModified();
                        }
                    }
                }, false);

            }
        }

        if (urlCheck.indexOf("workopolis.com/candidatesourcing") != -1) {
            if (eventSet) {
                eventSet = false;
                var resumeContainer = document.querySelector('#resume');
                resumeContainer.addEventListener("DOMSubtreeModified", function () {
                    var dt1 = new Date();

                    var diff = Math.abs(dlastCheck - dt1);

                    if (diff > 70) {
                        OnDomSubtreeModified();
                    }
                }, false);
            }
        }

        //Set click events for Universobit
        if (urlCheck.indexOf("universobit.com.ar/empresas") != -1) {
            if (eventSet) {
                eventSet = false;
                    ListenForPageEventsUniversoBit();
                }
        }

        if (urlCheck.indexOf(".termscout.com") !== -1) {
            //only setup first bar drop
            if (eventSet) {
                eventSet = false; //event are setup - dont go again
                ListenForPageEventsTermScout();
            }
        }
        
        const hostName = window?.location?.hostname;
        if (hostName && hostName.endsWith(".linkedin.com") && eventSet) {
            eventSet = false;
            linkedInHelper.ListenForPageEventsLinkedIn();       
        }

        if (urlCheck.indexOf("go.zoominfo.com" ) !== -1 || urlCheck.indexOf("app.zoominfo.com") !== -1) {
            zoomInfoEvents.SetupZoomInfoV2Events();
            eventSet = false;
        }

        if (urlCheck.indexOf(".entelo.com") !== -1) {
            SetupEnteloEvents();
            eventSet = false;
        }

        if (urlCheck.indexOf(".crunchbase.com") !== -1) {
            crunchbaseHelper.ListenForPageEventsCrunchbase();
            eventSet = false;
        }

        if (urlCheck.indexOf("recruitment.careerbuilder.com/candidates/candidates-list") !== -1) {
            careerBuilderEvents.SetupCareerBuilderSearchListParser();
        }
    }


    function OnDomSubtreeModified() {
        dlastCheck = new Date();
        var url = document.URL.toLowerCase();


        //Monster
        if ((url.indexOf("hiring.monster.") > -1 && url.indexOf("searchresults.aspx?") > -1) ^ (url.indexOf("hiring.monster.") > -1 && url.indexOf("/jcm/candidates") > -1) ^ (url.indexOf("arbeitgeber.monster.de") > -1 && url.indexOf("/jcm/") > -1)) {
            if (url.indexOf("#s=grid") == -1) {
                var checkOnProfile = '' + BarJsGetValueByXpath('//*[@id="ctl00_ctl00_ContentPlaceHolderBase_ContentPlaceHolderRight_CndTop_lblRecordPortion"]');
                if (checkOnProfile == '') {
                    checkOnProfile = '' + BarJsGetValueByXpath('//*[@id="ctl00_ctl00_ContentPlaceHolderBase_ContentPlaceHolderRight_ctl00_lblRecordPortion"]');
                }
                checkOnProfile = checkOnProfile.toLowerCase();
                if (checkOnProfile.indexOf('(1 of 1 candidates)') > -1 ^ checkOnProfile.indexOf('(1 von 1 kandidaten)') > -1) {
                    if (barLoadedCount == 0) {
                        var name = '' + BarJsGetValueByXpath('//*[@id="ctl00_ctl00_ContentPlaceHolderBase_ContentPlaceHolderRight_lld_Detail_controlDetailTop_lbName"]');
                        if (name == '') {
                            name = '' + BarJsGetValueByXpath('//*[@id="CandidateDetailTop_lbName"]');
                        }
                        if (name != '' && name != lastSearchRunWhenDomModified) {

                            //console.log("search 3");

                            proc.LoadFrame(name, 1);
                            lastSearchRunWhenDomModified = name;
                        }
                    }
                }
                else {

                    proc.HideFrame();
                }
            }
            else {

                proc.HideFrame();

            }
        }


        //Oil and Gas Job Search
        if (url.indexOf('oilandgasjobsearch.com/candidate') > -1) {
            dropBarUsingIframe();
        }

        if (url.indexOf('recruit.identified.com/prospects') > -1) {
            var checkOnProfile = BarJsGetValueByXpath('//*[@id="jobs-view"]/div[1]/h3').toLowerCase();
            if (checkOnProfile === 'work experience') {
                if (barLoadedCount == 0) {
                    setTimeout(function () {
                        dropBarUsingRulesEngine();
                    }, 250);
                }
            }
            else {
                proc.HideFrame();
            }
        }

        if (url.indexOf('recruit.identified.com/search') > -1) {
            proc.HideFrame();
        }

        if (url.indexOf('zoominfo.com/') > -1) {
            window.onhashchange = function () {

            if (document.URL.indexOf("#!search/profile") > -1) {
                dropBarUsingRulesEngine();
            } else {
                proc.HideFrame();
            }
        };
    }
    if (url.indexOf('workopolis.com/candidatesourcing') > -1) {
        if (url.indexOf('/#!/resumes') > -1) {
            dropBarUsingRulesEngine(url);
        }
        else {
            proc.HideFrame();
        }
    }
    if (url.indexOf('xing.com/xtm/') > -1) {
        if (url.indexOf('/profile') > -1) {
            dropBarUsingRulesEngine(url);
        }
        else {
            lastSearchRunWhenDomModified = "";
            proc.HideFrame();
        }
    }
    }

    function dropBarUsingIframe() {
        var url = document.URL.toLowerCase();
        if (url.indexOf("oilandgasjobsearch.com/candidate") > -1) {
            var localIframes = 0;
            if (document.getElementsByTagName("iframe").length > 0) {

                var iframes = document.getElementsByTagName("iframe");
                for (var i = 0; i < iframes.length; i++) {

                    var this_iframe = iframes[i];
                    var frameDomain = '';
                    if (SplitString(this_iframe.src, '/').length > 2) {
                        frameDomain = SplitString(this_iframe.src, '/')[2];
                    }

                    //Only bother to evaluate if iframe comes from the same place as the site itself
                    if (this_iframe.src.indexOf('/Candidate/Details/') > -1) {
                        //if (frameDomain == document.domain) {
                        localIframes++;
                        setTimeout(function () {
                            if (document.URL.toLowerCase().indexOf("/candidate/cvsearch") > -1) {
                                var path = "//*[@id='candidatedetails']/div/div[1]/div[2]/table/tbody/tr[2]/td[2]";
                                var frameDoc = this_iframe.contentDocument;

                                var element = frameDoc.evaluate(path, frameDoc, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
                                if (element) {
                                    if (element.innerText.length > 0) {

                                        //console.log("search 2");

                                        proc.LoadFrame(element.innerText, 1);
                                        return;
                                    }
                                }

                            }
                        }, 1000);
                    }
                }
                //End loop

                if (localIframes == 0) {
                    proc.HideFrame();
                }
            }
        }
    }

    function isInBlackList(comparisonURL) {
        ///<summary>Checks through an array blacklisted pages. Returns a bool. </summary>
        //Z 28/11/14 - the #s=grid exception means the events are not wired in Monster so removed
        var blacklist = [
            "www.oilandgasjobsearch.com/Candidate/Details/",
            //"#s=grid",
            "#s=pageGridState"
        ];
        for (var i = 0; i < blacklist.length; i++) {
            if (comparisonURL.indexOf(blacklist[i].toLowerCase()) > -1) {
                return true;
            }
        }

        return false;
    }

    function TitleSplit(splitter, splitIndex) {
        var name = "";
        var title = document.title;
        if (title.indexOf(splitter) > -1) {
            name = title.split(splitter)[splitIndex];
        }
        return name;
    }

    function FindNameGooglePlus() {
        var _name = "";
        var title = document.title;
        if (title.indexOf("-") > -1) {
            _name = title.split('-')[0];
        }
        return _name;
    }

    function FindNameDice() {
        var name = "";
        var title = document.title;
        if (title.indexOf("About") > -1) {
            if (title.split("About ").length > 1) {
                name = title.split("About ")[1];
            }
        }
        return name;

    }

    function FindNameFacebook() {
        var name = BarJsGetValueByXpath('/HTML/BODY/DIV[2]/DIV[2]/DIV/DIV/DIV[2]/DIV[2]/DIV[2]/DIV/DIV/DIV/DIV[3]/DIV/DIV[2]/DIV[2]/H2');
        return name;
    }


    function FindNameLinkedinRecruiter() {
        var elems = document.getElementsByTagName('span'), i;
        var ret = '';

        for (i in elems) {
            if ((" " + elems[i].className + " ").indexOf(" " + ("given-name") + " ") > -1) {
                ret = elems[i].innerText.trim();


            }
        }

        return ret;
    }

    function FindNameLinkedinCompany() {
        var elems = document.getElementsByTagName('div'), i;
        var name = '';

        for (i in elems) {
            if ((" " + elems[i].className + " ").indexOf(" " + ("name-wrapper") + " ") > -1) {
                name = elems[i].innerText.trim();


            }
        }

        if (name == '') {
            name = BarJsGetValueByXpath("//*[@id='body']/div[2]/div[1]/div[1]/div/h1");
        }

        return name;
    }


    function FindName() {
        var elems = document.getElementsByTagName('span'), i;
        var ret = '';

        for (i in elems) {
            if ((" " + elems[i].className + " ").indexOf(" " + ("full-name") + " ") > -1) {
                ret = elems[i].innerText;


            }
        }

        return ret;
    }

    function FindNameXing() {
        var elems = document.getElementsByTagName('span'), i;
        var ret = '';

        for (i in elems) {
            if ((" " + elems[i].className + " ").indexOf(" " + ("given-name") + " ") > -1) {
                ret = ret + elems[i].innerText;
            }
            if ((" " + elems[i].className + " ").indexOf(" " + ("family-name") + " ") > -1) {
                ret = ret + " " + elems[i].innerText;
            }

        }

        return ret;
    }
    function FindNameXingCompany() {
        var title = document.title;
        var name = title.split('|')[0];

        return name.trim();
    }

    function FindNameCVLibrary() {
        var title = document.title;
        var name = title.split('-')[0];

        return name.trim();
    }

    function SplitString(input, splitter) {
        return input.split(splitter);
    }

    function BarJsGetValueByXpath(path) {

        var valueStr = '';
        try {

            var element = document.evaluate(path, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;

            if (element != null) {

                if (element.innerText) {
                    valueStr = htmlspecialchars_decode(element.innerText).trim();
                }
                else if (element.value) {
                    valueStr = htmlspecialchars_decode(element.value).trim();
                }
                else if (element.textContent)
                {
                    valueStr = element.textContent;
                }
            }

        }
        catch (error) {

        }

        //Have a retry process in case the xpath we used doesn't return a node value
        if (!valueStr) {
            valueStr = EbstaXpath.GetStringByAnyType(path);
        }

        return valueStr;
    }

    function htmlspecialchars_decode(string, quote_style) {
        // http://kevin.vanzonneveld.net
        // +   original by: Mirek Slugen
        // +   improved by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
        // +   bugfixed by: Mateusz "loonquawl" Zalega
        // +      input by: ReverseSyntax
        // +      input by: Slawomir Kaniecki
        // +      input by: Scott Cariss
        // +      input by: Francois
        // +   bugfixed by: Onno Marsman
        // +    revised by: Kevin van Zonneveld (http://kevin.vanzonneveld.net)
        // +   bugfixed by: Brett Zamir (http://brett-zamir.me)
        // +      input by: Ratheous
        // +      input by: Mailfaker (http://www.weedem.fr/)
        // +      reimplemented by: Brett Zamir (http://brett-zamir.me)
        // +    bugfixed by: Brett Zamir (http://brett-zamir.me)
        // *     example 1: htmlspecialchars_decode("<p>this -&gt; &quot;</p>", 'ENT_NOQUOTES');
        // *     returns 1: '<p>this -> &quot;</p>'
        // *     example 2: htmlspecialchars_decode("&amp;quot;");
        // *     returns 2: '&quot;'
        var optTemp = 0,
        i = 0,
        noquotes = false;
        if (typeof quote_style === 'undefined') {
            quote_style = 2;
        }
        string = string.toString().replace(/&lt;/g, '<').replace(/&gt;/g, '>');
        var OPTS = {
            'ENT_NOQUOTES': 0,
            'ENT_HTML_QUOTE_SINGLE': 1,
            'ENT_HTML_QUOTE_DOUBLE': 2,
            'ENT_COMPAT': 2,
            'ENT_QUOTES': 3,
            'ENT_IGNORE': 4
        };
        if (quote_style === 0) {
            noquotes = true;
        }
        if (typeof quote_style !== 'number') { // Allow for a single string or an array of string flags
            quote_style = [].concat(quote_style);
            for (i = 0; i < quote_style.length; i++) {
                // Resolve string input to bitwise e.g. 'PATHINFO_EXTENSION' becomes 4
                if (OPTS[quote_style[i]] === 0) {
                    noquotes = true;
                } else if (OPTS[quote_style[i]]) {
                    optTemp = optTemp | OPTS[quote_style[i]];
                }
            }
            quote_style = optTemp;
        }
        if (quote_style & OPTS.ENT_HTML_QUOTE_SINGLE) {
            string = string.replace(/&#0*39;/g, "'"); // PHP doesn't currently escape if more than one 0, but it should
            // string = string.replace(/&apos;|&#x0*27;/g, "'"); // This would also be useful here, but not a part of PHP
        }
        if (!noquotes) {
            string = string.replace(/&quot;/g, '"');
        }
        // Put this in last place to avoid escape being double-decoded
        string = string.replace(/&amp;/g, '&');

        return string;
    }

    var wizardclosed = true; 

    function FrameWorker() {
        /**
         * Sends a message to background to ensure this website isn't blacklisted for dropping the bar
         * @param {} functionToCallOnSuccess 
         * @returns {} 
         */
        var shouldFrameLoad = function(functionToCallOnSuccess) {
            chrome.extension.sendMessage({ MessageType: "ShouldWeShowBar", Url: document.URL }, function (response) {
                if (response === true) {
                    functionToCallOnSuccess();
                }
            });
        }

        /**
         * Determines if we should use the ReplaceFrameLocationByContentWindow process to change the bar
         * state, instead of altering the iframe src directly as is standard.
         * This replaces the location of the bar frame "inside" the frame, instead of altering
         * the src of the frame - what this does is change the bars location but without affecting the page history
         * This at the moment is an alternative way to change the bar on pages where us doing it by altering the frames src
         * attribute was causing issues with history
         * @returns {boolean} - true / false
         */
        function ShouldWeUseReplaceFrameLocationByContentWindow() {
            var websiteArray = [
                "talentsearch.seek.com.au"
            ];

            return websiteArray.findIndex(x => document.URL.toLowerCase().indexOf(x) > -1) > -1;
        }

        /**
         * Starts the intial process of dropping the Ebsta Bar
         * @param {} name 
         * @param {} typeOfSearch 
         * @returns {} 
         */
        this.LoadFrame = function (name, typeOfSearch) {
            var callBack = (function() { this.insertBar(name, typeOfSearch); }).bind(this);
            shouldFrameLoad(callBack);
        }

        /**
         * Inserts the bar into a webpage
         * @param {} name 
         * @param {} typeOfSearch 
         * @returns {} 
         */
        this.insertBar = function(name, typeOfSearch){
            //console.time('FrameWorkerLoadFrame');
            //console.log("bar.js: Got here");
            //if (barLoadedCount == 0) {        
            var stdLocationBar = false;
            var showNewBar = true;

            var loadWizards = true;

            if (trialLicenceFound + "" == "true" && LicenceFound == "Free") {
                loadWizards = false;
            }

            var loadbar = true;
            //Stop the bar intially appearing when in Gmail when not in an email
            if (document.URL.toLowerCase().indexOf("mail.google.com") != -1) {
                stdLocationBar = true;
                if (gmailBar + "" == "true") {
                    loadbar = false; 
                }
            }
            //console.log("bar.js: Got here 2");

            // email tracking has been clicked
            var unknownWebsiteValue = 400;
            var knownWebsiteValue = 401;
            if (typeOfSearch == knownWebsiteValue || typeOfSearch == unknownWebsiteValue) {

                showNewBar = true;
                loadbar = true;
            }

            if (stdLocationBar) {
                if (abRules.ShowNewBar + "" == "1")
                {
                    showNewBar = true;
                }
            }
            else {
                if (abRules.ShowNewBarEveryWhere + "" == "1") {
                    showNewBar = true; 
                }
            }
        
            if (loadbar) {

                var additionalQuery = '';

                //for searchtype 4 (custom bar search) we have concatenated the email and the name so we extract the name to pass through.
                if (typeOfSearch === 4) {
                    var n = name.search(",,,");
                    var personname = name.substring(n + 3);
                    additionalQuery = '&personname=' + personname;
                }

                //amend the name variable so it only shows the email address and not the name as well.
                var n = name.search(",,,");

                if (n > -1) {
                    name = name.substring(0, n);
                }

                //Show LicenceFound.
                if (!this._view) {
                    barLoadedCount++;
                    if (document.URL.toLowerCase().indexOf('.xing.') > -1) {
                        this._view = document.body.insertBefore(document.createElement("iframe"), document.body.childNodes[2]);
                    }
                    else {
                        this._view = document.body.insertBefore(document.createElement("iframe"), document.body.firstChild);
                    }
                    if (showNewBar) 
                    {
                        this._view.id = "ebsta-frame";
                        this._view.style.position = "fixed";
                        this._view.style.height = "50px";
                        this._view.height = "50px";

                        if (document.URL.toLowerCase().indexOf("mail.google.com") != -1) {
                            this._view.style.zIndex = EBSTA_GMAIL_BAR_ZINDEX;
                        } else {
                            this._view.style.zIndex = "100001";
                        }
                    
                        this._view.style.left = "0";
                        this._view.style.right = "0";
                        this._view.style.top = "0";
                        this._view.style.display = "inline";
                        this._view.style.width = "100%";
                    
                        this._view.scrolling = "no";
                        this._view.setAttribute("frameborder", "0");
                        this._view.width = "100%";
                        this._view.style.display = "inline";
                    }
                    else {
                        this._view.id = "ebsta-frame";
                        this._view.style.position = "fixed";
                        this._view.style.zIndex = "100001";
                        this._view.style.left = "0";
                        this._view.style.right = "0";
                        this._view.style.top = "0";
                        this._view.style.display = "inline";
                        this._view.style.width = "100%";
                        this._view.style.height = "30px";
                        this._view.scrolling = "no";
                        this._view.setAttribute("frameborder", "0");
                        this._view.width = "100%";
                        this._view.height = "30px";
                        this._view.style.display = "inline";
                    }
                    //}    

                    if (document.URL.indexOf("mail.google.com") != -1) {
                        //this._view.style.zIndex = "0";

                        var headStyle = document.createElement("style");
                        headStyle.innerText = "#ci-sidepanel{margin-top:50px;}";
                        headStyle.removeAttribute("style");

                        document.head.insertBefore(headStyle, document.head.lastChild);

                    }

                    if (showNewBar) 
                    {
                        if (!ShouldWeUseReplaceFrameLocationByContentWindow()) {
                            this._view.src = chrome.extension.getURL("/html/bar-1.html" + "?st=" + typeOfSearch + "&changeView=1&name=" + name + '&co=' + isChatterOnly + '&lic=' + LicenceFound + additionalQuery);
                        } else {
                            this._view.contentWindow.location.replace(chrome.extension.getURL("/html/bar-1.html" + "?st=" + typeOfSearch + "&changeView=1&name=" + name + '&co=' + isChatterOnly + '&lic=' + LicenceFound + additionalQuery));
                        }
                    }
                    else {
                        this._view.src = chrome.extension.getURL("/html/bar.html" + "?st=" + typeOfSearch + "&changeView=1&name=" + name + '&co=' + isChatterOnly + '&lic=' + LicenceFound + additionalQuery);
                    }
                    ////
                
                    //if (document.URL.indexOf(".linkedin.") == -1) {

                    FixHeaderIssues(showNewBar);
                    //}


                } else {
                    barLoadedCount++;
                   // console.log("bar.js: Got here 5");
                    //We should always be showing the new bar in Gmail and Linkedin
                    if (document.URL.toLowerCase().indexOf("mail.google.com") != -1 || document.URL.toLowerCase().indexOf(".linkedin.") != -1) {
                        showNewBar = true;
                    }

                    if (showNewBar) {

                        if (document.URL.toLowerCase().indexOf("mail.google.com") != -1) {
                            this._view.style.height = "600px";
                            this._view.height = "600px";
                            this._view.style.zIndex = "0";
                            document.body.style.marginTop = "600px";
                        } else {
                            this._view.height = "50px";
                            this._view.style.height = "50px";
                            this._view.style.zIndex = "100001";
                            document.body.style.marginTop = "50px";
                        }
                    
                        this._view.style.display = "inline";                    
                        if (!ShouldWeUseReplaceFrameLocationByContentWindow()) {
                            this._view.src = chrome.extension.getURL("/html/bar-1.html" + "?st=" + typeOfSearch + "&changeView=1&name=" + name + '&co=' + isChatterOnly + '&lic=' + LicenceFound + additionalQuery);
                        } else {
                            this._view.contentWindow.location.replace(chrome.extension.getURL("/html/bar-1.html" + "?st=" + typeOfSearch + "&changeView=1&name=" + name + '&co=' + isChatterOnly + '&lic=' + LicenceFound + additionalQuery));
                        }
                    }
                    else {
                        this._view.height = "30px";
                        this._view.style.height = "30px";
                        this._view.style.display = "inline";
                        document.body.style.marginTop = "30px";
                        this._view.src = chrome.extension.getURL("/html/bar.html" + "?st=" + typeOfSearch + "&changeView=1&name=" + name + '&co=' + isChatterOnly + '&lic=' + LicenceFound + additionalQuery);
                    }

                    FixHeaderIssues(showNewBar);                   
                }
            }
            //console.timeEnd('FrameWorkerLoadFrame');
        }; 
   
        this.ReRunSearch = function () {
            if (!ShouldWeUseReplaceFrameLocationByContentWindow()) {
                if (this._view && this._view.src != null) {
                    this._view.src = this._view.src.replace("changeView=1", "changeView=0");
                }
            } else {
                //it does not like us working cross origin here and trying to replace the 
                //location inside the frame, starts affecting the rest of the widnow then and potentially we may see
                //[Exception: DOMException: Blocked a frame with origin "....." from accessing a cross-origin frame.]
                //so keep it simple and just re drop the bar
                dropBarUsingRulesEngine();
            }
        };

        this.HideFrame = function () {
            //console.log("reached the call to HideFrame");
            barLoadedCount = 0;
            this._view = null;

            //this._view.style.display = "none";
            //        document.body.style.marginTop = "0px";
            if (document.getElementById("ebsta-frame") != null) {
                document.body.removeChild(document.getElementById("ebsta-frame"));
                var currentTop = document.body.style.marginTop.replace("px", "");

                var newTop = (currentTop - 30) + "px";


                if (document.getElementsByClassName('global-nav').length > 0) {
                    document.getElementsByClassName('global-nav')[0].style.marginTop = "0";
                }

                document.body.style.marginTop = newTop;


            }
            UndoFixHeaderIssues();

        };

        this.DisplayFrame = function () {
            var showNewBar1 = false;
            if (this._view.height == "600px") {
                this._view.height = "600px";
                this._view.style.height = "600px";
                this._view.style.display = "inline";
                document.body.style.marginTop = "50px";
                showNewBar1 = true;
            }
            else {
                this._view.height = "30px";
                this._view.style.height = "30px";
                this._view.style.display = "inline";
                document.body.style.marginTop = "30px";
        
            }
            FixHeaderIssues(showNewBar1);
        };

        this.RightClickSearchLoadBar = function (name) {
            emailAddressToSearchJsonParse = "";
            if (ValidateEmail(name)) {
                emailAddressToSearchJsonParse = name;
            }

            fullnameToSearchJsonParse = "";

            //Custom Bar Search. Check if we are doing a default or custom bar search. searchtype = type 4.
            if (useDefaultBarSearch)
            {
                this.LoadFrame(name, 3);
            }
            else
            {
                var nameandemail = "" + ",,," + name;

                this.LoadFrame(nameandemail,4)
            }
        };
    }
    
    // When inserting/removing the bar the navigation panel on the left hand side becomes out of sync
    // This function resizes the window by css in order to trigger Gmail's own scripting which helps resize elements on the page
    function FixGoogleLeftPaneSection() {
        setTimeout(function () {
            var changeHeightToCallGmailScript = false;

            //Putting some classes on the main dom element will help us only call this function when the bar first appears/disappears
            if ($('#ebsta-frame').length == 0 && !$('.aAX').hasClass("eb-BarRemoved")) {
                changeHeightToCallGmailScript = true;
                $('.aAX').addClass("eb-BarRemoved").removeClass('eb-BarAdded');
            }
            else if ($('#ebsta-frame').length > 0 && !$('.aAX').hasClass("eb-BarAdded")) {
                changeHeightToCallGmailScript = true;
                $('.aAX').addClass("eb-BarAdded").removeClass('eb-BarRemoved');
            }

            if(changeHeightToCallGmailScript){
                $('.aAX').css("height", "99%");
                setTimeout(function () {
                    $('.aAX').css("height", "100%");
                }, 500);
                //Test if we are using a Chromebook. If we are we want to call the window resize event as this seems to be
                //the only thing that fixes an issue with a long gmail folder list overhanging the GoogleHangout buttons.
                if (navigator != null && typeof navigator != 'undefined' ) {
                    var useragent = navigator.userAgent;
                    if (useragent) {
                        if (/\bCrOS\b/.test(useragent)) {
                            window.dispatchEvent(new Event('resize'));
                        }
                    }
                }
            }
        }, 2000);
    }

    /**
     * Contains a centralized object to help contain the way we change margins on elements to show the bar on certain websites
     * This helps us keep track of what elements we've tried to change on a site and be able to to manipulate this after if we need to
     */
    var GLOBAL_MARGIN_HELPERS = {
        XingMarginHelper: null,
        MonsterMarginHelper: null,
        LinkedInSalesNav: null,
        StandardMarginHelper: null
    };

    /**
     * Module to apply margin offsets so that the bar appears above all content
     */
    function FixMarginHelper(barHeight, classesToChange, querySelectorsToChange, fixPositionTop) {

        var lBarHeight = barHeight;
        var classToChangeArray = classesToChange;
        var querySelectorsToChangeArray = querySelectorsToChange;
        var isFixPositionTop = fixPositionTop;

        /**
         * Either applys or removes the margin / position top from the element specified
         * @param {} element 
         * @param {} addMargin 
         * @returns {} 
         */
        function ApplyOrRemoveMargin(element, addMargin) {
            if (!isFixPositionTop) {
                element.style.marginTop = addMargin ? lBarHeight : 0;
            } else {
                element.style.top = addMargin ? lBarHeight : 0;
            }
        }

        /**
         * Loops through all classes specified and either adds or removes what we've changed to the DOM to dislay the bar
         * @param {} addMargin 
         * @returns {} 
         */
        function SearchAndChangeByClass(addMargin, elementsToLookFor) {
            elementsToLookFor.forEach(function (className) {
                var classFound = document.getElementsByClassName(className)[0];
                if (classFound) {
                    ApplyOrRemoveMargin(classFound, addMargin);
                }
            });
        }

        /**
         * Finds the element by using the QuerySelector functionality to find a more specific item
         * @param {} addMargin 
         * @returns {} 
         */
        function SearchAndChangeByQuerySelector(addMargin, selectorsTolookFor) {
            selectorsTolookFor.forEach(function (selector) {
                var elemFound = document.querySelector(selector);
                if (elemFound) {
                    ApplyOrRemoveMargin(elemFound, addMargin);
                }
            });
        }

        /**
         * Searches for elements by Class and changes the maring on elements to help the bar appear
         * @param {} barHeight 
         * @returns {} 
         */
        function FixHeadersByClassName() {
            if (classToChangeArray != null) {
                SearchAndChangeByClass(true, classToChangeArray);
            }
        }

        /**
        * Applies margins to the DOM by finding elements using QuerySelector
        * @param {} barHeight 
        * @returns {} 
        */
        function FixHeadersByQuerySelector() {
            if (querySelectorsToChangeArray != null) {
                SearchAndChangeByQuerySelector(true, querySelectorsToChangeArray);
            }
        }

        /**
        * Undoes any classes we changed to the DOM to display the page as it normally would look
        * @returns {} 
        */
        function UndoFixByClassName() {
            if (classToChangeArray != null) {
                SearchAndChangeByClass(false, classToChangeArray);
            }
        }

        /**
        * Undoes any classes we changed to the DOM to display the page as it normally would look
        * @returns {} 
        */
        function UndoFixByQuerySelector() {
            if (querySelectorsToChangeArray != null) {
                SearchAndChangeByQuerySelector(false, querySelectorsToChangeArray);
            }
        }

        return {
            FixHeaderIssues: function () {
                FixHeadersByClassName();
                FixHeadersByQuerySelector();
            },

            UndoFix: function () {
                UndoFixByClassName();
                UndoFixByQuerySelector();
            },
            /**
            * Calls the apply or remove margin with a interval, that tries n times to find
            * an element on the page to run the function on. Use on pages where elements are loaded in asynchronous
            * @param {any} querySelector - Query selector to find element, e.g ".fooClass" or div[data-cy="foo"]
            * @param {any} addMargin - true (add margin) / false (remove margin)
            */
            ApplyOrRemoveMarginWithRetry: function (querySelector, addMargin) {
                var retryCount = 0;
                var retryLimit = 20;
                var waitBetweenTry = 500;
                var marginApplied = false;

                //start retry - up to n times each n seconds, or until we have fixed the modal
                var retryInterval = setInterval(function () {
                    if (retryCount < retryLimit && !marginApplied) {
                        retryCount++;
                        var element = document.querySelector(querySelector);
                        if (element) {
                            ApplyOrRemoveMargin(element, addMargin);
                            marginApplied = true;
                        }
                    } else {
                        clearInterval(retryInterval);
                    }
                }, waitBetweenTry);
            }
        }
    }

    function FixHeaderIssues(showNewBar2) {
    
        //Clear previous body margins: If the ebsta bar is show. Clear body margins, before they are set to ebstas
        var bodyFixApplied = false;
        var fixNeeded = false;
        if (document.URL.indexOf("mail.google.com") != -1) {


            $('#ebsta-frame').each(function () {
                var offstTop1 = $(this).offset().top;
                
                if (offstTop1 != 0) {
                    $(this).offset().top = "0px";
                    document.body.style.marginTop = "0px";
                    
                    fixNeeded = true;
                  
                }
            });

            FixGoogleLeftPaneSection();
        }
        
        var barheight = "30px"; 
    
        if (showNewBar2) {
            barheight = "50px";
            if (!fixNeeded) {
                document.body.style.marginTop = "50px";
            }
        }
        else {
            document.body.style.marginTop = "30px";
        }
        
        if (document.URL.indexOf("mail.google.com") != -1) {

            var gNav = document.evaluate("/html/body/div[5]/div[2]/div", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            if (gNav != null) {
                gNav.style.position = "fixed";
            } else { //cirus window changes our xpaths...
                var gNavCirus = document.evaluate("//*[@id='ciAppContainer']/div[6]/div[2]/div", document, null, XPathResult.ANY_TYPE, null).iterateNext();
                if (gNavCirus != null) {
                    gNavCirus.style.position = "fixed";
                }
            }
            if (!fixNeeded) {
                document.body.style.marginTop = barheight;
            }

            //Fix horixtonal scrollbar and quick icons at the bottom left from disappearing when bar drops
            $('.Tm.aeJ, .nH.oy8Mbf.nn.aeN').each(function ()
            {
                //If we've already changed this don't do it again
                if (!$(this).hasClass("ebChange"))
                {
                    $(this).css("height", "-=" + barheight);
                    $(this).addClass("ebChange");
                }
            });
        }

        if (document.URL.indexOf("app.datafox.co") != -1) {
            var foxNav = document.evaluate("//*[@id='SPheader']", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            if (foxNav != null) {
                foxNav.style.marginTop = barheight;
            }

            var foxNav2 = document.evaluate("//*[@id='ember885']", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            if (foxNav2 != null) {
                foxNav2.style.marginTop = "90px";
            }        
        }

        if (document.getElementById("header") != null) {
            document.getElementById("header").style.marginTop = barheight;
        }
        if (document.getElementsByClassName("global-nav").length > 0 && document.URL.indexOf('linkedin.com/talent') === -1) {
            document.getElementsByClassName("global-nav")[0].style.marginTop = barheight;
        }
        if (document.URL.indexOf("theitjobboard") > -1) {
            if (document.getElementsByClassName("main").length > 0) {
                document.getElementsByClassName("main")[0].style.marginTop = "-" + barheight; // Fix
            }
        }
        if (document.URL.indexOf("doximity") > -1) {
            if (document.getElementsByClassName("mast grid").length > 0) {
                document.getElementsByClassName("mast grid")[0].style.marginTop = barheight;
            }
        }
        if (document.URL.indexOf(".3sourcing.") > -1) {
            if (document.getElementsByClassName("topbar").length > 0) {
                document.getElementsByClassName("topbar")[0].style.top = barheight;
            }
        }
        if (document.getElementById("blueBar") != null) {
            document.getElementById("blueBar").style.marginTop = barheight;
        }
        if (document.getElementById("hdMargin") != null) {
            document.getElementById("hdMargin").style.marginTop = barheight;
        }
        if (document.getElementById("dialogTopBar") != null) {
            document.getElementById("dialogTopBar").style.marginTop = barheight;
            if (document.getElementById("icon") != null) {
                document.getElementById("icon").style.marginTop = barheight;
            }
        }

        if (document.URL.indexOf('www.experteer.co.uk/recruiting/candidate') != -1) {

            var background = document.querySelector('html');
            var content = document.getElementById('pjpp_body');

            if(background != null) {
                background.style.backgroundPositionY = barheight;
            }

            if (content != null) {
                content.style.marginTop = "36px";
            }
        }

        if (document.URL.indexOf('plus.google.com') != -1) {
            /* Google+ Layout Adjustments*/
            var gNav = document.evaluate("/html/body/div[2]/div[1]", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gWhiteNav = document.evaluate("//*[@id='contentPane']/div/div[1]/div", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gSideNav = document.evaluate("//*[@id='content']/div[2]/div[1]", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gIconSound = document.evaluate("/html/body/div[2]/div[4]", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gIconHangout = document.evaluate("/html/body/div[2]/div[4]/div[1]/div[2]", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gContent = document.evaluate("//*[@id='content']", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            //var horizNavBar = document.evaluate("//*[@id='contentPane']/div/div[2]/div/div/div[1]/div[2]/div", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            //var addPeopleAndHangouts = document.evaluate("/html/body/div[2]/div[4]/div[1]/div[2]", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            //var hangOutPopUp = document.evaluate("//*[@id='ozIdRtRibbonChatRoster']", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            //var profileNavBar = document.evaluate('//*[@id="content"]/div[2]/div[2]', document, null, XPathResult.ANY_TYPE, null).iterateNext();
                
            if (gNav != null) {
                gNav.style.marginTop = barheight;
            }

            if (gWhiteNav != null) {
                gWhiteNav.style.marginTop = barheight;
            }

            if (gSideNav != null) {
                gSideNav.style.marginTop = barheight;
            }

            if (gIconSound != null) {
                gIconSound.style.marginTop = "74px";
            }

            if (gIconHangout != null) {
                gIconHangout.style.marginTop = "-88px";
            }

            if (gContent != null) {
                gContent.style.marginTop = barheight;
            }

        

        }
        if (document.URL.indexOf("dice.com") > -1) {
            if (document.getElementById("header-wrap") != null) {
                document.getElementById("header-wrap").style.marginTop = "15px";
            }
        }
        if (document.URL.indexOf("nursingjobs.us") > -1) {
            if (document.getElementsByClassName("navbar").length > 0) {
                document.getElementsByClassName("navbar")[0].style.marginTop = barheight;
            }
        }

        if (document.URL.indexOf("gagein.com/dragon/") != -1) {
            if (document.getElementsByClassName("lyt_member_header").length > 0) {
                document.getElementsByClassName("lyt_member_header")[0].setAttribute("style", "top:30px !important");
            }
            if (document.getElementsByClassName("lyt_manage_header").length > 0) {
                document.getElementsByClassName("lyt_manage_header")[0].setAttribute("style", "top:68px !important");
            }

        }

        if (document.URL.indexOf('source.gild.com/profiles/') != -1) {

            var header = document.getElementById('tuna-application-bar');

            if (header != null) {
                header.style.marginTop = barheight;
            }
        }

        //Attach listener to searchBox for keydown events and move results box on appearance
        if (document.URL.indexOf('linkedin.com') != -1) {  
            
            //Set the top position for search ovarlay when the Ebsta bar is turned on
            linkedInHelper.SetOverlayPositioning(barheight);

            var searchBox = document.getElementById('main-search-box');

            if (searchBox != null) {

                searchBox.addEventListener("keydown", function () {

                    var resultsBox = document.getElementById('universal-typeahead-results');

                    if (resultsBox != null) {
                        resultsBox.style.marginTop = "50px";
                    }

                });
            }

            //new LinkedIn navbar - if find assume new linkedin layout
            var headNavExtend = document.getElementById('extended-nav');
            if (headNavExtend != null) {
                headNavExtend.style.marginTop = barheight;

                //apply body this body fix only when new linkedin
                document.body.style.marginTop = (50 + headNavExtend.clientHeight) + "px";

                //track we have applied - stop overwrite
                bodyFixApplied = true;
       
                //fix for modals on new linkedin
                var ebstaBar = document.getElementById("ebsta-frame");
                if (ebstaBar != null);
                {
                    ebstaBar.style.zIndex = "100";
                }

            }
        
            if (document.URL.indexOf('linkedin.com/company') !== -1 || document.URL.indexOf('linkedin.com/in/') !== -1) {
                var headerBannerDiv = document.getElementById('header-banner');
                if (headerBannerDiv) {
                    headerBannerDiv.style.marginTop = "50px";
                }

                var headerNavDiv = document.getElementById('header-navigation');
                if (headerNavDiv) {
                    headerNavDiv.style.marginTop = "50px";
                }
            }

            //new style school site
            if (document.URL.indexOf('linkedin.com/school') !== -1) {
                var headerBannerDiv = document.getElementById('header-banner');
                if (headerBannerDiv) {
                    headerBannerDiv.style.marginTop = barheight;

                    document.body.style.marginTop = barheight;

                    //track we have applied - stop overwrite
                    bodyFixApplied = true;
                }
            }

            if (document.URL.indexOf('linkedin.com/sales') != -1) {
                var mainBodyDiv = document.getElementById('body');
                if (mainBodyDiv) {
                    mainBodyDiv.style.marginTop = "167px";
                }

                GLOBAL_MARGIN_HELPERS.LinkedInSalesNav = new FixMarginHelper(barheight, ["global-nav-primary-section", "global-nav-background"], null, false);
                GLOBAL_MARGIN_HELPERS.LinkedInSalesNav.FixHeaderIssues();
            }

            // Linkedin Recruiter
            if (document.URL.indexOf('linkedin.com/talent') !== -1) {
                let profileSideBarContainer = document.getElementsByClassName('pagination-header')[1];
                if (profileSideBarContainer) {
                    profileSideBarContainer.style.marginTop = barheight;
                    profileSideBarContainer.style.height = "calc(100vh - " + barheight;
                }
            }

            if (document.URL.toLowerCase().indexOf('linkedin.com/recruiter') === -1
                && document.URL.indexOf('linkedin.com/talent') === -1 && document.URL.indexOf('linkedin.com/sales') === -1) {
                barheight = "120px";
            }

            if (document.URL.indexOf('linkedin.com/profile/preview') != -1) {
                var profilePreviewPane = document.getElementById('preview-ctx');
                if (profilePreviewPane) {
                    profilePreviewPane.style.marginTop = "50px";
                }
            }
        }

        if (document.URL.indexOf("universobit.com.ar/empresas") != -1) {
            if (document.getElementsByClassName("menuGeneralEmpresasContainer").length > 0) {
                document.getElementsByClassName("menuGeneralEmpresasContainer")[0].setAttribute("style", "margin-top:" + barheight);
            }
        }

        if (document.URL.indexOf(".beyond.com") != -1) {
            if (document.getElementsByClassName("navbar-bs3 navbar-center-bs3 navbar-fixed-top-bs3").length > 0) {
                document.getElementsByClassName("navbar-bs3 navbar-center-bs3 navbar-fixed-top-bs3")[0].setAttribute("style", "margin-top:" + barheight);
            }

            if (document.getElementsByClassName("subhead").length > 0) {
                document.getElementsByClassName("subhead")[0].setAttribute("style", "margin-top:100px!important");
            }
        }

        if (document.URL.indexOf("data.channel-navigator.com") !== -1) {
            if (document.getElementsByClassName("navbar navbar-fixed-top  navbar-inverse noprint").length > 0) {
                document.getElementsByClassName("navbar navbar-fixed-top  navbar-inverse noprint")[0].setAttribute("style", "margin-top:" + barheight);
            }

            if (document.getElementsByClassName("mainContainer row-fluid").length > 0) {
                document.getElementsByClassName("mainContainer row-fluid")[0].setAttribute("style", "margin-top:140px");
            }
        }

        if (document.URL.indexOf(".termscout.com/") !== -1) {
            var navBarByClass = document.getElementsByClassName("headerPage navbar-fixed-top ng-scope");
            if (navBarByClass.length > 0) {
               navBarByClass[0].style.top = barheight;
            }
        }

        if (document.URL.indexOf(".xing.com") > -1) {
            //find first class that contains Header-headerWrapper- currently there will be an id appended at the end, eg
            //b54cdd76 seems liable to change
            GLOBAL_MARGIN_HELPERS.XingMarginHelper = new FixMarginHelper(barheight, null,
                ["[class*='Header-headerWrapper-']"], true);
            GLOBAL_MARGIN_HELPERS.XingMarginHelper.FixHeaderIssues();
        }

        if (document.URL.indexOf("hiring.monster.com/supersearch") > -1) {
            GLOBAL_MARGIN_HELPERS.MonsterMarginHelper = new FixMarginHelper(barheight, ["solid-nav"], ["m-popup.profile .popup-container"], false);
            GLOBAL_MARGIN_HELPERS.MonsterMarginHelper.FixHeaderIssues();
        }

        if (document.URL.indexOf("crunchbase.com") > -1) {
            GLOBAL_MARGIN_HELPERS.StandardMarginHelper = new FixMarginHelper(barheight, null, ["html > body header",
                ".mat-elevation-z5.mat-drawer", ".mat-elevation-z5.layout-row", "app-header"], false);
            GLOBAL_MARGIN_HELPERS.StandardMarginHelper.FixHeaderIssues();
        }

        if (!bodyFixApplied) {
            if (!fixNeeded) {
                document.body.style.marginTop = barheight;
            } else {

                setTimeout(function () {
                    document.body.style.marginTop = barheight;
                }, 15);
            }
        }
    }
    
    function UndoFixHeaderIssues() {
        
        //LogThis("undo header");
        if (document.getElementById("header") != null) {
            document.getElementById("header").style.marginTop = "0px";
        }
        if (document.getElementsByClassName("global-nav").length > 0) {
            document.getElementsByClassName("global-nav")[0].style.marginTop = "0px";
        }
        if (document.URL.indexOf("theitjobboard") > -1) {
            if (document.getElementsByClassName("main").length > 0) {
                document.getElementsByClassName("main")[0].style.marginTop = "0px";
            }
        }
        if (document.URL.indexOf("doximity") > -1) {
            if (document.getElementsByClassName("mast grid").length > 0) {
                document.getElementsByClassName("mast grid")[0].style.marginTop = "0px";
            }
        }
        if (document.URL.indexOf(".3sourcing.") > -1) {
            if (document.getElementsByClassName("topbar").length) {
                document.getElementsByClassName("topbar")[0].style.top = "0px";
            }
        }
        if (document.getElementById("blueBar") != null) {
            document.getElementById("blueBar").style.marginTop = "0px";
        }
        if (document.getElementById("hdMargin") != null) {
            document.getElementById("hdMargin").style.marginTop = "0px";
        }
        if (document.getElementById("dialogTopBar") != null) {
            document.getElementById("dialogTopBar").style.marginTop = "0px";
            if (document.getElementById("icon") != null) {
                document.getElementById("icon").style.marginTop = "0px";
            }

        }

        if (document.URL.indexOf('www.experteer.co.uk/recruiting/candidate') != -1) {

            var background = document.querySelector('html');
            var content = document.getElementById('pjpp_body');

            if (background != null) {
                background.style.backgroundPositionY = "0px";
            }

            if (content != null) {
                content.style.marginTop = "0px";
            }
        }

        if (document.URL.indexOf('plus.google.com') != -1) {
            /* Google+ Layout Adjustments*/
            var gNav = document.evaluate("/html/body/div[2]/div[1]", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gWhiteNav = document.evaluate("//*[@id='contentPane']/div/div[1]/div", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gSideNav = document.evaluate("//*[@id='content']/div[2]/div[1]", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gIconSound = document.evaluate("/html/body/div[2]/div[4]", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gIconHangout = document.evaluate("/html/body/div[2]/div[4]/div[1]/div[2]", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            var gContent = document.evaluate("//*[@id='content']", document, null, XPathResult.ANY_TYPE, null).iterateNext();


            gNav.style.marginTop = "0px";
            gWhiteNav.style.marginTop = "0px";
            gSideNav.style.marginTop = "0px";
            gIconSound.style.marginTop = "0px";
            gIconHangout.style.marginTop = "0px";
            gContent.style.marginTop = "0px";

        }
        if (document.URL.indexOf("dice.com") > -1) {
            if (document.getElementById("header-wrap") != null) {
                document.getElementById("header-wrap").style.marginTop = "0px";
            }
        }
        if (document.URL.indexOf("nursingjobs.us") > -1) {
            if (document.getElementsByClassName("navbar").length > 0) {
                document.getElementsByClassName("navbar")[0].style.marginTop = "0px";
            }
        }

        if (document.URL.indexOf("gagein.com/dragon/") != -1) {
            if (document.getElementsByClassName("lyt_member_header").length > 0) {
                document.getElementsByClassName("lyt_member_header")[0].setAttribute("style", "top:0px !important");
            }
            if (document.getElementsByClassName("lyt_manage_header").length > 0) {
                document.getElementsByClassName("lyt_manage_header")[0].setAttribute("style", "top:38px !important");
            }
        }
    
        if (document.URL.indexOf("app.datafox.co") != -1) {
            var foxNav = document.evaluate("//*[@id='SPheader']", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            if (foxNav != null) {
                foxNav.style.marginTop = "0px";
            }

            var foxNav2 = document.evaluate("//*[@id='ember885']", document, null, XPathResult.ANY_TYPE, null).iterateNext();
            if (foxNav2 != null) {
                foxNav2.style.marginTop = "0px";
            }
        }

        if (document.URL.indexOf("universobit.com.ar/empresas") != -1) {
            if (document.getElementsByClassName("menuGeneralEmpresasContainer").length > 0) {
                document.getElementsByClassName("menuGeneralEmpresasContainer")[0].setAttribute("style", "margin-top:0px");
            }
        }

        if (document.URL.indexOf(".beyond.com") != -1) {
            if (document.getElementsByClassName("navbar-bs3 navbar-center-bs3 navbar-fixed-top-bs3").length > 0) {
                document.getElementsByClassName("navbar-bs3 navbar-center-bs3 navbar-fixed-top-bs3")[0].setAttribute("style", "margin-top:0px");
            }

            if (document.getElementsByClassName("subhead").length > 0) {
                document.getElementsByClassName("subhead")[0].setAttribute("style", "margin-top:51px");
            }
        }

        if (document.URL.indexOf("mail.google.com/mail") != -1)
        {
            FixGoogleLeftPaneSection()

            //Fixes horixtonal scrollbar and quick icons at the bottom left
            $('.Tm.aeJ, .nH.oy8Mbf.nn.aeN').each(function ()
            {
                if ($(this).hasClass("ebChange"))
                {
                    $(this).css("height", "+=50");
                    $(this).removeClass("ebChange");
                }
            });
        }

        if (document.URL.toLowerCase().indexOf('linkedin.com/') !== -1)
        {
            //Set the search ovarlay position back to its original value when the Ebsta bar is turned off
            linkedInHelper.UnsetOverlayPositioning();

            if (document.URL.indexOf('linkedin.com/sales') != -1) {
                var mainBodyDiv = document.getElementById('body');
                if (mainBodyDiv) {
                    mainBodyDiv.style.marginTop = "97px";
                }
            }

            if (document.URL.indexOf('linkedin.com/profile/preview') != -1) {
                var profilePreviewPane = document.getElementById('preview-ctx');
                if (profilePreviewPane) {
                    profilePreviewPane.style.marginTop = "0px";
                }
            }

            if (document.URL.indexOf('linkedin.com/company') !== -1 || document.URL.indexOf('linkedin.com/in/') !== -1) {
                var headerBannerDiv = document.getElementById('header-banner');
                if (headerBannerDiv) {
                    headerBannerDiv.style.marginTop = "0px";
                }

                var headerNavDiv = document.getElementById('header-navigation');
                if (headerNavDiv) {
                    headerNavDiv.style.marginTop = "0px";
                }
            }

            //new linkedin layout nav
            var headNavExtend = document.getElementById('extended-nav');
            if (headNavExtend != null) {
                headNavExtend.style.marginTop = "0px";
            }

            //new style school site
            if (document.URL.indexOf('linkedin.com/school') !== -1) {
                var headerBannerDiv = document.getElementById('header-banner');
                if (headerBannerDiv) {
                    headerBannerDiv.style.marginTop = "0px";
                }
            }

            if (document.URL.indexOf('.linkedin.com/sales') !== -1) {
                if (GLOBAL_MARGIN_HELPERS.LinkedInSalesNav) {
                    GLOBAL_MARGIN_HELPERS.LinkedInSalesNav.UndoFix();
                }
            }

            // Linkedin Recruiter
            if (document.URL.indexOf('linkedin.com/talent') !== -1) {
                let profileSideBarContainer = document.getElementsByClassName('pagination-header')[1];
                if (profileSideBarContainer) {
                    profileSideBarContainer.style.marginTop = "0px";
                    profileSideBarContainer.style.height = "";
                }
            }
        }

        if (document.URL.indexOf("data.channel-navigator.com") !== -1) {
            if (document.getElementsByClassName("navbar navbar-fixed-top  navbar-inverse noprint").length > 0) {
                document.getElementsByClassName("navbar navbar-fixed-top  navbar-inverse noprint")[0].setAttribute("style", "margin-top:0px");
            }

            if (document.getElementsByClassName("mainContainer row-fluid").length > 0) {
                document.getElementsByClassName("mainContainer row-fluid")[0].setAttribute("style", "margin-top:90px");
            }
        }

        if (document.URL.indexOf(".termscout.com/") !== -1) {
            var navBarByClass = document.getElementsByClassName("headerPage navbar-fixed-top ng-scope");
            if (navBarByClass.length > 0) {
                navBarByClass[0].style.top = 0;
            }
        }

        if (document.URL.indexOf(".xing.com") > -1) {
            if (GLOBAL_MARGIN_HELPERS.XingMarginHelper) {
                GLOBAL_MARGIN_HELPERS.XingMarginHelper.UndoFix();
            }
        }

        if (document.URL.indexOf("hiring.monster.com/supersearch") > -1) {
            if (GLOBAL_MARGIN_HELPERS.MonsterMarginHelper) {
                GLOBAL_MARGIN_HELPERS.MonsterMarginHelper.UndoFix();
            }
        }

        if (document.URL.indexOf(".crunchbase.com") > -1) {
            if (GLOBAL_MARGIN_HELPERS.StandardMarginHelper) {
                GLOBAL_MARGIN_HELPERS.StandardMarginHelper.UndoFix();
            }
        }

        document.body.style.marginTop = "0px";
    }

    var proc = new FrameWorker();

    //Create a line sperator
    function CreateSeperator() {
        var sep = CreateSpan("", "ebstaZenSeperator");

        sep.style.display = "inline-block";
        sep.style.width = "3px";
        sep.style.height = "13px";
        sep.style.borderLeft = "1px solid #8b9496";

        return sep;
    }

    function GetAccessDetails() {
        var url = document.URL.toLowerCase();

        if (url.indexOf('https://console.ebsta.com/salesforce/') != -1) {

            if (url.indexOf('default.aspx') != -1 ^ url.indexOf('defaultsandbox.aspx') != -1) {
                var webUser = '';
                if(document.getElementById('defId_hdWebUserId') != null) {
                    webUser = document.getElementById('defId_hdWebUserId').value;
                }
                var sessionCode = '';
                if (document.getElementById('defId_hdSessionCode') != null) {
                    sessionCode = document.getElementById('defId_hdSessionCode').value;
                }
                var crmId = '';
                if (document.getElementById('defId_hdcrmId') != null) {
                    crmId = document.getElementById('defId_hdcrmId').value;
                }
                var firstname = '';
                if (document.getElementById('defId_hdfirstname') != null) {
                    firstname = document.getElementById('defId_hdfirstname').value;
                }
                var surname = '';
                if (document.getElementById('defId_hdsurname') != null) {
                    surname = document.getElementById('defId_hdsurname').value;
                }
                var crmurl = '';
                if (document.getElementById('defId_hdcrmurl') != null) {
                    crmurl = document.getElementById('defId_hdcrmurl').value;
                }
                var searchurl = "" + crmurl;
                var addurl = "" + crmurl;
                var updateurl = "" + crmurl;
                var selecturl = "" + crmurl;
                try {
                    searchurl = document.getElementById('defId_hdsearchurl').value;
                    addurl = document.getElementById('defId_hdaddurl').value;
                    updateurl = document.getElementById('defId_hdupdateurl').value;
                    selecturl = document.getElementById('defId_hdselecturl').value;
                } catch (error) {

                }
                var ebstalicence = '';
                if (document.getElementById('defId_hdebstalicence') != null) {
                    ebstalicence = document.getElementById('defId_hdebstalicence').value;
                }

                var databaseTypeId = '';
                if (document.getElementById('defId_hdDatabaseTypeId') != null) {
                    databaseTypeId = document.getElementById('defId_hdDatabaseTypeId').value;
                }

                var locale = '';
                if (document.getElementById('defId_hdLocale') != null) {
                    locale = document.getElementById('defId_hdLocale').value;
                }

                var chatterOnly = '';
                if (document.getElementById('defId_hdChatterOnly') != null) {
                    chatterOnly = document.getElementById('defId_hdChatterOnly').value;
                }

                if (webUser.length != 0) {
              
                    chrome.extension.sendMessage({ MessageType: "Auth", SessionCode: "" + sessionCode, WebUserId: "" + webUser, CrmId: "" + crmId, Firstname: "" + firstname, Surname: "" + surname, CrmUrl: "" + crmurl, SearchUrl: "" + searchurl, AddUrl: "" + addurl, UpdateUrl: "" + updateurl, SelectUrl: "" + selecturl, EbstaLicence: "" + ebstalicence, DatabaseTypeId: "" + databaseTypeId, Locale: "" + locale, ChatterOnly: "" + chatterOnly }, function (response) {

                    });
                }
            }
        }
    }

    //This function allows is to enable correct workflow in settings page after a user authorises gmail calendar sync
    function PostGmailAuth() {
        var url = document.URL.toLowerCase();
        if (url.indexOf('https://console.ebsta.com/google/calendarauthresponse.aspx') != -1)
        {
            var message = document.getElementById('ResponseMsg');

            var messageValue = message.value;

            if (messageValue.indexOf('1') != -1) {
                chrome.extension.sendMessage({ MessageType: "PostGmailAuth", AuthId: "true" }, function (response) { });
            }
        }
    }

    function PostGmailFullAuth() {
        var url = document.URL.toLowerCase();
        if (url.indexOf('https://console.ebsta.com/google/fullauthresponse.aspx') != -1) {
            var message = document.getElementById('AuthResult');

            var messageValue = message.value;

            if (messageValue.indexOf('1') != -1) {
                chrome.extension.sendMessage({ MessageType: "PostGmailFullAuth", Auth: "2" }, function (response) { });
            }
        }
    }

    function PassBackSLookup(entityType, db, searchUsed) {
   
    }


    function SetSaveOnSend() {
        //LogThis('SetSaveOnSend');
        AddButtonDelay();
    }

    function FormatNumber(number) {
        switch (number) {
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
            case 6:
            case 7:
            case 8:
            case 9:
                return "0" + number;
                break;
            default:
                return number;
                break;
        }
    }


    function FormatMonth(month) {
        switch (month) {
            case 0:
                return 'January';
            case 1:
                return 'February';
            case 2:
                return 'March';
            case 3:
                return 'April';
            case 4:
                return 'May';
            case 5:
                return 'June';
            case 6:
                return 'July';
            case 7:
                return 'August';
            case 8:
                return 'September';
                break;
            case 9:
                return 'October';
                break;
            case 10:
                return 'November';
                break;
            case 11:
                return 'December';
                break;
            default:
                return '';
                break;
        }
    }

    function StripHtml(html) {
        html = html.replace(/<br>/g, '\r\n');
        var tmp = document.createElement("DIV");
        tmp.innerHTML = html;
        return tmp.textContent || tmp.innerText || "";
    }

    function AttachListenerToMeetingSchedulerInsertLink(element) {
        if (element) {
            element.addEventListener('click', function() {

                chrome.extension.sendMessage({
                    MessageType: "GetMeetingSchedulerUrl"
                }, function (response) {

                });

                var meetingSchedulerPopup = this.closest('#sfEbstaMeetingSchedulerPopup');

                if (meetingSchedulerPopup !== null) {
                    meetingSchedulerPopup.style.display = 'none';
                }
            });
        }
    }

    function AttachListenerToMeetingSchedulerSetup(element) {
        if (element) {
            element.addEventListener('click', function () {

                if (webUserId) {

                    var redirectUrl = "https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=8";
                    window.open(redirectUrl, "_blank");
                }
            });
        }
    }

    function AttachClickThroughListenerToElement(element, uri) {
        if (element) {
            element.addEventListener('click', function () {

                window.open(uri, "_blank");
            });
        }
    }

    function GetOrCreateMeetingSchedulerPopup() {
        var popup = document.getElementById('sfEbstaMeetingSchedulerPopup');
        if (popup) {
            return popup;
        } else {
            popup = CreateElement("div", "sfEbSendCampaignMgrStarterPopup sfEbComposeMenu", "", "sfEbstaMeetingSchedulerPopup");

            // div for inserting the users calendar booking link eg https://booking.ebsta.com/eddbrownrigg.
            var divInsertLink = CreateElement("div", "sfEbComposeMenuOption ebsta-booking-link-insert", null, null);
            divInsertLink.appendChild(CreateElement("div", 'fa fa-hand-pointer-o  fa-lg ebsta-paddingright-ten', null, null));
            divInsertLink.appendChild(CreateElement("div", null, "Insert Calendar Link", null));
            popup.appendChild(divInsertLink);

            // event listener on click for select button
            AttachListenerToMeetingSchedulerInsertLink(divInsertLink);

            //div for link to the setup page in conse.
            var divSetup = CreateElement("div", "sfEbComposeMenuOption ebsta-booking-link-setup", null, null);
            divSetup.appendChild(CreateElement("div", 'fa fa-cog  fa-lg ebsta-paddingright-ten', null, null));
            divSetup.appendChild(CreateElement("div", null, "Setup", null));
            AttachListenerToMeetingSchedulerSetup(divSetup);
            popup.appendChild(divSetup);

            //div to click through to knowledge base article
            var divKnowledgeBase = CreateElement("div", "sfEbComposeMenuOption ebsta-booking-link-help", null, null);
            divKnowledgeBase.appendChild(CreateElement("div", 'fa fa-question-circle fa-lg ebsta-paddingright-ten', null, null));
            divKnowledgeBase.title = 'Check out our knowledgebase article';
            divKnowledgeBase.appendChild(CreateElement("div", null, "Help", null));
            AttachClickThroughListenerToElement(divKnowledgeBase, 'https://ebstalimited.zendesk.com/hc/en-us/articles/115002502828-Using-Meeting-Scheduler');
            popup.appendChild(divKnowledgeBase);

            document.body.appendChild(popup);

            HideElement(popup);

            return popup;
        }
    }

    var meetingSchedulerLinkObject = (function () {

        var currentCaret = null;

        return {
            SetCaret: function () {
                currentCaret = null;
                currentCaret = this.saveSelection();
            },
            saveSelection: function() {
                if (window.getSelection) {
                    var sel = window.getSelection();
                    if (sel.getRangeAt && sel.rangeCount) {
                        return sel.getRangeAt(0);
                    }
                } else if (document.selection && document.selection.createRange) {
                    return document.selection.createRange();
                }
                return null;
            },
            restoreSelection: function(range) {
                if (range) {
                    if (window.getSelection) {
                        var sel = window.getSelection();
                        sel.removeAllRanges();
                        sel.addRange(range);
                    } else if (document.selection && range.select) {
                        range.select();
                    }
                }
            },
            InsertMeetingSchedulerLinkIntoComposeWindow: function (composeWindow, link) {

                this.restoreSelection(currentCaret);
                composeWindow.focus();
                this.InsertHtmlAtCurserPosition(link);
            },
            // insert html at the posisiton of the curser
            // courtesy of http://stackoverflow.com/questions/6690752/insert-html-at-caret-in-a-contenteditable-div?noredirect=1&lq=1
            InsertHtmlAtCurserPosition: function(element) {
                var sel, range, html;
                if (window.getSelection) {
                    sel = window.getSelection();
                    if (sel.getRangeAt && sel.rangeCount) {
                        range = sel.getRangeAt(0);
                        range.deleteContents();

                        range.insertNode(element);
                        sel.removeAllRanges();
                        range = range.cloneRange();
                        range.selectNode(element);
                        range.collapse(false);
                        sel.addRange(range);
                    }
                } else if (document.selection && document.selection.createRange) {
                    range = document.selection.createRange();
                    range.pasteHTML(text);
                    range.select();
                }
            }
        }
    }());

    function SetUpComposeEvents() {

	$("body").undelegate(".sfEbstaMeetingSchedulerBtn", "click");
        $("body").delegate(".sfEbstaMeetingSchedulerBtn", "click", function() {

            // set all other present meeting scheduler buttons to be current-compose false
            $('.sfEbstaMeetingSchedulerBtn').attr("current-compose", "false");

            $(this).attr("current-compose", "true");

            SaveCaretPosition();

            var popup = GetOrCreateMeetingSchedulerPopup();

            GmailTools.Compose.InsertMenu(this, popup);

            ShowElement(popup);
        });

        $(".sfGetEmailTemplates").click(function (e) {
            e.stopImmediatePropagation();
            $('.sfGetEmailTemplates').attr("current-compose", "false");
            $(this).attr("current-compose", "true");
            $('#ebstaBox').remove();
            createCampaignMgrObject().RemoveCampaignManagerModal();
            if (abRules.ShowGmailTemplates + "" == "1")
            {
                AddEmailTemplatesBox(e);
            }
        });
    }

    //Updates the Gmail tracking button to show/not show when a user chooses plain text/html emails
    function UpdateGmailTrackingButtonForPlainTextEmails(composeScreenDivElement, hideGmailTrackingOption) {
        //Find the gmail tracking button first
        $(composeScreenDivElement).closest('.aSt, .AD, .adB').find('.sfEbstaGmailEmailTracking').each(function () {
            if (!hideGmailTrackingOption)
            {
                //Remove the hover function
                $(this).unbind('mouseenter mouseleave mouseover');
                //Hide the help text if it's showing
                //Amar: 03/02/2015 - This wont work any more - we only have one tooltip element now in the body and move this as needed to the button element on mouseenter
                //$(this).closest('.ebstaTD').find('span[data-tooltip-id="ebTrackingPlainText"]:visible').fadeOut(200);

                if ($(this).hasClass('sfEbstaGmailEmailTrackingDisabled'))
                {
                    $(this).removeClass('sfEbstaGmailEmailTrackingDisabled');
                    $(this).removeClass('sfEbstaGmailEmailTrackingSelected').addClass('sfEbstaGmailEmailTrackingNotSelected');
                }
            }
            else if (hideGmailTrackingOption)
            {
                if (!$(this).hasClass('sfEbstaGmailEmailTrackingDisabled'))
                {
                    //If if it is a plaintext compose - create a no tracking warning tooltip
                    $(this).addClass('sfEbstaGmailEmailTrackingDisabled');
                    $(this).removeClass('sfEbstaGmailEmailTrackingSelected').addClass('sfEbstaGmailEmailTrackingNotSelected');

                    SetupHoverForTrackingIconForPlainTextEmails();
                }

            }
        })
    }

    /**
     * Setup the tooltip for disabled email tracking button.
     * @param {} toolTipContainerClassName 
     * @param {} triggerClassName 
     * @returns {} 
     */
    function SetupEmailTrackingUpgradeTooltip(toolTipContainerClassName, triggerClassName)
    {
        var toolTipContent = "Email Tracking is available on our Growth plan. <br /><a href='https://www.ebsta.com/salesforce-gmail-integration/' target='_blank'>Upgrade Now</a>";

        SetupComposeIconToolTip(toolTipContainerClassName, triggerClassName, toolTipContent);
    }

    /**
     * Setup the tooltip for disabled send later button.
     * @param {} toolTipContainerClassName 
     * @param {} triggerClassName 
     * @returns {} 
     */
    function SetupSendLaterUpgradeTooltip(toolTipContainerClassName, triggerClassName) {
        var toolTipContent = "Send Later is available on our Growth plan. <br /><a href='https://www.ebsta.com/salesforce-gmail-integration/' target='_blank'>Upgrade Now</a>";

        SetupComposeIconToolTip(toolTipContainerClassName, triggerClassName, toolTipContent);
    }

    /**
     * Setup the tooltip for disabled campaign manager button.
     * @param {} toolTipContainerClassName 
     * @param {} triggerClassName 
     * @returns {} 
     */
    function SetupCampaignManagerUpgradeTooltip(toolTipContainerClassName, triggerClassName) {
        var toolTipContent = "Cadences are available on our Growth plan. <br /><a href='https://www.ebsta.com/salesforce-gmail-integration/' target='_blank'>Upgrade Now</a>";

        SetupComposeIconToolTip(toolTipContainerClassName, triggerClassName, toolTipContent);
    }

    /**
     * Setup the tooltip for disabled meeting scheduler button.
     * @param {} toolTipContainerClassName 
     * @param {} triggerClassName 
     * @returns {} 
     */
    function SetupMeetingSchedulerUpgradeTooltip(toolTipContainerClassName, triggerClassName) {
        var toolTipContent = "Calendar Booking is available on our Growth plan. <br /><a href='https://www.ebsta.com/salesforce-gmail-integration/' target='_blank'>Upgrade Now</a>";

        SetupComposeIconToolTip(toolTipContainerClassName, triggerClassName, toolTipContent);
    }

    /**
     * Setup a new tooltip for a compose icon element.
     * @param {} toolTipContainerClassName 
     * @param {} triggerClassName 
     * @param {} toolTipContent 
     * @returns {} 
     */
    function SetupComposeIconToolTip(toolTipContainerClassName, triggerClassName, toolTipContent)
    {
        var toolTipTrackingDiv;

        //If we dont have a plaintext warning tooltip - create one
        if ($('.' + toolTipContainerClassName).length < 1) {            
            //Send to function to create tooltip element.
            toolTipTrackingDiv = CreateWizardTooltipHover(toolTipContent, 'ebTrackingPlainText', 'toolTipLowerArrow', '', "" + toolTipContainerClassName);
            //Initially hide the warning element
            toolTipTrackingDiv.style.display = "none";

            //attach it to body - we will position it on mouseenters.
            document.body.appendChild(toolTipTrackingDiv);

            //Create click event for the close button
            $('.' + toolTipContainerClassName + ' .toolTipCloseHover').on("click", function () {
                $(this).closest('.' + toolTipContainerClassName).hide();
            });

        } else {
            //If we find a tracking plain text tooltip - store the element in local var
            toolTipTrackingDiv = $('.' + toolTipContainerClassName)[0];
        }

        //If we havent found a tracking tooltip element, but havent created one either - something is wrong. Dont bind events - get out.
        if (typeof toolTipTrackingDiv == 'undefined' || toolTipTrackingDiv == null) {
            return;
        }

        //Remove the binding from disabled tracking button for the current tooltipd id + create tooltip 
        $("." + triggerClassName).off("mouseover");
        $("." + triggerClassName).on("mouseover", function (event) {
            //Get the email tracking button that was hovered / mouseover

            //If this is a email tracking disabled button
            if ($(this)[0].className.indexOf("" + triggerClassName) != -1) {

                //Get the position of the current hovered / mouseover button
                var parentPos = $(this).offset();

                // positions the tooltip
                $('.' + toolTipContainerClassName).css({
                    top: parentPos.top - 95,
                    left: parentPos.left - 75
                }).show();

            }
        });

        //Remove the binding from disabled tracking buttons - then rebind to all the ones that currently exsist
        $("." + triggerClassName).off("mouseleave");
        $("." + triggerClassName).on("mouseleave", function (evt) {
            var elementClassName = "";

            //Check the class of the element where the mouseleave occured.
            if (evt.toElement != null && typeof evt.toElement != 'undefined') {
                if (evt.toElement.className != null && typeof evt.toElement.className != 'undefined') {
                    elementClassName = evt.toElement.className;
                }
            }

            //if the element is not the tooltip container or the tooltip arrow / any part of the tooltip - hide the tooltipcontainer
            if (elementClassName.indexOf("" + toolTipContainerClassName) == -1 && elementClassName.indexOf("ebWizardTooltip") == -1) {
                $('.' + toolTipContainerClassName).hide();
            }
        });

        //Remove the binding from disabled tracking buttons - then rebind to all the ones that currently exsist
        $("." + toolTipContainerClassName).off("mouseleave");
        $("." + toolTipContainerClassName).on("mouseleave", function (evt) {
            var elementClassName = "";

            //Check the class of the element where the mouseleave occured.
            if (evt.toElement != null && typeof evt.toElement != 'undefined') {
                if (evt.toElement.className != null && typeof evt.toElement.className != 'undefined') {
                    elementClassName = evt.toElement.className;
                }
            }

            //If this element is not the inner tickbox or the disabled tracking button - hide to tooltip container
            if (elementClassName.indexOf("ebsta-feature-icon-disabled") == -1) {
                $('.' + toolTipContainerClassName).hide();
            }
        });
    }

    //Displays the plain text email message when hovering over
    function SetupHoverForTrackingIconForPlainTextEmails() {

        var toolTipTrackingDiv;

        //If we dont have a plaintext warning tooltip - create one
        if ($('.toolPlainTextTipContainer').length < 1) {
            //Setting content
            var plainTextNotifyTxt = "You cannot track plain text emails. <a href='https://ebsta.zendesk.com/entries/75733259-Does-Ebsta-track-plain-text-emails-' target='_blank'>Learn more.</a>";
            //Send to function to create tooltip element.
            toolTipTrackingDiv = CreateWizardTooltipHover(plainTextNotifyTxt, 'ebTrackingPlainText', 'toolTipLowerArrow', '', "toolPlainTextTipContainer");
            //Initially hide the warning element
            toolTipTrackingDiv.style.display = "none";

            //attach it to body - we will position it on mouseenters.
            document.body.appendChild(toolTipTrackingDiv);

            //Create click event for the close button
            $('.toolPlainTextTipContainer .toolTipCloseHover').on("click", function () {
                $(this).closest('.toolPlainTextTipContainer').hide();
            });

        } else {
            //If we find a tracking plain text tooltip - store the element in local var
            toolTipTrackingDiv = $('.toolPlainTextTipContainer')[0];
        }

        //If we havent found a tracking tooltip element, but havent created one either - something is wrong. Dont bind events - get out.
        if (typeof toolTipTrackingDiv == 'undefined' || toolTipTrackingDiv == null) {
            return;
        }

        //Remove the binding from disabled tracking button for the current tooltipd id + create tooltip 
        $(".sfEbstaGmailEmailTrackingDisabled").off("mouseover");
        $(".sfEbstaGmailEmailTrackingDisabled").on("mouseover", function (event) {
            //Get the email tracking button that was hovered / mouseover

            //If this is a email tracking disabled button
            if ($(this)[0].className.indexOf("sfEbstaGmailEmailTrackingDisabled") != -1) {
                
                //Get the position of the current hovered / mouseover button
                var parentPos = $(this).offset();

                // positions the tooltip
                $('.toolPlainTextTipContainer').css({
                    top: parentPos.top - 85,
                    left: parentPos.left - 75
                }).show();
                
            }
        });

        //Remove the binding from disabled tracking buttons - then rebind to all the ones that currently exsist
        $(".sfEbstaGmailEmailTrackingDisabled").off("mouseleave");
        $(".sfEbstaGmailEmailTrackingDisabled").on("mouseleave", function (evt) {
            var elementClassName = "";

            //Check the class of the element where the mouseleave occured.
            if (evt.toElement != null && typeof evt.toElement != 'undefined') {
                if (evt.toElement.className != null && typeof evt.toElement.className != 'undefined') {
                    elementClassName = evt.toElement.className;
                }
            }

            //if the element is not the tooltip container or the tooltip arrow / any part of the tooltip - hide the tooltipcontainer
            if (elementClassName.indexOf("toolPlainTextTipContainer") == -1 && elementClassName.indexOf("ebWizardTooltip") == -1) {
                $('.toolPlainTextTipContainer').hide();
            }
        });

        //Remove the binding from disabled tracking buttons - then rebind to all the ones that currently exsist
        $(".toolPlainTextTipContainer").off("mouseleave");
        $(".toolPlainTextTipContainer").on("mouseleave", function (evt) {
            var elementClassName = "";

            //Check the class of the element where the mouseleave occured.
            if (evt.toElement != null && typeof evt.toElement != 'undefined') {
                if (evt.toElement.className != null && typeof evt.toElement.className != 'undefined') {
                    elementClassName = evt.toElement.className;
                }
            }

            //If this element is not the inner tickbox or the disabled tracking button - hide to tooltip container
            if (elementClassName.indexOf("sfEbstaGmailEmailTrackingDisabled ") == -1) {
                $('.toolPlainTextTipContainer').hide();
            }
        });

    }

 //This is called from CheckLicence once the message has been received
//Additionally passing through an intervalidentifier so we can clear down our interval timer
    function InsertGoogleSendButtons(intervalIdentifier) {
        if (intervalIdentifier === undefined) {
            intervalIdentifier = 0; // default value
        }
        //Only add buttons if this user is logged in
        if (LoggedIn) {
           // console.time("tdButtonAdding");
            $('td[class*="gU Up"]').each(function () {
		        var ebstaComposeButtonsClosure = EbstaComposeButtons();
                ebstaComposeButtonsClosure.Initialize($(this)[0]);
            });
           // console.timeEnd("tdButtonAdding");

            $(document).ready(function () {
                $(".eq").each(function () {

                    $(this).attr("style", "left:340px");

                });

                $(".oG").each(function () {

                    $(this).attr("style", "float:none");

                });

                $(".aWR").each(function () {

                    $(this).attr("style", "float:none");

                });
            });

            SetUpComposeEvents();
        }

        //LogThis("Resize Gmail 5");
        SetupResizeWindowEventsForGmailCompose();
        //console.timeEnd('InsertGoogleSendButtons');
    }
    
 	//We first throw a request to check the user is logged in and this will call InsertGoogleSendButtons if the user is verfied and logged in
//Additionally passing through an intervalidentifier so we can clear down our interval timer
    function AddGoogleSendButtons(intervalIdentifier) {
        if (intervalIdentifier === undefined) {
            intervalIdentifier = 0; // default value
        }
        InsertGoogleSendButtons(intervalIdentifier);
    }
	
	//Takes a div and looks to see if this is a plain text email
	function IsThisAPlainTextEmail(composeDiv) {
        var composeDiv = $(composeDiv).closest('.I5').find('input[type="hidden"][name="ishtml"]');
        if (composeDiv.length)
        {
            if (composeDiv.val() == 0) {
                return true;
            }
            else
            {
                return false;
            }
        }
    }

    function AddGoogleEmailTemplatesButton() {

        var td = Createtd("gU ebstaTD");
        
        var spn = CreateSpan("", "");
        spn.className = "sfGetEmailTemplates";
        spn.placeholder = "Click here to use salesforce email templates";
        
        var imgSpn = CreateSpan("", "");
        imgSpn.className = "sfEbstaGmailEmailTemplatesBtn sfEbstaGmailEmailTemplatesNotSelected";
        imgSpn.title = "Salesforce Email Templates";

        if (buttonMode === 1) {
            spn.classList.add("sfGetEmailTemplates-style1");
            imgSpn.classList.add("sfEbstaGmailEmailTemplatesBtn-style1");
        }

        spn.appendChild(imgSpn);

        td.appendChild(spn);

        return td;
    }
    
/**
 * Fires a request to check if the user already has a Calendar Sync running
 * @returns {} 
 */
function DisplayCalendarSyncIcon() {

    if (!calendarSyncLoaded && LoggedIn) {
        calendarSyncLoaded = true;
        chrome.extension.sendMessage({ MessageType: "CheckCalendarSyncSetup" }, function (response) { });
    }
}

  //Logs a users Gmail address in the database for the option to turn Ebsta off for individual accounts
    function LogUsersGmailAddress() {
   
        setTimeout(function () {

            var emailAddress = "";
            emailAddress = GetGmailUsersEmailAddress();
            if (emailAddress.indexOf("@") > -1) {
                if (chrome.extension.sendMessage({
                    MessageType: "LogUsersGmailAddress",
                    EmailAddress: emailAddress
                }, function (response) {

                }));
            }
        }, 5000);
    }

    window.addEventListener("DOMContentLoaded", function () {
        //On page load fire our checks off to get certain values
        Setup();
        
        if (document.URL.toLowerCase().indexOf("mail.google.com/mail") != -1 && document.URL.toLowerCase().indexOf("accounts.google.com") == -1) {

            //Look for elements to determine if a user is using gmail for business or personl gmail
            var gmailType = "LogGmailType-Personal";

            setTimeout(function () {
                //Find if there are any advert sections
                if (document.getElementsByClassName("ae6").length == 0) {
                    gmailType = "LogGmailType-Business";
                }

                //Log which type of Gmail it is
                chrome.extension.sendMessage({
                    MessageType: "LogGmailType",
                    LogType: gmailType
                }, function (response) {

                });
                
            }, 3000);

            chrome.extension.sendMessage({
                MessageType: "LogGmailView"
            }, function (response) {

            });
            
        }

        if (document.URL.toLowerCase().indexOf("mail.google.com/mail") != -1 || document.URL.toLowerCase().indexOf(".google.com/calendar") != -1
            || document.URL.toLowerCase().indexOf("contacts.google.com") != -1 || document.URL.toLowerCase().indexOf(".google.com/contacts") != -1) {
            
            
            //Setup certain things for Google Calendar
            if (document.URL.toLowerCase().indexOf(".google.com/calendar") != -1) {
                if (GoogleCalendarTools.IsThisNewGoogleCalendarLayout()) {
                    ChangeOnboardingButtonOnResize();
                    SetupWindowResizeListenerForGoogleCalendar();
                } else {
                    GoogleCalendarTools.SetupEvents();
                }
            }

            
        }

        // We should test for Gmail.
        //NewEmailEventSetup();
        PostGmailAuth();
        GetAccessDetails();
        PostGmailFullAuth();

        if (document.URL.indexOf("talentsearch.seek.com.au") != -1) {
            SetupSeekEvents();
        }

        listenForPageEvents();

        //console.timeEnd("DomComplete");
    }, false);

    chrome.extension.onMessage.addListener(
        function (request, sender, sendResponse) {
            if (request.MessageType == "RestyleBar") {
                var ebstaBar = document.getElementById("ebsta-frame");
                ebstaBar.style.zIndex = "" + request.ZIndexValue;

                var currentUrl = '';

                if(document.URL != null && typeof document.URL != "undefined")
                {
                    currentUrl = document.URL;
                }else{
                    currentUrl = request.Url;
                }

                var newZIndexValue = "" + request.ZIndexValue;

                if (newZIndexValue == "0") {
                    ebstaBar.style.height = "50px";
                } else {
                    ebstaBar.style.height = "600px";
                }

                if (currentUrl.indexOf("mail.google.com") === -1) {
                    ebstaBar.style.zIndex = "2147483647"; //"100001";
                } else {
                    ebstaBar.style.zIndex = EBSTA_GMAIL_BAR_ZINDEX;
                }
            }            

            if (request.MessageType == "InjectCalendarSyncStatus") {

                if (request.CalendarSyncObject != null) {

                    var calendarObject = request.CalendarSyncObject;

                    var json = JSON.parse(calendarObject);
                    var result = json.d;

                    var errorMessage = result.ErrorMessage;

                    if (errorMessage == "NOSYNCFOUND") {
                        CreateOnboardRestartBtnGoogleCalendar();
                        ChangeOnboardingButtonOnResize();
                    }
                }
            }

            if (request.MessageType == "UpdateDefaultBarSearch") {
                if (request.Value != null) {
                    useDefaultBarSearch = request.Value;
                }
            }
            
            if (request.MessageType == "DisplayGmailMessage") {

                var msg = document.getElementById("sfgmailId");
                var count = document.getElementsByClassName("sfBoxMsg");
                var message = request.MessageContent;
                var MsgCss = request.MsgCss;
                //Remove adding email to SF message
                if (msg != null) {
                    if (msg.textContent.indexOf("Adding") >= 0) {
                        $(msg).remove();
                        msg = null;
                    }
                }

                if (msg == null) {

                    msg = CreateDivWithId("", "sfBoxMsg" + MsgCss, "sfgmailId");
                    msg.style.visibility = "visible";

                    //If the error when adding a task was that they tried to add to a converted lead then let them know this
                    if (message.indexOf("cannot reference converted lead") != -1)
                    {
                        //Clear cache so we don't keep showing the record as a Lead for the next time they try
                        chrome.extension.sendMessage({
                            MessageType: "ClearLocalStorageEmailSearchResults"
                        }, function (response) {
                        });

                        var textNode = document.createTextNode("Error Adding: Lead has been Converted, cannot add to a Converted Lead");
                        var textNode2 = document.createTextNode("Please refresh the page and try again or use the Complex Add Feature");
                        box.appendChild(textNode);
                        box.appendChild(CreateBr());
                        box.appendChild(CreateBr());
                        box.appendChild(textNode2);

                        //Remove Lead attributes from Complex Add box
                        $('.sfComplexAdd').each(function () {
                            if ($(this).attr('sftype') == 'Lead') {
                                $(this).attr('sftype',"");
                                $(this).attr('sfemailaddress',"");
                            }
                        });
                    }
                    else
                    {
                        msg.innerText = message;
                    }
                    (document.body || document.documentElement).appendChild(msg);

                } else {
                    var content = msg.textContent;
                    if (content.indexOf("other") == -1)
                    {
                        msg.textContent = content + " and 1 other.";
                    }
                    else
                    {
                        var numberTxt = content.substring(content.indexOf("and") + 4, content.indexOf("and") + 6);
                        var numberInt = parseInt(numberTxt);
                        numberInt++;
                        msg.textContent = content.substring(0, content.indexOf("and")) + " and " + numberInt + " others.";
                    }
                    //var height = (count.length+1) * 5 - 1; 

                    //var box = CreateDivWithId("", "sfBoxMsg" + MsgCss, "sfgmailId"+height);
                    //box.style.visibility = "visible";
                    //box.innerText = message;
                    
                    //box.style.top = height + "%";
                    
                    //(document.body || document.documentElement).appendChild(box);
                    //$(box).delay(9000).fadeOut(1000, function () { $(this).remove(); });
                }

                //Find where the Google 'Send Message Notifcation' appears and locate it underneath that
                $('.b8:visible').closest('.nH').each(function () {
                    var additionalOffset = 10;

                    //If the google message is evident  then increase the offset then increase the additionoffset so it's not overlapping the message in anyway
                    $(this).find('.vh').each(function () {
                        if ($(this).text()) {
                            additionalOffset += $(this).outerHeight();
                        }
                    })

                    $(msg).css('top', $(this).offset().top + additionalOffset);
                });

                // Remove message after certain period
                setTimeout(function() {
                    var msg1 = document.getElementById("sfgmailId");
                    $(msg1).fadeOut();
                    $(msg1).fadeOut(function () { $(this).remove(); });
                }, 3000);


            }

            //Hide all Ebsta gmail elements and refresh local variable settings
            if (request.MessageType === "HideGmailButtons") {
                if (document.URL.toLowerCase().indexOf("mail.google.com/mail") != -1 || document.URL.toLowerCase().indexOf("google.calendar.com") != -1
                    || document.URL.toLowerCase().indexOf(".google.com/contacts") != -1 || document.URL.toLowerCase().indexOf("contacts.google.com") != -1) {

                    GmailTools.RemoveAllEbstaButtons();
                    GoogleTools.Header.RemoveButtons();
                }

                if (request.LogUserOut === true) {
                    LoggedIn = false;
                }
            }

            if (request.MessageType === "SendDetailForParsingBack" && request.CustomGmailRules) {
                sendResponse(SendDetailForParsingBack(request.CustomGmailRules));
            }
        });


    function RenderTemplateFoldersResult(entityType, resultObject, lastUsedEmailFolder) {
       
        var header1 = document.createElement('div');
        header1.setAttribute("class", "sfEmailTemplateFolderHeader");
        header1.appendChild(document.createTextNode('Folder'));

        $("#emailTemplateBox .sfRowContent").append(header1);

        //loop through folders.
        if (entityType == "gettemplatefolders") {
            if (resultObject != null) {
                if (resultObject.d.Values != null) { 
                    if (resultObject.d.Values.length > 0) { 

                        var dataArray = [];

                        //for selecting last folder.
                        var valueExists = false;

                        var lastFolderName = "";

                        // LOOPS THROUGH THE OBJECT 
                        for (var i = 0; i < resultObject.d.Values.length; i++) {
                            var jsonString = resultObject.d.Values[i];
                         
                            var currValueObject = JSON.parse(jsonString);
                          
                            // checks if the object name or id is null
                            if (currValueObject.Name != null && currValueObject.Id != null) {

                                currValueObject.text = currValueObject.Name;
                               
                                currValueObject.value = currValueObject.Id;

                                if (lastUsedEmailFolder != null && typeof lastUsedEmailFolder != 'undefined' && lastUsedEmailFolder != 'undefined') {
                                    if (currValueObject.Id == lastUsedEmailFolder) {
                                        valueExists = true;
                                        lastFolderName = currValueObject.Name;
                                    }
                                }

                                dataArray.push(currValueObject);
                            }
                        }

                        // creates the html dropdown menu
                        var dropDown = EbstaCustomDropdown.CreateCustomDropDown("newdropdown", dataArray[0], dataArray, RunTemplateFolderChangeEvent);
                                           
                        // append the new dropdown
                        $("#emailTemplateBox .sfRowContent").append(dropDown);


                        if (lastUsedEmailFolder.length > 0 && valueExists) {
                            //set the value.
                            $('#newdropdown').attr("selectedvalue", lastUsedEmailFolder);
                            //set the text.
                            $("#newdropdown .ebsta-selectedtext").text(lastFolderName);

                        }

                    } else { 
                        folderSelectList = CreateDiv("No folders available..", "sfFolderSelectList");
                        $("#emailTemplateBox .sfRowContent").append(folderSelectList);
                        //No request for templates will be fired off, so remove the loader
                        $('.sfEmailTemplateOptions .resultsLoader').remove();
                    }
                } else { 
                    folderSelectList = CreateDiv("No folders available..", "sfFolderSelectList");
                    $("#emailTemplateBox .sfRowContent").append(folderSelectList);
                    //No request for templates will be fired off, so remove the loader
                    $('.sfEmailTemplateOptions .resultsLoader').remove();
                }
            } else { 
                folderSelectList = CreateDiv("No folders available..", "sfFolderSelectList");
                $("#emailTemplateBox .sfRowContent").append(folderSelectList);
                //No request for templates will be fired off, so remove the loader
                $('.sfEmailTemplateOptions .resultsLoader').remove();
            }

        }

        // to load the selected value in the dropdown menu
        RunTemplateFolderChangeEvent();
}

    function RunTemplateFolderChangeEvent()
    {
        var selectValue = $('#newdropdown').attr("selectedvalue");

        chrome.extension.sendMessage({
            MessageType: "GetTemplatesForFolderById",
            FolderId: selectValue
        });

        // Get templates loader
        var loadingResults = CreateDiv("Loading templates...", "sfLoadingResults");

        loadingResults.className = 'resultsLoader';
        loadingResults.setAttribute("style", "display:inline-block;margin-left:10px;margin-top:15px;font-size:14px");
        var loadingGif = CreateImg(chrome.extension.getURL("/images/ajax-loader.gif"), "Loading templates...");
        loadingResults.appendChild(loadingGif);

        var templateList = document.getElementById('sfTemplatesList');
        templateList.innerHTML = ''; //Empty the list
        templateList.appendChild(loadingResults);
    }


    //Shows a new dialog box where the user can select a associated contact/lead with the selected email template
    function LoadContactOrLeadSuggestionsBoxForEmailTemplates(templateId)
    {
        var boxDiv = CreateDiv("", "sfEmailTemplateSuggestions");

        var divSuggestRow = CreateRow("sfEmailTemplatePeopleSuggestions");
        divSuggestRow.setAttribute("id", "sfRowComplete");

        var header1 = document.createElement('h3');
        header1.setAttribute("class", "sfEmailTemplateSuggestionsHeader");
        header1.appendChild(document.createTextNode('Choose a Contact or Lead'));
        divSuggestRow.appendChild(header1);

        //Create Search Div
        divSuggestRow.appendChild(CreateSearchBoxForEmailTemplateSearch());

        var header1 = document.createElement('h3');
        header1.setAttribute("class", "sfPeopleSuggestionsHeader");
        header1.appendChild(document.createTextNode('People Suggestions'));
        divSuggestRow.appendChild(header1);

        var divSuggestRowLeft = CreateDivWithId("", "sfEmailTemplateRowWhoSuggestLeft", "");
        divSuggestRow.appendChild(divSuggestRowLeft);

        boxDiv.appendChild(divSuggestRow);

        //Prepend the Div
        $('#emailTemplateBox').prepend(boxDiv);
        //(document.body || document.documentElement).appendChild(boxDiv);

        $('.sfEmailTemplateSuggestions .sfBtCancel, .sfEmailTemplateSuggestions .sftitle-close').click(function () {
            $('.sfEmailTemplateSuggestions').remove();
        });

        GetContactLeadSuggestionsForEmailTemplates(templateId);
    }

    // Create the Search Box
    function CreateSearchBoxForEmailTemplateSearch()
    {
        var divrow2 = CreateRow("sfEmailTemplateSearch");
        AddListenerForComplexAddRow(divrow2, true);
        //Who Search Box
        var customInputDiv = CreateDiv("", "sfWhoSearch CAObjectContainer");
        customInputDiv.id = "EbstaTxtSearchWhoIdTemplates";

        var whoIdElem = CreateTxtbox("", "sfSearchWhoTextBox CATextInput");
        whoIdElem.setAttribute("placeholder", "Search Name");
        TriggerEventOnEnter(whoIdElem);

        var searchButton = CreateSpan("", "sfWhoSearchBtn magnifySearchIcon", "");
        customInputDiv.appendChild(whoIdElem);
        customInputDiv.appendChild(searchButton);
        AddListenerForWhoSearchButton(searchButton);

        var SALeftSegmentContainer = document.createElement('div');
        SALeftSegmentContainer.className = 'SALeftSegment CASegment';
        SALeftSegmentContainer.appendChild(customInputDiv);

        //What Search Box
        var customInputDiv2 = CreateDiv("", "sfWhatSearch CAObjectContainer");
        customInputDiv2.id = "EbstaTxtSearchWhatIdTemplates";

        var whatIdElem = CreateTxtbox("", "sfSearchWhatTextBox CATextInput", "sfWhatId");
        whatIdElem.setAttribute("placeholder", "Search Related To");
        TriggerEventOnEnter(whatIdElem);
        var searchButton = CreateSpan("", "sfWhatSearchBtn magnifySearchIcon", "");

        AddListenerForWhatSearchButton(searchButton);

        var chooseObjectsToSearchButton = CreateObjectSearchMenu();
        chooseObjectsToSearchButton.id = "EbstaBtFilterObjectsTemplates";

        customInputDiv2.appendChild(searchButton);
        customInputDiv2.appendChild(chooseObjectsToSearchButton);
        customInputDiv2.appendChild(whatIdElem);
        customInputDiv2.appendChild(CreateBr());

        var SARightSegmentContainer = document.createElement('div');
        SARightSegmentContainer.className = 'SARightSegment CASegment';
        SARightSegmentContainer.appendChild(customInputDiv2);

        divrow2.appendChild(SALeftSegmentContainer);
        divrow2.appendChild(SARightSegmentContainer);

       // console.time("CheckIfWhoAndWhatFieldsAreEnabled");
        CheckIfWhoAndWhatFieldsAreEnabled(divrow2);
       // console.timeEnd("CheckIfWhoAndWhatFieldsAreEnabled");

        GetComplexAddSettings();

        return divrow2;
    }

    function GetContactLeadSuggestionsForEmailTemplates(templateId)
    {
        //Checks if the user actually has access to the whoid field of a task object before getting suggestions
        if (whoWhatEnabledObject != null) {
            if (!whoWhatEnabledObject.whoEnabled) {
                return false;
            }
        }

        //Put some loader of some type here

        chrome.extension.sendMessage({
            MessageType: "GetLocalStorageEmailSearchResult"
        }, function (response) {
            var json = null;

            if (response && IsValidJSON(response.json)) {
                json = JSON.parse(response.json);
            }

            //Get all recipients to search for
            var listOfEmails = $(".sfEmailTemplateOptions").attr("emailaddresses");
            var emailSuggestTemplateDiv = null;

            if ($('.sfEmailTemplateRowWhoSuggestLeft').length)
            {
                emailSuggestTemplateDiv = $('.sfEmailTemplateRowWhoSuggestLeft');
                $(emailSuggestTemplateDiv).hide();
            }

            var emailRecipients = new Array();
            emailRecipients = listOfEmails.split(",");

            var newEmailSearchArray = new Array();

            var isThisTheFirstFoundRecord = true;

            var countOfSuggestionsFound = 0;

            if (json && json.d && json.d.Values) {
                for (var i = 0; i < emailRecipients.length; i++) {
                    if (emailRecipients[i].length === 0) break;
                    var emailaddress = emailRecipients[i];
                    var boolFoundEmailInLocalStorage = false;

                    for (var j = 0; j < json.d.Values.length; j++) {

                        var parsedJson = JSON.parse(json.d.Values[j]);

                        var emailFromJson = "";
                        var recordType = "";
                        var name = "";
                        var whoId = "";

                        if (parsedJson["Email-0"] != null) {
                            emailFromJson = parsedJson["Email-0"];
                        }

                        if (parsedJson.Type) {
                            recordType = parsedJson.Type;
                        }

                        if (parsedJson.Name) {
                            name = parsedJson.Name;
                        }

                        if (parsedJson.Id) {
                            whoId = parsedJson.Id;
                        }

                        if (recordType != null && typeof recordType != "undefined") {
                            if (recordType.toLowerCase() == 'contact' || recordType.toLowerCase() == 'lead') { //process people only!!!

                                //Email addr. from list can be mixed case: if list is empty we will have empty str
                                if (emailaddress.toLowerCase() == emailFromJson.toLowerCase()) {
                                    boolFoundEmailInLocalStorage = true;
                                    //If this isn't the first record append to list of suggestions
                                    if (!isThisTheFirstFoundRecord) {

                                        countOfSuggestionsFound++;

                                        var emailDiv = CreateDiv("", "sfEmailSuggest CABoxStyleContainer");

                                        var crossIcon = CreateAddIcon();
                                        $(crossIcon).removeClass("AddSuggestionIcon");
                                        $(crossIcon).addClass("AddSuggestionIconWhite");

                                        var boxTextLabelContent = CheckStringLength(name + " (" + emailaddress + ")", 37);

                                        var style = "sfSuggest" + recordType + " CABoxStyle";

                                        var emailSpan = CreateComplexAddObjectDiv(boxTextLabelContent, recordType, style, crossIcon);

                                        emailDiv.setAttribute("WhoId", whoId);
                                        emailDiv.setAttribute("recordType", recordType);
                                        emailDiv.setAttribute("sfEmailAdd", emailaddress);
                                        emailDiv.setAttribute("sfName", name);
                                        emailDiv.appendChild(emailSpan);

                                        AddEventListenerToComplexAddWhoObject(emailDiv);

                                        if (emailSuggestTemplateDiv) {
                                            emailSuggestTemplateDiv.append(emailDiv);
                                        }
                                    }
                                    //if this is the first record found then put this straight into the search box
                                    else if (isThisTheFirstFoundRecord) {
                                        isThisTheFirstFoundRecord = false;

                                        var crossIcon = CreateCrossIcon();
                                        AddListenerToWhoButtonToRemove(crossIcon);

                                        var boxTextLabelContent = CheckStringLength(name + " (" + emailaddress + ")", 37);

                                        var whoIdButton = CreateComplexAddObjectDiv(boxTextLabelContent, recordType, "CABoxStyle sfInitSuggestion sfSuggest" + recordType.charAt(0).toUpperCase() + recordType.substring(1, recordType.length), crossIcon);
                                        whoIdButton.setAttribute("sfWhoId", whoId);
                                        whoIdButton.setAttribute("recordType", recordType);
                                        whoIdButton.setAttribute("recordName", name);
                                        whoIdButton.setAttribute("email", emailaddress);

                                        $('.sfEmailTemplateSearch').find('.sfSearchWhoTextBox').remove();
                                        $('.sfEmailTemplateSearch').find('.sfWhoSearch').find('.CABoxStyle,.magnifySearchIcon').remove();

                                        $('.sfEmailTemplateSearch').find('.sfWhoSearch').append(whoIdButton);
                                    }
                                }
                            }
                        }
                    }

                    if (!boolFoundEmailInLocalStorage) {
                        newEmailSearchArray.push(emailaddress);
                    }
                }
            } else {
                newEmailSearchArray = emailRecipients;
            }

            //If No Results found in localstorage lets do a live check to make sure they don't actually exists
            if (newEmailSearchArray.length > 0) {

                //Lets now do a live check
                var searchTerm = "";

                for (var i = 0; i < newEmailSearchArray.length; i++) {
                    searchTerm += newEmailSearchArray[i] + " OR ";
                }

                searchTerm = searchTerm.substring(0, searchTerm.length - 4).trim();

                var loadingResults = CreateDiv("Loading other potential matches... ", "sfLoadingResults");
                loadingResults.className = 'resultsLoader';
                loadingResults.setAttribute("style", "display:inline-block;margin-left:10px;margin-top:0px;");
                var loadingGif = CreateImg(chrome.extension.getURL("/images/ajax-loader.gif"), "Loading other potential matches...");

                loadingResults.appendChild(loadingGif);

                //Run Search
                chrome.extension.sendMessage({
                    MessageType: "SearchComplexAdd-Who",
                    SearchTerm: searchTerm
                }, function (response) {
                    DisplaySearchResults("", response, searchTerm)
                });

                $('#emailTemplateBox > .sfEmailTemplateSuggestions > .sfEmailTemplatePeopleSuggestions > .sfPeopleSuggestionsHeader').append(loadingResults);
            } else {
                if (countOfSuggestionsFound == 0) $('#emailTemplateBox').find(".sfPeopleSuggestionsHeader").css("display", "none");
                $(emailSuggestTemplateDiv).show();
            }
        });

    }

    function RenderMeetingSchedulerLink(entityType, resultObject) {


        var calendarUrl = "";

        if (resultObject != null && typeof resultObject != "undefined") {
            var values = resultObject.d.Values;

            if(values.length > 0) {
                calendarUrl = values[0];
            }
        }

        if (calendarUrl.length > 0) {


            var fullUrl = "Schedule a meeting with me here: https://booking.ebsta.com/" + calendarUrl;

            var currentCompose = $("*[current-compose*='true'").closest('.iN').find('.Am.Al');


            var textNode = document.createTextNode(fullUrl);

            var ebstaTemplatePre = document.createElement("span");
            ebstaTemplatePre.className = "ebstaTemplatePre";
            $(ebstaTemplatePre).css("font-family", "inherit");
            $(ebstaTemplatePre).css("font-size", "inherit");
            //gmail allready adds a spacing at the botton - dont double up.
            $(ebstaTemplatePre).css("margin", "0px");

            ebstaTemplatePre.appendChild(textNode);

            meetingSchedulerLinkObject.InsertMeetingSchedulerLinkIntoComposeWindow(currentCompose, ebstaTemplatePre);
        } else {


            var redirectUrl = "https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=8";
            window.open(redirectUrl, "_blank");
        }
    }

    function EmailTemplateSelectResult(entityType, resultObject) {
        if (resultObject.d.Values[0] != null) {
            if (resultObject.d.Values[0].length > 0) {
                var jsonString = resultObject.d.Values[0];
                var currValueObject = JSON.parse(jsonString);

                var ebstaTemplateObj = document.createElement("DIV");
                ebstaTemplateObj.id = "ebstaTemplateObject";
                ebstaTemplateObj.className = "ebstaTemplateObject";

                //Put template ID into an attribute
                if (currValueObject.Id) {
                    ebstaTemplateObj.className += " ebstatemplateid-" + currValueObject.Id;
                    $(ebstaTemplateObj).attr('ebstatemplateid', currValueObject.Id);
                }

                var ebstaTemplatePre = document.createElement("PRE");
                ebstaTemplatePre.className = "ebstaTemplatePre";
                //Gmail has there own styling for elements within <pre> tags, these inline styles help ignore those
                ebstaTemplatePre.style = "font-family: inherit; font-size: inherit; margin: 0px";

                var currentCompose = $("*[current-compose*='true'").closest('.iN').find('.Am.Al');
                var currentComposeCopy = currentCompose;

                // Catch if a user has selected a font from the settings page as this creates a new div 
                // called .gmail_default and has font settings inlined styled which we need to use for when we append the template text
                if ($("*[current-compose*='true'").closest('.iN').find('.Am.Al > .gmail_default').length > 0) {
                    currentCompose = $("*[current-compose*='true'").closest('.iN').find('.Am.Al > .gmail_default');
                }

                var currentSubject = null;
                $("*[current-compose*='true'").closest('.aaZ, .I5').find(".aoD.az6 input[name='subjectbox']").each(function(){
                    if (currentSubject == null) {
                        currentSubject = $(this);
                        return false;
                    }
                });

                //console.time("upsertEbstaTemplateObj");
                //Remove any one ebsta template that may exist - we will replace with new.
                if (currentCompose.length > 0) {
                    var rem = currentCompose[0].querySelectorAll("[id*='ebstaTemplateObject']");
                    for (var i = 0; i < rem.length; i++) rem[i].remove();
                }

                //Check if we have the additional space before signature.
                var currentComposeCheckSpace = $(currentCompose).find("br[clear='all']").next('div').html();

                //if we do. lets remove it.
                if (currentComposeCheckSpace == '<br>') {
                    $(currentCompose).find("br[clear='all']").next('div').remove();
                }

                if (currValueObject.Subject != null) {
                    var decodedString = decodeHTMLEntity(htmlspecialchars_decode(currValueObject.Subject));
                    $(currentSubject).val(decodedString);

                    
                    //Sets value for the hidden subject field so we parse it correctly into our DB
                    $(currentSubject).closest('.aaZ, .I5').find("*[name='subject']").val(decodedString);
                }
                else {

                }

                if (currValueObject.TemplateType == "text") {
                    ebstaTemplatePre.innerText = $.trim(currValueObject.Body);
                    ebstaTemplateObj.appendChild(ebstaTemplatePre);
                    $(currentCompose).prepend(ebstaTemplateObj);
                }

                if (currValueObject.TemplateType == "custom") {
                    if (currValueObject.HtmlValue.length > 0) {
                        ebstaTemplateObj.innerHTML = currValueObject.HtmlValue;
                        $(currentCompose).prepend(ebstaTemplateObj);
                    } else if (currValueObject.Body.length > 0) {
                        ebstaTemplateObj.innerHTML = currValueObject.Body;
                        $(currentCompose).prepend(ebstaTemplateObj);
                    }
                }

                if (currValueObject.TemplateType == "html") {
                    if (currValueObject.HtmlValue.length > 0) {
                        //Removing CDATA tags returned by SF, these cause all sorts of issues where anything within them is commented out when the HTML is parsed.
                        //CDATA tags returned by SF also contain HTML, which causes broken HTML tags because tags inside the CDATA is commented out
                        ebstaTemplateObj.innerHTML = currValueObject.HtmlValue.replace(/<!\[CDATA\[/gim, "").replace(/]]>/gim, "");
                        $(currentCompose).prepend(ebstaTemplateObj);
                    } else if (currValueObject.Body.length > 0) {
                        ebstaTemplateObj.innerHTML = currValueObject.Body;
                        $(currentCompose).prepend(ebstaTemplateObj);
                    }
                }

                //Rescale the height in the compose window after append so you can scroll down in the compose window
                var composeHeight = $(currentCompose).find('.ebstaTemplatePre').closest('.GP').css('height');
                $(currentCompose).find('.ebstaTemplatePre').closest('.GP').css('max-height', composeHeight);
                $(currentCompose).find('.ebstaTemplatePre').closest('.qz').css('max-height', composeHeight);

                $("#emailTemplateBox").remove();
                //console.timeEnd("upsertEbstaTemplateObj");

                // Sometimes template information makes the compose window extend larger, 
                // this triggers the focus event which does this straight after we append it
                if (currentComposeCopy.length > 0) {
                    var composeSection = currentComposeCopy[0];
                    if (composeSection != null) {
                        composeSection.focus();
                    }
                }

            } else {
                showGmailMsg('Could not load template..', 'failure');
                $("#emailTemplateBox").remove();
            }
            
        } else {
            showGmailMsg('Could not load template..', 'failure');
            $("#emailTemplateBox").remove();
        }
    }

    function showGmailMsg(msg, cssClass) {
        var msg = msg;
        var MsgCss =cssClass
    
        var box = CreateDivWithId("", "sfBoxMsg" + cssClass, "sfgmailId");
        box.style.visibility = "visible";
        box.innerText = msg;

        (document.body || document.documentElement).appendChild(box);

        setTimeout(function () {
            var msg1 = document.getElementById("sfgmailId");
            //msg1.style.visibility = "hidden";
            $(msg1).fadeOut();
            $(msg1).fadeOut(function () { $(this).remove(); });
        }, 9000);
    }

    /**
     * renders a list of salesforce templates into an element
     * @param {} templates
     * @returns {} 
     */
    function RenderListOfSalesforceTemplates(templateJsonList) {

        var templateListContainer = document.getElementsByClassName("sfTemplateListContainer")[0];
        templateListContainer.innerHTML = '';
        for (var i = 0; i < templateJsonList.length; i++) {
            var currValueObject = templateJsonList[i];

            if (currValueObject.Name != null && currValueObject.Id != null) {
                var template = document.createElement("DIV");

                var nameContainer = CreateDiv("", "sfTemplateListName");
                nameContainer.appendChild(CreateDiv(currValueObject.Name, "ebsta-sftemplatename"));
                if (currValueObject.Description != null) {
                    var description = htmlspecialchars_decode(decodeHTMLEntity(currValueObject.Description));
                    var descriptionDiv = CreateDiv(description, "ebsta-sftemplatedescription ebsta-ellipsis");
                    descriptionDiv.title = description;
                    nameContainer.appendChild(descriptionDiv);
                }

                var typeContainer = CreateDiv("", "sfTemplateListType");
                if (currValueObject.TemplateType != null) {
                    typeContainer.innerHTML = currValueObject.TemplateType;
                }

                var sentContainer = CreateDiv("", "ebsta-sftemplatetrackingstatscell");

                var openrateContainer = CreateDiv("", "ebsta-sftemplatetrackingstatscell");

                var clickRateContainer = CreateDiv("", "ebsta-sftemplatetrackingstatscell");

                var replyRateContainer = CreateDiv("", "ebsta-sftemplatetrackingstatscell");

                if (currValueObject.NoTrackingStats === true) {
                    sentContainer.innerHTML = "-";
                    openrateContainer.innerHTML = "-";
                    clickRateContainer.innerHTML = "-";
                    replyRateContainer.innerHTML = "-";
                } else {
                    if (currValueObject.Sent != null) {
                        sentContainer.innerHTML = currValueObject.Sent;
                    }

                    if (currValueObject.OpenRate != null) {
                        openrateContainer.innerHTML = parseInt(currValueObject.OpenRate) + "%";
                    }

                    if (currValueObject.ClickRate != null) {
                        clickRateContainer.innerHTML = parseInt(currValueObject.ClickRate) + "%";
                    }

                    if (currValueObject.ReplyRate != null) {
                        replyRateContainer.innerHTML = parseInt(currValueObject.ReplyRate) + "%";
                    }
                }

                template.appendChild(nameContainer);
                template.appendChild(typeContainer);
                template.appendChild(sentContainer);
                template.appendChild(openrateContainer);
                template.appendChild(clickRateContainer);

                //render reply data.
                if (renderReplyData + "" === "true") {
                    template.appendChild(replyRateContainer);
                }

                template.id = currValueObject.Id;
                template.className = "sfTemplateListItem";
                templateListContainer.appendChild(template);

            }
        }

        $(".sfTemplateListItem").undelegate("click");
        $(".sfTemplateListItem").on("click", function (event) {
            event.stopImmediatePropagation();

            $('.sfEBComplexAddSearchObjectChoiceDiv').hide();

            var whoId = "";
            var whoRecordType = "";
            //Lets find if a user has selected anyone in the search box
            $('.sfEmailTemplateSearch').find('.sfWhoSearch').find('.CABoxStyle').each(function () {
                whoId = $(this).attr('sfwhoid');
                whoRecordType = $(this).attr('recordtype');
            });

            var whatId = "";
            var whatRecordTypeID = "";
            //Lets find if a user has selected anyone in the search box
            $('.sfEmailTemplateSearch').find('.sfWhatSearch').find('.CABoxStyle').each(function () {
                whatId = $(this).attr('whatid');
                whatRecordTypeID = $(this).attr('recordtype');
                if (whatRecordTypeID.toLowerCase() == "opp") {
                    whatRecordTypeID = "Opportunity";
                }
            });

            //Remove any previous loaders
            $('.sfEmailTemplateOptions .resultsLoader').remove();

            //Get template loader
            var loadingResults = CreateDiv("Loading template... ", "sfLoadingResults");
            loadingResults.className = 'resultsLoader';
            loadingResults.setAttribute("style", "display:inline-block;margin-left:10px;margin-top:15px;font-size:14px");
            var loadingGif = CreateImg(chrome.extension.getURL("/images/ajax-loader.gif"), "Loading template...");
            loadingResults.appendChild(loadingGif);

            var templateList = document.getElementById('sfTemplatesList');
            templateList.appendChild(loadingResults);

            chrome.extension.sendMessage({
                MessageType: "GetEmailTemplateById",
                TemplateId: $(this).attr("id"),
                WhoId: whoId,
                WhoRecordType: whoRecordType,
                WhatId: whatId,
                WhatRecordType: whatRecordTypeID,
            });
        });
    }

    function FolderTemplatesResult(entityType, resultObject) {
        var templateListContainer = CreateDiv("", "sfTemplateListContainer");
        var templateListHeaders = CreateDiv("", "sfTemplateListHeader");
        var templateList = document.getElementById("sfTemplatesList");

        templateListHeaders.appendChild(CreateDiv("Name", "sfTemplateNameHeader"));
        templateListHeaders.appendChild(CreateDiv("Type", "sfTemplateTypeHeader"));

        var sentHeader = CreateDiv("Sent", "ebsta-sftemplatetrackingstatscell");
        sentHeader.setAttribute("data-sortfield", "Sent");
        sentHeader.appendChild(CreateElement("span", "fa fa-sort ebsta-template-icon-sort"));
        templateListHeaders.appendChild(sentHeader);

        var openRateHeader = CreateDiv("Open Rate", "ebsta-sftemplatetrackingstatscell");
        openRateHeader.setAttribute("data-sortfield", "OpenRate");
        openRateHeader.appendChild(CreateElement("span", "fa fa-sort ebsta-template-icon-sort"));
        templateListHeaders.appendChild(openRateHeader);

        var clickRateHeader = CreateDiv("Click Rate", "ebsta-sftemplatetrackingstatscell");
        clickRateHeader.setAttribute("data-sortfield", "ClickRate");
        clickRateHeader.appendChild(CreateElement("span", "fa fa-sort ebsta-template-icon-sort"));
        templateListHeaders.appendChild(clickRateHeader);

        var replyRateHeader = CreateDiv("Reply Rate", "ebsta-sftemplatetrackingstatscell");
        if (renderReplyData + "" === "true") {            
            replyRateHeader.setAttribute("data-sortfield", "ReplyRate");
            replyRateHeader.appendChild(CreateElement("span", "fa fa-sort ebsta-template-icon-sort"));
            templateListHeaders.appendChild(replyRateHeader);
        }

        if (resultObject != null) {
            if (resultObject.d.Values != null) {
                if (resultObject.d.Values.length > 0) {
                    var templateJsonList = [];
                    for (var i = 0; i < resultObject.d.Values.length; i++) {
                        templateJsonList.push(JSON.parse(resultObject.d.Values[i]));
                    }
                    var emailTrackingStatsSortManager = ObjectListSortManager(templateJsonList);

                    templateList.innerHTML = ""; //Empty the list
                    templateList.appendChild(templateListHeaders);
                    templateList.appendChild(templateListContainer);

                    RenderListOfSalesforceTemplates(templateJsonList);

                    $(templateList).hide().slideDown();

                    var templateSortFunction = function (evnt) {
                        var target = evnt.target;
                        if (target) {
                            var container = target.closest('.ebsta-sftemplatetrackingstatscell');
                            if (container) {
                                var sortField = container.dataset['sortfield'];
                                if (sortField) {
                                    emailTrackingStatsSortManager.sort(sortField);
                                    RenderListOfSalesforceTemplates(templateJsonList);
                                }
                            }
                        }
                    }

                    // add sort to sent, open rate and click rate columns
                    sentHeader.removeEventListener('click', templateSortFunction);
                    sentHeader.addEventListener('click', templateSortFunction);

                    openRateHeader.removeEventListener('click', templateSortFunction);
                    openRateHeader.addEventListener('click', templateSortFunction);

                    clickRateHeader.removeEventListener('click', templateSortFunction);
                    clickRateHeader.addEventListener('click', templateSortFunction);

                    if (renderReplyData + "" === "true") {
                        replyRateHeader.removeEventListener('click', templateSortFunction);
                        replyRateHeader.addEventListener('click', templateSortFunction);
                    }

                    $('.sfEmailTemplateOptions .resultsLoader').remove();

                    if ($(this).attr("id") != null && typeof $(this).attr("id") != "undefined") {
                        lastTemplateSelected = $(this).attr("id");
                    }
                } else {
                    templateList.innerHTML = ""; //Empty the list
                    templateList.appendChild(CreateDiv("No templates available in this folder", "sfTemplatesNoFoundError"));
                    $('.sfEmailTemplateOptions .resultsLoader').remove();
                }
            } else {
                templateList.innerHTML = ""; //Empty the list
                templateList.appendChild(CreateDiv("No templates available in this folder", "sfTemplatesNoFoundError"));
                $('.sfEmailTemplateOptions .resultsLoader').remove();
            }
        } else {
            templateList.innerHTML = ""; //Empty the list
            templateList.appendChild(CreateDiv("No templates available in this folder", "sfTemplatesNoFoundError"));
            $('.sfEmailTemplateOptions .resultsLoader').remove();
        }
        
    }

    function decodeHTMLEntity(str) {
        return str.replace(/&#(\d+);/g, function(match, dec) {
            return String.fromCharCode(dec);
        });
    };

    function getParameterByNameFromUriString(name, uriString) {

        name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");

        var regexS = "[\\?&]" + name + "=([^&#]*)";
        var regex = new RegExp(regexS);
        var results = regex.exec(uriString);    

        if (results == null)
            return "";
        else
            return decodeURIComponent(results[1].replace(/\+/g, " "));
    }
    
    //returns true if string contains number
    function ContainsNumber(stringToCheck) {
        if (stringToCheck != null) {
            var num = stringToCheck.match(/\d+/g);
            if (num != null) {
                return true;
            }
        }
        return false;
    }

    function IsTracked(sender) {

        var response = 0;

        if ($("img[src$='ebsta.gif']").length)
        {
            response = 1;
        }

        return response;
    }

    function getMessageId(sender) {

        var messageId = "";

        if ($(sender).hasClass('sfComplexAdd') || $(sender).hasClass('sfQuickAdd') || $(sender).hasClass('sfQuickAdded') || $(sender).hasClass('sfComplexAdded')) {
            messageId = GmailTools.MessageView.GetMessageId($(sender).closest('.gs'));
        }

        return messageId;
    }

    function createMessageObject(sender) {
        var messageId = "";
        var attachments = new Array();
        var messageApiObj = "";

        if ($(sender).is('.sfComplexAdd, .sfComplexAddSend, .sfComplexAddSmallCompose, .sfComplexAdded')) {
            //Find the attachments for this email
            $(sender).closest('.adn.ads').find('.hq.gt .aQH').children('span').each(function () {
                attachments.push({ Name: $(this).find('.aV3').text(), Id: getParameterByNameFromUriString('attid', $(this).attr('download_url')) });
            });

            messageApiObj = ({ MessageId: messageId, Attachments: attachments });
        }

        if ($(sender).is('.sfComplexAdd, .sfComplexAddSend, .sfComplexAddSmallCompose')) {
            //Find the attachments for this email

            //loading gif for 500ms (we ideally need to delay here but causes loading issues)

            $(sender).closest('.iN').find('.GM .dL').each(function () {

                var attachmentRowLoader = $(this).find('.dR');

                //if an attachment is still uploading we should not be adding this as a selectable option in complex add.
                if (attachmentRowLoader != null && typeof attachmentRowLoader != "undefined" && attachmentRowLoader != "undefined") {
                    if (attachmentRowLoader.is(':visible')) {
                        // loader is visible.
                    } else {
                        attachments.push({ Name: $(this).find('.vI').text(), Id: getParameterByNameFromUriString('attid', $(this).children('.dO').attr('href')) });
                    }
                } else { //if null we have not found a loader.
                    attachments.push({ Name: $(this).find('.vI').text(), Id: getParameterByNameFromUriString('attid', $(this).children('.dO').attr('href')) });
                }
            });

            messageApiObj = ({ MessageId: messageId, Attachments: attachments });
        }

        return messageApiObj;
    }

    function checkGmailAuth() {
        chrome.extension.sendMessage({
            MessageType: "GmailAuth"
        }, function (response) {
                
        });
    }

    function GmailAuthResult(entityType, resultObject) {
        //var resultObj = JSON.parse(resultObject.d.Values);
        ////LogThis("GmailAuthResult");
        ////LogThis(resultObj);
        //The result from the gmail auth
    }

    function gmailAuthCheckResult(gmailAuth, webUserId) {
        if (gmailAuth != 'true') {
          
            var authUrl = 'https://console.ebsta.com/Google/ActivateGoogle.aspx?wb=' + webUserId;
            window.open(authUrl, '_blank');
        }
    }

    // known websites: same behaviour
    // unknown: empty bar should be displayed
    function IsKnownWebsite(url, websiteUrls) {

        var urlLower = url.toLowerCase();
        var arrayLength = websiteUrls.length;

        for (var i = 0; i < arrayLength; i++) {
            if (urlLower.indexOf(websiteUrls[i].toLowerCase()) != -1) {
                return true;
            }
        }
        return false;
    }

    function IsBarLoaded() {
        return !!document.getElementById("ebsta-frame");
    }

    // invocation comes from Main.js
    function ShowEmailTrackingSummaryBar() {

        var unknownWebsiteValue = 400;
        var knownWebsiteValue = 401;

        var barLoaded = IsBarLoaded();
        if (barLoaded) {

            // trigger the event 
            // iframe cannot be modified directly, need to call CleverBarLocalProcess.js
            chrome.extension.sendMessage({ MessageType: "OpenCloseTrackingPanelMessage" });
            return;
        }

        var knownWebsite = IsKnownWebsite(document.URL, [".linkedin.com", "mail.google.com/mail", "accounts.google.com"]);

        // passing parameters via query string
        var tag = "EmailTracking";
        if (knownWebsite) {
            //console.log("search 11");
            proc.LoadFrame(tag, knownWebsiteValue);
        } else {
            //console.log("search 12");
            proc.LoadFrame(tag, unknownWebsiteValue);
        }
    }

    function CreateWizardTooltipHover(tooltipContent, tooltipName, tooltipType, tooltipPosition, containerClassName) {

        var toolTipContainer = CreateSpan("", "" + containerClassName);

        var tooltipSpan = CreateSpan("", "ebWizardTooltip");

        $(tooltipSpan).append(tooltipContent);    

        var tooltipClose = CreateSpan("", "toolTipCloseHover");

        $(tooltipSpan).prepend(tooltipClose);

        var tooltipClasses = tooltipSpan.className + ' ' + tooltipType + ' ' + tooltipPosition;

        tooltipSpan.setAttribute("class", tooltipClasses);

        tooltipSpan.setAttribute("data-tooltip-id", tooltipName);

        toolTipContainer.appendChild(tooltipSpan);       

        return toolTipContainer;
    }

    function setWizardTooltipShown(tooltipId) {
        chrome.extension.sendMessage({ MessageType: tooltipId }, function (response) {
            
        });
    }
        
    //Handle the response for auth check for mailbox
    function HandleGmailAuthForMailboxResponse(needsGAuth, calendarSyncBubblePress, contactSyncBubblePress) {
        if (!needsGAuth) {
            isAccountAuthorised = true;

            GetSetup();

            if (enableMutationObservers + "" == "true") {
                EbstaGmailMutationObserver(["NewComposeWindowDetected"]);
            }

            GmailTools.InjectProxy();
        }

        SetupAuthOnboardingWizards(needsGAuth, calendarSyncBubblePress, contactSyncBubblePress);

        // Create the buttons after we've done all the rendering as this causes some incorrect 
        // behavior on the button if this is selected before we get this request back
        GoogleTools.Header.InsertButtons();
    }
    
    function GetSetup()
    {
        SetMsgInbox();
        LoadComplete();

        // Show the bad boy.
        if (document.URL.indexOf("Compose") != -1) {
            AddGoogleSendButtons();
        }
        //compose=new = when you have a compose window open in the normal gmail window
        //view=cm = when you have opened a compose window in a new tab, e.g when you have google + gmail email handler
        if (document.URL.indexOf("compose=new") != -1 || document.URL.indexOf("view=cm") !== -1) {
            AddButtonDelay();
        }

        GmailHoverCardSearch.Initiate();
    }
    
/**
 * Module for run searches when a user hovers over a contact triggering the display of a Gmail Hover Card
 */
var GmailHoverCardSearch = (function GmailHoverCardSearch() {
    /**
     * Last email we search for, to help dupe checks from happening
     */
    var lastHoverSearchEmail = "";
    
    /**
     * As gmail usually hides the hovercards rather than deleting it, this determines if the hovercard is within the viewport
     * @param {} element 
     * @returns {} 
     */
    function IsHoverCardVisible(element) {
        return element.getBoundingClientRect().y > 0;
    }
    
    /**
     * Gmail elements have attributes containing the name and email, this function will find those attributes
     * @param {} element 
     * @returns {} 
     */
    function FindEmailAndNameFromElement(element) {
        var email = null, name = null;

        if (element) {
            email = element.getAttribute("email");
            name = element.getAttribute("name");
        }

        return {
            Email: email,
            Name: name
        }
    }
    
    /**
     * Fires a new bar search if the email is a valid email and hasn't previously been run
     * @param {} hoverOverElement 
     * @returns {} 
     */
    function RunSearch(hoverOverElement) {
        var contactResult = FindEmailAndNameFromElement(hoverOverElement);

        //Retry process if we don't find it on the first attempt
        if (contactResult.Email === null) {
            contactResult = FindEmailAndNameFromElement(hoverOverElement.closest(".vN.bfK.a3q"));
        }

        if (ValidateEmail(contactResult.Email) && contactResult.Email !== lastHoverSearchEmail) {
            lastHoverSearchEmail = contactResult.Email;
            emailAddressToSearchJsonParse = contactResult.Email;

            if (contactResult.Name !== "me") {
                fullnameToSearchJsonParse = contactResult.Name;
            } else {
                fullnameToSearchJsonParse = "";
            }

            // Custom Bar Search we will pass through the name as well as the email address. searchtype = 4.
            if (useDefaultBarSearch) {
                proc.LoadFrame(contactResult.Email, 1);
            } else {
                var nameandemail = contactResult.Email + ",,," + contactResult.Name;
                proc.LoadFrame(nameandemail, 4);
            }
        }
    }
    
    /**
     * Checks where the mouse is currently, and gets where the user is hovered over in order to find the email address
     * Note: We can't access the contents of the Iframe where the hovercard is so we need to use this way to find the email address
     * @returns {} 
     */
    function FindEmailAndSearch() {
        var hoverOverElement = GetInnermostHoveredElement();
        if (hoverOverElement) {
            RunSearch(hoverOverElement);
            return true;
        }
        return false;
    }

    /**
     * Call back function handle the mutations received for the old Gmail UI (PRE 04/05/2018)
     * @param {} mutations 
     * @returns {} 
     */
    function OldUiObserverCallBack(mutations) {        

        for (var i = 0; i < mutations.length; i++) {
            var mutation = mutations[i];
            if (mutation.target && mutation.target.nodeName === "IFRAME" && HasClass(mutation.target, "Gp")) {
                if (IsHoverCardVisible(mutation.target)) {
                    if (FindEmailAndSearch()) break;
                }
            }
        }
    }

    /**
     * The callback function for when the observer detects any mutations within the New Gmail UI object
     * @param {} mutations 
     * @returns {} 
     */
    function NewUiObserverCallBack(mutations) {
        for (var i = 0; i < mutations.length; i++) {
            var mutation = mutations[i];
            if (mutation.target) {
                if (IsHoverCardVisible(mutation.target)) {
                    if (FindEmailAndSearch()) break;
                }
            }
        }
    }

    /**
     * Mutation Observer callbacks for different types of Gmail UI
     */
    var uiTypeObserverCallBacks = {
        NewUi: NewUiObserverCallBack,
        OldUi: OldUiObserverCallBack
    }

    /**
     * Which function to call when mutation observer is called
     */
    var observerCallBackFunction = uiTypeObserverCallBacks.OldUi;

    /**
     * Mutation observer setup
     */
    // TODO: MUTATION
    var hoverCardMutationObserver = new window.MutationObserver(function (mutations) {
        observerCallBackFunction(mutations);
    });

    /**
     * Returns the DOM element which contains the hovercard details
     * @returns {} 
     */
    function FindGmailHoverCardDivNewUi() {
        return document.getElementById("__HC_94253229");
    }

    /**
     * Sets up the observer if we find the element corresponding to the new Gmail UI
     * @returns {} 
     */
    function SetGmailHoverCardObserverNewUi() {
        RetryWithCallback(60, 1, function() {
            return FindGmailHoverCardDivNewUi() != null;
        }, function () {
            hoverCardMutationObserver.disconnect();
            observerCallBackFunction = uiTypeObserverCallBacks.NewUi;
            hoverCardMutationObserver.observe(FindGmailHoverCardDivNewUi(), { attributes: true, subtree: true });
        });
    }

    /**
     * Sets a mutation observer on the parent Div which contains the Gmail Contact Hover Card for the Old UI (Pre-04/05/2018)
     * @returns {} 
     */
    function SetGmailHoverCardObserverOldUi() {

        var hoverCardContainer = null;

        // PRE NEW GMAIL UI - 04/05/2018
        // The HoverCard iframe is inserted into an empty DIV which exists on initial load, so here 
        // we find it and attach the mutation observer to it
        for (var i = 0; i < document.body.childNodes.length; i++) {
            var node = document.body.childNodes[i];
            if (node.nodeName === "DIV" && node.attributes && node.attributes.length === 0) {
                //SAFETY CHECK - THERE SHOULD ONLY BE ONE EMPTY DIV, IF WE FIND MULTIPLE SET BACK TO NULL SO NOTHING HAPPENS FROM HERE ON
                if (hoverCardContainer !== null) {
                    hoverCardContainer = null;
                    break;
                }

                hoverCardContainer = node;
            }
        }

        // pass in the target node, as well as the observer options
        if (hoverCardContainer) {
            hoverCardMutationObserver.observe(hoverCardContainer, { attributes: true, subtree: true });
        }
    }

    return {
        /**
         * Runs a bar search
         * @param {} hoverOverElement 
         * @returns {} 
         */
        RunSearch: function (hoverOverElement) {
            RunSearch(hoverOverElement);
        },
        /**
         * Sets the module up
         * @returns {} 
         */
        Initiate: function () {
            //Try the old ui before the new Ui, as the new UI will disconnect the observer if it's currently connected
            SetGmailHoverCardObserverOldUi();
            SetGmailHoverCardObserverNewUi();
        }
    }
})();


    
    //Sets up the divs for the onboarding process
    function SetupAuthOnboardingWizards(userNeedsToAuthFirst, calendarSyncBubblePress, contactSyncBubblePress) {
        //console.time("SetupAuthOnboardingWizards");
        //Remove any exisitng ebsta onbarding wizards
        $("#eb-wizard-onboarding").remove();
        //Create container for modals
        var wizardContainerId = CreateModalContainer("eb-wizard-onboarding");
        //create modal elements
        var ebstaModals = CreateOnboardingWizardElements();
        //Get the container element from DOM
        var wizardElemen = document.getElementById(wizardContainerId);

        //if we have it - add the onboarding elements to it
        if (wizardElemen != null && typeof wizardElemen != "undefined") {
            wizardElemen.appendChild(ebstaModals);

            //Lets create the steps of the Wizard
            CreateOnBoardingSteps();
            if (abRules.EnableContactSync + '' === "0") {
                ChangeElementsForAbTesting();
            }

            //set up events for modals
            SetupEbstaModalEvents();
            //If the user selected the Google Calendar Sync option in the bubble and the user is authed then open the onboarding at calendar sync setup page
            if (currentMailBoxActive =='true' && !userNeedsToAuthFirst && (calendarSyncBubblePress || contactSyncBubblePress))
            {
                if (calendarSyncBubblePress || document.URL.indexOf("google.com/calendar") != -1)
                {
                LogEvent("Onboarding Appear OnLoad Calendar Start");
                OpenOnboardingModal(3);
            } 
                else if (contactSyncBubblePress || (document.URL.indexOf("contacts.google.com") != -1 || document.URL.indexOf(".google.com/contacts") !== -1))
                {
                    ("Onboarding Appear OnLoad Contact Sync Start");
                    OpenOnboardingModal(8);
                }
            } 
            //check if we create wizard with intent of immidiate start - i.e if we checked for auth and registed needs auth
            else if (userNeedsToAuthFirst || (currentMailBoxActive=='false' && (calendarSyncBubblePress || contactSyncBubblePress)))
            {
                //open auth stage
                OpenOnboardingModal(1);
                LogEvent("Onboarding Appear OnLoad");
            }
            else
            {
                $('#eb-wizard-onboarding,.eb-modal-bodyoverlay').hide();
            }

            currentMailBoxActive = false;
        }
        ActionOnContinueButtonClick();

        //console.timeEnd("SetupAuthOnboardingWizards");
    }
    
    //Changes certain elements within Ebsta onboarding wizard for people without access to certain features
    function ChangeElementsForAbTesting() {
        //Change Headers so they fit nicely
        $('#eb-wizard-onboarding .nav > li > a').css('padding', '10px 15px').css('width', '0').css('height', '0');
        $('#eb-wizard-onboarding .eb-wizard-modalpage').css('height', '631px');
    }
    
    //Create overall container for modals - and body overlay
    function CreateModalContainer(containerId)
    {
        //overlay shows when class eb-modal-open is added to body
        var className = "eb-modal-container";

        if (abRules.EnableCampainMgr || abRules.EnableMeetingScheduler) {
             className = className + " eb-onboarding-flexibleSize";
        }

        var wizardContainer = CreateDivWithId("", className, containerId);

        //append container to body
        document.body.appendChild(wizardContainer);

        if ($('body .eb-modal-bodyoverlay').length < 1) {
            var bodyOverlay = CreateDiv("", "eb-modal-bodyoverlay");

            document.body.appendChild(bodyOverlay);
        }

        return containerId;
    }
    
    //Create the elements for the onboarding - append the to div and return
    function CreateOnboardingWizardElements()
    {
        var createdModals = document.createElement("DIV");

        createdModals.appendChild(CreateModalAuthPage());

        createdModals.appendChild(CreateOnBoardingWizardMainDiv());

        return createdModals;
    }

    //Main Div to hold the contents of the wizard steps
    function CreateOnBoardingWizardMainDiv()
    {
        var mainContentDiv = CreateDivWithId("", "eb-wizard-modalpage panel panel-default ", "eb-wizard-stepbystepprocess");

        //Create Header
        var title = CreateDiv("", "eb-onboardingtextheader panel-heading");
        title.appendChild(CreateDiv("", "eb-onboardinglogo"));

        var closeIconDiv = CreateDiv("", "ebsta-glyphicon ebsta-glyphicon-close-icon");
        $(closeIconDiv).on("click", function () {
            ClearModals(true);
            GetSetup();
        });
        title.appendChild(closeIconDiv);
        mainContentDiv.appendChild(title);

        var mainBody = CreateDiv("", "panel-body");
        mainBody.appendChild(CreateOnBoardingOptionHeader());
        mainBody.appendChild(CreateOnBoardingProgressBar());

        //Steps Container Div to hold all the steps
        var stepsDivHolder = CreateDiv("","eb-Steps");
        mainBody.appendChild(stepsDivHolder);

        //buttons container div to hold all the buttons
        var buttonsDiv = CreateDiv("","eb-onboardingbuttons");

        //Continue button
        var continueButton = CreateButton("", "Continue", "ebsta-btn ebsta-btn-lg ebsta-btn-success eb-continuebtn");
        buttonsDiv.appendChild(continueButton);

        //Back Button
        var backButton = CreateButton("", "Back", "ebsta-btn ebsta-btn-lg ebsta-btn-default eb-backbtn");
        buttonsDiv.appendChild(backButton);

        //Skip Sync Button
        var skipCalendarSyncButton = CreateButton("", "Skip", "ebsta-btn ebsta-btn-lg ebsta-btn-warning eb-skipsyncbtn");
        skipCalendarSyncButton.id = "EbstaBtSkipCalendarSyncSetup";

        $(skipCalendarSyncButton).hide();
        $(skipCalendarSyncButton).on("click", function () {
            var stepname = $('.ebstaStep:visible').attr('stepname');
            if (stepname == "Sync Calendars") {
            ShowEbstaStep(8);
            LogEvent("Onboarding Click - Skip Calendar Sync");
            } else if (stepname == "Sync Contacts") {
                ShowEbstaStep(13);
                LogEvent("Onboarding Click - Skip Contact Sync");
            } else if (stepname == "Contact Sync Listview Setup Screen") {
                ShowEbstaStep(13);
                LogEvent("Onboarding Click - Skip Listview Setup Screen");
            }
            $(this).blur();
        });
        buttonsDiv.appendChild(skipCalendarSyncButton);

        //Refresh TimeZones button
        var refreshTimezones = CreateButton("", "Refresh", "ebsta-btn ebsta-btn-lg ebsta-btn-warning eb-refreshtimezonesbtn");
        $(refreshTimezones).hide();
        $(refreshTimezones).on("click", function () {
            SendMessageToGetCalendarSyncSettings();
            $(this).blur();
            LogEvent("Onboarding Click - Refresh Timezones");
        });
        buttonsDiv.appendChild(refreshTimezones);

        //Sync Now Button
        var syncNowButton = CreateButton("", "Sync Now", "ebsta-btn ebsta-btn-lg ebsta-btn-success eb-syncnowbtn");
        $(syncNowButton).hide();
        $(syncNowButton).on("click", function () {
            chrome.extension.sendMessage({ MessageType: "Onboarding-SyncCalendarNow" });
            $(this).blur();
            LogEvent("Onboarding Click - Sync Now");
            UpdateSyncNowButton(false);
        });
        buttonsDiv.appendChild(syncNowButton);

        //Stop Calendar Sync Button
        var stopCalendarySyncButton = CreateButton("", "Stop Calendar Sync", "ebsta-btn ebsta-btn-lg ebsta-btn-danger eb-stopcalendarsyncbtn");
        $(stopCalendarySyncButton).hide();
        $(stopCalendarySyncButton).on("click", function () {
            $(this).blur();
            UpdateCalendarSettingsPage(false);
            $('.eb-calsync-calendarlist input:checkbox').removeAttr('checked');
            chrome.extension.sendMessage({ MessageType: "Onboarding-StopCalendarSync" });
            isCalendarSyncLive = false;
            LogEvent("Onboarding Click - Stop Calendar Sync");
        });
        buttonsDiv.appendChild(stopCalendarySyncButton);

        //Setup Calendar Sync Button
        var calendarySyncButton = CreateButton("", "Calendar Sync Settings", "ebsta-btn ebsta-btn-lg ebsta-btn-success eb-setupcalendarsyncbtn");
        calendarySyncButton.id = "EbstaBtCalendarSyncSettings";

        $(calendarySyncButton).hide();
        $(calendarySyncButton).on("click", function () {
            ShowEbstaStep(4);
            $(this).blur();
            LogEvent("Onboarding Click - Setup Calendar Sync");
        });
        buttonsDiv.appendChild(calendarySyncButton);

        //Setup email campaign Button
        var emailCampaignSetup = CreateButton("", "Email Cadence Setup", "ebsta-btn ebsta-btn-lg ebsta-btn-success eb-setupemailcampaignbtn");
        $(emailCampaignSetup).hide();
        $(emailCampaignSetup).on("click", function () {
            if (webUserId != null) {
                var uri = "https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=30";
                OpenNewTab(uri);
            }
        });
        buttonsDiv.appendChild(emailCampaignSetup);

        //Setup meeting scheduler Button
        var meetingSchedulerSetup = CreateButton("", "Meeting Scheduler Setup", "ebsta-btn ebsta-btn-lg ebsta-btn-success eb-setupmeetingschedulerbtn");
        $(meetingSchedulerSetup).hide();
        $(meetingSchedulerSetup).on("click", function () {
            if (webUserId != null) {
                var redirectUrl = "https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=8";
                OpenNewTab(redirectUrl);
            }
        });
        buttonsDiv.appendChild(meetingSchedulerSetup);

        buttonsDiv = SetupContactSyncButtons(buttonsDiv);

        mainBody.appendChild(buttonsDiv);
        mainContentDiv.appendChild(mainBody);

        return mainContentDiv;
}

/** Module to contain the all elements around our onboarding wizard we display on Google pages */
var googleOnboardingWizard = (function () {

    /** Contact Sync module for the onboardingwizard*/
    let contactSync = (function () {
        let listViewEbstaSearchBox = null;

        /**
         * Creates and sets up the search bar for Contact Sync
         * @param {DOMElement} mainContentContainer - The DOM element which contains the results if there is any or an alert to state we weren't able to retrieve any results
         * @param {DOMElement} loadingGif - The DOM containing the loading gif we'll display when a user runs a search
         * @param {DOMElement} resultsTable - The table containg the results that we'll need to hide during the search
         * @returns {DOMElement} - The search bar DOM element
         */
        function CreateContactSyncSearchDOM(mainContentContainer, loadingGif, resultsTable) {
            let searchDiv = CreateEbstaSearchBox("Search for Contact List View by name");

            (function ContactSyncSearchClosure(mainContainer, loadingGifDom, resultsTableDom) {
                let searchValidators = [];
                searchValidators.push(new EbstaSearchBoxValidator(function (text) {
                    return text.length >= 2;
                }, "Please enter at least 2 characters."));

                /**
                 * Runs a search for Contact List Views by name
                 * @param {function} ebstaSearchBoxCallback - A function provided back from the EbstaSearchBox which we need to call when we've finished searching so the SearchBox reflects a state of "finished"
                 */
                function Search(ebstaSearchBoxCallback) {
                    if (mainContainer !== null) {
                        var errorDiv = mainContainer.getElementsByClassName("alert")[0];
                        if (errorDiv) {
                            errorDiv.remove();
                        }
                    }
                    HideElement(resultsTableDom);
                    ShowElementBlock(loadingGifDom);
                    var searchValue = listViewEbstaSearchBox.GetText();

                    //Run the search, with a callback that hides the loading gif as well as calls the callback provided from the EbstaSearchBox setup
                    GetContactSyncListViews(searchValue, function () {
                        ebstaSearchBoxCallback();
                        HideElement(loadingGifDom);
                    }, resultsTableDom);
                }
                listViewEbstaSearchBox = EbstaSearchBox(searchDiv, Search, null, searchValidators);
            })(mainContentContainer, loadingGif, resultsTable);

            return searchDiv;
        }

        /**
         * Creates the DOM where we'll search and show the list views the user is able to sync
         * @param {int} ebstaStep - The  number of this screen in the sequence for the wizard
         * @returns {DOMElement} - The DOM that needs to be rendered
         */
        function CreateOnBoardingStepContactSyncListViewSetup(ebstaStep) {
            var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep + " ebstaStepContactSyncSetting");
            contentDiv.setAttribute("stepname", "Contact Sync Listview Setup Screen");

            contentDiv.appendChild(CreateElement("h3", "page-header eb-page-header text-left", "Pick your Contact Sync Settings"));

            var paragraphText = CreateElement("p", "eb-textcontent", "Please select the Contacts you wish to sync from Salesforce to Google from your custom Salesforce Contact list views below.", null);
            contentDiv.appendChild(paragraphText);

            var loadingGif = CreateOnBoardingLoadingGif("We're currently retrieving your Contact list views from Salesforce, please wait...");
            var resultsTable = CreateTableForListViewContactSync();

            var previewEventDiv = CreateDiv("", "eb-contactsync-selectlistviews");
            previewEventDiv.appendChild(resultsTable);
            previewEventDiv.appendChild(loadingGif);

            contentDiv.appendChild(CreateContactSyncSearchDOM(previewEventDiv, loadingGif, resultsTable));

            HideElement(loadingGif);
            HideElement(resultsTable);

            contentDiv.appendChild(previewEventDiv);

            return contentDiv;
        }

        /** Fire another search off using same criteria on the search box */
        function ReRunSearch() {
            listViewEbstaSearchBox.Search();
        }

        return {
            CreateOnBoardingStepContactSyncListViewSetup: CreateOnBoardingStepContactSyncListViewSetup,
            ReRunSearch: ReRunSearch
        };
    })();

    return {
        ContactSync: contactSync
    };

})();

    //Create extra buttons within OnBoarding for Contact Sync
    function SetupContactSyncButtons(buttonsDiv)
    {
        //Setup Contact Sync Button
        var contactSyncButton = CreateButton("", "Contact Sync Settings", "ebsta-btn ebsta-btn-lg ebsta-btn-success eb-setupcontactsyncbtn");
        contactSyncButton.id = "EbstaBtContactSyncSettings";

        $(contactSyncButton).hide();
        $(contactSyncButton).on("click", function () {
            ShowEbstaStep(9);
            $(this).blur();
            LogEvent("Onboarding Click - Setup Contact Sync");
        });
        buttonsDiv.appendChild(contactSyncButton);

        //Setup Another Sync button
        var setupAnotherSyncButton = CreateButton("", "Setup Another Contact Sync", "ebsta-btn ebsta-btn-lg ebsta-btn-success eb-setupanothercontactsync");
        $(setupAnotherSyncButton).hide();
        $(setupAnotherSyncButton).on("click", function () {
            googleOnboardingWizard.ContactSync.ReRunSearch();
            ShowEbstaStep(9);
            $(this).blur();
            LogEvent("Onboarding Click - Setup Another Sync");
        });
        buttonsDiv.appendChild(setupAnotherSyncButton);

        return buttonsDiv;
    }
    
    //Creates the process bar for the inboarding process
    function CreateOnBoardingProgressBar() {
        var mainProgressDiv = CreateDiv("", "progress eb-progress");

        var barProgress = CreateDiv("", "progress-bar eb-progressbar");
        $(barProgress).attr("role", "progressbar");
        $(barProgress).attr("aria-valuenow", "21");
        $(barProgress).attr("aria-valuemin", "0");
        $(barProgress).attr("aria-valuemax", "100");
        $(barProgress).css("width", "25%");
        $(barProgress).attr("role", "progressbar");

        var completeSpan = CreateSpan("33% Complete", "sr-only");

        barProgress.appendChild(completeSpan);

        mainProgressDiv.appendChild(barProgress);

        return mainProgressDiv;
    }
    
    //Creates the Iframe for the youtube videos
    function CreateVideoFrameForOnBoarding(videoUrl, className, id) {
        var videoFrame = CreateElement('iframe', className, null, id);
        videoFrame.src = videoUrl;
        videoFrame.frameBorder = 0;

        return videoFrame;
    }


    //Creates all the steps for the onboarding process
    function CreateOnBoardingSteps()
    {
        if ($('.eb-Steps').length == 1)
        {
            var stepsHolderDiv = $('.eb-Steps')[0];
            var ebstaStep = 2;
            stepsHolderDiv.appendChild(CreateOnBoardingStepAddEmailsToSalesforce(ebstaStep++));
            stepsHolderDiv.appendChild(CreateOnBoardingStepSyncCalendarsMainScreen(ebstaStep++));
            stepsHolderDiv.appendChild(CreateOnBoardingStepSyncCalendarFirstStep(ebstaStep++));
            stepsHolderDiv.appendChild(CreateOnBoardingStepSyncCalendarSecondStep(ebstaStep++));
            stepsHolderDiv.appendChild(CreateOnBoardingStepSyncCalendarConfirmSettings(ebstaStep++));
            stepsHolderDiv.appendChild(CreateOnBoardingStepSyncCalendarCompletion(ebstaStep++));

            if (abRules.EnableContactSync + '' === "1") {
                stepsHolderDiv.appendChild(CreateOnBoardingStepContactSyncIntroScreen(ebstaStep++));
                stepsHolderDiv.appendChild(googleOnboardingWizard.ContactSync.CreateOnBoardingStepContactSyncListViewSetup(ebstaStep++));
                stepsHolderDiv.appendChild(CreateOnBoardingStepContactSyncFieldMapping(ebstaStep++));
                stepsHolderDiv.appendChild(CreateOnBoardingStepContactSyncPreview(ebstaStep++));
                stepsHolderDiv.appendChild(CreateOnBoardingStepContactSyncCompletion(ebstaStep++));
            }

            stepsHolderDiv.appendChild(CreateOnBoardingStepSalesforceEmailTracking(ebstaStep++));
            stepsHolderDiv.appendChild(CreateOnBoardingStepSalesforceTemplates(ebstaStep++));
            stepsHolderDiv.appendChild(CreateOnBoardingStepSalesforceInsideGmail(ebstaStep++));

            //email campaign step
            if (abRules.EnableCampainMgr + '' === "1") {
                stepsHolderDiv.appendChild(CreateOnBoardingStepCampaignManager(16));
            }
            
            // calendar booking step
            if (abRules.EnableMeetingScheduler + '' === "1") {
                stepsHolderDiv.appendChild(CreateOnBoardingStepMeetingScheduler(17));
            }

            $('.ebstaStep').hide();
            $('.ebstaStep2').show();

            SetListenerUpOnContinueButton(false, false, false, false);
            SetListenerUpOnBackButton();
        }
    }

    // create onboarding step for meeting scheduler
    function CreateOnBoardingStepMeetingScheduler(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep);
        contentDiv.setAttribute("stepname", "Meeting Scheduler");

        var header = CreateElement("h3", "page-header eb-page-header text-left", "Meeting Scheduler");
        var icon = CreateElement('i', 'fa fa-question-circle eb-onboarding-knowledgebase-icon', null, null);
        icon.title = 'Click to check out our knowledgebase article';
        AttachClickThroughListenerToElement(icon, 'https://ebstalimited.zendesk.com/hc/en-us/articles/115002502828-Using-Meeting-Scheduler');
        header.appendChild(icon);
        contentDiv.appendChild(header);

        var paragraphText = CreateElement("p", "eb-textcontent", "Share your calendar with a simple link to effortlessly arrange meetings without the traditional back and forth.", null);
        contentDiv.appendChild(paragraphText);
        contentDiv.appendChild(CreateBr());

        return contentDiv;
    }

    // create onboarding step for campaign manager
    function CreateOnBoardingStepCampaignManager(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep);
        contentDiv.setAttribute("stepname", "Email Cadences");

        var header = CreateElement("h3", "page-header eb-page-header text-left", "Email Cadences");
        var icon = CreateElement('i', 'fa fa-question-circle eb-onboarding-knowledgebase-icon', null, null);
        icon.title = 'Click to check out our knowledgebase article';
        AttachClickThroughListenerToElement(icon, 'https://ebstalimited.zendesk.com/hc/en-us/articles/115002519188-Using-Email-Campaign');
        header.appendChild(icon);

        contentDiv.appendChild(header);
        contentDiv.appendChild(CreateElement("p", "eb-textcontent", "Build automatic Email Cadences and create follow-up tasks for targeted lists of prospects or customers.", null));
        contentDiv.appendChild(CreateBr());

        return contentDiv;
    }

    //Creates the header for the on boarding wizard with options to proceed through each step
    function CreateOnBoardingOptionHeader() {
        var header = CreateDiv("", "panel-body eb-optionheader");

        var mainList = document.createElement("ul");
        mainList.className = "nav nav-pills";

        var stepNo = 2;

        // 'add emails to salesforce' header
        var mainListItem = document.createElement("li");
        mainListItem.className = "ebsta-onboarding-header";
        var link = CreateHyperlink(stepNo, "#", "Add Emails to Salesforce");
        link.setAttribute("onClick", "return false;");
        mainListItem.appendChild(link);
        mainListItem.addEventListener("click", function (event) { 
            UpdateOnBoardingTab(event, null); 
            ShowEbstaStep(2); 
            LogEvent("Onboarding Click - Header (Add Emails to Salesforce)");
        });
        mainList.appendChild(mainListItem);
        stepNo++;

        // 'sync calendars' header
        var mainListItem2 = document.createElement("li");
        mainListItem2.className = "ebsta-onboarding-header";
        var link = CreateHyperlink(stepNo, "#", "Sync Calendars");
        link.setAttribute("onClick", "return false;");
        mainListItem2.appendChild(link);
        mainListItem2.addEventListener("click", function (event) {
            UpdateOnBoardingTab(event, null);
            ShowEbstaStep(3);
            LogEvent("Onboarding Click - Header (Sync Calendars)");
        });
        mainList.appendChild(mainListItem2);
        stepNo++;

        //Helps counter for the number of steps included if COntact Sync is disabled for this user
        var contactSyncOffset = 5;

        // 'contact sync' header
        if (abRules.EnableContactSync + '' === "1") {
            var mainListItem3 = document.createElement("li");
            mainListItem3.className = "ebsta-onboarding-header";
            var link = CreateHyperlink(stepNo, "#", "Sync Contacts");
            link.setAttribute("onClick", "return false;");
            mainListItem3.appendChild(link);
            mainListItem3.addEventListener("click", function(event) {
            UpdateOnBoardingTab(event, null);
            ShowEbstaStep(8);
                LogEvent("Onboarding Click - Header (Sync Contacts)");
        });
            mainList.appendChild(mainListItem3);
            contactSyncOffset = 0;
        }
        stepNo++;

        // 'email tracking' header
        var mainListItem4 = document.createElement("li");
        mainListItem4.className = "ebsta-onboarding-header";
        var link = CreateHyperlink(stepNo, "#", "Email Tracking");
        link.setAttribute("onClick", "return false;");
        mainListItem4.appendChild(link);
        mainListItem4.addEventListener("click", function (event) {
            UpdateOnBoardingTab(event, null);
            ShowEbstaStep(13 - contactSyncOffset);
            LogEvent("Onboarding Click - Header (Email Tracking)");
        });
        mainList.appendChild(mainListItem4);
        stepNo++;

        // 'salesforce templates' header
        var mainListItem5 = document.createElement("li");
        mainListItem5.className = "ebsta-onboarding-header";
        var link = CreateHyperlink(stepNo, "#", "Salesforce Templates");
        link.setAttribute("onClick", "return false;");
        mainListItem5.appendChild(link);
        mainListItem5.addEventListener("click", function (event) {
            UpdateOnBoardingTab(event, null);
            ShowEbstaStep(14 - contactSyncOffset);
            LogEvent("Onboarding Click - Header (Salesforce Templates)");
        });
        mainList.appendChild(mainListItem5);
        stepNo++;

        // 'salesforce inside gmail' header
        var mainListItem6 = document.createElement("li");
        mainListItem6.className = "ebsta-onboarding-header";
        var link = CreateHyperlink(stepNo, "#", "Salesforce inside Gmail");
        link.setAttribute("onClick", "return false;");
        mainListItem6.appendChild(link);
        mainListItem6.addEventListener("click", function (event) {
            UpdateOnBoardingTab(event, null);
            ShowEbstaStep(15 - contactSyncOffset);
            LogEvent("Onboarding Click - Header (Salesforce inside Gmail)");
        });
        mainList.appendChild(mainListItem6);
        stepNo++;

        // 'campaigns' header
        if (abRules.EnableCampainMgr + '' === "1") {
            var mainListItem7 = document.createElement("li");
            mainListItem7.className = "ebsta-onboarding-header";
            var mainListItem7Link = CreateHyperlink(stepNo, "#", "Email Cadences");
            mainListItem7Link.setAttribute("onClick", "return false;");
            mainListItem7.appendChild(mainListItem7Link);
            mainListItem7.addEventListener("click", function (event) {
                UpdateOnBoardingTab(event, null);
                ShowEbstaStep(16);
                LogEvent("Onboarding Click - Header (Email cadences)");
            });
            mainList.appendChild(mainListItem7);
            stepNo++;
        }

        // 'meeting scheduler' header
        if (abRules.EnableMeetingScheduler + '' === "1") {
            var mainListItem8 = document.createElement("li");
            mainListItem8.className = "ebsta-onboarding-header";
            var mainListItem8Link = CreateHyperlink(stepNo, "#", "Meeting Scheduler");
            mainListItem8Link.setAttribute("onClick", "return false;");
            mainListItem8.appendChild(mainListItem8Link);
            mainListItem8.addEventListener("click", function (event) {
                UpdateOnBoardingTab(event, null);
                ShowEbstaStep(17);
                LogEvent("Onboarding Click - Header (Meeting Scheduler)");
            });
            mainList.appendChild(mainListItem8);
            stepNo++;
        }

        //var mainListItem6 = document.createElement("li");
        //mainListItem6.appendChild(CreateHyperlink("7", "#", "Share Ebsta"));
        //mainListItem6.addEventListener("click", function () { ShowEbstaStep(11) });

        header.appendChild(mainList);
        return header;
    }
    
    //Updates the header tab to be selected
    function UpdateOnBoardingTab(event, step)
    {
        if (event) {
            if (event.toElement) {
                $(event.toElement).closest('ul').find('li.active').removeClass('active');
                $(event.toElement).closest('li').addClass("active");
            }
        } else if(step) {
            if ($(".eb-optionheader").length > 0) {
                if ($(".eb-optionheader").find('a.' + step).length > 0) {
                    $(".eb-optionheader").find('a.' + step).closest('ul').find('li.active').removeClass('active');
                    $(".eb-optionheader").find('a.' + step).closest('li').addClass("active");
                }
            }
        }
    }
    
    //Show a specified ebsta step in the Onboarding process
    function ShowEbstaStep(stepNo) {
        RemoveVideoIframesFromOnboardingProcess();
        $(".ebstaStep").hide();
        $(".ebstaStep" + stepNo).fadeIn(function()
        {
            InsertVideoIframeForStep();
        });

        //Run search and show all contact lists on Contact Sync Setting load
        if (stepNo == 9 && googleOnboardingWizard && googleOnboardingWizard.ContactSync) {
            googleOnboardingWizard.ContactSync.ReRunSearch();
        }
        
        if (stepNo == 3) {
            $('.eb-setupcalendarsyncbtn').fadeIn();
        }

        if (stepNo == 8) {
            if (abRules.EnableContactSync + '' === "1") {
                $('.eb-setupcontactsyncbtn').fadeIn();
            }
        }

        ActionOnContinueButtonClick();
    }

    // Steps forward through the OnBoarding Process
    // -- Refresh settings parameter helps decipher if the button needs to refresh the settings for the pre-requisie check, everything else just proceeds to the next step
    // -- Preview settings parameter helps let us know that the continue button is now the 'Preview Settings' button and to action appropriately on the click event
    // -- Confirm settings parameter helps let us know that user wishes to confirm the settings previously chosen
    // -- Finish - This is the last click to close the box down
    function SetListenerUpOnContinueButton(refreshsettings, previewSettings, confirmSettings, finish)
    {
        if ($('.eb-continuebtn').length == 1)
        {
            $('.eb-continuebtn').off();

            $('.eb-continuebtn').on("click", function ()
            {
                $(this).blur();
                if (refreshsettings)
                {
                    SendMessageToCheckFieldsForCalendarSync(true);
                    $('.ebstaStep:visible').find('.ebNewLoader').show(); //Show normal messages and loader
                    $('.ebstaStep:visible').find('.alert-danger, .alert-info, .eb-fieldErrorTable').hide(); //hide error messages
                    LogEvent("Onboarding Click - Refresh Settings");
                }
                else if (finish)
                {
                    RemoveVideoIframesFromOnboardingProcess();
                    $('#eb-wizard-onboarding, .eb-modal-bodyoverlay').hide();
                    LogEvent("Onboarding Click - Finish");
                    GetSetup();
                }
                else
                {
                    var logEventText = "";

                    if (previewSettings)
                    {
                        logEventText = "Onboarding Click - Preview Settings";

                        if ($('.eb-calsync-calendarlist-calendarsettings input:checked').length > 0)
                        {
                            SavePreviewSettingsForCalendarySync();
                            $('.ebstaStep:visible').find('.alert-danger').fadeOut();
                            $('.ebstaStep:visible').find('.eb-calsync-calendarlist').scrollTop();
                        }
                        else
                        {
                            $('.ebstaStep:visible').find('.alert-danger').fadeIn();
                            $('.eb-calsync-calendarlist').css("max-height", "240px");
                            LogEvent(logEventText);
                            return false;
                        }
                    }

                    //Send message to say user has confirmed settings
                    if (confirmSettings) {
                        chrome.extension.sendMessage({
                            MessageType: "CalSyncConfirmSettings",
                            GmailAccount: GetGmailUsersEmailAddress(),
                            IsOldAuthTokenBeingUsed: global_usingOldAuthToken
                        });
                        logEventText = "Onboarding Click - Confirm Settings";
                        $('.eb-authwizard-restart-calendar').remove();
                    }

                    RemoveVideoIframesFromOnboardingProcess();

                    var currentStep = $(".ebstaStep:visible");

                    //Bring next step in
                    $(currentStep).next().fadeIn(500);

                    //Hide last step
                    $(currentStep).hide();

                    InsertVideoIframeForStep();

                    if (logEventText.length == 0)
                    {
                        LogEvent("Onboarding Click - Continue (" + $(currentStep).attr('stepname') + ")");
                    }
                    else
                    {
                        LogEvent(logEventText);
                    }

                    ActionOnContinueButtonClick();
                }
            });
        }
    }
    
    //Sets a listener on the back button for onboarding process
    function SetListenerUpOnBackButton()
    {
        if ($('.eb-backbtn').length == 1)
        {
            $('.eb-backbtn').off();

            $('.eb-backbtn').on("click", function ()
            {
                RemoveVideoIframesFromOnboardingProcess();
                var currentStep = $(".ebstaStep:visible");
                $(currentStep).prev().fadeIn(500, function () {
                    InsertVideoIframeForStep();
                });
                $(currentStep).hide();
                ActionOnContinueButtonClick(true);
                $(this).blur();
                LogEvent("Onboarding Click - Back (" + $(currentStep).attr('stepname') + ")");
            });
        }
    }
    
    //Updates the progress bar during OnBoarding process
    function UpdateProgressBar(percentage) {
        $('.eb-progressbar').css('width', percentage+"%");
    }

    //Action on continue button click which helps decide what needs to happen on the click event for all steps
    function ActionOnContinueButtonClick(backButtonPress)
    {
        var currentStep = $(".ebstaStep:visible");
        if (currentStep) {
            var campaignMgrEnabled = abRules.EnableCampainMgr + '' === "1";
            var meetingSchedulerEnabled = abRules.EnableMeetingScheduler + '' === "1";

            var progressBarPercentage = 15;

            if (campaignMgrEnabled) {
                progressBarPercentage = progressBarPercentage - 5;
            }
            if (meetingSchedulerEnabled) {
                progressBarPercentage = progressBarPercentage - 5;
            }

            if (currentStep.length > 0)
            {
                var refreshSettings = false;
                var previewSettings = false;
                var confirmSettings = false;
                var finishSettings = false;
                var contineButtonText = "Continue";
                var backButtonText = "Back";

                //Hide and show certain buttons
                $('.eb-backbtn, .eb-continuebtn').show();
                $('.eb-setupcalendarsyncbtn, .eb-refreshtimezonesbtn, .eb-stopcalendarsyncbtn, ' +
                    '.eb-skipcalendarsyncbtn, .eb-setupcontactsyncbtn, .eb-syncnowbtn,' +
                    '.eb-setupanothercontactsync, .eb-skipsyncbtn, .eb-setupemailcampaignbtn, .eb-setupmeetingschedulerbtn').hide();//We only want to show these on specific pages
                $('.ebstaStep').find('.alert-danger').hide();
                $('.eb-skipsyncbtn').removeClass('eb-skipcontactsettings');

                if ($(currentStep).attr("stepname") === "Add Emails to Salesforce") {

                    UpdateProgressBar(progressBarPercentage);
                    UpdateOnBoardingTab(null, 2);
                    $('.eb-backbtn').hide();
                }
                else if ($(currentStep).attr("stepname") === "Sync Calendars") {

                    progressBarPercentage = 20;
                    // off set based on ab flag
                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 3;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 3;
                    }

                    UpdateProgressBar(progressBarPercentage);
                    UpdateOnBoardingTab(null, 3);
                    $('.eb-setupcalendarsyncbtn, .eb-skipsyncbtn').show();
                    $('.eb-continuebtn').hide();
                }
                //Pre-Requisite check for Calendar Sync
                else if ($(currentStep).attr("stepname") === "Calendar Sync Field Check")
                {
                    if (backButtonPress)
                    {
                        ShowEbstaStep(3)
                        $(currentStep).hide();
                    }
                    else
                    {
                        SendMessageToCheckFieldsForCalendarSync(false);
                    }
                    progressBarPercentage = 24;

                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 5;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 5;
                    }

                    UpdateProgressBar(progressBarPercentage);
                }
                //Preview Settings
                else if ($(currentStep).attr("stepname") === "Calendar Sync Settings")
                {
                    //Only call this function if this wasn't the back button being pressed
                    if (!backButtonPress) {
                        SendMessageToGetCalendarSyncSettings();
                    } else {
                        UpdateCalendarSettingsPage(isCalendarSyncLive);
                    }
                    contineButtonText = "Preview Settings"; //Change continue button text
                    previewSettings = true;
                    progressBarPercentage = 28;
                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 5;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 5;
                    }

                    UpdateProgressBar(progressBarPercentage);
                    ResetCalendarSettingsConfirmSettingsStep();
                    $('.eb-calsync-confirmprevieweventslist').remove();
                    $('.eb-refreshtimezonesbtn').fadeIn();
                    $('.eb-calsync-calendarlist').css('max-height','');
                    }
                //Confirm Settings
                else if ($(currentStep).attr("stepname") === "Calendar Sync Preview Events")
                {
                    if (backButtonPress)
                    {
                        ShowEbstaStep(3)
                        $(currentStep).hide();
                    }
                    else
                    {
                        contineButtonText = "Confirm Settings";
                        backButtonText = "Change Settings";
                        UpdateConfirmSettingsScreen();
                        confirmSettings = true;
                        progressBarPercentage = 33;
                        if (campaignMgrEnabled) {
                            progressBarPercentage = progressBarPercentage - 5;
                        }
                        if (meetingSchedulerEnabled) {
                            progressBarPercentage = progressBarPercentage - 5;
                        }

                        UpdateProgressBar(progressBarPercentage);
                        $('.alert-danger').hide();
                    }
                }
                else if ($(currentStep).attr("stepname") === "Calendar Sync Complete") {
                    progressBarPercentage = 35;
                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 5;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 5;
                    }

                    UpdateProgressBar(progressBarPercentage);
                    if (backButtonPress)
                    {
                        ShowEbstaStep(3)
                        $(currentStep).hide();
                    }
                }
                    //Contact Sync Start
                else if ($(currentStep).attr("stepname") === "Sync Contacts") {
                    $('.eb-skipsyncbtn,.eb-setupcontactsyncbtn').show();
                    $('.eb-continuebtn').hide();
                    progressBarPercentage = 37;
                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 5;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 5;
                    }

                    UpdateProgressBar(progressBarPercentage);
                    UpdateOnBoardingTab(null, 4);
                }
                else if ($(currentStep).attr("stepname") === "Contact Sync Listview Setup Screen") {
                    $('.eb-continuebtn').hide();
                    $('.eb-skipsyncbtn').addClass('eb-skipcontactsettings');
                    progressBarPercentage = 40;
                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 6;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 6;
                    }

                    UpdateProgressBar(progressBarPercentage);
                }
                else if ($(currentStep).attr("stepname") === "Contact Sync List Field Mapping") {
                    progressBarPercentage = 43;
                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 6;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 6;
                    }

                    UpdateProgressBar(progressBarPercentage);
                    contineButtonText = "Preview Settings";
                }
                else if ($(currentStep).attr("stepname") === "Contact Sync Preview Screen") {
                    if (backButtonPress) {
                        ShowEbstaStep(8)
                        $(currentStep).hide();
                    } else {
                        var listViewName = $('.ebstaStep10 .eb-contactsync-fieldmappingtablesparentdiv').attr('listviewname');
                        $('.eb-confirmcontactlistname b').text(listViewName);
                        GetContactsForPreviewSettingsScreen(listViewName);
                        $('#eb-wizard-onboarding .eb-continuebtn').hide();
                    }
                    progressBarPercentage = 46;
                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 6;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 6;
                    }

                    UpdateProgressBar(progressBarPercentage);
                    contineButtonText = "Confirm Settings";
                    backButtonText = "Change Settings";
                }
                else if ($(currentStep).attr("stepname") === "Contact Sync Complete") {
                    $('.eb-setupanothercontactsync').show();
                    if (backButtonPress) {
                        ShowEbstaStep(8)
                        $(currentStep).hide();
                    } else {
                        ConfirmContactSyncSettings($('#eb-wizard-onboarding .eb-contactsyncpreviewcontacts-table').attr('contactSyncRuleId'));
                        progressBarPercentage = 50;
                        if (campaignMgrEnabled) {
                            progressBarPercentage = progressBarPercentage - 6;
                        }
                        if (meetingSchedulerEnabled) {
                            progressBarPercentage = progressBarPercentage - 6;
                        }

                        UpdateProgressBar(progressBarPercentage);
                        UpdateOnBoardingTab(null, 4);
                    }
                }
                else if ($(currentStep).attr("stepname") === "Email Tracking") {
                    if (abRules.EnableContactSync + '' === "1") {
                        progressBarPercentage = 65;
                    } else {
                        progressBarPercentage = 48;
                    }
                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 12;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 12;
                    }

                    UpdateProgressBar(progressBarPercentage);
                    UpdateOnBoardingTab(null, 5);
                }
                else if ($(currentStep).attr("stepname") === "Salesforce Templates") {
                    if (abRules.EnableContactSync + '' === "1") {
                        progressBarPercentage = 75;
                    } else {
                        progressBarPercentage = 66;
                    }

                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 12;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 12;
                    }

                    UpdateProgressBar(progressBarPercentage);
                    UpdateOnBoardingTab(null, 6);
                }
                else if ($(currentStep).attr("stepname") === "Salesforce inside any web page") {
                    progressBarPercentage = 100;
                    if (campaignMgrEnabled) {
                        progressBarPercentage = progressBarPercentage - 15;
                    }
                    if (meetingSchedulerEnabled) {
                        progressBarPercentage = progressBarPercentage - 15;
                    }

                    UpdateProgressBar(progressBarPercentage);
                    if (abRules.EnableCampainMgr + '' === "0" && abRules.EnableMeetingScheduler + '' === "0") {
                        finishSettings = true;
                        contineButtonText = "Finish";
                    }
                    UpdateOnBoardingTab(null, 7);
                }
                else if ($(currentStep).attr("stepname") === "Email Cadences") {
                    $('.eb-setupemailcampaignbtn').show();
                    UpdateOnBoardingTab(null, 8);
                    if (abRules.EnableMeetingScheduler + '' === "0") {
                        UpdateProgressBar(100);
                        finishSettings = true;
                        contineButtonText = "Finish";
                    } else {
                        UpdateProgressBar(85);
                    }
                }
                else if ($(currentStep).attr("stepname") === "Meeting Scheduler") {
                    UpdateOnBoardingTab(null, 9);
                    $('.eb-setupmeetingschedulerbtn').show();
                    finishSettings = true;
                    contineButtonText = "Finish";
                    UpdateProgressBar(100);
                }

                //Update what action the continue button should look like and work
                SetListenerUpOnContinueButton(refreshSettings, previewSettings, confirmSettings, finishSettings);
                $('.eb-continuebtn').text(contineButtonText);
                $('.eb-backbtn').text(backButtonText);
            }
        }
    }

    //1st Step of for Wizard OnBoarding process
    function CreateOnBoardingStepAddEmailsToSalesforce(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep);
        contentDiv.setAttribute("stepname", "Add Emails to Salesforce");

        var header = CreateElement("h3", "page-header eb-page-header text-left", "Add Emails to Salesforce");
        var icon = CreateElement('i', 'fa fa-question-circle eb-onboarding-knowledgebase-icon', null, null);
        icon.title = 'Click to check out our knowledgebase article';
        AttachClickThroughListenerToElement(icon, 'https://ebstalimited.zendesk.com/hc/en-us/articles/115000096093-Add-Email-to-Salesforce');
        header.appendChild(icon);

        contentDiv.appendChild(header);

        contentDiv.appendChild(CreateElement("p","eb-textcontent", "Watch our video to see how you can quickly add emails to any Salesforce record at the click of a button.", null));

        //contentDiv.appendChild(CreateVideoFrameForOnBoarding("https://www.youtube.com/embed/ztpi5BZHRSc?rel=0&vq=hd720&showinfo=0", "eb-step1-video eb-onboard-video"));

        contentDiv.appendChild(CreateBr());

        return contentDiv;
    }

    //2nd Step of for Wizard OnBoarding process (Start of Calendar Sync)
    function CreateOnBoardingStepSyncCalendarsMainScreen(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep);
        contentDiv.setAttribute("stepname", "Sync Calendars");

        var header = CreateElement("h3", "page-header eb-page-header text-left", "Sync Calendars");
        var icon = CreateElement('i', 'fa fa-question-circle eb-onboarding-knowledgebase-icon', null, null);
        icon.title = 'Click to check out our knowledgebase article';
        AttachClickThroughListenerToElement(icon, 'https://ebstalimited.zendesk.com/hc/en-us/articles/229296068-Syncing-your-Google-Salesforce-Calendars');
        header.appendChild(icon);

        contentDiv.appendChild(header);

        var paragraphText = CreateElement("p", "eb-textcontent", "Sync events between your Google and Salesforce calendars. Click ", null);

        paragraphText.appendChild(CreateElement("b", null, "Calendar Sync Settings", null));
        paragraphText.appendChild(CreateTextNode(" to get started."));
        contentDiv.appendChild(paragraphText);

        //contentDiv.appendChild(CreateVideoFrameForOnBoarding("https://www.youtube.com/embed/LswIagiIWMA?rel=0&amp;showinfo=0", "eb-step2-video eb-onboard-video"));

        contentDiv.appendChild(CreateBr());

        return contentDiv;
    }
    
    //Creates the loading gif used for the onboarding process
    function CreateOnBoardingLoadingGif(text) {
        var loadingDiv = CreateDiv("", "ebNewLoader");

        var loadingGif = CreateDiv("", "ebNewLoaderGif");

        var divText = CreateDiv(text, "ebNewLoaderText text-center");

        loadingDiv.appendChild(loadingGif);

        loadingDiv.appendChild(divText);

        return loadingDiv;
    }

    //1st step for Calendar Sync Setup process - Pre-Requiste Check
    function CreateOnBoardingStepSyncCalendarFirstStep(ebstaStep)
    {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep + " ebstaStepCalendarSyncSetting");
        contentDiv.setAttribute("stepname", "Calendar Sync Field Check");
        
        contentDiv.appendChild(CreateOnBoardingLoadingGif("We're currently checking your Salesforce settings, please wait..."));

        //Errors above the table
        var errorText = CreateDiv("", "alert alert-danger text-center");
        errorText.appendChild(CreateElement("p","eb-textcontent", "We found the following errors that will stop the Calendar Sync process."));
        errorText.appendChild(CreateElement("p", "eb-textcontent", "Until these issues have been resolved, the sync process will not work."));
        $(errorText).hide();
        contentDiv.appendChild(errorText);

        //Error Table
        var fieldErrorsTable = CreateElement("table","table table-bordred table-striped eb-fieldErrorTable",null, null);
        var fieldErrorsTableBody = document.createElement("tbody");
        fieldErrorsTableBody.appendChild(CreateRowInFieldErrorTable("OwnerId"));
        fieldErrorsTableBody.appendChild(CreateRowInFieldErrorTable("StartDateTime"));
        fieldErrorsTableBody.appendChild(CreateRowInFieldErrorTable("EndDateTime"));
        fieldErrorsTableBody.appendChild(CreateRowInFieldErrorTable("Subject"));
        fieldErrorsTable.appendChild(fieldErrorsTableBody);
        $(fieldErrorsTable).hide();

        contentDiv.appendChild(fieldErrorsTable);

        //Help Text below the table
        var helpTextDiv = CreateDiv("", "alert alert-info text-center");
        var helpText = CreateElement("p", "eb-textcontent", null, null);
        helpText.appendChild(CreateTextNode("Please "));
        helpText.appendChild(CreateHyperlink("alert-link", "https://ebsta.zendesk.com/entries/81383489-How-do-I-customise-Salesforce-fields-shown-in-the-Salesforce-Event-Layout-", "click here"));
        helpText.appendChild(CreateTextNode(" to see our knowledge base article to help resolve the issue."));

        var helpText2 = CreateElement("p", "eb-textcontent", null, null);
        helpText2.appendChild(CreateTextNode("Press "));
        helpText2.appendChild(CreateElement("b", null, "Refresh Settings", null));
        helpText2.appendChild(CreateTextNode(" below once the changes have been made."));

        helpTextDiv.appendChild(helpText);
        helpTextDiv.appendChild(helpText2);
        $(helpTextDiv).hide();

        contentDiv.appendChild(helpTextDiv);

        return contentDiv;
    }
    
    //Creates a row in the table for the field errors
    function CreateRowInFieldErrorTable(fieldname) {
        var fieldErrorTableRow = CreateElement("tr","eb-fieldErrorTableRow", null, null);
        var fieldErrorTableColumn = CreateElement("td", "eb-fieldErrorTableCell", null, null);

        var textSpan = CreateSpan("You don't have access to the ","");
        textSpan.appendChild(CreateElement("b", null, fieldname, null));
        textSpan.appendChild(CreateSpan(" field against your Salesforce Event"));

        fieldErrorTableColumn.appendChild(textSpan);

        $(fieldErrorTableColumn).attr('sffieldname', fieldname);

        $(fieldErrorTableColumn).prepend(CreateSpan("", "eb-onboarding-calsyncprerequisiteerror ebsta-glyphicon ebsta-glyphicon-cross"));

        fieldErrorTableRow.appendChild(fieldErrorTableColumn);

        return fieldErrorTableRow;
    }

    //2ND step for Calendar Sync Setup process where a user will see the global settings and a list of thier calendars
    function CreateOnBoardingStepSyncCalendarSecondStep(ebstaStep) {

        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep + " ebstaStepCalendarSyncSetting");
        contentDiv.setAttribute("stepname", "Calendar Sync Settings");

        var headerContent = CreateDiv("", "eb-onboarding-settingsheadercontent");

        headerContent.appendChild(CreateElement("h3", "page-header eb-page-header text-left", "Pick your Calendar Sync Settings"));

        var headerDiv = CreateDiv("", "eb-onboarding-calendarsettingsheader");

        var leftColumnDiv = CreateDiv("", "col-xs-6");
        leftColumnDiv.appendChild(CreateElement("h5", "eb-onboarding-currentstatusheader text-left", "Current Sync Status:"));
        leftColumnDiv.appendChild(CreateElement("h5", "eb-onboarding-currentstatustext text-left", "Loading..."));

        //Recurring Events option
        var recurringEventDiv = CreateDiv("", "eb-onboarding-syncrecurringeventsoption");
        recurringEventDiv.appendChild(CreateElement("h5", "text-left", "Do Not Sync Recurring Events:"));
        var reucrringInputDiv = CreateElement("input", null, null, null);
        reucrringInputDiv.setAttribute('type', 'checkbox');
        recurringEventDiv.appendChild(reucrringInputDiv);
        leftColumnDiv.appendChild(recurringEventDiv);

        headerDiv.appendChild(leftColumnDiv);

        var rightColumnDiv = CreateDiv("", "col-xs-6");
        rightColumnDiv.appendChild(CreateElement("h5", "eb-onboarding-lastsyncheader text-left", "Last Sync:"));
        rightColumnDiv.appendChild(CreateElement("h5", "eb-onboarding-lastsynctext text-left", "Loading..."));

        //Private Events option
        var privateEventDiv = CreateDiv("", "eb-onboarding-syncprivateeventsoption");
        privateEventDiv.appendChild(CreateElement("h5", "text-left", "Do Not Sync Private Events:"));
        var privateInputDiv = CreateElement("input", null, null, null);
        privateInputDiv.setAttribute('type', 'checkbox');
        privateEventDiv.appendChild(privateInputDiv);
        rightColumnDiv.appendChild(privateEventDiv);

        headerDiv.appendChild(rightColumnDiv);

        headerContent.appendChild(headerDiv);
        contentDiv.appendChild(headerContent);

        var errorText = CreateDiv("You must select at least 1 calendar to sync.", "alert alert-danger");
        $(errorText).hide();
        headerContent.appendChild(errorText);

        contentDiv.appendChild(CreateListOfCalendersDiv());

        contentDiv.appendChild(CreateElement("section", null, null, "eb-onboarding-calendarsyncadminmodemessage"));

        return contentDiv;
    }

    //Last page where user gets to see a preview of events that will Sync
    function CreateOnBoardingStepSyncCalendarConfirmSettings(ebstaStep)
    {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep + " ebstaStepCalendarSyncSetting");
        contentDiv.setAttribute("stepname", "Calendar Sync Preview Events");

        contentDiv.appendChild(CreateElement("h3", "page-header eb-page-header text-left", "Calendar Sync Preview"));

        var leftColumnDiv = CreateDiv("", "col-xs-6 eb-calsync-confirmcalendars");
        leftColumnDiv.appendChild(CreateElement("h4", "eb-calsync-globalheading text-left", "Select Calendar"));

        var rightColumnDiv = CreateDiv("", "col-xs-6 eb-calsync-confirmsyncdirection");

        var dropDownCalendarSelectDiv = CreateDiv("", "eb-calsync-confirmcalendarsselectdiv");

        var dropDownCalendarSelect = CreateElement("select","form-control", null, null);

        CreateListenerOnBoardingConfirmSettingsSelectCalendar(dropDownCalendarSelect);

        dropDownCalendarSelectDiv.appendChild(dropDownCalendarSelect);
        leftColumnDiv.appendChild(dropDownCalendarSelectDiv);

        contentDiv.appendChild(leftColumnDiv);
        contentDiv.appendChild(rightColumnDiv);

        contentDiv.appendChild(CreateElement("p", "eb-calsync-confirmprevieweventtext", "Ebsta will sync the following Calendar Events."));

        var previewEventDiv = CreateDiv("", "eb-calsync-confirmpreviewevents");

        previewEventDiv.appendChild(CreateOnBoardingLoadingGif("We're just retrieving a preview of the events we will sync, please wait..."));

        var errorText = CreateDiv("There are currently no Events to preview the sync to / from this Calendar.", "alert alert-info text-center");
        $(errorText).hide();
        previewEventDiv.appendChild(errorText);

        contentDiv.appendChild(previewEventDiv);

        return contentDiv;
    }

    //Completion Page when calendar sync has succesfully been setup
    function CreateOnBoardingStepSyncCalendarCompletion(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep + " ebstaStepCalendarSyncSetting");
        contentDiv.setAttribute("stepname", "Calendar Sync Complete");

        var tickDiv = CreateDiv("", "ebsta-alert ebsta-alert-success");

        tickDiv.appendChild(CreateElement("p", "eb-paragraph text-left ebsta-glyphicon ebsta-glyphicon-tick", null, null));

        var text = "Congratulations! You've successfully setup Ebsta's Calendar Sync.";

        tickDiv.appendChild(CreateElement("p", "eb-textcontent text-left", text, null));

        var text2 = "Ebsta will now sync your Google and Salesforce Calendars once every hour."

        tickDiv.appendChild(CreateElement("p", "eb-textcontent text-left", text2, null));

        contentDiv.appendChild(tickDiv);

        return contentDiv;
    }


/**
 * Creates the first step for Contact Sync process
 * @param {int} ebstaStep - The  number of this screen in the sequence for the wizard
 * @returns {DOMElement} - The DOM we need to render
 */
function CreateOnBoardingStepContactSyncIntroScreen(ebstaStep) {
    var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep);
    contentDiv.setAttribute("stepname", "Sync Contacts");

    var header = CreateElement("h3", "page-header eb-page-header text-left", "Sync Contacts");
    var icon = CreateElement('i', 'fa fa-question-circle eb-onboarding-knowledgebase-icon', null, null);
    icon.title = 'Click to check out our knowledgebase article';
    AttachClickThroughListenerToElement(icon, 'https://ebstalimited.zendesk.com/hc/en-us/articles/231719588-Contact-Import');
    header.appendChild(icon);

    contentDiv.appendChild(header);

    var paragraphText = CreateElement("p", "eb-textcontent", "Sync your contacts from Salesforce to your Google Contacts. Click ", null);

    paragraphText.appendChild(CreateElement("b", null, "Contact Sync Settings", null));
    paragraphText.appendChild(CreateTextNode(" to get started."));
    contentDiv.appendChild(paragraphText);

    contentDiv.appendChild(CreateImg(chrome.extension.getURL("imgs/contactsyncscreengrab.png"), "Contact Sync", "eb-onboard-screenshot"));

    contentDiv.appendChild(CreateBr());

    return contentDiv;
}

    //Field Mapping screen for Contact Sync
    function CreateOnBoardingStepContactSyncFieldMapping(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep + " ebstaStepContactSyncSetting");
        contentDiv.setAttribute("stepname", "Contact Sync List Field Mapping");

        contentDiv.appendChild(CreateElement("h3", "page-header eb-page-header text-left", "Choose your Field Mappings"));

        var paragraphText = CreateElement("p", "eb-textcontent", "Please select which Salesforce Contact fields you wish to sync to Google.", null);
        contentDiv.appendChild(paragraphText);

        //contentDiv.appendChild(CreateOnBoardingLoadingGif("We're currently checking your Salesforce settings, please wait..."));

        var fieldMappingTable = CreateDiv("", "eb-contactsync-fieldmappings");
        fieldMappingTable.appendChild(CreateTablesForFieldMappingsContactSync());

        contentDiv.appendChild(fieldMappingTable);

        return contentDiv;
    }

    //Preview screen for Contact Sync
    function CreateOnBoardingStepContactSyncPreview(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep + " ebstaStepContactSyncSetting");
        contentDiv.setAttribute("stepname", "Contact Sync Preview Screen");

        contentDiv.appendChild(CreateElement("h3", "page-header eb-page-header text-left", "Contact Sync Preview"));

        var paragraphText = CreateElement("p", "eb-textcontent eb-confirmcontactlistname", "Ebsta will sync the following Contacts from your ", null);

        paragraphText.appendChild(CreateElement("b", null, null, null));
        paragraphText.appendChild(CreateTextNode(" list in Salesforce."));
        contentDiv.appendChild(paragraphText);

        contentDiv.appendChild(CreateDiv("Ebsta found no contacts for this list.", "alert alert-danger has-feedback text-center"));
        contentDiv.appendChild(CreateDiv("", "alert alert-warning has-warning has-feedback text-center"));

        contentDiv.appendChild(CreateOnBoardingLoadingGif("We're just retrieving a preview of the Contacts we will sync, please wait..."));

        var previewContactTable = CreateDiv("", "eb-contactsync-previewcontact");
        previewContactTable.appendChild(CreateTableForContactSyncPreviewSettings());
        contentDiv.appendChild(previewContactTable);

        return contentDiv;
    }

    //Completion Page when contact sync has succesfully been setup
    function CreateOnBoardingStepContactSyncCompletion(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep);
        contentDiv.setAttribute("stepname", "Contact Sync Complete");

        var tickDiv = CreateDiv("", "ebsta-alert ebsta-alert-success");

        tickDiv.appendChild(CreateElement("p", "eb-paragraph text-left ebsta-glyphicon ebsta-glyphicon-tick", null, null));

        var text = "Congratulations! You've successfully setup Ebsta's Contact Sync.";

        tickDiv.appendChild(CreateElement("p", "eb-textcontent text-left", text, null));

        var text2 = "Ebsta will now sync your Salesforce contacts once every hour."

        tickDiv.appendChild(CreateElement("p", "eb-textcontent text-left", text2, null));

        contentDiv.appendChild(tickDiv);

        return contentDiv;
    }

    //SalesforceTemplates for OnBoarding Process
    function CreateOnBoardingStepSalesforceEmailTracking(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep);
        contentDiv.setAttribute("stepname", "Email Tracking");

        var header = CreateElement("h3", "page-header eb-page-header text-left", "Email Tracking");
        var icon = CreateElement('i', 'fa fa-question-circle eb-onboarding-knowledgebase-icon', null, null);
        icon.title = 'Click to check out our knowledgebase article';
        AttachClickThroughListenerToElement(icon, 'https://ebstalimited.zendesk.com/hc/en-us/articles/229295888-Email-Open-and-Link-Tracking');
        header.appendChild(icon);

        contentDiv.appendChild(header);

        contentDiv.appendChild(CreateElement("p","eb-textcontent", "See when your emails are opened so you can better gauge how and when you should follow-up with your Contacts.", null));

        //contentDiv.appendChild(CreateVideoFrameForOnBoarding("https://www.youtube.com/embed/BYnoDmRXtso?rel=0&amp;showinfo=0", "eb-step8-video eb-onboard-video"));

        contentDiv.appendChild(CreateBr());

        return contentDiv;
    }

    //Email Tracking section of OnBoarding Process
    function CreateOnBoardingStepSalesforceTemplates(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep);
        contentDiv.setAttribute("stepname", "Salesforce Templates");

        var header = CreateElement("h3", "page-header eb-page-header text-left", "Salesforce Templates");
        var icon = CreateElement('i', 'fa fa-question-circle eb-onboarding-knowledgebase-icon', null, null);
        icon.title = 'Click to check out our knowledgebase article';
        AttachClickThroughListenerToElement(icon, 'https://ebstalimited.zendesk.com/hc/en-us/articles/231603647-Using-Salesforce-Templates');
        header.appendChild(icon);

        contentDiv.appendChild(header);

        contentDiv.appendChild(CreateElement("p", "eb-textcontent", "Ebsta gives you access to all your Salesforce Templates from within Gmail."));

        //contentDiv.appendChild(CreateVideoFrameForOnBoarding("https://www.youtube.com/embed/R7c1-xek7FM?rel=0&amp;showinfo=0", "eb-step9-video eb-onboard-video"));
            
        contentDiv.appendChild(CreateBr());

        return contentDiv;
    }

    //SalesforceTemplates for OnBoarding Process
    function CreateOnBoardingStepSalesforceInsideGmail(ebstaStep) {
        var contentDiv = CreateDiv("", "ebstaStep ebstaStep" + ebstaStep);
        contentDiv.setAttribute("stepname", "Salesforce inside any web page");

        var header = CreateElement("h3", "page-header eb-page-header text-left", "Salesforce inside any web page");
        var icon = CreateElement('i', 'fa fa-question-circle eb-onboarding-knowledgebase-icon', null, null);
        icon.title = 'Click to check out our knowledgebase article';
        AttachClickThroughListenerToElement(icon, 'https://ebstalimited.zendesk.com/hc/en-us/articles/231481667-Search-edit-update-Salesforce-Overview');
        header.appendChild(icon);
        contentDiv.appendChild(header);

        contentDiv.appendChild(CreateElement("p", "eb-textcontent", "Watch our video to see how Ebsta’s bubble allows you to view, edit and update Salesforce from any web page."));

        //contentDiv.appendChild(CreateVideoFrameForOnBoarding("https://www.youtube.com/embed/IV_3Z3SmqTc?rel=0&showinfo=0", "eb-step10-video eb-onboard-video wistia_embed"));

        contentDiv.appendChild(CreateBr());

        return contentDiv;
    }
    
    //Helper class to create the the options in Global Setting panel of Calendar Sync Setup
    function CreateOptionForGlobalSettingsPanel(text, checkBoxClassName) {
        var columnDiv = CreateDiv("", "col-sm-6 text-left");

        var checkBoxDiv = CreateDiv("", "checkbox");

        var checkBoxLabel = CreateElement("label", "", text, null);

        var inputBox = CreateElement("input", checkBoxClassName, null, null);
        inputBox.type = "checkbox";

        $(checkBoxLabel).prepend(inputBox);
        checkBoxDiv.appendChild(checkBoxLabel);
        columnDiv.appendChild(checkBoxDiv);

        return columnDiv;
    }

    //Creates the Calendar List Panel ready for the calendars to be appended when we recieve the response back
    function CreateListOfCalendersDiv() {
        var mainContentPanel = CreateDiv("", "panel panel-default");

        var subMainContentPanel = CreateDiv("", "panel-body");

        subMainContentPanel.appendChild(CreateElement("h4", "eb-calsync-calendarlistheader text-left", "List Of Calendars"));

        subMainContentPanel.appendChild(CreateOnBoardingLoadingGif("We're currently retrieving your calendars from Google, please wait.."));

        subMainContentPanel.appendChild(CreateRow("eb-calsync-calendarlist"));

        mainContentPanel.appendChild(subMainContentPanel);

        return mainContentPanel;
    }

    //Creates a new div for a Calendar in the onboarding process
    function CreateCalendarEntryInCalendarSyncSetup(calendarJson)
    {
        var calendarObject = GetValuesFromCalendarEntry(calendarJson);

        var mainCalendarDiv = CreateDiv("", "eb-calsync-calendarlist-calendar");
        mainCalendarDiv.setAttribute("calendarid", calendarObject.id);
        mainCalendarDiv.setAttribute("isprimary", calendarObject.IsPrimary);
        mainCalendarDiv.setAttribute("accessrole", calendarObject.AccessRole);
        mainCalendarDiv.setAttribute("calendartitle", calendarObject.GoogleCalendarSummaryName);

        var calendarSettingsDiv = CreateDiv("", "eb-calsync-calendarlist-calendarsettings");

        var leftColumnDiv = CreateDiv("", "col-xs-54");
        var rightColumnDiv = CreateDiv("", "col-xs-46");

        //Calendar Title
        var pDiv = CreateElement("p", null, null, null);
        var boldDiv = CreateElement("b", null, calendarObject.GoogleCalendarSummaryName, null);
        pDiv.appendChild(boldDiv);
        leftColumnDiv.appendChild(pDiv);
        
        //Text Box options for sync directions
        leftColumnDiv.appendChild(CreateOptionsForCalendar("Sync " + calendarObject.GoogleCalendarSummaryName+ " events to Salesforce", "eb-calsync-calendarlist-calendarsettingssynctosf", calendarObject.isSyncingToSF));
        leftColumnDiv.appendChild(CreateOptionsForCalendar("Sync Salesforce events to " + calendarObject.GoogleCalendarSummaryName, "eb-calsync-calendarlist-calendarsettingssynctogoogle", calendarObject.isSyncingToGoogle));

        calendarSettingsDiv.appendChild(leftColumnDiv);

        //TimeZone Information
        if (!calendarObject.TimeZonesMatch) {
            var warningDiv = CreateDiv("", "alert alert-warning has-warning has-feedback");

            var warningSign = CreateHyperlink("ebsta-glyphicon ebsta-glyphicon-question-mark", "https://ebsta.zendesk.com/entries/51738855-How-do-I-update-my-timezone-settings-in-Salesforce-and-or-Google-Calendar-", "");
            warningSign.setAttribute("target", "_blank");
            warningDiv.appendChild(warningSign);

            warningDiv.appendChild(CreateElement("p", null, "Salesforce TimeZone: " + calendarObject.SFtimeZone));
            warningDiv.appendChild(CreateElement("p", null, "Google TimeZone: " + calendarObject.GoogleTimeZone));
            rightColumnDiv.appendChild(warningDiv);
            calendarSettingsDiv.appendChild(rightColumnDiv);
        } else {
            $(leftColumnDiv).css("width", "100%");
        }
        
        mainCalendarDiv.appendChild(calendarSettingsDiv);

        $('.eb-calsync-calendarlist').append(mainCalendarDiv);
    }
    
    //Returns an object with values needed for the calendar display in Calendar Sync Setup
    function GetValuesFromCalendarEntry(calendar){

        var rtnObject = new Object();

        rtnObject.id = "";
        rtnObject.GoogleCalendarSummaryName = "";
        rtnObject.isSyncingToSF = false;
        rtnObject.isSyncingToGoogle = false;
        rtnObject.TimeZonesMatch = true;
        rtnObject.GoogleCalendarTimeZone = "";
        rtnObject.IsPrimary = false;
        rtnObject.AccessRole = "";
        rtnObject.SFtimeZone = "";
        rtnObject.GoogleTimeZone = "";

        if(calendar.id){
            rtnObject.id = calendar.id;
        }

        if (calendar.summary) {
            rtnObject.GoogleCalendarSummaryName = calendar.summary;
        }

        if (calendar.isSyncingToSalesforce != null) {
            rtnObject.isSyncingToSF = calendar.isSyncingToSalesforce;
        }

        if (calendar.isSyncingToGoogle != null) {
            rtnObject.isSyncingToGoogle = calendar.isSyncingToGoogle;
        }

        if (calendar.timeZonesMatch != null) {
            rtnObject.TimeZonesMatch = calendar.timeZonesMatch;
        }

        if (calendar.timeZone) {
            rtnObject.GoogleCalendarTimeZone = calendar.timeZone;
        }

        if (calendar.primary != null) {
            rtnObject.IsPrimary = calendar.primary;
        }

        if (calendar.accessRole != null) {
            rtnObject.AccessRole = calendar.accessRole;
        }

        if (calendar.SFtimeZone != null) {
            rtnObject.SFtimeZone = calendar.SFtimeZone;
        }

        if (calendar.timeZone != null) {
            rtnObject.GoogleTimeZone = calendar.timeZone;
        }

        return rtnObject;
    }

    //Helper class to create the the options for the input boxes for each calendar
    function CreateOptionsForCalendar(text, checkBoxClassName, checkTextBox) {

        var checkBoxDiv = CreateDiv("", "checkbox");

        var checkBoxLabel = CreateElement("label", "", text, null);

        var inputBox = CreateElement("input", checkBoxClassName, null, null);
        inputBox.type = "checkbox";

        if (checkTextBox) {
            $(inputBox).prop("checked", checkTextBox);
        }

        $(checkBoxLabel).prepend(inputBox);
        checkBoxDiv.appendChild(checkBoxLabel);

        return checkBoxDiv;
    }
    
    //Attach a listener to the select box on the confirm settings page of the onboarding process so the table updates when a user changes the selection
    function CreateListenerOnBoardingConfirmSettingsSelectCalendar(selectDiv) {
        $(selectDiv).off();

        $(selectDiv).on("change", function ()
        {
            //Find the selected calendarid from the select box
            var selectedVal = $(this).find('option:selected').attr("calendarid"); 
            
            //Hide all values beside the select box initially
            $('.eb-calsync-confirmsyncdirection').find("p").hide();
            
            //Find the text beside the select box to show the correct value
            $('.eb-calsync-confirmsyncdirection > p').filter(function (i) {
                return $(this).attr("googlecalendarname") == selectedVal;
            }).each(function ()
            {
                $(this).show();

                //hide all values in the preview events table
                $(".eb-confirmpreviewevents-table").find("p").hide();

                // Get the sync direction
                // 0 - No Sync, 1 - Both Ways, 2 - Google To Sf, 3 - SF To Google
                var syncdirection = $(this).attr("syncdirection");

                //Find the values associated with the selected direction in the preview events table and show them
                if (syncdirection)
                {
                    //Both Ways - show all
                    if (syncdirection == 1) {
                        $(".eb-confirmpreviewevents-table").find("td > p").filter(function (i) {
                            return $(this).attr("googlecalendarname") == selectedVal || $(this).attr("syncdirection") == 1;
                        }).fadeIn(100);
                    }
                     // Google to SF
                    else if (syncdirection == 2) {
                        $(".eb-confirmpreviewevents-table").find("td > p").filter(function (i) {
                             return $(this).attr("googlecalendarname") == selectedVal;
                        }).fadeIn(100);
                    }
                        // SF To Google
                    else if (syncdirection == 3) {
                        $(".eb-confirmpreviewevents-table").find("p[syncdirection='1']").fadeIn(100);
                    }
                }
            });

            //Need to show them first in order for them to be visible when looking for them again
            $('.eb-confirmpreviewevents-table').show();
            $('.eb-confirmpreviewevents-table tr').show();

            //Hide days with no active events
            $('.eb-confirmpreviewevents-table > tbody').find('tr').each(function ()
            {
                if ($(this).find("p:visible").length == 0) {
                    $(this).hide();
                }
            });

            //Shows no events error if there are no events
            if ($('.eb-confirmpreviewevents-table').find("tr:visible").length == 1)
            {
                $('.eb-calsync-confirmpreviewevents').find('.alert').fadeIn();
                $('.eb-confirmpreviewevents-table').hide();
            }
            else {
                $('.eb-calsync-confirmpreviewevents').find('.alert').hide();
                $('.eb-confirmpreviewevents-table').fadeIn(100);
            }
        });
    }
    
    //Updates the Confirm Settings screen with information taking from the preview settings screen
    function UpdateConfirmSettingsScreen()
    {
        //Clear Previous Calendars if they were set
        $('.eb-calsync-confirmcalendars').find('h5,p,select').empty();
        $('.eb-calsync-confirmsyncdirection').empty();

        if (global_calendarArray) {
            for (var i = 0 ; i < global_calendarArray.length; i++)
            {
                var calendar = global_calendarArray[i];

                var syncingDirectionText = "This Calendar is set to sync";
                var syncdirection = 0; //0 - No Sync, 1 - Both Way, 2 - Google To Sf, 3 - SF To Google

                if (calendar.syncToSf && calendar.syncToGoogle) {
                    syncingDirectionText += " both ways.";
                    syncdirection = 1;
                } else if (calendar.syncToSf) {
                    syncingDirectionText += " to Salesforce only.";
                    syncdirection = 2
                } else if (calendar.syncToGoogle) {
                    syncingDirectionText += " to Google only.";
                    syncdirection = 3
                }

                //Create the sync direction text and set the attribute to the calendar title so we can show the chosen one from the select box
                var syncDirectionParagraph = CreateElement("p", "eb-calsync-calendarsyncsdirection", syncingDirectionText, null);
                syncDirectionParagraph.setAttribute("googleCalendarName", calendar.calendarId);
                syncDirectionParagraph.setAttribute("syncdirection", syncdirection);
                $('.eb-calsync-confirmsyncdirection').append(syncDirectionParagraph);

                //Only put the calendar into the list if it was chosen at the preview stage
                if (syncdirection != 0) {
                    var option = CreateElement("option", null, calendar.calendarTitle, null);
                    option.setAttribute("value", calendar.calendarTitle);
                    option.setAttribute("calendarid", calendar.calendarId);
                    $('.eb-calsync-confirmcalendarsselectdiv select').append(option);
                }
                
            }
        }

        $('.eb-calsync-confirmcalendars select').trigger("change");
    }
    
    //Creates the main table for the preview events display
    function CreateTableForPreviewEvents()
    {
        var tableDiv = CreateElement("table", "table table-striped table-bordered table-condensed eb-confirmpreviewevents-table text-left", null, null);

        //Table Header
        var tableHeadDiv = CreateElement("thead", null, null, null);
        var headerRow = CreateElement("tr", null, null, null);
        headerRow.appendChild(CreateElement("th", "eb-confirmpreviewevents-tabledateheader", "Date", null));
        headerRow.appendChild(CreateElement("th", null, "Google Calendar to Salesforce", null));
        headerRow.appendChild(CreateElement("th", null, "Salesforce to Google Calendar", null));
        tableHeadDiv.appendChild(headerRow);
        tableDiv.appendChild(tableHeadDiv);

        tableDiv.appendChild(document.createElement("tbody"));
        $(tableDiv).hide();
        return tableDiv;
    }
    
    //Render the preview of events we would sync over
    function RenderPreviewEventsCalendarSyncSetup(json, dateCheck)
    {
        //console.time("RenderPreviewEventsCalendarSyncSetup");
        if (dateCheck == global_dateCheckForPreviewEventRequestArray[0])
        {
            var parentDiv = CreateDiv("", "eb-calsync-confirmprevieweventslist");
            parentDiv.appendChild(CreateTableForPreviewEvents());

            $('.eb-calsync-confirmpreviewevents').append(parentDiv);

            if (json) {
                if (json.d) {
                    if (json.d.ListOfSortedEvents) {
                        InsertEventsIntoPreviewTable(json.d.ListOfSortedEvents);
                    }
                }
            }

            setTimeout(function () {
                $('.ebstaStep:visible').find('.ebNewLoader').hide();
                $('.eb-calsync-confirmcalendars select').trigger("change");
            }, 1000);

            //Reset array now that we've displayed the events
            global_dateCheckForPreviewEventRequestArray = new Array();
        }

        //console.timeEnd("RenderPreviewEventsCalendarSyncSetup");
    }
    
    //Returns the date in a readable format for preview events
    function ReturnDateForPreviewEvents(date) {

        var timestring = "";

        //Change 0 hours to 12 for display purposes
        if (date.getHours() == 0)
        {
            timestring += "12";
        }
        //Change to 12 hours format
        else if (date.getHours() > 12)
        {
            timestring += date.getHours() - 12;
        }
        else
        {
            timestring += date.getHours();
        }

        if (date.getMinutes() < 10) {
            timestring += ":0" + date.getMinutes();
        } else {
            timestring += ":" + date.getMinutes();
        }

        if (date.getHours() < 12) {
            timestring += "am";
        } else {
            timestring += "pm";
        }

        return timestring;
    }

    //Inserts events into the preview table
    function InsertEventsIntoPreviewTable(events)
    {
        //console.time("InsertEventsIntoPreviewTable");
        //Variables for mapping dates and times
        var g_days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
        var g_months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
        
        var lastDateInputted = new Date();

        for (var i = 0; i < events.length; i++)
        {
            //If this is a private event lets only disaply this if the user passed the IsActive field check
            if (events[i].Private && !global_doesUserHavePrivateEventField)
            {
                continue;
            }

            //Helps make sure we pick up the recurring event series creator rather than each instance of the event as Salesforce create an individual event within a recurring events
            var parentRecurringEvent = false;
            if (events[i].RecurringEvent) {
                if (events[i].RecurringEvent) {
                    parentRecurringEvent = true;
                    if (events[i].Engine == 1) {
                        if (events[i].RecurringEventId != events[i].SalesforceId) {
                            parentRecurringEvent = false;
                        }
                    }
                }
            }

            if (!events[i].DeletedEvent && (parentRecurringEvent || events[i].RecurringEventId == null))
            {
                var subject = "";
                var fromEngine = "";
                var toEngine = "";
                var privateEvent = "no";
                var recurringEvent = "no";
                var status = "What here?!";

                if (events[i].Subject) {
                    subject = events[i].Subject;
                }

                var startDate = new Date();
                var endDate = new Date();

                //If this is an AllDayEvent just look for the startdate
                if (events[i].AllDayEvent)
                {
                    if (events[i].StartDate) {
                        //Date format: Date(1430348400000+0100)
                        //Regex to extract the numbers out to parse into time
                        var re = /-?\d+/;
                        var m = re.exec(events[i].StartDate);
                        if (m.length > 0) {
                            startDate = new Date(parseInt(m[0]));
                        }
                    }
                }
                else
                {
                    if (events[i].StartRaw) {
                        startDate = new Date(events[i].StartRaw);
                    }

                    if (events[i].EndRaw) {
                        endDate = new Date(events[i].EndRaw);
                    }
                }

                //Beautify the date string so we can display the date
                var startDateString = g_days[startDate.getDay()] + " " + startDate.getDate() + " " + g_months[startDate.getMonth()] + " " + startDate.getFullYear();
                var endDateString = g_days[endDate.getDay()] + " " + endDate.getDate() + " " + g_months[endDate.getMonth()] + " " + endDate.getFullYear();

                var startTime = "";
                var endTime = "";

                if (!events[i].AllDayEvent)
                {
                    startTime = ReturnDateForPreviewEvents(startDate);
                    endTime = ReturnDateForPreviewEvents(endDate);
                }

                var dateRow;

                //Find if we've created a row for this date already
                if ($('.eb-confirmpreviewevents-table').find("tr[eventstartdate='" + startDateString + "']").length == 0) {
                    dateRow = CreateElement("tr", null, null, null);

                    $(dateRow).append(CreateElement("td", "eb-confirmpreviewevents-tabledatecell", startDateString, null)).hide().fadeIn(100);
                    $(dateRow).append(CreateElement("td", "eb-confirmpreviewevents-tablegoogletosfcell", null, null)).hide().fadeIn(100);
                    $(dateRow).append(CreateElement("td", "eb-confirmpreviewevents-tablesftogooglecell", null, null)).hide().fadeIn(100);

                    dateRow.setAttribute("eventstartdate", startDateString);
                    $('.eb-confirmpreviewevents-table > tbody').append(dateRow);
                }
                else {
                    dateRow = $('.eb-confirmpreviewevents-table').find("tr[eventstartdate='" + startDateString + "']")[0];
                }

                if (events[i].Engine)
                {
                    var subjectAndTimeDiv;

                    if (startTime.length > 0) {
                        subjectAndTimeDiv = CreateElement("p", null, CheckStringLength(subject, 20) + " (" + startTime + " - " + endTime + ")");
                    } else {
                        subjectAndTimeDiv = CreateElement("p", null, CheckStringLength(subject, 20) + " (All Day Event)");
                    }

                    if (events[i].RecurringEvent) {
                        var recurringEventIcon = CreateSpan("", "ebsta-glyphicon ebsta-glyphicon-recurringevent");
                        $(subjectAndTimeDiv).append(recurringEventIcon);
                    }

                    if (events[i].Private) {
                        var privateEventIcon = CreateSpan("", "ebsta-glyphicon ebsta-glyphicon-privateevent");
                        $(subjectAndTimeDiv).append(privateEventIcon);
                    }

                    if (events[i].Engine == 1) {
                        fromEngine = "Salesforce";
                        toEngine = "Google";
                        $(dateRow).find('.eb-confirmpreviewevents-tablesftogooglecell').append(subjectAndTimeDiv);
                    }
                    else if (events[i].Engine == 2) {
                        fromEngine = "Google";
                        toEngine = "Salesforce";
                        subjectAndTimeDiv.setAttribute("googlecalendarname", events[i].GoogleCalendarName);
                        $(dateRow).find('.eb-confirmpreviewevents-tablegoogletosfcell').append(subjectAndTimeDiv);
                    }

                    subjectAndTimeDiv.setAttribute("syncdirection", events[i].Engine);
                }

                $('.eb-confirmpreviewevents-table > tbody').append(dateRow);
            }
        }

        //console.timeEnd("InsertEventsIntoPreviewTable");
    }
    
    //Gets all the details from the calendar settings page and saves them into the DB to potential be confirmed after
    function SavePreviewSettingsForCalendarySync() {
        //Find the global settings
        global_syncPrivateEvents = $('#eb-wizard-onboarding .eb-onboarding-syncprivateeventsoption input').prop('checked');
        global_syncRecurringEvents = $('#eb-wizard-onboarding .eb-onboarding-syncrecurringeventsoption input').prop('checked');
        global_syncColourEvents = "Not Supported Just Yet";

        //Get Calendar Details
        global_calendarArray = new Array();

        $('.eb-calsync-calendarlist-calendar').each(function () {
            var calendarObject = new Object();

            calendarObject.calendarId = $(this).attr('calendarid');
            calendarObject.isPrimary = $(this).attr('isprimary');
            calendarObject.accessRole = $(this).attr('accessrole');
            calendarObject.calendarTitle = $(this).attr('calendartitle');

            calendarObject.syncToSf = $(this).find('.eb-calsync-calendarlist-calendarsettingssynctosf').prop('checked');
            calendarObject.syncToGoogle = $(this).find('.eb-calsync-calendarlist-calendarsettingssynctogoogle').prop('checked');

            global_calendarArray.push(calendarObject);
        });

        //keep a track of the time of when user selected confirm settings do we can match it on the way back (helps prevent mutliple clicks of preview settings and multiple results returning)
        var nowTime = new Date().getTime();
        global_dateCheckForPreviewEventRequestArray.unshift(nowTime);

        //Send the message to save the preview settings
        chrome.extension.sendMessage({
            MessageType: "CalSyncSavePreviewSettings",
            GmailAccount: GetGmailUsersEmailAddress(),
            Calendars: global_calendarArray,
            PrivateEvents: global_syncPrivateEvents,
            RecurringEvents: global_syncRecurringEvents,
            ColourEvents: global_syncColourEvents,
            DateCheck: nowTime,
            IsOldAuthTokenBeingUsed: global_usingOldAuthToken
        });
    }

    
    //Removes all iframe video elements from onboarding process
    function RemoveVideoIframesFromOnboardingProcess() {
        //console.time("RemoveVideoIframesFromOnboardingProcess");
        $("#eb-wizard-onboarding").find(".ebstaStep").find('.eb-onboard-video').remove();
        //console.timeEnd("RemoveVideoIframesFromOnboardingProcess");
    }
    
    //Looks at the current step of the onboarding process and then inserts the video content
    function InsertVideoIframeForStep() {
        var currentStepVisible = $('#eb-wizard-onboarding').find('.ebstaStep:visible');

        if ($(currentStepVisible).find('.eb-onboard-video').length === 0) {
            var stepname = $(currentStepVisible).attr('stepname');
            if (stepname) {
                var videoSrc = "";

                switch (stepname) {
                case "Add Emails to Salesforce":
                    videoSrc = "https://www.youtube.com/embed/ztpi5BZHRSc?rel=0&showinfo=0";
                    break;
                case "Sync Calendars":
                    videoSrc = "https://www.youtube.com/embed/LswIagiIWMA?rel=0&showinfo=0";
                    break;
                case "Email Tracking":
                    videoSrc = "https://www.youtube.com/embed/BYnoDmRXtso?rel=0&showinfo=0";
                    break;
                case "Salesforce Templates":
                    videoSrc = "https://www.youtube.com/embed/R7c1-xek7FM?rel=0&showinfo=0";
                    break;
                case "Salesforce inside any web page":
                    videoSrc = "https://www.youtube.com/embed/IV_3Z3SmqTc?rel=0&showinfo=0";
                    break;
                    case "Email Cadences":
                    videoSrc = "https://www.youtube.com/embed/VJUzLh6ZG4M?rel=0&showinfo=0";
                    break;
                case "Meeting Scheduler":
                    videoSrc = "https://www.youtube.com/embed/vq-3c27YXWc?rel=0&showinfo=0";
                    break;
                default:
                    break;
                }

                if (videoSrc) {
                    var videoElement = CreateVideoFrameForOnBoarding(videoSrc, "eb-onboard-video", null);
                    $(currentStepVisible).find('.eb-textcontent').after(videoElement).hide().fadeIn();
                }
            }
        }
    }

    function SetupEbstaModalEvents() {

        $(".eb-wizard-noconnect").off("click");
        $(".eb-wizard-noconnect").on("click", function () {
            //Event for dont auth now btn - call funct to update setting for this mailbox
            HandleDontAuthNowEvent();
            LogEvent("Onboarding Click - No Thanks");
        });

        //Setup event for the auth button
        SetupGoogleAuthClickEvent();
    }

    //set up click events for the trial/upgrade button.
    function SetupUpgradeClickEvent() {
        
        //on click redirect user to the ebsta pricing page in a new tab.
        $("body").undelegate(".ebsta-trialmessage", "click");
        $("body").delegate(".ebsta-trialmessage", "click", function () {

            var redirectUrl = "https://www.ebsta.com/pricing/";
            window.open(redirectUrl, "_blank");
        });
    }
    
/**
* Creates an extra button when in Google Calendar page to prompt user to setup a Calendar Sync
* @returns {} 
*/
function CreateOnboardRestartBtnGoogleCalendar()
{
    if ($('.eb-authwizard-restart-calendar').length < 1) {
        
        var calendarSyncDiv = CreateElement('button', 'ebsta-btn ebsta-btn-primary ebsta-btn eb-authwizard-restart-calendar btn-success', 'Sync Calendar with Salesforce', null);
        
        //Old Google Calendar Site Pre-Dec 2017
        if (GoogleCalendarTools.IsThisNewGoogleCalendarLayout()) {
            var topRightNavigationDiv = document.getElementById("topRightNavigation");
            var clawIcon = CreateElement("span", "newClawSmallLogo");
            calendarSyncDiv.appendChild(clawIcon);

            $(topRightNavigationDiv).children().first().prepend(calendarSyncDiv);
        } else {
            if (!renderTrialCountdown) {
                var calendarSyncHeaderDiv = CreateElement("div", "ebsta-google-header-container");
                calendarSyncHeaderDiv.appendChild(calendarSyncDiv);
                //New Google Calendar Site Dec 2017
                AddClass(calendarSyncDiv, "ebsta-btn-lg");
                var ebstaSettingsButton = document.getElementsByClassName("ebsta-settings-parentdiv")[0];
                if (ebstaSettingsButton) {
                    insertBefore(calendarSyncHeaderDiv, ebstaSettingsButton);
                }
            }

            GoogleCalendarTools.UpdateEbstaButtonsState();
        }

        SetupOnboardingRestartEvent();
    }
}
    function HandleDontAuthNowEvent() {
        //console.time("HandleDontAuthNowEvent");
        var currentUserGmail = GetGmailUsersEmailAddress();
        ClearModals(false);

        chrome.extension.sendMessage({ MessageType: "UpdateLocalGmail", GoogleAccountEmail: currentUserGmail, isActive: 0 });

        // AS user does not auth, check if we should run gmail in bar only mode
        chrome.extension.sendMessage({ MessageType: "GetAccountActive", EmailAddress: currentUserGmail }, function (response) {
            if (response && response.DropBarWithNoAuth === true) {
                GmailTools.SetupGmailBarOnlyMode();
            }
        });
    }

    function SetupOnboardingRestartEvent() {
        //Event for auth / onbarding restart btn click - open onboarding stage 1
        $(".eb-authwizard-restart, .eb-authwizard-restart-calendar").off("click");
        $(".eb-authwizard-restart, .eb-authwizard-restart-calendar").on("click", function (event) {

            //if not logged in - login
            if (!LoggedIn) {
                chrome.extension.sendMessage({ MessageType: "OpenSFAuthTabURI", URI: "https://console.ebsta.com/Salesforce/RequestAccess.aspx?ver=10" });
            }
            else {

                event.stopImmediatePropagation();

                //If we dont have the onbarding wizards - create them and start wizards 
                if ($("#eb-wizard-onboarding").length < 1) {
                    SetupAuthOnboardingWizards(!isAccountAuthorised, false);
                } else {
                    //if we allready have them - open auth stage
                    if (isAccountAuthorised) {
                        //Google Calendar
                        if (document.URL.indexOf("google.com/calendar") != -1) {
                            OpenOnboardingModal(3);
                        } else if (document.URL.indexOf("contacts.google.com") != -1 || document.URL.indexOf(".google.com/contacts") !== -1) {
                            OpenOnboardingModal(8);
                        } else {
                            OpenOnboardingModal(2);
                        }
                    } else {
                        OpenOnboardingModal(1);
                    }
                    ResetCalendarSettingsSteps();
                }

                //Lets log which site they were on when the clicked the button
                var gmailOrGoogleCalendar = "Gmail";
                if (document.URL.indexOf("google.com/calendar") != -1) {
                    gmailOrGoogleCalendar = "Calendar";
                } else if (document.URL.indexOf("contacts.google.com") != -1) {
                    gmailOrGoogleCalendar = "Contacts";
                }
                LogEvent("Onboarding Click - Restart Onboarding (" + gmailOrGoogleCalendar + ")");
            }
        });
    }
    
    //Resets the onboarding settings so they're default
    function ResetCalendarSettingsSteps() {
        ResetCalendarSettingsChooseSettingsStep();
        ResetCalendarSettingsConfirmSettingsStep()
    }
    
    //Preview Settings reset
    function ResetCalendarSettingsChooseSettingsStep() {
        $('.ebstaStep5:not(#eb-onboarding-calendarsyncadminmodemessage)').children().show();
        $('#eb-onboarding-calendarsyncadminmodemessage').hide();
        $('.ebstaStep5.ebstaStep5:not(#eb-onboarding-calendarsyncadminmodemessage)').children().show();
        $('.ebstaStep5 .alert, .eb-stopcalendarsyncbtn, .eb-syncnowbtn').hide();
        $('.ebstaStep5 .eb-calsync-calendarlist').empty().css('max-height', ''); //calendar list
        $('.ebstaStep5 .ebNewLoader').show();
        $('.ebstaStep5 .eb-onboarding-currentstatustext').text("Loading...").removeClass('text-success').removeClass('text-danger');
        $('.ebstaStep5 .eb-onboarding-calendarsettingsheader input').prop('checked', false);
        $('.ebstaStep5 .eb-onboarding-lastsyncheader, .ebstaStep5 .eb-onboarding-lastsynctext').css('visibility', 'hidden');
    }
    
    //Confirm Settings reset
    function ResetCalendarSettingsConfirmSettingsStep() {
        $('.eb-calsync-confirmcalendars').find('h5,p').empty(); //calendar list
        $('.ebstaStep6 .eb-calsync-confirmprevieweventstable').remove(); //Events Table
        $('.ebstaStep6 .ebNewLoader').show();
    }

    function OpenOnboardingModal(openStage) {
        //console.log("openonboardingModal");
        //console.time("OpenOnboardingModal");
        //Open a onboarding stage
        var bodyClasses = document.body.className;

        //Add the modal open class (shows the fade overlay) to body
        if (bodyClasses.indexOf('eb-modal-open') == -1) {
            if (bodyClasses.length > 0) {
                bodyClasses += " eb-modal-open";
            } else {
                bodyClasses += " eb-modal-open";
            }

            document.body.setAttribute("class", bodyClasses)
        }

        //Hide any existing modal page
        $(".eb-wizard-modalpage, #eb-wizard-stepbystepprocess").hide();

        //Hide all steps then show the required step
        $(".ebstaStep").hide();

        if (openStage > 1)
        {
            $("#eb-wizard-stepbystepprocess").fadeIn();
            $(".ebstaStep" + openStage).fadeIn();
            ActionOnContinueButtonClick();
        }
        else 
        {
            $('#eb-wizard-authpart1').show();
            if (openStage == "AuthFailed") {
                $('#eb-wizard-authpart1').css('max-height', '445px').find('.eb-onboarding-gmailautherror').fadeIn();
            } else {
                $('#eb-wizard-authpart1').css('max-height', '380px').find('.eb-onboarding-gmailautherror').hide();
            }
        }

        //Show the onbarding container
        $("#eb-wizard-onboarding, .eb-modal-bodyoverlay").show();

        ActionOnContinueButtonClick();
        InsertVideoIframeForStep();
        //console.timeEnd("OpenOnboardingModal");
    }

    function SetupGoogleAuthClickEvent(bindToSelector) {

        $('.eb-wizard-gauth').on("click", function () {

            SetupAuthMessageListener();

            LogEvent("Onboarding Click - Connect to Ebsta");

            var origin = "https://mail.google.com";

            //Setup the auth url
            var googleAuthUrl = "https://console.ebsta.com/Google/FullAuth.aspx?wb=" + webUserId + "&origin="+origin+"&gmailAddress="+GetGmailUsersEmailAddress();

            //Open a auth popup - new window
            var popUpObj = window.open(googleAuthUrl,
             "ModalPopUp",
            "toolbar=no," +
            "scrollbars=no," +
            "location=no," +
            "statusbar=no," +
            "menubar=no," +
            "resizable=0," +
            "width=750," +
            "height=650," +
            "left=380," +
            "top=200"
            );

            //focus on the new window.
            popUpObj.focus();

            //Wait for half a second and check if the popup has been blocked / closed
            setTimeout(function () {
                if (!popUpObj || popUpObj.outerHeight === 0 || popUpObj.closed) {
                    //If so lets use chrome tabs api to open a new auth tab instead.
                    chrome.extension.sendMessage({ MessageType: "OpenGoogleAuthTabURI", URI: googleAuthUrl });
                }
            }, 500)
        });
    }


    function HandleGoogleAuthCompleted() {
        //When auth is complete - open onboarding stage at the relevant point depending on what site you're on
        if (document.URL.indexOf("google.com/calendar") != -1) {
            OpenOnboardingModal(3);
        } else if (document.URL.indexOf("contacts.google.com") != -1 || document.URL.indexOf(".google.com/contacts") !== -1) {
            OpenOnboardingModal(8);
        } else {
            OpenOnboardingModal(2);

            //Activate the mutation obeserver now we're authed
            if (enableMutationObservers + "" == "true") {
                // TODO: MUTATION
                EbstaGmailMutationObserver(["NewComposeWindowDetected"]);
            }
        }

        var currentUserGmail = GetGmailUsersEmailAddress();

        //Update global variable so we know to skip authorisation in the future
        isAccountAuthorised = true;

        GmailTools.InjectProxy();

        //Now we're authed we need to append some more buttons into the header depending on the users allowances
        GoogleTools.Header.ReRenderButtons();

        //Update the dont auth now for this user to false - we want auth for this mailbox 
        chrome.extension.sendMessage({ MessageType: "UpdateGoogleAccountDontAuthNow", GoogleDontAuthNow: 0, GoogleAccountEmail: currentUserGmail });
    }

    function HandleGoogleAuthFailed() {
        OpenOnboardingModal("AuthFailed");
    }

    function CreateButton(buttonId, buttonText, buttonClass) {
        var button = document.createElement("button");
        button.setAttribute("class", buttonClass);
        button.innerText = buttonText;

        if (buttonId.length > 0) {
            button.setAttribute("id", buttonId);
        }

        return button;
    }

    function CreateModalAuthPage() {

        //The the current users email addr.
        var currentUserEmail = GetGmailUsersEmailAddress();

        //create the overall container for this modal
        var wizardAuthPart = CreateDivWithId("", "eb-wizard-modalpage", "eb-wizard-authpart1");
        wizardAuthPart.setAttribute("data-onboard-stage", "1");

        //Create the modal heading - with botton border
        var wizardAuthHeaderCont = CreateDiv("", "panel panel-default eb-wizard-authpart1-connect");

        var title = CreateDiv("", "eb-onboardingtextheader panel-heading");

        title.appendChild(CreateDiv("", "eb-onboardinglogo"));

        //var closeIconDiv = CreateDiv("", "ebsta-glyphicon ebsta-glyphicon-close-icon");
        //$(closeIconDiv).on("click", function () {
        //    ClearModals(true);
        //});
        //title.appendChild(closeIconDiv);

        wizardAuthHeaderCont.appendChild(title);

        var body = CreateDiv("", "panel-body");

        var contentDiv = CreateDiv("", "row text-center");

        contentDiv.appendChild(CreateElement("h1", null ,"Welcome to Ebsta", null));

        //Create the Circle Images
        var iconRow = CreateRow("row imagerow");

        var firstColumn = CreateDiv("", "col-xs-3");

        var circle1 = CreateDiv("", "eb-white-circle eb-intro-circle1");
        firstColumn.appendChild(circle1);
        firstColumn.appendChild(CreateElement("h4", null, "Add Emails", null));
        iconRow.appendChild(firstColumn);

        var secondColumn = CreateDiv("", "col-xs-3");

        var circle2 = CreateDiv("", "eb-white-circle eb-intro-circle2");
        secondColumn.appendChild(circle2);
        secondColumn.appendChild(CreateElement("h4", null, "Sync Calendars", null));
        iconRow.appendChild(secondColumn);

        if (abRules.EnableContactSync + '' === "1") {
            var thirdColumn = CreateDiv("", "col-xs-3");
            var circle3 = CreateDiv("", "eb-white-circle eb-intro-circle3 ebsta-glyphicon");
            thirdColumn.appendChild(circle3);
            thirdColumn.appendChild(CreateElement("h4", null, "Sync Contacts", null));
            iconRow.appendChild(thirdColumn);
            $(iconRow).css('width', '790px');
        }

        var fourthCol = CreateDiv("", "col-xs-3");

        var circle4 = CreateDiv("", "eb-white-circle eb-intro-circle4");
        fourthCol.appendChild(circle4);
        fourthCol.appendChild(CreateElement("h4", null, "Track Emails", null));
        iconRow.appendChild(fourthCol);

        contentDiv.appendChild(iconRow);

        //Error Message
        var errorText = CreateDiv("An error occured while connecting your Google account account to Ebsta. \n Please try again or contact support@ebsta.com for more help.", "alert alert-danger eb-onboarding-gmailautherror");
        $(errorText).hide();
        contentDiv.appendChild(errorText);

        //Create the option buttons
        var buttonRow = CreateRow("row buttonrow text-right");

        var noThanksButton = CreateButton("", "No Thanks", "ebsta-btn ebsta-btn-lg ebsta-btn-default eb-wizard-noconnect");
        buttonRow.appendChild(noThanksButton);

        var loginButton = CreateButton("", "Connect your Google Account to Ebsta", "ebsta-btn ebsta-btn-lg ebsta-btn-success eb-wizard-gauth");
        buttonRow.appendChild(loginButton);

        contentDiv.appendChild(buttonRow);

        body.appendChild(contentDiv);

        wizardAuthHeaderCont.appendChild(body);

        wizardAuthPart.appendChild(wizardAuthHeaderCont);

        return wizardAuthPart;
    }

    function ClearModals(checkForAuthPage) {
        //console.time("ClearModals");
        //Clearing any existing modals

        //Check if the auth window was current window - if so count it as a dont auth this mailbox event
        if (checkForAuthPage) {
            if ($("*[data-onboard-stage='1']:visible").length > 0) {
                //HandleDontAuthNowEvent();
            }
            LogEvent("Onboarding Click - Exit Out");
        }

        //Hide any modal and its container
        $('.eb-modal-container').fadeOut(400, function (){
            RemoveVideoIframesFromOnboardingProcess();
        });
        $('.eb-modal-bodyoverlay').fadeOut(400);
        //Remove the modal open class from body- this hides the fade overlay
        $('body').removeClass("eb-modal-open");        

        //console.timeEnd("ClearModals");
    }


    function HandleBarOpenWizardModals() {
        //Send the user to console login auth page if they're not logged in
        if (!LoggedIn) {
            chrome.extension.sendMessage({ MessageType: "OpenSFAuthTabURI", URI: "https://console.ebsta.com/Salesforce/RequestAccess.aspx?ver=10" });
        } else {
            //Handles open wizard modals request from bar
            //console.time("HandleBarOpenWizardModals");
            //If we dont have the onbarding wizards - create them
            if ($("#eb-wizard-onboarding").length < 1) {
                SetupAuthOnboardingWizards(false);
            }

            //Then start wizards from stage 2 - post auth
            OpenOnboardingModal(2);
            //console.timeEnd("HandleBarOpenWizardModals");
        }
    }
    
    //Sets up custom events we need to happen within Seek.com
    function SetupSeekEvents(){
        SeekLoadBarOnPageLoad();
        //Attach click listener to whole document as profiles load async so unable to attach to each individual profile
        document.addEventListener("click", function (event) {
            //runs the initial seek opt function - if this does find elements to drop the bar
            //then chances are you are on the new seek version, and we should try that way
            if (!SeekWasProfileFoundAndBarDropped()) {
                talentsearchSeek2019VersionHelper.PageClickHandler(event);
            }
        });
    }
    
    //Finds elements on page load to help drop the bar in SEEK
    function SeekLoadBarOnPageLoad() {
        //wait for the result list to be loaded - set retry limit and track count, retry every n secons
        var listLoaded = false;
        var newSeekProfileOpenTab = false;
        var retryCount = 0;
        var retryLimit = 20;
        var waitBetweenTry = 500;

        //start retry - up to n times each n seconds, or until bar drops
        var retryInterval = setInterval(function () {
            if (retryCount < retryLimit && !listLoaded && !newSeekProfileOpenTab) {
                retryCount++;
                //look for the results loaded element - if there load is ok
                //set loaded to true - next pass drops bar
                var listCount = document.getElementsByClassName("results__loaded").length;

                //this is for the 2019 version of Seek - if we can find this element it means a profile
                //has already been opened, and we should drop the bar using the rules
                //we need to keep both opts working at this point, as both are still available
                if (listCount === 0) {
                    if (document.querySelectorAll(talentsearchSeek2019VersionHelper.GetModalQuerySelector).length > 0) {
                        newSeekProfileOpenTab = true;
                    }
                }

                if (listCount > 0) {
                    listLoaded = true;
                }
            } else {
                //success or give up, clear interval and stop
                clearInterval(retryInterval);
                if (listLoaded) { //if you got here via the list count (old seek)
                    SeekWasProfileFoundAndBarDropped();
                } else { //otherwise you are probably on the new seek and we can drop using the rules engine
                    dropBarUsingRulesEngine();
                    //after dropping bar - fix the modal position
                    talentsearchSeek2019VersionHelper
                        .FixModalPosition(talentsearchSeek2019VersionHelper.GetModalQuerySelector, true);
                }
            }

        }, waitBetweenTry);
    }
    
    var global_LastSeekNameSearch = "";

    /**
     *  * Finds the visible profile so we can drop the bar for that person.
     * This will run for "old" version of seek, we now have some people on a new version
     * so will return a true / false if function was able to find a name to search for.
     * If not we can run out new function for the new version of seek.
     * @returns { boolean } - True of false if we have run bar drop
     * */
    function SeekWasProfileFoundAndBarDropped() {
        var foundOpenProfile = document.querySelectorAll("div[ui-view='profile-details']");
        if (foundOpenProfile) {
            if (foundOpenProfile.length > 0) {
                //There should only be 1 open at a time so we'll take the first
                var profileDiv = foundOpenProfile[0];

                var name = profileDiv.querySelectorAll("span[class='profile-header__text']");

                if (name) {
                    if (name.length > 0) {
                        var nameText = name[0].textContent;

                        //Trim the full stop at the end if it exists
                        if (nameText.length > 1) {
                            if (nameText.substr(nameText.length - 1) == ".") {
                                nameText = nameText.substring(0, nameText.length - 1);
                            }
                        }

                        //If the last search was the same name then don't fire it again
                        if (nameText != global_LastSeekNameSearch) {
                            //console.log("search 14");
                            proc.LoadFrame(nameText, 1);
                            global_LastSeekNameSearch = nameText;
                        }
                        return true;
                    }
                }
            }
        }
        return false;
    }

// Goes through customgmail rules to see if we hit any so we know to go into 
// contentget.js and reparse content with our custom optimisation
function Does_Email_Have_Custom_Rules_Associated_With_It(rules) {
    for (var i = 0; i < rules.length; i++) {
        if (BarJsGetValueByXpath(rules[i])) return true;
    }
    return false;
}

//Creates a span which referecess a CSS class to loading a blue spinner loading gif
function CreateBlueSpanSpinnerLoader() {
    return CreateElement("span", "eb-spinnerloaderblue", null, null);
}

//Takes a key event type and and element and triggers an event on that element
function SimulateKeyEventOnElement(eventType, element, key, char) {
    var evt = new KeyboardEvent(eventType, { bubbles: true,cancelable: true, key: key, char: char });
    element.dispatchEvent(evt);
}

//Takes a mouse event type and and element and triggers an event on that element
function SimulateMouseEventOnElement(eventType, element) {
    var evt = new MouseEvent(eventType, {bubbles: true,cancelable: true,view: window});
    element.dispatchEvent(evt);
}

/**
 * Injects a style at the HEAD section which lasts for however long has been specified in the lengthOfTimeRuleShouldStayFor parameter
 * @param {} rule 
 * @param {} lengthOfTimeRuleShouldStayFor 
 * @returns {} 
 */
function InjectTemporaryStyle(rule, lengthOfTimeRuleShouldStayFor) {
    var temporaryStyleElement = CreateElement("style", null, rule, null);
    document.head.appendChild(temporaryStyleElement);
    setTimeout(function() {
        temporaryStyleElement.remove();
    }, lengthOfTimeRuleShouldStayFor);
}


//Listen for events on TermScout
function ListenForPageEventsTermScout() {
    //store last search valid hash 
    var termScoutLastHash = "";
    //fire bar drop on url change
    window.addEventListener("hashchange", function () {
        var hash = document.location.hash;
        if (hash != null) {
            var hash = '' + hash.toLowerCase();
            if (hash.indexOf("#/scouts/") !== -1 || hash.indexOf("#/research/company/") !== -1) {
                if (hash !== termScoutLastHash) {
                    dropBarUsingRulesEngine();
                }
                termScoutLastHash = hash; //store current hash, multiple changes pr. page load
            } else {
                //wipe last hash when hiding bar
                termScoutLastHash = "";
                proc.HideFrame();
            }
        }
    });
}

/**
 * Creates a closure for the Ebsta Buttons we insert into a compose window
 * @returns {} 
 */
function EbstaComposeButtons() {
    var lSendLaterClosure = new SendLaterObject(), //Send later details
        lCampaignManagerClosure = null, //Campaign manager details
        lComplexAddClosure = new ComplexAdd(), //Contains details chosen for Complex Add
        lEmailTrackingButton = null, // The Email Tracking button
        lEmailTrackingState = false, //Is email tracking set
        lEbstaSendButton = null, //Ebsta sent button
        lGmailSendButton = null; // Gmail sent button

    if (globalCompanySettings.DisableCampaigns !== true) {
        lCampaignManagerClosure = new createCampaignMgrObject(GmailTools);
    } else {
        //if campaign disabled create object that will always return .HasCampaignBeenSelected = false
        lCampaignManagerClosure = new createDisabledCampaignMgrObject();
    }

    /**
     * Checks if SendLater, ComplexAdd or Campaigns have been set in order to know if we should show our own button or gmails default send button
     * @returns {} 
     */
    function ShowOrHideEbstasOwnSendButton() {
        if (lCampaignManagerClosure.HasCampaignBeenSelected() || lSendLaterClosure.IsScheduled() || lComplexAddClosure.HasComplexAddBeenSet()) {
            HideElement(lGmailSendButton);
            lEbstaSendButton.style.display = null;
        } else {
            lGmailSendButton.style.display = null;
            HideElement(lEbstaSendButton);
        }
    }

    /**
     * Click event on the complex add button
     * @param {} complexAddButton 
     * @returns {} 
     */
    function ComplexAddButtonClickEvent() {
        lComplexAddClosure.SetupAndCreateComplexAdd(this, lComplexAddClosure, lCampaignManagerClosure.HasCampaignBeenSelected(), ShowOrHideEbstasOwnSendButton);
    }

    /**
     * Looks within the compose window to see if a template has been selected and gets the id (Stripped from TrackGmailEmailTracking)
     * @returns {} 
     */
    function GetTemplateIdSelected() {
        var composeWindow = GetComposeWindowFromElement(lGmailSendButton);
        if (composeWindow != null) {
            var ebstaTemplateElement = composeWindow.getElementsByClassName("ebstaTemplateObject");
            if (ebstaTemplateElement.length > 0) {
                var ebstatemplateidAttribute = ebstaTemplateElement[0].getAttribute("ebstatemplateid");
                if (ebstatemplateidAttribute != null) {
                    return ebstatemplateidAttribute;
                }
            }
        }
        return null;
    }

    /**
     * Click Event handler for our own send button within the compose window
     * @returns {} 
     */
    function EbstaSendButtonClickEvent() {
        var composeSubject = GetSubjectLineInGmailComposeOrReplyWindow(lEbstaSendButton);
        var isSendLaterScheduler = lSendLaterClosure.IsScheduled();

        //Campaign Manager Process
        if (lCampaignManagerClosure.HasCampaignBeenSelected()) {
            lCampaignManagerClosure.SaveContactToCampaignComposeMode(lSendLaterClosure.GetScheduledDate(), lEmailTrackingState, lComplexAddClosure.GetComplexAddValues());
            return true;
        }

        //If email tracking is set then set this email up to be tracked
        if (lEmailTrackingState === true) {
            InsertImgForGmailEmailTracking(lEbstaSendButton, composeSubject, !isSendLaterScheduler);
        }

        //Send Later Only
        if (isSendLaterScheduler) {
            var contacts = GmailTools.Compose.GetContactsFromComposeWindow(lEbstaSendButton, null);
            if (lSendLaterClosure.VerifyRecipients(contacts)) {
                lSendLaterClosure.ScheduleEmail(lEbstaSendButton, lComplexAddClosure.GetComplexAddValues(), lEmailTrackingState, GetTemplateIdSelected());
            }
            return true;
        }

        //Sending and adding of emails
        if (lComplexAddClosure.HasComplexAddBeenSet()) {
            lComplexAddClosure.SetOriginalSubject(composeSubject);

            var complexAddValues = lComplexAddClosure.GetComplexAddValues();

            /*
             * When the user sets an email to be saved to salesforce, this will keep track of the URL at
             * the time the user pressed the action button to send, and as the process may take a while
             * the url could change between now and the actual request being sent up to our servers (currently just for logging purposes)
             */
            complexAddValues.RequestedFromUrl = document.location.href;

            // When opening emails and compose screens using CTRL-Click or Shift-Click they open in a different window 
            // but are still related to the main instance of Gmail. When you send an email through these screens our proxy 
            // catches the request and all important messageid we need to get the message information is picked up in main Gmail window.
            // So we're now sending the complex add object to background.js and we'll pick it up once gmailproxy.js finds the request we need and sends us back the id,
            // once we find that ID we'll fetch the object we just sent to background.js and follow the same procedure we use to add a complex add object

            chrome.extension.sendMessage({
                MessageType: "AddSaveOnSendObject",
                Record: complexAddValues,
                DateInMilliseconds: new Date().getTime(),
                Mailbox: GmailTools.GetUserName()
            });
        }

        if (lEmailTrackingState === true) {
            LoggingEmailSend();
        }

        SimulateMouseEventOnElement("click", lGmailSendButton);
    }

    //Set click event up on the compose window's "Send Email" button so that it respects if a user pressed the Send
    var sendEmailButton = lComplexAddClosure.GetSendEmailButton();
    if (sendEmailButton != null) {
        sendEmailButton.addEventListener("click", function () {
            if(VerifyCustomFieldInputs(this)) {
                SaveComplexAddNew(sendEmailButton.ComplexAddObject);
                lComplexAddClosure.CloseComplexAddScreen();
                EbstaSendButtonClickEvent();
            }
        });
    }

    /**
     * Inserts Ebsta own Send button within Gmail so we can control the events when COmplex Add, Send Later or Campaigns have been selected
     * @param {} button 
     * @returns {} 
     */
    function CreateEbstaOwnSendButton(buttonsContainer) {
        var gmailSendButton = buttonsContainer.getElementsByClassName("aoO");
        if (gmailSendButton.length > 0) {
            lGmailSendButton = gmailSendButton[0];
            lEbstaSendButton = lGmailSendButton.cloneNode(true);
            lEbstaSendButton.innerText = "Send";
            lEbstaSendButton.removeAttribute("id");
            AddClass(lEbstaSendButton, "sfEbSendBtn");
            insertAfter(lEbstaSendButton, lGmailSendButton);
            HideElement(lEbstaSendButton);
            lEbstaSendButton.addEventListener("click", EbstaSendButtonClickEvent);
        }
    }

    /**
     * Uses the lEmailTrackingState variable to know how to set the button
     * @returns {} 
     */
    function UpdateEmailTrackingButton(buttonElement) {
        if (lEmailTrackingState) {
            RemoveClass(buttonElement, "sfEbstaGmailEmailTrackingNotSelected");
            AddClass(buttonElement, "sfEbstaGmailEmailTrackingSelected");
            buttonElement.setAttribute("title", "Email Tracking On");
        } else {
            RemoveClass(buttonElement, "sfEbstaGmailEmailTrackingSelected");
            AddClass(buttonElement, "sfEbstaGmailEmailTrackingNotSelected");
            buttonElement.setAttribute("title", "Email Tracking Off");
        }
    }

    /**
     * Click event for the Email Tracking button
     */
    function EmailTrackingButtonClickEvent() {
        if (HasClass(this, "sfEbstaGmailEmailTrackingNotSelected")) {
            lEmailTrackingState = true;
        }
        else if (HasClass(this, "sfEbstaGmailEmailTrackingSelected")) {

            //If we're turning off the tracking when a campaign is selected we need to show them an extra warning message
            if (lCampaignManagerClosure.HasCampaignBeenSelected()) {
                lCampaignManagerClosure.ShowEmailTrackingOffWarning(function() {
                    lEmailTrackingState = false;
                    UpdateEmailTrackingButton(lEmailTrackingButton);
                });
                return;
            } else {
                lEmailTrackingState = false;
            }
        }

        UpdateEmailTrackingButton(this);
    }

    /**
     * Creates the email tracking button
     * @returns {} 
     */
    function CreateEmailTrackingButton() {
        var td = Createtd("gU ebstaTD");

        lEmailTrackingButton = CreateDiv("", "sfEbstaGmailEmailTracking sfEbstaGmailEmailTrackingNotSelected");
        lEmailTrackingButton.placeholder = "Click here to track your email";
        lEmailTrackingButton.title = "Email Tracking Off";

        var imgSpn = CreateSpan("", "");
        imgSpn.className = "sfEbstaGmailEmailTrackingButton";

        //Set tick box to be enabled if the user has previously used the tracking option
        if (lastTrackingChoice != null && lastTrackingChoice.toString() == "true") {
            lEmailTrackingState = true;
            UpdateEmailTrackingButton(lEmailTrackingButton);
        }

        if (buttonMode === 1) {
            lEmailTrackingButton.classList.add("sfEbstaGmailEmailTracking-style1");
            imgSpn.className = "sfEbstaGmailEmailTrackingButton-style1";
        }

        lEmailTrackingButton.appendChild(imgSpn);

        //only add the click event if the button is enabled.
        if (abRules.EnableGmailEmailTracking + "" == "1") {
            lEmailTrackingButton.addEventListener("click", EmailTrackingButtonClickEvent);
        }

        td.appendChild(lEmailTrackingButton);
        return td;
    }

    /**
     * Creates the Templats Button
     * @returns {} 
     */
    function AddGoogleEmailTemplatesButton() {

        var td = Createtd("gU ebstaTD");

        var spn = CreateSpan("", "");
        spn.className = "sfGetEmailTemplates";
        spn.placeholder = "Click here to use salesforce email templates";

        var imgSpn = CreateSpan("", "");
        imgSpn.className = "sfEbstaGmailEmailTemplatesBtn sfEbstaGmailEmailTemplatesNotSelected";
        imgSpn.title = "Salesforce Email Templates";

        if (buttonMode === 1) {
            spn.classList.add("sfGetEmailTemplates-style1");
            imgSpn.classList.add("sfEbstaGmailEmailTemplatesBtn-style1");
        }

        spn.appendChild(imgSpn);
        td.appendChild(spn);
        return td;
    }

    /**
     * Inserts a button at the end of the ebsta button list
     * @param {} button 
     * @returns {} 
     */
    function InsertButtonIntoComposeWindowOnTheEnd(composeButtonsContainer, elementToAdd) {
        //Insert at the end end
        var ebstaButtons = composeButtonsContainer.getElementsByClassName("ebstaTD");
        if (ebstaButtons && ebstaButtons.length > 0) {
            var lastEbstaContainer = ebstaButtons[ebstaButtons.length - 1];
            if (lastEbstaContainer) {
                insertAfter(elementToAdd, lastEbstaContainer);
            }
        }
    }

    /**
     * Create buttons and insert them into Gmail Compose container given
     * @param {} composeButtonsContainer 
     * @returns {} 
     */
    function Initialize(buttonWithinComposeScreen, ebstaComposeButtonsClosure) {

        //console.log("Initialize()");
        //console.log(buttonWithinComposeScreen);
        //console.log(ebstaComposeButtonsClosure);

        var composeButtonsContainer = buttonWithinComposeScreen.closest(".n1tfz");

        //Before we start adding buttons, set that we have not added buttons yet.
        var toobarHasButtons = false;

        //console.log("composeButtonsContainer:");
        //console.log(composeButtonsContainer);

        if (typeof composeButtonsContainer == "undefined" || composeButtonsContainer == null)
        {
            //console.log("get the second value:");
            composeButtonsContainer = buttonWithinComposeScreen.closest(".btC");
        }

        //console.log("composeButtonsContainer 2:");
        //console.log(composeButtonsContainer);

        //Then check if the parent container indicates that we have
        if (HasClass(composeButtonsContainer,"ebstaBtnsAdded")) {
            toobarHasButtons = true;
        }

        //If we have not added buttons in this toolbar allready.
        if (!toobarHasButtons) {

            CreateEbstaOwnSendButton(composeButtonsContainer);

            //Mark that we have created buttons for this toolbar allready - dont want to be doing it multiple times.
            AddClass(composeButtonsContainer,"ebstaBtnsAdded");

            var complexAddButton = Createtd("gU ebstaTD");

            var spn = CreateSpan("", "");
            spn.setAttribute("style", "");

            var spnComplexAdd = CreateDiv("", "sfComplexAddSmallCompose googleButtonStyleLargeComposeScreen");
            spnComplexAdd.title = "Send + Add to Salesforce";

            if (buttonMode === 1) {
                spnComplexAdd.classList.add("sfComplexAddSend-style1");
            }

            spnComplexAdd.addEventListener("click", ComplexAddButtonClickEvent);

            lComplexAddClosure.SetComplexAddButton(spnComplexAdd);

            spn.appendChild(spnComplexAdd);
            complexAddButton.appendChild(spn);
            insertAfter(complexAddButton, lEbstaSendButton.closest(".gU.Up"));

            $(".ebstaCol").remove(); //clear any columns we have added already so we do not dupe.

            //set styling for the button table to we don't screw with google's layout.
            //add column groups to the table to match inserted cells.
            var colGroup1 = document.createElement('col');
            colGroup1.className = "ebstaCol";
            $(".IZ").find("colgroup").find(".Uf").before(colGroup1);

            //Inserts tickbox for email tracking selection
                var emailTrackingButton = CreateEmailTrackingButton();
                $(complexAddButton).after(emailTrackingButton);

                var colGroup2 = document.createElement('col');
                colGroup2.className = "ebstaCol";

                $(".IZ").find("colgroup").find(".Uf").before(colGroup2);

            if (abRules.EnableGmailEmailTracking + "" != "1") {
                //add the disabled class.
                $(".sfEbstaGmailEmailTracking").addClass("ebsta-feature-icon-disabled");

                //remove the tracking classes if this is disabled.
                $(".sfEbstaGmailEmailTracking").removeClass("sfEbstaGmailEmailTrackingNotSelected").removeClass("sfEbstaGmailEmailTrackingSelected");

                //set up the tooltip for the disabled feature.
                SetupEmailTrackingUpgradeTooltip("ebsta-tooltip-emailtracking", "sfEbstaGmailEmailTracking");
            }else if ($(composeButtonsContainer).closest('.I5').find('.oG').text() == "Plain text") { //Lets look for the 'Plain Text' wording which appears on compose load to help update the tracking icon.
                UpdateGmailTrackingButtonForPlainTextEmails($(composeButtonsContainer), true);
            }

            //Insert email templates button
            if (abRules.ShowGmailTemplates + "" == "1") {
                var templatesBtn = AddGoogleEmailTemplatesButton();
                $(complexAddButton).after(templatesBtn);

                var colGroup3 = document.createElement('col');
                colGroup3.className = "ebstaCol";

                $(".IZ").find("colgroup").find(".Uf").before(colGroup3);
            }

            //Insert Send Later Buttons.
            var addClickEvent = true;
            
            if (abRules.EnableSendLater + "" != "1") {
                addClickEvent = false;
            }

            lSendLaterClosure.GetScheduleDetailRequest(lEbstaSendButton, ebstaComposeButtonsClosure);
            var sendLaterBtn = lSendLaterClosure.CreateSendLaterButton(addClickEvent);
            insertBefore(sendLaterBtn, complexAddButton);

            if (!addClickEvent) {
                $(".sfEbSendLaterBtn").addClass("ebsta-feature-icon-disabled");

                //set up the tooltip for the disabled feature.
                SetupSendLaterUpgradeTooltip("ebsta-tooltip-sendlater", "sfEbSendLaterBtn");
            }

            //insert campaign manager button. 
            //if globalCompanySettings.DisableCampaigns lCampaignManagerClosure will be an empty object returning 
            //only .HasCampaignBeenSelected() false 
            if (lCampaignManagerClosure != null && !globalCompanySettings.DisableCampaigns) {
                lCampaignManagerClosure.Initialize();
                lCampaignManagerClosure.SetCampaignSendLimit(globalCampaignLimit);
                lCampaignManagerClosure.SetEbstaComposeButtonsClosure(ebstaComposeButtonsClosure);
                InsertButtonIntoComposeWindowOnTheEnd(composeButtonsContainer, lCampaignManagerClosure.CreateCampaignManagerButton());

                var colGroup5 = document.createElement('col');
                colGroup5.className = "ebstaCol";

                $(".IZ").find("colgroup").find(".Uf").before(colGroup5);
            }

            //if campaigns are disabled, disbale the button and add tooltip.
            if (abRules.EnableCampainMgr + "" != "1") {
                $(".sfEbstaCampaignMgrButton").addClass("ebsta-feature-icon-disabled");

                //set up the tooltip for the disabled feature.
                SetupCampaignManagerUpgradeTooltip("ebsta-tooltip-campaignmanager", "sfEbstaCampaignMgrButton");
            }

            //Insert Meeting Scheduler Buttons
                var meetingSchedulerButton = document.createElement("div");
                meetingSchedulerButton.className = "sfEbstaMeetingSchedulerBtn unselectable";
                meetingSchedulerButton.unselectable = "on";
                meetingSchedulerButton.title = "Insert Calendar Link";

                var meetingSchedulerButtonSpan = document.createElement("span");
                //meetingSchedulerButtonSpan.className = "ebsta-glyphicon ebsta-glyphicons-calendar";

                if (buttonMode === 1) {
                    meetingSchedulerButton.classList.add("sfEbstaMeetingSchedulerBtn-style1");
                    meetingSchedulerButtonSpan.className = "ebsta-glyphicon";
                }

                meetingSchedulerButton.appendChild(meetingSchedulerButtonSpan);

                var td2 = Createtd("gU ebstaTD");
                td2.appendChild(meetingSchedulerButton);

                InsertButtonIntoComposeWindowOnTheEnd(composeButtonsContainer, td2);

                var colGroup4 = document.createElement('col');
                colGroup4.className = "ebstaCol";

                $(".IZ").find("colgroup").find(".Uf").before(colGroup4);

            //if meeting scheduler are disabled, disbale the button and add tooltip.
            if (abRules.EnableMeetingScheduler + "" != "1") {
                meetingSchedulerButton.classList.add("ebsta-feature-icon-disabled");

                //set up the tooltip for the disabled feature.
                SetupMeetingSchedulerUpgradeTooltip("ebsta-tooltip-meetingscheduler", "sfEbstaMeetingSchedulerBtn");
            }

            //remove/overwrite the click events on the disabled buttons.
            $('.ebsta-feature-icon-disabled').click(function () { return false; });
        }
    }

    return {
        Initialize : function(composeButton) {
            Initialize(composeButton, this);
            lSendLaterClosure.SetlEbstaComposeButtonsClosure(this);
        },
        UpdateEmailTrackingState : function(value) {
            lEmailTrackingState = value;
        },
        UpdateEmailTrackingButtonWithState : function(state) {
            this.UpdateEmailTrackingState(state);
            UpdateEmailTrackingButton(lEmailTrackingButton);
        },
        GetEmailTrackingState : function() {
            return lEmailTrackingState;
        },
        GetSendLaterClosure : function() {
            return lSendLaterClosure;
        },
        GetComplexAddClosure: function () {
            return lComplexAddClosure;
        },
        /**
         * Sets the complex add values for this local complex add closure from another given closure
         * @param {} complexAddClosure 
         * @returns {} 
         */
        SetComplexAddValues : function(complexAddClosure) {
            if (complexAddClosure != null) {
                var complexAddValues = complexAddClosure.GetComplexAddValues();
                lComplexAddClosure.SetComplexAddValues(complexAddValues);
                lComplexAddClosure.UpdateComplexAddButton(!complexAddClosure.IsComplexAddValuesEmpty());
            }
        },
        /**
         * Hide Gmails send button and displays our own send button so we can handle the actions being chosen
         * @returns {} 
         */
        UpdateSendEmailButton: function () {
            ShowOrHideEbstasOwnSendButton();
        }
    }
}

/**
 * Sets up a mutation observer on the body of Gmail and can handle multiple events such as;
 * - Detecting new compose windows being created as well as reply/forward windows
 * @param {} arrayOfEventsToActOn 
 * @returns {} 
 */
function EbstaGmailMutationObserver(arrayOfEventsToActOn) {

    /**
     * NewComposeWindowDetected callback event fired when selected as an event to pick up
     * @param {} mutations 
     * @returns {} 
     */
    function NewComposeWindowDetectedCallback(mutations) {
        mutations.forEach(function (mutation) {
            if (mutation.addedNodes != null && mutation.addedNodes.length > 0) {
                for (var i = 0; i < mutation.addedNodes.length; i++) {
                    var addedNode = mutation.addedNodes[i];
                    if (addedNode.parentElement != null) {
                        //Find the M9 class as this is a container for both normal compose and reply/forward windows
                        var isThisComposeWindow = addedNode.closest(".M9");
                        if (isThisComposeWindow != null) {
                            AddGoogleSendButtons();
                        }
                    }
                }
            }
        });
    }

    /**
     * List of Events you can use with this observer with callbacks defined within the object
     */
    var listOfEvents = {
        "NewComposeWindowDetected": NewComposeWindowDetectedCallback
    }

    /**
     * Sets up the mutation observer on the body of Gmail and loops through the events set 
     * when the EbstaGmailMutationObserver has initially been setup
     * @returns {} 
     */
    function SetMutationObserverForGmail() {
        // TODO: MUTATION
        var observer = new window.MutationObserver(function (mutations) {
            if (arrayOfEventsToActOn != null) {
                for (var i = 0; i < arrayOfEventsToActOn.length; i++) {
                    listOfEvents[arrayOfEventsToActOn[i]](mutations);
                }
            }
        });

        // configuration of the observer:
        var config = { childList: true };

        // pass in the target node, as well as the observer options
        observer.observe(document.body, config);
    }

    SetMutationObserverForGmail();
}

/**
 * Module for Google wide specific functionality
 */
var GoogleTools = (function GoogleTools() {
    var globalsInjectedScript = null;

    var googleModule = null;

    /**
     * Functionality around the top header on Google sites
     */
    var header = (function() {

        /**
         * Helps ensure we only added the buttons once
         */
        var wasButtonsInserted = false;

        /**
         * Store the DOM object for the main Ebsta header button
         */
        var mainEbstaHeaderButton = null;

        /**
         * Keeps a reference to all the ebsta buttons we've added apart from the main Ebsta button which appears all the time
         */
        var additionalButtonsArray = new Array();

        /**
         * Returns an element of which we need to insert our buttons into
         * @returns {} 
         */
        function FindContainerToInsertEbstaButtons() {
            var prependTo = null;

            if ($('div[role="banner"], #onegoogbar, .yzy1pe').length > 0) {
                if ($('div[role="banner"], #onegoogbar, .yzy1pe').children().length > 1) {
                    if ($('div[role="banner"], #onegoogbar, .yzy1pe').children(":first").children().length > 1) {
                        prependTo = $('div[role="banner"], #onegoogbar, .yzy1pe').children(":first").children(":first");
                    }
                }
            }

            if (prependTo == null) {
                if ($('.yzy1pe').length > 0) {
                    prependTo = $('.yzy1pe');
                }
            }

            if (prependTo == null) {
                var headerElem = document.getElementsByClassName("gb_Nf gb_Mg");
                if (headerElem) {
                    prependTo = headerElem;
                }
            }

            if (prependTo && prependTo.length === 1) return prependTo;

            //Find the container which has the buttons that appear in the top right of Gmail
            let headerButtonsSection = document.querySelector("header[role=\"banner\"] > div:nth-child(2) > div:nth-child(3)");
            if (headerButtonsSection) {
                return headerButtonsSection;
            }

            //Catch old Contacts view
            if (document.URL.indexOf("google.com/contacts") !== -1) {
                let googleAppsMenuIcon = document.getElementsByClassName("gb_qc")[0];
                if (googleAppsMenuIcon) {
                    let menuContainer = googleAppsMenuIcon.closest("div[data-ogsr-up]");
                    if (menuContainer) {
                        return menuContainer;
                    }
                }
            }
            
            return null;
        }

        /**
         * Creates and inserts the main header Ebsta button
         * @param {} elementToPrependButtonsTo 
         * @returns {} The Ebsta button DOM element
         */
        function InsertMainEbstaButton(elementToPrependButtonsTo) {
            var mainEbstaButton = CreateDiv("", "ebsta-google-header-container eb-authwizard-restart-parentdiv");
            mainEbstaButton.appendChild(CreateButton("EbstaBtOnboarding", "Ebsta", "ebsta-btn ebsta-btn-primary ebsta-btn-lg eb-authwizard-restart"));
            $(elementToPrependButtonsTo).addClass("ebsta-button-row-holder").prepend(mainEbstaButton);

            SetupOnboardingRestartEvent();

            return mainEbstaButton;
        }

        /**
         * Creates and sets up the Campaign Button for user within the Gmail header
         * @returns {DOMElement|null} - If the Campaigns feature is enabled for the user then we'll return the DOM if not we'll return null
         */
        function CreateCampaignButton() {
            if (abRules.EnableCampainMgr && globalCompanySettings.DisableCampaigns !== true) {
                let campaignManagementObject = createCampaignMgrObject(GmailTools);

                if (campaignManagementObject) {
                    campaignManagementObject.Initialize();
                    campaignManagementObject.SetCampaignSendLimit(globalCampaignLimit);
                    campaignManagementObject.SetEbstaComposeButtonsClosure(new EbstaComposeButtons());
                    return campaignManagementObject.CreateGmailHeaderCampaignButton();
                }
            }
            return null;
        }

        /**
         * Creates and sets the Ebsta Settings button up
         * @returns {} 
         */
        function CreateSettingsButton() {
            var settingsButton = CreateElement("button", "ebsta-btn ebsta-btn-primary ebsta-btn-lg ebsta-settings");
            settingsButton.title = "Settings";
            settingsButton.id = "EbstaBtSettings";
            settingsButton.appendChild(CreateElement("span", "fa fa-cog"));

            var divContainer = CreateElement("div", "ebsta-google-header-container ebsta-settings-parentdiv");
            divContainer.appendChild(settingsButton);
            divContainer.addEventListener("click", function() {
                window.open("https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=crmintegration&objectLinker=Account", "_blank");
            });

            return divContainer;
        }

        /**
         * Creates and sets the Email Tracking Analysis button up
         * @returns {} 
         */
        function CreateEbstaTrackingButton() {
            var trackingAnalysisButton = CreateElement("button", "ebsta-btn ebsta-btn-primary ebsta-btn-lg ebsta-tracking-analysis");
            trackingAnalysisButton.title = "Tracking Analysis";
            trackingAnalysisButton.id = "EbstaBtTrackingAnalysis";
            trackingAnalysisButton.appendChild(CreateElement("span", "fa fa-line-chart"));

            var divContainer = CreateElement("div", "ebsta-google-header-container ebsta-tracking-analysis-parentdiv");
            divContainer.appendChild(trackingAnalysisButton);
            divContainer.addEventListener("click", function() {
                window.open("https://console.ebsta.com/LoginSalesforce.aspx?k=" + webUserId + "&j=130", "_blank");
            });

            return divContainer;
        }

        /**
         * Creates and sets the Trial Countdown button up
         * @returns {} 
         */
        function CreateTrialCountDownButton() {
            var trialMessage = "Trial expires in " + daysRemainingOnTrial + " days";

            if (daysRemainingOnTrial + "" === "1") {
                trialMessage = "Trial expires in " + daysRemainingOnTrial + " day";
            }

            var btTrialContainer = CreateDiv("", "ebsta-google-header-container ebsta-trialmessage-parentdiv");
            btTrialContainer.appendChild(CreateButton("", trialMessage, "ebsta-btn ebsta-btn-primary ebsta-btn-lg ebsta-trialmessage"));
            btTrialContainer.addEventListener("click", function() {
                window.open("https://www.ebsta.com/pricing/", "_blank");
            });
            return btTrialContainer;
        }

        /**
         * Inserts buttons other than main Ebsta button i.e Campaigns/Settings/Tracking/TrialExpiry
         * @returns {} 
         */
        function InsertAdditionalButtons() {
            //We only want the extra buttons when a user is logged in currently
            if (LoggedIn) {
                var lastButtonInserted = CreateSettingsButton();
                additionalButtonsArray.push(lastButtonInserted);
                insertBefore(lastButtonInserted, mainEbstaHeaderButton);

                //Only add the tracking and campaigns button if a user is authed into gmail and on Gmail
                if (document.URL.toLowerCase().indexOf("mail.google.com/mail") !== -1 && isAccountAuthorised) {
                    var trackingButton = CreateEbstaTrackingButton();
                    if (trackingButton) {
                        insertBefore(trackingButton, lastButtonInserted);
                        lastButtonInserted = trackingButton;
                        additionalButtonsArray.push(lastButtonInserted);
                    }

                    var campaignButton = CreateCampaignButton();
                    if (campaignButton) {
                        insertBefore(campaignButton, lastButtonInserted);
                        lastButtonInserted = campaignButton;
                        additionalButtonsArray.push(lastButtonInserted);
                    }
                }

                if (renderTrialCountdown === true) {
                    var trialCountDownButton = CreateTrialCountDownButton();
                    insertBefore(trialCountDownButton, lastButtonInserted);
                    additionalButtonsArray.push(trialCountDownButton);
                }

                //Check if we're on the calendar page and then check if we should show an extra button prompting the user to sync their calendar
                if (document.URL.indexOf("google.com/calendar") > -1) {
                    //Becuase of width contraints we want to show the countdown over the sync calendet button
                    if (!renderTrialCountdown) {
                        DisplayCalendarSyncIcon();
                    }
                    GoogleCalendarTools.UpdateEbstaButtonsState();
                }
            }
        }

        /**
         * Inserts our buttons into the header of Google sites
         * @returns {} 
         */
        function InsertButtons() {
            if (!wasButtonsInserted) {

                var elementToPrependButtonsTo = FindContainerToInsertEbstaButtons();

                if (elementToPrependButtonsTo) {
                    wasButtonsInserted = true;
                    mainEbstaHeaderButton = InsertMainEbstaButton(elementToPrependButtonsTo);
                    InsertAdditionalButtons();
                    ChangeOnboardingButtonOnResize();
                }
            }
        }

        /**
         * To ensure the buttons exists create a retry process to ensure we find 
         * a element we can add the buttons to
         * @returns {} 
         */
        function InsertButtonsRetry() {
            if (!wasButtonsInserted && !GmailTools.IsThisAExternalGmailComposeScreen()) {
                RetryWithCallback(30, 1, function() {
                    if (FindContainerToInsertEbstaButtons() != null) return true;
                    return false;
                }, InsertButtons, function() {
                    throw Error("Unable to find Google header to insert buttons.");
                });
            }
        }

        /**
         * Removes all buttons apart from the main ebsta button, to help when a user is logged out
         * @returns {} 
         */
        function RemoveButtons() {
            for (var i = 0; i < additionalButtonsArray.length; i++) {
                additionalButtonsArray[i].remove();
            }
            additionalButtonsArray = new Array();
        }

        /**
         * Re-renders the buttons within the header apart from the main Ebsta button
         * @returns {} 
         */
        function ReRenderButtons() {
            RemoveButtons();
            InsertAdditionalButtons();
        }

        return {
            InsertButtons: InsertButtonsRetry,
            RemoveButtons: RemoveButtons,
            ReRenderButtons: ReRenderButtons
        }
    })();

    /**
     * Message listener used for when we inject the js file which returns us the global variables we're looking for
     * @param {} event 
     * @returns {} 
     */
    function GoogleGlobalVariableMessageListener(event){
        if (event && event.data) {
            if (event.data.text) {
                var messageType = event.data.type;
                if (messageType === "GMAIL-GLOBALS") {
                    GmailTools.SetVariableData(event.data.text);
                } else if (messageType === "CALENDAR-NEWUI") {
                    GoogleCalendarTools.SetVariableData(event.data.text);
                } else if (messageType === "CONTACTS-NEWUI" || messageType === "CONTACTS-OLDUI") {
                    GoogleContactTools.SetVariableData(event.data.text);
                }
            }
        }
    };

    //Listen out for the messages we're firing with out injected script to get the the variables 
    window.addEventListener("message", GoogleGlobalVariableMessageListener);

    /**
     * Injects a script into Gmail so we can access its variables, this isn't possible using Content Scripts
     * @returns {} 
     */
    function InjectScriptToGetGlobals() {
        //If we've injected this already and wish to re-inject, remove the previous one first
        if (globalsInjectedScript) {
            globalsInjectedScript.remove();
        }
        globalsInjectedScript = document.createElement('script');
        globalsInjectedScript.type = "text/javascript";
        globalsInjectedScript.src = chrome.extension.getURL('js/googleglobalgetter.js');
        (document.head || document.documentElement).appendChild(globalsInjectedScript);
    }

    /**
     * Checks we have a valid authentication for this google account
     * @param {} emailAddress 
     * @returns {} 
     */
    function CheckAuthForGoogleAccount(emailAddress) {
        if (!gmailMailboxAuthChecked) {
            chrome.extension.sendMessage({ MessageType: "CheckGmailAuthForMailbox", MailboxAddress: emailAddress });
        }
    }

    /**
     * Checks if the current gmail account has already been set to be active
     * @returns {} 
     */
    function CheckIfGoogleAccountIsActive(currentEmailAccount) {
        chrome.extension.sendMessage({
            MessageType: "GetGmailAccounts",
            CurrentEmailAccount: currentEmailAccount
        },
        function ()
        {
            chrome.extension.sendMessage({ MessageType: "GetAccountActive", EmailAddress: currentEmailAccount }, function (response) {
                if (response != null) {

                    if (response.AccountActive + "" === "true"
                        || response.GoogleCalendarSyncBubbleClick + "" === "true"
                        || response.GoogleContactSyncBubbleClick + "" === "true") {
                        currentMailBoxActive = response.AccountActive;
                        //Checks to see if this mailbox has been already been authenticated and fires off onboarding process
                        CheckAuthForGoogleAccount(currentEmailAccount);

                        return true;
                    }
                    // gmailBar is set by the the disable bar in gmail option, if checked will return true
                    // so actually if this is true, means you have switched the bar off in gmail, and we should just do nothing
                    else if (response.DropBarWithNoAuth === true && gmailBar + "" === "false") {
                        GmailTools.SetupGmailBarOnlyMode();
                    }

                    //If gmailBar is true and the account isn't active then it means that
                    //we don't want to show the dropdown bar or the ebsta gmail buttons
                    if (gmailBar + "" === "false" || response.AccountActive + "" === "true") {
                        header.InsertButtons();
                    }

                } else {
                    //If we reach here we only want to add our generic buttons which appear when you're not logged in and/or authed
                    header.InsertButtons();
                }
            });
        });
    }

    /**
     * We call the Initialize function on the module until it returns true.
     * The initilize function on the module should only return true when it's ready to be setup.
     * When it's happy and returns true if will then run more checks on what state to set the page up.
     * @param {} module - This needs to be an object which has an Initilize and GetUserName functions
     * @returns {} 
     */
    function InitializeRetry(module) {
        googleModule = module;
        RetryWithCallback(60, 0.5,
        function () {
            InjectScriptToGetGlobals();
            return googleModule.Initialize() === true;
        },
        function () {
            globalsInjectedScript.remove();
            window.removeEventListener("message", GoogleGlobalVariableMessageListener);
            CheckIfGoogleAccountIsActive(googleModule.GetUserName());
        },
        function () {
            globalsInjectedScript.remove();
            window.removeEventListener("message", GoogleGlobalVariableMessageListener);
            throw new Error("Unable to find Email Address");
        });
    }

    /**
     * Sets up the google page
     * @returns {} 
     */
    function Initialize() {

        // When we redirect to the login/auth process for Google it includes the redirect url 'mail.google.com' which 
        // helps kick this function off so making sure we don't do anything in this screen if it's in the login/auth process
        if (document.URL.indexOf("accounts.google.com") === -1) {
            if (LoggedIn) {
                if (document.URL.indexOf("mail.google.com") !== -1) {
                    InitializeRetry(GmailTools);
                } else if (document.URL.indexOf("calendar.google.com") !== -1) {
                    InitializeRetry(GoogleCalendarTools);
                } else if (document.URL.indexOf("google.com/contacts") !== -1 || document.URL.indexOf("contacts.google.com") !== -1) {
                    InitializeRetry(GoogleContactTools);
                }


            } else {
                header.InsertButtons();
            }
        }
    }

    /**
     * Gets the Username from the given module
     * @returns {} 
     */
    function GetEmailAddress() {
        if (googleModule) return googleModule.GetUserName();
        else return null;
    }

    return {
        CheckIfGoogleAccountIsActive: CheckIfGoogleAccountIsActive,
        GetEmailAddress: GetEmailAddress,
        Header: header,
        InjectScriptToGetGlobals: InjectScriptToGetGlobals,
        Initialize: Initialize
    }
})();

/**
 * Module for all things in Google Calendar
 */
var GoogleCalendarTools = (function GoogleCalendarTools() {
    var googleHeaderElement = null,
        googleCalVariableData = null,
        googleUsername = null;

    /**
     * Hides the ebsta buttons if the user is in the calendar event detail page
     * Note: Currently only set for the newer view of Google Calendar (Dec 2017)
     * @param {} headerElement 
     * @returns {} 
     */
    function ShowOrHideEbstaButtonsInGoogleCalendar(headerElement) {
        var functionToCall = null;
        if (headerElement && HasClass(headerElement, "PTLZrb")) {
            functionToCall = HideElement;
        } else {
            functionToCall = ShowElement;
        }

        if (functionToCall) {
            var ebstaButtons = headerElement.getElementsByClassName("ebsta-google-header-container");
            for (var i = 0; i < ebstaButtons.length; i++) {
                functionToCall(ebstaButtons[i]);
            }
        }
    }

    /**
     * Sets up a mutation observer on an element and detects an changes on the class of the element
     * on that detection it calls the function specified
     * @param {} headerElement 
     * @returns {} 
     */
    function ObserveClassChangeElement(element, functionToCall) {
        if (element) {
            // TODO: MUTATION
            var observer = new MutationObserver(function () {
                functionToCall();
            });

            observer.observe(element, {
                attributes: true,
                attributeFilter: ['class'],
                childList: false,
                characterData: false
            });
        }
    }

    /**
     * Searches the google variable to find a valid email or 
     * @returns {} 
     */
    function SetUsername() {
        var emailUsername;
        try {
            emailUsername = googleCalVariableData[0][2];
            if (ValidateEmail(emailUsername)) {
                googleUsername = emailUsername;
                return true;
            }
        } catch (error) {
            
        }

        try {
            emailUsername = googleCalVariableData[0][3];
            if (ValidateEmail(emailUsername)) {
                googleUsername = emailUsername;
                return true;
            }
        } catch (error) {

        }
    }

    /**
     * Initial setup for Google Calendar, needs to return a bool so GoogleTools knows if it has succeeded in finding it what it needs to get up an running
     * @returns {} 
     */
    function Initialize() {
        SetUsername();
        if (googleUsername != null) return true;
        return false;
    }

    return {
        Initialize: Initialize,
        GetUserName: function() {
            return googleUsername;
        },
        SetVariableData: function (data) { googleCalVariableData = data; },
        /**
         * Detects if this is the newer layout of Google Calendar, currently only looks for a 
         * header element which doesn't exist on the old calendar layout
         * @returns {} 
         */
        IsThisNewGoogleCalendarLayout : function() {
            return document.getElementsByTagName("header")[0] == null;
        },

        /**
         * Sets a number of events when within Google Calendar
         * @returns {} 
         */
        SetupEvents : function () {
            googleHeaderElement = document.getElementsByTagName("header")[0];
            if (googleHeaderElement) {
                ObserveClassChangeElement(googleHeaderElement,
                    function() { ShowOrHideEbstaButtonsInGoogleCalendar(googleHeaderElement) });
            }
        },

        /**
         * Keeps the Ebsta buttons in the header updated depending on what state the Calendar page is in
         * @returns {} 
         */
        UpdateEbstaButtonsState: function() {
            if (googleHeaderElement) {
                ShowOrHideEbstaButtonsInGoogleCalendar(googleHeaderElement);
            }
        }
    }
})();

/**
 * Module for all things in Google Contacts
 */
var GoogleContactTools = (function GoogleContactTools() {
    var googleContactData = null,
        googleUsername = null;

    /**
     * Looks through the global data received to find an email
     * Google Contacts has two different places due to two different UI's in action
     * @returns {} 
     */
    function SetUsername() {
        if (googleContactData) {
            //New UI returns a object usually
            var email = googleContactData["oPEP7c"];
            if (ValidateEmail(email)) {
                googleUsername = email;
                return true;
            }

            if (typeof googleContactData === "object") {
                try {
                    googleContactData.keys(googleContactData).forEach(function(key, index) {
                        var keyValue = googleContactData[key];
                        if (typeof keyValue === "string" && ValidateEmail(keyValue)) {
                            googleUsername = email;
                            return true;
                        }
                    });
                } catch (error) {
                    
                }
            }

            //Old UI
            try {
                var emailVal = googleContactData[0][10][5];
                if (ValidateEmail(emailVal)) {
                    googleUsername = emailVal;
                    return true;
                }
            } catch (err) {
                
            }
        }
    }

    /**
     * Initial setup for Google Contact, needs to return a bool so GoogleTools knows if it has succeeded in finding it what it needs to get up an running
     * @returns {} 
     */
    function Initialize() {
        SetUsername();
        if (googleUsername != null) return true;
        return false;
    }

    return {
        Initialize: Initialize,
        GetUserName: function () {
            return googleUsername;
        },
        SetVariableData: function (data) { googleContactData = data; },
    }
})();
/**
 * Module to contain generic functions we use in Ebsta Modals such as (Complex Add, Templates and Campaigns)
 */
var EbstaModalTools = (function EbstaModalTools() {

    /**
     * Returns a DOM Element which is used to create Help Icons
     * @param {} className 
     * @param {} href 
     * @param {} title 
     * @returns {} 
     */
    this.CreateHelpIcon = function (className, href, title) {
        var helpIcon = CreateElement("a", "ebsta-help-icon " + className, null, null);

        if (href) {
            helpIcon.href = href;
        }

        if (title) {
            helpIcon.title = title;
        }

        // Use FontAwesome questionmark icon
        var faQuestionIcon = document.createElement("i");
        faQuestionIcon.className = 'fa fa-question-circle fa-2x';

        // Append the fontawesome icon to the hyperlink
        helpIcon.appendChild(faQuestionIcon);

        return helpIcon;
    }

    return this;
})();


/**
 * All events used within Entelo
 * @returns {} 
 */
function SetupEnteloEvents() {
    //Keep this in its own closure so we can keep track of what elements we've added to
    (function () {
        /**
         * Last Element clicked
         */
        var lastClickedElement = null;

        /**
         * Custom attribute we'll append to the click element so we can gather information from the right record
         */
        var ebstaParserAttrName = "data-ebstaparser";

        /**
         * Removes the custom attribute from the previously chosen element, if there was one
         * @returns {} 
         */
        function RemoveEbstaParserAttr() {
            if (lastClickedElement) {
                lastClickedElement.removeAttribute(ebstaParserAttrName);
            }
        }

        /**
         * Adds our custom parser attribute to an element given
         * @param {} element 
         * @returns {} 
         */
        function AddCustomAttrToElement(element) {
            element.setAttribute(ebstaParserAttrName, "ebsta-current");
        }

         /** 
         * Checkes if the clicked element has the profile url attribute
         * which will send us to the full profile so we should just drop that bar that way
         * @param {} element
         * @returns {} 
         */
        function HandleUrlProfileClick(element) {
            //false unless we specfically find profile-url attribute
            var isProfile = false;
            if (element) {
                var dataTestAttr = element.getAttribute("data-test");
                if (dataTestAttr != null) {
                    isProfile = (dataTestAttr.toLowerCase() == "profile-url");
                }
            }
            return isProfile;
        }

        /**
        * Add Click listener to entire DOM as its an SPA we can't attach it to specific 
        * elements as this will be removed and added over and over agian
        */
        document.addEventListener("mouseup", function (event) {
            if (event.target && event.button === 0) {

                //Check first if this is a click to full profile - if so just go for bar drop
                var isProfileUrlClick = HandleUrlProfileClick(event.target);
                //if this is a profile url click - just go straight to bar drop
                //e.g even if the click is within the same profile container
                if (isProfileUrlClick) {
                    dropBarUsingRulesEngine();
                } else {
                    //Find the Contact container to ensure it was a click event on a contact
                    var profileContainer = event.target.closest("div[data-test='new-serp']");
                    if (profileContainer) {
                        //Check if the profile container already has the ebsta parser class, if so just user
                        //clicking on within the same profile, so ignore that
                        var profileHasEbstaAttribute = profileContainer.hasAttribute(ebstaParserAttrName);

                        //Find the search result container so we can esnure we only trigger when clicking inside this container
                        //this looks to ensure that the event element clicked is within the childeren of the search result container
                        var searchResultContainer = event.target.closest(".index__SearchResults__results__25PFX");

                        //ensure we are clicking in the profile container, and not within the same container
                        //and that this is not a click to view the full profile
                        if (profileContainer && searchResultContainer && !profileHasEbstaAttribute) {
                            RemoveEbstaParserAttr();
                            lastClickedElement = profileContainer;
                            AddCustomAttrToElement(profileContainer);
                            dropBarUsingRulesEngine();
                        }
                    }
                }
            }
        });
    })();
}

/**
 * LinkedIn helper class
 * @returns {} 
 */
var linkedInHelper = (function () {

    let hideBarPages = [
        '.linkedin.com/search/results/',
        '/discover/recruiterSearch?searchContextId' // Search results page for Linkedin Recruiter
    ];

    /**
     * checks to see if a page on linked in is in list of pages of where we should hide the bar
     * @param {string} url - The url to check against out list with
     * @returns {boolean} - If we should hide the bar
     */
    let hideBarForUrl = function (url) {
        for (var i = 0; i < hideBarPages.length; i++) {
            if (url.indexOf(hideBarPages[i]) !== -1) {
                return true;
            }
        }
        return false;
    };

    // Create an observer instance linked to the callback function  
    let searchBoxOverlayObserver = null;

    let pageEventInterval = null;

    /** Setup page events for LinkedIn */
    function ListenForPageEventsLinkedIn() {
        //set intial location
        var oldLocation = location.href;
        pageEventInterval = setInterval(function () {
            //compare old location and new location - if different run bar drop
            if (location.href != null && location.href.length > 0) {
                if (location.href != oldLocation && !sharedLinkedInHelper.DoUrlsContainTheSameId(location.href, oldLocation)) {
                    oldLocation = location.href;
                    //drop bar unless in list of pages to hide bar on
                    //if in list remove to stop breaking things like search page modals
                    if (!hideBarForUrl(location.href)) {
                        dropBarUsingRulesEngine();
                    } else {
                        proc.HideFrame();
                    }
                }
            }
        }, 500);
    }

    function StopListeningForPageEventsLinkedIn() {
        clearInterval(pageEventInterval);
    }

    /**
    * Fix search overlay positioning when the bar is dropped
    * so the search input window is not overlayed
    */
    var setOverlayPositioning = function (barHeight) {

        // If the overlay already exists on the page apply the offset so it moves when the bar is dropped
        const searchBoxOverlay = document.querySelector('.search-box__overlay--hidden, .search-box__overlay');
        if (searchBoxOverlay) {
            searchBoxOverlay.style.top = barHeight;
        }

        // Already set up so stop here
        if (searchBoxOverlayObserver) return;

        // Select the node that will be observed for mutations
        var nodeToObserve = document.getElementById('global-nav');

        if (nodeToObserve) {
            // Options for the observer (which mutations to observe)
            var optionsToObserve = {
                childList: true,
                subtree: true
            };

            // Callback function to execute when mutations are observed
            var callback = function (mutations) {
                mutations.forEach(function (mutation) {
                    if (mutation.addedNodes && mutation.addedNodes.length > 0) {
                        for (var i = 0; i < mutation.addedNodes.length; i++) {
                            if (mutation.addedNodes[i].className) {
                                if (mutation.addedNodes[i].className.indexOf('search-box__overlay') !== -1) {
                                    if (IsBarLoaded()) {
                                        mutation.addedNodes[i].style.top = barHeight;
                                    } else {
                                        mutation.addedNodes[i].style.top = 0;
                                    }
                                }
                            }
                        }
                    }
                });     
            };

            // Create an observer instance linked to the callback function  
            // TODO: MUTATION
            searchBoxOverlayObserver = new MutationObserver(callback);

            // Start observing the target node for configured mutations
            searchBoxOverlayObserver.observe(nodeToObserve, optionsToObserve);
        }
    }


    /**
    * Reverse search overlay back to its original position when the bar is turned off
    */
    var unsetOverlayPositioning = function () {
        var searchBoxOverlay = document.querySelector('.search-box__overlay--hidden, .search-box__overlay');
        if (searchBoxOverlay) {
            searchBoxOverlay.style.top = "";
        } 

        if (searchBoxOverlayObserver) {
            searchBoxOverlayObserver.disconnect();
            searchBoxOverlayObserver = null;
        }
    }

    return {
        StopListeningForPageEventsLinkedIn: StopListeningForPageEventsLinkedIn,
        ListenForPageEventsLinkedIn: ListenForPageEventsLinkedIn,
        SetOverlayPositioning: setOverlayPositioning,
        UnsetOverlayPositioning: unsetOverlayPositioning
    };
})();

/**
 * Crunchbase helper class
 * @returns {} 
 */
var crunchbaseHelper = (function () {

    var hideBarPages = [
        '.crunchbase.com/search/'
    ];

    /** @description Checks if we should remove the bar on a specific page
     * @param {string} url url of the page you are on
     * @returns {boolean} - If returns true, the bar should be removed on this page
     */
    hideBarForUrl = function (url) {
        for (var i = 0; i < hideBarPages.length; i++) {
            if (url.indexOf(hideBarPages[i]) !== -1) {
                return true;
            }
        }
        return false;
    },
    /** @description Determines if two urls match before the first hash
     * @param {string} oldLocation the first url, the last page we dropped the bar
     * @param {string} currentLocation the second url, the page you are on now
     * @returns {boolean} - If the two urls match - it means you have not changed page only the hash has changed
     */
    locationCompareWithoutHash = function(oldLocation, currentLocation) {
        if (oldLocation != null && currentLocation != null) {
            return oldLocation.split("#")[0] === currentLocation.split("#")[0];
        }
        return false;
    }

    return {
        /** @description Sets up page page events on crunchbase, check on an internal 
         * if user has navigated to a new profile*/
        ListenForPageEventsCrunchbase: function () {
            //set intial location
            var oldLocation = location.href;
            setInterval(function () {
                //compare old location and new location - if different run bar drop
                if (location.href != null && location.href.length > 0) {
                    if (!locationCompareWithoutHash(oldLocation, location.href)) {
                        oldLocation = location.href;
                        //drop bar unless in list of pages to hide bar on
                        //if in list remove to stop breaking things like search page modals
                        if (!hideBarForUrl(location.href)) {
                            dropBarUsingRulesEngine();
                        } else {
                            proc.HideFrame();
                        }
                    }
                }
            }, 1000);
        }
    }
})();

// setup indeed resume handler.
var resumesIndeedHelper;
if (document.URL.toLowerCase().indexOf("resumes.indeed.com/search") !== -1) {
    /**
 * A module to handle the resumes indeed optmization
 * */
    resumesIndeedHelper = (function () {

        /** 
         * Rezemp-ResumeSearchPage-middleContainer is the main content container
         * This follows bootstrap style to column design. Second column is the profile preview */
        var previewContainerSelector = ".rezemp-ResumeSearchPage-middleContainer > div:nth-child(2)"

        /**
         * The inner element of the preview container, this gets created the initial time you
         *  hover over a profile. This is the basis of our first bar drop, and then the 
         *  mutation observer that will listen to the name being changes in the profile
         * */
        var previewContainerOutClass = "rezemp-ResumePreview";

        /**
         * The selector for the profiles name headline element, this gets changed
         * every time you view a new profile, so we need to observe changes to this element also
         * */
        var previewContainerNameSelector = ".rezemp-u-h2";

        var setupResumesIndeedObservers = function () {
            //The outer profile element - this is created on page load, but 
            //we will observe when the content is actually inserted here.
            var profileContainer = document.querySelector(previewContainerSelector);
            if (profileContainer !== null) {
                indeedResumesOuterObserver.observe(profileContainer, { subtree: true, childList: true })
            }
        }

        /**
         * The outer observer. Listen for the initial profile panel being created
         * */
        // TODO: MUTATION
        var indeedResumesOuterObserver = new MutationObserver(function (mutations) {
            mutations.forEach(function (mutation) {

                if (mutation.type === "childList") {
                    if (mutation.addedNodes && mutation.addedNodes.length > 0) {
                        for (var index = 0; index < mutation.addedNodes.length; index++) {
                            if (mutation.addedNodes[index].classList) {
                                if (mutation.addedNodes[index].classList.contains(previewContainerOutClass)) {
                                    dropBarUsingRulesEngine();
                                    //inner observer - after the first creation of the profile, observe when the name is changed 
                                    //within the preview element
                                    var nameNode = mutation.addedNodes[index].querySelector(previewContainerNameSelector);
                                    if (nameNode !== null) {
                                        //init inner observer
                                        indeedResumesInnerObserver.observe(nameNode, {
                                            characterData: true, subtree: true, childList: true
                                        });
                                    }
                                }
                            }
                        }
                    }
                }
            });
        });

        /**
         * The inner observer. Listen for the name being changed within the profile panel
        * @returns {}
        */
        // TODO: MUTATION
        var indeedResumesInnerObserver = new MutationObserver(function (mutations) {
            mutations.forEach(function (mutation) {

                if (mutation.type === "characterData") {
                    if (mutation.target && mutation.target.textContent) {
                        var pastContent = mutation.target.textContent;

                        //Run a timeout, because you are on a list of people that you can be scrolling through 
                        //quickly, so hold for n seconds and check that the mutation that triggered you to get here
                        //has the same text content change that is currently on the page - if so then drop the bar
                        //if not the user is still moving trough so dont drop
                        setTimeout(function () {
                            //find the name element within the outer element selector
                            var currentContent = document.querySelector(previewContainerSelector + " " + previewContainerNameSelector);
                            if (currentContent !== null && currentContent.innerText) {
                                if (pastContent === currentContent.innerText) {
                                    dropBarUsingRulesEngine();
                                }
                            }
                        }, 500);
                    }
                }
            });
        });

        return {
            /**
            * Sets up two oberservers on indeed, one that listens to the first profile being opened
            * then one observer that listens for changes to the profile name element within this first profile
            * container. Observer 1 allows us to drop the bar the first time and pick up the name element
            * to then register observer 2.
            * @returns {}
            */
            SetupResumesIndeedObservers: setupResumesIndeedObservers
        }
    })();
}


/**
 * As of June 2019 customers shifting to new version of Seek.
 * This class handles the opt of this version.
 * */
var talentsearchSeek2019VersionHelper = (function () {

    /**
    * Custom class we'll append to the click element so we can gather information from the right record
    */
    let ebstaParserClassName = "Ebsta_Parser";

    /**
     * Stores the previous profile card that dropped the bar
     */
    let previousProfileContainer = null;

    /**
     * Selector for the side bar modal which shows the actual profile
     */
    let modalQuerySelector = "#modal-root div";

    /**
     * Init a FixMarginHelper wher bar height is 50px and we are changing the margin-top property.
    */
    let fixMarginHelper = FixMarginHelper('50px', '', '', 3);

    /**
     * Removes the custom class from the previously chosen element, if there was one
     */
    function RemoveEbstaParserClass() {
        if (previousProfileContainer) {
            previousProfileContainer.classList.remove(ebstaParserClassName);
        }
    }

    /**
     * Adds our custom parser class to an element given
     * @param {any} element - element to add the class to
     */
    function AddCustomClassToElement(element) {
        element.classList.add(ebstaParserClassName);
    }

    return {
        /**
         * On click events look for the closest profile card, and check if this is the
         * profile tab that was clicked last bar drop, if not, then drop the bar again
         * @param {any} clickEvent - the full client event that was triggered
         */
        PageClickHandler: function (clickEvent) {
            var lastClickedProfile = clickEvent.target.closest('div[data-cy="profile-card"]');
            if (lastClickedProfile) {
                //check if current candidate detail card is the same as previously clicked card
                //to prevent bar drop if clicking in same candidate card
                if (lastClickedProfile && previousProfileContainer) {
                    if (lastClickedProfile.innerText != previousProfileContainer.innerText) {
                        RemoveEbstaParserClass();
                        AddCustomClassToElement(lastClickedProfile);
                        dropBarUsingRulesEngine();
                        previousProfileContainer = lastClickedProfile;
                    }
                } else if (lastClickedProfile) {
                    AddCustomClassToElement(lastClickedProfile);
                    dropBarUsingRulesEngine();
                    previousProfileContainer = lastClickedProfile;
                }

                //fix the position of the sidebar modal - as we will have dropped the bar by now
                fixMarginHelper.ApplyOrRemoveMarginWithRetry(modalQuerySelector, true);
            }
        },
        /**
         * Fix the position of the side bar modal after a bar drop
         */
        FixModalPosition: function () {
            fixMarginHelper.ApplyOrRemoveMarginWithRetry(modalQuerySelector, true);
        },
        /**
         * Returns the selector used to find the sidebar modal
         */
        GetModalQuerySelector: modalQuerySelector
    };
})();

/**
* Closure for all careerBuilderEvents
* @returns{}
* */
const careerBuilderEvents = (function () {

    var doWeNeedToSetMouseEvent = true;

    var lastClickedElement = null;

    /**
    * Custom class we'll append to the click element so we can gather information from the right record
    */
    var ebstaParserClassName = "Ebsta_Parser";

    /**
     * Removes the custom class from the previously chosen element, if there was one
     * @returns {} 
     */
    function RemoveEbstaParserClass() {
        if (lastClickedElement) {
            lastClickedElement.classList.remove(ebstaParserClassName);
        }
    }

    /**
     * Adds our custom parser class to an element given
     * @param {} element 
     * @returns {} 
     */
    function AddCustomClassToElement(element) {
        element.classList.add(ebstaParserClassName);
    }

    /**
    * Add Click listener to entire DOM as its an SPA 
    */
    function SetupCareerBuilderSearchListParser() { 
        if (doWeNeedToSetMouseEvent) {
            document.addEventListener("mouseup", function (event) {
                if (event.target && event.button === 0) {
                    //Find the Contact container to ensure it was a click event on a contact
                    var profileContainer = event.target.closest("candidate-list-item");                
                    if (profileContainer) {
                        RemoveEbstaParserClass();
                        lastClickedElement = profileContainer;
                        AddCustomClassToElement(profileContainer);
                        dropBarUsingRulesEngine();
                    }
                }
            });
        }
    doWeNeedToSetMouseEvent = false;
    };
    return {
        SetupCareerBuilderSearchListParser: SetupCareerBuilderSearchListParser
    }
})();